Package: distr
Version: 2.9.7
Date: 2025-01-11
Title: Object Oriented Implementation of Distributions
Description: S4-classes and methods for distributions.
Authors@R: c(person("Florian", "Camphausen", role="ctb", comment="contributed as student in
            the initial phase --2005"), person("Matthias", "Kohl", role=c("aut", "cph")),
            person("Peter", "Ruckdeschel", role=c("cre", "cph"),
            email="peter.ruckdeschel@uni-oldenburg.de"), person("Thomas", "Stabla",
            role="ctb", comment="contributed as student in the initial phase --2005"),
            person("R Core Team", role = c("ctb", "cph"), comment="for source file ks.c/
            routines 'pKS2' and 'pKolmogorov2x'"))
Depends: R(>= 3.4), methods, graphics, startupmsg(>= 1.0.0), sfsmisc
Suggests: distrEx, svUnit (>= 0.7-11), knitr, distrMod, ROptEst
Imports: stats, grDevices, utils, MASS
Enhances: RobAStBase
VignetteBuilder: knitr
ByteCompile: yes
Encoding: UTF-8
License: LGPL-3
URL: http://distr.r-forge.r-project.org/
LastChangedDate: {$LastChangedDate: 2024-11-05 21:59:54 +0100 (Di, 05
        Nov 2024) $}
LastChangedRevision: {$LastChangedRevision: 1482 $}
VCS/SVNRevision: 1482
NeedsCompilation: yes
Packaged: 2025-01-11 19:21:28 UTC; ruckdesc
Author: Florian Camphausen [ctb] (contributed as student in the initial phase
    --2005),
  Matthias Kohl [aut, cph],
  Peter Ruckdeschel [cre, cph],
  Thomas Stabla [ctb] (contributed as student in the initial phase
    --2005),
  R Core Team [ctb, cph] (for source file ks.c/ routines 'pKS2' and
    'pKolmogorov2x')
Maintainer: Peter Ruckdeschel <peter.ruckdeschel@uni-oldenburg.de>
Repository: CRAN
Date/Publication: 2025-01-12 23:30:02 UTC
Built: R 4.3.3; aarch64-apple-darwin20; 2025-01-24 14:59:25 UTC; unix
Archs: distr.so.dSYM

useDynLib(distr, .registration = TRUE, .fixes = "C_")
import("methods")
import("stats")
importFrom("grDevices", "dev.list", "dev.new", "xy.coords", "dev.off")
importFrom("graphics", "plot", "abline", "layout", "legend", "lines", "mtext", "par", "points", "title")
importFrom("MASS", "ginv")
importFrom("utils", "str")
importFrom("sfsmisc", "D1ss")
import("startupmsg")

export("Beta", "Binom", "Cauchy", "Chisq",  
       "Dirac","Exp", "DExp", "Fd", "Gammad", 
       "Geom", "Hyper", "Lnorm", "Logis", 
       "Nbinom", "Norm", "Pois", "RtoDPQ", 
       "RtoDPQ.d", "Td", "Unif", "Weibull", "Arcsine", 
       "distroptions", "getdistrOption", "simplifyr",
       "Lattice", "DiscreteDistribution",
       "EmpiricalDistribution",
       "LatticeDistribution", "EuclideanSpace", "Reals", 
       "Naturals", "standardMethods", 
       "distrARITH", "distrMASK", "getLabel", "devNew")
export("AbscontDistribution")
export("DistrList", "UnivarDistrList")
export("makeAbscontDistribution") 
exportClasses("rSpace", "EuclideanSpace", "Reals", 
              "Naturals")
exportClasses("Parameter")
exportClasses("OptionalParameter", "OptionalMatrix")
exportClasses("BetaParameter", "BinomParameter", 
              "CauchyParameter", "ChisqParameter",
              "DiracParameter", "ExpParameter", 
              "FParameter", "GammaParameter", 
              "HyperParameter",
              "LogisParameter", "LnormParameter",
              "NbinomParameter", "NormParameter", 
              "PoisParameter", "TParameter",
              "UnifParameter", "WeibullParameter", 
              "UniNormParameter")
exportClasses("Distribution")
exportClasses("UnivariateDistribution", "AbscontDistribution", 
              "DiscreteDistribution", "LatticeDistribution", 
              "AffLinAbscontDistribution", "AffLinDiscreteDistribution", 
              "AffLinLatticeDistribution", "AffLinDistribution", "Lattice",
              "Beta", "Binom", "Cauchy", "Chisq", "Dirac", 
              "DExp", "Exp", "Fd", "Gammad", "Geom", 
              "Hyper", "Logis", "Lnorm", "Nbinom", "Norm", 
              "Pois", "Td", "Unif", "Weibull", "Arcsine",
              "ExpOrGammaOrChisq")
exportClasses("UnivDistrListOrDistribution")
exportClasses("CompoundDistribution")
exportClasses("DistrList", 
              "UnivarDistrList")
exportClasses("OptionalNumeric", "PosSemDefSymmMatrix",
              "PosDefSymmMatrix",
              "Symmetry", "DistributionSymmetry",
              "NoSymmetry", "EllipticalSymmetry", "SphericalSymmetry",
              "DistrSymmList")
exportMethods("Max", "Max<-", "Min", "Min<-", "d", "df", 
              "df<-", "df1", "df1<-", "df2", 
              "df2<-", "dimension", "dimension<-", "name", 
              "name<-", "img", "k", "k<-", 
              "lambda", "lambda<-", "liesIn", "location", 
              "location<-", "m", "m<-", "mean", 
              "mean<-", "meanlog", "meanlog<-", "n", 
              "n<-", "ncp", "ncp<-", "p", "param", 
              "prob", "prob<-", "q", "r", "rate", "p.l", "q.r",
              "rate<-", "scale", "scale<-", "sd", "sd<-", 
              "sdlog", "sdlog<-", "shape", "shape<-", 
              "shape1", "shape1<-", "shape2", "shape2<-", 
              "size", "size<-", "support", "initialize", 
              "print", "plot", "+", "-", "/", "*", "coerce",
              "Math", "log", "log10", "gamma", "lgamma", "digamma", 
              "dim", "show", "convpow", "pivot", "sign",
              "lattice", "width", "Length", "pivot<-", 
              "width<-", "Length<-", "liesInSupport",
              "isOldVersion", "conv2NewVersion", "gaps",
              "gaps<-", "setgaps", "getLow", "getUp")
exportClasses("UnivarMixingDistribution",
              "UnivarLebDecDistribution",
              "AffLinUnivarLebDecDistribution",
              "AcDcLcDistribution")
exportMethods("mixCoeff", "mixCoeff<-", "mixDistr", "mixDistr<-",
              "discretePart", "discretePart<-", "acPart", "acPart<-",
              "discreteWeight", "discreteWeight<-", "acWeight", "acWeight<-",
              "p.discrete", "d.discrete", "q.discrete", "r.discrete",
              "p.ac", "d.ac", "q.ac", "r.ac", "p.r", "q.l")
exportMethods("decomposePM", "simplifyD", "showobj")
exportMethods("Truncate","Minimum","Maximum","Huberize")
exportMethods("solve", "sqrt")
exportMethods("type", "SymmCenter", "Symmetry", ".logExact", ".lowerExact")
export("UnivarMixingDistribution", "UnivarLebDecDistribution")
export("RtoDPQ.LC", "flat.LCD", "flat.mix")
exportMethods("abs","exp","^")
exportMethods("NumbOfSummandsDistr","SummandsDistr")
export("CompoundDistribution")
export("PosDefSymmMatrix","PosSemDefSymmMatrix")
export("NoSymmetry", "EllipticalSymmetry", "SphericalSymmetry",
       "DistrSymmList") 
export("qqbounds","igamma")
exportMethods("qqplot")
export(".isEqual",".isEqual01", ".inArgs", ".fillList", 
       ".presubs", ".makeLenAndOrder", ".DistrCollapse")
export("samplesize", "distribution", "samplesize<-")	   

##########################################################
## Demo: Instructive destructive example
##########################################################
require(distr)
options("newDevice"=TRUE)

## package "distr" encourages 
## consistency but does not 
## enforce it---so in general  
## d o   n o t   m o d i f y
## slots d,p,q,r!

N <- Norm()
B <- Binom()
N@d <- B@d
plot(N) 
### consequence: the slots of N
## are no longer consistent!!

require(distr)
options("newDevice"=TRUE)

if(!isGeneric("E")) 
   setGeneric("E", 
         function(object, fun) standardGeneric("E"))

setMethod("E",
        signature(object = "AbscontDistribution", 
                    fun = "function"),
        function(object, fun){
            integrand <- function(x) fun(x) * d(object)(x)
            return(integrate(f = integrand,
                             lower = q.l(object)(0),
                             upper = q.l(object)(1))$value)
          })

setMethod("E",
          signature(object = "DiscreteDistribution", 
                    fun = "function"),
          function(object, fun){
            supp = support(object)
            return(sum(fun(supp) * d(object)(supp)))
          })



# Example
id <- function(x) x
sq <- function(x) x^2

# Expectation and Variance of Binom(6,0.5)
B <- Binom(6, 0.5)
E(B, id)
E(B, sq) - E(B, id)^2

# Expectation and Variance of Norm(1,1)
N <- Norm(1, 1)
E(N, id)
E(N, sq) - E(N, id)^2

require(distr)
options("newDevice"=TRUE)

N <- Norm(0,1)
U <- Unif(0,1)
U2 <- U + U 
U4 <- U2 + U2
U8 <- U4 + U4
U12 <- U4 + U8
NormApprox <- U12 - 6

x <- seq(-4,4,0.001)

opar <- par(mfrow = c(2,1), no.readonly = TRUE)

plot(x, d(NormApprox)(x),
     type = "l",
     xlab = "",
     ylab = "Density",
     main = "Exact and approximated density")
lines(x, d(N)(x),
      col = "red")
legend("topleft",
       legend = c("NormApprox", "Norm(0,1)"),
       fill = c("black", "red"))

plot(x, d(NormApprox)(x) - d(N)(x),
     type = "l",
     xlab = "",
     ylab = "\"black\" - \"red\"",
     col = "darkgreen",
     main = "Error")
lines(c(-4,4), c(0,0))

cat("Hit <enter> to continue...")
readline()


f.n <- function(z, n){
    sapply(X = z,
           FUN = function(z, n){
               ind <- 0:n
               sum((-1)^ind*choose(n, ind)*(z-ind)^(n-1)*(z > ind))/factorial(n-1)
           },
           n = n)
}

z <- seq(from = 0, to = 6, length = 1000)

par(mfrow = c(1,1))

plot(z, f.n(z, n=1), type = "l", ylab = expression(paste(f[n](z))), lwd = 2, col = 1)
lines(z, f.n(z, n=2), lwd = 2, col = 2)
lines(z, f.n(z, n=3), lwd = 2, col = 3)
lines(z, f.n(z, n=4), lwd = 2, col = 4)
lines(z, f.n(z, n=5), lwd = 2, col = 5)
lines(z, f.n(z, n=6), lwd = 2, col = 6)
lines(z, dnorm(z, mean = 3), col = "orange", lwd = 2)
legend("topleft", xjust = 1, legend = c(expression(paste(f[1])), expression(paste(f[2])),
                        expression(paste(f[3])), expression(paste(f[4])),
                        expression(paste(f[5])), expression(paste(f[6])), "N(0,1)"),
       fill = c(1:6, "orange"), ncol = 2)
title("Densitiy of the sum of uniform distributed RV's")

par(opar)

require(distr)
options("newDevice"=TRUE)
Pause <- function() { cat("Hit <enter> to continue..."); readline()}

#################################################
## Comparison 1 - Exact, FFT and RtoDPQ 
#################################################

N1 <- Norm(0,3)
N2 <- Norm(0,4)
rnew1 <- function(n) r(N1)(n) + r(N2)(n) 

#  calls exact formula -> N(0,5)
X <- N1 + N2 
# calls FFT
Y <- N1 + as(N2, "AbscontDistribution") 
# appoximation via RtoDPQ
Z <- new("AbscontDistribution", r = rnew1) 

# density-plot
x <- seq(-15,15,0.01)
plot(x, d(X)(x), type = "l", lwd = 3, xlab = "", ylab = "density",  
      main = "Comparison 1", col = "black")
lines(x, d(Y)(x), col = "yellow")
lines(x, d(Z)(x), col = "red")
legend("topleft",  legend = c("Exact", "FFT-Approximation", 
                                             "RtoDQP-Approximation"),
           fill = c("black", "yellow", "red"))
Pause()

#################################################
## Comparison 2 - "Exact" Formula and RtoDPQ
#################################################

B <- Binom(size = 6, prob = 0.5) * 10
N <- Norm()
rnew2 <- function(n) r(B)(n) + r(N)(n)

# calls "exact" fomula
Y <- B + N 
# appoximation via RtoDPQ
Z <- new("AbscontDistribution", r = rnew2) 

# density-plot
x  <- seq(-5,65,0.01)
plot(x, d(Y)(x), type = "l", xlab = "", ylab = "density",
      main = "Comparison 2", col = "black")
lines(x, d(Z)(x), col = "red")
legend("topleft", legend = c("Exact", "RtoDQP-Approximation"),
           fill = c("black", "red"))

## ----knitRPreparations,include=FALSE------------------------------------------
library(knitr)
opts_chunk$set(tidy=FALSE)

## ----Prepa0, include=FALSE, results="hide"------------------------------------
require(distr)

## ----Prepa, echo=FALSE, results="asis"------------------------------
## preparation: set option withSweave to TRUE
require(distr)
distroptions(withSweave = TRUE)
options(width=70)

## ----exam1, eval = TRUE, fig.width=8.0, fig.height=6.5--------------
require(distr)
N <- Norm(mean = 2, sd = 1.3)
P <- Pois(lambda = 1.2)
Z <- 2*N + 3 + P
Z
plot(Z, panel.first = grid(), lwd=3)
p(Z)(0.4)
q(Z)(0.3)
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
Zs <- r(Z)(50)
Zs

## ----DiscrDist, eval = TRUE-----------------------------------------
D <- DiscreteDistribution(supp = c(1,5,7,21), prob = c(0.1,0.1,0.6,0.2))
D
plot(D, panel.first = grid(lwd=2), lwd = 3)

## ----AbscDist, eval = TRUE------------------------------------------
AC <- AbscontDistribution(d = function(x) exp(-abs(x)^3), withStand = TRUE)
AC
plot(AC, panel.first = grid(lwd=2), lwd = 3)

## ----AllClass1, results="asis", echo=TRUE---------------------------
## Class: BinomParameter
setClass("BinomParameter",
          representation = representation(size = "numeric", prob = "numeric"),
          prototype = prototype(size = 1, prob = 0.5, name =
                      gettext("Parameter of a Binomial distribution")
                      ),
          contains = "Parameter"
          )

## ----AllClass2, results="asis", echo=TRUE---------------------------
## Class: binomial distribution
setClass("Binom",
          prototype = prototype(
                      r = function(n){ rbinom(n, size = 1,prob = 0.5) },
                      d = function(x, log = FALSE){
                              dbinom(x, size = 1, prob = 0.5, log = log)
                                          },
                      p = function(q, lower.tail = TRUE, log.p = FALSE ){
                              pbinom(q, size = 1, prob = 0.5,
                                     lower.tail = lower.tail, log.p = log.p)
                                          },
                      q = function(p, lower.tail = TRUE, log.p = FALSE ){
                              qbinom(p, size = 1, prob = 0.5,
                                     lower.tail = lower.tail, log.p = log.p)
                                          },
                      img = new("Naturals"),
                      param = new("BinomParameter"),
                      support = 0:1,
                      lattice = new("Lattice",
                                pivot = 0, width = 1, Length = 2, name =
                                gettext(
                                  "lattice of a Binomial distribution"
                                       )
                                ),
                     .logExact = TRUE,
                     .lowerExact = TRUE
                      ),
          contains = "LatticeDistribution"
          )

## ----BinomDist1, results="asis", echo=TRUE--------------------------
## Access Methods
setMethod("size", "BinomParameter", function(object) object@size)
setMethod("prob", "BinomParameter", function(object) object@prob)
## Replace Methods
setReplaceMethod("size", "BinomParameter",
                  function(object, value){ object@size <- value; object})
setReplaceMethod("prob", "BinomParameter",
                  function(object, value){ object@prob <- value; object})

## ----AllGenerics, results="asis", echo=TRUE-------------------------
if(!isGeneric("size"))
   setGeneric("size", function(object) standardGeneric("size"))
if(!isGeneric("prob"))
   setGeneric("prob", function(object) standardGeneric("prob"))

## ----BinomDist2, results="asis", echo=TRUE--------------------------
setValidity("BinomParameter", function(object){
  if(length(prob(object)) != 1)
    stop("prob has to be a numeric of length 1")
  if(prob(object) < 0)
    stop("prob has to be in [0,1]")
  if(prob(object) > 1)
    stop("prob has to be in [0,1]")
  if(length(size(object)) != 1)
    stop("size has to be a numeric of length 1")
  if(size(object) < 1)
    stop("size has to be a natural greater than 0")
  if(!identical(floor(size(object)), size(object)))
    stop("size has to be a natural greater than 0")
  else return(TRUE)
})

## ----BinomDist3, results="asis", echo=TRUE--------------------------
Binom <- function(size = 1,prob = 0.5) new("Binom", size = size, prob = prob)

## ----BinomDist4, results="asis", echo=TRUE--------------------------
## Convolution for two binomial distributions Bin(n1,p1) and Bin(n2,p2)
## Distinguish cases
## p1 == p2 und p1 != p2


setMethod("+", c("Binom","Binom"),
          function(e1,e2){
            newsize <- size(e1) + size(e2)

            if(isTRUE(all.equal(prob(e1),prob(e2))))
               return(new("Binom", prob = prob(e1), size = newsize,
                          .withArith = TRUE))

            return(as(e1, "LatticeDistribution") + e2)
          })

## ----Prepa2, echo=FALSE, results="asis"-----------------------------
## preparation: set option withSweave to TRUE
require(distrEx)

## ----Expect, results="asis", echo=TRUE------------------------------
setMethod("E", signature(object = "Binom",
                         fun = "missing",
                         cond = "missing"),
    function(object, low = NULL, upp = NULL, ...){
    if(!is.null(low)) if(low <= min(support(object))) low <- NULL
    if(!is.null(upp)) if(upp >= max(support(object))) upp <- NULL
    if(is.null(low) && is.null(upp))
        return(size(object)*prob(object))
    else{
        if(is.null(low)) low <- -Inf
        if(is.null(upp)) upp <- Inf
        if(low == -Inf){
           if(upp == Inf) return(size(object)*prob(object))
           else return(m1df(object, upper = upp, ...))
        }else{
           E1 <- m1df(object, upper = low, ...)
           E2 <- if(upp == Inf)
                    size(object)*prob(object) else m1df(object, upper = upp, ...)
           return(E2-E1)
        }
    }
   })

## ----var, results="asis", echo=TRUE---------------------------------
setMethod("var", signature(x = "Binom"),
    function(x,...){
    dots <- match.call(call = sys.call(sys.parent(1)),
                       expand.dots = FALSE)$"..."
    fun <- NULL; cond <- NULL; low <- NULL; upp <- NULL
    if(hasArg(low)) low <- dots$low
    if(hasArg(upp)) upp <- dots$upp
    if(hasArg(fun)||hasArg(cond)||!is.null(low)||!is.null(upp))
        return(var(as(x,"DiscreteDistribution"),...))
    else
        return(size(x)*prob(x)*(1-prob(x)))
    })

## ----skew, results="asis", echo=TRUE--------------------------------
setMethod("skewness", signature(x = "Binom"),
    function(x,  ...){
    dots <- match.call(call = sys.call(sys.parent(1)),
                       expand.dots = FALSE)$"..."
    fun <- NULL; cond <- NULL; low <- NULL; upp <- NULL
    if(hasArg(low)) low <- dots$low
    if(hasArg(upp)) upp <- dots$upp
    if(hasArg(fun)||hasArg(cond)||!is.null(low)||!is.null(upp))
       return(skewness(as(x,"DiscreteDistribution"),...))
    else
        return((1-2*prob(x))/sqrt(size(x)*prob(x)*(1-prob(x))))
    })

## ----kurt, results="asis", echo=TRUE--------------------------------
setMethod("kurtosis", signature(x = "Binom"),
    function(x,  ...){
    dots <- match.call(call = sys.call(sys.parent(1)),
                       expand.dots = FALSE)$"..."
    fun <- NULL; cond <- NULL; low <- NULL; upp <- NULL
    if(hasArg(low)) low <- dots$low
    if(hasArg(upp)) upp <- dots$upp
    if(hasArg(fun)||hasArg(cond)||!is.null(low)||!is.null(upp))
       return(kurtosis(as(x,"DiscreteDistribution"),...))
    else
        p <- prob(x)
        return((1-6*p*(1-p))/(size(x)*p*(1-p)))
    })

# test for equality of the saved result and the actual result of a dontrun example
test.qqplot1 <- function() {
  # we compute the actual result
  P <- Pois(5)
  B <- Binom(size=2000,prob=5/2000)
  runit.dontrunQQPlot1.actual <- qqplot(B,P, nosym.pCI=TRUE)
  
  # we load the stored result
  #   we assume that this test is called from within the script in the upper directory
  load("unitTests/runit.dontrunQQPlot1.save")
  
  # we compare the stored result with the calculated one
  #   (a comparison with identical (ignoring the environment) gives FALSE...
  result <- all.equal(runit.dontrunQQPlot1.actual,
                      runit.dontrunQQPlot1.save)
  
  # we check whether the result is TRUE and if not, we write the message
  #   coming from the result
  checkEquals(is.logical(result) && result, TRUE, msg=paste(result, sep="", collapse="\n"))
}


# test for equality of the saved result and the actual result of a dontrun example
test.qqplot2 <- function() {
  # we compute the actual result
  mylist <- UnivarLebDecDistribution(discretePart=Binom(3,.3), acPart=Norm(2,2),
                                     acWeight=11/20)
  mylist2 <- mylist+0.1
  
  runit.dontrunQQPlot2.actual <- qqplot(mylist,mylist2,nosym.pCI=TRUE)
  
  # we load the stored result
  #   we assume that this test is called from within the script in the upper directory
  load("unitTests/runit.dontrunQQPlot2.save")
  
  # we compare the stored result with the calculated one
  #   (a comparison with identical (ignoring the environment) gives FALSE...
  result <- all.equal(runit.dontrunQQPlot2.actual,
                      runit.dontrunQQPlot2.save)
  
  # we check whether the result is TRUE and if not, we write the message
  #   coming from the result
  checkEquals(is.logical(result) && result, TRUE, msg=paste(result, sep="", collapse="\n"))
}


# test for equality of the saved result and the actual result of a dontrun example
test.qqplot3 <- function() {
  # we compute the actual result
  mylist3 <- UnivarMixingDistribution(Unif(0,0.3),Unif(0.6,1),mixCoeff=c(0.8,0.2))
  mylist4 <- UnivarMixingDistribution(Unif(0,0.3),Unif(0.6,1),mixCoeff=c(0.6,0.4))
  
  runit.dontrunQQPlot3.actual <- qqplot(mylist3,mylist4,nosym.pCI=TRUE)
  
  # we load the stored result
  #   we assume that this test is called from within the script in the upper directory
  load("unitTests/runit.dontrunQQPlot3.save")
  
  # we compare the stored result with the calculated one
  #   (a comparison with identical (ignoring the environment) gives FALSE...
  result <- all.equal(runit.dontrunQQPlot3.actual,
                      runit.dontrunQQPlot3.save)
  
  # we check whether the result is TRUE and if not, we write the message
  #   coming from the result
  checkEquals(is.logical(result) && result, TRUE, msg=paste(result, sep="", collapse="\n"))
}

###########################################################
## Demo: Convolution of normal distributions   
###########################################################
require(distr)
options("newDevice"=TRUE)

## initialize two normal distributions
A <- Norm(mean=1, sd=2)
B <- Norm(mean=4, sd=3) 

## convolution via addition of moments
AB <- A+B

## casting of A,B as absolutely continuous distributions
## that is, ``forget'' that A,B are normal distributions
A1 <- as(A, "AbscontDistribution")
B1 <- as(B, "AbscontDistribution")

## for higher precision we change the global variable
## "TruncQuantile" from 1e-5 to 1e-8
oldeps <- getdistrOption("TruncQuantile")
eps <- 1e-8
distroptions("TruncQuantile" = eps)
## support of A1+B1 for FFT convolution is
## [q.l(A1)(TruncQuantile),
##  q.l(B1)(TruncQuantile, lower.tail = FALSE)]

## convolution via FFT
AB1 <- A1+B1

#############################
## plots of the results
#############################
par(mfrow=c(1,3))
low <- q.l(AB)(1e-15)
upp <- q.l(AB)(1e-15, lower.tail = FALSE)
x <- seq(from = low, to = upp, length = 10000)

## densities
plot(x, d(AB)(x), type = "l", lwd = 5)
lines(x , d(AB1)(x), col = "orange", lwd = 1)
title("Densities")
legend("topleft", legend=c("exact", "FFT"), 
        fill=c("black", "orange"))

## cdfs
plot(x, p(AB)(x), type = "l", lwd = 5)
lines(x , p(AB1)(x), col = "orange", lwd = 1)
title("Cumulative distribution functions")
legend("topleft", legend=c("exact", "FFT"), 
        fill=c("black", "orange"))

## quantile functions
x <- seq(from = eps, to = 1-eps, length = 1000)
plot(x, q.l(AB)(x), type = "l", lwd = 5)
lines(x , q.l(AB1)(x), col = "orange", lwd = 1)
title("Quantile functions")
legend(0, q.l(AB)(eps, lower.tail = FALSE),
       legend = c("exact", "FFT"), 
        fill = c("black", "orange"))

## Since the plots of the results show no 
## recognizable differencies, we also compute 
## the total variation distance of the densities 
## and the Kolmogorov distance of the cdfs

## total variation distance of densities
total.var <- function(z, N1, N2){
    0.5*abs(d(N1)(z) - d(N2)(z))
}
dv <- integrate(f = total.var, lower = -Inf, 
                upper = Inf, rel.tol = 1e-8, 
                N1 = AB, N2 = AB1)
cat("Total variation distance of densities:\t")
print(dv) # 4.25e-07

### meanwhile realized in package "distrEx" 
### as TotalVarDist(N1,N2)

## Kolmogorov distance of cdfs 
## the distance is evaluated on a random grid
z <- r(Unif(Min=low, Max=upp))(1e5)
dk <- max(abs(p(AB)(z)-p(AB1)(z)))
cat("Kolmogorov distance of cdfs:\t", dk, "\n") 
# 2.03e-07

### meanwhile realized in package "distrEx" 
### as KolmogorovDist(N1,N2)

## old distroptions
distroptions("TruncQuantile" = oldeps)

##########################################################
## Demo: n-fold convolution of absolutely continuous            
##       probability distributions
##########################################################
require(distr)
options("newDevice"=TRUE)

### from version 1.9 of distr on available in the package
### already

if(!isGeneric("convpow")) 
    setGeneric("convpow", 
    function(D1, ...) standardGeneric("convpow"))

##########################################################
## Function for n-fold convolution
## -- absolute continuous distribution -- 
##########################################################

##implentation of Algorithm 3.4. of
#P. Ruckdeschel, M. Kohl (2014): General Purpose Convolution Algorithm for
# Distributions in S4 Classes by Means of FFT. J. Statist. Softw. 59(4), 1-25.


setMethod("convpow",
          signature(D1 = "AbscontDistribution"),
          function(D1, N){
            if((N<1)||(abs(floor(N)-N)>.Machine$double.eps))
              stop("N has to be a natural greater than 0")
            
            m <- getdistrOption("DefaultNrFFTGridPointsExponent")

    ##STEP 1

            lower <- ifelse((q.l(D1)(0) > - Inf), q.l(D1)(0),
                             q.l(D1)(getdistrOption("TruncQuantile"))
                           ) 
            upper <- ifelse((q.l(D1)(1) < Inf), q.l(D1)(1),
                             q.l(D1)(getdistrOption("TruncQuantile"),
                                   lower.tail = FALSE)
                           )

    ##STEP 2

            M <- 2^m
            h <- (upper-lower)/M
            if(h > 0.01)
              warning(paste("Grid for approxfun too wide,", 
                      "increase DefaultNrFFTGridPointsExponent"))
            x <- seq(from = lower, to = upper, by = h)
            p1 <- p(D1)(x)

    ##STEP 3

            p1 <- p1[2:(M + 1)] - p1[1:M]

    ##STEP 4
    
            ## computation of DFT
            pn <- c(p1, numeric((N-1)*M))
            fftpn <- fft(pn)

    ##STEP 5

            ## convolution theorem for DFTs
            pn <- Re(fft(fftpn^N, inverse = TRUE)) / (N*M)
            pn <- (abs(pn) >= .Machine$double.eps)*pn
            i.max <- N*M-(N-2)
            pn <- c(0,pn[1:i.max])
            dn <- pn / h
            pn <- cumsum(pn)

    ##STEP 6(density)

            ## density 
            x <- seq(from = N*lower+N/2*h, 
                     to = N*upper-N/2*h, by=h)            
            x <- c(x[1]-h,x[1],x+h)
            dnfun1 <- approxfun(x = x, y = dn, yleft = 0, yright = 0)

    ##STEP 7(density)
 
            standardizer <- sum(dn) - (dn[1]+dn[i.max+1]) / 2
            dnfun2 <- function(x) dnfun1(x) / standardizer

    ##STEP 6(cdf)
    
            ## cdf with continuity correction h/2
            pnfun1 <- approxfun(x = x+0.5*h, y = pn, 
                                yleft = 0, yright = pn[i.max+1])

    ##STEP 7(cdf)
   
            pnfun2 <- function(x) pnfun1(x) / pn[i.max+1]


            ## quantile with continuity correction h/2
            yleft <- ifelse(((q.l(D1)(0) == -Inf)|
                             (q.l(D1)(0) == -Inf)), -Inf, N*lower)
            yright <- ifelse(((q.l(D1)(1) == Inf)|
                              (q.l(D1)(1) == Inf)), Inf, N*upper)
            w0 <- options("warn")
            options(warn = -1)
            qnfun1 <- approxfun(x = pnfun2(x+0.5*h), y = x+0.5*h, 
                                yleft = yleft, yright = yright)
            qnfun2 <- function(x){ 
            ind1 <- (x == 0)*(1:length(x))
            ind2 <- (x == 1)*(1:length(x))
            y <- qnfun1(x)
            y <- replace(y, ind1[ind1 != 0], yleft)
            y <- replace(y, ind2[ind2 != 0], yright)
            return(y)
            }
            options(w0)

            rnew = function(N) apply(matrix(r(e1)(n*N), ncol=N), 1, sum)

            return(new("AbscontDistribution", r = rnew, d = dnfun1, p = pnfun2, 
                        q = qnfun2, .withArith = TRUE,
                     .withSim = FALSE))
})

## initialize a normal distribution
A <- Norm(mean=0, sd=1)

## convolution power
N <- 10 

## convolution via FFT 
AN <- convpow(as(A,"AbscontDistribution"), N)
##  ... for the normal distribution , 'convpow' has an "exact"
##      method by version 1.9 so the as(.,.)  is needed to
##      see how the algorithm above works

## convolution exact
AN1 <- Norm(mean=0, sd=sqrt(N))

## plots of the results
eps <- getdistrOption("TruncQuantile")
par(mfrow=c(1,3))
low <- q.l(AN1)(eps)
upp <- q.l(AN1)(eps, lower.tail = FALSE)
x <- seq(from = low, to = upp, length = 10000)

## densities
plot(x, d(AN1)(x), type = "l", lwd = 5)
lines(x , d(AN)(x), col = "orange", lwd = 1)
title("Densities")
legend("topleft", legend=c("exact", "FFT"), 
        fill=c("black", "orange"))

## cdfs
plot(x, p(AN1)(x), type = "l", lwd = 5)
lines(x , p(AN)(x), col = "orange", lwd = 1)
title("Cumulative distribution functions")
legend("topleft", legend=c("exact", "FFT"), 
        fill=c("black", "orange"))

## quantile functions
x <- seq(from = eps, to = 1-eps, length = 1000)
plot(x, q.l(AN1)(x), type = "l", lwd = 5)
lines(x , q.l(AN)(x), col = "orange", lwd = 1)
title("Quantile functions")
legend("topleft",
       legend = c("exact", "FFT"), 
        fill = c("black", "orange"))

require(distr)
options("newDevice"=TRUE)

## argument names conformal to use in distrEx

if(!isGeneric("Range"))
    setGeneric("Range",
    function(object, ...) standardGeneric("Range"))
    

setMethod("Range",
          signature(object = "AbscontDistribution",
          e2 = "numeric"),
          function(object, e2){
            if ((e2 <= 0) || !isTRUE(all.equal(e2,floor(e2))))
               stop("second argument needs to be a positive natural")

            ## new random number function
            rnew <- function(n){
              rn1 <- matrix(r(object)(n*e2),n,e2)
              m <- apply(rn1,1,min)
              M <- apply(rn1,1,max)
              M-m
            }

            fnt <- function(u0,s) d(object)(s)*(p(object)(s+u0)-p(object)(s))^(e2-1)*e2
            fu <- function(u) integrate(fnt, lower=-Inf, upper=Inf, u0=u)$value*(u>0)
            fnt0 <- function(u0,s) d(object)(s)*d(object)(s+u0)*(p(object)(s+u0)-p(object)(s))^(e2-2)*e2*(e2-1)
            fu0 <- function(u) integrate(fnt0, lower=-Inf, upper=Inf, u0=u)$value*(u>0)
            xgrid <- seq(0,
                         q.l(object)(1e-6, lower.tail = FALSE)-q.l(object)(1e-6),
                         length = getdistrOption("DefaultNrGridPoints")/10)
            fx <- sapply(xgrid, fu)
            pnew <- approxfun(xgrid, fx, yleft = 0, yright = 1)
            fx0 <- sapply(xgrid, fu0)
            dnew <- approxfun(xgrid, fx0, yleft = 0, yright = 0)

            ## new quantile function
            lower <- q.l(object)(0)
            upper <- q.l(object)(1)

            maxquantile = q.l(object)(1e-6, lower.tail = FALSE)
            minquantile = q.l(object)(1e-6)

            qfun1 <- function(x){
              if(x == 0) return(lower)
              if(x == 1) return(upper)
              fun <- function(t) pnew(t) - x
              uniroot(f = fun,
                  interval = c(0,
                               maxquantile-minquantile))$root
            }
            qfun2 <- function(x)
              sapply(x, qfun1)

            return(new("AbscontDistribution", r = rnew,
                   d = dnew, p = pnew, q = qfun2))
          })

N <- Norm(mean = 0, sd = 1)

R <- Range(N, 5)
plot(R)

cat("Hit <enter> to continue...")
readline()

setMethod("Range",
          signature(object = "DiscreteDistribution",
          e2 = "numeric"),
          function(object, e2){
            if ((e2 <= 0) || !isTRUE(all.equal(e2,floor(e2))))
               stop("second argument needs to be a positive natural")

            supp <- support(object)
            suppnew <- sort(unique(as.vector(outer(supp,supp,"-"))))
            suppnew <- suppnew[suppnew>=0]
            print(suppnew)
            
            ## new random number function
            rnew <- function(n){
              rn1 <- matrix(r(object)(n*e2),n,e2)
              m <- apply(rn1,1,min)
              M <- apply(rn1,1,max)
              M-m
            }

            fnt <- function(u0,s) (p(object)(s+u0)-p(object)(s)+d(object)(s))^e2 -
                                  (p(object)(s+u0)-p(object)(s))^e2
            pnew <- function(x) sapply(x, function(u) sum(fnt(u, s = supp)))*(x>=0)
            dnew <- function(x){
             (pnew(x)-pnew(x-getdistrOption("DistrResolution")*100))*(d(object)(x)>0)*(x>=0)
              }

            cumprob <- pnew(suppnew)
            qnew <- function(x){ suppnew[sum(cumprob<x)+1] }

            return(new("DiscreteDistribution", r = rnew, d = dnew, p = pnew,
                   q = qnew, support = suppnew))

          })

B1 <- Binom(6, 0.5)
R <- Range(B1, 2)
plot(R)

#####################################################
## Demo: Stationary Regressor Distribution of an AR(1)    
##       Process
#####################################################
require(distr)
options("newDevice"=TRUE)

## Approximation of the stationary regressor 
## distribution of an AR(1) process 
##       X_t = phi X_{t-1} + V_t 
## where V_t i.i.d N(0,1) and phi\in(0,1)
## We obtain 
##    X_t = \sum_{j=1}^\infty phi^j V_{t-j}
## i.e., X_t \sim N(0,1/(1-phi^2))
phi <- 0.5

## casting of V as absolutely continuous distributions
## that is, ``forget'' that V is a normal distribution
V <- as(Norm(), "AbscontDistribution")

## for higher precision we change the global variable
## "TruncQuantile" from 1e-5 to 1e-8
oldeps <- getdistrOption("TruncQuantile")
eps <- 1e-8
distroptions("TruncQuantile" = eps)

## Computation of the approximation 
##      H=\sum_{j=1}^n phi^j V_{t-j}
## of the stationary regressor distribution 
## (via convolution using FFT)
H <- V
n <- 15 
## may take some time
for(i in 1:n){Vi <- phi^i*V; H <- H + Vi } 

## the stationary regressor distribution (exact)
X <- Norm(sd=sqrt(1/(1-phi^2)))

#############################
## plots of the results
#############################
par(mfrow=c(1,3))
low <- q.l(X)(1e-15)
upp <- q.l(X)(1e-15, lower.tail = FALSE)
x <- seq(from = low, to = upp, length = 10000)

## densities
plot(x, d(X)(x),type = "l", lwd = 5)
lines(x , d(H)(x), col = "orange", lwd = 1)
title("Densities")
legend("topleft", legend=c("exact", "FFT"), 
        fill=c("black", "orange"))

## cdfs
plot(x, p(X)(x),type = "l", lwd = 5)
lines(x , p(H)(x), col = "orange", lwd = 1)
title("Cumulative distribution functions")
legend("topleft", legend=c("exact", "FFT"), 
        fill=c("black", "orange"))

## quantile functions
x <- seq(from = eps, to = 1-eps, length = 1000)
plot(x, q.l(X)(x),type = "l", lwd = 5)
lines(x , q.l(H)(x), col = "orange", lwd = 1)
title("Quantile functions")
legend( "topleft", 
        legend=c("exact", "FFT"), 
        fill=c("black", "orange"))

## Since the plots of the results show no 
## recognizable differencies, we also compute 
## the total variation distance of the densities 
## and the Kolmogorov distance of the cdfs

## total variation distance of densities
total.var <- function(z, N1, N2){
    0.5*abs(d(N1)(z) - d(N2)(z))
}
dv <- integrate(f = total.var, lower = -Inf, 
                upper = Inf, rel.tol = 1e-5, 
                N1=X, N2=H)
cat("Total variation distance of densities:\t")
print(dv) # 8.0e-06

### meanwhile realized in package "distrEx" 
### as TotalVarDist(N1,N2)


## Kolmogorov distance of cdfs 
## the distance is evaluated on a random grid
z <- r(Unif(Min=low, Max=upp))(1e5)
dk <- max(abs(p(X)(z)-p(H)(z)))
cat("Kolmogorov distance of cdfs:\t", dk, "\n") 
# 3.7e-05

### meanwhile realized in package "distrEx" 
### as KolmogorovDist(N1,N2)


## old distroptions
distroptions("TruncQuantile" = oldeps)

################################
##
## Class: NbinomParameter
##
################################

## Access Methods
setMethod("size", "NbinomParameter", function(object) object@size)
setMethod("prob", "NbinomParameter", function(object) object@prob)
## Replace Methods
setReplaceMethod("size", "NbinomParameter", 
                  function(object, value){ object@size <- value; object})
setReplaceMethod("prob", "NbinomParameter", 
                  function(object, value){ object@prob <- value; object})

setValidity("NbinomParameter", function(object){
  if(length(prob(object)) != 1)
    stop("prob has to be a numeric of length 1")    
  if(prob(object) <= 0)
    stop("prob has to be in (0,1)")
  if(prob(object) >= 1)
    stop("prob has to be in (0,1)")
  if(length(size(object)) != 1)
    stop("size has to be a numeric of length 1")    
  if(size(object) < 0)
    stop("size has to be non-negative")
#  if(!identical(floor(size(object)), size(object)))
#    stop("size has to be a not negative natural")
  else return(TRUE)
})


################################
##
## Class: negative binomial distribution
##
################################

Nbinom <- function(size = 1,prob = 0.5) new("Nbinom", size = size, prob = prob)

## wrapped access methods
setMethod("prob", "Nbinom", function(object) prob(param(object)))
setMethod("size", "Nbinom", function(object) size(param(object)))
## wrapped replace methods
setMethod("prob<-", "Nbinom", 
           function(object, value) 
               new("Nbinom", prob = value, size = size(object)))
setMethod("size<-", "Nbinom", 
           function(object, value) 
               new("Nbinom", prob = prob(object), size = value))

setMethod("+", c("Nbinom","Nbinom"),
          function(e1,e2){
            newsize <- size(e1) + size(e2)
            
            if(isTRUE(all.equal(prob(e1),prob(e2))))    
              return(new("Nbinom", size = newsize, prob = prob(e1), 
                         .withArith = TRUE))           
            return(as(e1, "LatticeDistribution") + e2)
          })

################################################################
# QQ - Plot functions in package distr
################################################################

setMethod("qqplot", signature(x = "UnivariateDistribution",
                              y = "UnivariateDistribution"), function(x, y,
                              n = 30, withIdLine = TRUE, withConf = TRUE,
    withConf.pw  = withConf,  withConf.sim = withConf,
    plot.it = TRUE, xlab = deparse(substitute(x)),
    ylab = deparse(substitute(y)), ...,
    width = 10, height = 5.5, withSweave = getdistrOption("withSweave"),
    mfColRow=TRUE, n.CI = n,
    col.IdL = "red", lty.IdL = 2, lwd.IdL = 2,
    alpha.CI = .95, exact.pCI = (n<100), exact.sCI = (n<100), nosym.pCI = FALSE,
    col.pCI = "orange", lty.pCI = 3, lwd.pCI = 2, pch.pCI = par("pch"), cex.pCI = par("cex"),
    col.sCI = "tomato2", lty.sCI = 4, lwd.sCI = 2, pch.sCI = par("pch"), cex.sCI = par("cex"),
    cex.pch = par("cex"), col.pch = par("col"),
    jit.fac = 0, check.NotInSupport = TRUE,
    col.NotInSupport = "red", with.legend = TRUE, legend.bg = "white",
    legend.pos = "topleft", legend.cex = 0.8, legend.pref = "", 
    legend.postf = "", legend.alpha = alpha.CI, debug = FALSE, withSubst = TRUE){


    args0 <- list(x = x, y = y, n = n, withIdLine = withIdLine,
     withConf = withConf, withConf.pw  = withConf.pw,
     withConf.sim = if(missing(withConf.sim)) {
        if(missing(withConf)) NULL else withConf} else withConf.sim,
     plot.it = plot.it, xlab = xlab, ylab = ylab, width = width,
     height = height, withSweave = withSweave, mfColRow=mfColRow,
     n.CI = if(!missing(n.CI)) n.CI else if(!missing(n)) n else NULL,
     col.IdL = col.IdL, lty.IdL = lty.IdL, lwd.IdL = lwd.IdL,
     alpha.CI = alpha.CI,
     exact.pCI = if(!missing(exact.pCI)) exact.pCI else if(!missing(n)) (n<100) else NULL,
     exact.sCI = if(!missing(exact.pCI)) exact.sCI else if(!missing(n)) (n<100) else NULL,
     nosym.pCI = nosym.pCI, col.pCI = col.pCI, lty.pCI = lty.pCI,
     lwd.pCI = lwd.pCI, pch.pCI = pch.pCI, cex.pCI = cex.pCI, col.sCI = col.sCI,
     lty.sCI = lty.sCI, lwd.sCI = lwd.sCI, pch.sCI = pch.sCI, cex.sCI = cex.sCI,
     cex.pch = cex.pch, col.pch = col.pch, jit.fac = jit.fac,
     check.NotInSupport = check.NotInSupport,
     col.NotInSupport = col.NotInSupport, with.legend = with.legend,
     legend.bg = legend.bg, legend.pos = legend.pos, legend.cex = legend.cex,
     legend.pref = legend.pref, legend.postf = legend.postf,
     legend.alpha = if(!missing(legend.alpha)) legend.alpha else if(!missing(alpha.CI)) alpha.CI else NULL,
     debug = debug, withSubst = withSubst)



    mc <- match.call(call = sys.call(sys.parent(1)))
    dots <- match.call(call = sys.call(sys.parent(1)), expand.dots = FALSE)$"..."

    plotInfo <- list(call = mc, dots=dots, args=args0)


    xcc <- as.character(deparse(mc$x))
    ycc <- as.character(deparse(mc$y))
    if(missing(xlab)) mc$xlab <- xcc
    if(missing(ylab)) mc$ylab <- ycc

    mcl <- as.list(mc)[-1]
    mcl$withSweave <- NULL
    mcl$mfColRow <- NULL
    mcl$debug <- NULL

   .mpresubs <- if(withSubst){
                 function(inx) 
                    .presubs(inx, c("%C", "%A", "%D" ),
                          c(as.character(class(x)[1]), 
                            as.character(date()), 
                            xcc))
                }else function(inx) inx

    force(x)

    pp <- ppoints(n)
    xc <- q.l(x)(pp)
    yc <- q.l(y)(pp)

    col.pch <- rep(col.pch,length.out=n)

    if(check.NotInSupport){
       xco <- sort(xc)
       nInSupp <- .NotInSupport(xc,y)
       if(length(nInSupp)){
          col.pch[nInSupp] <- col.NotInSupport
       }
    }


    oxc <- 1:length(xc)
    xc.o <- xc
    yc.o <- yc
    ord.x <- order(xc)

    if("support" %in% names(getSlots(class(x)))){
       xc <- jitter(xc, factor=jit.fac)
       oxc <- order(xc)
       xc <- xc[oxc]
       }

    if("support" %in% names(getSlots(class(y))))
       yc <- sort(jitter(yc, factor=jit.fac))

    mcl$x <- xc
    mcl$y <- yc

    mcl <- .deleteItemsMCL(mcl)
    mcl$cex <- .makeLenAndOrder(cex.pch,ord.x)
    mcl$col <- .makeLenAndOrder(col.pch,ord.x)

    mcl$xlab <- .mpresubs(mcl$xlab)
    mcl$ylab <- .mpresubs(mcl$ylab)

    if (!is.null(eval(mcl$main)))
        mcl$main <- .mpresubs(eval(mcl$main))
    if (!is.null(eval(mcl$sub)))
        mcl$sub <- .mpresubs(eval(mcl$sub))


    if (!withSweave){
           devNew(width = width, height = height)
    }
    opar <- par("mfrow", no.readonly = TRUE)
    if(mfColRow) on.exit(do.call(par, list(mfrow=opar)))
    if(mfColRow) opar1 <- par(mfrow = c(1,1), no.readonly = TRUE)

    ret <- do.call(stats::qqplot, args=mcl)
    qq.usr <- par("usr")
    qqb <- NULL
    if(withIdLine){
       if(plot.it)abline(0,1,col=col.IdL,lty=lty.IdL,lwd=lwd.IdL)
       if(#is(y,"AbscontDistribution") &&
       withConf){
          xy <- unique(sort(c(xc.o,yc.o)))
          xy <- xy[!.NotInSupport(xy,y)]
          lxy <- length(xy)
          if(is(y,"DiscreteDistribution")){
             n0 <- min(n.CI, length(support(y)))
             n1 <- max(n0-lxy,0)
             if (n1 >0 ){
                 notyetInXY <- setdiff(support(y), xy)
                 xy0 <- sample(notyetInXY, n1)
                 xy <- sort(c(xy,xy0))
             }
          }else{
             if(lxy < n.CI){
                n1 <- (n.CI-lxy)%/%3
                xy0 <- seq(min(xy),max(xy),length=n1)
                xy1 <- r(y)(n.CI-lxy-n1)
                xy <- sort(c(xy,xy0,xy1))
             }
          }
        qqplotInfo <- list(xy.0=xy, y.0=y, 
                         withConf.pw=withConf.pw, 
                         withConf.sim=withConf.sim, 
                         alpha.CI=alpha.CI ,
                         col.pCI = col.pCI , lty.pCI = lty.pCI , 
                         lwd.pCI = lwd.pCI , pch.pCI = pch.pCI, 
                         cex.pCI = cex.pCI , 
                         col.sCI = col.sCI , lty.sCI = lty.sCI , 
                         lwd.sCI = lwd.sCI , pch.sCI = pch.sCI, 
                         cex.sCI = cex.sCI , 
                         n = n , 
                         exact.sCI = exact.sCI, exact.pCI = exact.pCI,
                  nosym.pCI = nosym.pCI, with.legend = with.legend,
                  legend.bg = legend.bg, legend.pos = legend.pos,
                  legend.cex = legend.cex, legend.pref = legend.pref,
                  legend.postf = legend.postf, legend.alpha = legend.alpha, debug = debug,
                  args.stats.qqplot = mcl
                  )
       if(plot.it){
           qqb <- .confqq(xy, y, datax=TRUE, withConf.pw, withConf.sim, alpha.CI,
                      col.pCI, lty.pCI, lwd.pCI, pch.pCI, cex.pCI,
                      col.sCI, lty.sCI, lwd.sCI, pch.sCI, cex.sCI,
                  n, exact.sCI = exact.sCI, exact.pCI = exact.pCI,
                  nosym.pCI = nosym.pCI, with.legend = with.legend,
                  legend.bg = legend.bg, legend.pos = legend.pos,
                  legend.cex = legend.cex, legend.pref = legend.pref,
                  legend.postf = legend.postf, legend.alpha = legend.alpha,
                  debug = debug)
          }else{
           qqb <- qqbounds(sort(unique(xy)),y,alpha.CI,n,withConf.pw, withConf.sim,
                   exact.sCI,exact.pCI,nosym.pCI,debug)
          }
       }
    }
    qqplotInfo <- c(plotInfo, ret=ret, usr=qq.usr, qqplotInfo=qqplotInfo, qqb=qqb)
    class(qqplotInfo) <- c("qqplotInfo","DiagnInfo")
    return(invisible(qqplotInfo))
    })
    

##setMethod("+", c("AbscontDistribution","DiscreteDistribution"),
##          function(e1,e2){               
##            rfun = function(n) r(e1)(n) + r(e2)(n)
##            AbscontDistribution( r = rfun)
##          })


##setMethod("+", c("DiscreteDistribution","AbscontDistribution"),
##          function(e1,e2){
##            rfun = function(n) r(e1)(n) + r(e2)(n)
##            AbscontDistribution(r = rfun)
##          })




setMethod("+", c("numeric", "UnivariateDistribution"),
          function(e1, e2){
            e2 + e1
          })

setMethod("-", c("numeric", "UnivariateDistribution"),
          function(e1, e2){
            -1*e2 + e1
          })

setMethod("*", c("numeric", "UnivariateDistribution"),
          function(e1, e2){
            e2 * e1
          })

setMethod("-", c("UnivariateDistribution", "UnivariateDistribution"),
          function(e1,e2){
            e1+(-e2)
          })
setMethod("-", c("UnivariateDistribution", "missing"),
          function(e1){
            e1*(-1)
          })


setMethod("-", c("UnivariateDistribution", "numeric"),
          function(e1, e2){
            return(e1 + (-e2))
          })

setMethod("/", c("UnivariateDistribution", "numeric"),
          function(e1, e2){
            if(e2 == 0) stop("division by zero")
            return(e1 * (1/e2))
          })


setMethod("+", c("AbscontDistribution", "DiscreteDistribution"),
          function(e1,e2){
            return(e2 + e1)
          })

setMethod("+", c("LatticeDistribution", "DiscreteDistribution"),
          function(e1,e2){
            return(e2 + as(e1,"DiscreteDistribution"))
          })


setMethod("+", c("DiscreteDistribution","AbscontDistribution"),
function(e1,e2){
     rfun <- function(n){}
     body(rfun) <- substitute({ f(n) + g(n) },
                                list(f = e1@r, g = e2@r))
     grid <- support(e1)
     probab <- d(e1)(grid)

     lower1 <- getLow(e1)
     upper1 <- getUp(e1)
     lower2 <- getLow(e2)
     upper2 <- getUp(e2)

     lower <- lower1 + lower2
     upper <- upper1 + upper2
     x <- seq(from = lower, to = upper,
              length = getdistrOption("DefaultNrGridPoints"))
     h <- x[2]-x[1]
           ### to avoid double accounting for density boundary 
           ### points we jitter a little:
     dpn <- outer(x+rnorm(x)*sd(x)*getdistrOption("DistrResolution"), 
                  grid, function(x,y) d(e2)(x - y)) %*% probab
     
     ### treatment if density of e2 has singularities
     if(any(idx <- (dpn*h >= 10*length(dpn)))){
        Lx <- length(dpn)
        idx <- seq(Lx)[idx]
        Li <- length(idx)
        xx <- x[idx]
        xxp <- ifelse(upper2-xx < .Machine$double.eps, xx-h/4 , xx+h/2)
        xxm <- ifelse(xx-lower2 < .Machine$double.eps, xx+h/4 , xx-h/2)
        xx0 <- sort(c(xxm,xxp))
        jdx <- 2*seq(Li)-1
        pn <- outer(xx0, grid, function(x,y) p(e2)(x - y)) %*% probab/(xxp-xxm)
        dpn[idx] <- diff(pn)[jdx]
     }

     dfun <- .makeDNew(x, dpn, h=1, standM="int" )

     if(is(e2,"Chisq")){
        pfun <- .makePNew(x, dpn, h,
                          .notwithLArg(e1)||.notwithLArg(e2))
     }else{
#
     pl <- outer(x, grid, function(x,y) p(e2)(x - y)) %*% probab
     pu <- if (.inArgs("lower.tail", p(e2)))
               outer(x, grid,
                     function(x,y) p(e2)(x - y, lower.tail = FALSE)) %*% probab
           else
               outer(x, grid, function(x,y) 1-p(e2)(x - y)) %*% probab
     ## cdf (steps 5--7)
     pfun <- .makePNew(x, NULL, h,
                        .notwithLArg(e1)||.notwithLArg(e2), pxl = pl , pxu = pu)
     }
     ## quantile function
     yL <-  if ((q.l(e1)(0) == -Inf)||(q.l(e2)(0) == -Inf))
          -Inf else lower
     yR <-  if ((q.l(e1)(1) ==  Inf)||(q.l(e2)(1) ==  Inf))
           Inf else upper
     
     ## contintuity correction
     px.l <- pfun(x + 0.5*h)
     px.u <- pfun(x + 0.5*h, lower.tail = FALSE)

#     print(summary(x))
#     print(summary(px.l))
#     print(summary(px.u))
     qfun <- .makeQNew(x + 0.5*h, px.l, px.u,
                       .notwithLArg(e1)||.notwithLArg(e1), yL, yR)

     object <- AbscontDistribution( r = rfun, d = dfun, p = pfun,
                    q = qfun, .withSim = FALSE, .withArith = TRUE)

     if(is(e1@Symmetry,"SphericalSymmetry")&& 
        is(e2@Symmetry,"SphericalSymmetry"))
           object@Symmetry <- SphericalSymmetry(SymmCenter(e1@Symmetry)+
                                                 SymmCenter(e2@Symmetry))   
     object
     }) 


setMethod("+", c("numeric", "LatticeDistribution"),
          function(e1, e2){
            e2 + e1
          })

setMethod("-", c("numeric", "LatticeDistribution"),
          function(e1, e2){
            -1*e2 + e1
          })

setMethod("*", c("numeric", "LatticeDistribution"),
          function(e1, e2){
            e2 * e1
          })

setMethod("-", c("LatticeDistribution", "LatticeDistribution"),
          function(e1,e2){
            e1+(-e2)
          })

setMethod("-", c("LatticeDistribution", "missing"),
          function(e1){
            e1*(-1)
          })


setMethod("-", c("LatticeDistribution", "numeric"),
          function(e1, e2){
            return(e1 + (-e2))
          })

setMethod("/", c("LatticeDistribution", "numeric"),
          function(e1, e2){
            if(e2 == 0) stop("division by zero")
            return(e1 * (1/e2))
          })
setMethod("-", c("UnivariateDistribution", "LatticeDistribution"),
          function(e1,e2){
            e1+(-e2)
          })
setMethod("-", c("LatticeDistribution", "UnivariateDistribution"),
          function(e1,e2){
            e1+(-e2)
          })

################################
##
## Class: ExpParameter
##
################################

## Access methods
setMethod("rate", "ExpParameter", function(object) object@rate)
## Replace methods
setReplaceMethod("rate", "ExpParameter", 
        function(object, value){ object@rate <- value; object})


setValidity("ExpParameter", function(object){
  if(length(rate(object)) != 1)
    stop("rate has to be a numeric of length 1")    
  if(rate(object) <= 0)
    stop("rate has to be positive")
  else return(TRUE) })


################################
##
## Class: exponential distribution
##
################################

Exp <- function(rate = 1) new("Exp", rate = rate)

## wrapped access methods
setMethod("rate", "Exp", function(object) rate(param(object)))

## wrapped replace methods
setMethod("rate<-", "Exp", function(object, value) new("Exp", rate = value))

setMethod("*", c("Exp","numeric"),
       function(e1, e2){
         if (length(e2)>1) stop("length of operator must be 1")
         if(isTRUE(all.equal(e2,0)))  
            return(new("Dirac", location = 0, .withArith = TRUE))
         if(e2 > 0) return(new("Exp", rate = rate(e1) / e2, .withArith = TRUE))
         return(-1 * as(Exp(rate = rate(e1) / abs(e2)), "AbscontDistribution"))
       })

setMethod("shape", "Exp", function(object)1)

### defines the inverse function of digamma called igamma for simplicity

## an extensive grid of x-values
.xg <- sort(c(10^(-70:-1),qexp(unique(pmin(seq(0,1,length=5e3)+1e-10,1-1e-10))),qcauchy(seq(0.999,1-1e-10,length=5e3))))
.dxg <- digamma(.xg)
igamma <- approxfun(.dxg,.xg)
rm(.xg,.dxg)

################################
##
## Class: LogisParameter
##
################################
## Access Methods
setMethod("location", "LogisParameter", function(object) object@location)
setMethod("scale", "LogisParameter", 
           function(x, center = TRUE, scale = TRUE) x@scale)
## Replace Methods
setReplaceMethod("location", "LogisParameter", 
                  function(object, value){ object@location <- value; object})
setReplaceMethod("scale", "LogisParameter", 
                  function(object, value){ object@scale <- value; object})

setValidity("LogisParameter", function(object){
  if(length(location(object)) != 1)
    stop("location has to be a numeric of length 1")    
  if(length(scale(object)) != 1)
    stop("scale has to be a numeric of length 1")    
  if(scale(object) <= 0)
    stop("scale has to be positive")
  else return(TRUE)
})


################################
##
## Class: logistic distribution
##
################################

Logis <- function(location = 0, scale = 1) 
         new("Logis", location = location, scale = scale)

## wrapped access methods
setMethod("location", "Logis", function(object) location(param(object)))
setMethod("scale", "Logis", 
           function(x, center = TRUE, scale = TRUE) scale(param(x)))

## wrapped replace methods
setMethod("location<-", "Logis", function(object, value) 
           new("Logis", location = value, scale = scale(object)))
setMethod("scale<-", "Logis", function(object, value) 
           new("Logis", location = location(object), scale = value))

## extra Methoden for Logistic distribution
setMethod("+", c("Logis","numeric"),
          function(e1, e2){
            if (length(e2)>1) stop("length of operator must be 1")
            new("Logis", location = location(e1) + e2, 
                 scale = scale(e1), .withArith = TRUE) 
          })

setMethod("*", c("Logis","numeric"),
          function(e1, e2){
            if (length(e2)>1) stop("length of operator must be 1")
            if(isTRUE(all.equal(e2,0)))  
               return(new("Dirac", location = 0, .withArith = TRUE))
            new("Logis", location = location(e1) * e2, 
                 scale = scale(e1) * abs(e2), .withArith = TRUE)
          })

.OkTyp.flat.LCD <-  c("DiscreteDistribution", "AbscontDistribution",
                      "UnivarLebDecDistribution", "UnivarMixingDistribution")

flat.LCD <- function(..., mixCoeff = NULL, withgaps = getdistrOption("withgaps")){
    ldots <- list(...)
    l <- length(ldots)
    ep <- getdistrOption("TruncQuantile")
    if(is.null(mixCoeff))
       mixCoeff <- rep(1,l)/l
    else{ if (l!=length(mixCoeff))
          stop("argument 'mixCoeff' and the mixing distributions must have the same length")
          if(any(mixCoeff < -ep) || sum(mixCoeff)>1+ep)
             stop("mixing coefficients are no probabilities")
        }
    if(!all(as.logical(lapply(ldots, function(x)is(x,"UnivarLebDecDistribution")))))
       stop("all list elements must be of class 'UnivarLebDecDistribution'")

    if(any(mixCoeff > (1-getdistrOption("TruncQuantile"))))
         return(ldots[[which.max(mixCoeff)]])
        
    ep <- getdistrOption("TruncQuantile")
    
    ldots <- ldots[mixCoeff >ep]
    l <- length(ldots)
    mixCoeff <- mixCoeff[mixCoeff >ep]
        
    mixDistr.c <- lapply(ldots, function(x)acPart(x))
    mixDistr.d <- lapply(ldots, function(x)discretePart(x))

    mixCoeff0.c <-  as.vector(unlist(lapply(ldots, function(x)
                        acWeight(x))))* mixCoeff
    mixCoeff0.d <-  as.vector(unlist(lapply(ldots, function(x)
                        discreteWeight(x))))* mixCoeff

    w.c <- sum(mixCoeff0.c)
    w.d <- sum(mixCoeff0.d)
    w.c <- w.c/(w.c+w.d)
    w.d <- 1-w.c

    mixCoeff.c <- mixCoeff0.c/(w.c+(w.c==0))
    mixCoeff.d <- mixCoeff0.d/(w.d+(w.d==0))

    mixDistr.c <- mixDistr.c[mixCoeff.c >ep]
    mixCoeff.c <- mixCoeff.c[mixCoeff.c >ep]
    l.c <- length(mixDistr.c)
    
    mixDistr.d <- mixDistr.d[mixCoeff.d >ep]
    mixCoeff.d <- mixCoeff.d[mixCoeff.d >ep]
    l.d <- length(mixDistr.d)
    finSupport <- c(TRUE,TRUE)
    if(l.d>0){
       mixDistr.dfs <- sapply(mixDistr.d, function(x) x@.finSupport)
       finSupport <- apply(mixDistr.dfs,1,all)
    }
    if(l.c){
    rnew.c <- .rmixfun(mixDistr = mixDistr.c, mixCoeff = mixCoeff.c)
    pnew.c <- .pmixfun(mixDistr = mixDistr.c, mixCoeff = mixCoeff.c)
    dnew.c <- .dmixfun(mixDistr = mixDistr.c, mixCoeff = mixCoeff.c)
    qnew.c <- .qmixfun(mixDistr = mixDistr.c, mixCoeff = mixCoeff.c,
                       Cont = TRUE, pnew = pnew.c)
    .withSim   <- any(as.logical(lapply(ldots, function(x) x@.withSim)))    
    f.c <- AbscontDistribution( r = rnew.c, d = dnew.c, p = pnew.c,
                q = qnew.c, 
                .withSim = .withSim, .withArith = TRUE)
    if(withgaps && is.null(gaps(f.c))) setgaps(f.c)
    }
    else f.c <- Norm()            

    if(l.d){
    .withSim   <- any(as.logical(lapply(ldots, function(x) x@.withSim)))

    suppList <- lapply(mixDistr.d, function(x) x@support)
    supp <- unique(sort(as.vector(unlist(suppList))))

    dnew.d <- .dmixfun(mixDistr = mixDistr.d, mixCoeff = mixCoeff.d, 
                       withStand = TRUE, supp = supp)

    
    f.d <- if(sum(dnew.d(supp))<1-getdistrOption("TruncQuantile"))
              Dirac(0) else
           {if (length(supp)==1) Dirac(supp)
            else DiscreteDistribution(supp = supp, prob = dnew.d(supp),
                           .withSim = FALSE, .withArith = TRUE)}
    }else f.d <- Dirac(0)            

    f.d@.finSupport <- finSupport

    UnivarLebDecDistribution(
                     discretePart = f.d, acPart = f.c,
                     discreteWeight = w.d, acWeight = w.c)
}

flat.mix <- function(object){
    mixDistr <- object@mixDistr
    mixCoeff <- object@mixCoeff
    isOkTyp <- function(x) any(as.logical(lapply(.OkTyp.flat.LCD, 
                                           function(y) is(x,y))))
    if(!all(as.logical(lapply(mixDistr, isOkTyp))))
       stop(gettextf("all list elements must be of one of the following classes\n"),
            paste("'",.OkTyp.flat.LCD,"'", sep ="", collapse=", "))
    mixDistr2 <- mixDistr
    for(i in seq(length(mixDistr)))
        {if ( is(mixDistr[[i]],"UnivarMixingDistribution") &&
             !is(mixDistr[[i]],"UnivarLebDecDistribution"))
             mixDistr2[[i]] <- flat.mix(mixDistr[[i]])
         else mixDistr2[[i]] <- as(mixDistr[[i]],"UnivarLebDecDistribution")
         }
    erg <- do.call(flat.LCD, c(mixDistr2, alist(mixCoeff = mixCoeff)))
    simplifyD(erg)
}

################################
##
## Class: Distribution
##
################################

## Access Methods
setMethod("img", "Distribution", function(object) object@img)
setMethod("param", "Distribution", function(object) object@param)
setMethod("r", "Distribution", function(object) object@r)
setMethod("d", "Distribution", function(object) object@d)
setMethod("p", "Distribution", function(object) object@p)
setMethod("p.r", "Distribution", function(object) object@p)
setMethod("q", "Distribution", 
           function(save = "default", status = 0, runLast = TRUE) save@q)
           ### odd arg-list due to existing function in base package 
setMethod("q.l", "Distribution", 
           function(object) object@q)

setMethod("p", "Distribution", function(object) object@p)
setMethod(".lowerExact", "Distribution", function(object){ 
             er <- is(try(slot(object, ".lowerExact"), silent = TRUE), "try-error")
             if(er){ object0 <- conv2NewVersion(object)
                     objN <- paste(substitute(object))
                     warning(gettextf("'%s' was generated in an old version of this class.\n",
                                     objN),
                            gettextf("'%s' has been converted to the new version",objN),
                            gettextf(" of this class by a call to 'conv2NewVersion'.\n")
                            )           
                    eval.parent(substitute(object<-object0))                    
                    return(object0@.lowerExact)}
             object@.lowerExact})
setMethod(".logExact", "Distribution", function(object){
             er <- is(try(slot(object, ".logExact"), silent = TRUE), "try-error")
             if(er){ object0 <- conv2NewVersion(object)
                     objN <- paste(substitute(object))
                     warning(gettextf("'%s' was generated in an old version of this class.\n",
                                     objN),
                            gettextf("'%s' has been converted to the new version",objN),
                            gettextf(" of this class by a call to 'conv2NewVersion'.\n")
                            )           
                    eval.parent(substitute(object<-object0))
                    return(object0@.logExact)}
             object@.logExact})
setMethod("Symmetry", "Distribution", function(object){
             er <- is(try(slot(object, "Symmetry"), silent = TRUE), "try-error")
             if(er){ object0 <- conv2NewVersion(object)
                     objN <- paste(substitute(object))
                     warning(gettextf("'%s' was generated in an old version of this class.\n",
                                     objN),
                            gettextf("'%s' has been converted to the new version",objN),
                            gettextf(" of this class by a call to 'conv2NewVersion'.\n")
                            )           
                    eval.parent(substitute(object<-object0))
                    return(object0@Symmetry)}
             object@Symmetry})


################################
##
## Class: UnivariateDistribution
##
################################


setMethod("dim", "UnivariateDistribution", function(x)1)

################################
##
## Class: Parameter
##
################################


## Access Methoden
setMethod("name", "Parameter", function(object) object@name)
## Replace Methoden
setReplaceMethod("name", "Parameter", 
                 function(object, value){ object@name <- value; object})

################################
##
## Class: BinomParameter
##
################################


## Access Methods
setMethod("size", "BinomParameter", function(object) object@size)
setMethod("prob", "BinomParameter", function(object) object@prob)
## Replace Methods
setReplaceMethod("size", "BinomParameter", 
                  function(object, value){ object@size <- value; object})
setReplaceMethod("prob", "BinomParameter", 
                  function(object, value){ object@prob <- value; object})


setValidity("BinomParameter", function(object){
  if(length(prob(object)) != 1)
    stop("prob has to be a numeric of length 1")    
  if(prob(object) < 0)
    stop("prob has to be in [0,1]")
  if(prob(object) > 1)
    stop("prob has to be in [0,1]")
  if(length(size(object)) != 1)
    stop("size has to be a numeric of length 1")    
  if(size(object) < 1)
    stop("size has to be a natural greater than 0")
  if(!identical(floor(size(object)), size(object)))
    stop("size has to be a natural greater than 0")    
  else return(TRUE)
})


################################
##
## Class: binomial distribution
##
################################

Binom <- function(size = 1,prob = 0.5){
   if(length(size)!=1 || length(prob)!=1)
      stop("Arguments 'size' and 'prob' must be of length 1")
   if(!.isInteger(size) || size < 1 )
      stop("Argument 'size' must be a positive integer")
   if(prob < 0  || prob > 1 )
      stop("Argument 'prob' must be in [0,1]")
   if(!.isEqual01(prob)) return(new("Binom", size = size, prob = prob))
   if(prob < 0.1) return(Dirac(0)) else return(Dirac(size))
}

## wrapped access methods
setMethod("prob", "Binom", function(object) prob(param(object)))
setMethod("size", "Binom", function(object) size(param(object)))
## wrapped replace methods
setMethod("prob<-", "Binom", 
           function(object, value) new("Binom", prob = value, 
                                        size = size(object)))
setMethod("size<-", "Binom", 
           function(object, value) new("Binom", prob = prob(object), 
                                        size = value))

## Convolution for two binomial distributions Bin(n1,p1) and Bin(n2,p2)
## Distinguish cases 
## p1 == p2 und p1 != p2


setMethod("+", c("Binom","Binom"),
          function(e1,e2){
            newsize <- size(e1) + size(e2)
            
            if(isTRUE(all.equal(prob(e1),prob(e2))))    
               return(new("Binom", prob = prob(e1), size = newsize, 
                          .withArith = TRUE))
            
            return(as(e1, "LatticeDistribution") + e2)
          })

setMethod("simplifyD", "UnivarMixingDistribution",
           function(object){
                if(length(object@mixDistr)==1)
                   return(object@mixDistr[[1]])
                ep <- getdistrOption("TruncQuantile")
                mixC.old <- mixCoeff(object)
                if(any(mixC.old > (1 - ep)))
                   {
                    object@mixCoeff <- 1
                    object@mixDistr <- mixDistr[mixC.old > (1 - ep)]
                    return(flat.mix(object))
                   }
                if(all(mixC.old< ep))
                   return(flat.mix(object))
                mixC.new <- mixC.old[mixC.old>ep]
                mixC.new <- mixC.new/sum(mixC.new)
                mixD.new <- mixDistr(object)[mixC.old>ep]
                object@mixCoeff <- mixC.new
                object@mixDistr <- new("UnivarDistrList", mixD.new)
                return(flat.mix(object))}
           )

setMethod("simplifyD", "UnivarLebDecDistribution",
           function(object){
                ep <- getdistrOption("TruncQuantile")
                if(acWeight(object)> (1 - ep))
                   return(acPart(object))
                if(acWeight(object)< ep)
                   return(discretePart(object))
                return(object)}
           )

setMethod("simplifyD", "AbscontDistribution", 
           function(object)object)
setMethod("simplifyD", "DiscreteDistribution", 
           function(object)object)

################################
##
## Class: TParameter
##
################################

## Access Methods
setMethod("df", "TParameter", function(x, ...) x@df)
setMethod("ncp", "TParameter", function(object) object@ncp)
## Replace Methods
setReplaceMethod("df", "TParameter", 
                  function(object, value){ object@df <- value; object})
setReplaceMethod("ncp", "TParameter", 
                  function(object, value){ object@ncp <- value; object})


setValidity("TParameter", function(object){
  if(length(df(object)) != 1)
    stop("df has to be a numeric of length 1")    
  if(df(object) <= 0)
    stop("df has to be positive")
  if(length(ncp(object)) != 1)
    stop("ncp has to be a numeric of length 1")      
  else return(TRUE)
})

################################
##
## Class: Student distribution
##
################################

Td <- function(df = 1, ncp = 0) new("Td", df = df, ncp = ncp)

## wrapped access methods
setMethod("df", "Td", function(x, ...) df(param(x)))
setMethod("ncp", "Td", function(object) ncp(param(object)))

## wrapped replace methods
setMethod("df<-", "Td", 
           function(object, value) new("Td", df = value, ncp = ncp(object)))
setMethod("ncp<-", "Td", 
           function(object, value) new("Td", df = df(object), ncp = value))

################################
##
## virtal Class: rSpace
##
################################

## Access methods
setMethod("name", "rSpace", function(object) object@name)
## Replace methods
setReplaceMethod("name", "rSpace", 
                 function(object, value){ object@name <- value; object})

################################
##
## Class: EuclideanSpace
##
################################

EuclideanSpace <- function(dimension = 1){ 
    new("EuclideanSpace", dimension = dimension)
}

## Access methods
setMethod("dimension", "EuclideanSpace", function(object) object@dimension)
## Replace methods
setReplaceMethod("dimension", "EuclideanSpace", 
                 function(object, value){ object@dimension <- value; object})

## Initialize method
setMethod("initialize", "EuclideanSpace",
          function(.Object, dimension = 1) {
            .Object@dimension <-  dimension
            .Object@name <- gettext("Euclidean Space")
            validObject(.Object)
            .Object
          })



setValidity("EuclideanSpace", function(object){
  if(dimension(object) < 1)
    stop("dimension has to be a natural greater than 0")
  if(!identical(floor(dimension(object)), dimension(object)))
    stop("dimension has to be a natural greater than 0")    
})



## extra methods

setMethod("liesIn", signature(object = "EuclideanSpace", x = "numeric"), 
          function(object, x){
            if(dimension(object) ==  length(x))
              return(TRUE)
            return(FALSE)
          })


Reals <- function(){ new("Reals") }

################################
##
## Class: Naturals
##
################################

Naturals <- function(){ new("Naturals") }

setMethod("liesIn", signature(object = "Naturals", x = "numeric"), 
          function(object, x){
            if(length(x) !=  1)
              return(FALSE)
            if(x <= 0)
              return(FALSE)
            if(!identical(floor(x), x+ 0.0))
              return(FALSE)
            else return(TRUE)
          })


################################
##
## Class: Lattice
##
################################


Lattice <- function(pivot = 0, width = 1, Length = 2, name = "a lattice")
   {
   L <- Length; w <- width; p <- pivot
   if(!identical(L+0.0,floor(L)) || length(L)>1 || L<1)
       stop("Length must be a positive integer")  ### may be Inf!!!
   if(length(w)>1 || (abs(width) <= getdistrOption("DistrResolution")) ||
                    !is.finite(w) )
       stop("width must be a non-zero, finite numeric of length 1") 
   if(length(p)>1 || !is.finite(p))
       stop("pivot must be a finite numeric of length 1") 
   new("Lattice", pivot=pivot, width = width, Length = Length,  name = name)
    }


setMethod("Length","Lattice", function(object) object@Length)
setMethod("width","Lattice", function(object) object@width)
setMethod("pivot","Lattice", function(object) object@pivot)

setReplaceMethod("Length","Lattice", function(object,value)
   {if(!identical(value,floor(value)) || length(value)>1 || value<1)
               stop("Length must be a positive integer")
    object@Length <- value; object})

setReplaceMethod("width","Lattice", function(object,value)
    {if(length(value)>1 || (abs(value) <= getdistrOption("DistrResolution")) ||
                    !is.finite(value) )
       stop("width must be a non-zero, finite numeric of length 1") 
    object@width <- value; object})

setReplaceMethod("pivot","Lattice", function(object,value)
    {if(length(value)>1 || !is.finite(value))
       stop("pivot must be a finite numeric of length 1") 
     object@pivot <- value; object})

### for working under R < 2.8.0
if(getRversion()<'2.8.0'){
    devNew <- function(...){
        if(length(dev.list())>0)
        get(getOption("device"))(...)
    }
}else{
    devNew <- function(...){
        if(length(dev.list())>0){
           if(!is.null(getOption("newDevice"))){
               nrOpen <- length(grDevices::dev.list())
               if(getOption("newDevice")==TRUE) {
                  if(interactive()){
                      while(nrOpen >20){
                         invisible(readline(prompt=
                         paste(gettext(
                         "Too many open graphic devices; please shut some."),
                         "\n", gettext(
                         "When you have shut some devices, press [enter] to continue"),
                         "\n", sep="")))
                         nrOpen <- length(grDevices::dev.list())
                      }
                  }else{
                      if(nrOpen >20){
                         while(nrOpen<-length(grDevices::dev.list())>5)
                             grDevices::dev.off(which=grDevices::dev.list()[2])
                      }
                  }
                  dev.new(...)
               }
           }
        }
    }
}
options("newDevice"=FALSE)

#------------------------------------------------------------------------------
### getLow / getUp methods
#------------------------------------------------------------------------------

 setMethod("getLow", "AbscontDistribution",
            function(object, eps = getdistrOption("TruncQuantile")) {
                 q0 <- q.l(object)(0)
                 if (q0 > - Inf){
                   return(q0)
                }else{
                   qF <- q.l(object)
                   qe <- qF(eps)
                   if (!is.na(qe) && qe > -Inf)
                      return(qe)
                   else{
                      if(.inArgs("log.p",qF))
                         return(qF(p = .fm(x = .5, f = q.l(object)),
                               log.p = TRUE))
                      else
                         return(qF(p = exp(.fm(x = .5, f = q.l(object)))))
                   }
                }  
            })
 setMethod("getUp", "AbscontDistribution",
            function(object, eps = getdistrOption("TruncQuantile")) {
                 q1 <- q.l(object)(1)
                 if (q1 < Inf){
                   return(q1)
                 }else{
                    qF <- q.l(object)
                    if (.inArgs("lower.tail", qF)){
                          qe <- qF(eps, lower.tail = FALSE)
                          if (!is.na(qe) && qe < Inf)
                               return(qe)
                          else{
                             if(.inArgs("log.p",qF))
                                return(qF(p = .fM2(x = .5, f = q.l(object)),
                                      log.p = TRUE))
                             else
                                return(qF(p = exp(.fM2(x = .5, f = q.l(object)))))
                          }
                      }else{
                          qe <- q.l(object)(1-eps)
                          if (!is.na(qe) && qe < Inf)
                               return(qe)
                          else{
                             if(.inArgs("log.p",qF))
                               return(qF(p = .fM(x = .5, f = q.l(object)),
                                      log.p = TRUE))
                             else
                               return(qF(p = exp(.fM(x = .5, f = q.l(object)))))
                          }
                      }
                 }
            })
 setMethod("getLow", "DiscreteDistribution",
            function(object, ...) min(support(object)) )
 setMethod("getUp", "DiscreteDistribution",
            function(object, ...) max(support(object)) )

 setMethod("getLow", "LatticeDistribution",
            function(object, ...){ 
                lattice <- lattice(object) 
                qF <- q.l(object)
                if(is.finite(Length(lattice)) || width(lattice)>0)
                   return(min(support(object)))
                if(.inArgs("log.p",qF))
                   return(qF(p = .fm(x = .5, f = q.l(object)), log.p = TRUE))
                else 
                   return(qF(p = exp(.fm(x = .5, f = q.l(object)))))
                })
 setMethod("getUp", "LatticeDistribution",
            function(object, ...){
                lattice <- lattice(object) 
                if(is.finite(Length(lattice)) || width(lattice)<0)
                   return(max(support(object)))
                qF <- q.l(object)
                if (.inArgs("lower.tail", qF)){
                    if(.inArgs("log.p",qF))
                       return(qF(p = .fM(x = .5, f = q.l(object)),
                                        log.p = TRUE))
                    else
                       return(qF(p = exp(.fM(x = .5, f = q.l(object)))))
                }
                if(.inArgs("log.p",qF))
                   return(qF(p = .fM2(x = .5, f = q.l(object)),
                             lower.tail = FALSE, log.p = TRUE))               
                else
                   return(qF(p = exp(.fM2(x = .5, f = q.l(object))),
                             lower.tail = FALSE))               
                
                })   

 setMethod("getLow", "UnivarLebDecDistribution",
            function(object, eps = getdistrOption("TruncQuantile")) 
                     min(getLow(discretePart(object)),
                         getLow(acPart(object), eps = eps)))
 setMethod("getUp", "UnivarLebDecDistribution",
            function(object, eps = getdistrOption("TruncQuantile")) 
                     max(getUp(discretePart(object)),
                         getUp(acPart(object), eps = eps)))

 setMethod("getLow", "UnivarMixingDistribution",
            function(object, eps = getdistrOption("TruncQuantile")){ 
                     l <- length(mixCoeff)
                     low <- Inf
                     for(i in 1:l){
                         if(!is(try(low0 <- getLow(mixDistr[[i]], eps = eps), 
                                    silent = TRUE), "try-error"))
                            low <- min(low,low0)
                         else {
                            if(!is(try(low0 <- getLow(mixDistr[[i]]), 
                                       silent = TRUE), "try-error"))
                               low <- min(low,low0)                         
                         }
                     }    
                     return(low)})
 setMethod("getUp", "UnivarMixingDistribution",
            function(object, eps = getdistrOption("TruncQuantile")){ 
                     l <- length(mixCoeff)
                     up <- -Inf
                     for(i in 1:l){
                         if(!is(try(up0 <- getUp(mixDistr[[i]], eps = eps), 
                                    silent = TRUE), "try-error"))
                            up <- max(up,up0)
                         else {
                            if(!is(try(up0 <- getUp(mixDistr[[i]]), 
                                       silent = TRUE), "try-error"))
                               up <- max(up,up0)
                         }
                     }    
                     return(up)})

################################
##
## Class: WeibullParameter
##
################################

## Access Methods
setMethod("shape", "WeibullParameter", 
           function(object) object@shape)
setMethod("scale", "WeibullParameter", 
           function(x, center = TRUE, scale = TRUE) x@scale)
           ### odd arg-list due to existing function in base package 

## Replace Methods
setReplaceMethod("shape", "WeibullParameter", 
                  function(object, value){ object@shape <- value; object})
setReplaceMethod("scale", "WeibullParameter", 
                  function(object, value){ object@scale <- value; object})

setValidity("WeibullParameter", function(object){
  if(length(shape(object)) != 1)
    stop("shape has to be a numeric of length 1")    
  if(shape(object) <= 0)
    stop("shape has to be positive")
  if(length(scale(object)) != 1)
    stop("scale has to be a numeric of length 1")      
  if(scale(object) <= 0)
    stop("scale has to be positive")
  else return(TRUE)
})

################################
##
## Class: Weibull distribution
##
################################

Weibull <- function(shape = 1, scale = 1) 
                    new("Weibull", shape = shape, scale = scale)

## wrapped access methods
setMethod("shape", "Weibull", 
           function(object) shape(param(object)))
setMethod("scale", "Weibull", 
           function(x, center = TRUE, scale = TRUE) scale(param(x)))
           ### odd arg-list due to existing function in base package 

## wrapped replace methods
setMethod("shape<-", "Weibull", 
           function(object, value) 
                    new("Weibull", shape = value, scale = scale(object)))
setMethod("scale<-", "Weibull", 
           function(object, value) 
                    new("Weibull", shape = shape(object), scale = value))

setMethod("*", c("Weibull","numeric"),
          function(e1, e2){
                if(isTRUE(all.equal(e2,0)))
                   return(new("Dirac", location = 0, .withArith = TRUE))
                if(e2 > 0)
                   return(new("Weibull", shape = shape(e1),
                               scale = scale(e1) * e2, .withArith = TRUE))
                return(-1 * as(Weibull(shape = shape(e1),
                                    scale = scale(e1) * (-e2)),
                               "AbscontDistribution")
                      )
          })

################################
##
## Class: UnifParameter
##
################################


## Access Methods
setMethod("Min", "UnifParameter", function(object) object@Min)
setMethod("Max", "UnifParameter", function(object) object@Max)
## Replace Methods
setReplaceMethod("Min", "UnifParameter", 
                  function(object, value){ object@Min <- value; object})
setReplaceMethod("Max", "UnifParameter", 
                  function(object, value){ object@Max <- value; object})


setValidity("UnifParameter", function(object){
  if(length(Min(object)) != 1)
    stop("Min has to be a numeric of length 1")    
  if(length(Max(object)) != 1)
    stop("Max has to be a numeric of length 1")    
  if(Min(object) >= Max(object))
    stop("Min has to be less than Max")
  else return(TRUE)
})


################################
##
## Class: uniform distribution
##
################################

Unif <- function(Min = 0, Max = 1) new("Unif", Min = Min, Max = Max)

## wrapped access methods
setMethod("Min", "Unif", function(object) Min(param(object)))
setMethod("Max", "Unif", function(object) Max(param(object)))

## wrapped replace methods
setMethod("Min<-", "Unif", 
           function(object, value) new("Unif", Min = value, Max = Max(object)))
setMethod("Max<-", "Unif", 
           function(object, value) new("Unif", Min = Min(object), Max = value))

## extra methods for Unif distribution
setMethod("+", c("Unif","numeric"),
          function(e1, e2){
            if (length(e2)>1) stop("length of operator must be 1")
            new("Unif", Min = Min(e1) + e2, Max = Max(e1) + e2, 
                .withArith = TRUE) 
          })

setMethod("*", c("Unif","numeric"),
          function(e1, e2){
            if (length(e2)>1) stop("length of operator must be 1")
            if(e2 == 0) return(new("Dirac", location = 0, .withArith = TRUE))
            if(e2 > 0) 
              new("Unif", Min = Min(e1) * e2, Max = Max(e1) * e2, 
                   .withArith = TRUE)
            else
              new("Unif", Min = Max(e1) * e2, Max = Min(e1) * e2, 
                  .withArith = TRUE)
          })

################################
##
## Class: FParameter
##
################################


## Access Methods
setMethod("df1", "FParameter", function(object) object@df1)
setMethod("df2", "FParameter", function(object) object@df2)
setMethod("ncp", "FParameter", function(object) object@ncp)
## Replace Methods
setReplaceMethod("df1", "FParameter", 
                  function(object, value){ object@df1 <- value; object})
setReplaceMethod("df2", "FParameter", 
                  function(object, value){ object@df2 <- value; object})
setReplaceMethod("ncp", "FParameter", 
                  function(object, value){ object@ncp <- value; object})

setValidity("FParameter", function(object){
  if(length(df1(object)) != 1)
    stop("df1 has to be a numeric of length 1")    
  if(df1(object) <= 0)
    stop("df1 has to be positive")
  if(length(df2(object)) != 1)
    stop("df2 has to be a numeric of length 1")    
  if(df2(object) <= 0)
    stop("df2 has to be positive")
  if(length(ncp(object)) != 1)
    stop("ncp has to be a numeric of length 1")      
  else return(TRUE)
})

################################
##
## Class: F distribution
##
################################

Fd <- function(df1 = 1, df2 = 1, ncp = 0) 
               new("Fd", df1 = df1, df2 = df2, ncp = ncp)

## wrapped access methods
setMethod("df1", "Fd", function(object) df1(param(object)))
setMethod("df2", "Fd", function(object) df2(param(object)))
setMethod("ncp", "Fd", function(object) ncp(param(object)))

## wrapped replace methods
setMethod("df1<-", "Fd", function(object, value) 
           new("Fd", df1 = value, df2 = df2(object), ncp=ncp(object)))
setMethod("df2<-", "Fd", function(object, value) 
           new("Fd", df1 = df1(object), df2 = value, ncp=ncp(object)))
setMethod("ncp<-", "Fd", function(object, value) 
           new("Fd", df1 = df1(object), df2 = df2(object), ncp=value))

makeAbscontDistribution <- function(object, gaps = NULL,
                       param = NULL, img = NULL,
                   withgaps = getdistrOption("withgaps"),
                   ngrid = getdistrOption("DefaultNrGridPoints"),
                   ep = getdistrOption("TruncQuantile")){
   if(!is(object,"UnivariateDistribution"))
        stop("Argument 'object' must be of class 'UnivariateDistribution'")
   if(missing(img)) img0 <- img(object)
   if(is.null(img)) img0 <- img(object)
   pfun <- p(object)
   low0 <- q.l(object)(0)*1.001
   up0 <- q.l(object)(1)*1.001
   low1 <- getLow(object,ep)*1.001
   up1 <- getUp(object,ep)*1.001
   wS <- object@.withSim
   wA <- object@.withArith
   lE <- .lowerExact(object)
   loE <- .logExact(object)
   AbscontDistribution(p = pfun, gaps = gaps, param = param, img = img0,
                   .withSim = wS, .withArith = wA, .lowerExact = lE,
                   .logExact = loE, withgaps = withgaps, low1 = low1, up1 = up1,
                   low = low0, up = up0, ngrid = ngrid,
                   ep = ep)
   }

   

###############################################################################
# Methods for Absolutely Continuous Distributions
###############################################################################

## (c) P.R. 300408

AbscontDistribution <- function(r = NULL, d = NULL, p = NULL, q = NULL,
                   gaps = NULL, param = NULL, img = new("Reals"),
                   .withSim = FALSE, .withArith = FALSE,
                    .lowerExact = FALSE, .logExact = FALSE,
                   withgaps = getdistrOption("withgaps"),
                   low1 = NULL, up1 = NULL, low = -Inf, up =Inf,
                   withStand = FALSE,
                   ngrid = getdistrOption("DefaultNrGridPoints"),
                   ep = getdistrOption("TruncQuantile"),
                   e = getdistrOption("RtoDPQ.e"),
                   Symmetry = NoSymmetry() 
                  )
{ if(missing(r) && missing(d) && missing(p) && missing(q))
    stop("At least one of arg's r,d,p,q must be given")

  d1 <-  d
  wS <- .withSim
  wA <- .withArith
  q.l0 <- q
  if(is.null(r)){
      if(is.null(q)){
          if(is.null(p)){
              if(is.null(low1)){
                  i <- 0; x0 <- -1
                  while(d(x0)> ep && i < 20) x0 <- x0 * 2
                  low1 <- x0
              }
              if(is.null(up1)){
                  i <- 0; x0 <- 1
                  while(d(x0)> ep && i < 20) x0 <- x0 * 2
                  up1 <- x0
              }
              ### new: allow for non standardized d functions i.e. 
              ### possibly, int d(x) dx != 1
              if(withStand){
                  if(is(try(
                     stand <- integrate(d,-Inf,Inf)$value,
                     silent=TRUE),
                     "try-error")){
                     if(is(try(
                         stand <- integrate(d, low1, up1)$value,
                         silent = TRUE),
                         "try-error")){
                          n1 <- 2*ngrid+1
                          n1.odd <- seq(1, n1, by = 2)
                          n1.even <- (1:n1)[-n1.odd]
                          x <- seq(low1,up1, length = n1)
                          h <- diff(x)[1]
                          dx <- d(x)
                          stand <- (4*sum(dx[n1.even])+
                                    2*sum(dx[n1.odd])-
                                    dx[1]-rev(dx)[1])*h/6 
                     }
                  }
                  d0 <- d
                  if(.inArgs("log",d0))
                     d1 <- function(x, log = FALSE){
                                 d00 <- d0(x, log = log)
                                 d00 <- if(log) d00 - log(stand) else d00 / stand
                                 return(d00)
                           }      
                  else
                     d1 <- function(x, log = FALSE){
                                 d00 <- d0(x)/stand
                                 if(log) d00 <- log(d00) 
                                 return(d00)
                           }      
              }
              p <- .D2P(d = d1, ql = low1, qu = up1,  ngrid = ngrid)
              q <- q.l0 <- .P2Q(p = p, ql = low1, qu = up1,  ngrid = ngrid,
                        qL = low, qU = up)
              r <- function(n) q.l0(runif(n))
          }else{ 
              if(is.null(low1)){
                  i <- 0; x0 <- -1
                  while(p(x0)> ep && i < 20) x0 <- x0 * 2
                  low1 <- x0
              }
              if(is.null(up1)){
                  i <- 0; x0 <- 1
                  while(p(x0)< 1-ep && i < 20) x0 <- x0 * 2
                  up1 <- x0
              }

              q <- q.l0 <- .P2Q(p = p, ql = low1, qu = up1,  ngrid = ngrid,
                       qL = low, qU = up)
              r <- function(n) q.l0(runif(n))
              if( is.null(d))
                 d <- .P2D(p = p, ql = low1, qu = up1,  ngrid = ngrid)
          }
      }else{
          q.l0 <- q
          if(is.null(p))
             p <- .Q2P(q.l0, ngrid = ngrid)
          r <- function(n) q.l0(runif(n))
          if( is.null(d)){
              if(is.null(low1))
                 low1 <- q.l0(ep)
              if(is.null(up1))
                 up1 <- q.l0(1-ep)
              d <- .P2D(p = p, ql = low1, qu = up1,  ngrid = ngrid)
              }
      }
  }else{
      if(is.null(d)){
          if(is.null(p)){
              if(is.null(q)){
                  erg <- RtoDPQ(r = r, e = e, n = ngrid)
                  wS <- TRUE
                  d <- erg$d; p <- erg$p; q<- q.l0<- erg$q
              }else{
                  q.l0 <- q
                  p <- .Q2P(q, ngrid = ngrid)
                  if( is.null(d)){
                      if(is.null(low1))
                         low1 <- q.l0(ep)
                      if(is.null(up1))
                         up1 <- q.l0(1-ep)
                      d <- .P2D(p = p, ql = low1, qu = up1,  ngrid = ngrid)
                  }
              }
          }else{
              if(is.null(q)){
                  if(is.null(low1)){
                      i <- 0; x0 <- -1
                      while(p(x0)> ep && i < 20) x0 <- x0 * 2
                      low1 <- x0
                  }
                  if(is.null(up1)){
                      i <- 0; x0 <- 1
                      while(p(x0)< 1-ep && i < 20) x0 <- x0 * 2
                      up1 <- x0
                  }
                  q <- q.l0 <- .P2Q(p = p, ql = low1, qu = up1,  ngrid = ngrid,
                           qL = low, qU = up)
                  d <- .P2D(p = p, ql = low1, qu = up1,  ngrid = ngrid)
              }
          }
      }else{
          if(is.null(p)){
              if(is.null(q)){
                  if(is.null(low1)){
                      i <- 0; x0 <- -1
                      while(d(x0)> ep && i < 20) x0 <- x0 * 2
                      low1 <- x0
                      }
                  if(is.null(up1)){
                      i <- 0; x0 <- 1
                      while(d(x0)> ep && i < 20) x0 <- x0 * 2
                      up1 <- x0
                      }

              ### new: allow for non standardized d functions i.e. 
              ### possibly, int d(x) dx != 1
              
                  if(withStand){
                      if(is(try(
                          stand <- integrate(d, -Inf, Inf)$value,
                                silent=TRUE),
                            "try-error")){
                          if(is(try(
                              stand <- integrate(d, low1, up1)$value,
                                    silent = TRUE),
                                "try-error")){
                              n1 <- 2*ngrid+1
                              n1.odd <- seq(1,n1, by=2)
                              n1.even <- (1:n1)[-n1.odd]
                              x <- seq(low1,up1, length = n1)
                              h <- diff(x)[1]
                              dx <- d(x)
                              stand <- (4*sum(dx[n1.even])+
                                        2*sum(dx[n1.odd])-
                                        dx[1]-rev(dx)[1])*h/6 
                          }
                      }
                      d0 <- d
                      if(.inArgs("log",d0))
                         d1 <- function(x, log = FALSE){
                                     d00 <- d0(x, log = log)
                                     d00 <- if(log) d00 - log(stand) else d00 / stand
                                     return(d00)
                               }      
                      else
                         d1 <- function(x, log = FALSE){
                                     d00 <- d0(x)/stand
                                     if(log) d00 <- log(d00) 
                                     return(d00)
                               }      
                  }
        
                  p <- .D2P(d = d1, ql = low1, qu=up1,  ngrid = ngrid)
                  q <- q.l0 <- .P2Q(p = p, ql = low1, qu=up1,  ngrid = ngrid,
                            qL = low, qU = up)
              }else
                  p <- .Q2P(q, ngrid = ngrid)
          }else{
              if(is.null(q)){
                  if(is.null(low1)){
                      i <- 0; x0 <- -1
                      while(p(x0)> ep && i < 20) x0 <- x0 * 2
                      low1 <- x0
                  }
                  if(is.null(up1)){
                      i <- 0; x0 <- 1
                      while(p(x0)< 1-ep && i < 20) x0 <- x0 * 2
                      up1 <- x0
                  }
                  q <- q.l0 <- .P2Q(p = p, ql = low1, qu=up1,  ngrid = ngrid,
                            qL = low, qU = up)
              }          
          }
      }
  }
  obj <- new("AbscontDistribution", r = r, d = d1, p = p, q = q.l0,
      gaps = gaps, param = param, img = img, .withSim = wS,
      .withArith = wA, .lowerExact = .lowerExact, .logExact = .logExact,
      Symmetry = Symmetry)

  if(is.null(gaps) && withgaps) setgaps(obj)
  if(!is.null(obj@gaps)&&length(obj@gaps)){
     obj@q <- .modifyqgaps(pfun = obj@p, qfun = obj@q, gaps = obj@gaps)
  }else{
     if(exists("..q0fun", envir=environment(obj@q)))
        obj@q <- get("..q0fun", envir=environment(obj@q))
  }
  return(obj)
}

## Access Methods

setMethod("gaps", signature(object = "AbscontDistribution"),  
           function(object) object@gaps)

## ReplaceMethods

setReplaceMethod("gaps", signature(object = "AbscontDistribution"),  
          function(object, value)
                  {dsvalue <- deparse(substitute(value))
                   if(!is.null(value)){
                      if(!is.matrix(value))
                         stop("value must either be a matrix or NULL")
                      if(!ncol(value)==2)
                         stop("if matrix, value must have 2 columns")
                      l <- length(value)
                      if(!identical(1:l, order(c(t(value)))))
                         stop(gettextf("c(t(%s)) must be increasing",
                              dsvalue))
                      colnames(value) <- c("from", "to")
                      }
                   object@gaps <- value; object})
                                                             

setMethod("setgaps", signature(object = "AbscontDistribution"), 
function(object, exactq = 6, ngrid = 50000, ...){
       object1 <- object
       lower <- getLow(object, eps = getdistrOption("TruncQuantile")*2)
       upper <- getUp(object, eps = getdistrOption("TruncQuantile")*2)
       #lower <- 0 ; upper <- 8
       dist <- upper - lower
       low1 <- max(q.l(object)(0),lower-0.1*dist)
       upp1 <- min(q.l(object)(1),upper+0.1*dist)
       grid <- seq(from = low1, to = upp1, length = ngrid) 
       dxg <- d(object)(grid)
       
       ix <-  1:ngrid
       ix0 <- (dxg < 1/10^exactq)&(grid>=lower)&(grid<=upper)     
       if(any(ix0)){
          ixc <- ix[ix0]
          dixc <- c(2,diff(ixc))
          dixc0 <- dixc>1
          l2 <- length(ixc)
          ixc2 <- seq(l2)
          ixcl <- ixc2[dixc0]
          ixcr <- c(ixcl[-1]-1,l2)
          gridl <- grid[ixc[ixcl]]
          gridr <- grid[ixc[ixcr]]
          
          mattab.d <- cbind(gridl, gridr)
          
          ox <- order(mattab.d[,1])
          mattab.d <- matrix(mattab.d[ox,], ncol = 2)
          mattab.d <- .consolidategaps(mattab.d)
          if(nrow(mattab.d)==0) mattab.d <- NULL
          if(length(mattab.d)==0) mattab.d <- NULL
          } else mattab.d <- NULL
          finit <- if(is.null(dim(mattab.d))) 0 else
                   apply(mattab.d, 1, function(x) all(is.finite(x)))
          mattab.d <- if(sum(finit)>0) mattab.d[finit,,drop=FALSE] else NULL
          eval(substitute( "slot<-"(object,'gaps', value = mattab.d)))
       return(invisible())
})
 
## Arithmetics

setMethod("+", c("AbscontDistribution","AbscontDistribution"),
function(e1,e2){
            ### Step 1 : Truncation
            
            lower <- min(getLow(e1), getLow(e2))
            upper <- max(getUp(e1) , getUp(e2))

            ### Step 2 : Discretizing

            n <- getdistrOption("DefaultNrFFTGridPointsExponent")
            h <- (upper-lower)/2^n

            dpe1 <- .discretizeP(e1, lower, upper, h)
            dpe2 <- .discretizeP(e2, lower, upper, h)

            x <- seq(from = 2*lower, to = 2*upper, by = h)

            ### Step 3 : Zero-Padding

            dpe1 <- c(dpe1, numeric(2^n))
            dpe2 <- c(dpe2, numeric(2^n))

            ## Step 4: computation of DFT

            ftpe1 <- fft(dpe1); ftpe2 <- fft(dpe2)
            ## convolution theorem for DFTs
            d2 <- c(0,Re(fft(ftpe1*ftpe2, inverse = TRUE)) / length(ftpe1))

            ## density & cdf (steps 5--7)
            dfun <- .makeDNew(x, d2, h)
            pfun <- .makePNew(x, d2, h, .notwithLArg(e1)||.notwithLArg(e2) )


            ## quantile function
            yL <-  if ((q.l(e1)(0) == -Inf)||(q.l(e2)(0) == -Inf))
                 -Inf else getLow(e1)+getLow(e2)
            yR <-  if ((q.l(e1)(1) ==  Inf)||(q.l(e2)(1) ==  Inf))
                  Inf else getUp(e1)+getUp(e2)

            px.l <- pfun(x + 0.5*h)
            px.u <- pfun(x + 0.5*h, lower.tail = FALSE)
            
            qfun <- .makeQNew(x + 0.5*h, px.l, px.u,
                              .notwithLArg(e1)||.notwithLArg(e2), yL, yR)
            
            rfun <- function(n){}
            body(rfun) <- substitute({ f(n) + g(n) },
                                     list(f = e1@r, g = e2@r))

            object <- AbscontDistribution(r = rfun, d = dfun, p = pfun,
                          q = qfun, .withSim = FALSE, .withArith = TRUE)

            rm(d2, dpe1,dpe2, ftpe1,ftpe2)
            rm(h, px.l, px.u, rfun, dfun, qfun, pfun, upper, lower)
            if(is(e1@Symmetry,"SphericalSymmetry")&& 
               is(e2@Symmetry,"SphericalSymmetry"))
               object@Symmetry <- SphericalSymmetry(SymmCenter(e1@Symmetry)+
                                                     SymmCenter(e2@Symmetry))   
            object
          })


###setMethod("m1df", "AbscontDistribution",
###   function(object){
###     lower <- q.l(object)(TruncQuantile)
###     upper <- q.l(object)(1 - TruncQuantile)
###     
###     gitter.x <- seq(from = lower, to = upper, length = DefaultNrGridPoints)
###     
###    integrand <- function(x) x * d(object)(x)
###     
###     tmp <- function(t) integrate(integrand, lower = lower, upper = t)$value
###     
###     gitter.y <- sapply(gitter.x, tmp)
###     
###     approxfun(gitter.x, gitter.y, rule = 2)
###   })


###setMethod("m2df", "AbscontDistribution", 
###   function(object){
###     lower <- q.l(object)(TruncQuantile)
###     upper <- q.l(object)(1 - TruncQuantile)
###     
###     gitter.x <- seq(from = lower, to = upper, length = DefaultNrGridPoints)
###     
###     integrand <- function(x) x^2 * d(object)(x)
###     
###     tmp <- function(t) integrate(integrand, lower = lower, upper = t)$value
###     
###     gitter.y <- sapply(gitter.x, tmp)
###     
###     approxfun(gitter.x, gitter.y, rule = 2)
###   })

## binary operators for absolut continuous distributions


setMethod("*", c("AbscontDistribution","numeric"),
          function(e1, e2) {Distr <-  .multm(e1,e2, "AbscontDistribution")                               
                            if(is(Distr, "AffLinDistribution"))
                                 Distr@X0 <- e1
                            if(is(e1@Symmetry,"SphericalSymmetry"))
                               Distr@Symmetry <- 
                                 SphericalSymmetry(SymmCenter(e1@Symmetry)*e2)

                            Distr})
setMethod("+", c("AbscontDistribution","numeric"),
           function(e1, e2) {Distr <-  .plusm(e1,e2, "AbscontDistribution")                               
                            if(is(Distr, "AffLinDistribution"))
                                 Distr@X0 <- e1
                            if(is(e1@Symmetry,"SphericalSymmetry"))
                               Distr@Symmetry <- 
                                 SphericalSymmetry(SymmCenter(e1@Symmetry)+e2)
                            Distr})                            
setMethod("*", c("AffLinAbscontDistribution","numeric"),
          function(e1, e2){Distr <-  .multm(e1,e2, "AffLinAbscontDistribution")
                           if(is(e1@Symmetry,"SphericalSymmetry"))
                               Distr@Symmetry <- 
                                 SphericalSymmetry(SymmCenter(e1@Symmetry)*e2)
                           Distr                         })
setMethod("+", c("AffLinAbscontDistribution","numeric"),
           function(e1, e2){Distr <-  .plusm(e1,e2, "AffLinAbscontDistribution")
                            if(is(e1@Symmetry,"SphericalSymmetry"))
                               Distr@Symmetry <- 
                                 SphericalSymmetry(SymmCenter(e1@Symmetry)+e2)
                            Distr                         })

## Group Math for absolutly continuous distributions
setMethod("Math", "AbscontDistribution",
          function(x){            

            rnew <- function(n, ...){}           
            body(rnew) <- substitute({ f(g(n, ...)) },
                              list(f = as.name(.Generic), g = x@r))
            
            n <- 10^getdistrOption("RtoDPQ.e")+1
            u <- seq(0,1,length=n+1); u <- (u[1:n]+u[2:(n+1)])/2
            y <- callGeneric(q.l(x)(u))
            DPQnew <- RtoDPQ(r=rnew, y=y)
                       
            object <- AbscontDistribution(d = DPQnew$d, p = DPQnew$p, 
                           r = rnew, q = DPQnew$q,
                           .withSim = TRUE, .withArith = TRUE)
            object
          })

## exact: abs for absolutly continuous distributions
setMethod("abs", "AbscontDistribution",
    function(x){
       if (.isEqual(p(x)(0),0)) return(x)
       xx <- x
       rnew <- function(n, ...){}
       body(rnew) <- substitute({ abs(g(n, ...)) }, list(g = xx@r))
       
       isSym0 <- FALSE
       if(is(Symmetry(xx),"SphericalSymmetry"))
          if(.isEqual(SymmCenter(Symmetry(xx)),0))
             isSym0 <- TRUE  
       
       if(isSym0){
          if (is.null(gaps(xx)))
              gapsnew <- NULL
          else {gapsnew <- gaps[gaps[,2]>=0,]
                VZW <- gapsnew[,1] <= 0 
                gapsnew[VZW,1] <- 0
                gapsnew <- .consolidategaps(gapsnew)}
          dOx <- d(xx)

          dxlog <- if("log" %in% names(formals(dOx))) 
                        quote({dOx(x, log = TRUE)})
                   else quote({log(dOx(x))})
          pxlog <- if("log.p" %in% names(formals(p(x))) && 
                       "lower.tail" %in% names(formals(p(x)))) 
                        quote({p(xx)(q, lower.tail = FALSE, log.p = TRUE)})
                   else
                        quote({log(1-p(xx)(q))})

          qxlog <- if("lower.tail" %in% names(formals(q.l(xx))))
                          quote({qx <- if(lower.tail)
                                          q.l(xx)((1+p1)/2)
                                       else
                                          q.l(xx)(p1/2,lower.tail=FALSE)})
                      else
                          quote({qx <- q.l(xx)(if(lower.tail) (1+p1)/2 else 1-p1/2)})
          if("lower.tail" %in% names(formals(q.l(xx)))&&
             "log.p" %in% names(formals(q.l(xx))))
              qxlog <- quote({qx <- if(lower.tail) q.l(xx)((1+p1)/2)
                                       else
                                          q.l(xx)(if(log.p)p-log(2)
                                               else p1/2,lower.tail=FALSE,log.p=log.p)}) 
          dnew <- function(x, log = FALSE){}
          body(dnew) <- substitute({
                    dx <- (dxlog0 + log(2))*(x>=0)
                    if (!log) dx <- exp(dx)
                    dx[x<0] <- if(log) -Inf else 0
                    return(dx)
                    }, list(dxlog0 = dxlog))
            
          pnew <- function(q, lower.tail = TRUE, log.p = FALSE){}
          body(pnew) <- substitute({
                    if (!lower.tail){
                        px <- (log(2) + pxlog0)*(q>=0)
                        if(!log.p) px <- exp(px)
                    }else{
                        px <- pmax(2 * p(x)(q) - 1,0)
                        if(log.p) px <- log(px)
                    }
                    return(px)            
            }, list(pxlog0 = pxlog))

          qnew <- function(p, lower.tail = TRUE, log.p = FALSE){}
          body(qnew) <- substitute({
                   p1 <- if(log.p) exp(p) else p 
                   qxlog0
                   qx[p1<0] <- NaN
                   if (any((p1 < -.Machine$double.eps)|(p1 > 1+.Machine$double.eps)))
                   warning(gettextf("q method of %s produced NaN's ", objN))
                   return(qx)
            }, list(qxlog0 = qxlog, objN= quote(.getObjName(1))))
                   
       }else{
            if (is.null(gaps(xx)))
                gapsnew <- NULL
            else {VZW <- gaps(xx)[,1] <= 0 & gaps(xx)[,2] >= 0
                  gapsnew <- t(apply(abs(gaps(xx)), 1, sort))
                  gapsnew[VZW,2] <- pmin(-gaps(xx)[VZW,1], gaps(x)[VZW,2])
                  gapsnew[VZW,1] <- 0
                  gapsnew <- .consolidategaps(gapsnew)}
            
            lower <- max(0, getLow(xx))
            upper <- max(-getLow(xx) , abs(getUp(xx)))

            n <- getdistrOption("DefaultNrFFTGridPointsExponent")
            h <- (upper-lower)/2^n

            x.g <- seq(from = lower, to = upper, by = h)

            dnew <- function(x, log = FALSE){
                    o.warn <- getOption("warn"); 
                    on.exit(options(warn=o.warn))
                    options(warn = -1)
                    dx <- (x>=0) * (d(xx)(x) + d(xx)(-x))                     
                    if (log) dx <- log(dx)
                    return(dx)
            }
            
            pxlow <- if("lower.tail" %in% names(formals(p(xx))))
                        substitute({p(xx)(q, lower=FALSE)})
                   else
                        substitute({1-p(xx)(q)})

            pnew <- function(q, lower.tail = TRUE, log.p = FALSE){}
            body(pnew) <- substitute({
                    px <- if (lower.tail)
                            (q>=0) * (p(xx)(q) - p(xx)(-q))                    
                          else pxlow0 + p(xx)(-q)
                    if (log.p) px <- log(px)
                    return(px)
            }, list(pxlow0 = pxlow))

            px.l <- pnew(x.g + 0.5*h)
            px.u <- pnew(x.g + 0.5*h, lower.tail = FALSE)
            
            yR <- max(q.l(xx)(1), abs(q.l(xx)(0)))

            qnew <- .makeQNew(x.g + 0.5*h, px.l, px.u,
                              notwithLLarg = FALSE,  lower, yR)
    
            lowerExact <- FALSE
 
    }
    object <- AbscontDistribution( r = rnew, p = pnew, q = qnew, d = dnew, 
                     gaps = gapsnew,  .withSim = xx@.withSim, .withArith = TRUE,
                     .lowerExact = .lowerExact(x), .logExact = FALSE)
    object
    })

## exact: exp for absolutly continuous distributions
setMethod("exp", "AbscontDistribution",
           function(x) .expm.c(x))


### preliminary to export special functions
if (getRversion()>='2.6.0'){ 

setMethod("log", "AbscontDistribution",
           function(x, base = exp(1)) {
           xs <- as.character(deparse(match.call(
                 call = sys.call(sys.parent(1)))$x))
           ep <- getdistrOption("TruncQuantile")
           basl <- log(base)
           if(p(x)(0)>ep) 
                stop(gettextf("log(%s) is not well-defined with positive probability ", xs))
           else return(.logm.c(x)/basl)})
                       
                       
setMethod("log10", "AbscontDistribution",
          function(x) log(x = x, base = 10))

setMethod("sign", "AbscontDistribution",
          function(x){ 
       DiscreteDistribution(supp=c(-1,0,1), 
              prob=c(p(x)(0), 0, p(x)(0, lower=FALSE)))                     
          })



setMethod("digamma", "AbscontDistribution",
          function(x){
            rnew <-  function(n, ...){}
            body(rnew) <- substitute({ digamma(g(n, ...)) }, list(g = x@r))
            px0 <- p(x)(0)
            if(px0>0) stop("argument of 'digamma' must be concentrated on positive values")
            xx <- x
                    
            pnew <- function(q, lower.tail = TRUE, log.p = FALSE){
                    iq <- igamma(q) 
                    px <- p(xx)(iq, lower.tail = lower.tail, log.p = log.p)
                    return(px)
            }
            dnew <- function(x, log = FALSE){
                    ix <- igamma(x)
                    dx <- d(xx)(ix, log = log)
                    nx <- trigamma(ix)
                    if(log) dx <- dx - log(nx)
                    else dx <- dx/nx
                    return(dx)
            }
            
            .x <- sort(c(qexp(unique(pmin(seq(0,1,length=5e4)+1e-10,1-1e-10))),
                       -abs(rnorm(1e4)),
                       qcauchy(seq(0.999,1-1e-10,length=5e3),lower.tail=FALSE)))
            i <- 0; x0 <- 1
            while(pnew(x0,lower.tail = FALSE)>  getdistrOption("TruncQuantile") && i < 20) 
                 x0 <- x0 * 2
             up1 <- x0
            i <- 0; x0 <- -1
            while(pnew(x0)> getdistrOption("TruncQuantile") && i < 20) 
                 x0 <- x0 * 2
             low1 <- x0
          

            
            qnew <- .P2Q(p = pnew, xx =.x,
                         ql = low1, qu=up1,  
                         ngrid = getdistrOption("DefaultNrGridPoints"),
                            qL = -Inf, qU = Inf)
 
            
            object <- AbscontDistribution( r = rnew, d = dnew, p = pnew, q=qnew,
                           .withSim = TRUE, .withArith = TRUE, .logExact = FALSE)
            object
          })

setMethod("lgamma", "AbscontDistribution",
          function(x){
            rnew <- function(n, ...){}
            body(rnew) <- substitute({ lgamma(g(n, ...)) }, list(g = x@r))

            n <- 10^getdistrOption("RtoDPQ.e")+1
            u <- seq(0,1,length=n+1); u <- (u[1:n]+u[2:(n+1)])/2
            y <- lgamma(q.l(x)(u))
            DPQnew <- RtoDPQ(r=rnew, y=y)
            
            object <- AbscontDistribution( r = rnew, d = DPQnew$d, p = DPQnew$p,
                           q=DPQnew$q, .withSim = TRUE, .withArith = TRUE)
            object
          })

setMethod("gamma", "AbscontDistribution",
          function(x){
            rnew <- function(n, ...){}
            body(rnew) <- substitute({ gamma(g(n, ...)) }, list(g = x@r))
            n <- 10^getdistrOption("RtoDPQ.e")+1
            u <- seq(0,1,length=n+1); u <- (u[1:n]+u[2:(n+1)])/2
            y <- gamma(q.l(x)(u))
            DPQnew <- RtoDPQ(r=rnew, y=y)

            object <- AbscontDistribution( r = rnew, d = DPQnew$d, p = DPQnew$p,
                           q=DPQnew$q, .withSim = TRUE, .withArith = TRUE)
            object
          })
          
setMethod("sqrt", "AbscontDistribution",
            function(x) x^0.5)

}

#------------------------------------------------------------------------
# new p.l, q.r methods
#------------------------------------------------------------------------

setMethod("p.l", signature(object = "AbscontDistribution"),  
           function(object) p(object))

setMethod("q.r", signature(object = "AbscontDistribution"),  
           function(object){
                if(!is.null(gaps(object))&&length(gaps(object)))
                   .modifyqgaps(pfun = p(object), qfun = q.l(object),
                                gaps = gaps(object), leftright = "right")
                else
                    q.l(object)
            })

setMethod("liesInSupport", signature(object = "DiscreteDistribution",
                                     x = "numeric"),
    function(object, x, checkFin = FALSE){
       supp <- support(object)
       erg <- x %in% supp
       if(checkFin && !all(object@.finSupport)){
          lO <- q.l(object)(0)
          rO <- q.l(object)(1)
          lS <- min(supp)
          rS <- max(supp)

          ll.f <- is.finite(lO)
          lr.f <- is.finite(rO)

          ll <- object@.finSupport[1]
          lr <- object@.finSupport[2]

          lln <- !ll.f|!object@.finSupport[1]|is.na(object@.finSupport[1])
          lrn <- !lr.f|!object@.finSupport[2]|is.na(object@.finSupport[2])


          erg <- erg | ((x>=lO) & (x<lS) & ll) | ((x<=rO) & (x>rS) & lr) |
                       ((x<lS) & lln) | ((x>rS) & lrn)
         }
       return(erg)
       })

setMethod("liesInSupport", signature(object = "Distribution",
                                     x = "matrix"),
    function(object, x, checkFin = FALSE){
            ret <- liesInSupport(object,as.vector(x),checkFin)
#            dim(ret) <- dim(x)
            return(ret)
    })

setMethod("liesInSupport", signature(object = "AbscontDistribution",
                                     x = "numeric"),
    function(object, x, checkFin = FALSE){
         ## checkFin not used here but for simple coercing agains x ="matrix"
        if(!is.nan(q.l(object)(0)))
            low <- q.l(object)(0)
        else
            low <- q.l(object)(10*.Machine$double.eps)
        if(!is.nan(q.l(object)(1)))
            upp <- q.l(object)(1)
        else
            upp <- q.l(object)(1-10*.Machine$double.eps)

        (x >= low)&(x <= upp)
    })

#setMethod("liesInSupport", signature(object = "AbscontDistribution",
#                                     x = "matrix"),
#    function(object, x){
#        if(ncol(x) != 1)
#            stop("'x' has wrong dimension")
#       else{
#          ret <- liesInSupport(object,as.vector(x),checkFin)
#            dim(ret) <- dim(x)
#            return(ret)
#       }
#    })

setMethod("liesInSupport", signature(object = "LatticeDistribution",
                                     x = "numeric"),
    function(object, x, checkFin = FALSE){
       erg <- x %in% support(object)
       if(checkFin && !all(object@.finSupport)){
          lO <- q.l(object)(0)
          rO <- q.l(object)(1)
          lln <- !object@.finSupport[1]|is.na(object@.finSupport[1])
          lrn <- !object@.finSupport[2]|is.na(object@.finSupport[2])

          L <- lattice(object)
          vals <- (x-pivot(L))/width(L)
          ep <- .Machine$double.eps
          erg <- .isInteger(vals) & (vals>=-ep) & (vals<=Length(L)-1+ep)
          erg <- erg & ((x>=lO)|lln) & ((x<=rO)|lrn)
         }
       return(erg)
       })

setMethod("liesInSupport", signature(object = "UnivarLebDecDistribution",
                                     x = "numeric"),
    function(object, x, checkFin = FALSE){
       ac <- acPart(object)
       dc <- discretePart(object)
       ep <- .Machine$double.eps
       acl <- rep(TRUE, length(x))
       dcl <- TRUE
       if(acWeight(object)>ep) acl <- liesInSupport(ac,x)
       if(discreteWeight(object)>ep) dcl <- liesInSupport(dc,x, checkFin)
       return(acl|dcl)
       })

setMethod("liesInSupport", signature(object = "UnivarMixingDistribution",
                                     x = "numeric"),
    function(object, x, checkFin = FALSE){
       obj <- flat.mix(object)
       return(liesInSupport(obj,x,checkFin))
       })

.alwaysInSupp <- function(object, x, checkFin = TRUE) is.finite(x)
.posInSupp <- function(object, x, checkFin = TRUE) (x>=0)
.stricposInSupp <- function(object, x, checkFin = TRUE) (x>0)

setMethod("liesInSupport", signature(object = "ExpOrGammaOrChisq",
                                     x = "numeric"),.posInSupp)
setMethod("liesInSupport", signature(object = "Lnorm",
                                     x = "numeric"),.posInSupp)
setMethod("liesInSupport", signature(object = "Fd",
                                     x = "numeric"),.posInSupp)

setMethod("liesInSupport", signature(object = "Norm",
                                     x = "numeric"),.alwaysInSupp)
setMethod("liesInSupport", signature(object = "DExp",
                                     x = "numeric"),.alwaysInSupp)
setMethod("liesInSupport", signature(object = "Cauchy",
                                     x = "numeric"),.alwaysInSupp)
setMethod("liesInSupport", signature(object = "Td",
                                     x = "numeric"),.alwaysInSupp)
setMethod("liesInSupport", signature(object = "Logis",
                                     x = "numeric"),.alwaysInSupp)
setMethod("liesInSupport", signature(object = "Weibull",
                                     x = "numeric"),.alwaysInSupp)

setMethod("liesInSupport", signature(object = "Unif",
                                     x = "numeric"),
  function(object, x, checkFin = TRUE){(x>=Min(object))&(x<=Max(object))})
setMethod("liesInSupport", signature(object = "Beta",
                                     x = "numeric"),
  function(object, x, checkFin = TRUE){(x>=0)&(x<=1)})

setMethod("*", c("AcDcLcDistribution","AcDcLcDistribution"),
function(e1,e2){

 e1 <- .ULC.cast(e1)
 e2 <- .ULC.cast(e2)

#         if( is(e1,"AffLinUnivarLebDecDistribution"))
#             e1 <- as(e1, "UnivarLebDecDistribution")
#         if( is(e2,"AffLinUnivarLebDecDistribution"))
#             e2 <- as(e2, "UnivarLebDecDistribution")
#
#         if( is(e1,"AbscontDistribution"))
#             e1 <- as(as(e1,"AbscontDistribution"), "UnivarLebDecDistribution")
#         if( is(e2,"AbscontDistribution"))
#             e2 <- as(as(e2,"AbscontDistribution"), "UnivarLebDecDistribution")
#         if(is(e1,"DiscreteDistribution"))
#             e1 <- as(as(e1,"DiscreteDistribution"), "UnivarLebDecDistribution")
#         if(is(e2,"DiscreteDistribution"))
#             e2 <- as(as(e2,"DiscreteDistribution"), "UnivarLebDecDistribution")

         ep <- getdistrOption("TruncQuantile")

         e1DC <- decomposePM(e1)
         e2DC <- decomposePM(e2)
         w12pp <- e1DC$pos$w*e2DC$pos$w
         w12mm <- e1DC$neg$w*e2DC$neg$w
         w12pm <- e1DC$pos$w*e2DC$neg$w
         w12mp <- e1DC$neg$w*e2DC$pos$w
         mixCoeff <- c(w12pp,w12mm,w12pm,w12mp)
         mixCoeff <- c(mixCoeff,1-sum(mixCoeff))

         e12pp <- if(w12pp>ep)
                       as(exp(log(e1DC$pos$D)+log(e2DC$pos$D)),
                          "UnivarLebDecDistribution")
                  else as(Dirac(1), "UnivarLebDecDistribution")
         e12pp.f <- discretePart(e1DC$pos$D)@.finSupport[2] &
                    discretePart(e2DC$pos$D)@.finSupport[2]
         d12pp <- discretePart(e12pp)
         d12pp@.finSupport <- e12pp.f
         discretePart(e12pp) <- d12pp

         e12mm <- if(w12mm>ep)
                       as(exp(log(-e1DC$neg$D)+log(-e2DC$neg$D)),
                          "UnivarLebDecDistribution")
                  else as(Dirac(1), "UnivarLebDecDistribution")
         e12mm.f <- discretePart(e1DC$neg$D)@.finSupport[1]&
                    discretePart(e2DC$neg$D)@.finSupport[1]
         d12mm <- discretePart(e12mm)
         d12mm@.finSupport <- e12mm.f
         discretePart(e12mm) <- d12mm

         e12pm <- if(w12pm>ep)
                       as(-exp(log(e1DC$pos$D)+log(-e2DC$neg$D)),
                          "UnivarLebDecDistribution")
                  else as(Dirac(-1), "UnivarLebDecDistribution")
         e12pm.f <- discretePart(e1DC$pos$D)@.finSupport[2] &
                    discretePart(e2DC$neg$D)@.finSupport[1]
         d12pm <- discretePart(e12pm)
         d12pm@.finSupport <- e12pm.f
         discretePart(e12pm) <- d12pm

         if(identical(e1,e2)){
            e12mp <- e12pm
            e12mp.f <- e12pm.f
         }else{ e12mp <- if(w12mp>ep)
                            as( -exp(log(-e1DC$neg$D)+log(e2DC$pos$D)),
                                 "UnivarLebDecDistribution")
                       else as(Dirac(-1), "UnivarLebDecDistribution")
                e12mp.f <- discretePart(e1DC$neg$D)@.finSupport[1] &
                           discretePart(e2DC$pos$D)@.finSupport[2]
                d12mp <- discretePart(e12mp)
                d12mp@.finSupport <- e12mp.f
                discretePart(e12mp) <- d12mp
         }
         e12pm <- .del0dmixfun(e12pm)
         e12mp <- .del0dmixfun(e12mp)

         obj <- flat.LCD(mixCoeff = mixCoeff,
                         e12pp, e12mm, e12pm, e12mp,
                         as(Dirac(0),"UnivarLebDecDistribution"))

         dP <- discretePart(obj)
         dP@.finSupport <- c((w12pm+w12mp<ep^2)|(e12pm.f&e12mp.f),
                              (w12pp+w12pp<ep^2)|(e12pp.f&e12mm.f))
         discretePart(obj) <- dP

         if(getdistrOption("simplifyD"))
            obj <- simplifyD(obj)

         rnew <- function(n, ...){}
         body(rnew) <- substitute({ g1(n, ...) * g2(n, ...) },
                                    list(g1 = e1@r, g2 = e2@r))
         obj@r <- rnew

          if(is(e1@Symmetry,"SphericalSymmetry")&&
             is(e2@Symmetry,"SphericalSymmetry"))
             if(.isEqual(SymmCenter(e1@Symmetry),0) && 
                .isEqual(SymmCenter(e2@Symmetry),0))
                 obj@Symmetry <-  SphericalSymmetry(0)   

         return(obj)
         })

setMethod("/", c("numeric",
                 "AcDcLcDistribution"),
function(e1,e2){
  i <- 1; stopit <- FALSE; sL <- length(sys.calls())
  e2s <- "e2"
  while(!stopit && i <= sL){
    i <- i + 1
	trcall <- sys.call(sys.parent(i))
    myc <- paste(as.list(trcall)[[1]])
	if(any(myc == "/")){
	   e2s <- as.character(deparse(match.call(call=trcall)$e2))
	   stopit <- TRUE
	}
  }  
 e2 <- .ULC.cast(e2)

#         if( is(e2,"AffLinUnivarLebDecDistribution"))
#             e2 <- as(e2, "UnivarLebDecDistribution")

#         if( is(e2,"AbscontDistribution"))
#             e2 <- as(as(e2, "AbscontDistribution"),
#                      "UnivarLebDecDistribution")

#         if( is(e2,"DiscreteDistribution"))
#             e2 <- as(as(e2, "DiscreteDistribution"),
#                      "UnivarLebDecDistribution")

         if (discreteWeight(e2)>getdistrOption("TruncQuantile"))
            if (d.discrete(e2)(0)>getdistrOption("TruncQuantile"))
                stop(gettextf("1 / %s is not well-defined with positive probability ", e2s))

         e2DC <- decomposePM(e2)
         w2p <- e2DC$pos$w
         w2m <- e2DC$neg$w

         e2p <- as(exp(-log(e2DC$pos$D)), "UnivarLebDecDistribution")
         d2p <- discretePart(e2p)
         d2p@.finSupport <- c(TRUE,TRUE) # as both pos&neg part are bounded away from 0
         discretePart(e2p) <- d2p

         e2m <- as(-exp(-log(-e2DC$neg$D)), "UnivarLebDecDistribution")
         d2m <- discretePart(e2m)
         d2m@.finSupport <- c(TRUE,TRUE) # as both pos&neg part are bounded away from 0
         discretePart(e2m) <- d2m

         e2D <- flat.LCD(mixCoeff = c(w2p, w2m), e2p, e2m)

         dP <- discretePart(e2D)
         dP@.finSupport <- c(TRUE,TRUE) # as both pos&neg part are bounded away from 0
         discretePart(e2D) <- dP


         if(getdistrOption("simplifyD"))
            e2D <- simplifyD(e2D)

         obj <- e1*e2D

         rnew <- function(n, ...){}
         body(rnew) <- substitute({ g1 / g2(n, ...) },
                                    list(g1 = e1, g2 = e2@r))
         obj@r <- rnew

          if(is(e2@Symmetry,"SphericalSymmetry"))
             if(.isEqual(SymmCenter(e2@Symmetry),0))
                  obj@Symmetry <-  SphericalSymmetry(0)   



         return(obj)
         })

setMethod("/", c("AcDcLcDistribution",
                 "AcDcLcDistribution"),
function(e1,e2){
  i <- 1; stopit <- FALSE; sL <- length(sys.calls())
  e2s <- "e2"
  while(!stopit && i <= sL){
    i <- i + 1
	trcall <- sys.call(sys.parent(i))
    myc <- paste(as.list(trcall)[[1]])
	if(any(myc == "/")){
	   e2s <- as.character(deparse(match.call(call=trcall)$e2))
	   stopit <- TRUE
	}
  }  

#         if( is(e2,"AbscontDistribution"))
#             e2 <- as(as(e2,"AbscontDistribution"), "UnivarLebDecDistribution")

#         if( is(e2,"DiscreteDistribution"))
#             e2 <- as(as(e2,"DiscreteDistribution"), "UnivarLebDecDistribution")

 e2 <- .ULC.cast(e2)

         if (discreteWeight(e2)>getdistrOption("TruncQuantile"))
         if (d.discrete(e2)(0)>getdistrOption("TruncQuantile"))
            stop(gettextf("1 / %s is not well-defined with positive probability ", e2s))

         obj <- e1 * (1/e2)

         rnew <- function(n, ...){}
         body(rnew) <- substitute({ g1(n, ...) / g2(n, ...) },
                                    list(g1 = e1@r, g2 = e2@r))
         obj@r <- rnew

          if(is(e1@Symmetry,"SphericalSymmetry")&&
             is(e2@Symmetry,"SphericalSymmetry"))
             if(.isEqual(SymmCenter(e1@Symmetry),0) && 
                .isEqual(SymmCenter(e2@Symmetry),0))
                 obj@Symmetry <-  SphericalSymmetry(0)   

         return(obj)
         })

setMethod("^", c("AcDcLcDistribution","Integer"),
function(e1,e2){

           if(.isEqual(e2,0)) return(Dirac(1))
           if(.isEqual(e2,1)) return(e1)

           ep <- getdistrOption("TruncQuantile")
           d00 <- discretePart(e1)@d(0)
           d0 <- discreteWeight(e1)*d00
           if(d0 > ep){
                if(.isEqual(d00,1)){
                   e1 <- acPart(e1)
                }else{
                   su <- support(discretePart(e1))
                   pr <- d(discretePart(e1))(su)
                   acW <- acWeight(e1)
                   discreteP <- DiscreteDistribution(
                                supp = su[su!=0],
                                prob = pr[su!=0]/(1-d00))
                   e1 <- UnivarLebDecDistribution(acPart = acPart(e1),
                         discretePart = discreteP, acWeight = acW)
                }
               }

           f.0 <- discretePart(e1)@.finSupport
           if(e2 > 0){
              e1pf <- f.0[2] & (f.0[1]|(e2%%2 == 1L))
              e1mf <- f.0[1] | (e2%%2 == 0L)
           }else{
              e1pf <- e1mf <- TRUE
           }

           e1DC <- decomposePM(e1)
           mixCoeff <- c(e1DC$pos$w,e1DC$neg$w)
           mixCoeff <- mixCoeff/sum(mixCoeff)
           e1p <- if(mixCoeff[1]>ep)
                    as(exp(e2*log(e1DC$pos$D)),"UnivarLebDecDistribution")
                 else as(Dirac(1), "UnivarLebDecDistribution")
           d1p <- discretePart(e1p)
           d1p@.finSupport <- c(TRUE,e1pf)
           discretePart(e1p) <- d1p
           e1m <- if(mixCoeff[2]>ep)
                    as((-1)^e2*exp(e2*log(-e1DC$neg$D)),"UnivarLebDecDistribution")
                 else as(Dirac((-1)^e2), "UnivarLebDecDistribution")
           d1m <- discretePart(e1m)
           d1m@.finSupport <- c(e1mf,TRUE)
           discretePart(e1m) <- d1m
           erg <- flat.LCD(mixCoeff = mixCoeff, e1p, e1m)

#
           if(d0 > ep){
                if(.isEqual(d00,1)){
                    erg <- UnivarLebDecDistribution(acPart = acPart(erg),
                           discretePart = Dirac(0), acWeight = acW)
                }else{
                    su <- support(discretePart(erg))
                    su0 <- c(su,0)
                    o <- order(su0)
                    pr <- c(d(discretePart(erg))(su) * (1-d00), d00)
                    suo <- su0[o]
                    pro <- pr[o]
                    discreteP <- DiscreteDistribution(supp = suo, prob = pro)
                    discreteP@.finSupport <- c(e1mf,e1pf)
                    erg <- UnivarLebDecDistribution(acPart = acPart(erg),
                           discretePart = discreteP, acWeight = acW)
                }
             }
           if(getdistrOption("simplifyD"))
                erg <- simplifyD(erg)

           rnew <- function(n, ...){}
           body(rnew) <- substitute({ g1(n, ...)^g2 },
                                    list(g1 = e1@r, g2 = e2))
           erg@r <- rnew
           return(erg)
           })

setMethod("^", c("AcDcLcDistribution","numeric"),
function(e1,e2){
  i <- 1; stopit <- FALSE; sL <- length(sys.calls())
  e1s <- "e1"; e2s <- "e2"
  while(!stopit && i <= sL){
    i <- i + 1
	trcall <- sys.call(sys.parent(i))
    myc <- paste(as.list(trcall)[[1]])
	if(any(myc == "^")){
	   mc <- match.call(call=trcall)
	   e1s <- as.character(deparse(mc$e1))
	   e2s <- as.character(deparse(mc$e2))
	   stopit <- TRUE
	}
  }  

  if (length(e2)>1) stop("length of operator must be 1")
  if (isTRUE(all.equal(e2,1))) return(e1)
  if (isTRUE(all.equal(e2,0))) return(Dirac(1))


 e1 <- .ULC.cast(e1)
#  if( is(e1,"AbscontDistribution") || is(e1,"DiscreteDistribution") ||
#      is(e1,"AffLinUnivarLebDecDistribution"))
#      e1 <- as(e1, "UnivarLebDecDistribution")

  if (e2<0) return((1/e1)^(-e2))

  if (.isNatural(e2, tol = 1e-10))
      return(get("^")(e1 = e1, e2 = as(e2,"Integer")))

  ep <- getdistrOption("TruncQuantile")
  d00 <- discretePart(e1)@d(0)
  d0 <- discreteWeight(e1)*d00
  p0 <- p(e1)(0)

  if ((p0 > ep && e2 < 0) || (p0 > d0+ep))
      stop(gettextf("%s^%s is not well-defined with positive probability ",
                    e1s, e2s))

  ### special treatment if e2>=0 and d.discrete(e1)(0)>0

  d1 <- discretePart(e1)
  if(d0 > ep){
     if(.isEqual(d00,1)){
        e1 <- acPart(e1)
     }else{
        su <- support(d1)
        pr <- d(d1)(su)
        acW <- acWeight(e1)#/(1-d0)
        discreteP <- DiscreteDistribution(
                     supp = su[su!=0],
                     prob = pr[su!=0]/(1-d00))
        discreteP@.finSupport <- c(TRUE, d1@.finSupport)
        e1 <- UnivarLebDecDistribution(acPart = acPart(e1),
              discretePart = discreteP, acWeight = acW)
     }
   }

   erg <- exp( e2 * log(e1))

   ### special treatment if e2>=0 and d.discrete(e1)(0)>0
   if(d0 > ep){
      if(.isEqual(d00,1)){
          erg <- UnivarLebDecDistribution(acPart = acPart(erg),
                 discretePart = Dirac(0), acWeight = acW)
      }else{
          acW <- acWeight(erg)
          erg.d <- discretePart(erg)
          su <- support(erg.d)
          su0 <- c(su,0)
          o <- order(su0)
          pr <- c(d(erg.d)(su) * (1-d00), d00)
          suo <- su0[o]
          pro <- pr[o] #/(1-acW)
          discreteP <- DiscreteDistribution(supp = suo, prob = pro)
          discreteP@.finSupport <- c(TRUE, d1@.finSupport)
          erg <- UnivarLebDecDistribution(acPart = acPart(erg),
                      discretePart = discreteP, acWeight = acW)
     }
   }

  if(getdistrOption("simplifyD"))
            erg <- simplifyD(erg)

  rnew <- function(n, ...){}
  body(rnew) <- substitute({ g1(n, ...)^g2 },
                            list(g1 = e1@r, g2 = e2))
  erg@r <- rnew

  return(erg)
  }
)

setMethod("^", c("AcDcLcDistribution","Dirac"),
function(e1,e2)e1^location(e2))

setMethod("^", c("AcDcLcDistribution","AcDcLcDistribution"),
function(e1,e2){
 ### check if there are problems
  i <- 1; stopit <- FALSE; sL <- length(sys.calls())
  e1s <- "e1"; e2s <- "e2"
  while(!stopit && i <= sL){
    i <- i + 1
	trcall <- sys.call(sys.parent(i))
    myc <- paste(as.list(trcall)[[1]])
	if(any(myc == "/")){
	   mc <- match.call(call=trcall)
	   e1s <- as.character(deparse(mc$e1))
	   e2s <- as.character(deparse(mc$e2))
	   stopit <- TRUE
	}
  }  
# if( is(e1,"AffLinUnivarLebDecDistribution"))
#     e1 <- as(e1, "UnivarLebDecDistribution")
# if( is(e2,"AffLinUnivarLebDecDistribution"))
#     e2 <- as(e2, "UnivarLebDecDistribution")

# if( is(e1,"AbscontDistribution"))
#     e1 <- as(as(e1,"AbscontDistribution"), "UnivarLebDecDistribution")
# if( is(e2,"AbscontDistribution"))
#     e2 <- as(as(e2,"AbscontDistribution"), "UnivarLebDecDistribution")


# if( is(e1,"DiscreteDistribution"))
#     e1 <- as(as(e1,"DiscreteDistribution"), "UnivarLebDecDistribution")
# if( is(e2,"DiscreteDistribution"))
#     e2 <- as(as(e2,"DiscreteDistribution"), "UnivarLebDecDistribution")

 e1 <- .ULC.cast(e1)
 e2 <- .ULC.cast(e2)


 ep <- getdistrOption("TruncQuantile")

 if(p(e2)(0)-discreteWeight(e2)*d.discrete(e2)(0)>ep)
    { ## must be able to work with negative exponents

         if (d.discrete(e1)(0)*discreteWeight(e1) > ep)
            stop(gettextf("%s^%s is not well-defined with positive probability ",
                 e1s, e2s))

         if ((discreteWeight(e2)>1-ep) && all(.isInteger(support(e2)))){
              Dlist <- lapply(support(e2), function(x)
                  as(do.call("^",list(e1=e1,e2=x)), "UnivarLebDecDistribution"))
              erg <- as(simplifyD( do.call(flat.LCD,
                        c(Dlist, alist(mixCoeff = d.discrete(e2)(support(e2)))))),
                        "UnivarLebDecDistribution")
              if(getdistrOption("simplifyD")) erg <- simplifyD(erg)
              return(erg)
              }

         if (p(e1)(0) > ep)
            stop(gettextf("%s^%s is not well-defined with positive probability ",
                 e1s, e2s))
    }

 if(p(e1)(0)>ep)
    { ## works only for purely natural e2


         if ((discreteWeight(e2)>1-ep) && all(.isInteger(support(e2)))){
              Dlist <- lapply(support(e2), function(x)
                  as(do.call("^",list(e1=e1,e2=x)), "UnivarLebDecDistribution"))
              erg <- as(simplifyD( do.call(flat.LCD,
                        c(Dlist, alist(mixCoeff = d.discrete(e2)(support(e2)))))),
                        "UnivarLebDecDistribution")
              if(getdistrOption("simplifyD")) erg <- simplifyD(erg)
              return(erg)
              }

         stop(gettextf("%s^%s is not well-defined with positive probability ",
                 e1s, e2s))
    }

 le1 <- log(e1)
 le <- le1 * e2
 erg <- exp(le)

 if(getdistrOption("simplifyD")) erg <- simplifyD(erg)

 rnew <- function(n, ...){}
 body(rnew) <- substitute({ g1(n, ...)^g2(n, ...) },
                            list(g1 = e1@r, g2 = e2@r))
 erg@r <- rnew

 return(erg)
})


setMethod("^", c("numeric","AcDcLcDistribution"),
function(e1,e2){
 ### check if there are problems
  i <- 1; stopit <- FALSE; sL <- length(sys.calls())
  e1s <- "e1"; e2s <- "e2"
  while(!stopit && i <= sL){
    i <- i + 1
	trcall <- sys.call(sys.parent(i))
    myc <- paste(as.list(trcall)[[1]])
	if(any(myc == "^")){
	   mc <- match.call(call=trcall)
	   e1s <- as.character(deparse(mc$e1))
	   e2s <- as.character(deparse(mc$e2))
	   stopit <- TRUE
	}
  }  

 e2 <- .ULC.cast(e2)
 #e2 <- .if( is(e2,"AffLinUnivarLebDecDistribution"))
 #    e2 <- as(e2, "UnivarLebDecDistribution")
 #if( is(e2,"AbscontDistribution"))
 #    e2 <- as(as(e2,"AbscontDistribution"), "UnivarLebDecDistribution")
 #if( is(e2,"DiscreteDistribution"))
 #    e2 <- as(as(e2,"DiscreteDistribution"), "UnivarLebDecDistribution")

 ep <- getdistrOption("TruncQuantile")
 if(p(e2)(0)-discreteWeight(e2)*d.discrete(e2)(0)>ep)
    { ## must be able to work with negative exponents

         if (abs(e1) < ep)
             stop(gettextf("%s^%s is not well-defined with positive probability ",
                 e1s, e2s))

         if ((discreteWeight(e2)>1-ep) && all(.isInteger(support(e2)))){
              erg <- DiscreteDistribution(e1^support(e2),
                                          d.discrete(e2)(support(e2)))
              erg@.finSupport <- c(TRUE, discretePart(e2)@.finSupport[2])
              if(e1<0){
                 de2 <- discretePart(e2)
                 su <- support(e2)
                 oddS <- su[su%%2==1L]
                 podd <- sum(d.discrete(e2)(oddS))
                 peven <- 1-podd
                 erg@.finSupport <- c(de2@.finSupport[2]|(podd<ep^2),
                                         de2@.finSupport[2]|(peven<ep^2))
              }
              if(!getdistrOption("simplifyD"))
                  erg <- as(erg,"UnivarLebDecDistribution")
              return(erg)
             }

         if (e1 < -ep)
            stop(gettextf("%s^%s is not well-defined with positive probability ",
                 e1s, e2s))
    }
 if(e1< -ep)
    { ## works only for purely natural e2

         if ((discreteWeight(e2)>1-ep) && all(.isInteger(support(e2)))){
              erg <- DiscreteDistribution(e1^support(e2),
                                          d.discrete(e2)(support(e2)))
              erg@.finSupport <- c(TRUE, discretePart(e2)@.finSupport[2])
              if(e1<0){
                 de2 <- discretePart(e2)
                 su <- support(e2)
                 oddS <- su[su%%2==1L]
                 podd <- sum(d.discrete(e2)(oddS))
                 peven <- 1-podd
                 erg@.finSupport <- c(de2@.finSupport[2]|(podd<ep^2),
                                         de2@.finSupport[2]|(peven<ep^2))
              }
             if(!getdistrOption("simplifyD"))
                 erg <- as(erg,"UnivarLebDecDistribution")
             return(erg)
             }

         stop(gettextf("%s^%s is not well-defined with positive probability ",
                 e1s, e2s))
    }
  le1 <- log(e1)
  le <- le1 * e2
  erg <- exp(le)
  if(getdistrOption("simplifyD")) erg <- simplifyD(erg)

  rnew <- function(n, ...){}
  body(rnew) <- substitute({ g1^g2(n, ...) },
                            list(g1 = e1, g2 = e2@r))
  erg@r <- rnew

  return(erg)
})

setMethod("+", signature(e1="AcDcLcDistribution", e2="AcDcLcDistribution"),
           function(e1,e2)(.ULC.cast(e1)+(-.ULC.cast(e2))))
setMethod("-", signature(e1="AcDcLcDistribution", e2="AcDcLcDistribution"),
           function(e1,e2)(.ULC.cast(e1)+(-.ULC.cast(e2))))


setMethod("sign", "AcDcLcDistribution",
            function(x){ 
            if(is(x,"AbscontDistribution")) d0 <-0
            else if(is(x,"DiscreteDistribution")) d0 <- d(x)(0)
            else d0 <- d.discrete(as(x,UnivarLebDecDistribution))(0)
            pm <- p(x)(-getdistrOption("TruncQuantile"))
            pp <- p(x)(getdistrOption("TruncQuantile"), lower=FALSE)
            Symmetry <- NoSymmetry()
            if(.isEqual(pm,pp))
               Symmetry <- SphericalSymmetry(0)
            DiscreteDistribution(supp = c(-1,0,1), prob = c(pm, d0, pp))
            })

setMethod("sqrt", "AcDcLcDistribution",
            function(x) x^0.5)

setMethod("Math", "AcDcLcDistribution",
          function(x) callGeneric(.ULC.cast(x)))

################################
##
## Class: HyperParameter
##
################################


## Access Methods
setMethod("m", "HyperParameter", function(object) object@m)
setMethod("n", "HyperParameter", function(object) object@n)
setMethod("k", "HyperParameter", function(object) object@k)
## Replace Methods
setReplaceMethod("m", "HyperParameter", 
                  function(object, value){ object@m <- value; object})
setReplaceMethod("n", "HyperParameter", 
                  function(object, value){ object@n <- value; object})
setReplaceMethod("k", "HyperParameter", 
                  function(object, value){ object@k <- value; object})




setValidity("HyperParameter", function(object){
  if(length(m(object)) != 1)
    stop("m has to be a numeric of length 1")    
  if(m(object) < 0)
    stop("m has to be a not negative natural")
  if(length(n(object)) != 1)
    stop("n has to be a numeric of length 1")    
  if(n(object) < 0)
    stop("n has to be a not negative natural")
  if(length(k(object)) != 1)
    stop("k has to be a numeric of length 1")    
  if(k(object) < 0)
    stop("k has to be a not negative natural")
  
  if(!identical(floor(m(object)), m(object)))
    stop("m has to be a not negative natural")
  if(!identical(floor(n(object)), n(object)))
    stop("n has to be a not negative natural")
  if(!identical(floor(k(object)), k(object)))
    stop("k has to be a not negative natural")
  if(k(object) > m(object) + n(object))
    stop("k has to be less or equal than m + n")    
  else return(TRUE)
})



################################
##
## Class: hypergeometric distribution
##
################################

Hyper <- function(m = 1, n = 1, k = 1) new("Hyper", m = m, n = n, k = k)

## wrapped access methods
setMethod("m", "Hyper", function(object) m(param(object)))
setMethod("n", "Hyper", function(object) n(param(object)))
setMethod("k", "Hyper", function(object) k(param(object)))
## wrapped replace methods
setMethod("m<-", "Hyper", 
           function(object, value) 
                    new("Hyper", m = value, n = n(object), k = k(object)))
setMethod("n<-", "Hyper", 
           function(object, value) 
                    new("Hyper", m = m(object), n = value, k = k(object)))
setMethod("k<-", "Hyper", 
           function(object, value) 
                    new("Hyper", m = m(object), n = n(object), k = value))

### elementary version management:
##  detects missing slots

setMethod("isOldVersion", "ANY",
# isold <-
function(object) {
   if(!isClass(class(object)))
      stop("'isOldVersion()' only works for formal S4-Classes.")
   slotNames <- slotNames(object)
   ers <- sapply(slotNames, function(x)
                is(try(slot(object, x), silent = TRUE), "try-error"))
   error <- any(ers)
   if(error)
      { warning(gettextf(
        paste("Object '%s' was defined under a deprecated version of class '%s'.",
              "Slot[s] %s is[are] missing.",
              "Try conv2NewVersion().", sep = "\n"),
              deparse(substitute(object)),
              class(object),
              paste("'",slotNames[ers],"'",sep="", collapse=", "))
               )
         return(TRUE)
   }else return(FALSE)     }
)

setMethod("conv2NewVersion", "ANY",
function(object) {
           tryobject <- new(class(object))
           slotNames <- slotNames(object)
           getIfExists <- function(x)
                    if (!is(try(slot(object, x), silent = TRUE),"try-error"))
                         slot(object,x) else slot(tryobject,x)
           lst <- sapply(slotNames, function(x) getIfExists(x))
           names(lst) <- slotNames
           #
           lst <- c(list(Class = class(object)), lst)
           if(is(try (myobj <- do.call("new", args = lst), silent = TRUE), "try-error")){
              myobj <- tryobject
              for(i in 2:length(lst)) 
                  slot(myobj, name=names(lst)[i]) <- lst[[i]]
           }           
           myobj
           })

setMethod("conv2NewVersion", "LatticeDistribution",
function(object) {
           slotNames <- slotNames(param(object))
           slotNames <- slotNames[slotNames!="name"]
           lst <- lapply(slotNames, function(x) slot(param(object),x))
           names(lst) <- slotNames
           lst <- c(list(Class = class(object)), lst)
           myobj <- do.call("new", args = lst)
           myobj
            })

setMethod("width", signature(object="Lattice"), function(object) object@width)
setMethod("Length", signature(object="Lattice"), function(object) object@Length)
setMethod("pivot", signature(object="Lattice"), function(object) object@pivot)

setReplaceMethod("width",  signature(object="Lattice"), 
          function(object, value) {object@width <- value; object})
setReplaceMethod("Length", signature(object="Lattice"), 
          function(object, value) {object@Length <- value; object})
setReplaceMethod("pivot",  signature(object="Lattice"), 
          function(object, value) {object@pivot <- value; object})

.distrExInstalled <- !all(is.na(packageDescription("distrEx")))

.distroptions <- list(
                      DefaultNrGridPoints = 2^12,
                      DistrResolution = 1e-6,
                      TruncQuantile = 1e-5,
                      DefaultNrFFTGridPointsExponent = 12,
                      RtoDPQ.e = 5, 
                      # new Warning-items P.R. 28.03.06
                      WarningArith = TRUE,
                      WarningSim = TRUE,
                      ## new Items from 2.0:
                      withgaps = TRUE,
                      simplifyD = TRUE,
                      DistrCollapse = TRUE,
                      withSweave = FALSE,
                      ## new Items after mail by Jacob van Etter, 27-02-09
                      DistrCollapse.Unique.Warn = FALSE,
                      use.generalized.inverse.by.default = TRUE,
                      ## new Item after annoying warnings with GEV 28-01-13
                      warn.makeDNew = TRUE
                      )
distroptions <- function(...) {
  if (nargs() == 0) return(.distroptions)
  current <- .distroptions
  temp <- list(...)
  if (length(temp) == 1 && is.null(names(temp))) {
    arg <- temp[[1]]
    switch(mode(arg),
           list = temp <- arg,
           character = return(.distroptions[arg]),
           stop("invalid argument: ", sQuote(arg)))
  }
  if (length(temp) == 0) return(current)
  n <- names(temp)
  if (is.null(n)) stop("options must be given by name")
  changed <- current[n]
  current[n] <- temp
  env <- if (sys.parent() == 0) asNamespace("distr") else parent.frame()
  assign(".distroptions", current, envir = env)
  invisible(current)
}

getdistrOption <- function(x)distroptions(x)[[1]]

options("newDevice" = FALSE)

# we only run the tests, if svUnit is available
if (require(svUnit, quietly=TRUE)) {
  pkg <- "distr"
  require("distr")
  
  # we must investigate whether R CMD check is running or not
  #   and if the check is running, whether a time limit exists
  RCMDCHECK <- FALSE
  RCMDCHECKCRAN <- FALSE
  
  for (actual.name in names(Sys.getenv())) {
    if (substr(actual.name, 1, 9) == "_R_CHECK_") {
      RCMDCHECK <- TRUE
      
      if (actual.name == "_R_CHECK_TIMINGS_") {
        RCMDCHECKCRAN <- (as.numeric(Sys.getenv("_R_CHECK_TIMINGS_")) > 0)
      }
    }
  }
  
  # we must determine the path for tests in the installation and outside installation
  if (RCMDCHECK) {
    ## Path to unit tests for R CMD check
    ## PKG.Rcheck/tests/../PKG/unitTests
    ## PKG.Rcheck/tests/unitTests
    
    # we determine the two paths
    pathTestsInInstallation <- system.file(package=pkg, "unitTests")
    pathTestsOutsideInstallation <- file.path(getwd(), "unitTests")
  } else {
    ## Path to unit tests for standalone running as script with "PKG/tests" as working directory
    ## PKG/tests/../inst/unitTests
    ## PKG/tests/unitTests
    
    # we determine the two paths
    pathTestsInInstallation <- file.path(getwd(), "..", "inst", "unitTests")
    pathTestsOutsideInstallation <- file.path(getwd(), "unitTests")
  }
  
  print(pathTestsInInstallation)
  print(pathTestsOutsideInstallation)
  
  # it depends whether we want to skip the long running tests or not
  if (RCMDCHECKCRAN) {
    mypkgSuite <- svSuiteList(packages=pkg, dirs=pathTestsInInstallation)
  } else {
    mypkgSuite <- svSuiteList(packages=pkg, dirs=c(pathTestsInInstallation, pathTestsOutsideInstallation))
  }
  
  unlink("report.txt")  # Make sure we generate a new report
  
  print(svSuiteList(packages=FALSE, dirs=c(pathTestsInInstallation, pathTestsOutsideInstallation)))
  
  runTest(mypkgSuite, name = pkg)  # Run them...
  
  ## makeTestListFromExamples is in svUnit 0.7.8 or more
  #doRunExamples <- TRUE
  #svUnitVersion = as.integer(strsplit(installed.packages()[which(installed.packages()[, 'Package'] == "svUnit"), "Version"], "[\\.-]")[[1]])
  #if (svUnitVersion[1] == 0) {
  #  if (svUnitVersion[2] < 7) {
  #    doRunExamples <- FALSE
  #  } else {
  #    if (svUnitVersion[2] == 7)
  #      doRunExamples <- svUnitVersion[3] >= 8
  #  }
  #}
  #if(doRunExamples)
  #  runTest(tryCatch(makeTestListFromExamples(pkg, "../../pkg/man/"), error=function(e) NULL))
  
  
  protocol(Log(), type = "text", file = "report.txt")  # ... and write report
}

##########################################################
## Function for n-fold convolution          
## -- absolute continuous distribution --
##########################################################

##implentation of Algorithm 3.4. of
#P. Ruckdeschel, M. Kohl (2014): General Purpose Convolution Algorithm for
# Distributions in S4 Classes by Means of FFT. J. Statist. Softw. 59(4), 1-25.

setMethod("convpow",
          signature(D1 = "AbscontDistribution"),
          function(D1, N){
            if( !.isNatural0(N))
              stop("N has to be a natural (or 0)")
            if (N==0) return(Dirac(0))
            if (N==1) return(D1)
    ##STEP 1

            lower <- getLow(D1);  upper <- getUp(D1);

    ##STEP 2

    ## binary logarithm of the effective number of gridpoints
            m <- max(getdistrOption("DefaultNrFFTGridPointsExponent") -
                 floor(log(N)/log(2)),5)
            M <- 2^m
            Nl <-2^ceiling(log(N)/log(2))

            h <- (upper-lower)/M
            dp1 <- .discretizeP(D1, lower, upper, h)

    ##STEP 3

            dpn0 <- c(dp1, numeric((Nl-1)*M))
    ##STEP 4

            ## computation of DFT
            fftdpn <- fft(dpn0)

    ##STEP 5

            ## convolution theorem for DFTs
            dpn <- c(0,(Re(fft(fftdpn^N, inverse = TRUE)) / (Nl*M))[1:(N*M-N+2)])

            x <- seq(from = N*lower+N/2*h, to = N*upper-N/2*h, by = h)
            x <- c(x[1]-h, x[1], x+h)

            ## density  (steps 5--7)

            dfun <- .makeDNew(x, dpn, h)

            ## cdf (steps 5--7)
            pfun <- .makePNew(x, dpn, h, .notwithLArg(D1))

            ## continuity correction by h/2

            ## quantile function
            yL <-  if  (q.l(D1)(0) == -Inf) -Inf  else  N*lower
            yR <-  if  (q.l(D1)(1) ==  Inf)  Inf  else  N*upper
            px.l <- pfun(x + 0.5*h)
            px.u <- pfun(x + 0.5*h, lower.tail = FALSE)
            qfun <- .makeQNew(x + 0.5*h, px.l, px.u, .notwithLArg(D1), yL, yR)

            rfun = function(n) colSums(matrix(r(D1)(n*N), ncol=n))

            object <- new("AbscontDistribution", r = rfun, d = dfun, p = pfun,
                       q = qfun, .withArith = TRUE, .withSim = FALSE)

            if(is(D1@Symmetry,"SphericalSymmetry"))
               object@Symmetry <- SphericalSymmetry(N*SymmCenter(D1@Symmetry))   

            rm(m, dpn, dp1, dpn0, fftdpn)
            rm(h, px.u, px.l, rfun, dfun, qfun, pfun, upper, lower)
           return(object)
})

setMethod("convpow",
          signature(D1 = "LatticeDistribution"),
          function(D1, N, ep = getdistrOption("TruncQuantile")){
            if( !.isNatural0(N))
              stop("N has to be a natural (or 0)")
            if (N==0) return(Dirac(0))
            if (N==1) return(D1)

            if(!is.numeric(ep)) stop("argument 'ep' must be a numeric.")
            if(length(ep)!=1) stop("argument 'ep' must be a numeric of length 1.")
            if((ep<0)||(ep>1)) stop("argument 'ep' must be in (0,1).")

            w <- width(lattice(D1))

            supp0 <- support(D1)
            supp1 <- seq(by=abs(w),from=N*min(supp0),to=N*max(supp0))

            d1 <- d(D1)(supp0); d1 <- c(d1,numeric((length(supp0)-1)*(N-1)))

            ## computation of DFT
            ftde1 <- fft(d1)

            ## convolution theorem for DFTs
            newd <- Re(fft(ftde1^N, inverse = TRUE)) / length(ftde1)
            newd <- (newd >= .Machine$double.eps)*newd

            rsum.u <- min( sum( cumsum(rev(newd)) > ep/2)+1, length(supp1))
            rsum.l <- max( sum( cumsum(newd) < ep/2), 1)

            newd <- newd[rsum.l:rsum.u]
            newd <- newd/sum(newd)
            supp1 <- supp1[rsum.l:rsum.u]
            
            supp2 <- supp1[newd>ep]
            newd2 <- newd[newd>ep]
            newd2 <- newd2/sum(newd2)

            Symmetry <- NoSymmetry()
            if(is(D1@Symmetry,"SphericalSymmetry"))
               Symmetry <- SphericalSymmetry(N*SymmCenter(D1@Symmetry))   
            
            if( length(supp1) >= 2 * length(supp2))
               return(DiscreteDistribution(supp = supp2, prob = newd2,
                                           .withArith = TRUE, Symmetry = Symmetry))
            else  
               return(LatticeDistribution(supp = supp1, prob = newd,
                                          .withArith = TRUE, Symmetry = Symmetry))
})

###############################################################################
#
# new from 2.0: convpov for  AcDcLcDistribution
#
###############################################################################
#
setMethod("convpow",
          signature(D1 = "AcDcLcDistribution"),
          function(D1, N, ep = getdistrOption("TruncQuantile")){
            if( !.isNatural0(N))
              stop("N has to be a natural (or 0)")
            if (N==0) return(Dirac(0))
            if (N==1) return(D1)
        e1 <- as(D1, "UnivarLebDecDistribution")
        if(is(e1,"DiscreteDistribution")) return(convpow(e1,N))
        if(is(e1,"AbscontDistribution")) return(convpow(e1,N))

            if(!is.numeric(ep)) stop("argument 'ep' must be a numeric.")
            if(length(ep)!=1) stop("argument 'ep' must be a numeric of length 1.")
            if((ep<0)||(ep>1)) stop("argument 'ep' must be in (0,1).")

        aw1 <- acWeight(e1)
        dw1 <- 1-aw1
        dD1 <- discretePart(e1)
        aD1 <- acPart(e1)
        dD1 <- discretePart(e1)
        if(is(dD1,"LatticeDistribution"))
           dD1 <- as(dD1,"LatticeDistribution")
  #      dDm <- max(d.discrete(e1)(support(e1)))*dw1

        if(aw1<ep) return(convpow(dD1,N))
        if(dw1<ep) return(convpow(aD1,N))

        maxN <- ceiling(2*log(ep)/log(dw1))
        Nm <- min(maxN,N)
        Mm <- N%/%Nm
        Rm <- N-Mm*Nm
   
        sumM <- function(mm){
                db <- dbinom(0:mm, size = mm, prob = aw1)                
                im <- (0:mm)[db>ep^2]
                db <- db[db>ep^2]
                db <- db/sum(db)
                if(length(im)>1){
                      DList <- lapply(im,
                                function(x) {
                                   S.a <- convpow(aD1, x)
                                   S.d <- convpow(dD1, mm-x) #as(dD1,
                                          #  "DiscreteDistribution"), mm-x)
                                   as(S.a+S.d,"UnivarLebDecDistribution")
                               }) 
                      erg <- do.call(flat.LCD, c(DList, alist(mixCoeff = db)))
                }else{
                      DList <- as(convpow(aD1,im)+convpow(dD1,mm-im),"UnivarLebDecDistribution")           
                      erg <- flat.LCD(DList, mixCoeff = 1)
                      } 
                return(erg)
        }
        
        erg <- sumM(Nm)
        if(Mm>1) erg <- convpow(erg,Mm,ep=ep)
        if(Rm>0) erg <- sumM(Rm)+ as(erg,"UnivarLebDecDistribution")
        if(is(erg,"UnivarLebDecDistribution")) erg <- simplifyD(erg)

        if(is(D1@Symmetry,"SphericalSymmetry"))
              erg@Symmetry <- SphericalSymmetry(N*SymmCenter(D1@Symmetry))   
        return(erg)
})
#
###############################################################################
setMethod("convpow",
          signature(D1 = "DiscreteDistribution"),
          function(D1, N){
            if( !.isNatural0(N))
              stop("N has to be a natural (or 0)")
            if (N==0) return(Dirac(0))
            if (N==1) return(D1)
            if (N==2) return(D1+D1)
            DN1 <- convpow(D1,N%/%2)
            DN1 <- DN1 + DN1
            if (N%%2==1) DN1 <- DN1+D1 
            return(DN1)
            })
###############################################################################
            
setMethod("convpow",
          signature(D1 = "Norm"),
          function(D1, N) 
             {if( !.isNatural0(N))
                  stop("N has to be a natural (or 0)")
              if (N==0) return(Dirac(0))
              if(N==1)  D1 else Norm(mean = N*mean(D1), sd = sqrt(N)*sd(D1))}
           )

setMethod("convpow",
          signature(D1 = "Pois"),
          function(D1, N) 
             {if( !.isNatural0(N))
                  stop("N has to be a natural (or 0)")
              if (N==0) return(Dirac(0))
              if(N==1) D1 else  Pois(lambda=N*lambda(D1))
             }
          )

setMethod("convpow",
          signature(D1 = "Binom"),
          function(D1, N) 
             {if( !.isNatural0(N))
                  stop("N has to be a natural (or 0)")
              if (N==0) return(Dirac(0))
              if(N==1) D1 else  Binom(size=N*size(D1),prob=prob(D1))}
          )

setMethod("convpow",
          signature(D1 = "Nbinom"),
          function(D1, N) 
             {if( !.isNatural0(N))
                  stop("N has to be a natural (or 0)")
              if (N==0) return(Dirac(0))
              if(N==1) D1 else  Nbinom(size=N*size(D1),prob=prob(D1))}
          )

#setMethod("convpow",
#          signature(D1 = "Gammad"),
#          function(D1, N) 
#            {if((N<1)||(abs(floor(N)-N)>.Machine$double.eps))
#               stop("N has to be a natural greater than or equal to  1")
#              if(N==1) D1 else  Gammad(shape=N*shape(D1),scale=scale(D1))}
#          )

setMethod("convpow",
          signature(D1 = "Dirac"),
          function(D1, N) 
             {if( !.isNatural0(N))
                  stop("N has to be a natural (or 0)")
              if (N==0) return(Dirac(0))
              Dirac(location =N*location(D1))}
          )

setMethod("convpow",
          signature(D1 = "ExpOrGammaOrChisq"),
          function(D1, N) 
             {if( !.isNatural0(N))
                  stop("N has to be a natural (or 0)")
              if (N==0) return(Dirac(0))
              if(N==1) return(D1) 
                 else  if(is(D1,"Gammad")) 
                          {D1 <- as(D1,"Gammad")
                           return(Gammad(shape = N*shape(D1),
                                   scale = scale(D1))) }
                 else convpow(as(D1, "AbscontDistribution"),N)}
          )

 setMethod("convpow",
          signature(D1 = "Cauchy"),
          function(D1, N) 
             {if( !.isNatural0(N))
                  stop("N has to be a natural (or 0)")
              if (N==0) return(Dirac(0))
              if(N==1)  D1 else Cauchy(location = N*location(D1), 
                                       scale = N*scale(D1))}
           )

# generating function for class 'PosDefSymmMatrix'
PosSemDefSymmMatrix <- function(mat){
    if(is(mat, "PosDefSymmMatrix")) return(as(mat, "PosSemDefSymmMatrix"))
    if(!is.matrix(mat)) mat <- as.matrix(mat)
    new("PosSemDefSymmMatrix", mat)
}

PosDefSymmMatrix <- function(mat){ 
    if(!is.matrix(mat)) mat <- as.matrix(mat)
    new("PosDefSymmMatrix", mat)
}

################################
##
## Class: ChisqParameter
##
################################

## Access Methods
setMethod("df", "ChisqParameter", function(x, df1, df2, log = FALSE) x@df)
setMethod("ncp", "ChisqParameter", function(object) object@ncp)
## Replace Methods
setReplaceMethod("df", "ChisqParameter", function(object, value)
                 { object@df <- value; object})
setReplaceMethod("ncp", "ChisqParameter", function(object, value)
                 { object@ncp <- value; object})


setValidity("ChisqParameter", function(object){
  if(length(df(object)) != 1)
    stop("df has to be a numeric of length 1")    
  if(df(object) <= 0 + .Machine$double.eps^.5 )
    stop("df has to be positive")
  if(length(ncp(object)) != 1)
    stop("ncp has to be a numeric of length 1")    
  if(ncp(object) < 0)
    stop("ncp has to be not negative")
  else return(TRUE)  })


################################
##
## Class: Chi squared distribution
##
################################

Chisq <- function(df = 1, ncp = 0) new("Chisq", df = df, ncp = ncp)

## wrapped access methods
setMethod("df", "Chisq", function(x, df1, df2, log = FALSE) df(param(x)))
setMethod("ncp", "Chisq", function(object) ncp(param(object)))

## wrapped replace methods
setMethod("df<-", "Chisq", 
           function(object, value) 
               new("Chisq", df = value, ncp = ncp(object)))
setMethod("ncp<-", "Chisq", 
           function(object, value) new("Chisq", df = df(object), ncp = value))

setMethod("+", c("Chisq","Chisq"),
          function(e1,e2){
            newdf <- df(e1) + df(e2)
            newncp <- ncp(e1) + ncp(e2)
            return(new("Chisq", df = newdf, ncp = newncp, .withArith = TRUE))
          })

### new from version 1.9 on:

setMethod("shape", "Chisq", 
           function(object){ 
               if (isTRUE(all.equal(ncp(object),0))) 
                   df(object)/2
               else stop(gettextf("%s is not a Gamma distribution"),
                                   deparse(substitute(object))
                         )
                            }
           )

setMethod("scale", "Chisq", 
           function(x, center = TRUE, scale = TRUE){ 
               if (isTRUE(all.equal(ncp(x),0))) 
                                   2
               else stop(gettextf("%s is not a Gamma distribution"),
                                   deparse(substitute(object))
                         )
                            }
           )

###### end new version 1.9

.makedotsLowLevel <- function(dots){
       dots$sub <- dots$xlab <- dots$ylab <- dots$main <- dots$type <- NULL
       dots$xlim <- dots$ylim <- dots$yaxt <- dots$axes <- dots$xaxt <- NULL
       dots$panel.last <- dots$panel.first <- dots$frame.plot <- dots$ann <-NULL
       dots$log <- dots$bg <- dots$asp <- NULL
       return(dots)
}
.makedotsL <- function(dots){
    dots <- .makedotsLowLevel(dots)
    dots$pch <- dots$cex <- NULL
    dots
}
.makedotsP <- function(dots){
    dots <- .makedotsLowLevel(dots)
    dots$lwd <- dots$lty <- NULL
    dots
}
.makedotsPt <- function(dots){
    dots <- dots[names(dots) %in% c("bg", "lwd", "lty")]
    dots["pch"] <- NULL
    if (length(dots) == 0 ) dots <- NULL
    dots
}

################################
##
## Class: BetaParameter
##
################################


## Access methods

setMethod("shape1", "BetaParameter", function(object) object@shape1)
setMethod("shape2", "BetaParameter", function(object) object@shape2)
setMethod("ncp", "BetaParameter", function(object) object@ncp)

## Replace Methoden

setReplaceMethod("shape1", "BetaParameter", 
                  function(object, value){ object@shape1 <- value; object})
setReplaceMethod("shape2", "BetaParameter", 
                  function(object, value){ object@shape2 <- value; object})
setReplaceMethod("ncp", "BetaParameter", 
                  function(object, value){ object@ncp <- value; object})


setValidity("BetaParameter", function(object){
  if(length(shape1(object)) != 1)
    stop("shape1 has to be a numeric of length 1")    
  if(shape1(object) <= 0)
    stop("shape1 has to be positive")
  if(length(shape2(object)) != 1)
    stop("shape2 has to be a numeric of length 1")    
  if(shape2(object) <= 0)
    stop("shape2 has to be positive")
  if(length(ncp(object)) != 1)
    stop("ncp has to be a numeric of length 1")      
  else return(TRUE)
}
)


################################
##
## Class: BetaDistribution
##
################################

Beta <- function(shape1 = 1, shape2 = 2, ncp = 0) 
                 new("Beta", shape1 = shape1, shape2 = shape2, ncp = ncp)

## wrapped access methods
setMethod("shape1", "Beta", function(object) shape1(param(object)))
setMethod("shape2", "Beta", function(object) shape2(param(object)))
setMethod("ncp", "Beta", function(object) ncp(param(object)))

## wrapped replace methods
setMethod("shape1<-", "Beta", 
           function(object, value) new("Beta", shape1 = value, 
                                    shape2 = shape2(object), ncp = ncp(object)))
setMethod("shape2<-", "Beta", 
           function(object, value) new("Beta", shape1 = shape1(object), 
                                        shape2 = value, ncp = ncp(object)))
setMethod("ncp<-", "Beta", 
           function(object, value) new("Beta", shape1 = shape1(object), 
                                        shape2 = shape2(object), ncp = value))

setMethod("-", c("numeric","Beta"),
           function(e1, e2) {if(isTRUE(all.equal(e1,1))&&
                                isTRUE(all.equal(ncp(e2),0)))
                             return(Beta(shape1=shape2(e2),shape2=shape1(e2)))
                             else e1-as(e2,"AbscontDistribution")})

## borrowed from Andy Liaw's rfNews()


distrARITH <- function(library = NULL) 
{
    infoShow(pkg = "distr", filename = "ARITHMETICS", library = library)
}

distrMASK <- function(library = NULL) 
{
    infoShow(pkg = "distr", filename="MASKING", library = library)
}

############# preparations ################
# (.onload, .onattach ...)
### left out P.R. 01-09-11
##.onLoad <- function(lib, pkg) { # extended 03-28-06: P.R. 
##    require("methods", character = TRUE, quietly = TRUE)
##}



.onAttach <- function(library, pkg)
{
  unlockBinding(".distroptions", asNamespace("distr"))
  unlockBinding(".distrExInstalled", asNamespace("distr"))

## global variable needed for flat.R
##  unlockBinding(".OkTyp", asNamespace("distr"))
    msga <- gettext(
    "Attention: Arithmetics on distribution objects are understood as "
                   )
    msgb <- gettext(
    "operations on corresponding random variables (r.v.s); see distrARITH().\n"
                   )
    msgc <- gettext(
    "Some functions from package 'stats' are intentionally masked ---see distrMASK().\n"
                   )
    msgd <- gettext(
    "Note that global options are controlled by distroptions() ---c.f. ?\"distroptions\"."
                   )
buildStartupMessage(pkg = "distr", msga, msgb, msgc, msgd, library = library, 
                    packageHelp = TRUE, 
# MANUAL = "http://www.uni-bayreuth.de/departments/math/org/mathe7/DISTR/distr.pdf",
                    VIGNETTE = gettext(
"Package \"distrDoc\" provides a vignette to this package as well as to several extension packages; try vignette(\"distr\")."
                                      )
                   )
  invisible()
} 

################################
##
## Optional..-classes
##
################################
setClassUnion("OptionalMatrix", 
               c("matrix","NULL")
               )
### from Matthias' thesis / ROptEst
## optional numeric
setClassUnion("OptionalNumeric", c("numeric", "NULL"))

################################
##
## utility classes 
##
################################

setClass("Integer", contains ="numeric",
          validity = function(object) all(.isInteger(object)))

################################
##
## space classes 
##
################################

## virtal Class: rSpace
setClass("rSpace", 
          representation = representation(name = "character"), 
          prototype = prototype(name = gettext("a space")), 
          contains = "VIRTUAL"
          )

## Class: EuclideanSpace
setClass("EuclideanSpace", 
          representation = representation(dimension = "numeric"), 
          contains = "rSpace",
          prototype = prototype(dimension = 1, 
                                name = gettext("Euclidean Space")
                                )
         )

## Class: Reals
setClass("Reals",  
          contains = "EuclideanSpace"
          )


## Class: Lattice
setClass("Lattice", 
          representation = representation(pivot = "numeric", width = "numeric", 
                                          Length = "numeric"
### masking not possible here -> Length instead of length
                                          ),
          prototype = prototype(pivot = 0, width = 1, Length = 2, 
                                name = gettext("a default lattice")
                                ),
          contains = "rSpace"
         )

## Class: Naturals
setClass("Naturals", 
          contains = "Reals"
          )

################################
##
## parameter classes
##
################################

setClass("Parameter", 
          representation = representation(name = "character"), 
          prototype = prototype(name = gettext("a parameter"))
          )

setClassUnion("OptionalParameter", 
               c("Parameter","NULL")
               )


## Class: ChisqParameter
setClass("ChisqParameter", 
          representation = representation(df = "numeric", ncp = "numeric"), 
          prototype = prototype(df = 1, ncp = 0, 
                      name = gettext("Parameter of a Chisq distribution")
                      ), 
          contains = "Parameter"
          )

### Class: DiracParameter
setClass("DiracParameter", 
          representation = representation(location = "numeric"), 
          prototype = prototype(location = 0, 
                      name = gettext("Parameter of a Dirac distribution")
                      ), 
          contains = "Parameter"
          )

## Class: ExpParameter
setClass("ExpParameter", 
          representation = representation(rate = "numeric"), 
          prototype = prototype(rate = 1, name = 
                      gettext("Parameter of an Exponential distribution")
                      ),  
          contains = "Parameter"
          )

## Class: GammaParameter
setClass("GammaParameter", 
          representation = representation(shape = "numeric", scale = "numeric"), 
          prototype = prototype(shape = 1, scale = 1, 
                      name = gettext("Parameter of a Gamma distribution")
                      ), 
          contains = "Parameter"
          )

## Class: PoisParameter
setClass("PoisParameter", 
          representation = representation(lambda = "numeric"), 
          prototype = prototype(lambda = 1, 
                      name = gettext("Parameter of a Poisson distribution")
                      ), 
          contains = "Parameter"
          )

## Class: NbinomParameter
setClass("NbinomParameter", 
          representation = representation(size = "numeric", prob = "numeric"), 
          prototype = prototype(size = 1, prob = 0.5, name = 
                      gettext("Parameter of a Negative Binomial distribution")
                      ), 
          contains = "Parameter"
          )

## Class: HyperParameter
setClass("HyperParameter", 
          representation = representation(m = "numeric", n = "numeric", 
                                          k = "numeric"
                                          ), 
          prototype = prototype(m = 1, n = 1, k = 1, name = 
                      gettext("Parameter of a Hypergeometric distribution")
                      ), 
          contains = "Parameter"
          )

## Class: BinomParameter
setClass("BinomParameter", 
          representation = representation(size = "numeric", prob = "numeric"), 
          prototype = prototype(size = 1, prob = 0.5, name = 
                      gettext("Parameter of a Binomial distribution")
                      ), 
          contains = "Parameter"
          )

#-
## no longer needed: this is a negBinom with size 1 no longer 
#-
### !!! deprecated as of version 1.9 !!!
## defunct as of 2.8.0
##
## Class: GeomParameter   
#setClass("GeomParameter",
#          representation = representation(prob = "numeric"),
#          prototype = prototype(prob = 0.5, name =
#                      gettext("Parameter of a Geometric distribution")
#                      ),
#          contains = "Parameter"
#          )
### !!! end of deprecated !!! of defunct

## Class: CauchyParameter
setClass("CauchyParameter", 
          representation = representation(location = "numeric", 
                                          scale = "numeric"
                                          ), 
          prototype = prototype(location = 0, scale = 1, name = 
                      gettext("Parameter of a Cauchy distribution")
                      ), 
          contains = "Parameter"
          )

## Class: NormParameter
setClass("NormParameter", 
          representation = representation(mean = "numeric", sd = "vector"), 
          prototype = prototype(mean = 0, sd = 1, name = 
                      gettext("Parameter of a Normal distribution")
                      ), 
          contains = "Parameter"
          )

## Class: UniNormParameter
setClass("UniNormParameter", 
          prototype = prototype(name = 
                      gettext("Parameter of a univariate Normal distribution")
                      ), 
          contains = "NormParameter"
          )

## Class: UnifParameter
setClass("UnifParameter", 
          representation = representation(Min = "numeric", Max = "numeric"), 
          prototype = prototype(Min = 0, Max = 1, name =  
                      gettext("Parameter of a Uniform distribution")
                      ), 
          contains = "Parameter"
          )

## Class: FParameter
setClass("FParameter", 
          representation = representation(df1 = "numeric", df2 = "numeric", 
                                          ncp = "numeric"
                                          ), 
          prototype = prototype(df1 = 1, df2 = 1, ncp = 0, name = 
                      gettext("Parameter of a Fisher-Snedecor/F distribution")
                      ), 
          contains = "Parameter"
          )

## Class: TParameter
setClass("TParameter", 
          representation = representation(df = "numeric", ncp = "numeric"), 
          prototype = prototype(df = 1, ncp = 0, name = 
                      gettext("Parameter of a Student/T distribution")
                      ), 
          contains = "Parameter"
          )

## Class: LNormParameter
setClass("LnormParameter", 
          representation = representation(meanlog = "numeric",
                                          sdlog = "numeric"
                                          ), 
          prototype = prototype(meanlog = 0, meansd = 1, name =  
                      gettext("Parameter of a Log-Normal distribution")
                      ), 
          contains = "Parameter"
          )

## Class: BetaParameter
setClass("BetaParameter", 
          representation = representation(shape1 = "numeric", 
                                          shape2 = "numeric", ncp = "numeric"
                                          ), 
          prototype = prototype(shape1 = 1, shape2 = 1, ncp = 0, name = 
                      gettext("Parameter of a Beta distribution")
                      ), 
          contains = "Parameter"
          )

## Class: LogisParameter
setClass("LogisParameter", 
          representation = representation(location = "numeric", 
                                          scale = "numeric"
                                          ), 
          prototype = prototype(location = 0, scale = 1, name = 
                      gettext("Parameter of a Logistic distribution")
                      ), 
          contains = "Parameter"
          )

## Class: WeibullParameter
setClass("WeibullParameter", 
          representation = representation(shape = "numeric", 
                                          scale = "numeric"
                                          ), 
          prototype = prototype(shape = 1, scale = 1, name = 
                      gettext("Parameter of a Weibull distribution")
                      ), 
          contains = "Parameter"
          )

################################
##
## matrix classes
##
################################
## positive definite, symmetric matrices with finite entries
setClass("PosSemDefSymmMatrix", contains = "matrix",
            prototype = prototype(matrix(1)),
            validity = function(object){
                if(nrow(object) != ncol(object))
                    stop("no square matrix")
                if(any(!is.finite(object)))
                    stop("inifinite or missing values in matrix")
                if(!isTRUE(all.equal(object, t(object), .Machine$double.eps^0.5,
                                     check.attributes = FALSE)))
                    stop("matrix is not symmetric")
                if(!all(eigen(object)$values > -100*.Machine$double.eps))
                   stop("matrix is (numerically) not positive semi - definite")
               return(TRUE)
            })

## positive definite, symmetric matrices with finite entries
setClass("PosDefSymmMatrix", contains = "PosSemDefSymmMatrix",
            validity = function(object){
               if(!all(eigen(object)$values > 100*.Machine$double.eps))
                   stop("matrix is (numerically) not positive definite")
               valid <- getValidity(getClass("PosSemDefSymmMatrix"))
               valid(as(object, "PosSemDefSymmMatrix"))
               return(TRUE)
            })


################################
##
## symmetry classes
##
################################

### from Matthias' thesis / ROptEst / moved from distrMod

## class of symmetries
setClass("Symmetry", representation(type = "character",
                                    SymmCenter = "ANY"),
                     contains = "VIRTUAL")

## symmetry of distributions
setClass("DistributionSymmetry", contains = c("Symmetry", "VIRTUAL"))

## no symmetry
setClass("NoSymmetry", contains = "DistributionSymmetry",
            prototype = prototype(type = "non-symmetric distribution",
                                  SymmCenter = NULL))

## elliptical symmetry
setClass("EllipticalSymmetry", contains = "DistributionSymmetry",
            prototype = prototype(type = "elliptically symmetric distribution",
                                  SymmCenter = numeric(0)))

## spherical symmetry
setClass("SphericalSymmetry", contains = "EllipticalSymmetry",
            prototype = prototype(type = "spherically symmetric distribution",
                                  SymmCenter = numeric(0)))

## list of symmetry types
setClass(Class = "DistrSymmList",
            prototype = prototype(list(new("NoSymmetry"))),
            contains = "list",
            validity = function(object){
                nrvalues <- length(object)
                for(i in 1:nrvalues)
                    if(!is(object[[i]], "DistributionSymmetry"))
                        stop("element ", i, " is no 'DistributionSymmetry'")
                return(TRUE)
            })


################################
##
## distribution classes
##
################################

setClass("Distribution",
          representation = representation(
                      img = "rSpace",
                      param = "OptionalParameter",
                      r = "function",
                      d = "OptionalFunction",
                      p = "OptionalFunction",
                      q = "OptionalFunction", # extended by P.R. 28-03-06
                      .withSim = "logical",   ## 'internal' slots => no
                      .withArith = "logical",  ## accessor/replacement functions
                      .logExact = "logical",
                      .lowerExact = "logical",
                      Symmetry = "DistributionSymmetry"
                      ),
         prototype = prototype(
                     r = function(n){ rnorm(n, mean = 0, sd = 1) },
                     d = function(x, log = FALSE)
                            { dnorm(x, mean = 0, sd = 1, log = log) },
                     p = function(q, lower.tail = TRUE, log.p = FALSE )
                             { pnorm(q, mean = 0, sd = 1,
                                     lower.tail = lower.tail, log.p = log.p) },
                     q = function(p, lower.tail = TRUE, log.p = FALSE )
                             { qnorm(p, mean = 0, sd = 1,
                                     lower.tail = lower.tail, log.p = log.p) },
                     img = new("Reals"),
                     param = NULL,
                     .withArith = FALSE,
                     .withSim = FALSE,
                     .logExact = FALSE,
                     .lowerExact = FALSE,
                     Symmetry = new("NoSymmetry")
                     )
         )

## Class: UnivariateDistribution
setClass("UnivariateDistribution",  
          contains = "Distribution"
          )

### ---- absolutely continuous distributions ---- ###

## AbscontDistribution
setClass("AbscontDistribution", 
          representation = representation(gaps = "OptionalMatrix"),
          prototype = prototype(gaps = NULL),
          contains = "UnivariateDistribution"
          )



## inbetween-Class: ExpOrGammaOrChisq

#not quite virtual ...
setClass("ExpOrGammaOrChisq", 
          contains = c("AbscontDistribution", "VIRTUAL")
          )


## Class: exponential distribution
setClass("Exp",
          prototype = prototype(
                      r = function(n){ rexp(n, rate = 1) },
                      d = function(x, log = FALSE)
                                  { dexp(x, rate = 1, log = log) },
                      p = function(q, lower.tail = TRUE, log.p = FALSE )
                                  { pexp(q, rate = 1, lower.tail = lower.tail,
                                         log.p = log.p) },
                      q = function(q, lower.tail = TRUE, log.p = FALSE )
                                  { qexp(p, rate = 1, lower.tail = lower.tail,
                                         log.p = log.p) },
                      param = new("ExpParameter"),
                     .logExact = TRUE,
                     .lowerExact = TRUE
                      ),
          contains = "ExpOrGammaOrChisq"
          )

## Class: gamma distribution
setClass("Gammad",
          prototype = prototype(
                      r = function(n){ rgamma(n, shape = 1, scale = 1) },
                      d = function(x, log = FALSE){
                              dgamma(x, shape = 1, scale = 1, log = log)
                                          },
                      p = function(q, lower.tail = TRUE, log.p = FALSE ){
                              pgamma(q, shape = 1, scale = 1,
                                     lower.tail = lower.tail, log.p = log.p)
                                          },
                      q = function(p, lower.tail = TRUE, log.p = FALSE ){
                              qgamma(p, shape = 1, scale = 1,
                                     lower.tail = lower.tail, log.p = log.p)
                                          },
                      param = new("GammaParameter"),
                     .logExact = TRUE,
                     .lowerExact = TRUE
                      ),
          contains = "ExpOrGammaOrChisq"
          )

## Class: Chi squared distribution
setClass("Chisq",
          prototype = prototype(
                      r = function(n){ rchisq(n, df = 1, ncp = 0) },
                      d = function(x, log = FALSE)
                                  { dchisq(x, df = 1, ncp = 0, log = log) },
                      p = function(q, lower.tail = TRUE, log.p = FALSE )
                                  { pchisq(q, df = 1, ncp = 0,
                                           lower.tail = lower.tail,
                                           log.p = log.p) },
                      q = function(p, lower.tail = TRUE, log.p = FALSE )
                                  { qchisq(p, df = 1, ncp = 0,
                                           lower.tail = lower.tail,
                                           log.p = log.p) },
                      param = new("ChisqParameter"),
                     .logExact = TRUE,
                     .lowerExact = TRUE
                      ),
          contains = "ExpOrGammaOrChisq"
          )

## Class: Laplace or Double Exponential distribution
setClass("DExp",
          prototype = prototype(
                      r = function(n){
                              (2*rbinom(n ,size = 1, prob = 0.5)-1) *
                                 rexp(n, rate = 1)
                              },
                      d = function(x, log = FALSE)
                              { d0 <-  dexp(abs(x), rate = 1, log = log)
                                d0 <- if (log) d0-log(2) else d0 <- d0 / 2
                                return(d0) },
                      p = function(q, lower.tail = TRUE, log.p = FALSE ){
                                   if (!lower.tail) q <- -q
                                   p0 <- ifelse( q <= 0,
                                                 0.5*pexp(-q, rate = 1,
                                                          lower.tail = FALSE),
                                                 0.5 + 0.5*pexp( q, rate = 1)
                                                    )
                                   if (log.p)  p0 <- log(p0)
                                   return(p0)
                                   },
                      q = function(p, lower.tail = TRUE, log.p = FALSE ){
                              if (log.p) p <- exp(p)
                              if (!lower.tail) p <- 1-p
                              ifelse( p <= 0.25,
                                  -qexp(2*p, rate = 1, lower.tail = FALSE),
                                  ifelse( p <= 0.5,
                                      -qexp(1-2*p, rate = 1),
                                      ifelse( p <= 0.75   ,
                                          qexp(2*p - 1, rate = 1),
                                          qexp(2*(1-p), rate = 1,
                                               lower.tail = FALSE)
                                            )
                                         )
                                     )},
                      param = new("ExpParameter", name =
                      gettext("Parameter of a Laplace/Double Exponential distribution")
                                 ),
                     .logExact = TRUE,
                     .lowerExact = TRUE,
                     Symmetry = new("SphericalSymmetry", 
                                     type = "univariate symmetric distribution",
                                     SymmCenter = 0)
                      ),
          contains = "AbscontDistribution"
          )

## Class: CauchyDistribution
setClass("Cauchy",
          prototype = prototype(
                      r = function(n){ rcauchy(n,  location = 0, scale = 1) },
                      d = function(x, log = FALSE){
                              dcauchy(x,  location = 0, scale = 1, log = log)
                                          },
                      p = function(q, lower.tail = TRUE, log.p = FALSE ){
                              pcauchy(q,  location = 0, scale = 1,
                                      lower.tail = lower.tail, log.p = log.p)
                                          },
                      q = function(p, lower.tail = TRUE, log.p = FALSE ){
                              qcauchy(p,  location = 0, scale = 1,
                                      lower.tail = lower.tail, log.p = log.p)
                                          },
                      param = new("CauchyParameter"),
                     .logExact = TRUE,
                     .lowerExact = TRUE,
                     Symmetry = new("SphericalSymmetry", 
                                     type = "univariate symmetric distribution",
                                     SymmCenter = 0)
                      ),
          contains = "AbscontDistribution"
          )

## Class: normal distribution
setClass("Norm",
          prototype = prototype(
                      r = function(n){ rnorm(n, mean = 0, sd = 1) },
                      d = function(x, log = FALSE)
                              { dnorm(x, mean = 0, sd = 1, log = log) },
                      p = function(q, lower.tail = TRUE, log.p = FALSE )
                              { pnorm(q, mean = 0, sd = 1,
                                      lower.tail = lower.tail, log.p = log.p) },
                      q = function(p, lower.tail = TRUE, log.p = FALSE )
                              { qnorm(p, mean = 0, sd = 1,
                                      lower.tail = lower.tail, log.p = log.p) },
                      param = new("UniNormParameter"),
                     .logExact = TRUE,
                     .lowerExact = TRUE,
                     Symmetry = new("SphericalSymmetry", 
                                     type = "univariate symmetric distribution",
                                     SymmCenter = 0)
                      ),
          contains = "AbscontDistribution"
          )

## Class: lognormal distribution
setClass("Lnorm",
          prototype = prototype(
                      r = function(n){ rlnorm(n, meanlog = 0, sdlog = 1) },
                      d = function(x, log = FALSE){
                              dlnorm(x, meanlog = 0, sdlog = 1, log = log)
                                          },
                      p = function(q, lower.tail = TRUE, log.p = FALSE ){
                              plnorm(q, meanlog = 0, sdlog = 1,
                                     lower.tail = lower.tail, log.p = log.p)
                                          },
                      q = function(p, lower.tail = TRUE, log.p = FALSE ){
                              qlnorm(p, meanlog = 0, sdlog = 1,
                                     lower.tail = lower.tail, log.p = log.p)
                                          },
                      param = new("LnormParameter"),
                     .logExact = TRUE,
                     .lowerExact = TRUE
                      ),
          contains = "AbscontDistribution"
          )

## Class: uniform distribution
setClass("Unif",
          prototype = prototype(
                      r = function(n){ runif(n, min = 0, max = 1) },
                      d = function(x, log = FALSE)
                              { dunif(x,  min = 0, max = 1, log = log) },
                      p = function(q, lower.tail = TRUE, log.p = FALSE )
                              { punif(q,  min = 0, max = 1,
                                      lower.tail = lower.tail, log.p = log.p) },
                      q = function(p, lower.tail = TRUE, log.p = FALSE )
                              { qunif(p,  min = 0, max = 1,
                                      lower.tail = lower.tail, log.p = log.p) },
                      param = new("UnifParameter"),
                     .logExact = TRUE,
                     .lowerExact = TRUE,
                     Symmetry = new("SphericalSymmetry", 
                                     type = "univariate symmetric distribution",
                                     SymmCenter = .5)
                      ),
          contains = "AbscontDistribution"
          )

## Class: F distribution
setClass("Fd",
          prototype = prototype(
                      r = function(n){ rf(n,  df1 = 1, df2 = 1, ncp = 0) },
                      d = function(x, log = FALSE){
                              df(x, df1 = 1, df2 = 1, ncp = 0, log = log)
                                          },
                      p = function(q, lower.tail = TRUE, log.p = FALSE ){
                              pf(q, df1 = 1, df2 = 1, ncp = 0,
                                 lower.tail = lower.tail, log.p = log.p)
                                          },
                      q = function(p, lower.tail = TRUE, log.p = FALSE ){
                              qf(p, df1 = 1, df2 = 1, ncp = 0,
                                 lower.tail = lower.tail, log.p = log.p)
                                          },
                      param = new("FParameter"),
                     .logExact = TRUE,
                     .lowerExact = TRUE
                      ),
          contains = "AbscontDistribution"
          )

## Class: Student distribution
setClass("Td",
          prototype = prototype(
                      r = function(n){ rt(n,  df = 1, ncp = 0) },
                      d = function(x, log = FALSE)
                              { dt(x,  df = 1, ncp = 0, log = log) },
                      p = function(q, lower.tail = TRUE, log.p = FALSE )
                              { pt(q,  df = 1, ncp = 0,
                                   lower.tail = lower.tail, log.p = log.p) },
                      q = function(p, lower.tail = TRUE, log.p = FALSE )
                              { qt(p,  df = 1, ncp = 0,
                                   lower.tail = lower.tail, log.p = log.p) },
                      param = new("TParameter"),
                     .logExact = TRUE,
                     .lowerExact = TRUE,
                     Symmetry = new("SphericalSymmetry", 
                                     type = "univariate symmetric distribution",
                                     SymmCenter = 0)
                      ),
          contains = "AbscontDistribution"
          )


## Class: logistic distribution
setClass("Logis",
          prototype = prototype(
                      r = function(n){ rlogis(n, location = 0, scale = 1) },
                      d = function(x, log = FALSE){
                              dlogis(x, location = 0, scale = 1, log = log)
                                          },
                      p = function(q, lower.tail = TRUE, log.p = FALSE ){
                              plogis(q, location = 0, scale = 1,
                                     lower.tail = lower.tail, log.p = log.p)
                                          },
                      q = function(p, lower.tail = TRUE, log.p = FALSE ){
                              qlogis(p, location = 0, scale = 1,
                                     lower.tail = lower.tail, log.p = log.p)
                                          },
                      param = new("LogisParameter"),
                     .logExact = TRUE,
                     .lowerExact = TRUE
                      ),
          contains = "AbscontDistribution"
          )

## Class: BetaDistribution
setClass("Beta",
          prototype = prototype(
                      r = function(n){
                              rbeta(n,  shape1 = 1, shape2 = 1, ncp = 0)
                                     },
                      d = function(x, log = FALSE){
                              dbeta(x,  shape1 = 1, shape2 = 1, ncp = 0,
                                    log = log)
                                          },
                      p = function(q, lower.tail = TRUE, log.p = FALSE ){
                              pbeta(q,  shape1 = 1, shape2 = 1, ncp = 0,
                                    lower.tail = lower.tail, log.p = log.p)
                                          },
                      q = function(p, lower.tail = TRUE, log.p = FALSE ){
                              qbeta(p,  shape1 = 1, shape2 = 1, ncp = 0,
                                    lower.tail = lower.tail, log.p = log.p)
                                          },
                      param = new("BetaParameter"),
                     .logExact = TRUE,
                     .lowerExact = TRUE
                      ),
          contains = "AbscontDistribution"
          )

## Class: Weibull distribution
setClass("Weibull",
          prototype = prototype(
                      r = function(n){ rweibull(n, shape = 1, scale = 1) },
                      d = function(x, log = FALSE){
                              dweibull(x, shape = 1, scale = 1, log = log)
                                          },
                      p = function(q, lower.tail = TRUE, log.p = FALSE ){
                              pweibull(q, shape = 1, scale = 1,
                                       lower.tail = lower.tail, log.p = log.p)
                                          },
                      q = function(p, lower.tail = TRUE, log.p = FALSE ){
                              qweibull(p, shape = 1, scale = 1,
                                       lower.tail = lower.tail, log.p = log.p)
                                          },
                      param = new("WeibullParameter"),
                     .logExact = TRUE,
                     .lowerExact = TRUE
                      ),
          contains = "AbscontDistribution"
          )

## Class: Arcsine distribution
setClass("Arcsine",
          prototype = prototype(
                      r = function(n){ sin((runif(n)-.5)*pi) },
                      d = function(x, log = FALSE){
                              x0 <- (abs(x)<1-.Machine$double.eps)
                              x1 <- x^2*x0
                              d <-  x0/sqrt(1-x1)/pi
                              d[.isEqual(abs(x),1)] <- Inf
                              if(log) d<- log(d)
                              return(d)},
                      p = function(q, lower.tail = TRUE, log.p = FALSE ){
                              if(!lower.tail) q<- -q
                              q <- pmin(pmax(q,-1),1)
                              p <- asin(q)/pi+1/2
                              if(log.p) p <- log(p)
                              return(p)},
                      q = function(p, lower.tail = TRUE, log.p = FALSE ){
                              if(log.p) p <- exp(p)
                              p1 <- p
                              p1[p<0|p>1] <- 0.5
                              if(!lower.tail) p1 <- 1-p1
                              q <- sin( (p1-1/2)*pi)
                              q[p<0|p>1] <- NA
                              q[.isEqual(p,0)] <- -1
                              q[.isEqual(p,1)] <-  1
                              return(q)},
                     .logExact = TRUE,
                     .lowerExact = TRUE,
                     Symmetry = new("SphericalSymmetry", 
                                     type = "univariate symmetric distribution",
                                     SymmCenter = 0)
                      ),
          contains = "AbscontDistribution"
          )


## inbetween-Class: AffLinAbscontDistribution

setClass("AffLinAbscontDistribution", 
          representation = representation(a = "numeric", b = "numeric",
          X0 = "AbscontDistribution"),
          prototype = prototype(a = 1, b = 0, X0 = new("Norm")),
          contains = "AbscontDistribution"
          )

### ---- discrete distributions ---- ###

## DiscreteDistribution
setClass("DiscreteDistribution", 
          representation = representation(support = "numeric", .finSupport = "logical"),
          prototype = prototype(
                      r = function(n){ rbinom(n, size=1, prob=0.5) },
                      d = function(x, log = FALSE)
                              { dbinom(x, size=1, prob=0.5, log = log) },
                      p = function(q, lower.tail = TRUE, log.p = FALSE )
                              { pbinom(q, size=1, prob=0.5, 
                                       lower.tail = lower.tail, log.p = log.p) },
                      q = function(p, lower.tail = TRUE, log.p = FALSE )
                              { qbinom(p, size=1, prob=0.5, 
                                       lower.tail = lower.tail, log.p = log.p) },
                      img = new("Reals"),
                      support = 0:1,
                      .finSupport = c(TRUE,TRUE)
                      ), 
          contains = "UnivariateDistribution"
          )

## LatticeDistribution
setClass("LatticeDistribution", 
          representation = representation(lattice = "Lattice"),
          prototype = prototype(lattice = new("Lattice")),
          contains = "DiscreteDistribution"
          )


### Class: Dirac distribution
setClass("Dirac",
          prototype = prototype(
                      r = function(n){ array(0, n)},
                      d = function(x, log)
                              { y <- rep(0,length(x))
                                d0 <- as.numeric(x == y)
                                if(log) d0 <- log(d0)
                                return(d0)
                              },
                      p = function(q, lower.tail = TRUE, log.p = FALSE )
                              { p0 <- as.numeric(q + 10^-10 >= 0)
                                if (!lower.tail) p0 <- 1-p0
                                if (log.p) p0 <- log(p0)
                                return(p0)
                              },
                      q = function(p, lower.tail = TRUE, log.p = FALSE )
                             {  if (log.p) p <- exp(p)
                                if(any((p < 0)|(p > 1)))
                                   warning("q Method of class Dirac produced NaN's.")
                                q0 <- 0 * p
                                q0[(p<0) | (p>1)] <- NaN
                                return(q0)
                              },
                      param = new("DiracParameter"),
                      support = 0,
                      lattice = new("Lattice",
                                pivot = 0, width = 1, Length = 1, name =
                                gettext("lattice of a Dirac distribution")
                                ),
                     .logExact = TRUE,
                     .lowerExact = TRUE,
                     .finSupport = c(TRUE,TRUE)
                      ),
          contains = "LatticeDistribution"
          )

## Class: Poisson distribution
setClass("Pois",
          prototype = prototype(
                      r = function(n){ rpois(n, lambda = 1) },
                      d = function(x, log = FALSE)
                              { dpois(x, lambda = 1, log = log) },
                      p = function(q, lower.tail = TRUE, log.p = FALSE )
                              { ppois(q, lambda = 1, lower.tail = lower.tail,
                                      log.p = log.p) },
                      q = function(p, lower.tail = TRUE, log.p = FALSE )
                              { qpois(p, lambda = 1, lower.tail = lower.tail,
                                      log.p = log.p) },
                      img = new("Naturals"),
                      param = new("PoisParameter"),
                      support = seq( 0,
                                     qpois(getdistrOption("TruncQuantile"),
                                           lambda = 1, lower.tail = FALSE),
                                     by = 1
                                    ),
                      lattice = new("Lattice",
                                pivot = 0, width = 1, Length = Inf, name =
                                gettext("lattice of a Poisson distribution")
                                ),
                     .logExact = TRUE,
                     .lowerExact = TRUE,
                     .finSupport = c(TRUE,FALSE)
                      ),
          contains = "LatticeDistribution"
          )

## Class: negative binomial distribution
setClass("Nbinom",
          prototype = prototype(
                      r = function(n){ rnbinom(n, size = 1, prob = 0.5) },
                      d = function(x, log = FALSE){
                              dnbinom(x, size = 1, prob = 0.5, log = log)
                                          },
                      p = function(q, lower.tail = TRUE, log.p = FALSE ){
                              pnbinom(q, size = 1, prob = 0.5,
                                      lower.tail = lower.tail, log.p = log.p)
                                          },
                      q = function(p, lower.tail = TRUE, log.p = FALSE ){
                              qnbinom(p, size = 1, prob = 0.5,
                                      lower.tail = lower.tail, log.p = log.p)
                                          },
                      img = new("Naturals"),
                      param = new("NbinomParameter"),
                      support = seq( 0,
                                     qnbinom(
                                        getdistrOption("TruncQuantile"),
                                        size = 1, prob = 0.5, lower.tail = FALSE
                                            ),
                                     by = 1
                                     ),
                      lattice = new("Lattice",
                                pivot = 0, width = 1, Length = Inf, name =
                                gettext(
                                  "lattice of a Negative Binomial distribution"
                                       )
                                ),
                     .logExact = TRUE,
                     .lowerExact = TRUE,
                     .finSupport = c(TRUE,FALSE)
                      ),
          contains = "LatticeDistribution"
          )

## Class: hypergeometric distribution
setClass("Hyper",
          prototype = prototype(
                      r = function(nn){ rhyper(nn, m = 1, n = 1, k = 1) },
                      d = function(x, log = FALSE){
                              dhyper(x, m = 1, n = 1, k = 1, log = log)
                                          },
                      p = function(q, lower.tail = TRUE, log.p = FALSE ){
                              phyper(q, m = 1, n = 1, k = 1,
                                     lower.tail = lower.tail, log.p = log.p)
                                          },
                      q = function(p, lower.tail = TRUE, log.p = FALSE ){
                              qhyper(p, m = 1, n = 1, k = 1,
                                     lower.tail = lower.tail, log.p = log.p)
                                          },
                      img = new("Naturals"),
                      param = new("HyperParameter"),
                      support = 0:1,
                      lattice = new("Lattice",
                                pivot = 0, width = 1, Length = 2, name =
                                gettext(
                                  "lattice of a Hypergeometric distribution"
                                       )
                                ),
                     .logExact = TRUE,
                     .lowerExact = TRUE,
                     .finSupport = c(TRUE,TRUE)
                      ),
          contains = "LatticeDistribution"
          )

## Class: binomial distribution
setClass("Binom",
          prototype = prototype(
                      r = function(n){ rbinom(n, size = 1,prob = 0.5) },
                      d = function(x, log = FALSE){
                              dbinom(x, size = 1, prob = 0.5, log = log)
                                          },
                      p = function(q, lower.tail = TRUE, log.p = FALSE ){
                              pbinom(q, size = 1, prob = 0.5,
                                     lower.tail = lower.tail, log.p = log.p)
                                          },
                      q = function(p, lower.tail = TRUE, log.p = FALSE ){
                              qbinom(p, size = 1, prob = 0.5,
                                     lower.tail = lower.tail, log.p = log.p)
                                          },
                      img = new("Naturals"),
                      param = new("BinomParameter"),
                      support = 0:1,
                      lattice = new("Lattice",
                                pivot = 0, width = 1, Length = 2, name =
                                gettext(
                                  "lattice of a Binomial distribution"
                                       )
                                ),
                     .logExact = TRUE,
                     .lowerExact = TRUE,
                     .finSupport = c(TRUE,TRUE)
                      ),
          contains = "LatticeDistribution"
          )

## Class: geometric distribution
setClass("Geom",
          prototype = prototype(
                      r = function(n){ rgeom(n, prob = 0.5) },
                      d = function(x, log = FALSE)
                              { dgeom(x, prob = 0.5, log = log) },
                      p = function(q, lower.tail = TRUE, log.p = FALSE )
                              { pgeom(q, prob = 0.5, lower.tail = lower.tail,
                                      log.p = log.p) },
                      q = function(p, lower.tail = TRUE, log.p = FALSE )
                              { qgeom(p, prob = 0.5, lower.tail = lower.tail,
                                      log.p = log.p) },
                      param = new("NbinomParameter", name =
                              gettext("Parameter of a Geometric distribution")
                              ),
                      support = seq( 0,
                                     qgeom(getdistrOption("TruncQuantile"),
                                           prob = 0.5, lower.tail = FALSE),
                                     by = 1
                                    ),
                      lattice = new("Lattice",
                                pivot = 0, width = 1, Length = Inf, name =
                                gettext(
                                  "lattice of a Geometric distribution"
                                       )
                                ),
                     .logExact = TRUE,
                     .lowerExact = TRUE,
                     .finSupport = c(TRUE, FALSE)
                      ),
          contains = "Nbinom"
          )

### ---- List of distributions by M. Kohl ---- ###

setClass(Class = "DistrList", 
            prototype = prototype(list(new("Norm"))), 
            contains = "list",
            validity = function(object){
                nrvalues <- length(object)
                for(i in 1:nrvalues)
                    if(!is(object[[i]], "Distribution")) 
                        stop("element ", i, " is no 'Distribution'")
                return(TRUE) 
            })

## inbetween-Classes: AffLinDiscreteDistribution, AffLinLatticeDistribution

setClass("AffLinDiscreteDistribution", 
          representation = representation(a = "numeric", b = "numeric",
          X0 = "DiscreteDistribution"),
          prototype = prototype(a = 1, b = 0, X0 = new("Binom")),
          contains = "DiscreteDistribution"
          )

setClass("AffLinLatticeDistribution", 
          contains = c("LatticeDistribution", "AffLinDiscreteDistribution")
          )




################################
##
## Distribution List classes 
##
################################

setClass("UnivarDistrList", 
            prototype = prototype(list(new("Norm"))), 
            contains = "DistrList", 
            validity = function(object){
                nrvalues <- length(object)
                for(i in 1:nrvalues)
                    if(!is(object[[i]], "UnivariateDistribution"))
                        stop("element ", i, " is no 'UnivariateDistribution'")
                return(TRUE) 
            })


#### new from version 2.0: Mixing Distributions

################################
##
## Mixing Distribution classes 
##
################################

setClass("UnivarMixingDistribution",
            representation = representation(mixCoeff = "numeric",
                             mixDistr = "UnivarDistrList",
                             gaps = "OptionalMatrix",
                             support = "numeric",
                             Symmetry = "DistributionSymmetry",
                             .withArith = "logical",
                             .withSim = "logical",
                             .logExact = "logical",
                             .lowerExact = "logical"
                             ),
            prototype = prototype(mixCoeff = 1, 
                                  mixDistr = new("UnivarDistrList"),
                                  gaps = NULL,
                                  support = numeric(0),
                                  Symmetry = new("NoSymmetry"),
                                 .withArith = FALSE,
                                 .withSim = FALSE,
                                 .logExact = TRUE,
                                 .lowerExact = TRUE
                                  ),
            contains = "UnivariateDistribution",
            validity = function(object){
                if(any(object@mixCoeff< -.Machine$double.eps) || 
                   sum(object@mixCoeff)>1+.Machine$double.eps)
                   stop("mixing coefficients are no probabilities")
                return(TRUE)
            })

setClass("UnivarLebDecDistribution",
            representation = representation(mixCoeff = "numeric",
                             mixDistr = "UnivarDistrList"),
            prototype = prototype(mixCoeff = c("acWeight" = 1, 
                                               "discreteWeight" = 0),
                                  mixDistr = new("UnivarDistrList",
                                              list("acPart" = new("Norm"),
                                                   "discretePart" = new("Dirac")
                                                   )
                                  )),
            contains = "UnivarMixingDistribution",
            validity = function(object){
                if (length(object@mixCoeff)!=2)
                    stop("number of mixing components is not 2")
                if (!is(object@mixDistr[[1]], "AbscontDistribution"))
                    stop("first component must be absolutely continuous")
                if (!is(object@mixDistr[[2]], "DiscreteDistribution"))
                    stop("second component must be discrete")
                return(TRUE)
            })

setClass("AffLinUnivarLebDecDistribution",
          representation = representation(a = "numeric", b = "numeric",
          X0 = "UnivarLebDecDistribution"),
          prototype = prototype(a = 1, b = 0, 
                                X0 = new("UnivarLebDecDistribution")),
          contains = "UnivarLebDecDistribution"
          )

         
setClassUnion("UnivDistrListOrDistribution",
               c("UnivarDistrList","UnivariateDistribution"))

setClass("CompoundDistribution", representation=representation(
             NumbOfSummandsDistr = "DiscreteDistribution",
             SummandsDistr = "UnivDistrListOrDistribution"),
          prototype=prototype(NumbOfSummandsDistr = new("Pois"),
              SummandsDistr=new("Norm")),
          contains = "UnivarMixingDistribution"
         )

################################
##
## virtual Distribution class Unions 
##
################################

setClassUnion("AcDcLcDistribution", c("AbscontDistribution",
               "DiscreteDistribution", "UnivarLebDecDistribution",
               "CompoundDistribution"))

setClassUnion("AffLinDistribution", c("AffLinAbscontDistribution",
               "AffLinDiscreteDistribution", "AffLinUnivarLebDecDistribution"))

## access method
setMethod("type", "Symmetry", function(object) object@type)
setMethod("SymmCenter", "Symmetry", function(object) object@SymmCenter)

## generating function
NoSymmetry <- function(){ new("NoSymmetry") }

## generating function
EllipticalSymmetry <- function(SymmCenter = 0){ 
    new("EllipticalSymmetry", SymmCenter = SymmCenter) 
}

## generating function
SphericalSymmetry <- function(SymmCenter = 0){ 
    new("SphericalSymmetry", SymmCenter = SymmCenter) 
}

## extra methods
## binary operators


if(!isGeneric("simplifyr")) 
    setGeneric("simplifyr", 
                function(object, 
                         size = 10^getdistrOption("RtoDPQ.e")) 
                         standardGeneric("simplifyr")
              )

setMethod("simplifyr", "UnivariateDistribution", 
          function(object, size = 10^getdistrOption("RtoDPQ.e")){
            Sample <- r(object)(size)       
            rneu <- function(n) sample(x = Sample, size = n, replace = TRUE)
            eval.parent(substitute(object@r<-rneu))           
          })


## function to automatically generate, starting from simulations, density, 
## quantile function and cdf
## first version for absolutely continuous, second for discrete distributions

## we use 10^RtoDPQExponent random numbers to generate new distr
## density should use DefaultNrGridPoints equally spaced points for evaluation

RtoDPQ <- function(r, e = getdistrOption("RtoDPQ.e"),
                      n = getdistrOption("DefaultNrGridPoints"), y = NULL){
  zz <- if(!is.null(y)) y else r(10^e)
  zz <- zz[!is.na(zz)]
  
  dxy <-  xy.coords(density(zz, n = n))
  dfun <- .makeDNew(dxy$x, dxy$y, standM = "int")

  pf0 <- function(x, y, yleft, yright) ecdf(x)
  pfun <- .makePNew(x=zz, dx=0, notwithLLarg=TRUE, myPf = pf0)
            ## quantile function

  yL <-  min(zz);   yR <-  max(zz); rm(zz)
  px.l <- pfun(dxy$x);   px.u <- pfun(dxy$x, lower.tail = FALSE)
  qfun <- .makeQNew(dxy$x, px.l, px.u, TRUE, yL, yR)

  rm(px.l, px.u, dxy, pf0)
  list(dfun = dfun, pfun = pfun, qfun = qfun)}


RtoDPQ.d <- function(r, e = getdistrOption("RtoDPQ.e")){
  zz <- r(10^e)
  X <- table(zz)
  rm(zz)

  supp <- as.numeric(names(X))
  prob <- X/(10^e)
  rm(X)

  len = length(supp)

  if(len > 1){
    if(min(diff(supp)) <
           getdistrOption("DistrResolution"))
       stop("grid too narrow --> change DistrResolution")
  }

  dfun <- .makeDNew(supp, prob, Cont = FALSE)
  pfun <- .makePNew(supp, prob, TRUE, Cont = FALSE)
  qfun <- .makeQNew(supp, cumsum(prob), rev(cumsum(rev(prob))),
                      TRUE, min(supp), max(supp), Cont = FALSE)

  list(dfun = dfun, pfun = pfun, qfun = qfun)
}

### new from 2.0:

RtoDPQ.LC <- function(r, e = getdistrOption("RtoDPQ.e"),
                      n = getdistrOption("DefaultNrGridPoints"), y = NULL){

  zz <- if(!is.null(y)) y else r(10^e)
  hasDis <- FALSE
  zz.nr <- zz

  zz.T <- table(zz)
  zz.T1 <- zz.T[zz.T>1]
  zz.replic <- as.numeric(names(zz.T1))
  w.d <- sum(zz %in% zz.replic)/10^e
  rm(zz.T)

  f.d <- Dirac(0)
  if(w.d){
   hasDis <- TRUE
   zz.nr <- zz[! zz %in% zz.replic]
   d.r <- zz.T1/sum(zz.T1)
   f.d <- DiscreteDistribution(supp = zz.replic, prob = d.r,
                     .withSim = TRUE, .withArith = TRUE,
                     .lowerExact = FALSE, .logExact = FALSE)
   f.d@.finSupport <- c(TRUE,TRUE)
   rm(d.r,zz.replic,zz.T1)
  }
  rm(zz)
  
  if(1-w.d){
  dxy <-  xy.coords(density(zz.nr, n = n))
  dcfun <- .makeDNew(dxy$x, dxy$y, standM = "int")

  pf0 <- function(x, y, yleft, yright) ecdf(x)
  pcfun <- .makePNew(x=zz.nr, dx=0, notwithLLarg=TRUE, myPf = pf0)
            ## quantile function

  yL <-  min(zz.nr);   yR <-  max(zz.nr); rm(zz.nr)
  px.l <- pcfun(dxy$x);   px.u <- pcfun(dxy$x, lower.tail = FALSE)
  qcfun <- .makeQNew(dxy$x, px.l, px.u, TRUE, yL, yR)

  rm(px.l, px.u, dxy, pf0)
  f.c <- AbscontDistribution( r= function(n) qcfun(runif(n)),
             d=dcfun, p = pcfun, q = qcfun, .withSim = TRUE,
             .withArith = TRUE, .lowerExact = FALSE, .logExact = FALSE)
  }
  else f.c <-Norm()
  UnivarLebDecDistribution(discretePart = f.d, acPart = f.c,
                           discreteWeight = w.d)
  }

####################################################################################


###########################################################

setMethod("Minimum",
          signature(e1 = "AbscontDistribution",
                    e2 = "AbscontDistribution"),
          function(e1, e2, ...){

            ##new gaps
            newgaps <- .mergegaps2(gaps(e1),gaps(e2))

            ## new random number function
            rnew <- function(n){
              rn1 <- r(e1)(n)
              rn2 <- r(e2)(n)
              ifelse(rn1 < rn2, rn1, rn2)
            }

            ## new cdf
            pnew <- function(q, lower.tail = TRUE, log.p = FALSE){
              p1 <- p(e1)(q, lower.tail = FALSE)
              p2 <- p(e2)(q, lower.tail = FALSE)
              p0 <- if(lower.tail) 1 - p1 * p2  else p1 * p2
              if (log.p) p0 <- log(p0)
              return(p0)
            }

            ## new density
            dnew <- function(x, log = FALSE){
              d1 <- d(e1)(x)
              d2 <- d(e2)(x)
              p1 <- p(e1)(x, lower.tail = FALSE)
              p2 <- p(e2)(x, lower.tail = FALSE)
              d0 <- d1 * p2 + d2 * p1
              if(log) d0 <- log(d0)
              return(d0)
            }

            ## new quantile function
            qL1 <- min(getLow(e1), getLow(e2))
            qU1 <- max(getUp(e1), getUp(e2))
            n <- getdistrOption("DefaultNrGridPoints")
            h <- (qU1-qL1)/n
            xseq <- seq(from = qL1, to = qU1, by = h)
            px.l <- pnew(xseq, lower.tail = TRUE)
            px.u <- pnew(xseq, lower.tail = FALSE)
            qL2 <- min(q.l(e1)(0),q.l(e2)(0))
            qU2 <- max(q.l(e1)(1),q.l(e2)(1))

            qnew <- .makeQNew(xseq, px.l, px.u, FALSE, qL2, qU2)

            return(AbscontDistribution( r = rnew, gaps = newgaps,
                   d = dnew, p = pnew, q = qnew, .withArith = TRUE,
                   .withSim = e1@.withSim|e2@.withSim))
          })

setMethod("Minimum",
          signature(e1 = "DiscreteDistribution",
                    e2 = "DiscreteDistribution"),
          function(e1, e2, ...){

           supp1 <- support(e1)
           supp2 <- support(e2)
           suppMax <- min(max(supp1), max(supp2))
           supp <- union(supp1[supp1 <= suppMax], supp2[supp2 <= suppMax])
           len <- length(supp)
           if(length(usupp <- unique(supp)) < len){
              warning("collapsing to unique support values")
              supp <- sort(usupp)
              len <- length(supp)
           }else{
              o <- order(supp)
              supp <- supp[o]
           }
          d1 <- d(e1)(supp)
          d2 <- d(e2)(supp)
          p1 <- p(e1)(supp,lower.tail = FALSE)
          p2 <- p(e2)(supp,lower.tail = FALSE)
          d0 <- d1*p2 + d2*p1 + d1*d2
          res <- DiscreteDistribution(supp=supp, prob=d0, .withArith= TRUE)
          res@.finSupport <- c(e1@.finSupport[1]&e2@.finSupport[1],
                               e1@.finSupport[2]|e2@.finSupport[2])
          res
          })

setMethod("Minimum",
          signature(e1 = "AbscontDistribution",
                    e2 = "Dirac"),
          function(e1, e2, withSimplify = getdistrOption("simplifyD")){
            M <- location(e2)
            aw <- p(e1)(M)
            if (aw > getdistrOption("TruncQuantile"))
                  aD <- Truncate(e1, lower = -Inf, upper = M)
            else aD <- Norm()
            erg <- UnivarLebDecDistribution(acPart = aD, discretePart = e2,
                   acWeight = aw)
            if(withSimplify) simplifyD(erg)
            return(erg)})


setMethod("Minimum",
          signature(e1 = "AcDcLcDistribution",
                    e2 = "AcDcLcDistribution"),
          function(e1, e2, withSimplify = getdistrOption("simplifyD")){

        e1 <- as(e1, "UnivarLebDecDistribution")
        e2 <- as(e2, "UnivarLebDecDistribution")

        aw1 <- acWeight(e1)
        dw1 <- 1-aw1
        aw2 <- acWeight(e2)
        dw2 <- 1-aw2
        aD1 <- acPart(e1)
        aD2 <- acPart(e2)
        dD1 <- discretePart(e1)
        dD2 <- discretePart(e2)

        ep <- getdistrOption("TruncQuantile")

        aaw <- aw1*aw2
        aaD <- if(aaw>ep) Minimum(aD1,aD2)  else Norm()
        aaD <- as(aaD,"UnivarLebDecDistribution")

        ddw <- dw1*dw2
        ddD <- if(ddw>ep) Minimum(dD1,dD2)  else Dirac(0)

        ddD <- as(ddD,"UnivarLebDecDistribution")

        adw <- aw1*dw2
        if(adw > ep){
           Dlist <- lapply(support(dD2), function(x) Minimum(aD1,Dirac(x)))
           adD <- as(simplifyD( do.call(flat.LCD,
                        c(Dlist, alist(mixCoeff = d(dD2)(support(dD2)))))),
                        "UnivarLebDecDistribution")
        }else adD <- as(Dirac(0),"UnivarLebDecDistribution")

        if(identical(e1,e2)) {daw <-adw; daD <- adD}
        else{
            daw <- aw2*dw1
            if(daw > ep){
               Dlist <- lapply(support(dD1), function(x) Minimum(aD2,Dirac(x),
                               withSimplify = FALSE))
               daD <- as(simplifyD( do.call(flat.LCD,
                            c(Dlist, alist(mixCoeff = d(dD1)(support(dD1)))))),
                            "UnivarLebDecDistribution")
            }else daD <- as(Dirac(0),"UnivarLebDecDistribution")

        }
        mixCoeff <- c(aaw,ddw,adw,daw)
        mixCoeff <- mixCoeff/sum(mixCoeff)
#        print(c(aaw,ddw,adw,daw,sum(c(aaw,ddw,adw,daw))))
#        print(list(aaD,ddD,adD,daD))
        erg <- flat.LCD(aaD,ddD,adD,daD,mixCoeff=c(aaw,ddw,adw,daw))
        if(withSimplify) simplifyD(erg)
        return(erg)
      })


setMethod("Minimum",
          signature(e1 = "AbscontDistribution",
          e2 = "numeric"),
          function(e1, e2, ...){
            if (abs(e2)< .Machine$double.eps) return(Dirac(Inf))
            if (!.isNatural(e2))
               stop("second argument needs to be a natural (or 0)")
            if (e2==1) return(e1)
            ## new random number function

            rnew <- function(n){
              rn1 <- matrix(r(e1)(n*e2),n,e2)
              apply(rn1,1,min)
            }

            ## new cdf
            pnew <- function(q, lower.tail = TRUE, log.p = FALSE){
              p0 <- if(lower.tail)
                  1 - (p(e1)(q, lower.tail = FALSE))^e2 else
                  (p(e1)(q, lower.tail = FALSE))^e2
              if (log.p) p0 <- log(p0)
              return(p0)
            }

            ## new density
            dnew <- function(x, log = FALSE){
              d0 <- e2 * (p(e1)(x, lower.tail = FALSE))^(e2-1) * (d(e1)(x))
              if (log) d0 <- log(d0)
              return(d0)
            }

            ## new quantile function
            qL <- q.l(e1)(0)
            qU <- q.l(e1)(1)

            ql <- getLow(e1)
            qu <- getUp(e1)
            n <- getdistrOption("DefaultNrGridPoints")
            h <- (qu-ql)/n
            xseq <- seq(from = ql, to = qu, by = h)
            px.l <- pnew(xseq, lower.tail = TRUE)
            px.u <- pnew(xseq, lower.tail = FALSE)

            qnew <- .makeQNew(xseq, px.l, px.u, FALSE, qL, qU)
            
            return(AbscontDistribution( r = rnew,
                   d = dnew, p = pnew, q = qnew, gaps = gaps(e1),
                   .withArith = TRUE))
          })

setMethod("Minimum", signature(e1 = "DiscreteDistribution",
                               e2 = "numeric"),
    function(e1, e2, ...){
        if (abs(e2)< .Machine$double.eps) return(Dirac(Inf))
        if (!.isNatural(e2))
               stop("second argument needs to be a natural (or 0)")
        if (e2==1) return(e1)

        ## new support
        supp <- support(e1)
        pnew <- 1 - (p(e1)(supp, lower.tail = FALSE))^e2
        dnew <- c(pnew[1],diff(pnew))
        res <- DiscreteDistribution(supp = supp, prob = dnew, .withArith = TRUE)
        res@.finSupport = e1@.finSupport
        res
    })

setMethod("Minimum",
          signature(e1 = "AcDcLcDistribution",
                    e2 = "numeric"),
          function(e1, e2, withSimplify = getdistrOption("simplifyD")){

        if (!.isNatural0(e2))
               stop("second argument needs to be a natural (or 0)")
        if (e2==0) return(Dirac(Inf))
        if (e2==1) return(e1)
        e1 <- as(e1, "UnivarLebDecDistribution")

        aw1 <- acWeight(e1)
        aD1 <- acPart(e1)
        dD1 <- discretePart(e1)
        ep <- getdistrOption("TruncQuantile")
        if(aw1<ep) return(Minimum(dD1,e2))
        if(1-aw1<ep) return(Minimum(aD1,e2))

        DList <- lapply(seq(e2+1)-1,
                               function(x) {Mi1 <- Minimum(aD1, x,
                                                   withSimplify=FALSE)
                                            Mi2 <- Minimum(dD1, e2-x,
                                                   withSimplify=FALSE)
                                           as(Minimum(Mi1, Mi2,
                                                withSimplify=FALSE),
                                              "UnivarLebDecDistribution")
                                           })
        erg <- do.call(flat.LCD, c(DList,
                        alist(mixCoeff = dbinom(0:e2, size = e2, prob = aw1))))
        if(withSimplify) simplifyD(erg)
        return(erg)
      })

setMethod("Maximum",
          signature(e1 = "AcDcLcDistribution",
                    e2 = "AcDcLcDistribution"),
                    function(e1,e2, withSimplify = getdistrOption("simplifyD"))
                    -Minimum(-e1,-e2, withSimplify = withSimplify))
setMethod("Maximum",
          signature(e1 = "AcDcLcDistribution",
                    e2 = "numeric"),
                    function(e1,e2, withSimplify = getdistrOption("simplifyD"))
                    -Minimum(-e1,e2, withSimplify = withSimplify))

#######################################################################

setMethod("decomposePM", "AbscontDistribution",
      function(object){
         ep <- getdistrOption("TruncQuantile")
         p0 <- as.vector(object@p(0))
         p01 <- if(.inArgs("lower.tail",object@p))
                object@p(0, lower.tail = FALSE) else 1-p0
         neg <- if(p0>ep){
                qnew.n <- function(p, lower.tail = TRUE,
                               log.p = FALSE){}
                body(qnew.n) <- substitute({
                              if (log.p) p <- exp(p)
                              if(!lower.tail) p <- 1-p
                              if (any((p < -.Machine$double.eps)|
                                      (p > 1+.Machine$double.eps)))
                                  warning(gettextf(
                                  "q method of %s produced NaN's ", objN))
                              i01 <- (-.Machine$double.eps<=p)&
                                     (p<=1+.Machine$double.eps)
                              p0x <- p[i01] ## only values in [0,1] are used
                              q0  <- p*0
                              q0[!i01] <- NaN
                              q0[ i01] <- pmin(object@q(p0x*p0),0)
                              return(as.numeric(q0))},
                            list(objN = quote(.getObjName(1))))
                AbscontDistribution(
                    p = function(q, lower.tail = TRUE, log.p = FALSE){
                               px <- object@p(q, lower.tail = lower.tail,
                                          log.p = FALSE)
                               if(lower.tail)
                                  px <- pmin(p0,px)/p0
                               else
                                  px <- pmax(0,px-p01)/p0
                               if(log.p) px <- log(px)
                               return(px)},
                    d = function(x, log = FALSE){
                               dx <- object@d(x, log = log)
                               d0 <- (x<=0)
                               if(log) dx+log(d0)-log(p0)
                                  else dx*d0/p0},
                    q = qnew.n,
                    r = function(n) qnew.n(runif(n))
                    )} else -abs(Norm())
         pos <- if(p01>ep){
                qnew <- function(p, lower.tail = TRUE,
                               log.p = FALSE){}
                body(qnew) <- substitute({
                              if (log.p) p <- exp(p)
                              if(!lower.tail) p <- 1-p
                              if (any((p < -.Machine$double.eps)|
                                      (p > 1+.Machine$double.eps)))
                                  warning(gettextf(
                                  "q method of %s produced NaN's ", objN))
                              i01 <- (-.Machine$double.eps<=p)&
                                     (p<=1+.Machine$double.eps)
                              p0x <- p[i01] ## only values in [0,1] are used
                              q0  <- p*0
                              q0[!i01] <- NaN
                              q0[ i01] <- pmax(object@q(pmin(p0+p0x*p01,1)),0)
                              return(as.numeric(q0))},
                            list(objN = quote(.getObjName(1))))
                AbscontDistribution(
                    p = function(q, lower.tail = TRUE, log.p = FALSE){
                               px <- object@p(q, lower.tail = lower.tail,
                                          log.p = FALSE)
                               if(lower.tail)
                                  px <- pmax(0,px-p0)/p01
                               else
                                  px <- pmin(p01,px)/p01
                               if(log.p) px <- log(px)
                               return(px)},
                    d = function(x, log = FALSE){
                               dx <- object@d(x, log = log)
                               d0 <- (x>=0)
                               if(log) dx+log(d0)-log(p01)
                                  else dx*d0/p01},
                    q = qnew,
                    r = function(n) qnew(runif(n))
                    )} else abs(Norm())
         list(
         "neg"=list(D = neg, w = p0*(p0>ep)),
         "pos"=list(D = pos, w = p01*(p01>ep))
          )})

setMethod("decomposePM", "DiscreteDistribution",
      function(object){
         supp <- support(object)

         suppp <- supp[supp>0]
         suppm <- supp[supp<0]

         pp <- max(1-p(object)(0),0)
         pm <- max(p(object)(0)-d(object)(0),0)

         p0 <- max(d(object)(0),0)

         Dm <- if(length(suppm))
               DiscreteDistribution(supp = suppm, prob = d(object)(suppm)/pm)
         else  Dirac(-1)
         Dm@.finSupport <- c(object@.finSupport[1],TRUE)
         Dp <- if(length(suppp))
              DiscreteDistribution(supp = suppp, prob = d(object)(suppp)/pp)
         else Dirac(1)
         Dp@.finSupport <- c(TRUE, object@.finSupport[2])

         D0 <- Dirac(0)
         list("neg" = list( D = Dm, w = pm),
              "0"   = list( D = D0, w = p0),
              "pos" = list( D = Dp, w = pp))
          })

setMethod("decomposePM", "UnivarLebDecDistribution", function(object){
         p.a <- acPart(object)
         p.d <- discretePart(object)

         w.a <- acWeight(object)
         w.d <- discreteWeight(object)

         p.a.DC <- if(w.a) decomposePM(p.a) else
                       list(pos = list(D =  Exp(), w = 1/2),
                            neg = list(D = -Exp(), w = 1/2))
         p.d.DC <- if(w.d) decomposePM(p.d) else
                       list(pos = list(D =  Dirac(1),  w = 0),
                            neg = list(D =  Dirac(-1), w = 0),
                            "0" = list(D = Dirac(0), w = 1))


         w.p0 <- p.d.DC$pos$w * w.d + p.a.DC$pos$w * w.a
         w.m0 <- p.d.DC$neg$w * w.d + p.a.DC$neg$w * w.a
         w.p <- w.p0 + (w.p0==0)
         w.m <- w.m0 + (w.m0==0)

         pos <- UnivarLebDecDistribution(
                 discretePart = p.d.DC$pos$D,
                 acPart =  p.a.DC$pos$D,
                 discreteWeight = max(min(p.d.DC$pos$w*w.d/w.p,1),0))
         neg <- UnivarLebDecDistribution(
                 discretePart = p.d.DC$neg$D,
                 acPart =  p.a.DC$neg$D,
                 discreteWeight = max(min(p.d.DC$neg$w*w.d/w.m,1),0))
         list( "pos" = list( D = pos, w = w.p0),
               "neg" = list( D = neg, w = w.m0),
               "0" = list( D = Dirac(0), w = 1-w.m0-w.p0))

})

############################ plot #######################

setMethod("plot", signature(x = "AffLinUnivarLebDecDistribution", y = "missing"),
    function(x, width = 10, height = 5.5, withSweave = getdistrOption("withSweave"),
             xlim = NULL, ylim = NULL, ngrid = 1000,
             verticals = TRUE, do.points = TRUE,
             main = FALSE, inner = TRUE, sub = FALSE,
             bmar = par("mar")[1], tmar = par("mar")[3], ...,
             cex.main = par("cex.main"), cex.inner = 1.2,
             cex.sub = par("cex.sub"), col.points = par("col"),
             col.hor = par("col"), col.vert = par("col"),
             col.main = par("col.main"), col.inner = par("col.main"),
             col.sub = par("col.sub"),  cex.points = 2.0,
             pch.u = 21, pch.a = 16, mfColRow = TRUE, to.draw.arg = NULL,
             withSubst = TRUE){

      mc <- as.list(match.call(call = sys.call(sys.parent(1)), expand.dots = TRUE)[-1])
      dots <- match.call(call = sys.call(sys.parent(1)),
                        expand.dots = FALSE)$"..."
      ret <- do.call(getMethod("plot",
              signature(x="UnivarLebDecDistribution",y="missing")), args = mc)
      ret$dots <- dots
      ret$call <- mc
      ret$args <- list(x = x, width = width, height = height,
             withSweave = withSweave, xlim = xlim, ylim = ylim, ngrid = ngrid,
             verticals = verticals, do.points = do.points,
             main = main, inner = inner, sub = sub, bmar = bmar, tmar = tmar,
             cex.main = cex.main, cex.inner = cex.inner,
             cex.sub = cex.sub, col.points = col.points,
             col.hor = col.hor, col.vert = col.vert,
             col.main = col.main, col.inner = col.inner,
             col.sub = col.sub,  cex.points = cex.points,
             pch.u = pch.u, pch.a = pch.a, mfColRow = mfColRow, to.draw.arg = to.draw.arg,
             withSubst = withSubst)
      return(invisible(ret))
})

setMethod("plot", signature(x = "UnivarLebDecDistribution", y = "missing"),
    function(x, width = 10, height = 14.5, withSweave = getdistrOption("withSweave"),
             xlim = NULL, ylim = NULL, ngrid = 1000,
             verticals = TRUE, do.points = TRUE,
             main = FALSE, inner = TRUE, sub = FALSE,
             bmar = par("mar")[1], tmar = par("mar")[3], ...,
             cex.main = par("cex.main"), cex.inner = 0.9,
             cex.sub = par("cex.sub"), col.points = par("col"),
             col.hor = par("col"), col.vert = par("col"),
             col.main = par("col.main"), col.inner = par("col.main"),
             col.sub = par("col.sub"),  cex.points = 2.0,
             pch.u = 21, pch.a = 16, mfColRow = TRUE, to.draw.arg = NULL,
             withSubst = TRUE){

      mc <- match.call(call = sys.call(sys.parent(1)), expand.dots = TRUE)
      mc1 <- mc[-1]
      xc <- mc$x
      args0 <- list(x = x, width = width, height = height,
             withSweave = withSweave, xlim = xlim, ylim = ylim, ngrid = ngrid,
             verticals = verticals, do.points = do.points,
             main = main, inner = inner, sub = sub, bmar = bmar, tmar = tmar,
             cex.main = cex.main, cex.inner = cex.inner,
             cex.sub = cex.sub, col.points = col.points,
             col.hor = col.hor, col.vert = col.vert,
             col.main = col.main, col.inner = col.inner,
             col.sub = col.sub,  cex.points = cex.points,
             pch.u = pch.u, pch.a = pch.a, mfColRow = mfColRow, to.draw.arg = to.draw.arg,
             withSubst = withSubst)

      ### manipulating the ... - argument
      dots <- match.call(call = sys.call(sys.parent(1)),
                       expand.dots = FALSE)$"..."

      plotD <- getMethod("plot", signature(x = "DiscreteDistribution", 
                                           y = "missing"))
      plotC <- getMethod("plot", signature(x = "AbscontDistribution", 
                                           y = "missing"))

      to.draw <- 1:8
      names(to.draw) <- c("p","q","d.c","p.c","q.c","d.d","p.d","q.d")
      if(! is.null(to.draw.arg)){
         if(is.character(to.draw.arg)) 
            to.draw <- pmatch(to.draw.arg, names(to.draw))
         else if(is.numeric(to.draw.arg)) 
              to.draw <- to.draw.arg
      }
      l.draw <- length(to.draw)

      xlab0.d <- xlab0.c <- list("d"="x", "p"="q", "q"="p")
      ylab0.d <- ylab0.c <- list("d"="d(x)", "p"="p(q)", "q"="q(p)")

      pF <- expression({})
      if(!is.null(dots[["panel.first"]])){
          pF <- .panel.mingle(dots,"panel.first")
      }
      pF <- .fillList(pF, l.draw)
      pL <- expression({})
      if(!is.null(dots[["panel.last"]])){
          pL <- .panel.mingle(dots,"panel.last")
      }
      pL <- .fillList(pL, l.draw)
      dots$panel.first <- dots$panel.last <- NULL

      plotCount <- 1
      if(!is(x, "UnivarLebDecDistribution"))
          x <- .ULC.cast(x)

      if(is(x,"DiscreteDistribution")){
         mcl <- as.list(mc1)
         mcl$to.draw.arg <- (1:3)[( (6:8) %in%to.draw )] 
         mcl$ngrid <- NULL
         whichPFL <- mcl$to.draw.arg   
         mcl$panel.first <- pF[whichPFL]
         mcl$panel.last  <- pL[whichPFL]
         if(is.null(mcl$xlab)) mcl$xlab <- xlab0.d
         if(is.null(mcl$ylab)) mcl$ylab <- ylab0.d
         if(!is.logical(inner)){
                if(length(inner)!=3)
                   {inner <- .fillList(inner, 8)
                     mcl$inner <- inner[6:8]}
                }                          
         ret <- do.call(plotD, mcl)
         ret$dots <- dots
         ret$call <- mc
         ret$args <- args0
         return(invisible(ret))
      }
      
      if(is(x,"AbscontDistribution")){
         mcl <- as.list(mc1)
         mcl$col.hor <- NULL
         if(is.null(mcl$xlab)) mcl$xlab <- xlab0.c
         if(is.null(mcl$ylab)) mcl$ylab <- ylab0.c
         mcl$to.draw.arg <- (1:3)[( (3:5) %in%to.draw )]
         whichPFL <- mcl$to.draw.arg   
         mcl$panel.first <- pF[whichPFL]
         mcl$panel.last  <- pL[whichPFL]
            if(!is.logical(inner)){
                if(length(inner)!=3)
                   {inner <- .fillList(inner, 8)
                     mcl$inner <- inner[6:8]}
                }                          
         ret <- do.call(plotC, as.list(mcl))
         ret$dots <- dots
         ret$call <- mc
         ret$args <- args0
         return(invisible(ret))
      }
      
      
      if(.isEqual(x@mixCoeff[1],0)){
         x <- x@mixDistr[[2]]
         mcl <- as.list(mc1)
         if(is.null(mcl$xlab)) mcl$xlab <- xlab0.d
         if(is.null(mcl$ylab)) mcl$ylab <- ylab0.d
         mcl$x <- x
         mcl$to.draw.arg <- (1:3)[( (6:8) %in%to.draw )] 
         mcl$ngrid <- NULL
         whichPFL <- if(l.draw<8) mcl$to.draw.arg else 5+mcl$to.draw.arg  
         mcl$panel.first <- pF[whichPFL]
         mcl$panel.last  <- pL[whichPFL]
            if(!is.logical(inner)){
                if(length(inner)!=3)
                   {inner <- .fillList(inner, 8)
                     mcl$inner <- inner[6:8]}
                }                          
         ret <- do.call(plotD, as.list(mcl))
         ret$dots <- dots
         ret$call <- mc
         ret$args <- args0
         return(invisible(ret))
        }

      if(.isEqual(x@mixCoeff[1],1)){
         x <- x@mixDistr[[1]]
         mcl <- as.list(mc1)
         if(is.null(mcl$xlab)) mcl$xlab <- xlab0.c
         if(is.null(mcl$ylab)) mcl$ylab <- ylab0.c
         mcl$x <- x
         mcl$to.draw.arg <- (1:3)[( (3:5) %in%to.draw )] 
         mcl$col.hor <- NULL
         whichPFL <- if(l.draw<8) mcl$to.draw.arg else 2+mcl$to.draw.arg  
         mcl$panel.first <- pF[whichPFL]
         mcl$panel.last  <- pL[whichPFL]
            if(!is.logical(inner)){
                if(length(inner)!=3)
                   {inner <- .fillList(inner, 8)
                     mcl$inner <- inner[6:8]}
                }                          
         ret <- do.call(plotC, as.list(mcl))
         ret$dots <- dots
         ret$call <- mc
         ret$args <- args0
         return(invisible(ret))
        }

      plotInfo <- list(call = mc, dots=dots, args=args0)

      dots.for.points <- .makedotsPt(dots)

      dots.lowlevel <- .makedotsLowLevel(dots)
      dots.without.pch <- dots.lowlevel[! (names(dots.lowlevel) %in% c("pch", "col"))]
      dots.for.lines <- .makedotsL(dots)
      dots.v <- dots.for.lines
      dots.v$col <- NULL
     ###
     if(!is.logical(inner))
         {if(!is.list(inner))
              inner <- as.list(inner)
            #stop("Argument 'inner' must either be 'logical' or a 'list'")
          inner <- .fillList(inner,l.draw)          
         }
     cex <- if (hasArg("cex")) dots$cex else 1

     if (hasArg("cex") && missing(cex.points))
         cex.points <- 2.0 * cex

     if (hasArg("pch") && missing(pch.u))
          pch.u <- dots$pch
     if (hasArg("pch") && missing(pch.a))
          pch.a <- dots$pch

     if (hasArg("col") && missing(col.points))
         col.points <- dots$col
     if (hasArg("col") && missing(col.vert))
         col.vert <- dots$col
     if (hasArg("col") && missing(col.main))
        col.main <- dots$col
     if (hasArg("col") && missing(col.inner))
        col.inner <- dots$col
     if (hasArg("col") && missing(col.sub))
        col.sub <- dots$col

     if (!withSweave){
           devNew(width = width, height = height)
           }
     omar <- par("mar", no.readonly = TRUE)
 #    omar$cin <- omar$cra <- omar$csi <- omar$cxy <-  omar$din <- NULL
     if(mfColRow) (on.exit(par(omar, no.readonly = TRUE)))
     
     mainL <- FALSE
     subL <- FALSE
     lineT <- NA
     logpd <- logq <- ""

     slots <-  slotNames(param(x))
     slots <-  slots[slots != "name"]
     nrvalues <-  length(slots)
     if(nrvalues > 0){
           values <-  numeric(nrvalues)
       for(i in 1:nrvalues)
         values[i] <-  attributes(attributes(x)$param)[[slots[i]]]
       paramstring <-  paste(values, collapse = ", ")
       nparamstring <-  paste(slots, "=", values, collapse = ", ")
       qparamstring <- paste("(",paramstring,")",sep="")
     }
     else paramstring <- qparamstring <- nparamstring <- ""


     .mpresubs <- if(withSubst){ 
                    function(inx)
                    .presubs(inx, c("%C", "%D", "%N", "%P", "%Q", "%A"),
                          c(as.character(class(x)[1]),
                            as.character(date()),
                            nparamstring,
                            paramstring,
                            qparamstring,
                            as.character(deparse(xc))))
                  }else function(inx)inx
 
     .mp2 <- function(dlb = dots$xlab, lb0 = list(list("p"="q", "q"="p"),
                          list("d"="x", "p"="q", "q"="p"),
                          list("d"="x", "p"="q", "q"="p"))){
              if (!is.null(dlb)){
              if(is.call(dlb)) dlb <- dlb[-1]
              .mp <- if(is.list(dlb)) function(x,i){
                                if(is.call(x)) x <- eval(x)
                                if(length(i)==0) return(NULL)
                                i <- min(i)
                                if(is.character(x[[i]])){
                                   return(as.character(eval(.mpresubs(x[[i]]))))
                                }else{
                                res <- .mpresubs(x[[i]])
                                if(length(res)==0) return(NULL)
                                if(is.call(res)) res <- res[-1]
                                return(res)}
                                }else function(x,i){
                                  if(length(x)==1) return(x[1])
                                  res <- x[i]
                                  if(length(res)==0) return(NULL)
                                  if(is.na(res)) return(NULL)
                                  return(res)}
              iL <- 1:length(to.draw)
              force(lb0)
              .mp3 <- .mp(dlb,iL[to.draw==1])
              if(1%in%to.draw & !is.null(.mp3)) lb0[[1]][["p"]] <- .mp3
              .mp3 <- .mp(dlb,iL[to.draw==2])
              if(2%in%to.draw & !is.null(.mp3)) lb0[[1]][["q"]] <- .mp3
              .mp3 <- .mp(dlb,iL[to.draw==3])
              if(3%in%to.draw & !is.null(.mp3)) lb0[[2]][["d"]] <- .mp3
              .mp3 <- .mp(dlb,iL[to.draw==4])
              if(4%in%to.draw & !is.null(.mp3)) lb0[[2]][["p"]] <- .mp3
              .mp3 <- .mp(dlb,iL[to.draw==5])
              if(5%in%to.draw & !is.null(.mp3)) lb0[[2]][["q"]] <- .mp3
              .mp3 <- .mp(dlb,iL[to.draw==6])
              if(6%in%to.draw & !is.null(.mp3)) lb0[[3]][["d"]] <- .mp3
              .mp3 <- .mp(dlb,iL[to.draw==7])
              if(7%in%to.draw & !is.null(.mp3)) lb0[[3]][["p"]] <- .mp3
              .mp3 <- .mp(dlb,iL[to.draw==8])
              if(8%in%to.draw & !is.null(.mp3)) lb0[[3]][["q"]] <- .mp3
             }
             return(lb0)}

      xlab0 <- .mp2()
      xlab0.c <- xlab0[[2]]
      xlab0.d <- xlab0[[3]]
      dots$xlab <- NULL
      ylab0 <- .mp2(dlb = dots$ylab, lb0 = list(list("p"="p(q)", "q"="q(p)"),
                          list("d"="d(x)", "p"="p(q)", "q"="q(p)"),
                          list("d"="d(x)", "p"="p(q)", "q"="q(p)")))
      ylab0.c <- xlab0[[2]]
      ylab0.d <- ylab0[[3]]
      dots$ylab <- NULL

     if (hasArg("main")){
         mainL <- TRUE
         if (is.logical(main)){
             if (!main) mainL <-  FALSE
             else
                  main <- gettextf("Distribution Plot for %%A") ###
                          ### double  %% as % is special for gettextf
             }
         main <- .mpresubs(main)
         if (mainL) {
             if(missing(tmar))
                tmar <- 5
             if(missing(cex.inner))
                cex.inner <- .9
             lineT <- 0.6
             }
     }
     if (hasArg("sub")){
         subL <- TRUE
         if (is.logical(sub)){
             if (!sub) subL <-  FALSE
             else       sub <- gettextf("generated %%D")
                          ### double  %% as % is special for gettextf
         }
         sub <- .mpresubs(sub)
         if (subL)
             if (missing(bmar)) bmar <- 6
     }

     if(mfColRow){
        opar <- par("mfrow", mar = c(bmar,omar[2],tmar,omar[4]), no.readonly = TRUE)
        ## common:
        com.drw <- (1:2)[( (1:2) %in%to.draw )]
        disc.drw <-(1:3)[( (3:5) %in%to.draw )]
        cont.drw <-(1:3)[( (6:8) %in%to.draw )]
        lcom.drw <- length(com.drw)
        ldisc.drw <- length(disc.drw)
        lcont.drw <- length(cont.drw) 
        
        nrw.drw <- (lcom.drw>0)+(ldisc.drw>0)+(lcont.drw>0)
        le.drw <- c(lcom.drw,ldisc.drw,lcont.drw)
        
        tw.drw <- any(le.drw==2); th.drw <- any(le.drw==3);
        cmm.drw <- (1+tw.drw)*(1+th.drw*2)
        ma.drw <- NULL; man.drw <- 0
        if(lcom.drw>0){
           ma.drw <- rep( 1:lcom.drw, 
                          each = cmm.drw/length(com.drw))
           man.drw <- length(com.drw)
           }
        if(ldisc.drw>0){
           ma.drw <- c(ma.drw, rep(1:ldisc.drw + man.drw, 
                       each = cmm.drw/length(disc.drw)))
           man.drw <- man.drw + length(disc.drw)
           }
        if(lcont.drw>0){
           ma.drw <- c(ma.drw, rep(1:lcont.drw + man.drw, 
                       each = cmm.drw/length(cont.drw)))
           }
        if(nrw.drw >0 ) layout(matrix(ma.drw, byrow=TRUE,  nrow=nrw.drw))
     }else 
        opar <- par(mar = c(bmar,omar[2],tmar,omar[4]), no.readonly = TRUE)

     if(is.logical(inner)){
        inner.p <- if (inner)
                   .mpresubs(gettextf("CDF of %%C%%Q")) else ""
        inner.q <- if (inner)
                   .mpresubs(gettextf("Quantile function of %%C%%Q")) else ""
                          ### double  %% as % is special for gettextf
     }else{
        iL <- 1:length(to.draw[to.draw<=2])

        inner.p <- if(1%in%to.draw) .mpresubs(inner[[min(iL[to.draw==1])]]) else NULL
        inner.q <- if(2%in%to.draw) .mpresubs(inner[[min(iL[to.draw==2])]]) else NULL
     }


     lower0 <- min(getLow(x@mixDistr[[1]],
                      eps = getdistrOption("TruncQuantile")*2),
                  getLow(x@mixDistr[[2]]))
     upper0 <- max(getUp(x@mixDistr[[1]],
                      eps = getdistrOption("TruncQuantile")*2),
                  getUp(x@mixDistr[[2]]))
     me <- q.l(x)(1/2); s <- q.l(x)(3/4)-q.l(x)(1/4)
     lower1 <- me - 6 * s
     upper1 <- me + 6 * s
     lower <- max(lower0, lower1)
     upper <- min(upper0, upper1)


     ## ngrid  nr of gridpoints
     ## exactq two p-values are considered equal if difference is
     ## is less than 10^-exactq in abs. value

     dist <- upper - lower
     del <- getdistrOption("DistrResolution")
     supp <- support(x)

     if(hasArg("xlim"))
     {  if(length(xlim)!=2) stop("Wrong length of Argument xlim");
           grid <- seq(xlim[1], xlim[2], length = ngrid)
           supp <- supp[(supp >= xlim[1]) & (supp <= xlim[2])]
     }else{grid <- seq(from = lower - 0.1 * dist, to = upper + 0.1 * dist,
                       length = ngrid)
           supp <- support(x)
     }

     grid <- unique(sort( c(supp, supp-del , grid )))
     pxg <- p(x)(grid)


     
     if(hasArg("ylim"))
         { if (any(c(2,5,8) %in% to.draw) && any( c(1,3,4,6,7) %in% to.draw)){
                 if(! length(ylim) %in% c(2,4)) 
                     stop("Wrong length of Argument ylim")
           }else{
                 if(! length(ylim) == 2) 
                     stop("Wrong length of Argument ylim")
           }                  
           ylim <- matrix(ylim, 2,2)
           ylim2 <- ylim[,2]
           }
     else ylim2 <- c(-0.05,1.05)

     if(hasArg("log"))
         {logpd <- dots$log
          logq <- gsub("u","y",gsub("y","x",gsub("x", "u", logpd)))
          if(length(grep("y",logpd))){
             ylim2 <- c(max(min(pxg[pxg>0]), ylim2[1]),
                               ylim2[2])
             }
          }

     if(!verticals){
         grid <- unique(sort( c(supp-del/2, grid )))
         grid[.isIn(grid,cbind(supp-del/2,supp-del/2))] <- NA
         pxg <- p(x)(grid)
     }else{
         xv <- as.vector(t(cbind(supp-del,supp,NA)))
         pxv <- p(x)(xv)
     }

     plotInfo$to.draw <- to.draw
     plotInfo$panelFirst <- pF
     plotInfo$panelLast <- pL

     o.warn <- getOption("warn"); options(warn = -1)
     if(1 %in% to.draw){
        on.exit(options(warn=o.warn))
        dots.lowlevel$panel.first <- pF[[plotCount]]
        dots.lowlevel$panel.last  <- pL[[plotCount]]
        dots.lowlevel$xlim <- xlim
        plotInfo$pplot$plot <- c(list(x = grid, pxg, type = "l",
             ylim = ylim2, ylab = ylab0[[1]][["p"]], xlab = xlab0[[1]][["p"]], log = logpd),
             dots.lowlevel)
        do.call(plot, c(list(x = grid, pxg, type = "l",
             ylim = ylim2, ylab = ylab0[[1]][["p"]], xlab = xlab0[[1]][["p"]], log = logpd),
             dots.lowlevel))
        plotInfo$pplot$usr <- par("usr")

        dots.lowlevel$panel.first <- dots.lowlevel$panel.last <- NULL
        dots.lowlevel$xlim <- NULL
        plotCount <- plotCount + 1
        options(warn = o.warn)
   
        pxg.d <- p(x)(supp)
        pxg.d0 <- p(x)(supp-del)
        if(do.points){
           do.call(points, c(list(x = supp, y = pxg.d, pch = pch.a,
                     cex = cex.points, col = col.points), dots.for.points))
           do.call(points, c(list(x = supp-del, y = pxg.d0, pch = pch.u,
                     cex = cex.points, col = col.points), dots.for.points))
           plotInfo$pplot$points.u <- c(list(x = supp, y = pxg.d, pch = pch.a,
                     cex = cex.points, col = col.points), dots.for.points)
           plotInfo$pplot$points.a <- c(list(x = supp-del, y = pxg.d0, pch = pch.u,
                     cex = cex.points, col = col.points), dots.for.points)
        }
        if(verticals){
            do.call(lines, c(list(x = xv, y = pxv, col = col.vert),
                    dots.v))
            plotInfo$pplot$vlines <- c(list(x = xv, y = pxv, col = col.vert),
                    dots.v)
        }
   
        title(main = inner.p, line = lineT, cex.main = cex.inner,
              col.main = col.inner)
        plotInfo$pplot$title <- list(main = inner.p, line = lineT,
                  cex.main = cex.inner, col.main = col.inner)
     }
     ### quantiles

     ### fix finite support bounds
     ixg  <-  grid>=max(q.l(x)(0),lower) & grid <= min(q.l(x)(1),upper)
     pxg  <-   pxg[ixg]
     grid <-  grid[ixg]
     if(is.finite(q.l(x)(0))) {grid <- c(q.l(x)(0),grid); pxg <- c(0,pxg)}
     if(is.finite(q.l(x)(1))) {grid <- c(grid,q.l(x)(1)); pxg <- c(pxg,1)}

#     ### fix constancy regions of p(x)
#     if(isOldVersion(x)) x <- conv2NewVersion(x)
#
#     if(length(pxv))
#         do.call(lines, c(list(x = pxv, y = xv), dots.without.pch))
     
     if(!is.null(gaps(x))){
        i.not.gap    <- !.isIn(grid,gaps(x))
        ndots <- nrow(gaps(x))
        pu1 <- p(x)(gaps(x)[,1])
        if (verticals){
             xu <- c(gaps(x)[,1],gaps(x)[,2], grid[i.not.gap])
             pu <- c(rep(pu1,2), pxg[i.not.gap])
        }else{
             xu <- c(gaps(x)[,1],rep(NA,ndots),gaps(x)[,2], grid[i.not.gap])
             pu <- c(rep(pu1,3), pxg[i.not.gap])
        }
        #
        o <- order(pu)
        po <- pu[o]
        xo <- xu[o]
     }else{
        po <- pxg
        xo <- grid
     }

     if(2 %in% to.draw){
        options(warn = -1)
        dots.without.pch$panel.first <- pF[[plotCount]]
        dots.without.pch$panel.last  <- pL[[plotCount]]
        plotInfo$qplot$plot <- c(list(x = po, xo, type = "n",
             xlim = ylim2, ylim = xlim, ylab = ylab0[[1]][["q"]], xlab = xlab0[[1]][["q"]],
             log = logq), dots.without.pch)
        do.call(plot, c(list(x = po, xo, type = "n",
             xlim = ylim2, ylim = xlim, ylab = ylab0[[1]][["q"]], xlab = xlab0[[1]][["q"]],
             log = logq), dots.without.pch), envir = parent.frame(2))
        plotInfo$qplot$usr <- par("usr")
        plotCount <- plotCount + 1
        dots.without.pch$panel.first <- dots.without.pch$panel.last <- NULL
        options(warn = o.warn)
   
   
        title(main = inner.q, line = lineT, cex.main = cex.inner,
              col.main = col.inner)
        plotInfo$qplot$title <- c(main = inner.q, line = lineT,
             cex.main = cex.inner, col.main = col.inner)

        options(warn = -1)
        do.call(lines, c(list(x=po, y=xo), dots.for.lines))
   #    if (verticals && !is.null(gaps(x))){
   #         do.call(lines, c(list(rep(pu1,2), c(gaps(x)[,1],gaps(x)[,2]),
   #                 col = col.vert), dots.without.pch))
   #     }
        plotInfo$qplot$lines <- c(list(x=po, y=xo), dots.for.lines)

        options(warn = o.warn)
   
   
        if (verticals && !is.null(gaps(x))){
                pu <- rep(pu1,3)
                xu <- c(gaps(x)[,1],gaps(x)[,2],rep(NA,ndots))
                o <- order(pu)
                do.call(lines, c(list(pu[o], xu[o],
                        col = col.vert), dots.v))
                plotInfo$qplot$vlines <- c(list(pu[o], xu[o],
                        col = col.vert), dots.v)
         }
        if(!is.null(gaps(x)) && do.points){
            do.call(points, c(list(x = pu1, y = gaps(x)[,1], pch = pch.a,
                    cex = cex.points, col = col.points), dots.for.points) )
            do.call(points, c(list(x = pu1, y = gaps(x)[,2], pch = pch.u,
                    cex = cex.points, col = col.points), dots.for.points) )
            plotInfo$qplot$points.u <- c(list(x = pu1, y = gaps(x)[,1], pch = pch.a,
                    cex = cex.points, col = col.points), dots.for.points)
            plotInfo$qplot$points.a <- c(list(x = pu1, y = gaps(x)[,2], pch = pch.u,
                    cex = cex.points, col = col.points), dots.for.points)
         }

        if(do.points){
           if(is.finite(q.l(x)(0))){
              do.call(points, c(list(x = 0, y = q.l(x)(0), pch = pch.u,
                   cex = cex.points, col = col.points), dots.for.points) )
              plotInfo$qplot$points0 <- c(list(x = 0, y = q.l(x)(0), pch = pch.u,
                   cex = cex.points, col = col.points), dots.for.points)
           }
           if(is.finite(q.l(x)(1))){
              do.call(points, c(list(x = 1, y = q.l(x)(1), pch = pch.a,
                   cex = cex.points, col = col.points), dots.for.points) )
              plotInfo$qplot$points0 <- c(list(x = 1, y = q.l(x)(1), pch = pch.a,
                   cex = cex.points, col = col.points), dots.for.points)
           }
        }
        if (mainL){
            mtext(text = main, side = 3, cex = cex.main, adj = .5,
                  outer = TRUE, padj = 1.4, col = col.main)
            plotInfo$mainL <- list(text = main, side = 3, cex = cex.main, adj = .5,
                  outer = TRUE, padj = 1.4, col = col.main)
        }
        if (subL){
            mtext(text = sub, side = 1, cex = cex.sub, adj = .5,
                  outer = TRUE, line = -1.6, col = col.sub)
            plotInfo$subL <- list(text = sub, side = 1, cex = cex.sub, adj = .5,
               outer = TRUE, line = -1.6, col = col.sub)
        }
      }
    mc.ac <- mc1
    if(!is.logical(inner)) 
       mc.ac$inner <- lapply(inner[3:5], function(x) 
                             if(is.character(x))
                                as.character(eval(.mpresubs(x)))
                             else .mpresubs(x)) 
     mc.ac$xlab <- xlab0.c
     mc.ac$ylab <- ylab0.c
     mc.ac$mfColRow <- FALSE
     mc.ac$main <- FALSE
     mc.ac$sub <- FALSE
     mc.ac$x <- NULL 
     mc.ac$withSweave <- TRUE 
     mc.ac$to.draw.arg <- (1:3)[( (3:5) %in%to.draw )] 
     if(is.null(mc.ac$cex.inner))  mc.ac$cex.inner <- 0.9

     whichPFL <- plotCount-1+mc.ac$to.draw.arg  
     mc.ac$panel.first <- pF[whichPFL]
     mc.ac$panel.last  <- pL[whichPFL]

     plotInfo$ac <- do.call(plotC, c(list(acPart(x)),mc.ac), envir = parent.frame(2))
     plotCount <- plotCount + 3

     mc.di <- mc1
     if(!is.logical(inner)) 
         mc.di$inner <- lapply(inner[6:8], function(x) 
                               if(is.character(x))
                                  as.character(eval(.mpresubs(x)))
                               else .mpresubs(x)) 
     mc.di$xlab <- xlab0.d
     mc.di$ylab <- ylab0.d
     mc.di$mfColRow <- FALSE
     mc.di$main <- FALSE
     mc.di$sub <- FALSE
     mc.di$x <- NULL
     mc.di$ngrid <- NULL
     mc.di$withSweave <- TRUE 
     mc.di$to.draw.arg <- (1:3)[( (6:8) %in%to.draw )]
     if(is.null(mc.di$cex.inner))  mc.di$cex.inner <- 0.9

     whichPFL <- plotCount-1+mc.di$to.draw.arg  
     mc.di$panel.first <- pF[whichPFL]
     mc.di$panel.last  <- pL[whichPFL]
     plotInfo$di <- do.call(plotD, c(list(discretePart(x)),mc.di), envir = parent.frame(2))
     plotCount <- plotCount + 3
     plotInfo$plotCount <- plotCount
     class(plotInfo) <- c("plotInfo","DiagnInfo")
     return(invisible(plotInfo))
   }
   )

setMethod("plot", signature(x = "CompoundDistribution", y = "missing"),
           function(x, ...) {
           mc <- as.list(match.call(call = sys.call(sys.parent(1)), 
                            expand.dots = TRUE)[-1])
           ret <- do.call(getMethod("plot",signature(x = "UnivarLebDecDistribution",
                                      y = "missing")),args=mc)
           ret$call <- mc
           return(invisible(ret))
           })

################################
##
## Class: LnormParameter
##
################################

## Access Methods
setMethod("meanlog", "LnormParameter", function(object) object@meanlog)
setMethod("sdlog", "LnormParameter", function(object) object@sdlog)
## Replace Methods
setReplaceMethod("meanlog", "LnormParameter", 
                  function(object, value){ object@meanlog <- value; object})
setReplaceMethod("sdlog", "LnormParameter", 
                  function(object, value){ object@sdlog <- value; object})

setValidity("LnormParameter", function(object){
  if(length(sdlog(object)) != 1)
    stop("sdlog has to be a numeric of length 1")    
  if(sdlog(object) <= 0)
    stop("sdlog has to be positive")
  else return(TRUE)
}           
)

################################
##
## Class: lognormal distribution
##
################################

Lnorm <- function(meanlog = 0, sdlog = 1) 
             new("Lnorm", meanlog = meanlog, sdlog = sdlog)

## wrapped access methods
setMethod("meanlog", "Lnorm", function(object) meanlog(param(object)))
setMethod("sdlog", "Lnorm", function(object) sdlog(param(object)))

## wrapped replace methods
setMethod("meanlog<-", "Lnorm", 
           function(object, value) 
               new("Lnorm", meanlog = value, sdlog = sdlog(object)))
setMethod("sdlog<-", "Lnorm", 
           function(object, value) 
               new("Lnorm", meanlog = meanlog(object), sdlog = value))

setMethod("*", c("Lnorm","numeric"),
       function(e1, e2){
         if (length(e2)>1) stop("length of operator must be 1")
         if(isTRUE(all.equal(e2,0)))  
            return(new("Dirac", location = 0, .withArith = TRUE))
         nL <- new("Lnorm", meanlog = meanlog(e1) + log(abs(e2)), 
                        sdlog = sdlog(e1), .withArith = TRUE)
         if(e2 > 0) 
            return(nL)
         else 
            return(getMethod("*", c("AbscontDistribution","numeric"))(e1,e2))
            #return(-1 * as(nL, "AbscontDistribution"))
       })

getLabel <- function (x, withnames = TRUE)
{
    cl <- as.character(class(x)[1])
    slots <-  slotNames(param(x))
    slots <-  slots[slots != "name"]
    qparamstring <-""
    nrvalues <-  length(slots)
    if(nrvalues > 0){
          values <-  numeric(nrvalues)
      for(i in 1:nrvalues)
        values[i] <-  attributes(attributes(x)$param)[[slots[i]]]
    if( withnames)
        nparamstring <-  paste(slots, "=", values, collapse = ", ")
   else
        nparamstring <-  paste(values, collapse = ", ")
    qparamstring <- paste("(",nparamstring,")",sep="")
  return(paste(cl,qparamstring,sep=""))
  }
} 

setMethod("Truncate", "AbscontDistribution",
          function(object, lower = -Inf, upper = Inf){
            ep <- .Machine$double.eps^2
            if(lower >= upper+ep) 
               stop("Argument 'lower' must be smaller than argument 'upper'")
            newgaps <- gaps(object)
            if(!is.null(newgaps)){
               newgaps[,1] <- pmax(newgaps[,1],lower)
               newgaps[,2] <- pmin(newgaps[,1],upper)
               newgaps <- newgaps[newgaps[,1]<newgaps[,2],]
               newgaps <- if(nrow(newgaps)==0) NULL else 
                            .consolidategaps(newgaps)
               }
            
            if(lower == -Inf && upper == Inf) return(object)

            if(.logExact(object)){
               if(p(object)(lower)>=0.5 && upper < Inf) 
                  return(Truncate(Truncate(object,lower=lower),
                                  upper=upper))
               if(p(object)(upper)<=0.5 && lower > -Inf) 
                  return(Truncate(Truncate(object,upper=upper),
                                  lower=lower))
               if(lower == -Inf)     erg <- .trunc.up(object, upper)
               else if(upper == Inf) erg <- .trunc.low(object, lower)
            
               if(lower == -Inf || upper == Inf){
                 rnew <- erg$r
                 pnew <- erg$p
                 dnew <- erg$d
                 qnew <- erg$q
            
                 X <- AbscontDistribution( r = rnew, gaps = newgaps,
                      d = dnew, p = pnew, q = qnew, .withArith = TRUE,
                      .withSim = object@.withSim)
                 X@.logExact <- TRUE; X@.lowerExact <- .lowerExact(object)
                 return(X)
               }
            }

            if((lower <= getLow(object)+ep)&&(upper >= getUp(object)-ep))
               return(object)

            
            ## new random number function
            rnew <- function(n){
                 rn <- r(object)(n)
                 while(TRUE){
                   rn[rn < lower+ep] = NA
                   rn[rn > upper-ep] = NA
                   index = is.na(rn)
                   if(!(any(index))) break
                   rn[index] = r(object)(sum(index))
                 }
                 rn}

            ## new cdf
            plower <- p(object)(lower)
            pupper <- p(object)(upper)
            restmass <-  pupper-plower
            if(restmass < getdistrOption("TruncQuantile"))
               stop("too little mass between args 'lower' and 'upper'")

            pnew <- .makeP(substitute(e1, list(e1 = object)),
                           substitute(alist(q = pmax(pmin(q,upper),lower)),
                                      list(upper=upper, lower=lower)),
                           fac = 1/restmass,
                           fac2 = substitute(ifelse(lower.tail,fa1,fa2),
                                      list(fa1 = -plower/restmass,
                                           fa2 = 1 - (1-plower)/restmass)))

            ## new density

            dnew <- .makeD(substitute(e1, list(e1 = object)),
                           substitute(alist(x = x)),
                           stand = restmass,
                           fac = substitute((x<=upper & x > lower),
                                     list(lower=lower,upper=upper)))

            # new quantile
            qL1 <- max(getLow(object), lower)
            qU1 <- min(getUp(object), upper)
            n <- getdistrOption("DefaultNrGridPoints")
            h <- (qU1-qL1)/n
            xseq <- seq(from = qL1, to = qU1, by = h)
            px.l <- pnew(xseq, lower.tail = TRUE)
            px.u <- pnew(xseq, lower.tail = FALSE)
            qL2 <- max(q.l(object)(0),lower)
            qU2 <- min(q.l(object)(1),upper)

            qnew <- .makeQNew(xseq, px.l, px.u, FALSE, qL2, qU2)

            erg <- AbscontDistribution( r = rnew, gaps = newgaps,
                   d = dnew, p = pnew, q = qnew, .withArith = TRUE,
                   .withSim = object@.withSim,  
                   .lowerExact = .lowerExact(object))

            if(is(object@Symmetry,"SphericalSymmetry"))
                    if(.isEqual(lower+upper,2*SymmCenter(object@Symmetry))) 
                       erg@Symmetry <- SphericalSymmetry(SymmCenter(object@Symmetry))        

            return(erg)
          })

setMethod("Truncate", "LatticeDistribution",
          function(object, lower = -Inf, upper = Inf){
            ep <- .Machine$double.eps^2
            if(lower == -Inf && upper == Inf) return(object)
            if(lower >= upper+ep) 
               stop("Argument 'lower' must be smaller than argument 'upper'")
            .finS <- object@.finSupport
            .finS[1] <- .finS[1]|(lower > -Inf)
            .finS[2] <- .finS[2]|(upper <  Inf)
            if(is.finite(Length(lattice(object)))||
               !.logExact(object)||
               (width(lattice(object)) < 0 && 
                      lower > q.l(object)(getdistrOption("TruncQuantile")))||
               (width(lattice(object)) > 0 && 
                      upper < q.l(object)(getdistrOption("TruncQuantile"),
                                        lower.tail = FALSE))               
               ){
               erg <- getMethod("Truncate","DiscreteDistribution")(object, 
                                 lower, upper)
               erg <- LatticeDistribution(DiscreteDistribution = erg, check = FALSE)
               erg@.finSupport <- .finS
               erg
            }else{
               if(p(object)(lower)>=0.5 && upper < Inf) 
                  return(Truncate(Truncate(object,lower=lower),
                                  upper=upper))
               if(p(object)(upper)<=0.5 && lower > -Inf) 
                  return(Truncate(Truncate(object,upper=upper),
                                  lower=lower))
               if(lower == -Inf)     erg <- .trunc.up(object, upper)
               else if(upper == Inf) erg <- .trunc.low(object, lower)
            
               if(lower == -Inf || upper == Inf){
                 rnew <- erg$r
                 pnew <- erg$p
                 dnew <- erg$d
                 qnew <- erg$q
            
                 
                 m <- max(getLow(object),lower)
                 M <- min(getUp(object),upper)
                 if(M<m)
                    stop("too little mass between args 'lower' and 'upper'")
                 lattice <- lattice(object)
                 p <- pivot(lattice)
                 w <- width(lattice)
                 M1 <- ceiling(abs(M-p)/abs(w))
                 m1 <- floor(abs(p-m)/abs(w))
                 s1 <- if(m1>1 && m<p) 
                          rev(seq(from = p, by = -abs(w), length.out = m1+1)) else NULL
                 S1 <- if(M1>1 && M>p) 
                          seq(from = p, by = abs(w), length.out = M1)[-1] else NULL
                 support <- sort(unique(c(s1,p,S1)))
                 support <- support[support<=M & support>=m]
                 
                 X <- LatticeDistribution(check = FALSE, 
                       DiscreteDistribution = new("DiscreteDistribution", 
                          r = rnew, d = dnew, p = pnew, q = qnew, 
                          .withArith = TRUE, .withSim = object@.withSim,
                          .logExact = TRUE, .lowerExact = .lowerExact(object),
                          support = support))

                X@.finSupport <- .finS
                if(is(object@Symmetry,"SphericalSymmetry"))
                      if(.isEqual(lower+upper,2*SymmCenter(object@Symmetry))) 
                       X@Symmetry <- SphericalSymmetry(SymmCenter(object@Symmetry))        
                return(X)
               }
            }
          })

setMethod("Truncate", "DiscreteDistribution",
          function(object, lower = -Inf, upper = Inf){
            ep <- .Machine$double.eps^2
            if(lower >= upper+ep) 
               stop("Argument 'lower' must be smaller than argument 'upper'")
            if((lower <= getLow(object))&&(upper >= getUp(object)))
               return(object)
            .finS <- object@.finSupport
            .finS[1] <- .finS[1]|(lower > -Inf)
            .finS[2] <- .finS[2]|(upper < Inf)
            supp <- support(object)
            newsupport <- supp[supp<=upper & supp>=lower]
            if(! length(newsupport))
               stop("too little mass between args 'lower' and 'upper'")
            pnewsupport <- d(object)(newsupport)/sum(d(object)(newsupport))
            erg <- DiscreteDistribution(supp = newsupport, prob = pnewsupport,
                     .withArith = TRUE, .withSim = object@.withSim,
                     .lowerExact = .lowerExact(object), 
                     .logExact = .logExact(object))
            if(is(object@Symmetry,"SphericalSymmetry"))
               if(.isEqual(lower+upper,2*SymmCenter(object@Symmetry))) 
                  erg@Symmetry <- SphericalSymmetry(SymmCenter(object@Symmetry))        
            erg@.finSupport <- .finS
            erg      
          })

setMethod("Truncate", "UnivarLebDecDistribution",
          function(object, lower = -Inf, upper = Inf, 
                   withSimplify = getdistrOption("simplifyD")){
            ep <- .Machine$double.eps^2
            if(lower >= upper+ep) 
               stop("Argument 'lower' must be smaller than argument 'upper'")
            if((lower <= getLow(object))&&(upper >= getUp(object)))
               return(object)
            aD <- acPart(object)
            aw <- acWeight(object)
            dD <- discretePart(object)
            dw <- 1 - aw
            arestmass <- p(aD)(upper) - p(aD)(lower)
            drestmass <- p(dD)(upper) - p(dD)(lower)
            awnew <- arestmass/(drestmass+arestmass)
            aDnew <- Truncate(aD, lower, upper)
            dDnew <- Truncate(dD, lower, upper)
            Dnew <- UnivarLebDecDistribution(acPart = aDnew,
                        discretePart = dDnew, acWeight = awnew)
            if(withSimplify) Dnew <- simplifyD(Dnew)
            Dnew@.lowerExact  <- .lowerExact(aD) && .lowerExact(dD)
            Dnew@.logExact <- .logExact(aD) && .logExact(dD)
            
            if(is(object@Symmetry,"SphericalSymmetry"))
               if(.isEqual(lower+upper,2*SymmCenter(object@Symmetry))) 
                  Dnew@Symmetry <- SphericalSymmetry(SymmCenter(object@Symmetry))        
            Dnew})

### for working under R < 2.3.0
if(getRversion()<'2.3.0') 
{ ## ignore ncp 

 ###beta Distribution
 
   qbeta <- function(p, shape1, shape2, ncp = 0, lower.tail = TRUE, 
                     log.p = FALSE)
           {if(isTRUE(all.equal(ncp,0)))
               stats::qbeta(p, shape1, shape2, lower.tail, log.p)
            else
              {x <- c(0.0,1.0)
               pfun <- function(x)
                       { pbeta(x, shape1=shape1, shape2=shape2, ncp=ncp)}
               qfun <- P2Q(pfun,x)
               p <- ifelse(log.p,exp(p),p)
               p <- ifelse(lower.tail,p,1-p)
               qfun(p)
              }   
            }
  
  rbeta <- function(n, shape1, shape2, ncp = 0)
           {if(isTRUE(all.equal(ncp,0)))
               stats::rbeta(n, shape1, shape2)
            else
               {X <- rchisq(n,df=2*shape1,ncp=ncp)
                Y <- rchisq(n,df=2*shape2,ncp=0)
                X/(X+Y)}
           }
 
 ###F Distribution
 
  qf    <- function(p, df1, df2, ncp = 0, lower.tail = TRUE, log.p = FALSE)
           {if(isTRUE(all.equal(ncp,0)))
               stats::qf(p, df1, df2, lower.tail, log.p)
            else
              {TQ <- getdistrOption("TruncQuantile")
               xz <- qchisq(1-TQ,df=df1,ncp=ncp); xn<-qchisq(TQ,df=df2,ncp=0)
               x <- c(0,xz/xn*df2/df1)
               pfun <- function(x){pf(x, df1=df1, df2=df2, ncp=ncp)}
               qfun <- P2Q(pfun,x)
               p <- ifelse(log.p,exp(p),p)
               p <- ifelse(lower.tail,p,1-p)
               qfun(p)
              }   
           }
  rf    <- function(n, df1, df2, ncp = 0)
           {if(isTRUE(all.equal(ncp,0)))
                 stats::rf(n, df1, df2)
            else df2*rchisq(n, df=df1, ncp=ncp)/rchisq(n, df=df2, ncp=0)/df1
           }
 
  ###T Distribution
  
  qt    <- function(p, df, ncp = 0, lower.tail = TRUE, log.p = FALSE)
           {if(isTRUE(all.equal(ncp,0)))
               stats::qt(p, df, lower.tail, log.p)
            else
              {TQ <- getdistrOption("TruncQuantile")*2
               xz <- qnorm(1-TQ,mean=df); xn<-sqrt(qchisq(TQ,df=df,ncp=0)/df)
               x <- c(-xz/xn,xz/xn)
               pfun <- function(x){pt(x, df=df, ncp=ncp)}
               qfun <- P2Q(pfun,x)
               p <- ifelse(log.p,exp(p),p)
               p <- ifelse(lower.tail,p,1-p)
               qfun(p)
              }   
           }
  rt    <- function(n, df, ncp = 0)
           {if(isTRUE(all.equal(ncp,0)))
                 stats::rt(n, df)
            else rnorm(n,mean=ncp)/sqrt(rchisq(n,df=df)/df)
           }
}

# generating function for class 'DistrList'


DistrList <- function(..., Dlist){
    ldots <- list(...)
    if(!missing(Dlist)){
        Dlist.L <- as(Dlist, "list")
        if(!is(try(do.call(DistrList,args=Dlist.L),silent=TRUE),"try-error"))
            ldots <- c(ldots, Dlist.L)
       }
    l <- length(ldots)
    if(l==0) return(NULL) else{
       do.call(new, args=list("DistrList", c(ldots)))
    }
}


# coerce to "DistrList"
setAs(from = "Distribution", to = "DistrList", 
    def = function(from){ new("DistrList", list(from)) })


# generating function for class 'UnivarDistrList'
UnivarDistrList <- function(..., Dlist){
    ldots <- list(...)
    if(!missing(Dlist)){
        Dlist.L <- as(Dlist, "list")
        if(!is(try(do.call(UnivarDistrList,args=Dlist.L),silent=TRUE),"try-error"))
            ldots <- c(ldots, Dlist.L)
       }
    l <- length(ldots)
    if(l==0) return(NULL) else{
       do.call(new, args=list("UnivarDistrList", c(ldots)))
    }
}


# coerce to "UnivarDistrList"
setAs(from = "UnivariateDistribution", to = "UnivarDistrList", 
    def = function(from){ new("UnivarDistrList", list(from)) })

# -------- AbscontDistribution ---------- #

setMethod("plot", signature(x = "AbscontDistribution", y = "missing"),
   function(x, width = 10, height = 5.5, withSweave = getdistrOption("withSweave"),
            xlim = NULL, ylim = NULL, ngrid = 1000, verticals = TRUE,
            do.points = TRUE, main = FALSE, inner = TRUE, sub = FALSE, 
            bmar = par("mar")[1], tmar = par("mar")[3], ..., 
            cex.main = par("cex.main"), cex.inner = 1.2, 
            cex.sub = par("cex.sub"), col.points = par("col"), 
            col.vert = par("col"), col.main = par("col.main"), 
            col.inner = par("col.main"), col.sub = par("col.sub"), 
            cex.points = 2.0, pch.u = 21, pch.a = 16, mfColRow = TRUE,
            to.draw.arg = NULL, withSubst = TRUE){

     mc <- match.call(call = sys.call(sys.parent(1)))
     xc <- mc$x
     ### manipulating the ... - argument
     dots <- match.call(call = sys.call(sys.parent(1)),
                        expand.dots = FALSE)$"..."

      to.draw <- 1:3
      names(to.draw) <- c("d","p","q")
      if(! is.null(to.draw.arg)){
         if(is.character(to.draw.arg)) 
            to.draw <- pmatch(to.draw.arg, names(to.draw))
         else if(is.numeric(to.draw.arg)) 
              to.draw <- to.draw.arg
      }
      l.draw <- length(to.draw)

     
     pF <- expression({})
     if(!is.null(dots[["panel.first"]])){
         pF <- .panel.mingle(dots,"panel.first")
     }
     pF <- .fillList(pF, l.draw)
     pL <- expression({})
     if(!is.null(dots[["panel.last"]])){
          pL <- .panel.mingle(dots,"panel.last")
     }
     pL <- .fillList(pL, l.draw)

     plotInfo <- list(call = mc, dots=dots,
                      args = list(width = width, height = height,
                      withSweave = withSweave,
                      xlim = xlim, ylim = ylim, ngrid = ngrid,
                      verticals = verticals, do.points = do.points,
                      main = main, inner = inner, sub = sub,
                      bmar = bmar, tmar = tmar, cex.main = cex.main,
                      cex.inner = cex.inner, cex.sub = cex.sub,
                      col.points = col.points, col.vert = col.vert,
                      col.main = col.main, col.inner = col.inner,
                      col.sub = col.sub, cex.points = cex.points,
                      pch.u = pch.u, pch.a = pch.a, mfColRow = mfColRow,
                      to.draw.arg = to.draw.arg, withSubst = withSubst),
                      to.draw=to.draw, panelFirst = pF,
                      panelLast = pL)

     plotInfo$to.draw <- to.draw
     plotInfo$panelFirst <- pF
     plotInfo$panelLast <- pL

     dots$panel.first <- dots$panel.last <- NULL
     dots$col.hor <- NULL

     dots.for.points <- .makedotsPt(dots)

     dots.lowlevel <- .makedotsLowLevel(dots)
     dots.without.pch <- dots.lowlevel[! (names(dots.lowlevel) %in% c("col", "pch"))]
     if(!is(x,"AbscontDistribution"))
         x <- .ULC.cast(x)     
     ###
     if(!is.logical(inner))
         {if(!is.list(inner))
              inner <- as.list(inner)
            #stop("Argument 'inner' must either be 'logical' or a 'list'")
          inner <- .fillList(inner,l.draw)          
         }
     cex <- if (hasArg("cex")) dots$cex else 1

     if (hasArg("cex") && missing(cex.points)) 
         cex.points <- 2.0 * cex

     if (hasArg("pch") && missing(pch.u))
          pch.u <- dots$pch
     if (hasArg("pch") && missing(pch.a))
          pch.a <- dots$pch
     
     if (hasArg("col") && missing(col.points))
         col.points <- dots$col
     if (hasArg("col") && missing(col.vert))
         col.vert <- dots$col
     if (hasArg("col") && missing(col.main))
        col.main <- dots$col
     if (hasArg("col") && missing(col.inner))
        col.inner <- dots$col
     if (hasArg("col") && missing(col.sub))
        col.sub <- dots$col

     if (!withSweave){
           devNew(width = width, height = height)
           }
     omar <- par("mar", no.readonly = TRUE)
#     omar$cin <- omar$cra <- omar$csi <- omar$cxy <-  omar$din <- NULL
     if(mfColRow) on.exit(par(omar, no.readonly = TRUE))
     
     mainL <- FALSE
     subL <- FALSE
     lineT <- NA
     logpd <- logq <- ""

     slots <-  slotNames(param(x))
     slots <-  slots[slots != "name"]
     nrvalues <-  length(slots)
     if(nrvalues > 0){
           values <-  numeric(nrvalues)
       for(i in 1:nrvalues)
         values[i] <-  attributes(attributes(x)$param)[[slots[i]]]
       paramstring <-  paste(values, collapse = ", ")
       nparamstring <-  paste(slots, "=", values, collapse = ", ")    
       qparamstring <- paste("(",paramstring,")",sep="")
     }
     else paramstring <- qparamstring <- nparamstring <- ""

     .mpresubs <- if(withSubst){
             function(inx) 
                    .presubs(inx, c("%C", "%D", "%N", "%P", "%Q", "%A"),
                          c(as.character(class(x)[1]), 
                            as.character(date()), 
                            nparamstring, 
                            paramstring, 
                            qparamstring,
                            as.character(deparse(xc))))
            }else function(inx) inx
            
     xlab0 <- list("d"="x", "p"="q", "q"="p")
     iL <- 1:length(to.draw)
     .mp2 <- function(dlb = dots$xlab, lb0 = list("d"="x", "p"="q", "q"="p")){
              dlb0 <- eval(dlb)
              if (!is.null(dlb)){
              .mp <- if(is.list(dlb0)) function(x,i){
                                if(is.call(x)) x <- eval(x)
                                if(length(i)==0) return(NULL)
                                i <- min(i)
                                if(is.character(x[[i]])){
                                   return(as.character(eval(.mpresubs(x[[i]]))))
                                }else{
                                res <- .mpresubs(x[[i]])
                                if(length(res)==0) return(NULL)
                                if(is.call(res)) res <- res[-1]
                                return(res)}
                                }else function(x,i){
                                  if(length(x)==1) return(x[1])
                                  res <- x[i]
                                  if(length(res)==0) return(NULL)
                                  if(is.na(res)) return(NULL)
                                  return(res)}
              force(lb0)
              .mp3 <- .mp(dlb,iL[to.draw==1])
              if(1%in%to.draw & !is.null(.mp3)) lb0[["d"]] <- .mp3
              .mp3 <- .mp(dlb,iL[to.draw==2])
              if(2%in%to.draw & !is.null(.mp3)) lb0[["p"]] <- .mp3
              .mp3 <- .mp(dlb,iL[to.draw==3])
              if(3%in%to.draw & !is.null(.mp3)) lb0[["q"]] <- .mp3

             }
             return(lb0)}
     xlab0 <- .mp2()
     dots$xlab <- NULL
     ylab0 <- .mp2(dlb=dots$ylab, lb0=list("d"="d(x)", "p"="p(q)", "q"="q(p)"))
     dots$ylab <- NULL

     if (hasArg("main")){
         mainL <- TRUE
         if (is.logical(main)){
             if (!main) mainL <-  FALSE 
             else 
                  main <- gettextf("Distribution Plot for %%A") ### 
                          ### double  %% as % is special for gettextf
             }
         main <- .mpresubs(main)
         if (mainL) {
             if(missing(tmar))
                tmar <- 5
             if(missing(cex.inner)) 
                cex.inner <- .9 
             lineT <- 0.6
             }
     }
     if (hasArg("sub")){ 
         subL <- TRUE
         if (is.logical(sub)){
             if (!sub) subL <-  FALSE 
             else       sub <- gettextf("generated %%D")
                          ### double  %% as % is special for gettextf
         }
         sub <- .mpresubs(sub)
         if (subL) 
             if (missing(bmar)) bmar <- 6 
     }
     
     if(mfColRow)
         opar <- par(mfrow = c(1,l.draw), mar = c(bmar,omar[2],tmar,omar[4]), no.readonly = TRUE)
     else
         opar <- par(mar = c(bmar,omar[2],tmar,omar[4]), no.readonly = TRUE)
     
     if(is.logical(inner)){     
        inner.d <- if (inner) 
                   .mpresubs(gettextf("Density of %%C%%Q")) else ""
        inner.p <- if (inner) 
                   .mpresubs(gettextf("CDF of %%C%%Q")) else ""
        inner.q <- if (inner) 
                   .mpresubs(gettextf("Quantile function of %%C%%Q")) else ""
                          ### double  %% as % is special for gettextf
     }else{
        iL <- 1:length(to.draw)

        inner.d <- if(1%in%to.draw) .mpresubs(inner[[min(iL[to.draw==1])]]) else NULL
        inner.p <- if(2%in%to.draw) .mpresubs(inner[[min(iL[to.draw==2])]]) else NULL
        inner.q <- if(3%in%to.draw) .mpresubs(inner[[min(iL[to.draw==3])]]) else NULL
     }

     lower0 <- getLow(x, eps = getdistrOption("TruncQuantile")*2)
     upper0 <- getUp(x, eps = getdistrOption("TruncQuantile")*2)
     me <- q.l(x)(1/2); s <- q.l(x)(3/4)-q.l(x)(1/4)
     lower1 <- me - 6 * s
     upper1 <- me + 6 * s
     lower <- max(lower0, lower1)
     upper <- min(upper0, upper1)

     ## ngrid  nr of gridpoints
     ## exactq two p-values are considered equal if difference is
     ## is less than 10^-exactq in abs. value  

     dist <- upper - lower
     if(hasArg("xlim")) 
         {if(length(xlim)!=2) stop("Wrong length of Argument xlim");
             grid <- seq(xlim[1], xlim[2], length = ngrid)}
     else grid <- seq(from = lower - 0.1 * dist, to = upper + 0.1 * dist, 
                      length = ngrid)

     dxg <- d(x)(grid)
     pxg <- p(x)(grid)
     
     
     if(hasArg("ylim"))
         {if (3 %in% to.draw && any( c(1,2) %in% to.draw)){
                 if(! length(ylim) %in% c(2,4)) 
                     stop("Wrong length of Argument ylim")
           }else{
                 if(! length(ylim) == 2) 
                     stop("Wrong length of Argument ylim")
           }                  
           ylim <- matrix(ylim, 2,2)
           ylim1 <- ylim[,1]; ylim2 <- ylim[,2];
           }
     else {ylim1 <- c(0,max(dxg[dxg<50])); ylim2 <- c(-0.05,1.05)}

     if(hasArg("log"))
         {logpd <- dots$log
          logq <- gsub("u","y",gsub("y","x",gsub("x", "u", logpd)))
          if(length(grep("y",logpd))){ 
             ylim1 <- c(max(min(dxg[dxg>0]), ylim1[1]), 
                             ylim1[2])
             ylim2 <- c(max(min(pxg[pxg>0]), ylim2[1]), 
                             ylim2[2])
             }
          }

     plotCount <- 1
     o.warn <- getOption("warn"); options(warn = -1)
     if(1%in%to.draw){
         on.exit(options(warn=o.warn))
         dots.lowlevel$panel.first <- pF[[plotCount]]
         dots.lowlevel$panel.last  <- pL[[plotCount]]
         dots.lowlevel$xlim <- xlim
         plotInfo$dplot$plot <- c(list(x = grid, dxg, type = "l",
             ylim = ylim1,  ylab = ylab0[["d"]], xlab = xlab0[["d"]], log = logpd),
             dots.lowlevel)
         do.call(plot, c(list(x = grid, dxg, type = "l",
             ylim = ylim1,  ylab = ylab0[["d"]], xlab = xlab0[["d"]], log = logpd),
             dots.lowlevel))
         plotInfo$dplot$usr <- par("usr")
         dots.lowlevel$panel.first <- dots.lowlevel$panel.last <- NULL
         dots.lowlevel$xlim <- NULL
         plotCount <- plotCount + 1
         options(warn = o.warn)
     
         plotInfo$dplot$title <- list(main = inner.d, line = lineT,
               cex.main = cex.inner, col.main = col.inner)

         title(main = inner.d, line = lineT, cex.main = cex.inner,
               col.main = col.inner)
     
     
         options(warn = -1)
     }
     if(is.finite(q.l(x)(0))) {grid <- c(q.l(x)(0),grid); pxg <- c(0,pxg)}
     if(is.finite(q.l(x)(1))) {grid <- c(grid,q.l(x)(1)); pxg <- c(pxg,1)}

     if(2%in%to.draw){
        dots.lowlevel$panel.first <- pF[[plotCount]]
        dots.lowlevel$panel.last  <- pL[[plotCount]]
        dots.lowlevel$xlim <- xlim
        plotInfo$pplot$plot <- c(list(x = grid, pxg, type = "l",
             ylim = ylim2, ylab = ylab0[["p"]], xlab = xlab0[["p"]], log = logpd),
             dots.lowlevel)
        do.call(plot, c(list(x = grid, pxg, type = "l",
             ylim = ylim2, ylab = ylab0[["p"]], xlab = xlab0[["p"]], log = logpd),
             dots.lowlevel))
        plotInfo$pplot$usr <- par("usr")
        dots.lowlevel$panel.first <- dots.lowlevel$panel.last <- NULL
        dots.lowlevel$xlim <- NULL
        plotCount <- plotCount + 1
        options(warn = o.warn)
        plotInfo$pplot$title <- list(main = inner.p, line = lineT,
                  cex.main = cex.inner, col.main = col.inner)

        title(main = inner.p, line = lineT, cex.main = cex.inner,
              col.main = col.inner)
     }
     ### quantiles

     ### fix finite support bounds
     ixg  <-  grid>=max(q.l(x)(0),lower) & grid <= min(q.l(x)(1),upper)
     pxg  <-   pxg[ixg]
     grid <-  grid[ixg]

     ### fix constancy regions of p(x)
     if(isOldVersion(x)) x <- conv2NewVersion(x)
                 
     if(!is.null(gaps(x))){
        i.not.gap    <- !.isIn(grid,gaps(x))
        ndots <- nrow(gaps(x))
        pu1 <- p(x)(gaps(x)[,1])
        if (verticals){
             xu <- c(gaps(x)[,1],gaps(x)[,2], grid[i.not.gap])
             pu <- c(rep(pu1,2), pxg[i.not.gap])
        }else{
             xu <- c(gaps(x)[,1],rep(NA,ndots),gaps(x)[,2], grid[i.not.gap])
             pu <- c(rep(pu1,3), pxg[i.not.gap])
        }     
        #
        o <- order(pu)
        po <- pu[o]
        xo <- xu[o]
     }else{
        po <- pxg
        xo <- grid
     }
     
     if(3%in%to.draw){
        options(warn = -1)
        dots.lowlevel$panel.first <- pF[[plotCount]]
        dots.lowlevel$panel.last  <- pL[[plotCount]]
        plotInfo$qplot$plot <- c(list(x = po, xo, type = "n",
             xlim = ylim2, ylim = xlim, ylab = ylab0[["q"]], xlab = xlab0[["q"]],
             log = logq), dots.lowlevel)
        do.call(plot, c(list(x = po, xo, type = "n",
             xlim = ylim2, ylim = xlim, ylab = ylab0[["q"]], xlab = xlab0[["q"]],
             log = logq), dots.lowlevel))
        plotInfo$qplot$usr <- par("usr")
        dots.lowlevel$panel.first <- dots.lowlevel$panel.last <- NULL
        plotCount <- plotCount + 1
        options(warn = o.warn)
    
        
        plotInfo$qplot$title <- list(main = inner.q, line = lineT,
              cex.main = cex.inner, col.main = col.inner)
        title(main = inner.q, line = lineT, cex.main = cex.inner,
              col.main = col.inner)
        
        options(warn = -1)
            lines(po,xo, ...)
        if (verticals && !is.null(gaps(x))){
            pu <- rep(pu1,3)
            xu <- c(gaps(x)[,1],gaps(x)[,2],rep(NA,ndots))
            o <- order(pu)
            dots.without.pch0 <- dots.without.pch
            dots.without.pch0$col <- NULL
            do.call(lines, c(list(pu[o], xu[o], 
                    col = col.vert), dots.without.pch0))    
            plotInfo$qplot$vlines <- c(list(x=pu[o], y=xu[o],
                    col = col.vert), dots.without.pch0)
        }
        options(warn = o.warn)
     
        if(!is.null(gaps(x)) && do.points){
           do.call(points, c(list(x = pu1, y = gaps(x)[,1], pch = pch.a, 
                   cex = cex.points, col = col.points), dots.for.points) )
           do.call(points, c(list(x = pu1, y = gaps(x)[,2], pch = pch.u,
                   cex = cex.points, col = col.points), dots.for.points) )
           plotInfo$qplot$vpoints.l <- c(list(x=pu1, y=gaps(x)[,1],
                   pch = pch.a, cex = cex.points, col = col.points),
                   dots.for.points)
           plotInfo$qplot$vpoints.r <- c(list(x=pu1, y=gaps(x)[,2],
                   pch = pch.a, cex = cex.points, col = col.points),
                   dots.for.points)
        }
     }      
     if (mainL){
         mtext(text = main, side = 3, cex = cex.main, adj = .5, 
               outer = TRUE, padj = 1.4, col = col.main)                            
         plotInfo$mainL <- list(text = main, side = 3, cex = cex.main, adj = .5,
               outer = TRUE, padj = 1.4, col = col.main)
     }
     if (subL){
         mtext(text = sub, side = 1, cex = cex.sub, adj = .5,
               outer = TRUE, line = -1.6, col = col.sub)                            
         plotInfo$subL <- list(text = sub, side = 1, cex = cex.sub, adj = .5,
               outer = TRUE, line = -1.6, col = col.sub)
     }
   class(plotInfo) <- c("plotInfo","DiagnInfo")
   return(invisible(plotInfo))
   }
   )
# -------- DiscreteDistribution -------- #

setMethod("plot", signature(x = "DiscreteDistribution", y = "missing"),
    function(x,  width = 10, height = 5.5, withSweave = getdistrOption("withSweave"), 
             xlim = NULL, ylim = NULL, verticals = TRUE, do.points = TRUE, 
             main = FALSE, inner = TRUE, sub = FALSE,
             bmar = par("mar")[1], tmar = par("mar")[3], ..., 
             cex.main = par("cex.main"), cex.inner = 1.2, 
             cex.sub = par("cex.sub"), col.points = par("col"), 
             col.hor = par("col"), col.vert = par("col"), 
             col.main = par("col.main"), col.inner = par("col.main"), 
             col.sub = par("col.sub"),  cex.points = 2.0, 
             pch.u = 21, pch.a = 16, mfColRow = TRUE,
             to.draw.arg = NULL, withSubst = TRUE){

      mc <- match.call(call = sys.call(sys.parent(1)))
      xc <- mc$x
      ### manipulating the ... - argument
      dots <- match.call(call = sys.call(sys.parent(1)), 
                       expand.dots = FALSE)$"..."

      plotInfo <- list(call = mc, dots=dots,
                      args = list(width = width, height = height,
                         withSweave = withSweave,
                         xlim = xlim, ylim = ylim, verticals = verticals,
                         do.points = do.points, main = main, inner = inner,
                         sub = sub, bmar = bmar, tmar = tmar, cex.main = cex.main,
                         cex.inner = cex.inner, cex.sub = cex.sub,
                         col.points = col.points, col.hor = col.hor,
                         col.vert = col.vert, col.main = col.main,
                         col.inner = col.inner, col.sub = col.sub,
                         cex.points = cex.points, pch.u = pch.u,
                         pch.a = pch.a, mfColRow = mfColRow,
                         to.draw.arg = to.draw.arg, withSubst = withSubst))
      to.draw <- 1:3
      names(to.draw) <- c("d","p","q")
      if(! is.null(to.draw.arg)){
         if(is.character(to.draw.arg)) 
            to.draw <- pmatch(to.draw.arg, names(to.draw))
         else if(is.numeric(to.draw.arg)) 
              to.draw <- to.draw.arg
      }
      l.draw <- length(to.draw)

      pF <- expression({})
      if(!is.null(dots[["panel.first"]])){
         pF <- .panel.mingle(dots,"panel.first")
      }
      pF <- .fillList(pF, l.draw)
      pL <- expression({})
      if(!is.null(dots[["panel.last"]])){
          pL <- .panel.mingle(dots,"panel.last")
      }
      pL <- .fillList(pL, l.draw)

      plotInfo$to.draw <- to.draw
      plotInfo$panelFirst <- pF
      plotInfo$panelLast <- pL

      dots$panel.first <- dots$panel.last <- NULL

      dots$ngrid <- NULL

      dots.for.points <- .makedotsPt(dots)
      dots.lowlevel <- .makedotsLowLevel(dots)
      dots.without.pch <- dots.lowlevel[! (names(dots.lowlevel) %in% c("col", "pch"))]
      ###
     if(!is(x,"DiscreteDistribution"))
         x <- .ULC.cast(x)     
      
     if(!is.logical(inner))
         {if(!is.list(inner))
              inner <- as.list(inner)
            #stop("Argument 'inner' must either be 'logical' or a 'list'")
          inner <- .fillList(inner,l.draw)          
         }

     cex <- if (hasArg("cex")) dots$cex else 1

     if (hasArg("cex") && missing(cex.points)) 
         cex.points <- 2.0 * cex

     if (hasArg("pch") && missing(pch.u))
          pch.u <- dots$pch
     if (hasArg("pch") && missing(pch.a))
          pch.a <- dots$pch
     
     if (hasArg("col") && missing(col.points))
         col.points <- dots$col
     if (hasArg("col") && missing(col.vert))
         col.vert <- dots$col
     if (hasArg("col") && missing(col.hor))
         col.hor <- dots$col
     if (hasArg("col") && missing(col.main))
        col.main <- dots$col
     if (hasArg("col") && missing(col.inner))
        col.inner <- dots$col
     if (hasArg("col") && missing(col.sub))
        col.sub <- dots$col

     if (!withSweave){
           devNew(width = width, height = height)
           }
     omar <- par("mar", no.readonly = TRUE)
 #    omar$cin <- omar$cra <- omar$csi <- omar$cxy <-  omar$din <- NULL
     if(mfColRow) on.exit(par(omar, no.readonly = TRUE))
     
     mainL <- FALSE
     subL <- FALSE
     lineT <- NA
     logpd <- logq <- ""

      ## getting the parameter
     slots <-  slotNames(param(x))
     slots <-  slots[slots != "name"]
     nrvalues <-  length(slots)
     if(nrvalues > 0){
           values <-  numeric(nrvalues)
       for(i in 1:nrvalues)
         values[i] <-  attributes(attributes(x)$param)[[slots[i]]]
       paramstring <-  paste(values, collapse = ", ")
       nparamstring <-  paste(slots, "=", values, collapse = ", ")
       qparamstring <- paste("(",paramstring,")",sep="")
     }
     else paramstring <- qparamstring <- nparamstring <- ""


     .mpresubs <- if(withSubst){
             function(inx) 
                    .presubs(inx, c("%C", "%D", "%N", "%P", "%Q", "%A"),
                          c(as.character(class(x)[1]), 
                            as.character(date()), 
                            nparamstring, 
                            paramstring, 
                            qparamstring,
                            as.character(deparse(xc))))
            }else function(inx) inx

     xlab0 <- list("d"="x", "p"="q", "q"="p")
     iL <- 1:length(to.draw)
     .mp2 <- function(dlb = dots$xlab, lb0 = list("d"="x", "p"="q", "q"="p")){
              dlb0 <- eval(dlb)
              if (!is.null(dlb)){
              .mp <- if(is.list(dlb0)) function(x,i){
                                if(is.call(x)) x <- eval(x)
                                if(length(i)==0) return(NULL)
                                i <- min(i)
                                if(is.character(x[[i]])){
                                   return(as.character(eval(.mpresubs(x[[i]]))))
                                }else{
                                res <- .mpresubs(x[[i]])
                                if(length(res)==0) return(NULL)
                                if(is.call(res)) res <- res[-1]
                                return(res)}
                                }else function(x,i){
                                  if(length(x)==1) return(x[1])
                                  res <- x[i]
                                  if(length(res)==0) return(NULL)
                                  if(is.na(res)) return(NULL)
                                  return(res)}
              force(lb0)
              .mp3 <- .mp(dlb,iL[to.draw==1])
              if(1%in%to.draw & !is.null(.mp3)) lb0[["d"]] <- .mp3
              .mp3 <- .mp(dlb,iL[to.draw==2])
              if(2%in%to.draw & !is.null(.mp3)) lb0[["p"]] <- .mp3
              .mp3 <- .mp(dlb,iL[to.draw==3])
              if(3%in%to.draw & !is.null(.mp3)) lb0[["q"]] <- .mp3

             }
             return(lb0)}
     xlab0 <- .mp2()
     dots$xlab <- NULL
     ylab0 <- .mp2(dlb=dots$ylab, lb0=list("d"="d(x)", "p"="p(q)", "q"="q(p)"))
     dots$ylab <- NULL

     if (hasArg("main")){
         mainL <- TRUE
         if (is.logical(main)){
             if (!main) mainL <-  FALSE 
             else 
                  main <- gettextf("Distribution Plot for %%A") ### 
                          ### double  %% as % is special for gettextf
             }
         main <- .mpresubs(main)
         if (mainL) {
             if(missing(tmar))
                tmar <- 5
             if(missing(cex.inner)) 
                cex.inner <- .9 
             lineT <- 0.6
             }
     }
     if (hasArg("sub")){ 
         subL <- TRUE
         if (is.logical(sub)){
             if (!sub) subL <-  FALSE 
             else       sub <- gettextf("generated %%D")
                          ### double  %% as % is special for gettextf
         }
         sub <- .mpresubs(sub)
         if (subL) 
             if (missing(bmar)) bmar <- 6 
     }
     
     if(mfColRow)
        opar <- par(mfrow = c(1,l.draw), mar = c(bmar,omar[2],tmar,omar[4]), no.readonly = TRUE)
     else 
        opar <- par(mar = c(bmar,omar[2],tmar,omar[4]), no.readonly = TRUE)
     
     if(is.logical(inner)){     
        inner.d <- if (inner) 
                   .mpresubs(gettextf("Probability function of %%C%%Q")) else 
                   ""
        inner.p <- if (inner) 
                   .mpresubs(gettextf("CDF of %%C%%Q")) else ""
        inner.q <- if (inner) 
                   .mpresubs(gettextf("Quantile function of %%C%%Q")) else ""
                          ### double  %% as % is special for gettextf
     }else{
        iL <- 1:length(to.draw)

        inner.d <- if(1%in%to.draw) .mpresubs(inner[[min(iL[to.draw==1])]]) else NULL
        inner.p <- if(2%in%to.draw) .mpresubs(inner[[min(iL[to.draw==2])]]) else NULL
        inner.q <- if(3%in%to.draw) .mpresubs(inner[[min(iL[to.draw==3])]]) else NULL
     }
                              
      lower <- min(support(x))
      upper <- max(support(x))
      dist <-  upper - lower

      supp <- support(x); 
      
      if(hasArg("xlim")) 
            {if(length(xlim) != 2) 
                stop("Wrong length of Argument xlim")
             supp <- supp[(supp >= xlim[1]) & (supp <= xlim[2])]         

            }else{
            }

       dx <- d(x)(supp)

     if(hasArg("ylim"))
         {if (3 %in% to.draw && any( c(1,2) %in% to.draw)){
                 if(! length(ylim) %in% c(2,4)) 
                     stop("Wrong length of Argument ylim")
           }else{
                 if(! length(ylim) == 2) 
                     stop("Wrong length of Argument ylim")
           }                  
           ylim <- matrix(ylim, 2,2)
           ylim1 <- ylim[,1]; ylim2 <- ylim[,2];
           }else{
              ylim1 <- c(0, max(dx)) 
              ylim2 <- c(-0.05,1.05)
              }

      if(hasArg("log"))
          {logpd <- dots$log
           logq <- gsub("u","y",gsub("y","x",gsub("x", "u", logpd)))
           if(length(grep("y",logpd))){ 
              ylim1 <- c(max(min(dx[dx>0]), ylim1[1]), 
                              ylim1[2])
              ylim2 <- c(max(min(dx[dx>0]), ylim2[1]), 
                              ylim2[2])
              }
           }

       o.warn <- getOption("warn")
       options(warn = -1)
       on.exit(options(warn=o.warn))

     plotCount <- 1

     if(1%in%to.draw){
       dots.without.pch$panel.first <- pF[[plotCount]]
       dots.without.pch$panel.last  <- pL[[plotCount]]
       plotInfo$dplot$plot <- c(list(x = supp, dx, type = "h", pch = pch.a,
            ylim = ylim1, xlim=xlim, ylab = ylab0[["d"]], xlab = xlab0[["d"]],
            log = logpd), dots.without.pch)
       do.call(plot, c(list(x = supp, dx, type = "h", pch = pch.a,
            ylim = ylim1, xlim=xlim, ylab = ylab0[["d"]], xlab = xlab0[["d"]],
            log = logpd), dots.without.pch))
       plotInfo$dplot$usr <- par("usr")
       dots.without.pch$panel.first <- dots.without.pch$panel.last <- NULL
       plotCount <- plotCount + 1
       options(warn = o.warn)


       title(main = inner.d, line = lineT, cex.main = cex.inner,
             col.main = col.inner)
       plotInfo$dplot$title <- list(main = inner.d, line = lineT,
             cex.main = cex.inner, col.main = col.inner)

       if(do.points){
          do.call(points, c(list(x = supp, y = dx, pch = pch.a,
                  cex = cex.points, col = col.points), dots.for.points))
       plotInfo$dplot$points <- c(list(x = supp, y = dx, pch = pch.a,
                  cex = cex.points, col = col.points), dots.for.points)
       }
       options(warn = -1)
       }
     ngrid <- length(supp)

     supp1 <- if(ngrid>1) supp else c(-max(1,abs(supp))*.08,0)+supp
     psupp1 <- c(0,p(x)(supp1))

     if(2%in%to.draw){
       dots.without.pch$panel.first <- pF[[plotCount]]
       dots.without.pch$panel.last  <- pL[[plotCount]]
       plotInfo$pplot$plot <- c(list(x = stepfun(x = supp1, y = psupp1),
                     main = "", verticals = verticals,
                     do.points = FALSE,
                     ylim = ylim2, ylab = ylab0[["p"]], xlab = xlab0[["p"]],
                     col.hor = col.hor, col.vert = col.vert,
                     log = logpd), dots.without.pch)
       do.call(plot, c(list(x = stepfun(x = supp1, y = psupp1),
                     main = "", verticals = verticals, 
                     do.points = FALSE, 
                     ylim = ylim2, ylab = ylab0[["p"]], xlab = xlab0[["p"]],
                     col.hor = col.hor, col.vert = col.vert, 
                     log = logpd), dots.without.pch))
       plotInfo$pplot$usr <- par("usr")
       dots.without.pch$panel.first <- dots.without.pch$panel.last <- NULL
       plotCount <- plotCount + 1
       if(do.points)
          {if(ngrid>1){
              do.call(points, c(list(x = supp, y = psupp1[1:ngrid], pch = pch.u, 
                  cex = cex.points, col = col.points), dots.for.points))
              do.call(points, c(list(x = supp, y = psupp1[2:(ngrid+1)], pch = pch.a, 
                  cex = cex.points, col = col.points), dots.for.points))
              plotInfo$pplot$points.u <- c(list(x = supp, y = psupp1[1:ngrid], pch = pch.u,
                  cex = cex.points, col = col.points), dots.for.points)
              plotInfo$pplot$points.a <- c(list(x = supp, y = psupp1[2:(ngrid+1)], pch = pch.a,
                  cex = cex.points, col = col.points), dots.for.points)
              }else{
              do.call(points, c(list(x = supp, y = 0, pch = pch.u, 
                  cex = cex.points, col = col.points), dots.for.points))           
              do.call(points, c(list(x = supp, y = 1, pch = pch.a, 
                  cex = cex.points, col = col.points), dots.for.points))           
              plotInfo$pplot$points.u <- c(list(x = supp, y = 0, pch = pch.u,
                  cex = cex.points, col = col.points), dots.for.points)
              plotInfo$pplot$points.a <- c(list(x = supp, y = 1, pch = pch.a,
                  cex = cex.points, col = col.points), dots.for.points)
              }
           }       
       options(warn = o.warn)

       
       title(main = inner.p, line = lineT, cex.main = cex.inner, 
             col.main = col.inner)
       plotInfo$pplot$title <- c(main = inner.p, line = lineT,
             cex.main = cex.inner, col.main = col.inner)


       if(do.points){
          do.call(points, c(list(x = supp, 
                  y = c(0,p(x)(supp[-length(supp)])), pch = pch.u, 
                  cex = cex.points, col = col.points), dots.for.points))
          plotInfo$pplot$points <- c(list(x = supp,
                  y = c(0,p(x)(supp[-length(supp)])), pch = pch.u,
                  cex = cex.points, col = col.points), dots.for.points)
       }
     }

     if(3%in%to.draw){
       options(warn = -1)
       dots.without.pch$panel.first <- pF[[plotCount]]
       dots.without.pch$panel.last  <- pL[[plotCount]]
       plotInfo$qplot$plot <- c(list(x = stepfun(c(0,p(x)(supp)),
                            c(NA,supp,NA), right = TRUE),
            main = "", xlim = ylim2, ylim = c(min(supp),max(supp)),
            ylab = ylab0[["q"]], xlab = xlab0[["q"]],
            verticals = verticals, do.points = do.points,
            cex.points = cex.points, pch = pch.a,
            col.points = col.points,
            col.hor = col.hor, col.vert = col.vert,
            log = logq), dots.without.pch)
       do.call(plot, c(list(x = stepfun(c(0,p(x)(supp)),
                            c(NA,supp,NA), right = TRUE), 
            main = "", xlim = ylim2, ylim = c(min(supp),max(supp)),
            ylab = ylab0[["q"]], xlab = xlab0[["q"]],
            verticals = verticals, do.points = do.points, 
            cex.points = cex.points, pch = pch.a, 
            col.points = col.points,
            col.hor = col.hor, col.vert = col.vert, 
            log = logq), dots.without.pch))
       plotInfo$qplot$usr <- par("usr")
       dots.without.pch$panel.first <- dots.without.pch$panel.last <- NULL
       plotCount <- plotCount + 1
       options(warn = o.warn)

      
       title(main = inner.q, line = lineT, cex.main = cex.inner,
             col.main = col.inner)
       plotInfo$qplot$title <- c(main = inner.q, line = lineT,
             cex.main = cex.inner, col.main = col.inner)

       dots.without.pch0 <- dots.without.pch
       dots.without.pch0$col <- NULL

       do.call(lines, c(list(x = c(0,p(x)(supp[1])), y = rep(supp[1],2),  
                  col = col.vert), dots.without.pch0))           
       plotInfo$qplot$lines <- c(list(x = c(0,p(x)(supp[1])), y = rep(supp[1],2),
                  col = col.vert), dots.without.pch0)

       if(do.points){
           do.call(points, c(list(x = p(x)(supp[-length(supp)]),
                  y = supp[-1], pch = pch.u, cex = cex.points, 
                  col = col.points), dots.for.points))
           do.call(points, c(list(x = 0, y = supp[1], pch = pch.u, 
                  cex = cex.points, col = col.points), dots.for.points))
           plotInfo$qplot$points.u <- c(list(x = p(x)(supp[-length(supp)]),
                  y = supp[-1], pch = pch.u, cex = cex.points,
                  col = col.points), dots.for.points)
           plotInfo$qplot$points.a <- c(list(x = 0, y = supp[1], pch = pch.u,
                  cex = cex.points, col = col.points), dots.for.points)
       }
        
       if(verticals && ngrid>1)
          {dots.without.pch0 <- dots.without.pch
           dots.without.pch0$col <- NULL

           do.call(lines, c(list(x = rep(p(x)(supp[1]),2), y = c(supp[1],supp[2]),  
                  col = col.vert), dots.without.pch0))
           plotInfo$qplot$vlines <- c(list(x = rep(p(x)(supp[1]),2), y = c(supp[1],supp[2]),
                  col = col.vert), dots.without.pch0)
          }
       }                      
       
     if (mainL){
           mtext(text = main, side = 3, cex = cex.main, adj = .5, 
                 outer = TRUE, padj = 1.4, col = col.main)                            
           plotInfo$mainL <- list(text = main, side = 3, cex = cex.main, adj = .5,
               outer = TRUE, padj = 1.4, col = col.main)
     }
     if (subL){
           mtext(text = sub, side = 1, cex = cex.sub, adj = .5,
                 outer = TRUE, line = -1.6, col = col.sub)                            
           plotInfo$subL <- list(text = sub, side = 1, cex = cex.sub, adj = .5,
               outer = TRUE, line = -1.6, col = col.sub)
     }
   class(plotInfo) <- c("plotInfo","DiagnInfo")
   return(invisible(plotInfo))
   }
)

# -------- DistributionList   ---------- #

setMethod("plot", signature(x =  "DistrList", y = "missing"),
    function(x, ...){ 
        mc <- as.list(match.call(call = sys.call(sys.parent(1)),
                            expand.dots = TRUE)[-1])
        plotInfoList <- vector("list",length(x))
        plotInfoList$call <- mc
        for(i in 1:length(x)){
            #devNew()
            plotInfoList[[i]] <- plot(x[[i]],...)
        }
        class(plotInfoList) <- c("plotInfo","DiagnInfo")
        return(invisible(plotInfoList))
    })

################################
##
## Class: Laplace or Double Exponential distribution
##
################################


## wrapped access methods
setMethod("rate", "DExp", function(object) rate(param(object)))

## wrapped replace methods
setMethod("rate<-", "DExp", function(object, value) new("DExp", rate = value))

DExp <- function(rate = 1) {D <- new("DExp", rate = rate)
                            D@Symmetry <- SphericalSymmetry(0)
                            D}

setMethod("*", c("DExp","numeric"),
          function(e1, e2){
            if (length(e2)>1) stop("length of operator must be 1")
            if(isTRUE(all.equal(e2,0))) 
               return(new("Dirac", location = 0, .withArith = TRUE))
            else  
               return(new("DExp", rate = rate(e1) / abs(e2), .withArith = TRUE))
          })

#### as to whether to use Generating functions or to use initialize methods:
#### https://tolstoy.newcastle.edu.au/R/e2/devel/07/01/1976.html
                     
################################################################################
## SPACES
################################################################################

setMethod("initialize", "Reals",
          function(.Object) {
            .Object@dimension <-  1
            .Object@name <- gettext("Real Space")
            .Object
          })


setMethod("initialize", "Naturals",
          function(.Object) {
            .Object@dimension <-  1
            .Object@name <- gettext("Grid of Naturals")
            .Object
          })


################################################################################
## PARAMETERS
################################################################################

# defunct as of 2.8.0
#setMethod("initialize", "GeomParameter",
#          function(.Object, prob = .5) {
#            .Deprecated(new = "new(\"NbinomParameter\"(size = 1, prob, name)",
#                        package = "distr",
#                        msg = gettext(
#"Class 'GeomParameter' is no longer needed and will be replaced by \nclass 'NbinomParameter' soon."
#                        ))
#            .Object@prob <- prob
#            .Object@name <- gettext("Parameter of a Geometric distribution")
#            .Object
#          })
################################################################################
## DISTRIBUTIONS
################################################################################

## Class: UnivariateDistribution
###produces difficulties in coercing...:
#
#setMethod("initialize", "UnivariateDistribution",
#          function(.Object, r = NULL, d = NULL, p = NULL, q = NULL, 
#                    param = NULL, img = new("Reals"),
#                    .withSim = FALSE, .withArith = FALSE) {
#            if(is.null(r)) {
#              stop("You have at least to give the slot r.")
#              return(invisible())}
#            ### Attention: no checking!!!
#            .Object@img <- img
#            .Object@param <- param
#            .Object@d <- d
#            .Object@p <- p
#            .Object@q <- q
#           .Object@r <- r
#            .Object@.withSim <- .withSim
#            .Object@.withArith <- .withArith
#            .Object })

## class AbscontDistribution
setMethod("initialize", "AbscontDistribution",
          function(.Object, r = NULL, d = NULL, p = NULL, q = NULL, 
                   gaps = NULL, param = NULL, img = new("Reals"),
                   .withSim = FALSE, .withArith = FALSE,
                   .lowerExact = FALSE, .logExact = FALSE,
                   low1 = NULL, up1 = NULL, low = -Inf, up =Inf,
                   Symmetry = NoSymmetry()
                   ) {
            ## don't use this if the call is new("AbscontDistribution")
            LL <- length(sys.calls())
            if(sys.calls()[[LL-3]] == "new(toDef)")
               {return(.Object)}
            if(sys.calls()[[LL-3]] == "new(\"AbscontDistribution\")")
               {return(.Object)}
            
            if(is.null(r))
               warning("you have to specify slot r at least")
                          
            ## TOBEDONE Errorkanal
            
            dpq.approx <- 0
            
            dfun <- d
            pfun <- p
            qfun <- q
            
            if(is.null(d)) {
              .withSim <- TRUE
              dpq <- RtoDPQ(r)
              dpq.approx <- 1
              dfun <- dpq$dfun}
            
            if(is.null(p)) {
              .withSim <- TRUE
              if(dpq.approx == 0) {dpq <- RtoDPQ(r)}
              dpq.approx <- 1
              pfun <- dpq$pfun}
            
            if(is.null(q)) {
               ## quantile function
               rN <- NULL
               if(is.null(up1)) up1 <- max(rN <- r(10^getdistrOption("RtoDPQ.e")))
               if(is.null(low1)) {
                 low1 <- if(is.null(rN)) min(r(10^getdistrOption("RtoDPQ.e")))
                         else min(rN)}
                         
               h <- (up1-low1)/getdistrOption("DefaultNrFFTGridPointsExponent")
               x <-   seq(from = low1, to = up1, by = h)

               px.l <- pfun(x + 0.5*h)
               px.u <- pfun(x + 0.5*h, lower.tail = FALSE)
            
               qfun <- .makeQNew(x + 0.5*h, px.l, px.u, FALSE, low, up)
             }
            
            .Object@img <- img
            .Object@param <- param
            .Object@d <- dfun
            .Object@p <- pfun
            .Object@q <- qfun
            .Object@r <- r
            .Object@gaps <- gaps
            .Object@.withSim <- .withSim
            .Object@.withArith <- .withArith
            .Object@.logExact <- .logExact
            .Object@.lowerExact <- .lowerExact
            .Object@Symmetry <- Symmetry
            .Object })

## class AffLinAbscontDistribution
setMethod("initialize", "AffLinAbscontDistribution",
          function(.Object, r = NULL, d = NULL, p = NULL, q = NULL, gaps = NULL,
                   a = 1, b = 0, X0 = Norm(), param = NULL, img = new("Reals"),
                   .withSim = FALSE, .withArith = FALSE,
                   .lowerExact = FALSE, .logExact = FALSE,
                   Symmetry = NoSymmetry()) {
  X <- new("AbscontDistribution", r = r, d = d, p = p, q = q, gaps = gaps, 
           param = param, img = img, .withSim = .withSim, 
           .withArith = .withArith)
  .Object@gaps  <- X@gaps 
  .Object@img <- X@img
  .Object@param <- X@param
  .Object@a <- a
  .Object@b <- b
  .Object@X0 <- X0
  .Object@d <- X@d
  .Object@p <- X@p
  .Object@q <- X@q
  .Object@r <- X@r
  .Object@.withSim <- .withSim
  .Object@.withArith <- .withArith
  .Object@Symmetry <- Symmetry
  .Object
})

## Class: DiscreteDistribution
setMethod("initialize", "DiscreteDistribution",
          function(.Object, r = NULL, d = NULL, p = NULL, q = NULL, 
                    support = NULL, param = NULL, img = new("Reals"), 
                    .withSim = FALSE, .withArith = FALSE,
                   .lowerExact = FALSE, .logExact = FALSE,
                   .finSupport = c(TRUE,TRUE),
                   Symmetry = NoSymmetry()) {

            ## don't use this if the call is new("DiscreteDistribution")
            LL <- length(sys.calls())
            if(sys.calls()[[LL-3]] == "new(toDef)")
               {return(.Object)}
            if(sys.calls()[[LL-3]] == "new(\"DiscreteDistribution\")")
               {return(.Object)}
            
            if(is.null(r))
               warning("you have to specify slot r at least")
              
            if(is.null(support)) 
               .Object@support <- as.numeric(names(table(r(10^6))))
            
            else .Object@support <- support

#           len = length(support)
#
#            if(len > 1){
#              if(min(diff(support)) < getdistrOption("DistrResolution"))
#                 stop("grid too narrow --> change DistrResolution")
#            }

            dpq.approx <- 0

            dfun <- d
            pfun <- p
            qfun <- q

            if(is.null(d)) {
              .withSim <- TRUE
              dpq <- RtoDPQ.d(r)
              dpq.approx <- 1
              dfun <- dpq$dfun
            }

            if(is.null(p)) {
              .withSim <- TRUE
              if(dpq.approx==0) dpq <- RtoDPQ.d(r)
              dpq.approx <- 1
              pfun <- dpq$pfun
            }

            if(is.null(q)) {
              .withSim <- TRUE
              if(dpq.approx==0) dpq <- RtoDPQ.d(r)
              qfun <- dpq$qfun
            }

            .Object@img <- img
            .Object@param <- param
            .Object@d <- dfun
            .Object@p <- pfun
            .Object@q <- qfun
            .Object@r <- r
            .Object@.withSim <- .withSim
            .Object@.withArith <- .withArith
            .Object@.lowerExact <- .lowerExact
            .Object@.logExact <- .logExact
            .Object@Symmetry <- Symmetry
            .Object@.finSupport <- .finSupport
            .Object
          })

## Class: AffLinDiscreteDistribution
setMethod("initialize", "AffLinDiscreteDistribution",
          function(.Object, r = NULL, d = NULL, p = NULL, q = NULL, 
                   support = NULL, a = 1, b = 0, X0 = Binom(), param = NULL, 
                   img = new("Reals"), .withSim = FALSE, .withArith = FALSE,
                   .lowerExact = FALSE, .logExact = FALSE,
                   Symmetry = NoSymmetry(), .finSupport = c(TRUE,TRUE)) {
   ## don't use this if the call is new("DiscreteDistribution")
   LL <- length(sys.calls())
   if(sys.calls()[[LL-3]] == "new(\"AffLinDiscreteDistribution\")")
        X <- new("DiscreteDistribution")
   else X <- new("DiscreteDistribution", r = r, d = d, p = p, q = q, support = support, 
             param = param, img = img, .withSim = .withSim, 
            .withArith = .withArith, .finSupport = .finSupport)
  .Object@support  <- X@support 
  .Object@img <- X@img
  .Object@param <- X@param
  .Object@a <- a
  .Object@b <- b
  .Object@X0 <- X0
  .Object@d <- X@d
  .Object@p <- X@p
  .Object@q <- X@q
  .Object@r <- X@r
  .Object@.withSim <- .withSim
  .Object@.withArith <- .withArith
  .Object@.lowerExact <- .lowerExact
  .Object@.logExact <- .logExact
  .Object@Symmetry <- Symmetry
  .Object@.finSupport <- .finSupport
  .Object
})

## Class: LatticeDistribution
setMethod("initialize", "LatticeDistribution",
          function(.Object, r = NULL, d = NULL, p = NULL, q = NULL, 
                    support = NULL, lattice = NULL, param = NULL, 
                    img = new("Reals"), .withSim = FALSE, .withArith = FALSE,
                   .lowerExact = FALSE, .logExact = FALSE,
                   Symmetry = NoSymmetry(), .finSupport = c(TRUE,TRUE)) {


             LL <- length(sys.calls())
             if(sys.calls()[[LL-3]] == "new(\"LatticeDistribution\")")
             D <- new("DiscreteDistribution")
             else
             D <- new("DiscreteDistribution", r = r, d = d, p = p, 
                       q = q, support = support, param = param, img = img, 
                     .withSim = .withSim, .withArith = .withArith,
                     .finSupport = .finSupport)

            
             OS  <- D@support 

             #if(is.null(lattice))  
             #  {  if(! .is.vector.lattice(OS))
             #         stop("Support as given/generated is not a lattice.")
             #     .Object@lattice <- .make.lattice.es.vector(OS)
             #}else{
                  .Object@lattice <- if(is.null(lattice )) 
                          new("Lattice") else lattice
             #}


            .Object@support <- OS
            .Object@img <- D@img
            .Object@param <- D@param
            .Object@d <- D@d
            .Object@p <- D@p
            .Object@q <- D@q
            .Object@r <- D@r
            .Object@.withSim <- .withSim
            .Object@.withArith <- .withArith
            .Object@.lowerExact <- .lowerExact
            .Object@.logExact <- .logExact
            .Object@Symmetry <- Symmetry
            .Object@.finSupport <- .finSupport
            .Object
          })

## Class: AffLinLatticeDistribution
setMethod("initialize", "AffLinLatticeDistribution",
          function(.Object, r = NULL, d = NULL, p = NULL, q = NULL, 
                   support = NULL, lattice = NULL, a = 1, b = 0, X0 = Binom(), 
                   param = NULL, img = new("Reals"), .withSim = FALSE, 
                   .withArith = FALSE, .lowerExact = FALSE, .logExact = FALSE,
                   Symmetry = NoSymmetry(), .finSupport = c(TRUE, TRUE)) {

   LL <- length(sys.calls())
   if(sys.calls()[[LL-3]] == "new(\"AffLinLatticeDistribution\")")
        X <- new("LatticeDistribution")
   else X <- new("LatticeDistribution", r = r, d = d, p = p, q = q, 
                  support = support, lattice = lattice, param = param, 
                  img = img, .withSim = .withSim, 
                 .withArith = .withArith, .finSupport = .finSupport)

  .Object@support  <- X@support 
  .Object@lattice <-  X@lattice 
  .Object@img <- X@img
  .Object@param <- X@param
  .Object@a <- a
  .Object@b <- b
  .Object@X0 <- X0
  .Object@d <- X@d
  .Object@p <- X@p
  .Object@q <- X@q
  .Object@r <- X@r
  .Object@.withSim <- .withSim
  .Object@.withArith <- .withArith
  .Object@.lowerExact <- .lowerExact
  .Object@.logExact <- .logExact
  .Object@Symmetry <- Symmetry
  .Object@.finSupport <- .finSupport
  .Object
})

######### particular discrete distributions

### Class: Dirac distribution
setMethod("initialize", "Dirac",
          function(.Object, location = 0, .withArith = FALSE) {
            .Object@img <- new("Reals")
            .Object@param <- new("DiracParameter", location = location)
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){} 
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){}
            body(.Object@r) <- substitute({ rep(locationSub, n)},
                                            list(locationSub = location)
                                          )
            body(.Object@d) <- substitute(
                           { y <- rep(locationSub, length(x))
                             d0 <- mapply(function(x,y) 
                                          as.numeric(isTRUE(all.equal(x,y))),
                                          x = x, y = y)
                             if (log) d0 <- log(d0)
                             return(d0)
                           }, list(locationSub = location)
                                          )
            body(.Object@p) <- substitute(
                           {p0 <-as.numeric(q + .Machine$double.eps^.5 >= 
                                       locationSub)
                            if (!lower.tail) p0 <- 1-p0
                            if (log.p) p0 <- log(p0)
                            return(p0)
                            },
                            list(locationSub = location)
                                          )
            body(.Object@q) <- substitute( 
                { if (log.p) p <- exp(p)
                  if(any((p < 0)|(p > 1))) 
                     warning("q Method of class Dirac produced NaNs.")
                  q0 <- ifelse((p < 0)|(p > 1), NaN, locationSub) 
                  return(q0)
                },
                           list(locationSub = location)
                                          )
            .Object@support <- location
            .Object@lattice <- new("Lattice", pivot = location, width = 1, 
                                    Length = 1)
            .Object@.withArith <- .withArith
            .Object@.finSupport <- c(TRUE,TRUE)&(location> -Inf & location < Inf)
            .Object
          })

## Class: binomial distribution
setMethod("initialize", "Binom",
          function(.Object, size = 1, prob = 0.5, .withArith = FALSE) {
            .Object@img <- new("Naturals")
            .Object@param <- new("BinomParameter", size = size, prob = prob)
            .Object@support <- 0:size
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute(
                           { rbinom(n, size = sizeSub, prob = probSub) },
                             list(sizeSub = size, probSub = prob)
                                         )
            body(.Object@d) <- substitute(
                           { dbinom(x, size = sizeSub, prob = probSub, 
                                    log = log) },
                             list(sizeSub = size, probSub = prob)
                                         )
            body(.Object@p) <- substitute(
                           { pbinom(q, size = sizeSub, prob = probSub, 
                                    lower.tail = lower.tail, log.p = log.p) },
                             list(sizeSub = size, probSub = prob)
                                         )
            body(.Object@q) <- substitute(
                           { qbinom(p, size = sizeSub, prob = probSub, 
                                    lower.tail = lower.tail, log.p = log.p) },
                             list(sizeSub = size, probSub = prob)
                                         )
            .Object@support = 0:size
            .Object@lattice = new("Lattice", pivot = 0, width = 1,
                                   Length = size+1)
            .Object@.withArith <- .withArith
            .Object@.finSupport <- c(TRUE,TRUE)
            .Object
          })

## Class: hypergeometric distribution
setMethod("initialize", "Hyper",
          function(.Object, m = 1, n = 1, k = 1, .withArith = FALSE) {
            .Object@img <- new("Naturals")
            .Object@param <- new("HyperParameter", m = m, n = n, k = k)
            .Object@support <- 0:k
            .Object@r <- function(nn){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute(
                               { rhyper(nn, m = mSub, n = nSub, k = kSub) },
                                 list(mSub = m, nSub = n, kSub = k)
                                         )
            body(.Object@d) <- substitute(
                               { dhyper(x, m = mSub, n = nSub, k = kSub, 
                                        log = log) },
                                 list(mSub = m, nSub = n, kSub = k)
                                          )
            body(.Object@p) <- substitute(
                               { phyper(q, m = mSub, n = nSub, k = kSub, 
                                        lower.tail = lower.tail, log.p = log.p) 
                                        },
                                 list(mSub = m, nSub = n, kSub = k)
                                          )
            body(.Object@q) <- substitute(
                               { qhyper(p, m = mSub, n = nSub, k = kSub, 
                                        lower.tail = lower.tail, log.p = log.p) 
                                        },
                                 list(mSub = m, nSub = n, kSub = k)
                                          )
            .Object@support <-  seq(from = 0, to = min(k,m), by = 1)
            .Object@lattice <-  new("Lattice", pivot = 0, width = 1,
                                     Length = min(k,m)+1 )
            .Object@.withArith <- .withArith
            .Object@.finSupport <- c(TRUE,TRUE)
            .Object
          })

## Class: Poisson distribution 
setMethod("initialize", "Pois",
          function(.Object, lambda = 1, .withArith=FALSE) {
            .Object@img <- new("Naturals")
            .Object@param <- new("PoisParameter", lambda = lambda)
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute({ rpois(n, lambda = lambdaSub) },
                                            list(lambdaSub = lambda)
                                          )
            body(.Object@d) <- substitute({ dpois(x, lambda = lambdaSub, 
                                                  log = log) },
                                            list(lambdaSub = lambda)
                                          )
            body(.Object@p) <- substitute({ ppois(q, lambda = lambdaSub, 
                                                  lower.tail = lower.tail, 
                                                  log.p = log.p) },
                                            list(lambdaSub = lambda)
                                          )
            body(.Object@q) <- substitute({ qpois(p, lambda = lambdaSub, 
                                                  lower.tail = lower.tail, 
                                                  log.p = log.p) },
                                            list(lambdaSub = lambda)
                                          )
            .Object@support <- seq(from = 0, by = 1, to = 
                                   qpois(getdistrOption("TruncQuantile"),
                                         lambda = lambda, lower.tail = FALSE) 
                                         + 2
                                   )
            .Object@lattice <- new("Lattice", pivot = 0, width = 1, 
                                    Length = Inf)
            .Object@.withArith <- .withArith
            .Object@.finSupport <- c(TRUE,FALSE)
            .Object
          })

## Class: negative binomial distribution
setMethod("initialize", "Nbinom",
          function(.Object, size = 1, prob = 0.5, .withArith = FALSE) {
            .Object@img <- new("Naturals")
            .Object@param <- new("NbinomParameter", size = size, prob = prob)
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute(
                           { rnbinom(n, size = sizeSub, prob = probSub) },
                             list(sizeSub = size, probSub = prob)
                                         )
            body(.Object@d) <- substitute(
                           { dnbinom(x, size = sizeSub, prob = probSub, 
                                     log = log) },
                             list(sizeSub = size, probSub = prob)
                                         )
            body(.Object@p) <- substitute(
                           { pnbinom(q, size = sizeSub, prob = probSub, 
                                     lower.tail = lower.tail, log.p = log.p) },
                             list(sizeSub = size, probSub = prob)
                                         )
            body(.Object@q) <- substitute(
                           { qnbinom(p, size = sizeSub, prob = probSub, 
                                     lower.tail = lower.tail, log.p = log.p) },
                             list(sizeSub = size, probSub = prob)
                                         )
            .Object@.withArith <- .withArith
            .Object@support <-  seq(from = 0, by = 1, 
                                    to = qnbinom( size = size, prob = prob,
                                         getdistrOption("TruncQuantile"),
                                         lower.tail = FALSE)
                                    )
            .Object@lattice <-  new("Lattice", pivot = 0, width = 1, 
                                     Length = Inf)
            .Object@.finSupport <- c(TRUE,FALSE)
            .Object
          })

## Class: geometric distribution
setMethod("initialize", "Geom",
          function(.Object, prob = 0.5, .withArith = FALSE) {
            .Object@img <- new("Naturals")
            .Object@param <- new("NbinomParameter", name = 
                             gettext("Parameter of a Geometric distribution"),
                             prob = prob)
            .Object@support <- 0:qgeom(getdistrOption("TruncQuantile"), 
                                       prob = prob, lower.tail = FALSE)
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute({ rgeom(n, prob = probSub) },
                                          list(probSub = prob))
            body(.Object@d) <- substitute({ dgeom(x, prob = probSub, 
                                                  log = log) },
                                          list(probSub = prob))
            body(.Object@p) <- substitute({ pgeom(q, prob = probSub, 
                                                  lower.tail = lower.tail, 
                                                  log.p = log.p) },
                                          list(probSub = prob))
            body(.Object@q) <- substitute({ qgeom(p, prob = probSub, 
                                                  lower.tail = lower.tail, 
                                                  log.p = log.p) },
                                          list(probSub = prob))
            .Object@.withArith <- .withArith
            .Object@.finSupport <- c(TRUE,FALSE)
            .Object
          })


## --- particular absolutely continuous distributions


## Class: uniform distribution
setMethod("initialize", "Unif",
          function(.Object, Min = 0, Max = 1, .withArith = FALSE) {
            .Object@img <- new("Reals")
            .Object@param <- new("UnifParameter", Min = Min, Max = Max)
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute(
                           { runif(n, min = MinSub, max = MaxSub) },
                             list(MinSub = Min, MaxSub = Max)
                                          )
            body(.Object@d) <- substitute(
                           { dunif(x, min = MinSub, max = MaxSub, log = log) },
                             list(MinSub = Min, MaxSub = Max)
                                          )            
            body(.Object@p) <- substitute(
                           { punif(q, min = MinSub, max = MaxSub, 
                                   lower.tail = lower.tail, log.p = log.p) },
                             list(MinSub = Min, MaxSub = Max)
                                          )        
            body(.Object@q) <- substitute(
                           { qunif(p, min = MinSub, max = MaxSub, 
                                   lower.tail = lower.tail, log.p = log.p) },
                             list(MinSub = Min, MaxSub = Max)
                                          )                    
            .Object@.withArith <- .withArith
            .Object@Symmetry <- SphericalSymmetry(Min+Max/2)
            .Object
          })


## Class: normal distribution
setMethod("initialize", "Norm",
          function(.Object, mean = 0, sd = 1, .withArith = FALSE) {
            .Object@img <- new("Reals")
            .Object@param <- new("UniNormParameter", mean = mean, sd = sd)
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute(
                           { rnorm(n, mean = meanSub, sd = sdSub) },
                             list(meanSub = mean, sdSub = sd)
                                          )
            body(.Object@d) <- substitute(
                           { dnorm(x, mean = meanSub, sd = sdSub, log = log) },
                             list(meanSub = mean, sdSub = sd)
                                          )
            body(.Object@p) <- substitute(
                           { pnorm(q, mean = meanSub, sd = sdSub, 
                                   lower.tail = lower.tail, log.p = log.p) },
                             list(meanSub = mean, sdSub = sd)
                                          )
            body(.Object@q) <- substitute(
                           { qnorm(p, mean = meanSub, sd = sdSub, 
                                   lower.tail = lower.tail, log.p = log.p) },
                             list(meanSub = mean, sdSub = sd)
                                          )
            .Object@.withArith <- .withArith
            .Object@Symmetry <- SphericalSymmetry(mean)
            .Object
          })

## Class: lognormal distribution
setMethod("initialize", "Lnorm",
          function(.Object, meanlog = 0, sdlog = 1, .withArith = FALSE) {
            .Object@img <- new("Reals")
            .Object@param <- new("LnormParameter", meanlog = meanlog, 
                                  sdlog = sdlog)
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute(
                           { rlnorm(n, meanlog = meanlogSub, 
                                    sdlog = sdlogSub) },
                             list(meanlogSub = meanlog, sdlogSub = sdlog)
                                          )
            body(.Object@d) <- substitute(
                           { dlnorm(x, meanlog = meanlogSub, 
                                    sdlog = sdlogSub, log = log) },
                             list(meanlogSub = meanlog, sdlogSub = sdlog)
                                          )
            body(.Object@p) <- substitute(
                           { plnorm(q, meanlog = meanlogSub, sdlog = sdlogSub, 
                                    lower.tail = lower.tail, log.p = log.p) },
                             list(meanlogSub = meanlog, sdlogSub = sdlog)
                                          )
            body(.Object@q) <- substitute(
                           { qlnorm(p, meanlog = meanlogSub, sdlog = sdlogSub, 
                                    lower.tail = lower.tail, log.p = log.p) },
                             list(meanlogSub = meanlog, sdlogSub = sdlog)
                                          )
            .Object@.withArith <- .withArith
            .Object
          })

## Class: CauchyDistribution
setMethod("initialize", "Cauchy",
          function(.Object, location = 0, scale = 1) {
            .Object@img <- new("Reals")
            .Object@param <- new("CauchyParameter", location = location, 
                                  scale = scale)
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute(
                           { rcauchy(n, location = locationSub, 
                                     scale = scaleSub) },
                             list(locationSub = location, 
                                               scaleSub = scale)
                                          )
            body(.Object@d) <- substitute(
                           { dcauchy(x, location = locationSub, 
                                     scale = scaleSub, log = log) }, 
                             list(locationSub = location, scaleSub = scale)
                                          )
            body(.Object@p) <- substitute(
                           { pcauchy(q, location = locationSub, 
                                     scale = scaleSub, lower.tail = lower.tail, 
                                     log.p = log.p) },
                             list(locationSub = location, scaleSub = scale)
                                          )
            body(.Object@q) <- substitute(
                           { qcauchy(p, location = locationSub, 
                                     scale = scaleSub, lower.tail = lower.tail, 
                                     log.p = log.p) },
                             list(locationSub = location, scaleSub = scale)
                                          )
            .Object@.withSim   <- FALSE
            .Object@.withArith <- FALSE
            .Object@Symmetry <- SphericalSymmetry(location)
            .Object
          })

## Class: F distribution
setMethod("initialize", "Fd",
          function(.Object, df1 = 1, df2 = 1, ncp = 0) {
            .Object@img <- new("Reals")
            .Object@param <- new("FParameter", df1 = df1, df2 = df2, ncp = ncp)
            .Object@r <- function(n){}            
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            
            #### will probably change.... (when df for ncp!=0 available...)
            if((isTRUE(all.equal(ncp,0)))||getRversion()>'2.4.0')
               {df.0<- function(x, df1 = df1, df2 = df2, ncp = ncp, log = FALSE)
                       {stats::df(x = x, df1 = df1 , df2 = df2, ncp = ncp, 
                                  log = log)} 
               }
            else  
               {## for R < 2.4.0  df with ncp != 0:
                ### later perhaps with sfsmisc:
                  TQ <- getdistrOption("TruncQuantile")/2
                  xz <- qf(TQ, df1 = df1, df2 = df2, ncp = ncp, 
                           lower.tail = FALSE)
                  pfun <- function(x){pf(x, df1 = df1, df2 = df2, ncp = ncp)}
                  dfun <- .P2D(p=pfun, ql = 0, qu = xz)
                # by means of simulations
                # rfun <- function(n){rf(n, df1=df1, df2=df2, ncp=ncp)}
                # dfun <-R2D(rfun, nsim = 10^getdistrOption("RtoDPQ.e"), 
                #           n = getdistrOption("DefaultNrGridPoints"))
                df.0 <- function(x, df1 = df1, df2 = df2, ncp = ncp, 
                                 log = FALSE) {dfun(x)}
               }                  
            
            body(.Object@r) <- substitute(                    
                           { rf(n, df1 = df1Sub, df2 = df2Sub, ncp = ncpSub) },
                             list(df1Sub = df1, df2Sub = df2, ncpSub = ncp)                                         
                                          )
            body(.Object@d) <- substitute(
                           { df.0(x, df1 = df1Sub, df2 = df2Sub, ncp = ncpSub, 
                                  log = log)},
                             list(df1Sub = df1, df2Sub = df2, ncpSub = ncp)
                                           )
            body(.Object@p) <- substitute(
                           { pf(q, df1 = df1Sub, df2 = df2Sub, ncp = ncpSub, 
                                lower.tail = lower.tail, log.p = log.p) },
                             list(df1Sub = df1, df2Sub = df2, ncpSub = ncp)
                                          )
            body(.Object@q) <- substitute(
                           { qf(p, df1 = df1Sub, df2 = df2Sub, ncp = ncpSub, 
                                lower.tail = lower.tail, log.p = log.p) },
                             list(df1Sub = df1, df2Sub = df2, ncpSub = ncp)
                                          )
            .Object@.withArith <- FALSE
            .Object
          })

## Class: Student distribution
setMethod("initialize", "Td",
          function(.Object, df = 1, ncp = 0) {
            .Object@img <- new("Reals")
            .Object@param <- new("TParameter", df = df, ncp = ncp) 
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute({ rt(n, df = dfSub, ncp = ncpSub) }, 
                                          list(dfSub = df, ncpSub = ncp)
                                          )
            body(.Object@d) <- substitute(
                                       { dt(x, df = dfSub, ncp = ncpSub, 
                                            log = log) },
                                         list(dfSub = df, ncpSub = ncp)
                                          )
            body(.Object@p) <- substitute(
                                       { pt(q, df = dfSub, ncp = ncpSub, 
                                            lower.tail = lower.tail, 
                                            log.p = log.p) },
                                         list(dfSub = df, ncpSub = ncp)
                                          )
            body(.Object@q) <- substitute(
                                       { qt(p, df = dfSub, ncp = ncpSub, 
                                            lower.tail = lower.tail, 
                                            log.p = log.p) },
                                         list(dfSub = df, ncpSub = ncp)
                                          )
            .Object@.withArith <- FALSE
            .Object@Symmetry <- SphericalSymmetry(0)
            .Object
          })

## Class: Chi squared distribution
setMethod("initialize", "Chisq",
          function(.Object, df = 1, ncp = 0, .withArith = FALSE) {
            .Object@img <- new("Reals")
            .Object@param <- new("ChisqParameter", df = df, ncp = ncp)
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute(
                           { rchisq(n, df = dfSub, ncp = ncpSub) },
                             list(dfSub = df, ncpSub = ncp)
                                          )
            body(.Object@d) <- substitute(
                           { dchisq(x, df = dfSub, ncp = ncpSub, log = log) },
                             list(dfSub = df, ncpSub = ncp)
                                          )
            body(.Object@p) <- substitute(
                           { pchisq(q, df = dfSub, ncp = ncpSub, 
                                    lower.tail = lower.tail, log.p = log.p) },
                             list(dfSub = df, ncpSub = ncp)
                                          )
            body(.Object@q) <- substitute(
                           { qchisq(p, df = dfSub, ncp = ncpSub, 
                                    lower.tail = lower.tail, log.p = log.p) },
                             list(dfSub = df, ncpSub = ncp)
                                          )
            .Object@.withSim   <- FALSE
            .Object@.withArith <- .withArith
            .Object
          })

## Class: exponential distribution
setMethod("initialize", "Exp",
          function(.Object, rate = 1, .withArith = FALSE) {
            .Object@img <- new("Reals")
            .Object@param <- new("ExpParameter", rate = rate)
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute(
                           { rexp(n, rate = rateSub) },
                             list(rateSub = rate)
                                          )
            body(.Object@d) <- substitute(
                           { dexp(x, rate = rateSub, log = log) },
                             list(rateSub = rate)
                                          )
            body(.Object@p) <- substitute(
                           { pexp(q, rate = rateSub, 
                                  lower.tail = lower.tail, log.p = log.p) },
                             list(rateSub = rate)
                                          )
            body(.Object@q) <- substitute(
                           { qexp(p, rate = rateSub, 
                                  lower.tail = lower.tail, log.p = log.p) },
                             list(rateSub = rate)
                                          )
            .Object@.withSim   <- FALSE
            .Object@.withArith <- .withArith
            .Object
          })

## Class: Laplace or Double Exponential distribution
setMethod("initialize", "DExp",
          function(.Object, rate = 1, .withArith = FALSE) {
            .Object@img <- new("Reals")
            .Object@param <- new("ExpParameter", rate = rate)
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute(
                           { (2 * rbinom(n, size = 1, prob = 0.5) -1 ) * 
                              rexp(n, rate = rateSub) 
                           }, list(rateSub = rate)
                                          )
            body(.Object@d) <- substitute( 
                            { d0 <-  dexp(abs(x), rate = rateSub, log = log) 
                              d0 <- if (log) d0-log(2) else d0 <- d0 / 2
                              return(d0) },
                              list(rateSub = rate)
                                          )
            body(.Object@p) <- substitute(
                           { if (!lower.tail) q <- -q
                             p0 <- ifelse( q <= 0, 
                                           0.5 * pexp(-q, rate = rateSub,
                                                    lower.tail = FALSE),
                                           0.5 + 0.5*pexp( q, rate = rateSub)
                                           )
                             if (log.p)  p0 <- log(p0)       
                             return(p0)
                           }, list(rateSub = rate)
                                         )
            body(.Object@q) <- substitute(
                           {  if (log.p) p <- exp(p)
                              if (!lower.tail) p <- 1-p
                              q0 <- p
                              q0[p <=0.25] <- -qexp(2*p[p <=0.25], rate = rateSub, lower.tail =FALSE)
                              q0[p>0.25&p<=.50] <- -qexp(1-2*p[p>0.25&p<=.50], rate = rateSub)
                              q0[p>0.5&p<=.75] <- qexp(2*p[p>0.5&p<=.75] - 1, rate = rateSub)
                              q0[p>0.75] <- qexp(2*(1-p[p>0.75]), rate = rateSub, lower.tail = FALSE)
                              return(q0)
                           }, list(rateSub = rate)
                                          )
            .Object@.withSim   <- FALSE
            .Object@.withArith <- .withArith
            .Object@Symmetry <- SphericalSymmetry(0)
            .Object
          })

## Class: gamma distribution
setMethod("initialize", "Gammad",
          function(.Object, shape = 1, scale = 1, .withArith = FALSE) {
            .Object@img <- new("Reals")
            .Object@param <- new("GammaParameter", shape = shape, scale = scale)
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute(
                           { rgamma(n, shape = shapeSub, scale = scaleSub) },
                             list(shapeSub = shape, scaleSub = scale)
                                          )
            body(.Object@d) <- substitute(
                           { dgamma(x, shape = shapeSub, scale = scaleSub, 
                                    log = log) },
                             list(shapeSub = shape, scaleSub = scale)
                                          )
            body(.Object@p) <- substitute(
                           { pgamma(q, shape = shapeSub, scale = scaleSub, 
                                    lower.tail = lower.tail, log.p = log.p) },
                             list(shapeSub = shape, scaleSub = scale)
                                          )
            body(.Object@q) <- substitute(
                           { qgamma(p, shape = shapeSub, scale = scaleSub, 
                                    lower.tail = lower.tail, log.p = log.p) },
                             list(shapeSub = shape, scaleSub = scale)
                                          )
            .Object@.withSim   <- FALSE
            .Object@.withArith <- .withArith
            .Object
          })


## Class: BetaDistribution
setMethod("initialize", "Beta",
          function(.Object, shape1 = 1, shape2 = 1, ncp = 0) {
            .Object@img <- new("Reals")
            .Object@param <- new("BetaParameter", shape1 = shape1, 
                                  shape2 = shape2, ncp = ncp)
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute(
                           { rbeta(n, shape1 = shape1Sub, shape2 = shape2Sub, 
                                   ncp = ncpSub) },
                             list(shape1Sub = shape1, shape2Sub = shape2, 
                                  ncpSub = ncp)
                                          )
            body(.Object@d) <- substitute(
                           { dbeta(x, shape1 = shape1Sub, shape2 = shape2Sub, 
                                   ncp = ncpSub, log = log) },
                             list(shape1Sub = shape1, shape2Sub = shape2, 
                                  ncpSub = ncp)
                                          )
            body(.Object@p) <- substitute(
                           { pbeta(q, shape1 = shape1Sub, shape2 = shape2Sub, 
                                   ncp = ncpSub, lower.tail = lower.tail, 
                                   log.p = log.p) },
                             list(shape1Sub = shape1, shape2Sub = shape2, 
                                  ncpSub = ncp)
                                          )
            body(.Object@q) <- substitute(
                           { qbeta(p, shape1 = shape1Sub, shape2 = shape2Sub, 
                                   ncp = ncpSub, lower.tail = lower.tail, 
                                   log.p = log.p) },
                             list(shape1Sub = shape1, shape2Sub = shape2, 
                                  ncpSub = ncp)
                                          )
            .Object@.withSim   <- FALSE
            .Object@.withArith <- FALSE
            .Object
          })

## Class: logistic distribution
setMethod("initialize", "Logis",
          function(.Object, location = 0, scale = 1) {
            .Object@img <- new("Reals")
            .Object@param <- new("LogisParameter", location = location, 
                                  scale = scale)
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute(
                           { rlogis(n, location = locationSub, 
                                    scale = scaleSub) },
                             list(locationSub = location, scaleSub = scale)
                                         )
            body(.Object@d) <- substitute(
                           { dlogis(x, location = locationSub, scale = scaleSub, 
                                    log = log) },
                             list(locationSub = location, scaleSub = scale)
                                         )
            body(.Object@p) <- substitute(
                           { plogis(q, location = locationSub, scale = scaleSub, 
                                    lower.tail = lower.tail, log.p = log.p) },
                             list(locationSub = location, scaleSub = scale)
                                         )
            body(.Object@q) <- substitute(
                           { qlogis(p, location = locationSub, scale = scaleSub, 
                                    lower.tail = lower.tail, log.p = log.p) },
                             list(locationSub = location, scaleSub = scale)
                                         )
            .Object@.withArith <- FALSE
            .Object
          })

## Class: Weibull distribution
setMethod("initialize", "Weibull",
          function(.Object, shape = 1, scale = 1, .withArith = FALSE) {
            .Object@img <- new("Reals")
            .Object@param <- new("WeibullParameter", 
                                  shape = shape, scale = scale
                                  )
            .Object@r <- function(n){}
            .Object@d <- function(x, log = FALSE){}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){} 
            body(.Object@r) <- substitute(
                           { rweibull(n, shape = shapeSub, scale = scaleSub) },
                             list(shapeSub = shape, scaleSub = scale)
                                         )
            body(.Object@d) <- substitute(
                           { dweibull(x, shape = shapeSub, scale = scaleSub, 
                                      log = log) },
                             list(shapeSub = shape, scaleSub = scale)
                                         )
            body(.Object@p) <- substitute(
                           { pweibull(q, shape = shapeSub, scale = scaleSub, 
                                      lower.tail = lower.tail, log.p = log.p) },
                             list(shapeSub = shape, scaleSub = scale)
                                         )
            body(.Object@q) <- substitute(
                           { qweibull(p, shape = shapeSub, scale = scaleSub, 
                                      lower.tail = lower.tail, log.p = log.p) },
                             list(shapeSub = shape, scaleSub = scale)
                                         )
            .Object@.withArith <- .withArith
            .Object
          })

## Class: Arcsine distribution
setMethod("initialize", "Arcsine",
          function(.Object, .withArith = FALSE) {
            .Object@img <- new("Reals")
            .Object@r <- function(n){sin((runif(n)-.5)*pi)}
            .Object@d <- function(x, log = FALSE){ 
                              x0 <- (abs(x)<1-.Machine$double.eps)
                              x1 <- x^2*x0
                              d <-  x0/sqrt(1-x1)/pi
                              d[.isEqual(abs(x),1)] <- Inf
                              if(log) d<- log(d)
                              return(d)}
            .Object@p <- function(q, lower.tail = TRUE, log.p = FALSE){ 
                              if(!lower.tail) q<- -q
                              q <- pmin(pmax(q,-1),1)
                              p <- asin(q)/pi+1/2
                              if(log.p) p <- log(p)
                              return(p)} 
            .Object@q <- function(p, lower.tail = TRUE, log.p = FALSE){ 
                              if(log.p) p <- exp(p)
                              p1 <- p
                              p1[p<0|p>1] <- 0.5
                              if(!lower.tail) p1 <- 1-p1
                              q <- sin( (p1-1/2)*pi)
                              q[p<0|p>1] <- NA
                              q[.isEqual(p,0)] <- -1
                              q[.isEqual(p,1)] <-  1
                              return(q)}                      
            .Object@.withSim   <- FALSE
            .Object@.withArith <- .withArith
            .Object@Symmetry <- SphericalSymmetry(0)
            .Object
          })

setMethod("show", "Symmetry", 
    function(object){ 
        cat(gettextf("type of symmetry:\t%s\n", object@type))
        if(!is.null(object@SymmCenter))
            cat(gettext("center of symmetry:\n"))
            print(object@SymmCenter, quote = FALSE)
    })

#------  UnivariateDistribution ---------- #


setMethod("show", "UnivariateDistribution",
          function(object){
            cls <- class(object)[1]
            cat(showobj(object, className = cls))
            ws <- .IssueWarn(object@.withArith, object@.withSim)
            if(!is.null(ws$msgA)) warning(ws$msgA)
            if(!is.null(ws$msgS)) warning(ws$msgS)
            }
          )

setMethod("show", "LatticeDistribution",
          function(object){
            cls <- class(object)[1]
            cat(showobj(object, className = cls))
            ws <- .IssueWarn(object@.withArith, object@.withSim)
            if(!is.null(ws$msgA)) warning(ws$msgA)
            if(!is.null(ws$msgS)) warning(ws$msgS)
          })


setMethod("showobj", "UnivariateDistribution",
          function(object, className = class(object)[1]){
#            x <- object
            txt <- gettextf("Distribution Object of Class: %s\n", className)
            parameter = param(object)
            Names = slotNames(parameter)
            if(length(Names) > 1){
              for(i in Names[Names != "name"])
                txt <- c(txt,
                gettextf("%s: %s\n", i, slot(parameter, i)))                
            }
            #cat(txt)
            return(txt)
          })

setMethod("print", "UnivariateDistribution",
          function(x, ...)show(x))

#------ DistributionList  ---------- #

setMethod("show", "DistrList", 
    function(object){
       cls <- class(object)[1]
       cat(showobj(object, className = cls))
       ws <- .IssueWarn(any(unlist(lapply(object,function(x)x@.withArith))), 
                        any(unlist(lapply(object,function(x)x@.withSim))))
       if(!is.null(ws$msgA)) warning(ws$msgA)
       if(!is.null(ws$msgS)) warning(ws$msgS)
    
      }
    )

setMethod("showobj", "DistrList", 
    function(object,className = class(object)[1]){
        txt <- gettextf("An object of class \"%s\"\n", className)
        for(i in 1:length(object)){
            s <- showobj(object[[i]])
            les <- length(s)
            if(les >1)
                 st <- c(gettextf("[[%i]] ",i), rep("      :",les-1))
            else st <- gettextf("[[%i]] ",i)
            sts <- paste(st, gettextf("%s", s), sep = "")
            txt <- c(txt, sts)
        }
        return(txt)
    })

#------ UnivarMixingDistribution ---------- #


setMethod("show", "UnivarMixingDistribution",
          function(object){
            cls <- class(object)[1]
            cat(showobj(object, className = cls))
            ws <- .IssueWarn(object@.withArith, object@.withSim)
            if(!is.null(ws$msgA)) warning(ws$msgA)
            if(!is.null(ws$msgS)) warning(ws$msgS)
          }
         )


setMethod("showobj", "UnivarMixingDistribution",
          function(object, className = class(object)[1]){
              txt <- gettextf("An object of class \"%s\"\n", className)
              l <- length(mixCoeff(object))
              txt <- c(txt,
                     "---------------------------------------------\n")
              txt <- c(txt,
                  gettextf("It consists of  %i components \n", l))
              txt <- c(txt,
                     gettextf("Components: \n"))
 
              for(i in 1:l){
                 s <- showobj(mixDistr(object)[[i]])
                 les <- length(s)
                 st <- if (les>1) 
                     c(gettextf("[[%i]]", i), rep("      :",les-1))
                 else  gettextf("[[%i]]", i)
                 txt <- c(txt,
                          paste(st, gettextf("%s",s),sep=""))}
              txt <- c(txt,
                     "---------------------------------------------\n")
              txt <- c(txt, 
                          gettextf("Weights: \n"))
              txt <- c(txt, gettextf("%f",
                          round(mixCoeff(object),3)))
            txt<-c(txt,"\n ---------------------------------------------\n")
            return(txt)
            }
          )

setMethod("showobj", "CompoundDistribution",
          function(object, className = class(object)[1]){
              txt <- gettextf("An object of class \"%s\"\n\n", className)
              txt <- c(txt,
                     gettextf("The frequency distribution is:\n%s",
                              paste(showobj(NumbOfSummandsDistr(object)), collapse="")))
              txt <- c(txt,
                     gettextf("The summands distribution is/are:\n%s",
                              paste(showobj(SummandsDistr(object)), collapse="")))
              txt <- c(txt,
                     gettextf("\nThis Distribution is:\n%s",
                              paste(showobj(simplifyD(object)), collapse="")))                                            
            return(txt)
            }
          )


#------ UnivarLebDecDistribution ---------- #

setMethod("show", "UnivarLebDecDistribution",
          function(object){
           sc <- sys.call(sys.parent(1))
           if(identical(sc[[1]], as.name("show"))){
              objs <- as.character(deparse(match.call(call = sc)$object))
           }else{
              objs <- "<obj>"
           }
            cls <- class(object)[1]
            cat(showobj(object, className = cls, objs = objs))
           ws <- .IssueWarn(object@.withArith, object@.withSim)          
           if(!is.null(ws$msgA)) warning(ws$msgA)
           if(!is.null(ws$msgS)) warning(ws$msgS)
          })

setMethod("showobj", "UnivarLebDecDistribution",
          function(object, className = class(object)[1], objs){
              if(missing(objs)) objs <- "<obj>" 
              else if(length(grep("<S4 object ", objs))) objs <- "<obj>"
              txt <- gettextf("An object of class \"%s\"\n", className)
              txt <- c(txt,
                       gettextf("--- a Lebesgue decomposed distribution:\n\n"))
              l <- length(mixCoeff(object))
              txt <- c(txt,
                gettextf("   Its discrete part (with weight %f) is a\n", 
                  round(discreteWeight(object),3)))
              txt <- c(txt, showobj(discretePart(object)))
              txt <- c(txt,
              gettextf("This part is accessible with 'discretePart(%s)'.\n\n", 
                  objs))
              txt <- c(txt,
              gettextf("   Its absolutely continuous part (with weight %f) is a\n", 
                  round(acWeight(object),3)))
              txt <- c(txt, showobj(acPart(object)))
              txt <- c(txt,
              gettextf("This part is accessible with 'acPart(%s)'.\n", 
                  objs))
             return(txt)
          }          
          )

################################
##
## Class: GeomParameter
##                                
################################


## Access Methods

### Replaced by NbinomParameter ....
### pre v1.9 /deprecated  
### defunct as of 2.8.0
#setMethod("prob", "GeomParameter", function(object)
#     {.Defunct(new = "",
#                        package = "distr",
#                        msg = gettext(
#"Class 'GeomParameter' is no longer needed and will be replaced by \nclass 'NbinomParameter' soon."
#                                     )
#                 )
#      object@prob
#     }
#     )


## code is in NegbinomDistribution.R
# setMethod("prob", "NbinomParameter", function(object) object@prob)


## Replace Methods
### Replaced by NbinomParameter ....
### pre v1.9:  /deprecated
### defunct as of 2.8.0
#setReplaceMethod("prob", "GeomParameter",
#                             function(object, value)
#                                      {.Defunct(new = "",
#                                                   package = "distr",
#                                                   msg = gettext(
#"Class 'GeomParameter' is no longer needed and will be replaced by \nclass 'NbinomParameter' soon."
#                                                                )
#                                                  )
#                                       object@prob <- value;
#                                       object})

## code is in NegbinomDistribution.R
#setReplaceMethod("prob", "NbinomParameter",
#                  function(object, value)
#                          { object@prob <- value; object}
#                  )


### no longer needed from version 1.9 on
#setValidity("GeomParameter", function(object){
#  if(length(prob(object)) != 1)
#    stop("prob has to be a numeric of length 1")    
#  if(prob(object) <= 0)
#    stop("prob has to be in (0,1]")
#  if(prob(object) > 1)
#    stop("prob has to be in (0,1]")
#  else return(TRUE)
#}
#)

################################
##
## Class: geometric distribution 
##
################################

Geom <- function(prob = 0.5) new("Geom", prob = prob)

## wrapped access methods
setMethod("prob", "Geom", function(object) prob(param(object)))
## wrapped replace methods
setMethod("prob<-", "Geom", function(object, value) new("Geom", prob = value))

setMethod("size", "Geom", function(object) return(1))

setMethod("NumbOfSummandsDistr", signature="CompoundDistribution",
           function(object) object@NumbOfSummandsDistr)
setMethod("SummandsDistr", signature="CompoundDistribution",
           function(object) object@SummandsDistr)

CompoundDistribution<- function( NumbOfSummandsDistr, SummandsDistr, .withSim = FALSE,
                                 withSimplify = FALSE){

  Symmetry <- NoSymmetry()

  if(!is(NumbOfSummandsDistr,"DiscreteDistribution"))
    stop("Argument 'NumbOfSummandsDistr' must be of class 'DiscreteDistribution'")

  supp <- support(NumbOfSummandsDistr)
  if(!(all(.isInteger(supp))&&all(supp >=0)))
    stop("Support of 'NumbOfSummandsDistr' must be non neg. integers")

  if(!is(SummandsDistr,"UnivDistrListOrDistribution"))
    stop("Argument 'SummandsDistr' must be of class 'UnivDistrListOrDistribution'")

##20200918 can be deleted:  supp <- support(NumbOfSummandsDistr)

  supp <- as(supp,"integer")
  suppNot0 <- supp[supp!=0L]

  ## new 20200918 triggered by mail by Vlada Milchevskaya vmilchev@uni-koeln.de
  ## special treatment of case support is of length 1
  if(length(supp)==1L){
     if(supp[1]==0L) return(Dirac(0))
     return(convpow(SummandsDistr,supp[1]))
  }

  is0 <- 0 %in% supp
  lI <- vector("list", length(supp))
  if(is0) lI[[1]] <- Dirac(0)
  ##  bugfix :: bug detected by Wolfgang Kreitmeier <wkreitmeier@gmx.de> 29.07.2016
  if(length(suppNot0)){
     if(is(SummandsDistr,"UnivariateDistribution")){
#        dsuppNot0 <- c(suppNot0,diff(suppNot0))
#        S <- 0
        for (i in 1:length(suppNot0)){
#             x0 <- convpow(SummandsDistr,suppNot0[i])
             S <- convpow(SummandsDistr,suppNot0[i])
#             S <- S + x0
             lI[[i+is0]] <- S
        }
      Symmetry <- Symmetry(SummandsDistr)
     }else{
       supp <- min(supp):max(supp)
       if( (length(supp)!=length(SummandsDistr)) &&
           !(is0 && length(supp)==1+length(SummandsDistr)))
          stop("Lengths of support of 'NumbOfSummandDistr' and list in 'SummandDistr' do not match")
       if(is0 && length(supp)==length(SummandsDistr))
          SummandsDistr <- SummandsDistr[2:length(SummandsDistr)]
       S <- 0
       Symm1 <- Symmetry(SummandsDistr[[1]])
       SymmL <- is(Symm1, "SphericalSymmetry")
       SymmC <- if(SymmL) SymmCenter(Symm1) else NULL
       for(i in 1:(length(supp)-is0)){
           if(SymmL && i>1){
              SymmI <- Symmetry(SummandsDistr[[i]])
              SymmL <- is(SymmI, "SphericalSymmetry")
              if(SymmL)
                 SymmL <- .isEqual(SymmCenter(SymmI),SymmC)
           }
           S <- S + SummandsDistr[[i]]
           lI[[i+is0]] <- S
       }
       if(SymmL) Symmetry <- SphericalSymmetry(SymmC)
     }
  UV <- do.call("UnivarMixingDistribution",
                 args = c(list(mixCoeff = d(NumbOfSummandsDistr)(supp),
                               withSimplify = FALSE),
                               lI)
                 )
  obj <- new("CompoundDistribution",
              NumbOfSummandsDistr = NumbOfSummandsDistr,
              SummandsDistr = SummandsDistr,
              p = UV@p, r = UV@r, d = UV@d, q = UV@q,
              mixCoeff = UV@mixCoeff, mixDistr = UV@mixDistr,
              .withSim = .withSim, .withArith = TRUE,
              Symmetry = Symmetry)

   if(withSimplify) return(simplifyD(obj))
   else return(obj)

  }
}

setMethod("+", c("CompoundDistribution","numeric"),
          function(e1, e2) simplifyD(e1)+e2)
setMethod("*", c("CompoundDistribution","numeric"),
          function(e1, e2) simplifyD(e1)*e2)


setMethod("Math", "AcDcLcDistribution",
          function(x){
            callGeneric(simplifyD(.ULC.cast(x)))
          })

setAs("CompoundDistribution", "UnivarLebDecDistribution",
       function(from)simplifyD(from))







####################################

setMethod("sqrt", signature(x = "PosSemDefSymmMatrix"), function(x){
            er <- eigen(x)
            d <- sqrt(er$values)
            D <- if(length(d)==1) diag(1)*d else diag(d)
            return(PosSemDefSymmMatrix(er$vectors %*% D %*% t(er$vectors)))
})

################################
##
## Class: GammaParameter
##
################################


## Access Methods
setMethod("shape", "GammaParameter", function(object) object@shape)
setMethod("scale", "GammaParameter", 
           function(x, center = TRUE, scale = TRUE) x@scale)
           ### odd arg-list due to existing function in base package 

## Replace Methods
setReplaceMethod("shape", "GammaParameter", 
                  function(object, value){ object@shape <- value; object})
setReplaceMethod("scale", "GammaParameter", 
                  function(object, value){ object@scale <- value; object})


validGammaParameter <- function(object){
  if(length(shape(object)) != 1)
    stop("shape has to be a numeric of length 1")    
  if(shape(object) <= 0)
    stop("shape has to be positive")
  if(length(scale(object)) != 1)
    stop("scale has to be a numeric of length 1")    
  if(scale(object) <= 0)
    stop("scale has to be positive")
  else return(TRUE)
}

setValidity("GammaParameter", validGammaParameter)


################################
##
## Class: gamma distribution
##
################################

Gammad <- function(shape = 1, scale = 1) 
          new("Gammad", shape = shape, scale = scale)

## wrapped access methods
setMethod("shape", "Gammad", function(object) shape(param(object)))
setMethod("scale", "Gammad", 
           function(x, center = TRUE, scale = TRUE) scale(param(x)))
           ### odd arg-list due to existing function in base package 

## wrapped replace methods
setMethod("shape<-", "Gammad", 
           function(object, value) 
                    new("Gammad", shape = value, scale = scale(object)))
setMethod("scale<-", "Gammad", 
           function(object, value) 
                    new("Gammad", shape = shape(object), scale = value))

############################# Utils

.mergegaps <- function(gaps, support){
 if(is.null(gaps)) return(NULL)
 if(is.null(support)) return(gaps)
 if(length(support)==0) return(gaps)
 
 mm <- rbind(cbind(gaps[ ,1],1),
             cbind(gaps[ ,2],2),
             cbind(support,3))
 mm <- mm[order(mm[,1]),]
 jj <- 0
 ein <- FALSE
 gaps.new <- mm
 for(j in 1:nrow(mm))
  {if (mm[j,2] == 1 & !ein){
       jj <- jj + 1
       ein <- TRUE
       gaps.new[jj,1] <- mm[j,1]
       }
   if (mm[j,2] == 3 &  ein){
       gaps.new[jj,2] <- mm[j,1]
       jj <- jj + 1
       gaps.new[jj,1] <- mm[j,1]
       }
   if (mm[j,2] == 2)       {
       ein <- FALSE
       gaps.new[jj,2] <- mm[j,1]
       }
   }
 ga <- gaps.new[1:jj,,drop = FALSE]
 if(jj>0){
    ln <- seq(jj)
    ga <- matrix(ga[ln[ga[,1] != ga[,2]], ], ncol = 2)
    }
 else ga <- NULL
 return(ga)

}

.mergegaps2 <- function(gaps1, gaps2){
 if(is.null(gaps1)) return(NULL)
 if(is.null(gaps2)) return(NULL)
 mm <- rbind(cbind(gaps1[ ,1],1), 
             cbind(gaps1[ ,2],2),
             cbind(gaps2[ ,1],3),
             cbind(gaps2[ ,2],4))
 mm <- mm[order(mm[ ,1]), ]
 jj <- 0
 state1 <- 0
 state2 <- 0
 gaps.new <- mm
 for(j in 1:nrow(mm))
  {if (mm[j,2] == 1 && state2 == 0){
        state1 <- 1
       }
   if (mm[j,2] == 1 && state2 == 1){
        jj <- jj + 1
        gaps.new[jj,1] <- mm[j,1]
        state1 <- 1
       }
   if (mm[j,2] == 2 && state2 == 1){
        gaps.new[jj,2] <- mm[j,1]
        state1 <- 0
       }
   if (mm[j,2] == 2 && state2 == 0){
        state1 <- 0
       }
   if (mm[j,2] == 3 && state1 == 0){
        state2 <- 1
       }
   if (mm[j,2] == 3 && state1 == 1){
        jj <- jj + 1
        gaps.new[jj,1] <- mm[j,1]
        state2 <- 1
       }
   if (mm[j,2] == 4 && state1 == 1){
        gaps.new[jj,2] <- mm[j,1]
        state2 <- 0
       }
   if (mm[j,2] == 4 && state1 == 0){
        state2 <- 0
       }
   }
 erg <- if (jj > 0) gaps.new[1:jj, ,drop=FALSE] else NULL
 return(.consolidategaps(erg))

}

.consolidategaps <- function(gaps){
 if(is.null(gaps)) return(NULL)
 if(nrow(gaps)==0) return(NULL)
 if(nrow(gaps)==1) return(gaps)
 jj <- 1
 for(j in 1:(nrow(gaps)-1)){
   if (.isEqual(gaps[jj,2],gaps[j+1,1]))
      gaps[jj,2] <- gaps[j+1,2]
   else jj <- jj+1   
 }     
 return(gaps)
}

.pmixfun <- function(mixDistr, mixCoeff, leftright = "right"){
  l <- length(mixCoeff)
  return(function(q, lower.tail = TRUE, log.p = FALSE ){
  p0 <- as.vector(
           matrix(unlist(
              lapply(mixDistr, 
                  function(x){
                       p.lr <- if(pmatch(leftright, table=c("left","right"), 
                                         nomatch = 2)==2) p.l(x) 
                               else x@p
                       do.call(p.lr, list(q = q, lower.tail = lower.tail)) 
                  } 
              )),
              ncol = l, nrow = length(q)) %*% mixCoeff
           )
  if(log.p) p0 <- log(p0)
  return(p0)               
   })
}
.dmixfun <- function(mixDistr, mixCoeff, withStand = FALSE, supp = NULL){
  l <- length(mixCoeff)
  if(withStand) {su <- sum(as.vector(matrix(unlist(lapply(mixDistr, function(y)
                                               do.call(y@d, list(x = supp)))),
                                 ncol = l, nrow = length(supp)) %*% mixCoeff))
                }else su <- 1
 
  return(function(x, log = FALSE ){
         d0 <- 
         as.vector(matrix(unlist(lapply(mixDistr, function(y)
            do.call(y@d, list(x = x)))),
         ncol = l, nrow = length(x)) %*% mixCoeff)/su
         if(log) d0 <- log(d0)
         return(d0)
         })
}

.rmixfun <- function(mixDistr, mixCoeff){
  l <- length(mixCoeff)
  return(function(n){
  Un <- outer(sample(1:l, size = n, replace = TRUE, prob = mixCoeff),
                          1:l, function(x,y) x == y)
  mal <- lapply(mixDistr, function(x) x@r(n))
  ma <- matrix(unlist(mal), ncol = l, nrow = n)
  return(rowSums(Un*ma))})

}


.qmixfun <- function(mixDistr, mixCoeff, Cont = TRUE, pnew, gaps = NULL, 
                     leftright = "left"){
  l <- length(mixCoeff)
  if(l==0) return(NULL)
  loup <- .loupmixfun(mixDistr)

  n <- getdistrOption("DefaultNrGridPoints")
  up <- if(is.finite(loup$qu)) loup$qu else  1000 #min(loup$qu,1000)#getdistrOption("LARGE")
  lo <- if(is.finite(loup$ql)) loup$ql else -1000 #max(loup$ql,-1000)#getdistrOption("LARGE")

  h <- (up-lo)/n
  suppsA <- NULL
  for (i in 1:l){
    if(!is(try(su0 <- support(mixDistr[[i]]), silent=TRUE),"try-error"))
       suppsA <- c(suppsA,su0)
    }

  xseq <- c(seq(from = lo, to = up, by = h),
            suppsA,
            suppsA-getdistrOption("DistrResolution"))
  xseq <- sort(unique(xseq))          
  if(length(xseq)<2)
     xseq <- c(min(lo,up)-0.1,max(lo,up)+0.1)

  px.l <- pnew(xseq, lower.tail = TRUE)
  px.u <- pnew(xseq, lower.tail = FALSE)
  qnew <- .makeQNew(xseq, px.l, px.u, TRUE, lo, up, Cont = Cont)
  if(!is.null(gaps)&&length(gaps)){
      qnew <- .modifyqgaps(pfun = pnew, qfun = qnew, gaps = gaps, 
                           leftright = leftright)
  }else{
     if(exists("..q0fun", envir=environment(qnew)))
        qnew <- get("..q0fun", envir=environment(qnew))
  }
  return(qnew)
}


.loupmixfun <- function(mixDistr){
    if(length(mixDistr)==0) return(list(qL = NA, ql = NA, qU = NA, qu = NA))
    if(length(mixDistr)==1){
      q1 <- q.l(mixDistr[[1]])
      return(list(qL = q1(p = 0, lower.tail = TRUE),
                  ql = q1(p = getdistrOption("TruncQuantile"), lower.tail = TRUE),
                  qU = q1(p = 0, lower.tail = FALSE),
                  qu = q1(p = getdistrOption("TruncQuantile"), lower.tail =FALSE)
                  ))
    }
    qL0 <- as.vector(unlist(lapply(mixDistr, function(x)
                         do.call(x@q,list(p = 0, lower.tail = TRUE)))))
    qL1 <- as.vector(unlist(lapply(mixDistr, function(x)
                         do.call(x@q,list(p = getdistrOption("TruncQuantile"),
                                 lower.tail = TRUE)))))

## start patch 20200131 : in the last line added na.rm=TRUE
    qL1[!is.finite(qL1)] <- NA
    if(all(is.na(qL1))) qL1[1] <- Inf
    if(all(is.na(qL0))) qL0[1] <- Inf
    qL  <- min(qL0, na.rm = TRUE); ql <- min(qL1, na.rm = TRUE)
## end patch 20200131

    qU0 <- as.vector(unlist(lapply(mixDistr, function(x)
                         do.call(x@q,list(p = 0, lower.tail = FALSE)))))
    qU1 <- as.vector(unlist(lapply(mixDistr, function(x)
                         do.call(x@q,list(p = getdistrOption("TruncQuantile"),
                                 lower.tail = FALSE)))))

## start patch 20200131 : in the last line added na.rm=TRUE
    qU1[!is.finite(qU1)] <- NA
    if(all(is.na(qU1))) qU1[1] <- -Inf
    if(all(is.na(qU0))) qU0[1] <- -Inf
    qU  <- max(qU0, na.rm = TRUE); qu <- max(qU1, na.rm = TRUE)
## end patch 20200131

    return(list(qL = qL, ql = ql, qU = qU, qu = qu))
    }

.del0dmixfun <- function(mixDistr){
  dac <- mixDistr@mixDistr[[1]]@d
  if(!is.null(dac)){
      dnew <- function(x, log = FALSE, ...){
               d0 <- dac(x, log = log, ...)
               d0[.isEqual(x,0)] <-  0
               return(d0)
            }
      mixDistr@mixDistr[[1]]@d <- dnew
  }
  return(mixDistr)
}

.ULC.cast <- function(x){
         if( is(x,"AbscontDistribution"))
             x <- as(as(x,"AbscontDistribution"), "UnivarLebDecDistribution")
         if(is(x,"DiscreteDistribution"))
             x <- as(as(x,"DiscreteDistribution"), "UnivarLebDecDistribution")
         if(!is(x,"UnivarLebDecDistribution"))
            x <- as(x,"UnivarLebDecDistribution")
         return(x)
}

################################
##
## Class: PoisParameter
##
################################

## Access Methods
setMethod("lambda", "PoisParameter", function(object) object@lambda)
## Replace Methods
setReplaceMethod("lambda", "PoisParameter", 
                  function(object, value){ object@lambda <- value; object})

setValidity("PoisParameter", function(object){
  if(length(lambda(object)) != 1)
    stop("lambda has to be a numeric of length 1")    
  if(lambda(object) < 0)
    stop("lambda has to be not negative")
  else return(TRUE)
})

################################
##
## Class: Poisson distribution 
##
################################

Pois <- function(lambda = 1) new("Pois", lambda = lambda)

## wrapped access methods
setMethod("lambda", "Pois", function(object) lambda(param(object)))
## wrapped replace methods
setMethod("lambda<-", "Pois", 
           function(object, value) new("Pois", lambda = value))

setMethod("+", c("Pois","Pois"),
          function(e1,e2){
            new("Pois", lambda = lambda(e1) + lambda(e2), .withArith = TRUE)
          })

# test for equality of the saved result and the actual result of a dontrun example
test.operatorsMethods <- function() {
  # we compute the actual result
  N <- Norm(0,3)
  P <- Pois(4)
  runit.dontrunOperatorsMethods.actual <- N ^ P
  
  # we load the stored result
  #   we assume that this test is called from within the script in the upper directory
  load("unitTests/runit.dontrunOperatorsMethods.save")
  
  # we compare the stored result with the calculated one
  #   (a comparison with identical (ignoring the environment) gives FALSE...
  result <- all.equal(runit.dontrunOperatorsMethods.actual,
                      runit.dontrunOperatorsMethods.save)
  
  # we check whether the result is TRUE and if not, we write the message
  #   coming from the result
  checkEquals(is.logical(result) && result, TRUE, msg=paste(result, sep="", collapse="\n"))
}

################################################################
# QQ - Plot helper functions in package distr
################################################################


## helpers
.inGaps <- function(x,gapm){
  if(is.null(gapm)) return(rep(FALSE,length(x)))
  fct <- function(x,m){ m[,2]>=x & m[,1]<=x}
  sapply(x, function(y) length(which(fct(y,gapm)))>0)
}

.isReplicated <- function(x, tol=.Machine$double.eps){
  tx <- table(x)
  rx <- as.numeric(names(tx[tx>1]))
  sapply(x, function(y) any(abs(y-rx)<tol))
}

.NotInSupport <- function(x,D){
  if(length(x)==0) return(logical(0))
  nInSupp <- which(x < q.l(D)(0))
  nInSupp <- unique(sort(c(nInSupp,which(x > q.l(D)(1)))))

  nInSuppo <-
      if("support" %in% names(getSlots(class(D))))
         which( ! x %in% support(D)) else numeric(0)
  if("gaps" %in% names(getSlots(class(D)))){
         InGap <- which( .inGaps(x,gaps(D)))
         if("support" %in% names(getSlots(class(D))))
            nInSupp <- unique(sort(c(nInSupp, intersect(InGap,nInSuppo))))
         else
            nInSupp <- unique(sort(c(nInSupp, InGap)))
  }else{
         nInSupp <- unique(sort(c(nInSupp, nInSuppo)))
  }
  return((1:length(x)) %in% nInSupp)
}


.SingleDiscrete <- function(x,D){
  ## produces a logical vector of
  ##     0  : discrete mass point
  ##     1  : within continuous support
  ##     2  : left gap point
  ##     3  : right gap point
  ##     4  : not in support
  lx <- x * 0

  lx[.NotInSupport(x,D)] <- 4

  idx.0 <- ((x>q.l(D)(1)) | (x<q.l(D)(0)))
  iG <- rep(FALSE,length(x))

  if(is(D, "DiscreteDistribution")){
     return(lx)
  }
  if("gaps" %in% names(getSlots(class(D)))){
     if(!is.null(gaps(D))){
        lx[apply(sapply(gaps(D)[,1], function(u) .isEqual(u,x)),1,any)] <- 2
        lx[apply(sapply(gaps(D)[,2], function(u) .isEqual(u,x)),1,any)] <- 3
        iG <- .inGaps(x,gaps(D))
        lx[!idx.0 & !iG] <- 1
     }else{
        lx[!idx.0 & !iG] <- 1
     }
  }
  if("support" %in% names(getSlots(class(D)))){
     idx <- x %in% support(D)
     if("acPart" %in% names(getSlots(class(D))))
         idx.0 <- ((x>q.ac(D)(1)) | (x<q.ac(D)(0)))
     lx[idx & (idx.0|iG)] <- 0
  }

  return(lx)
}


.makeLenAndOrder <- function(x,ord){
   n <- length(ord)
   x <- rep(x, length.out=n)
   x[ord]
}


.pk2 <- if(getRversion()<"2.16.0") function(p0, n){
                 .C("pkolmogorov2x", p = as.double(p0),
                     as.integer(n), PACKAGE = "stats")$p
        }else function(p0,n){
                 .Call(C_pKolmogorov2x, #"pKolmogorov2x",
                       p0, n) #, PACKAGE = "stats")
        }
.pks2 <- if(getRversion()<"2.16.0") function(x, tol){
                 .C("pkstwo",
                    as.integer(1),
                    p = as.double(x), as.double(tol), PACKAGE = "stats")$p
        }else function(x, tol){
                 .Call(C_pKS2, #"pKS2",
                       p = x, tol) #, PACKAGE = "stats")
        }


.q2kolmogorov <- function(alpha,n,exact=(n<100), silent0 = TRUE){ ## Kolmogorovstat
 if(is.numeric(alpha)) alpha <- as.vector(alpha)
 else stop("Level alpha must be numeric.")
 if(any(is.na(alpha))) stop("Level alpha must not contain missings.")
 if(exact){
   fct <- function(p0){
   ### from ks.test from package stats:
      .pk2(p0,n) -alpha
   }
   i <- 0
   oK <- FALSE
   del <- 0.01
   while(!oK && i < 20){
       i <- i + 1
       res <- try(uniroot(fct,lower=del,upper=3*(1-del)/sqrt(n))$root*sqrt(n), silent=silent0)
       del <- del / 10
       if(!is(res, "try-error")) oK <- TRUE
   }
 }else{
   fct <- function(p0){
   ### from ks.test from package stats:
        1 - .pks2(p0,1e-09)-alpha  }
   i <- 0
   oK <- FALSE
   del <- 0.01
   while(!oK && i < 20){
       i <- i + 1
       res <- try(uniroot(fct,lower=del,upper=3*(1-del))$root, silent=silent0)
       del <- del / 10
       if(!is(res, "try-error")){
          oK <- TRUE
       }else{
          if(!silent0){
              cat("Problem in uniroot in .q2kolmogorov:\nSearch bounds were")
              print(c(lower=del,upper=sqrt(n)*(1-del)))
          }
       }
   }
 }
 return(res)
}

.BinomCI.in <- function(t,p.bi,x.i, del.i=0,D.i,n.i,alpha.i){
   p.bi.u <- p(D.i)(x.i+(t+del.i)/sqrt(n.i))
   p.bi.l <- p.l(D.i)(x.i-(t-del.i)/sqrt(n.i))
   d.r <- if(n.i*p.bi>floor(n.i*p.bi)) 0 else
        dbinom(x = n.i*p.bi, size = n.i, prob = pmax(p.bi.u,1))
   p.r <- pbinom(q = n.i*p.bi, size = n.i, prob = pmin(p.bi.u,1),lower.tail=FALSE)+d.r
   p.l <- pbinom(q = n.i*p.bi, size = n.i, prob = pmax(p.bi.l,0),lower.tail=FALSE)
   r <- p.r -p.l - alpha.i
#   print(c(r=r,p=p.bi,x=x.i,p.u=p.bi.u,p.l=p.bi.l,r.r=p.r,r.l=p.l,t=t,np=n*p.bi))
   r
  }


.BinomCI <- function(x,p.b,D,n,alpha,silent0 = TRUE){
  if(length(x)==0) return(NA)
  res <- sapply(1:length(x),
           function(i){
              .nm.i <- sqrt(n)*max(x[i],n-x[i]) + 1
              .fct.i <- function(t){
                         .BinomCI.in(t, p.bi = p.b[i], x.i = x[i], del.i = 0,
                                     D.i = D, n.i = n, alpha.i = alpha)
              }
              ii <- 0
              oK <- FALSE
              del <- 0.01
              while(ii < 20 && !oK){
                 ii <- ii + 1
                 res0 <- try(uniroot(.fct.i, tol = 1e-9, lower = del,
                                upper=.nm.i*(1-del))$root,
                             silent=silent0)
                 del <- del / 10
                 if(!is(res0, "try-error")){
                     oK <- TRUE
                 }else{
                     if(!silent0){
                         cat("Problem in uniroot in .BinomCI:\nSearch bounds were")
                         print(c(lower=del,upper=.nm.i*(1-del)))
                         cat("Further settings:\n")
                         print(c(i=i,p.bi = p.b[i], x.i = x[i], del=del))
                     }
                     res0 <- NA
                 }
              }
              return(res0)
           }
         )
  return(cbind(left=-res, right=res))
}

.BinomCI.nosym <- function(x,p.b,D,n,alpha,silent0 = TRUE){
  if(length(x)==0) return(NA)
  res0 <- sapply(1:length(x),
            function(i){
              .nm.i <- max(x[i],n-x[i])*sqrt(n) + 1
              .fct.i.d <- function(t,del.0){
                         .BinomCI.in(t, p.bi = p.b[i], x.i = x[i], del.i = del.0,
                                     D.i = D, n.i = n, alpha.i = alpha)
              }
              get.t <- function(del.o){
                 ii <- 0
                 oK <- FALSE
                 del.l <- 0.01
                 while(ii < 20 && !oK){
                    ii <- ii + 1
                    res0 <- try(uniroot(.fct.i.d, tol = 1e-9, lower = del.l,
                                        upper = .nm.i*(1-del.l),
                                        del.0 = del.o)$root, silent = silent0)
                    del.l <- del.l / 10
                    if(!is(res0, "try-error")){
                       oK <- TRUE
                    }else{
                       if(!silent0){
                           cat("Problem in uniroot in .BinomCI.nosym:\nSearch bounds were")
                           print(c(lower=del.l,upper=.nm.i*(1-del.l)))
                           cat("Further settings:\n")
                           print(c(i=i,p.bi = p.b[i], x.i = x[i], del.i=del.o,
                                   del.l=del.l))
                       }
                       res0 <- NA
                    }
                 }
                 return(res0)
              }
    res <- try(optimize(get.t, lower=-.nm.i,  upper = .nm.i, tol = 1e-9),
               silent = silent0)
    if(is(res,"try-error")){
       if(!silent0){
           cat("Problem in optimize in .BinomCI.nosym:\nSearch bounds were")
           print(c(lower=-.nm.i,upper=.nm.i))
           cat("Further settings:\n")
           print(c(i=i,p.bi = p.b[i], x.i = x[i]))
       }
       return(c(NA,NA))
    }
    t.o <- res$objective
    del <- res$minimum
    c(left=-t.o+del, right=t.o+del)
    })
   return(t(res0))
}


.q2pw <- function(x,p.b,D,n,alpha,exact=(n<100),nosym=FALSE, silent0=TRUE){
 if(exact){
    fct <- if(nosym) .BinomCI.nosym else .BinomCI
    ro <- fct(x,p.b,D,n,alpha,silent0)
    return(ro)
 }
 pq <- log(p.b)+log(1-p.b)
 if(is(D,"AbscontDistribution")){
    dp <- d(D)(x,log=TRUE)
    dsupp.p <- dsupp.m <- 1
 }else{ ## have E and sd available ?
    if(!.distrExInstalled) stop("")
    supp.ind <- sapply(x, function(y)
                 which.min(abs(y-support(D))))
    nx <- length(support(D))
    supp.ind.p <- pmax(supp.ind + 1 ,nx)
    supp.ind.m <- pmax(supp.ind - 1 ,1)
    dsupp.p <- support(D)[supp.ind.p] - support(D)[supp.ind]
    dsupp.m <- support(D)[supp.ind] - support(D)[supp.ind.m]
    m <- distrEx::E(D)
    s <- sqrt(distrEx::E(D, fun = function(x) (x-m)^2))
    dp <- log(pnorm((x+dsupp.p/2-m)/s) - pnorm((x-dsupp.m/2-m)/s))
 }
 ro <- exp(pq/2-dp)*(dsupp.p+dsupp.m)/2*qnorm((1+alpha)/2)
 return(cbind(left=-ro,right=ro))
}




.confqq <- function(x,D, datax = FALSE, withConf.pw  = TRUE,
                    withConf.sim = TRUE, alpha,
                    col.pCI, lty.pCI, lwd.pCI, pch.pCI, cex.pCI,
                    col.sCI, lty.sCI, lwd.sCI, pch.sCI, cex.sCI,
                    n,exact.sCI=(n<100),exact.pCI=(n<100), nosym.pCI = FALSE,
                    with.legend = TRUE, legend.bg = "white",
                    legend.pos = "topleft", legend.cex = 0.8,
                    legend.pref = "", legend.postf = "", 
                    legend.alpha = alpha, qqb0=NULL, transf0=NULL, debug = FALSE){

   x <- sort(unique(x))
   if("gaps" %in% names(getSlots(class(D))))
       {if(!is.null(gaps(D)))
            x <- sort(unique(c(x, gaps(D))))
       }
   SI <- .SingleDiscrete(x,D)
#   print(SI)
   SI.in <- SI<4
   SIi <- SI[SI.in]
   SI.c <- SIi>0
   x.in <- x[SI.in]
   x.c <- x.in[SI.c]
   x.d <- x.in[!SI.c]        
   tx.c <- if(is.null(transf0)) x.c else transf0(x.c)
   tx.d <- if(is.null(transf0)) x.d else transf0(x.d)

   qqb <- if(is.null(qqb0)) qqbounds(x,D,alpha,n,withConf.pw, withConf.sim,
                   exact.sCI,exact.pCI,nosym.pCI, debug) else qqb0
                   
   qqb$crit <- if(is.null(qqb0)) qqb$crit[SI.in,]

   if(qqb$err["pw"]){
      if(sum(SI.c)>0){
         if(datax){
            lines(tx.c, qqb$crit[SI.c,"pw.right"],
               col=col.pCI,lty=lty.pCI,lwd=lwd.pCI)
            lines(tx.c, qqb$crit[SI.c,"pw.left"],
               col=col.pCI,lty=lty.pCI,lwd=lwd.pCI)
         }else{
            lines(qqb$crit[SI.c,"pw.right"], tx.c,
               col=col.pCI,lty=lty.pCI,lwd=lwd.pCI)
            lines(qqb$crit[SI.c,"pw.left"], tx.c,
               col=col.pCI,lty=lty.pCI,lwd=lwd.pCI)
         }
      }
      if(sum(!SI.c)>0){
         if(datax){
            points(tx.d, qqb$crit[!SI.c,"pw.right"],
               col=col.pCI, pch=pch.pCI, cex = cex.pCI)
            points(tx.d, qqb$crit[!SI.c,"pw.left"],
               col=col.pCI, pch=pch.pCI, cex = cex.pCI)
         }else{
            points(qqb$crit[!SI.c,"pw.right"], tx.d,
               col=col.pCI, pch=pch.pCI, cex = cex.pCI)
            points(qqb$crit[!SI.c,"pw.left"], tx.d,
               col=col.pCI, pch=pch.pCI, cex = cex.pCI)
         }
      }
   }
   if(qqb$err["sim"]){
      if(sum(SI.c)>0){
         if(datax){
            lines(tx.c, qqb$crit[SI.c,"sim.right"],
               col=col.sCI,lty=lty.sCI,lwd=lwd.sCI)
            lines(tx.c, qqb$crit[SI.c,"sim.left"],
               col=col.sCI,lty=lty.sCI,lwd=lwd.sCI)
         }else{
            lines(qqb$crit[SI.c,"sim.right"], tx.c,
               col=col.sCI,lty=lty.sCI,lwd=lwd.sCI)
            lines(qqb$crit[SI.c,"sim.left"], tx.c,
               col=col.sCI,lty=lty.sCI,lwd=lwd.sCI)
         }
      }
      if(sum(!SI.c)>0){
         if(datax){
            points(tx.d, qqb$crit[!SI.c,"sim.right"],
                col=col.sCI, pch=pch.sCI, cex = cex.sCI)
            points(tx.d, qqb$crit[!SI.c,"sim.left"],
                col=col.sCI, pch=pch.sCI, cex = cex.sCI)
         }else{
            points(qqb$crit[!SI.c,"sim.right"], tx.d,
                col=col.sCI, pch=pch.sCI, cex = cex.sCI)
            points(qqb$crit[!SI.c,"sim.left"], tx.d,
                col=col.sCI, pch=pch.sCI, cex = cex.sCI)
         }
      }
   }
   if(with.legend){
      if( qqb$err["pw"] ||  qqb$err["sim"] ){
         expression1 <- substitute(
            legpf~nosym0~"pointw."~ex.p~alpha==alpha0~"%- conf. interval"~legpof,
            list(legpf = legend.pref, legpof = legend.postf,
                 ex.p = if(exact.pCI) "exact" else "asympt.",
                 alpha0 = round(legend.alpha*100,2),
                 nosym0 = if(nosym.pCI&&exact.pCI) "shortest asymm." else "symm"))
         expression2 <- substitute(
            legpf~"simult."~ex.s~alpha==alpha0~"%- conf. interval"~legpof,
            list(legpf = legend.pref, legpof = legend.postf,
                 ex.s = if(exact.sCI) "exact" else "asympt.",
                 alpha0 = round(legend.alpha*100,2)))

         lcl <- list()
         if(!qqb$err["sim"]){
            expression3 <- expression1
            lcl$pch <- if(sum(!SI.c)>0) pch.pCI else NULL
            lcl$lty <- if(sum(SI.c)>0)  lty.pCI else NULL
            lcl$col <- col.pCI
            lcl$lwd <- if(sum(SI.c)>0)  2 else NULL
         }
         if(!qqb$err["pw"]){
            expression3 <- expression2
            lcl$pch <- if(sum(!SI.c)>0) pch.sCI else NULL
            lcl$lty <- if(sum(SI.c)>0)  lty.sCI else NULL
            lcl$col <- col.sCI
            lcl$lwd <- if(sum(SI.c)>0)  2 else NULL
         }
         if( qqb$err["pw"] && qqb$err["sim"]){
            expression3 <- eval(substitute(expression(expression1, expression2)))
            lcl$pch <- if(sum(!SI.c)>0) c(pch.pCI, pch.sCI) else NULL
            lcl$lty <- if(sum(SI.c)>0)  c(lty.pCI, lty.sCI) else NULL
            lcl$col <- c(col.pCI,col.sCI)
            lcl$lwd <- if(sum(SI.c)>0)  2 else NULL
         }
         do.call(legend, c(list(legend.pos, legend = expression3, bg = legend.bg,
                                merge = FALSE, cex = legend.cex), lcl))
      }
   }
  return(invisible(qqb))
}

.deleteItemsMCL <- function(mcl){
    mcl$n <- NULL
    mcl$col.IdL <- mcl$alpha.CI <- mcl$lty.IdL <-  NULL
    mcl$col.NotInSupport <- mcl$check.NotInSupport <- NULL
    mcl$exact.sCI <- mcl$exact.pCI <- NULL
    mcl$withConf <- mcl$withConf.sim <- mcl$withConf.pw <- NULL
    mcl$withIdLine <- mcl$distance <- NULL
    mcl$col.pCI <- mcl$lty.pCI <- mcl$col.sCI <- mcl$lty.sCI <- NULL
    mcl$lwd.IdL <- mcl$lwd.pCI <- mcl$lwd.sCI <- NULL
    mcl$withLab <- mcl$lab.pts <- mcl$which.lbs <- NULL
    mcl$which.Order <- mcl$order.traf  <- NULL
    mcl$col.pch <- mcl$cex.pch  <- mcl$jit.fac <- NULL
    mcl$col.lbl <- mcl$cex.lbl  <- mcl$adj.lbl <- NULL
    mcl$exp.cex2.pch <- mcl$exp.cex2.lbl <- NULL
    mcl$exp.fadcol.pch <- mcl$exp.fadcol.lbl <- NULL
    mcl$nosym.pCI <- mcl$n.CI <- mcl$n.adj <- NULL
    mcl$legend.cex <- mcl$with.legend <- mcl$legend.bg <- NULL
    mcl$legend.pos <- mcl$legend.pref <- mcl$legend.postf <- NULL
    mcl$legend.alpha <- NULL
mcl}

############################## Accessor / Replacement functions

############################## Arithmetics

setMethod("*", c("AffLinUnivarLebDecDistribution","numeric"),
          function(e1, e2) {

          if (length(e2)>1) stop("length of operator must be 1")

          if (isTRUE(all.equal(e2,1))) return(e1)
          if (isTRUE(all.equal(e2,0)))
               return(new("Dirac", location = 0))
          
          if(.isEqual(e1@a*e2,1)&&.isEqual(e1@b,0)){
             obj <- e1@X0
             if(getdistrOption("simplifyD"))
                obj <- simplifyD(obj)
             return(obj)
          }   

          Distr <- UnivarLebDecDistribution(
                     discretePart = discretePart(e1)*e2,
                     acPart = acPart(e1)*e2,
                     discreteWeight = discreteWeight(e1),
                     acWeight = acWeight(e1))

          if(.isEqual(e1@a*e2,1)&&.isEqual(e1@b,0)){
             obj <- e1@X0
             if(getdistrOption("simplifyD"))
                obj <- simplifyD(obj)
             return(obj)
          }   

          Symmetry <- NoSymmetry()
          if(is(e1@Symmetry,"SphericalSymmetry"))
             Symmetry <- SphericalSymmetry(SymmCenter(e1@Symmetry) + e2)   

          object <- new("AffLinUnivarLebDecDistribution",
                    r = Distr@r, d = Distr@d, p = Distr@p,
                    q = Distr@q, X0 = e1@X0, mixDistr = Distr@mixDistr,
                    mixCoeff = Distr@mixCoeff,
                    a = e1@a*e2, b = e1@b, .withSim  = e1@.withSim,
                    .withArith = TRUE,
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1),
                     gaps = gaps(Distr), support = support(Distr),
                     Symmetry = Symmetry
                     )
          object})

setMethod("+", c("AffLinUnivarLebDecDistribution","numeric"),
          function(e1, e2) {
          if (length(e2)>1) stop("length of operator must be 1")
          if (isTRUE(all.equal(e2,0))) return(e1)

          if(.isEqual(e1@a,1)&&.isEqual(e1@b+e2,0)){
             obj <- e1@X0
             if(getdistrOption("simplifyD"))
                obj <- simplifyD(obj)
             return(obj)
          }   
          
          Distr <- UnivarLebDecDistribution(
                     discretePart = discretePart(e1)+e2,
                     acPart = acPart(e1)+e2,
                     discreteWeight = discreteWeight(e1),
                     acWeight = acWeight(e1))

          if(.isEqual(e1@a,1)&&.isEqual(e1@b+e2,0)){
             obj <- e1@X0
             if(getdistrOption("simplifyD"))
                obj <- simplifyD(obj)
             return(obj)
          }   
          
          Symmetry <- NoSymmetry()
          if(is(e1@Symmetry,"SphericalSymmetry"))
             Symmetry <- SphericalSymmetry(SymmCenter(e1@Symmetry) * e2)   

          object <- new("AffLinUnivarLebDecDistribution",
                    r = Distr@r, d = Distr@d, p = Distr@p,
                    q = Distr@q, X0 = e1@X0, mixDistr = Distr@mixDistr,
                    mixCoeff = Distr@mixCoeff,
                    a = e1@a, b = e1@b+e2, .withSim  = e1@.withSim,
                    .withArith = TRUE,
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1),
                     gaps = gaps(Distr), support = support(Distr),
                     Symmetry = Symmetry
                     )
          object})

################################################################################

## standardMethods

## generates standard methods for an existing class

## args:
##
## class         -    string containing the class names
## writetofile   -    logical value: TRUE, if output is to be written on file
## directory     -    target directory for the file to be created

## value:
##
## none

## Details:
##
## If output, too, is to be written to a file, standardMethods generates a file
## <classname>_StandardMethods.txt in the given directory

################################################################################

standardMethods <- function(class, writetofile = FALSE, directory){
  ClassRep <- getClassDef(class)
  SlotNames <- names(getSlots(ClassRep))
  nrSlots <- length(SlotNames)
  
  if(!nrSlots) return()
  
  part0 <- "if(!isGeneric(\""
  part1 <- "\")) setGeneric(\""
  part2 <- "\", function(object) standardGeneric(\""
  part3 <- "\"))"
  
  for(i in 1:nrSlots){
    string <- paste(part0, SlotNames[i], part1, SlotNames[i],
                    part2, SlotNames[i], part3, "\n", sep = "")
    if(writetofile) cat(string, 
                        file = paste(directory, class, 
                                     "_StandardMethods.txt", sep=""), 
                        append = FALSE)
    cat(string, sep = "")
  }    
  
  part1 <- "setMethod(\""
  part2 <- "\", \""
  part3 <- "\", function(object) object@"
  part4 <- ")"
  
  for(i in 1:nrSlots){
    string <- paste(part1, SlotNames[i], part2, class, 
                    part3, SlotNames[i], part4, "\n", sep = "")
    if(writetofile) cat(string, 
                        file = paste(directory, class, 
                                     "_StandardMethods.txt", sep=""), 
                        append = TRUE)
    cat(string, sep = "")
  }    
  
  part0 <- "if(!isGeneric(\""
  part1 <- "<-\")) setGeneric(\""
  part2 <- "<-\", function(object, value) standardGeneric(\""
  part3 <- "<-\"))"
  
  for(i in 1:nrSlots){
    string <- paste(part0, SlotNames[i], part1, SlotNames[i], 
                    part2, SlotNames[i], part3, "\n", sep = "")
    if(writetofile) cat(string, 
                        file = paste(directory, class, 
                                     "_StandardMethods.txt", sep=""), 
                        append = TRUE)
    cat(string, sep = "")
  }    
  
  part1 <- "setReplaceMethod(\""
  part2 <- "\", \""
  part3 <- "\", function(object, value){ object@"
  part4 <- " <- value; object})"
  
  for(i in 1:nrSlots){
    string <- paste(part1, SlotNames[i], part2, class, 
                    part3, SlotNames[i], part4, "\n", sep = "")
    if(writetofile) cat(string, 
                        file = paste(directory, class, 
                                     "_StandardMethods.txt", sep=""), 
                        append = TRUE)
    cat(string, sep = "")
  }    
}            



## Example

##setClass("testclass", representation(a = "numeric", b = "character"))
##standardMethods("testclass")
##directory = "C:/TMP/"  ## directory C:\TMP must exist ...
##standardMethods("testclass", writetofile = TRUE, directory = directory)

########################################################################
### intentional maskings --- purpose: add formal arg "..." to generic...
########################################################################

############################################################################
### ------------------------------------------------
### In this comment substitute 'xxx' by 'qqplot', 'df', and 'sd', respectively
### ------------------------------------------------
### We intentionally mask function 'xxx' from stats in order to add a formal
### argument "...". 
### functionality of 'stats::xxx' is completely retained, however
### for help to the original 'stats::xxx' function write
###       'help("xxx", package="stats")'                             
### for code to the original 'stats::xxx' function write
###       'stats::xxx'
############################################################################

## masking function df

df <- function(x, ...)
       {
        dots <- list(...)
        if(hasArg("df1")) df1 <- dots$"df1"
           else stop("Argument df1 missing")
        if(hasArg("df2")) df2 <- dots$"df2"
           else stop("Argument df2 missing")
        log.arg <- if(hasArg("log")) dots$"log" else FALSE 

        if(hasArg("ncp")) ncp <- dots$"ncp"
           else ncp <- 0

        if(isTRUE(all.equal(ncp,0))||(getRversion()>='2.4.0'))
             return(stats::df(x = x, df1 = df1, df2 = df2, 
                              log = log.arg, ncp=ncp))
        else
          ## preliminary version for ncp in df:
             { TQ <- getdistrOption("TruncQuantile")
               xz <- qchisq(1-TQ,df=df1,ncp=ncp)
               xn <- qchisq(TQ,df=df2,ncp=0)
               pfun <- function(x){pf(x, df1=df1, df2=df2, ncp=ncp)}
               dfun <- .P2D(p=pfun, ql=0, qu=df2*xz/xn/df1)
               #
               ## simulational alternative:
               #rfun <- function(x){rf(x, df1=df1, df2=df2, ncp=ncp)}
               #dfun <-R2D(rfun, nsim = 10^getdistrOption("RtoDPQ.e"),
               #            n = getdistrOption("DefaultNrGridPoints"))
               d <- dfun(x)
               if(log.arg==TRUE) return(log(d))
               else return(d)
              }
        }

## masking function sd

sd <- function(x, ...){
      dots <- list(...)
      na.rm <- ifelse(hasArg("na.rm"), dots$"na.rm", FALSE)
      stats::sd(x = x, na.rm = na.rm)
      }

## masking function qqplot

#if(!isGeneric("qqplot"))
    setGeneric("qqplot", function(x, y, ...) standardGeneric("qqplot"))

setMethod("qqplot", signature(x="ANY",y="ANY"), function(x, y,
    plot.it = TRUE, xlab = deparse(substitute(x)),
    ylab = deparse(substitute(y)), ...){
    mc <- match.call(call = sys.call(sys.parent(1)))
    if(missing(xlab)) mc$xlab <- xlab
    if(missing(ylab)) mc$ylab <- ylab
    mcl <- as.list(mc)[-1]
    return(invisible(do.call(stats::qqplot, args=mcl)))
    })

### ---------------------------
### "multiple-purpose generics" --- 
###   for  
###      + accessor method  
###      + stats/base function
###      + functional method (in case of 'sd' with additional package 'distrEx') 
### ---------------------------

### .. + stats function

if(!isGeneric("df")) 
    setGeneric("df", function(x, ...) standardGeneric("df"))

if(!isGeneric("sd")) 
    setGeneric("sd", function(x, ...) standardGeneric("sd"))

if(!isGeneric("mean")) 
    setGeneric("mean", function(x, ...) standardGeneric("mean"))

### .. + base function

if(!isGeneric("q")) 
    setGeneric("q", function(save = "default", status = 0, runLast = TRUE) 
                    standardGeneric("q"))

if(!isGeneric("scale")) 
    setGeneric("scale", function(x, center = TRUE, scale = TRUE) 
                                standardGeneric("scale"))


############################################################################
# Arithmetics
############################################################################

if(getRversion()<'2.9.0'){
if(!isGeneric("log")) 
    setGeneric("log", function(x, base) standardGeneric("log"), group = "Math")
if(!isGeneric("log10")) 
    setGeneric("log10", function(x) standardGeneric("log10"), group = "Math")
if(!isGeneric("gamma")) 
    setGeneric("gamma", function(x) standardGeneric("gamma"), group = "Math")
if(!isGeneric("lgamma")) 
    setGeneric("lgamma", function(x) standardGeneric("lgamma"), group = "Math")
}



############################################################################
# Access methods
############################################################################

if(!isGeneric("name")) 
    setGeneric("name", function(object) standardGeneric("name"))
if(!isGeneric("dimension")) 
   setGeneric("dimension", function(object) standardGeneric("dimension"))

if(!isGeneric("shape")) 
   setGeneric("shape", function(object) standardGeneric("shape"))

if(!isGeneric("rate")) 
   setGeneric("rate", function(object) standardGeneric("rate"))

if(!isGeneric("Length"))  
   setGeneric("Length", function(object) standardGeneric("Length"))
if(!isGeneric("width"))   
   setGeneric("width", function(object) standardGeneric("width"))
if(!isGeneric("pivot"))   
   setGeneric("pivot", function(object) standardGeneric("pivot"))
if(!isGeneric("lattice")) 
   setGeneric("lattice", function(object) standardGeneric("lattice"))

if(!isGeneric("location")) 
   setGeneric("location", function(object) standardGeneric("location"))


if(!isGeneric("ncp")) 
   setGeneric("ncp", function(object) standardGeneric("ncp"))

if(!isGeneric("lambda")) 
   setGeneric("lambda", function(object) standardGeneric("lambda"))


if(!isGeneric("size")) 
   setGeneric("size", function(object) standardGeneric("size"))
if(!isGeneric("prob")) 
   setGeneric("prob", function(object) standardGeneric("prob"))

if(!isGeneric("support")) 
   setGeneric("support", function(object) standardGeneric("support"))

if(!isGeneric("m")) 
   setGeneric("m", function(object) standardGeneric("m"))
if(!isGeneric("n")) 
   setGeneric("n", function(object) standardGeneric("n"))
if(!isGeneric("k")) 
   setGeneric("k", function(object) standardGeneric("k"))

if(!isGeneric("img")) 
   setGeneric("img", function(object) standardGeneric("img"))
if(!isGeneric("param")) 
   setGeneric("param", function(object) standardGeneric("param"))
if(!isGeneric("r")) 
   setGeneric("r", function(object) standardGeneric("r"))
if(!isGeneric("d")) 
   setGeneric("d", function(object) standardGeneric("d"))
if(!isGeneric("p")) 
   setGeneric("p", function(object) standardGeneric("p"))

if(!isGeneric("p.l"))
   setGeneric("p.l", function(object) standardGeneric("p.l"))
if(!isGeneric("q.r"))
   setGeneric("q.r", function(object) standardGeneric("q.r"))
if(!isGeneric("p.r"))
   setGeneric("p.r", function(object) standardGeneric("p.r"))
if(!isGeneric("q.l"))
   setGeneric("q.l", function(object) standardGeneric("q.l"))

if(!isGeneric("gaps"))
   setGeneric("gaps", function(object) standardGeneric("gaps"))

if(!isGeneric("Min")) 
   setGeneric("Min", function(object) standardGeneric("Min"))
if(!isGeneric("Max")) 
   setGeneric("Max", function(object) standardGeneric("Max"))

if(!isGeneric("df1")) 
   setGeneric("df1", function(object) standardGeneric("df1"))
if(!isGeneric("df2")) 
   setGeneric("df2", function(object) standardGeneric("df2"))

if(!isGeneric("meanlog")) 
   setGeneric("meanlog", function(object) standardGeneric("meanlog"))
if(!isGeneric("sdlog")) 
   setGeneric("sdlog", function(object) standardGeneric("sdlog"))

if(!isGeneric("shape1")) 
   setGeneric("shape1", function(object) standardGeneric("shape1"))
if(!isGeneric("shape2")) 
    setGeneric("shape2", function(object) standardGeneric("shape2"))

############################################################################
# Replacement methods
############################################################################

if(!isGeneric("name<-")) 
    setGeneric("name<-", 
                function(object, value) standardGeneric("name<-"))
if(!isGeneric("dimension<-")) 
   setGeneric("dimension<-", 
               function(object, value) standardGeneric("dimension<-"))

if(!isGeneric("gaps<-")) 
   setGeneric("gaps<-", function(object, value) standardGeneric("gaps<-"))

if(!isGeneric("shape<-")) 
   setGeneric("shape<-", function(object, value) standardGeneric("shape<-"))
if(!isGeneric("scale<-")) 
   setGeneric("scale<-", function(object, value) standardGeneric("scale<-"))

if(!isGeneric("rate<-")) 
   setGeneric("rate<-", function(object, value) standardGeneric("rate<-"))

if(!isGeneric("Length<-")) 
   setGeneric("Length<-", function(object, value) standardGeneric("Length<-"))
if(!isGeneric("width<-"))  
   setGeneric("width<-", function(object, value) standardGeneric("width<-"))
if(!isGeneric("pivot<-"))  
   setGeneric("pivot<-", function(object, value) standardGeneric("pivot<-"))

if(!isGeneric("location<-")) 
   setGeneric("location<-", function(object, value) 
                                     standardGeneric("location<-"))

if(!isGeneric("df<-")) 
   setGeneric("df<-", function(object, value) standardGeneric("df<-"))
if(!isGeneric("ncp<-")) 
   setGeneric("ncp<-", function(object, value) standardGeneric("ncp<-"))

if(!isGeneric("lambda<-")) 
   setGeneric("lambda<-", function(object, value) standardGeneric("lambda<-"))

if(!isGeneric("size<-")) 
   setGeneric("size<-", function(object, value) standardGeneric("size<-"))
if(!isGeneric("prob<-")) 
   setGeneric("prob<-", function(object, value) standardGeneric("prob<-"))

if(!isGeneric("m<-")) 
   setGeneric("m<-", function(object, value) standardGeneric("m<-"))
if(!isGeneric("n<-")) 
   setGeneric("n<-", function(object, value) standardGeneric("n<-"))
if(!isGeneric("k<-")) 
   setGeneric("k<-", function(object, value) standardGeneric("k<-"))

if(!isGeneric("mean<-")) 
   setGeneric("mean<-", function(object, value) standardGeneric("mean<-"))
if(!isGeneric("sd<-")) 
   setGeneric("sd<-", function(object, value) standardGeneric("sd<-"))

if(!isGeneric("Min<-")) 
   setGeneric("Min<-", function(object, value) standardGeneric("Min<-"))
if(!isGeneric("Max<-")) 
   setGeneric("Max<-", function(object, value) standardGeneric("Max<-"))

if(!isGeneric("df1<-")) 
   setGeneric("df1<-", function(object, value) standardGeneric("df1<-"))
if(!isGeneric("df2<-")) 
   setGeneric("df2<-", function(object, value) standardGeneric("df2<-"))

if(!isGeneric("meanlog<-")) 
   setGeneric("meanlog<-", function(object, value) 
                           standardGeneric("meanlog<-"))
if(!isGeneric("sdlog<-")) 
   setGeneric("sdlog<-", function(object, value) standardGeneric("sdlog<-"))

if(!isGeneric("shape1<-")) 
   setGeneric("shape1<-", function(object, value) standardGeneric("shape1<-"))
if(!isGeneric("shape2<-")) 
    setGeneric("shape2<-", function(object, value) standardGeneric("shape2<-"))

############################################################################
# generics to  "usual"  methods
############################################################################

if(!isGeneric("liesIn")) 
   setGeneric("liesIn", function(object, x) standardGeneric("liesIn"))

if(!isGeneric("liesInSupport")) 
   setGeneric("liesInSupport", function(object, x,...)
                               standardGeneric("liesInSupport"))
if(!isGeneric("convpow")) 
    setGeneric("convpow", function(D1, ...) standardGeneric("convpow"))

if(!isGeneric("getLow")) 
    setGeneric("getLow", function(object, ...) standardGeneric("getLow"))

if(!isGeneric("getUp")) 
    setGeneric("getUp", function(object, ...) standardGeneric("getUp"))

# general methods

if(!isGeneric("isOldVersion")) 
   setGeneric("isOldVersion", function(object) standardGeneric("isOldVersion"))

if(!isGeneric("conv2NewVersion")) 
   setGeneric("conv2NewVersion", 
               function(object) standardGeneric("conv2NewVersion"))
### setting gaps

if(!isGeneric("setgaps"))
   setGeneric("setgaps", function(object, ...) standardGeneric("setgaps"))

#### generics for log, log10, lgamma, gamma, digamma


if(getRversion()<'2.9.0'){
if(!isGeneric("log"))
   setGeneric("log") #, function(x, base) standardGeneric("log"))
if(!isGeneric("log10"))
   setGeneric("log10")
if(!isGeneric("lgamma"))
   setGeneric("lgamma")
if(!isGeneric("digamma"))
   setGeneric("digamma")
if(!isGeneric("gamma"))
   setGeneric("gamma")
if(!isGeneric("sign"))
   setGeneric("sign") #, function(x, base) standardGeneric("log"))
}

### new Generics from 2.0 on (in particular for Mixing Distributions)

# Accessor / Replacement Functions for UnivarMixingDistribution
if(!isGeneric("mixCoeff"))
   setGeneric("mixCoeff", function(object) standardGeneric("mixCoeff"))
if(!isGeneric("mixCoeff<-"))
   setGeneric("mixCoeff<-", function(object, value) standardGeneric("mixCoeff<-"))

if(!isGeneric("mixDistr"))
   setGeneric("mixDistr", function(object) standardGeneric("mixDistr"))
if(!isGeneric("mixDistr<-"))
   setGeneric("mixDistr<-", function(object, value) standardGeneric("mixDistr<-"))

# Accessor / Replacement Functions for [AffLin]UnivarLebDecDistribution

if(!isGeneric("discretePart"))
    setGeneric("discretePart", function(object) standardGeneric("discretePart"))
if(!isGeneric("discretePart<-"))
   setGeneric("discretePart<-", function(object, value) standardGeneric("discretePart<-"))

if(!isGeneric("acPart"))
   setGeneric("acPart", function(object) standardGeneric("acPart"))
if(!isGeneric("acPart<-"))
   setGeneric("acPart<-", function(object, value) standardGeneric("acPart<-"))

if(!isGeneric("discreteWeight"))
   setGeneric("discreteWeight", function(object) standardGeneric("discreteWeight"))
if(!isGeneric("discreteWeight<-"))
   setGeneric("discreteWeight<-", function(object, value) standardGeneric("discreteWeight<-"))

if(!isGeneric("acWeight"))
   setGeneric("acWeight", function(object) standardGeneric("acWeight"))
if(!isGeneric("acWeight<-"))
   setGeneric("acWeight<-", function(object, value) standardGeneric("acWeight<-"))

if(!isGeneric("p.discrete"))
    setGeneric("p.discrete", function(object, ...) standardGeneric("p.discrete"))
if(!isGeneric("d.discrete"))
    setGeneric("d.discrete", function(object, ...) standardGeneric("d.discrete"))
if(!isGeneric("q.discrete"))
    setGeneric("q.discrete", function(object) standardGeneric("q.discrete"))
if(!isGeneric("r.discrete"))
    setGeneric("r.discrete", function(object) standardGeneric("r.discrete"))

if(!isGeneric("p.ac"))
    setGeneric("p.ac", function(object, ...) standardGeneric("p.ac"))
if(!isGeneric("d.ac"))
    setGeneric("d.ac", function(object, ...) standardGeneric("d.ac"))
if(!isGeneric("q.ac"))
    setGeneric("q.ac", function(object) standardGeneric("q.ac"))
if(!isGeneric("r.ac"))
    setGeneric("r.ac", function(object) standardGeneric("r.ac"))

### Help functions

if(!isGeneric("decomposePM"))
   setGeneric("decomposePM", function(object) standardGeneric("decomposePM"))

if(!isGeneric("simplifyD"))
   setGeneric("simplifyD", function(object) standardGeneric("simplifyD"))

if(!isGeneric("Truncate"))
   setGeneric("Truncate", function(object, ...) standardGeneric("Truncate"))

### Arithmetic functions / Min[Max]imum, Huberization, Truncation

if(!isGeneric("Minimum"))
    setGeneric("Minimum",
    function(e1, e2, ...) standardGeneric("Minimum"))
if(!isGeneric("Maximum"))
    setGeneric("Maximum",
    function(e1, e2, ...) standardGeneric("Maximum"))

if(!isGeneric("Huberize"))
   setGeneric("Huberize", function(object, ...) standardGeneric("Huberize"))

### help function for show

if(!isGeneric("showobj"))
   setGeneric("showobj", function(object, ...) standardGeneric("showobj"))


if(!isGeneric("NumbOfSummandsDistr"))
    setGeneric("NumbOfSummandsDistr", function(object) 
                standardGeneric("NumbOfSummandsDistr"))
if(!isGeneric("SummandsDistr"))
    setGeneric("SummandsDistr", function(object) 
                standardGeneric("SummandsDistr"))

if(!isGeneric(".lowerExact"))
    setGeneric(".lowerExact", function(object) 
                standardGeneric(".lowerExact"))

if(!isGeneric(".logExact"))
    setGeneric(".logExact", function(object) 
                standardGeneric(".logExact"))

if(!isGeneric("type")){
    setGeneric("type", function(object) standardGeneric("type"))
}
if(!isGeneric("SymmCenter")){
    setGeneric("SymmCenter", function(object) standardGeneric("SymmCenter"))
}
if(!isGeneric("Symmetry")){
    setGeneric("Symmetry", function(object) standardGeneric("Symmetry"))
}
if(!isGeneric("distribution")){
    setGeneric("distribution", function(object) standardGeneric("distribution"))
}
if(!isGeneric("samplesize")){
    setGeneric("samplesize", function(object, ...) standardGeneric("samplesize"))
}
if(!isGeneric("samplesize<-")){
    setGeneric("samplesize<-",
        function(object, value) standardGeneric("samplesize<-"))
}

################################
##
## Class: CauchyParameter
##
################################


## Access methods
setMethod("location", "CauchyParameter", function(object) object@location)
setMethod("scale", "CauchyParameter", 
           function(x, center = TRUE, scale = TRUE) x@scale)
           ### odd arg-list due to existing function in base package 

## Replace Methods
setReplaceMethod("location", "CauchyParameter", 
                  function(object, value){ object@location <- value; object})
setReplaceMethod("scale", "CauchyParameter", 
                  function(object, value){ object@scale <- value; object})

setValidity("CauchyParameter", function(object){
  if(length(location(object)) != 1)
    stop("location has to be a numeric of length 1")    
  if(length(scale(object)) != 1)
    stop("scale has to be a numeric of length 1")    
  if(scale(object) <= 0)
    stop("scale has to be positive")
  else return(TRUE)
})



################################
##
## Class: CauchyDistribution
##
################################

Cauchy <- function(location = 0, scale = 1) 
                   new("Cauchy", location = location, scale = scale)

## wrapped access methods
setMethod("location", "Cauchy", function(object) location(param(object)))
setMethod("scale", "Cauchy", 
           function(x, center = TRUE, scale = TRUE) scale(param(x)))
           ### odd arg-list due to existing function in base package 

## wrapped replace methods
setMethod("location<-", "Cauchy", 
           function(object, value) 
               new("Cauchy", location = value, scale = scale(object)))
setMethod("scale<-", "Cauchy", 
           function(object, value) 
               new("Cauchy", location = location(object), scale = value))

## convolution operator for normal distributions

setMethod("+", c("Cauchy","Cauchy"),
          function(e1,e2){
              new("Cauchy", scale = scale(e1) + scale(e2), 
                   location = location(e1) + location(e2),  .withArith = TRUE)
          })

## some exact arithmetic methods
setMethod("+", c("Cauchy","numeric"),
          function(e1, e2){
              if (length(e2)>1) 
                  stop("length of operator must be 1")
              Cauchy(location = location(e1) + e2, scale = scale(e1))  
          })

setMethod("*", c("Cauchy","numeric"),
          function(e1, e2){
            if (length(e2)>1) 
                stop("length of operator must be 1")
            if(isTRUE(all.equal(e2,0))) 
               Dirac(location = 0) 
            else                
               Cauchy(location = location(e1) * e2, 
                      scale = scale(e1) * abs(e2))  
          })

setMethod("solve", signature(a = "ANY", b = "ANY"), function(a,b, 
             generalized = getdistrOption("use.generalized.inverse.by.default"),
             tol = .Machine$double.eps, ...) {
          if(!generalized|is.null(dim(a))) return(base::solve(a,b, tol = tol, ...))
          else if(nrow(a)==ncol(a)){
             abtry <- try({
                 ab <- base::solve(a,b, tol = tol, ...)
                 if(missing(b)){
                    dimnames(ab) <-  rev(dimnames(a))
                 }else{
                    names(ab) <-  colnames(a)
                 }
                 return(ab)
             }, silent = TRUE)
             if(!is(abtry, "try-error")) return(abtry)
          }
          if (!missing(b))
              if(!(length(b)==nrow(a))) stop("non-conformable arguments")
          a.m <- MASS::ginv(a)
          dimnames(a.m) <- rev(dimnames(a))
          if (missing(b)) return(a.m)
              else return(a.m %*% b)
          })

setMethod("solve", signature(a = "PosSemDefSymmMatrix", b = "ANY"), 
           function(a,b, 
               generalized = getdistrOption("use.generalized.inverse.by.default"), 
               tol = .Machine$double.eps, ...){
          if(!generalized) return(base::solve(a,b, tol = tol, ...))
          else{
            er <- eigen(a)
            d1 <- er$values
            d <- 1/d1[d1 > tol]
            ev <- er$vectors[,d1 > tol]
            A <- if (length(d)) ev %*% (t(ev)*d) else 0*a
            dimnames(A) <- dimnames(a)
            if(missing(b)) return(A)
            else return(A%*%b)}   
})

setMethod("solve", signature(a = "PosDefSymmMatrix", b = "ANY"), function(a,b, ...){
base::solve(a,b, ...)})

setMethod("Huberize", "AcDcLcDistribution",
          function(object, lower, upper, withSimplify = getdistrOption("simplifyD"))
                {Mi <- Minimum( object, Dirac(upper),
                                withSimplify = withSimplify)
                 M0 <- Maximum(Dirac(lower),Mi,
                        withSimplify = withSimplify)
                 if(is(object@Symmetry,"SphericalSymmetry"))
                    if(.isEqual(lower+upper,2*SymmCenter(object@Symmetry))) 
                       M0@Symmetry <- SphericalSymmetry(SymmCenter(object@Symmetry))        
                 M0})

.trunc.up <- function(object, upper){
   ep <- .Machine$double.eps^2
   plN <- p(object)(upper, lower.tail = TRUE, log.p=TRUE)
   rnew <- function(n){
           q.l(object)(plN-rexp(n), lower.tail = TRUE, log.p=TRUE)
   }
   pnew <- function(q, lower.tail = TRUE, log.p = FALSE){
           indNA <- is.na(q)
           q[indNA] <- mean(q[!indNA])
           ind <- (q > upper-ep)
           p0 <- q*0
           p0[ind] <- if(lower.tail) 1 else 0
           if(log.p) p0[ind] <- log(p0[ind])
           q1 <- q[!ind]
           p1 <- p(object)(q1, lower.tail = TRUE,
                           log.p = TRUE) - plN
           p0[!ind] <- if(!log.p || !lower.tail) exp(p1) else p1
           if(!lower.tail) p0[!ind] <- 1-p0[!ind]
           if(log.p && !lower.tail) p0[!ind] <- log(p0[!ind])
           p0[indNA] <- NA
           return(p0)
   }
   dnew <- function(x, log = FALSE){
           indNA <- is.na(x)
           x[indNA] <- mean(x[!indNA])
           ind <- (x > upper-ep)
           d0 <- x*0
           d0[ind] <- 0
           if(log) d0[ind] <- log(d0[ind])
           x1 <- x[!ind]
           d1 <- d(object)(x1,  log = TRUE)-plN
           d0[!ind] <- if(log) d1 else exp(d1)
           d0[indNA] <- NA
           return(d0)
   }
   qnew <- function(p, lower.tail = TRUE, log.p = FALSE){
           indNA <- is.na(p)
           p[indNA] <- 0.5
           p0 <- if(log.p) exp(p) else p
           ind1 <- (p0 < -ep) | p0>1+ep
           indis0 <- .isEqual(p0,0,ep)
           indis1 <- .isEqual(p0,1,ep)
           in01 <- !(ind1|indis0|indis1)
           q0 <- 0*p
           q0[ind1] <- NA
           q0[indis1] <- if(lower.tail)
                                  upper else q.l(object)(0)
           q0[indis0] <- if(lower.tail)
                                  q.l(object)(0) else upper
           p1 <- p[in01]
           if(log.p && lower.tail) p1l <- plN + p1
           else{ if(log.p) p1 <- exp(p1)
                 p1l <- plN + if(lower.tail) log(p1) else log(1-p1)
                }
           q0[in01] <- q.l(object)(p1l, log.p = TRUE)
           q0[indNA] <- NA
           return(q0)

   }
   return(list(r=rnew,p=pnew,d=dnew,q=qnew))
}

.trunc.low <- function(object, lower){
   ep <- .Machine$double.eps^2
   if(is(object,"DiscreteDistribution")){
      Pl <- p.l(object)
      Qr <- q.r(object)
   }else{
      Pl <- p(object)
      Qr <- q.l(object)
   }
   plN <- Pl(lower,  lower.tail = FALSE, log.p = TRUE)
   rnew <- function(n){
           Qr(plN-rexp(n), lower.tail = FALSE, log.p = TRUE)
   }
   pnew <- function(q, lower.tail = TRUE, log.p = FALSE){
           indNA <- is.na(q)
           q[indNA] <- mean(q[!indNA])
           ind <- (q < lower+ep)
           p0 <- q*0
           p0[ind] <- if(lower.tail) 0 else 1
           if(log.p) p0[ind] <- log(p0[ind])
           q1 <- q[!ind]
           p1 <- p(object)(q1, lower.tail=FALSE,
                           log.p = TRUE)-plN
           p0[!ind] <- if(!log.p || lower.tail) exp(p1) else p1
           if(lower.tail) p0[!ind] <- 1-p0[!ind]
           if(log.p && lower.tail) p0[!ind] <- log(p0[!ind])
           p0[indNA] <- NA
           return(p0)
   }
   dnew <- function(x, log = FALSE){
           indNA <- is.na(x)
           x[indNA] <- mean(x[!indNA])
           ind <- (x < lower+ep)
           d0 <- x*0
           d0[ind] <- 0
           if(log) d0[ind] <- log(d0[ind])
           x1 <- x[!ind]
           d1 <- d(object)(x1,  log = TRUE)-plN
           d0[!ind] <- if(log) d1 else exp(d1)
           d0[indNA] <- NA
           return(d0)
   }
   qnew <- function(p, lower.tail = TRUE, log.p = FALSE){
           indNA <- is.na(p)
           p[indNA] <- 0.5
           p0 <- if(log.p) exp(p) else p
           ind1 <- (p0 < -ep) | p0>1+ep
           indis0 <- .isEqual(p0,0,ep)
           indis1 <- .isEqual(p0,1,ep)
           in01 <- !(ind1|indis0|indis1)
           q0 <- 0*p
           q0[ind1] <- NA
           q0[indis1] <- if(lower.tail)
                                  q.l(object)(1) else lower
           q0[indis0] <- if(lower.tail)
                                  lower else q.l(object)(1)
           p1 <- p[in01]
           if(log.p && !lower.tail) p1l <- plN + p1
           else{ if(log.p) p1 <- exp(p1)
                 p1l <- plN + if(lower.tail) log(1-p1) else log(p1) }
           q0[in01] <- q.l(object)(p1l, lower.tail = FALSE, log.p = TRUE)
           q0[indNA] <- NA
           return(q0)
   }
   return(list(r=rnew,p=pnew,d=dnew,q=qnew))
}

UnivarLebDecDistribution <- function(acPart, discretePart, acWeight, discreteWeight,
                                     r = NULL, e = NULL, n = NULL, y = NULL){

    if(missing(discretePart) && missing(acPart) && missing(r) )
       stop("at least one of 'discretePart' resp. 'acPart'  resp. 'r' must be given")
    if(missing(discretePart) && missing(acPart))
       {if(missing(e)) e <- getdistrOption("RtoDPQ.e")
        if(missing(n)) n <- getdistrOption("DefaultNrGridPoints")
        return(RtoDPQ.LC(r = r, e = e, n = n, y = y))}

    if(missing(discretePart)) {discreteWeight <- 0; acWeight <- 1;
                               discretePart <- new("Dirac")}
    if(missing(acPart)) {discreteWeight <- 1; acWeight <- 0;
                        acPart <- new("Norm")}

    if(!missing(discretePart)&&!missing(acPart)){
       if(missing(discreteWeight) && missing(acWeight))
             stop("at least one of 'discreteWeight' resp. 'acWeight' must be given")
       if(missing(discreteWeight)) discreteWeight <- 1-acWeight
       if(missing(acWeight)) acWeight <- 1-discreteWeight
       if(discreteWeight <0 || acWeight<0 || acWeight+discreteWeight>1)
             stop("no proper weights given")
    }

## PR 2018 04 13
## detected by Tuomo.OJALA@3ds.com:
## in a loop the names of slots acWeight, discreteWeight will grow;
## fix this by setting the prior names to NULL
    names(acWeight) <- NULL
    names(discreteWeight) <- NULL

    if(discreteWeight > 1 - getdistrOption("TruncQuantile"))
       {return(
           new("UnivarLebDecDistribution", p = discretePart@p,
                r = discretePart@r, d = NULL, q = discretePart@q,
                mixCoeff = c(acWeight = 0, discreteWeight = 1),
                mixDistr = new("UnivarDistrList", list(acPart = acPart,
                                discretePart = discretePart)),
                support = support(discretePart),
                gaps = gaps(acPart)                
              )
             )}
    if(discreteWeight < getdistrOption("TruncQuantile"))
       return(
           new("UnivarLebDecDistribution", p = acPart@p,
                r = acPart@r, d = NULL, q = acPart@q,
                mixCoeff = c(acWeight = 1, discreteWeight = 0),
                mixDistr = new("UnivarDistrList", list(acPart = acPart,
                                discretePart = discretePart)),
                support = support(discretePart),
                gaps = gaps(acPart)
              )
             )

    mixDistr <- new("UnivarDistrList", list(acPart = acPart,
                     discretePart = discretePart))
    mixCoeff <- c(acWeight = acWeight, discreteWeight = discreteWeight)

    rnew <- function(n)
             {U <- rbinom(n, size = 1, prob = acWeight)
              AC <- acPart@r(n); DISCRETE <- discretePart@r(n)
              U*AC+(1-U)*DISCRETE}

    pnew <- function(q, lower.tail = TRUE, log.p = FALSE )
                 {p <- cbind(acPart@p(q, lower.tail = lower.tail),
                         discretePart@p(q, lower.tail = lower.tail))
                  p <- as.vector(p%*%mixCoeff)
                  if(log.p) p <- log(p)
                  return(p)
                 }

    supp <- discretePart@support
    gaps <- .mergegaps(acPart@gaps,discretePart@support)
    #mixDistr[[1]]@gaps <- gaps

    qL1 <- min(getLow(acPart), getLow(discretePart))
    qU1 <- max(getUp(acPart), getUp(discretePart))
    n <- getdistrOption("DefaultNrGridPoints")
    h <- (qU1-qL1)/n
    ep <- getdistrOption("DistrResolution")
    xseq <- unique(sort(c(seq(from = qL1, to = qU1, by = h),gaps-ep,gaps,
                   gaps+ep,supp-ep,supp, supp+ep )))
    px.l <- pnew(xseq, lower.tail = TRUE)
    px.u <- pnew(xseq, lower.tail = FALSE)

    qL2 <- min(acPart@q(0),discretePart@q(0))
    qU2 <- max(acPart@q(1),discretePart@q(1))

    qnew <- .makeQNew(xseq, px.l, px.u, FALSE, qL2, qU2)
    .withSim <- acPart@.withSim | discretePart@.withSim
    .withArith <- acPart@.withArith | discretePart@.withArith
    .lowerExact <- .lowerExact(acPart) & .lowerExact(discretePart)

    Symmetry <- NoSymmetry()
    if(is(acPart@Symmetry,"SphericalSymmetry")&& 
       is(discretePart@Symmetry,"SphericalSymmetry"))
       if(.isEqual(SymmCenter(Symmetry(acPart)),
                   SymmCenter(Symmetry(discretePart))))
       Symmetry <- SphericalSymmetry(SymmCenter(Symmetry(acPart)))   



    new("UnivarLebDecDistribution", p = pnew, r = rnew, d = NULL, q = qnew,
         mixCoeff = mixCoeff, mixDistr = mixDistr, .withSim = .withSim,
         .withArith = .withArith, .lowerExact = .lowerExact, support = supp,
         gaps = gaps, Symmetry = Symmetry)
}

############################## Accessor / Replacement functions

setMethod("discretePart", "UnivarLebDecDistribution",
           function(object) object@mixDistr[[2]])
setReplaceMethod("discretePart", "UnivarLebDecDistribution",
          function(object,value){
          obj <- UnivarLebDecDistribution(acPart = acPart(object),
                 discretePart = value,
                 acWeight = acWeight(object),
                 discreteWeight = discreteWeight(object))
          obj})

setMethod("discreteWeight", "UnivarLebDecDistribution",
           function(object) object@mixCoeff[2])
setReplaceMethod("discreteWeight", "UnivarLebDecDistribution",
          function(object,value){
          obj <- UnivarLebDecDistribution(acPart = acPart(object),
                 discretePart = discretePart(object),
                 acWeight = 1-value,
                 discreteWeight = value)
          obj})

setMethod("acPart", "UnivarLebDecDistribution",
           function(object) object@mixDistr[[1]])
setReplaceMethod("acPart", "UnivarLebDecDistribution",
          function(object,value){
          obj <- UnivarLebDecDistribution(acPart = value,
                 discretePart = discretePart(object),
                 acWeight = acWeight(object),
                 discreteWeight = discreteWeight(object))
          obj})

setMethod("acWeight", "UnivarLebDecDistribution",
           function(object) object@mixCoeff[1])
setReplaceMethod("acWeight", "UnivarLebDecDistribution",
          function(object,value){
          obj <- UnivarLebDecDistribution(acPart = acPart(object),
                 discretePart = discretePart(object),
                 acWeight = value,
                 discreteWeight = 1-value)
          obj})

#setMethod("support", "UnivarLebDecDistribution",
#           function(object) object@mixDistr[[2]]@support)

#setMethod("gaps", "UnivarLebDecDistribution",
#           function(object) object@mixDistr[[1]]@gaps)


setMethod("p.discrete", "UnivarLebDecDistribution",
           function(object, CondOrAbs="cond"){ 
                  CondOrAbs0 <- pmatch(CondOrAbs,c("cond","abs"),nomatch=1)
                  pd <- object@mixDistr[[2]]@p
                  if(CondOrAbs0==1)
                       return(pd)
                  else {wd <- discreteWeight(object)
                        return(function(q, lower.tail = TRUE, log.p = FALSE ){
                               wd * pd(q, lower.tail = lower.tail, log.p = log.p)
                        })
                  }   
           })
setMethod("d.discrete", "UnivarLebDecDistribution",
           function(object, CondOrAbs="cond"){ 
                  CondOrAbs0 <- pmatch(CondOrAbs,c("cond","abs"),nomatch=1)
                  dd <- object@mixDistr[[2]]@d
                  if(CondOrAbs0==1)
                       return(dd)
                  else {wd <- discreteWeight(object)
                        return(function(x, log = FALSE ){
                               wd * dd(x, log = log)
                        })
                  }   
           })
setMethod("q.discrete", "UnivarLebDecDistribution",
           function(object) object@mixDistr[[2]]@q)
setMethod("r.discrete", "UnivarLebDecDistribution",
           function(object) object@mixDistr[[2]]@r)
setMethod("p.ac", "UnivarLebDecDistribution",
           function(object, CondOrAbs="cond"){ 
                  CondOrAbs0 <- pmatch(CondOrAbs,c("cond","abs"),nomatch=1)
                  pd <- object@mixDistr[[1]]@p
                  if(CondOrAbs0==1)
                       return(pd)
                  else {wa <- acWeight(object)
                        return(function(q, lower.tail = TRUE, log.p = FALSE ){
                               wa * pd(q, lower.tail = lower.tail, log.p = log.p)
                        })
                  }   
           })
setMethod("d.ac", "UnivarLebDecDistribution",
           function(object, CondOrAbs="cond"){ 
                  CondOrAbs0 <- pmatch(CondOrAbs,c("cond","abs"),nomatch=1)
                  dd <- object@mixDistr[[1]]@d
                  if(CondOrAbs0==1)
                       return(dd)
                  else {wa <- acWeight(object)
                        return(function(x, log = FALSE ){
                               wa * dd(x, log = log)
                        })
                  }   
           })
setMethod("q.ac", "UnivarLebDecDistribution",
           function(object) object@mixDistr[[1]]@q)
setMethod("r.ac", "UnivarLebDecDistribution",
           function(object) object@mixDistr[[1]]@r)



setMethod("p.l", "UnivarLebDecDistribution", function(object){
           ep <- getdistrOption("TruncQuantile")
           w.d <- discreteWeight(object)
           w.a <- acWeight(object)   
           if(w.d<ep) return(p(object))
           mixCoeff <- c(w.a,w.d)
           p.a <- p(acPart(object))
           p.d <- p.l(discretePart(object))
           return(function(q, lower.tail = TRUE, log.p = FALSE){
                  p <- cbind(p.a(q, lower.tail = lower.tail),
                             p.d(q, lower.tail = lower.tail))
                  p <- as.vector(p%*%mixCoeff)
                  if(log.p) p <- log(p)
                  return(p)
                 })
       })

### right continuous quantile function

setMethod("q.r", "UnivarLebDecDistribution", function(object){
    ep <- getdistrOption("TruncQuantile")
    if(discreteWeight(object)<ep) return(q.l(object))
    supp <- support(object)
    gaps <- gaps(object)
    aP <- acPart(object)
    dP <- discretePart(object)
    pl <- p.l(object)
    qL1 <- min(getLow(aP), getLow(dP))
    qU1 <- max(getUp(aP), getUp(dP))
    n <- getdistrOption("DefaultNrGridPoints")
    h <- (qU1-qL1)/n
    xseq <- unique(sort(c(seq(from = qL1, to = qU1, by = h),
                   gaps-ep,gaps,gaps+ep,
                   supp-ep,supp, supp+ep )))
    px.l <- pl(q=xseq, lower.tail = TRUE)
    px.u <- pl(q=xseq, lower.tail = FALSE)

    qL2 <- min(aP@q(0),dP@q(0))
    qU2 <- max(aP@q(1),dP@q(1))

    return( .makeQNew(xseq, px.l, px.u, FALSE, qL2, qU2))
})






setMethod("prob", "UnivarLebDecDistribution", 
function(object) {pr0 <- prob(as(object@mixDistr[[2]],"DiscreteDistribution"))
                  d <- discreteWeight(object)
                  return(rbind("cond"=pr0,"abs"=d*pr0))})
                  


############################## setAs relations

setAs("AbscontDistribution", "UnivarLebDecDistribution",
       function(from) UnivarLebDecDistribution(acPart = from))

setAs("DiscreteDistribution", "UnivarLebDecDistribution",
       function(from) UnivarLebDecDistribution(discretePart = from))



############################## Arithmetics

setMethod("+", c("UnivarLebDecDistribution","UnivarLebDecDistribution"),
function(e1,e2){
         ep <- getdistrOption("TruncQuantile")

         simple <- 0

         if(discreteWeight(e1)>1-ep)
            {simple <- 1
             e10 <- discretePart(e1)
             if(is(e10,"Dirac"))
               return(e2+location(as(e10,"Dirac")))}
         if(discreteWeight(e1) < ep)
            {simple <- 1
             e10 <- acPart(e1)}

         if(discreteWeight(e2)>1-ep)
            {simple <- simple + 1
             e20 <- discretePart(e2)
             if(is(e20,"Dirac"))
               return(e1+location(as(e20,"Dirac")))}
         if(discreteWeight(e2) < ep)
            {simple <- simple + 1
             e20 <- acPart(e2)}

         if(simple == 2)
            return(as(e10+e20,"UnivarLebDecDistribution"))

         p1.a <- acPart(e1)
         p2.a <- acPart(e2)
         p1.d <- discretePart(e1)
         p2.d <- discretePart(e2)
         w1.a <- acWeight(e1)
         w2.a <- acWeight(e2)
         w1.d <- discreteWeight(e1)
         w2.d <- discreteWeight(e2)


         f1 <- if(w1.a*w2.a>ep)  p1.a + p2.a else Norm()
         f2 <- if(w1.a*w2.d>ep)  p1.a + p2.d else Norm()
         if (identical(e1,e2)) f3 <- f2
         else {f3 <- if(w1.d*w2.a>ep)  p1.d + p2.a else Norm()}
         f.d <- if(w1.d*w2.d>ep) p1.d + p2.d else Dirac(0)


         w.d <- w2.d * w1.d
         w.a <- 1 - w.d

         rnew <- function(n) {
                 U <- sample(1:3, size = n, replace = TRUE, prob =
                             c(w1.a*w2.a, w1.a*w2.d, w1.d*w2.a))
                 R1 <- r(f1)(n)
                 R2 <- r(f2)(n)
                 R3 <- r(f3)(n)
                 (U==1)*R1 + (U==2)*R2 + (U==3)*R3
                 }
         pnew <- function(q, lower.tail = TRUE, log.p = FALSE)
                     {p1 <- w1.a * w2.a *
                         p(f1)(q, lower.tail = lower.tail, log.p = log.p)
                      p2 <- w1.a * w2.d *
                         p(f2)(q, lower.tail = lower.tail, log.p = log.p)
                      p3 <- w1.d * w2.a *
                         p(f3)(q, lower.tail = lower.tail, log.p = log.p)
                      (p1 + p2 + p3)/w.a
                      }
         dnew <- function(x, log = FALSE)
                     {(w1.a * w2.a * d(f1)(x, log = log)+
                       w1.a * w2.d * d(f2)(x, log = log)+
                       w1.d * w2.a * d(f3)(x, log = log)) / w.a }

         qL2 <- min(f1@q(0),f2@q(0),f3@q(0))
         qU2 <- max(f1@q(1),f2@q(1),f3@q(1))

         qL <-  min(getLow(f1), getLow(f2), getLow(f3))
         qU <-  max(getUp(f1), getUp(f2), getUp(f3))
         n <- getdistrOption("DefaultNrGridPoints")
         h <- (qU-qL)/n
         xseq <- seq(from = qL, to = qU, by = h)
         px.l <- pnew(xseq, lower.tail = TRUE)
         px.u <- pnew(xseq, lower.tail = FALSE)

         qnew <- .makeQNew(xseq, px.l, px.u, FALSE, qL2, qU2)

         f.a <- AbscontDistribution(
                    r = rnew, p = pnew, q = qnew, d = dnew, .withSim = FALSE,
                    .withArith = TRUE)

         obj <- UnivarLebDecDistribution(discretePart = f.d, acPart = f.a,
                                  discreteWeight = w.d)
         if(getdistrOption("simplifyD"))
            obj <- simplifyD(obj)

         return(obj)
         })

setMethod("*", c("UnivarLebDecDistribution","numeric"),
          function(e1, e2) {
          if (length(e2)>1) stop("length of operator must be 1")

          if (isTRUE(all.equal(e2,1))) return(e1)
          if (isTRUE(all.equal(e2,0)))
               return(new("Dirac", location = 0))

          Distr <- UnivarLebDecDistribution(
                     discretePart = discretePart(e1)*e2,
                     acPart = acPart(e1)*e2,
                     discreteWeight = discreteWeight(e1),
                     acWeight = acWeight(e1))

          object <- new("AffLinUnivarLebDecDistribution",
                    r = Distr@r, d = Distr@d, p = Distr@p,
                    q = Distr@q, X0 = e1, mixDistr = Distr@mixDistr,
                    mixCoeff = Distr@mixCoeff,
                    a = e2, b = 0, .withSim  = e1@.withSim,
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1),
                    .withArith = TRUE, support= support(Distr),
                    gaps = gaps(Distr))
          object})

setMethod("+", c("UnivarLebDecDistribution","numeric"),
          function(e1, e2) {
          if (length(e2)>1) stop("length of operator must be 1")
          if (isTRUE(all.equal(e2,0))) return(e1)

          Distr <- UnivarLebDecDistribution(
                     discretePart = discretePart(e1)+e2,
                     acPart = acPart(e1)+e2,
                     discreteWeight = discreteWeight(e1),
                     acWeight = acWeight(e1))

          object <- new("AffLinUnivarLebDecDistribution",
                    r = Distr@r, d = Distr@d, p = Distr@p,
                    q = Distr@q, X0 = e1, mixDistr = Distr@mixDistr,
                    mixCoeff = Distr@mixCoeff,
                    a = 1, b = e2, .withSim  = e1@.withSim,
                    .withArith = TRUE, support= support(Distr),
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1),
                    gaps = gaps(Distr))
          object})

#setMethod("*", c("numeric","UnivarLebDecDistribution"),
#          function(e1, e2) e2*e1)
#setMethod("+", c("numeric","UnivarLebDecDistribution"),
#          function(e1, e2) e2+e1)


## Group Math for absolutly continuous distributions
setMethod("Math", "UnivarLebDecDistribution",
          function(x){
            rnew <- function(n, ...){}
            body(rnew) <- substitute({ f(g(n, ...)) },
                              list(f = as.name(.Generic), g = x@r))
            object <- UnivarLebDecDistribution(r = rnew)

            if(getdistrOption("simplifyD"))
               object <- simplifyD(object)
            object
          })

## ??? setGroupGeneric("exFct", def=function(object) standardGeneric("exFct"), group = list(exp, abs,...))

## exact: abs for absolutly continuous distributions
setMethod("abs", "UnivarLebDecDistribution",
          function(x)UnivarLebDecDistribution(
                     discretePart = abs(discretePart(x)),
                     acPart = abs(acPart(x)),
                     discreteWeight = discreteWeight(x),
                     acWeight = acWeight(x)))

## exact: exp for absolutly continuous distributions
setMethod("exp", "UnivarLebDecDistribution",
          function(x){
           if(acWeight(x)>1-getdistrOption("TruncQuantile"))
              {x <- force(acPart(x))
               return(as(exp(x),"UnivarLebDecDistribution"))}
           if(acWeight(x) < getdistrOption("TruncQuantile"))
              {x <- force(discretePart(x))
               return(as(exp(x),"UnivarLebDecDistribution"))}
           return(UnivarLebDecDistribution(
                     discretePart = exp(discretePart(x)),
                     acPart = exp(acPart(x)),
                     discreteWeight = discreteWeight(x),
                     acWeight = acWeight(x)))})


setMethod("log", "UnivarLebDecDistribution",
          function(x, base = exp(1)){
           xs <- as.character(deparse(match.call(
                 call = sys.call(sys.parent(1)))$x))
           ep <- getdistrOption("TruncQuantile")
           basl <- log(base)
           if(p(x)(0)>ep) 
                stop(gettextf("log(%s) is not well-defined with positive probability ", xs))
           
           if(acWeight(x)>1-getdistrOption("TruncQuantile")) 
              {x <- force(acPart(x))
               return(as(log(x, base = base),"UnivarLebDecDistribution"))}    
           if(acWeight(x) < getdistrOption("TruncQuantile")) 
              {x <- force(discretePart(x))
               return(as(log(x, base = base),"UnivarLebDecDistribution"))}    
           return(UnivarLebDecDistribution(
                     discretePart = log(discretePart(x), base = base),
                     acPart = log(acPart(x), base = base),
                     discreteWeight = discreteWeight(x),
                     acWeight = acWeight(x)))})

setMethod("log10", "UnivarLebDecDistribution",
          function(x) log(x = x, base = 10))

setMethod("sign", "UnivarLebDecDistribution",
          function(x){ 
          d0 <- d.discrete(as(x,UnivarLebDecDistribution))(0)
          DiscreteDistribution(supp=c(-1,0,1), 
              prob=c(p(x)(-getdistrOption("TruncQuantile")),
                     d0,
                     p(x)(getdistrOption("TruncQuantile"), lower=FALSE)))                     
          })

setMethod("sqrt", "UnivarLebDecDistribution",
            function(x) x^0.5)
#######################

################################################################################
#
#                  LatticeDistribution
#
################################################################################

LatticeDistribution <- function(lattice = NULL, supp = NULL, prob = NULL, 
                       .withArith = FALSE, .withSim = FALSE, 
                       DiscreteDistribution = NULL, check = TRUE,
                       Symmetry = NoSymmetry()){
    if(is(lattice,"Lattce")){
       if(width(lattice)>0){
          .finS <- c(TRUE,is.finite(Length(lattice)))
       }else{
          .finS <- c(is.finite(Length(lattice)), TRUE)
       }
    }else .finS <- c(TRUE,TRUE)
    if (is(DiscreteDistribution, "AffLinDiscreteDistribution"))
        {  D <- DiscreteDistribution
           if (is(lattice, "Lattice")) 
             { ### check consistency with support of DiscreteDistribution} 
              if (check){
                 if( !.is.consistent(lattice, support(D), eq.space = FALSE))         
                     stop(paste("Argument 'lattice' is inconsistent to",
                            " the support of argument 'DiscreteDistribution'." , 
                            sep = ""))
              }           
              return(new("AffLinLatticeDistribution", r = D@r, d = D@d,
                          q = D@q, p = D@p, support = D@support, 
                          a = D@a, b = D@b, X0 = D@X0,
                          lattice = lattice, .withArith = .withArith, 
                          .withSim = .withSim, img = D@img,
                          param = D@param, Symmetry = Symmetry,
                          .finSupport = .finS))
              }else{
               if (check){
                   if( !.is.vector.lattice(support(D)))
                       stop(paste("Support of argument 'DiscreteDistribution' ",
                              "is not a lattice.", sep = ""))
               }           
               return(new("AffLinLatticeDistribution", r = D@r, d = D@d, 
                          q = D@q, p = D@p, support = D@support, 
                          lattice = .make.lattice.es.vector(D@support), 
                          a = D@a, b = D@b, X0 = D@X0,
                          .withArith = .withArith, 
                          .withSim = .withSim, img = D@img,
                          param = D@param, Symmetry = Symmetry,
                          .finSupport = .finS))
              }                 
        }

    if (is(DiscreteDistribution, "DiscreteDistribution"))
        {  D <- DiscreteDistribution
           if (is(lattice, "Lattice")) 
             { ### check consistency with support of DiscreteDistribution} 
              if (check){
                  if( !.is.consistent(lattice, support(D), eq.space = FALSE))         
                     stop(paste("Argument 'lattice' is inconsistent to the",
                            " support of argument 'DiscreteDistribution'." , 
                            sep = ""))
              }           
              return(new("LatticeDistribution", r = D@r, d = D@d, 
                          q = D@q, p = D@p, support = D@support, 
                          lattice = lattice, .withArith = .withArith, 
                          .withSim = .withSim, img = D@img,
                          param = D@param, Symmetry = Symmetry,
                          .finSupport = .finS))
              }else{
               if (check){
                   if( !.is.vector.lattice(support(D)))
                     stop(paste("Support of argument 'DiscreteDistribution' is",
                              "not a lattice.", sep = " "))
               }           
 
               return(new("LatticeDistribution", r = D@r, d = D@d, 
                          q = D@q, p = D@p, support = D@support, 
                          lattice = .make.lattice.es.vector(D@support), 
                          .withArith = .withArith, 
                          .withSim = .withSim, img = D@img,
                          param = D@param, Symmetry = Symmetry,
                          .finSupport = .finS))
              }                 
        }

    if (is(lattice, "Lattice") && ! is.null(supp))
       {D <- DiscreteDistribution(supp = supp, prob = prob, 
                                   .withArith = .withArith, 
                                   .withSim = .withSim, Symmetry = Symmetry )
        
        if (check){
            if( !.is.consistent(lattice, supp, eq.space = FALSE))         
                stop("Argument 'lattice' is inconsistent to argument 'supp'.")
        }
        
        return(new("LatticeDistribution", r = r(D), d = d(D), 
                    q = q.l(D), p = p(D), support = supp,
                    lattice = lattice, .withArith = .withArith, 
                    .withSim = .withSim, Symmetry = Symmetry,
                    .finSupport = .finS))
       }

    if (is(lattice, "Lattice"))
       {if (is.finite(Length(lattice)))
             {if (is.null(prob))
                  prob <- rep(1/Length(lattice), Length(lattice))
              if (Length(lattice) == length(prob))
                 {supp <- seq( pivot(lattice), length = Length(lattice), 
                               by = width(lattice))
                  D <- DiscreteDistribution(supp = supp, prob = prob, 
                                             .withArith = .withArith, 
                                             .withSim = .withSim, 
                                             Symmetry = Symmetry )
                  return(new("LatticeDistribution", r = r(D), d = d(D), 
                          q = q.l(D), p = p(D), support = supp,
                          lattice = lattice, .withArith = .withArith, 
                          .withSim = .withSim, Symmetry = Symmetry,
                          .finSupport = .finS))
                  }else{ 
                   #if (check)
                       stop("Lengths of lattice and probabilities differ.")
                   #else return(D)
                   }    
              }else {if (is.null(prob))
                        stop(paste("Insufficient information given to ",
                                   "determine distribution.", sep = ""))
                     else{
                         supp <- seq( pivot(lattice), length = length(prob), 
                                     by = width(lattice))
                         D <- DiscreteDistribution(supp = supp, prob = prob, 
                                                   .withArith = .withArith, 
                                                   .withSim = .withSim, 
                                                   Symmetry = Symmetry )
                         return(new("LatticeDistribution", r = r(D), d = d(D), 
                                q = q.l(D), p = p(D), support = supp,
                                lattice = lattice, .withArith = .withArith, 
                                .withSim = .withSim, Symmetry = Symmetry,
                                .finSupport = .finS))
                        }                  
             }
       }else if (!is.null(supp))
            {if (is.null(prob)) prob <- supp*0+1/length(supp)
             D <- DiscreteDistribution(supp, prob, .withArith = .withArith, 
                                       .withSim = .withSim, Symmetry = Symmetry )
             if (check){
                 if (!.is.vector.lattice (supp))
                     stop("Argument 'supp' given is not a lattice.")
             }    
             return(new("LatticeDistribution", r = D@r, d = D@d, 
                             q = D@q, p = D@p, support = D@support, 
                             lattice = .make.lattice.es.vector(D@support), 
                             .withArith = D@.withArith, 
                             .withSim = D@.withSim, img = D@img,
                             param = D@param, Symmetry = Symmetry,
                            .finSupport = .finS))
            }else 
             stop("Insufficient information given to determine distribution.")
}


setMethod("lattice", "LatticeDistribution", function(object) object@lattice)


## canceling out of lattice points with mass 0
#setAs("LatticeDistribution", "DiscreteDistribution", 
#       def = function(from){
#    cF <- class(from)[1]
#    value <- if (cF!="LatticeDistribution") 
#                 new(cF) else new("DiscreteDistribution")
#    for (what in slotNames("DiscreteDistribution")) 
#         slot(value, what) <- slot(from, what)
#    supp.old <- from@support
#    o.warn <- getOption("warn"); options(warn = -2)
#    d.old <- from@d(from@support)
#    options(warn = o.warn)
#    supp.new <- supp.old[d.old > 0]
#    value@support <- supp.new
#    value
#       }
#)


setAs("AffLinLatticeDistribution","AffLinDiscreteDistribution", 
       def = function(from){
    value <- new("AffLinDiscreteDistribution")
    for (what in slotNames("AffLinDiscreteDistribution")) 
         slot(value, what) <- slot(from, what)
    supp.old <- from@support
    o.warn <- getOption("warn"); options(warn = -2)
    on.exit(options(warn=o.warn))
    d.old <- from@d(from@support)
    supp.new <- supp.old[d.old > 0]
    options(warn = o.warn)
    value@support <- supp.new
    value
       }
)

setMethod("+", c("LatticeDistribution", "LatticeDistribution"),
function(e1,e2){
            if(length(support(e1))==1) return(e2+support(e1))
            if(length(support(e2))==1) return(e1+support(e2))

### Lattice calculations:

            sup1 <- support(e1)
            sup2 <- support(e2)
            # left and right endpoint of convolution support
            su12.l <- sup1[1]+sup2[1]
            su12.r <- (rev(sup1))[1]+(rev(sup2))[1]

            l1 <- length(sup1)
            l2 <- length(sup2)

            lat1 <- lattice(e1)
            lat2 <- lattice(e2)
            L1 <- Length(lat1)
            L2 <- Length(lat2)
            w1 <- width(lat1)
            w2 <- width(lat2)


            ### take care if lattice is infinite
            L.inf <- !(is.finite(L1)&&is.finite(L2))
            if(L.inf){
               if(is.finite(L2)){
                  if(w1>0)
                     L.lr <- +1
                  else
                     L.lr <- -1   
               }else{    
                  if(is.finite(L1)){
                     if(w2>0)
                        L.lr <- +1
                     else
                        L.lr <- -1   
                  }else{
                     if(w1*w2>0) L.lr <- if(w1>0) +1 else -1
                     if(w1*w2<0) L.lr <- if(abs(su12.l)<abs(su12.r)) +1 else -1
                  }
               }
            }   


            e0 <- NULL
            tol0 <- .distroptions$DistrResolution/1000
            
            ## treat case separately when Discr + Discr is "faster"
            if(l1*l2 < 100){ 
                     d0 <- .convDiscrDiscr(e1,e2)
                     sup0 <- support(d0)
                     md <- min(diff(sup0))
                     sup00 <- seq(from=min(sup0),to=max(sup0),by=md)
                     sup0s <- intersect(sup00,sup0)
                     sup01 <- .inWithTol(sup00, sup0s, tol=tol0)
                     sup10 <- .inWithTol(sup0, sup0s, tol=tol0)
                     if(!all(sup10)) return(d0)
                     pr0 <- sup00*0
                     pr0[sup01] <- (prob(d0))[sup10]
                     pr0 <- pr0/sum(pr0)                              
                     lat <- Lattice(pivot = sup00[1], width = md, 
                                    Length = length(sup00))
                     e0 <- LatticeDistribution(supp = sup00, prob = pr0, 
                                               lattice = lat, check = FALSE)           
                     if(L.inf){
                         wa <- .getCommonWidth(abs(w1),abs(w2), tol=tol0)
                         e0@lattice <- if(L.lr>0){
                            Lattice(pivot = su12.l, width = wa, Length = Inf)
                                }else{ 
                            Lattice(pivot = su12.r, width = -wa, Length = Inf)}
                     }       
               }
 
            ## step 1 common width
              wa <- .getCommonWidth(abs(w1),abs(w2),
                      tol=tol0)


            ## treat case separately when no common support, i.e. when 
            ## w1/w2 is not "rational" enough
            
            if(is.null(wa))  return(.convDiscrDiscr(e1,e2))
            
            
            w0 <- ifelse(w1<0,-1,1) * wa
            pi1 <- pivot(lat1)
            pi2 <- pivot(lat2)                        
            ### Step 2
            supp0 <- seq(by = wa, from = min(sup1-pi1, sup2-pi2),
                                  to = max(sup1-pi1, sup2-pi2))
            s1 <- .inWithTol(supp0,sup1-pi1,tol0)
            s2 <- .inWithTol(supp0,sup2-pi2,tol0)
            d1 <- d2 <- 0*supp0
            d1[s1] <- prob(as(e1,"DiscreteDistribution"))
            d2[s2] <- prob(as(e2,"DiscreteDistribution"))

            L <- length(supp0)
            Ln <- 2^(ceiling(log(L)/log(2))+1)

            ### Step 3
            d1 <- c(d1, numeric(Ln-L))
            d2 <- c(d2, numeric(Ln-L))

            ##STEP 4
            ## computation of DFT
            ftde1 <- fft(d1)
            ftde2 <- fft(d2)

            ## convolution theorem for DFTs
            newd <- (Re(fft(ftde1*ftde2, inverse = TRUE)) / Ln)[1:(2*L+1)]
            newd <- (newd >= .Machine$double.eps^1.5)*newd


            ## reduction to relevant support
            supp1 <- seq(by = wa,
                         from = 2 * min(sup1-pi1, sup2-pi2),
                         to   = 2 * max(sup1-pi1, sup2-pi2))+pi1+pi2

            L1 <- length(supp1)
            newd <- newd[1:L1]

            if (L1 > getdistrOption("DefaultNrGridPoints")){
                rsum.u <- min( sum( rev(cumsum(rev(newd))) >=
                                    getdistrOption("TruncQuantile")/2)+1,
                               length(supp1)
                           )
                rsum.l <- 1 + sum( cumsum(newd) <
                                   getdistrOption("TruncQuantile")/2)
            }else{
                rsum.u <- min( sum( rev(cumsum(rev(newd))) >=
                                    .Machine$double.eps),
                               length(supp1)
                           )
                rsum.l <- 1 + sum( cumsum(newd) < .Machine$double.eps)

            }
            wi1 <- rsum.l:rsum.u
            newd <- newd[wi1]
            newd <- newd/sum(newd)
            supp1 <- supp1[wi1]

            wi2 <- newd > getdistrOption("TruncQuantile")
            supp2 <- supp1[wi2]
            newd2 <- newd[wi2]
            newd2 <- newd2/sum(newd2)

            Symmetry <- NoSymmetry()
            if(is(e1@Symmetry,"SphericalSymmetry")&& 
               is(e2@Symmetry,"SphericalSymmetry"))
               Symmetry <- SphericalSymmetry(SymmCenter(e1@Symmetry)+
                                              SymmCenter(e2@Symmetry))   

            if( length(supp1) >= 2 * length(supp2)){
               res <- DiscreteDistribution(supp = supp2, prob = newd2,
                                           .withArith = TRUE, Symmetry = Symmetry)
               res@.finSupport <- e1@.finSupport & e2@.finSupport
               return(res)
            }else{
               lat <- Lattice(pivot=supp1[1],width=wa, Length=length(supp1))

               e0 <- LatticeDistribution(supp = supp1, prob = newd,
                                         lattice = lat,
                                         .withArith = TRUE, Symmetry = Symmetry,
                                         check = FALSE)
               if(L.inf){
                  e0@lattice <- if(L.lr>0){ 
                            Lattice(pivot = su12.l, width = wa, Length = Inf) 
                         }else{ 
                            Lattice(pivot = su12.r, width = -wa, Length = Inf)}
               }
               e0@.finSupport <- e1@.finSupport & e2@.finSupport
               return(e0)
            }
          })

## extra methods
## binary operators

setMethod("+", c("LatticeDistribution", "numeric"),
          function(e1, e2) 
             {L <- lattice(e1)
              pivot(L) <- pivot(L) + e2
              Distr <- as(e1, "DiscreteDistribution") + e2 
              if(is(Distr, "AffLinDistribution"))
                    Distr@X0 <- e1

              Symmetry <- NoSymmetry()
              if(is(e1@Symmetry,"SphericalSymmetry"))
                 Symmetry <- SphericalSymmetry(SymmCenter(e1@Symmetry)+e2)   
              
              res <- LatticeDistribution(lattice = L,
                     DiscreteDistribution = Distr, Symmetry = Symmetry, 
                     check = FALSE)
              res@.finSupport <- e1@.finSupport & c(e2>(-Inf),(e2<Inf))
              return(res)
              })       

setMethod("*", c("LatticeDistribution", "numeric"),
          function(e1, e2) 
             {if (.isEqual(e2,0))
                  return(Dirac( location = 0 ))
              else{
                  L <- lattice(e1)
                  pivot(L) <- pivot(L) * e2
                  width(L) <- width(L) * e2
                  Distr <- as(e1, "DiscreteDistribution") * e2 
                  if(is(Distr, "AffLinDistribution"))
                        Distr@X0 <- e1
            
                  Symmetry <- NoSymmetry()
                  if(is(e1@Symmetry,"SphericalSymmetry"))
                     Symmetry <- SphericalSymmetry(SymmCenter(e1@Symmetry) * e2)   
              
                  res <- LatticeDistribution(lattice = L,
                          DiscreteDistribution = Distr, Symmetry = Symmetry, 
                          check = FALSE)
                  if(is.finite(e2)){
                      res@.finSupport <- e1@.finSupport
                  }else{
                      ep <- .Machine$double.eps
                      res@.finSupport <- c((p(e1)(0)<=  ep),(p(e1)(0)>=1-ep))
                  }
                  if(e2<0) res@.finSupport <- rev(res@.finSupport)
                  return(res)
                }
             }
          )              

setMethod("+", c("AffLinLatticeDistribution", "numeric"),
          function(e1, e2) 
             {L <- lattice(e1)
              pivot(L) <- pivot(L) + e2
              Symmetry <- NoSymmetry()
              if(is(e1@Symmetry,"SphericalSymmetry"))
                 Symmetry <- SphericalSymmetry(SymmCenter(e1@Symmetry) + e2)   
              res <- LatticeDistribution(lattice = L,
                     DiscreteDistribution = 
                        as(e1, "AffLinDiscreteDistribution") + e2,
                        Symmetry = Symmetry, 
                     check = FALSE)                     
              res@.finSupport <- e1@.finSupport & c(e2>(-Inf),(e2<Inf))
              return(res)
              })

setMethod("*", c("AffLinLatticeDistribution", "numeric"),
          function(e1, e2) 
             {if (isTRUE(all.equal(e2,0)))
                  return(Dirac( location = 0 ))
              else     
                { L <- lattice(e1)
                  pivot(L) <- pivot(L) * e2
                  width(L) <- width(L) * e2
                  Symmetry <- NoSymmetry()
                  if(is(e1@Symmetry,"SphericalSymmetry"))
                     Symmetry <- SphericalSymmetry(SymmCenter(e1@Symmetry) * e2)   
                  res <- LatticeDistribution(lattice = L,
                          DiscreteDistribution = 
                             as(e1, "AffLinDiscreteDistribution") * 
                             e2, Symmetry = Symmetry, 
                             check = FALSE)
                  if(is.finite(e2)){
                      res@.finSupport <- e1@.finSupport
                  }else{
                      ep <- .Machine$double.eps
                      res@.finSupport <- c((p(e1)(0)<=  ep),(p(e1)(0)>=1-ep))
                  }
                  if(e2<0) res@.finSupport <- rev(res@.finSupport)
                  return(res)
                }
             }
          )              

## to be exported: berechnet Konfidenzbänder, simultan und punktweise
qqbounds <- function(x,D,alpha,n,withConf.pw, withConf.sim,
                     exact.sCI=(n<100),exact.pCI=(n<100),
                     nosym.pCI = FALSE, debug = FALSE){
   x <- sort(unique(x))
   if("gaps" %in% names(getSlots(class(D))))
       {if(!is.null(gaps(D)))
            x <- sort(unique(c(x, gaps(D))))
       }
   c.c <- matrix(NA,nrow=length(x),ncol=4)
   colnames(c.c) <- c("sim.left","sim.right","pw.left","pw.right")

   SI <- .SingleDiscrete(x,D)
   SI.in <- SI<4
   SIi <- SI[SI.in]
   x.in <- x[SI.in]
   p.r <- p(D)(x.in)
   p.l <- p.l(D)(x.in)
   l.x <- length(x.in)
   if(debug){
     cat("the partition into discrete mass points and cont. points gives\n")
     print(SI)
     cat("x.in is\n")
     print(x.in)
     cat("number of mass points:\n")
     print(sum(SI.in))
     cat("p.r and p.l\n")
     print(cbind(p.r,p.l))
     cat("length of l.x\n")
     print(l.x)
     print(c(alpha=alpha,n=n,exact.sCI=exact.sCI))
   }

   silent0 <- !debug

   c.crit <- if(withConf.sim) try(.q2kolmogorov(alpha,n,exact.sCI, silent0),
                                   silent=silent0) else NULL
   c.crit.i <- if(withConf.pw) try(.q2pw(x.in,p.r,D,n,alpha,exact.pCI,nosym.pCI,
                                         silent0),silent=silent0) else NULL
   #print(cbind(c.crit,c.crit.i))
   if(debug){
      cat("returned c.crit is\n")
      print(str(c.crit))
      cat("returned c.crit.i is\n")
      print(str(c.crit.i))
   }
   te.i <- withConf.pw  & !is(c.crit.i,"try-error")
   te.s <- withConf.sim & !is(c.crit,  "try-error")

   if(te.s){
      c.crit.r <- q.r(D)(pmax(1-p.r-c.crit/sqrt(n),
                         # alternative: pmax(1-(1:l.x)/l.x-c.crit/sqrt(n),
                         getdistrOption("DistrResolution")),lower.tail=FALSE)
      c.crit.l <- q.l(D)(pmax(p.l-c.crit/sqrt(n),
                       # alternative: pmax(((1:l.x)-1)/l.x-c.crit/sqrt(n),
                       getdistrOption("DistrResolution")))
      c.crit.l[abs(c.crit.l)==Inf] <- NA
      c.crit.r[abs(c.crit.r)==Inf] <- NA
      c.crit.l[SIi == 2 | SIi == 3] <- NA
      c.crit.r[SIi == 2 | SIi == 3] <- NA
      c.c[SI.in,1:2] <- cbind(c.crit.l,c.crit.r)
   }
   if(te.i){
      c.crit.i <- x.in + c.crit.i/sqrt(n)
      c.crit.i[SIi == 2 | SIi == 3] <- NA
      c.c[SI.in,3:4] <- c.crit.i
      c.c[SI.in & abs(c.crit.i[,1])==Inf,3] <- NA
      c.c[SI.in & abs(c.crit.i[,2])==Inf,4] <- NA
   }
   return(list(crit = c.c, err=c(sim=te.s,pw=te.i)))
}
# returnlevelplot(xex,datax=FALSE,GEVFamilyMuUnknown(loc=es[1],shape=es[3],scale=es[2]))

###############################################################################
# Methods for Discrete Distributions
###############################################################################

## (c) Matthias Kohl: revised P.R. 030707

DiscreteDistribution <- function(supp, prob, .withArith = FALSE,
     .withSim = FALSE, .lowerExact = TRUE, .logExact = FALSE,
     .DistrCollapse =
                  getdistrOption("DistrCollapse"),
     .DistrCollapse.Unique.Warn =
                  getdistrOption("DistrCollapse.Unique.Warn"),
     .DistrResolution = getdistrOption("DistrResolution"),
     Symmetry = NoSymmetry()){
    if(!is.numeric(supp))
        stop("'supp' is no numeric vector")
    if(any(!is.finite(supp)))   # admit +/- Inf?
        stop("infinite or missing values in supp")
    len <- length(supp)
    if(missing(prob)){
        prob <- rep(1/len, len)
    }else{
        if(len != length(prob))
            stop("'supp' and 'prob' must have equal lengths")
        if(any(!is.finite(prob)))
            stop("infinite or missing values in prob")
        if(!identical(all.equal(sum(prob), 1,
                          tolerance = 2*getdistrOption("TruncQuantile")), TRUE))
            stop("sum of 'prob' has to be (approximately) 1")
        if(!all(prob >= 0))
            stop("'prob' contains values < 0")
    }

    o <- order(supp)
    supp <- supp[o]
    prob <- prob[o]
    rm(o)

    if(.DistrCollapse){
       if (len>1 && min(diff(supp))< .DistrResolution){
           erg <- .DistrCollapse(supp, prob, .DistrResolution)
           if (len>length(erg$prob) && .DistrCollapse.Unique.Warn)
               warning("collapsing to unique support values")
           prob <- erg$prob
           supp <- erg$supp
       }
    }else{
       usupp <- unique(supp)
       if(length(usupp) < len){
          if(.DistrCollapse.Unique.Warn)
             warning("collapsing to unique support values")
          prob <- as.vector(tapply(prob, supp, sum))
          supp <- sort(usupp)
          len <- length(supp)
          rm(usupp)
       }
       if(len > 1){
          if(min(diff(supp))< .DistrResolution)
             stop("grid too narrow --> change DistrResolution")
       }
    }
    rm(len)

    if(length(supp) == 1){
      rfun <- function(n){
        rep(supp, n)
      }
    }else{
      rfun <- function(n){
          sample(x = supp, size = n, replace = TRUE, prob = prob)
      }
    }

    dfun <- .makeDNew(supp, prob, Cont = FALSE)
    pfun <- .makePNew(supp, prob, .withSim, Cont = FALSE)
    qfun <- .makeQNew(supp, cumsum(prob), rev(cumsum(rev(prob))),
                      .withSim, min(supp), max(supp), Cont = FALSE)

    object <- new("DiscreteDistribution", r = rfun, d = dfun, q = qfun, p=pfun,
         support = supp, .withArith = .withArith, .withSim = .withSim,
         .lowerExact = .lowerExact, .logExact = .logExact, Symmetry = Symmetry)
}


setMethod("support", "DiscreteDistribution", function(object) object@support)

### left continuous cdf

setMethod("p.l", "DiscreteDistribution", function(object){
       if (.inArgs("lower.tail", p(object))){
           function(q, lower.tail = TRUE, log.p = FALSE){
                px <- p(object)(q, lower.tail = lower.tail)
                o.warn <- getOption("warn");
                on.exit(options(warn=o.warn))
                options(warn = -2)
                dx <- d(object)(.setEqual(q, support(object)))
                options(warn = o.warn)
                px0 <- pmax(px + if(lower.tail) -dx else  dx,0)
                if (log.p) px0 <- log(px0)
                return(px0)
                }
       }else{
           function(q, lower.tail = TRUE, log.p = FALSE){
                px <- p(object)(q)
                o.warn <- getOption("warn")
                on.exit(options(warn=o.warn))
                options(warn = -2)
                dx <- d(object)(.setEqual(q, support(object)))
                options(warn = o.warn)
                px0 <- pmax(if(lower.tail) px - dx else 1 - px + dx, 0)
                if (log.p) px0 <- log(px0)
                return(px0)
                }
       }
})

### right continuous quantile function

setMethod("q.r", "DiscreteDistribution", function(object){
    if (.inArgs("log.p", q.l(object))){
       if (.inArgs("lower.tail", q.l(object))){
           function(p, lower.tail = TRUE, log.p = FALSE){
                s <- support(object)
                psx <- p(object)(s, lower.tail = lower.tail,
                                 log.p = log.p)
                ps0 <- .setEqual(p, psx)

                o.warn <- getOption("warn"); options(warn = -2)
                on.exit(options(warn=o.warn))
                qx0 <- q.l(object)(ps0, lower.tail = lower.tail,
                                 log.p = log.p)
                options(warn = o.warn)

                m <- match(ps0, psx)
                n.ina.m <- !is.na(m)
                if(any(n.ina.m))
                   { M.n.ina.m  <- m[n.ina.m]
                     qx0[n.ina.m] <- (support(object))[pmin(M.n.ina.m+1,
                                                       length(s))]
                   }
                if(any(is.nan(qx0)))
                   warning("NaN's produced")
                return(qx0)
                }
       }else{
           function(p, lower.tail = TRUE, log.p = FALSE){
                s <- support(object)
                psx <- p(object)(s, log.p = log.p)
                if (lower.tail) p <- 1 - p
                ps0 <- .setEqual(p, psx)

                o.warn <- getOption("warn"); options(warn = -2)
                on.exit(options(warn=o.warn))
                qx0 <- q.l(object)(ps0, lower.tail = lower.tail,
                                 log.p = log.p)
                options(warn = o.warn)

                m <- match(ps0, psx)
                n.ina.m <- !is.na(m)
                if(any(n.ina.m))
                   { M.n.ina.m  <- m[n.ina.m]
                     qx0[n.ina.m] <- (support(object))[pmin(M.n.ina.m+1,
                                                       length(s))]
                   }
                if(any(is.nan(qx0)))
                   warning("NaN's produced")
                return(qx0)
                }
       }
    }else{
       if (.inArgs("lower.tail", q.l(object))){
           function(p, lower.tail = TRUE, log.p = FALSE){
                if (log.p) p <- exp(p)
                s <- support(object)
                psx <- p(object)(s, lower.tail = lower.tail)
                ps0 <- .setEqual(p, psx)

                o.warn <- getOption("warn"); options(warn = -2)
                on.exit(options(warn=o.warn))
                qx0 <- q.l(object)(ps0, lower.tail = lower.tail,
                                 log.p = log.p)
                options(warn = o.warn)

                m <- match(ps0, psx)
                n.ina.m <- !is.na(m)
                if(any(n.ina.m))
                   { M.n.ina.m  <- m[n.ina.m]
                     qx0[n.ina.m] <- (support(object))[pmin(M.n.ina.m+1,
                                                       length(s))]
                   }
                if(any(is.nan(qx0)))
                   warning("NaN's produced")
                return(qx0)
                }
       }else{
           function(p, lower.tail = TRUE, log.p = FALSE){
                if (log.p) p <- exp(p)
                s <- support(object)
                psx <- p(object)(s)
                if (lower.tail) p <- 1 - p
                ps0 <- .setEqual(p, psx)

                o.warn <- getOption("warn"); options(warn = -2)
                on.exit(options(warn=o.warn))
                qx0 <- q.l(object)(ps0, lower.tail = lower.tail,
                                 log.p = log.p)
                options(warn = o.warn)

                m <- match(ps0, psx)
                n.ina.m <- !is.na(m)
                if(any(n.ina.m))
                   { M.n.ina.m  <- m[n.ina.m]
                     qx0[n.ina.m] <- (support(object))[pmin(M.n.ina.m+1,
                                                       length(s))]
                   }
                if(any(is.nan(qx0)))
                   warning("NaN's produced")
                }
       }
    }
})



## Convolution Discrete Distributions

setMethod("+", c("DiscreteDistribution","DiscreteDistribution"),
function(e1,e2){

            if(length(support(e1))==1) return(e2+support(e1))
            if(length(support(e2))==1) return(e1+support(e2))
            e1.L <- as(e1, "LatticeDistribution")
            e2.L <- as(e2, "LatticeDistribution")
            if(is(e1.L, "LatticeDistribution") & is(e2.L, "LatticeDistribution"))
                  {w1 <- width(lattice(e1.L))
                   w2 <- width(lattice(e2.L))
                   W <- sort(abs(c(w1,w2)))
                   if (abs(abs(w1)-abs(w2))<getdistrOption("DistrResolution") ||
                       W[2] %% W[1] < getdistrOption("DistrResolution") )
                       res <- e1.L + e2.L
                       res@.finSupport <- e1.L@.finSupport&e2.L@.finSupport
                       return(res)
                  }
            res <- .convDiscrDiscr(e1,e2)
            res@.finSupport <- e1@.finSupport&e2@.finSupport
            return(res)
            })

setMethod("+", c("Dirac","DiscreteDistribution"),
      function(e1,e2){e2+location(e1)})


## binary operators for discrete distributions

setMethod("*", c("DiscreteDistribution","numeric"),
           function(e1, e2) { Distr <- .multm(e1,e2, "DiscreteDistribution")
                              if(is(Distr, "AffLinDistribution"))
                                 Distr@X0 <- e1

                              if(is(e1@Symmetry,"SphericalSymmetry"))
                                 Distr@Symmetry <-
                                   SphericalSymmetry(SymmCenter(e1@Symmetry)*e2)

                              if(is.finite(e2)){
                                 Distr@.finSupport <- e1@.finSupport
                              }else{
                                 ep <- .Machine$double.eps
                                 Distr@.finSupport <- c(p(e1)(0)<ep,p(e1)(0)>1-ep)
                              }
                              if(e2<0) Distr@.finSupport <- rev(Distr@.finSupport)
                              return(Distr)
                             })
setMethod("+", c("DiscreteDistribution","numeric"),
           function(e1, e2) { Distr <- .plusm(e1,e2, "DiscreteDistribution")
                              if(is(Distr, "AffLinDistribution"))
                                 Distr@X0 <- e1

                              if(is(e1@Symmetry,"SphericalSymmetry"))
                                 Distr@Symmetry <-
                                   SphericalSymmetry(SymmCenter(e1@Symmetry)+e2)

                              isfe2 <- c(e2 >(-Inf), e2<Inf)
                              Distr@.finSupport <- e1@.finSupport & isfe2
                              return(Distr)
                             })

setMethod("*", c("AffLinDiscreteDistribution","numeric"),
           function(e1, e2) {
                Distr <- .multm(e1,e2, "AffLinDiscreteDistribution")
                if(is(e1@Symmetry,"SphericalSymmetry"))
                      Distr@Symmetry <-
                        SphericalSymmetry(SymmCenter(e1@Symmetry)*e2)
                if(is.finite(e2)){
                   Distr@.finSupport <- e1@.finSupport
                }else{
                   ep <- .Machine$double.eps
                   Distr@.finSupport <- c(p(e1)(0)<ep,p(e1)(0)>1-ep)
                }
                if(e2<0) Distr@.finSupport <- rev(Distr@.finSupport)
                return(Distr)
                })
setMethod("+", c("AffLinDiscreteDistribution","numeric"),
           function(e1, e2) {
                Distr <- .plusm(e1,e2, "AffLinDiscreteDistribution")
                if(is(e1@Symmetry,"SphericalSymmetry"))
                      Distr@Symmetry <-
                        SphericalSymmetry(SymmCenter(e1@Symmetry)*e2)
                isfe2 <- c(e2 >(-Inf), e2<Inf)
                Distr@.finSupport <- e1@.finSupport & isfe2
                return(Distr)
                })

## Group Math for discrete distributions
setMethod("Math", "DiscreteDistribution",
          function(x){
            rnew <- function(n, ...){}
            body(rnew) <- substitute({ f(g(n, ...)) },
                                         list(f = as.name(.Generic), g = x@r))
            object <- new("DiscreteDistribution", r = rnew,
                           .withSim = TRUE, .withArith = TRUE)
            object@.finSupport <- x@.finSupport&NA
            object
          })
setMethod("Math", "Dirac",
          function(x){ loc <- location(x)
                       lc <- callGeneric(loc)
                       object <- Dirac(lc)
                       object@.finSupport <- x@.finSupport&NA
                       object
                       })

## exact: abs for discrete distributions
setMethod("abs", "DiscreteDistribution",function(x){

       rnew <- function(n, ...){}
       body(rnew) <- substitute({ abs(g(n, ...)) },
                                      list(g = x@r))

       xx <- x
       supportnew <- support(x)

       isSym0 <- FALSE
       if(is(Symmetry(x),"SphericalSymmetry"))
          if(.isEqual(SymmCenter(Symmetry(x)),0))
             isSym0 <- TRUE

       if(isSym0){
          supportnew <- supportnew[supportnew>=0]

          .lowerExact = .lowerExact(x)

          dxlog <- if("log" %in% names(formals(d(x))))
                        quote({dx <- d(xx)(x, log = TRUE)})
                   else quote({dx <-  log(d(xx)(x))})
          pxlog <- if("log.p" %in% names(formals(p(x))) &&
                       "lower.tail" %in% names(formals(p(x))))
                        quote({p(x)(q, lower.tail = FALSE, log.p = TRUE)})
                   else
                        quote({log(1-p(x)(q))})


          qxlog <- if("lower.tail" %in% names(formals(q.l(x))))
                          quote({qx <- if(lower.tail)
                                          q.l(x)((1+p1)/2)
                                       else
                                          q.l(x)(p1/2,lower.tail=FALSE)})
                      else
                          quote({qx <- q.l(x)(if(lower.tail) (1+p1)/2 else 1-p1/2)})
          if("lower.tail" %in% names(formals(q.l(x)))&&
             "log.p" %in% names(formals(q.l(x))))
              qxlog <- quote({qx <- if(lower.tail) q.l(x)((1+p1)/2)
                                       else
                                          q.l(x)(if(log.p)p-log(2)
                                               else p1/2,lower.tail=FALSE,log.p=log.p)})


          dnew <- function(x, log = FALSE){}
          body(dnew) <- substitute({
                    dxlog0
                    dx[x>0] <- dx+log(2)
                    if (!log) dx <- exp(dx)
                    dx[x<0] <- if(log) -Inf else 0
                    return(dx)
                    }, list(dxlog0 = dxlog))

          pnew <- function(q, lower.tail = TRUE, log.p = FALSE){}
          body(pnew) <- substitute({
                    if (!lower.tail){
                        px <- (log(2) + pxlog0)*(q>=0)
                        if(!log.p) px <- exp(px)
                    }else{
                        px <- pmax(2 * p(x)(q) - 1,0)
                        if(log.p) px <- log(px)
                    }
                    return(px)
            }, list(pxlog0 = pxlog))

          qnew <- function(p, lower.tail = TRUE, log.p = FALSE){}
          body(qnew) <- substitute({
                   p1 <- if(log.p) exp(p) else p
                   qxlog0
                   qx[p1<0] <- NaN
                   if (any((p1 < -.Machine$double.eps)|(p1 > 1+.Machine$double.eps)))
                   warning(gettextf("q method of %s produced NaN's ", objN))
                   return(qx)
            }, list(qxlog0 = qxlog, objN= quote(.getObjName(1))))

       }else{
            if (.isEqual(p.l(x)(0),0)) return(x)

            supportnew <- sort(unique(abs(supportnew)))

            dnew <- function(x, log = FALSE){
                    o.warn <- getOption("warn"); options(warn = -1)
                    on.exit(options(warn=o.warn))
                    dx <- (x>=0) * d(xx)(x) + (x>0) * d(xx)(-x)
                    options(warn = o.warn)
                    if (log) dx <- log(dx)
                    return(dx)
                 }

            pxlow <- if("lower.tail" %in% names(formals(p(x))))
                        substitute({p(x)(q, lower=FALSE)})
                   else
                        substitute({1-p(x)(q)})

            pnew <- function(q, lower.tail = TRUE, log.p = FALSE){}
            body(pnew) <- substitute({
                    px <- if (lower.tail)
                            (q>=0) * (p(x)(q) - p.l(x)(-q))
                          else pxlow0 + p.l(x)(-q)
                    if (log.p) px <- log(px)
                    return(px)
            }, list(pxlow0=pxlow))

            prob <- dnew(supportnew)

            qnew <- .makeQNew(supportnew, cumsum(prob),
                            rev(cumsum(rev(prob))), notwithLLarg = x@.withSim,
                            min(supportnew), max(supportnew), Cont = FALSE)

         }
         object <- new("DiscreteDistribution", r = rnew, p = pnew,
                        q = qnew, d = dnew, support = supportnew,
                        .withSim = x@.withSim, .withArith = TRUE,
                        .lowerExact = .lowerExact(x))
         object@.finSupport <- c(TRUE, all(x@.finSupport))
         object
})

## exact: eps for discrete distributions
setMethod("exp", "DiscreteDistribution",
           function(x){ obj <- .expm.d(x)
                        obj@.finSupport <- c(TRUE, x@.finSupport[2])
                        obj 
           }
           )


### preliminary to export special functions
if (getRversion()>='2.6.0'){

setMethod("log", "DiscreteDistribution",
           function(x, base = exp(1)) {
           xs <- as.character(deparse(match.call(
                 call = sys.call(sys.parent(1)))$x))
           ep <- getdistrOption("TruncQuantile")
           basl <- log(base)
           if(p(x)(0)>ep)
                stop(gettextf("log(%s) is not well-defined with positive probability ", xs))
           else{
                obj <- .logm.d(x)/basl
                obj@.finSupport <- c(TRUE, x@.finSupport[2])
                return(obj)
           }})

setMethod("log", "Dirac",
          function(x, base = exp(1)){
                       xs <- as.character(deparse(match.call(
                             call = sys.call(sys.parent(1)))$x))
                       loc <- location(x)
                       ep <- getdistrOption("TruncQuantile")
                       basl <- log(base)
                       if(loc < ep)
                          stop(gettextf("log(%s) is not well-defined with positive probability ", xs))
                       Dirac(log(loc)/basl)})

setMethod("log10", "DiscreteDistribution",
          function(x) log(x = x, base = 10))

setMethod("sign", "DiscreteDistribution",
          function(x){
          d0 <- d(x)(0)
          DiscreteDistribution(supp=c(-1,0,1),
              prob=c(p(x)(-getdistrOption("TruncQuantile")),
                     d0,
                     p(x)(getdistrOption("TruncQuantile"), lower=FALSE)))
          })


setMethod("digamma", "DiscreteDistribution",
          function(x){
            px0 <- p(x)(0)
            if(px0>0) stop("argument of 'digamma' must be concentrated on positive values")
            rnew <-  function(n, ...){}
            body(rnew) <- substitute({ digamma(g(n, ...)) }, list(g = x@r))

            object <- DiscreteDistribution(
                     supp=digamma(support(x)),
                     prob=prob(x), .withArith = TRUE)

            object@.finSupport <- c(TRUE, x@.finSupport[2])
            object
          })

setMethod("lgamma", "DiscreteDistribution",
          function(x){
            rnew = function(n, ...){}
            body(rnew) <- substitute({ lgamma(g(n, ...)) }, list(g = x@r))
            object <- new("DiscreteDistribution", r = rnew,
                           .withSim = TRUE, .withArith = TRUE)
            object@.finSupport <- c(TRUE, x@.finSupport[2])
            object
          })

setMethod("gamma", "DiscreteDistribution",
          function(x){
            rnew = function(n, ...){}
            body(rnew) <- substitute({ gamma(g(n, ...)) }, list(g = x@r))
            object <- new("DiscreteDistribution", r = rnew,
                           .withSim = TRUE, .withArith = TRUE)
            object@.finSupport <- c(TRUE, x@.finSupport[2])
            object
          })
setMethod("sqrt", "DiscreteDistribution",
            function(x) x^0.5)

}
setMethod("prob", "DiscreteDistribution",
function(object) {sp <- object@support
                  pr <- object@d(sp)
                  names(pr) <- paste(sp)
                  return(pr)
                  })
## Replace Methods
setReplaceMethod("prob", "DiscreteDistribution",
                  function(object, value){
                  return(DiscreteDistribution(supp = object@support,
                             prob = value,
                            .withArith = object@.withArith,
                            .withSim = object@.withSim,
                            .lowerExact = .lowerExact(object),
                            .logExact = .logExact(object)))}
                  )

#################################
###
### Class: DiracParameter
###
#################################


### Access Methods
setMethod("location", "DiracParameter", function(object) object@location)
### Replace Methods
setReplaceMethod("location", "DiracParameter", 
                  function(object, value){ object@location <- value; object})

validDiracParameter <- function(object){
  if(length(location(object)) != 1)
    stop("location has to be a numeric of length 1")    
  else return(TRUE)
}

setValidity("DiracParameter", validDiracParameter)



#################################
###
### Class: Dirac distribution
###
#################################

Dirac <- function(location = 0) new("Dirac", location = location)

### wrapped access methods
setMethod("location", "Dirac", function(object) location(param(object)))
### wrapped replace methods
setMethod("location<-", "Dirac", 
           function(object, value) new("Dirac", location = value))


## some exact arithmetic methods
setMethod("+", c("Dirac","numeric"),
          function(e1, e2){
            if (length(e2)>1) stop("length of operator must be 1")
            new("Dirac", location = location(e1) + e2, .withArith = TRUE)  
          })

setMethod("*", c("Dirac","numeric"),
          function(e1, e2){
            if (length(e2)>1) stop("length of operator must be 1")
            new("Dirac", location = location(e1) * e2, .withArith = TRUE)  
          })

setMethod("+", c("Dirac","Dirac"),
          function(e1,e2){
            new("Dirac", location = location(e1) + location(e2), 
                 .withArith = TRUE)
          })

setMethod("*", c("Dirac","Dirac"),
          function(e1,e2){
            new("Dirac", location = location(e1) * location(e2), 
                 .withArith = TRUE)
          })

setMethod("-", c("Dirac","Dirac"),
          function(e1,e2){
            new("Dirac", location = location(e1) - location(e2), 
                 .withArith = TRUE)
          })

setMethod("/", c("Dirac","Dirac"),
          function(e1,e2){
            if(isTRUE(all.equal(location(e2),0))) 
                stop("location parameter of divisor must not be 0")
            new("Dirac", location = location(e1) / location(e2), 
                .withArith = TRUE)
          })

setMethod("/", c("numeric","Dirac"),
          function(e1,e2){
            if(isTRUE(all.equal(location(e2),0))) 
                stop("location parameter of divisor must not be 0")
            new("Dirac", location = e1 / location(e2), 
                .withArith = TRUE)
          })

setMethod("+", c("Dirac","UnivariateDistribution"),
          function(e1,e2){
            location(e1) + e2
          })

setMethod("*", c("Dirac","UnivariateDistribution"),
          function(e1,e2){
            location(e1) * e2
          })

setMethod("+", c("UnivariateDistribution","Dirac"),
          function(e1,e2){
            location(e2) + e1
          })

setMethod("*", c("UnivariateDistribution","Dirac"),
          function(e1,e2){
            location(e2) * e1
          })

###############################################################################
## Generating function to generate empirical distribution given some data
###############################################################################

## simple wrapper to DiscreteDistribution
EmpiricalDistribution <- function(data, .withArith = FALSE,
                                  .withSim = FALSE, .lowerExact = TRUE, .logExact = FALSE,
                                  .DistrCollapse = 
                                    getdistrOption("DistrCollapse"),
                                  .DistrCollapse.Unique.Warn = 
                                    getdistrOption("DistrCollapse.Unique.Warn"),
                                  .DistrResolution = getdistrOption("DistrResolution"),
                                  Symmetry = NoSymmetry()){
  DiscreteDistribution(supp = data, .withArith = .withArith, .withSim = .withSim,
                       .lowerExact = .lowerExact, .logExact = .logExact, 
                       .DistrCollapse = .DistrCollapse, 
                       .DistrCollapse.Unique.Warn = .DistrCollapse.Unique.Warn,
                       .DistrResolution = .DistrResolution,
                       Symmetry = Symmetry)
}

#------------------------------------------------------------------------------
### internal help function to check whether numeric vectors are integers
#------------------------------------------------------------------------------

.isInteger  <- function(x, tol = .Machine$double.eps) abs(as.integer(x)-x)< tol
.isNatural  <- function(x, tol = .Machine$double.eps) .isInteger(x, tol) & (x>0)
.isNatural0 <- function(x, tol = .Machine$double.eps) .isInteger(x, tol) & (x>=0)
setAs("numeric","Integer",function(from) new("Integer",as.integer(from)))

#------------------------------------------------------------------------------
### internal help function to check if a vector can be made a lattice
#------------------------------------------------------------------------------

.is.vector.lattice <- function(x)
  {    ### is x equally spaced?
    all( sapply(diff(x), function(y)
         isTRUE(all.equal(y, diff(x)[1],
                tolerance = getdistrOption("DistrResolution"),
                check.attributes = FALSE))))
  }

### internal help function to check consistency of lattice with support:
.is.consistent <- function(lattice, support, eq.space = TRUE)
  {
   p <- pivot(lattice); l <- Length(lattice); w <- width(lattice)
   ds <- diff(support); ms <- min(support); Ms <- max(support)
   ### is support equally spaced?
   if (! .is.vector.lattice(support)  && eq.space)
      return(FALSE)
   ### are width of lattice and support consistent
   if (! isTRUE(all.equal(min(ds), abs(w), check.attributes = FALSE)))
      return(FALSE)
   ### pivot is left or right endpoint of support
   if ( isTRUE(all.equal(ms, p, check.attributes = FALSE)) || isTRUE(all.equal(Ms, p, check.attributes = FALSE)) )
      return(TRUE)

   if (isTRUE(all.equal(min((support[1]-p)%%w,w-(support[1]-p)%%w),0,
                        tolerance = getdistrOption("TruncQuantile"),
                        check.attributes = FALSE)))
      return(TRUE)
  return(FALSE)
  }

 ## generate a lattice from equally spaced vector
.make.lattice.es.vector <- function(x){
  new("Lattice", pivot = x[1], width = x[2]-x[1],
       Length = length(x))
}

#------------------------------------------------------------------------------
### internal help function to determin common lattice width
#------------------------------------------------------------------------------

.EuclidAlgo <- function(n1,n2){
   r<- 2
   m <- round(max(n1,n2))
   n <- round(min(n1,n2))
   if (n==1) return(1)
   while (r>1){
      r <- m %% n
      m <- n
      if(r==1) return(1)
      if(r==0) break
      n <- r
      }
   return(n)
}
.getCommonWidth <- function(x1,x2, tol=.Machine$double.eps){
  rI <- function(x) .isInteger(x, tol = x * tol)
  if(rI(x1)&&rI(x2)) return(.EuclidAlgo(x1,x2))
  if(rI(10000*x1) && rI(10000*x2)) return(.EuclidAlgo(10000*x1,10000*x2)/10000)
  n <- max(x1,x2); m <- min(x1,x2)
  if(rI(n/m)) return(m)
  vc <- 1:10000
  vecT <- sapply(vc,function(x) rI(x*(n/m)))
  vec <- m/vc
#  print(min(vc[vecT]))
  if(any(vecT)) return((vec/min(vc[vecT]))[1])
  return(NULL)  
}

#------------------------------------------------------------------------------
### %in% for numerics with tolerance
#------------------------------------------------------------------------------
.inWithTol <- function(x,y,tol=.Machine$double.eps){
   sapply(x, function(u) any(abs(u-y)<tol))
}                                                                         

#------------------------------------------------------------------------------
### brute force convolution
#------------------------------------------------------------------------------
.convDiscrDiscr <- function(e1,e2){
            convolutedsupport <- rep(support(e1), each = length(support(e2))) +
                                 support(e2)

            gridvalues1 <- d(e1)(support(e1)); gridvalues2 <- d(e2)(support(e2))
            convolutedvalues <- rep(gridvalues1, each = length(support(e2))) *
                                gridvalues2
            rm(gridvalues1,gridvalues2)

            tmptable <- data.frame(x = convolutedsupport, dx = convolutedvalues)
            rm(convolutedsupport,convolutedvalues)
            tmp <- tapply(tmptable$dx, tmptable$x, sum)
            rm(tmptable)

            supp.u <- as.numeric(names(tmp))
            prob.u <- as.numeric(tmp)

            o <- order(supp.u)
            supp <- supp.u[o]
            prob <- prob.u[o]

            #supp.u <- unique(supp)

            len <- length(supp)

            if(len > 1){
               if (min(diff(supp))< getdistrOption("DistrResolution")){
                   if (getdistrOption("DistrCollapse")){
                       erg <- .DistrCollapse(supp, prob, 
                                   getdistrOption("DistrResolution"))
                       if ( len > length(erg$prob) && 
                                getdistrOption("DistrCollapse.Unique.Warn") )
                            warning("collapsing to unique support values")         
                       prob <- erg$prob
                       supp <- erg$supp
                   }else
                    stop("grid too narrow --> change DistrResolution")
                }
            }

            rm(tmp, len)

            .withSim <- e1@.withSim || e2@.withSim

            rfun <- function(n) {}
            body(rfun) <- substitute({ f(n) + g(n) },
                                         list(f = e1@r, g = e2@r))

            dfun <- .makeDNew(supp, prob, Cont = FALSE)
            pfun <- .makePNew(supp, prob, .withSim, Cont = FALSE)
            qfun <- .makeQNew(supp, cumsum(prob), rev(cumsum(rev(prob))),
                      .withSim, min(supp), max(supp), Cont = FALSE)

            object <- new("DiscreteDistribution", r = rfun, d = dfun, p = pfun,
                           q = qfun, support = supp,
                           .withSim = .withSim, .withArith = TRUE)
            rm(rfun, dfun, qfun, pfun)

            if(is(e1@Symmetry,"SphericalSymmetry")&& 
               is(e2@Symmetry,"SphericalSymmetry"))
               object@Symmetry <- SphericalSymmetry(SymmCenter(e1@Symmetry)+
                                                     SymmCenter(e2@Symmetry))   

            object@.finSupport <- e1@.finSupport & e2@.finSupport
            object
          }

#------------------------------------------------------------------------------
### .fm, .fM, .fM2 functions
#------------------------------------------------------------------------------

.fM <- function(x,f){
   owarn <- getOption("warn")
   options("warn"=-1)
   on.exit(options("warn"=owarn))
   if(!.inArgs("log.p", f))
      f0 <- f
   else f0 <- function(x) f(log(x), log.p = TRUE)   
   xo <-  x
   x1 <- (1+xo)/2
   i <- 1
   while( i < 30)
   { i <- i+1
     while(!is.na(f1 <- f0(x1)) && f1 < Inf)
        {xo <- x1
         x1 <- (1+x1)/2
         }
     x1 <- (x1+xo)/2}
   log(xo)}
   
.fM2 <- function(x,f){
   owarn <- getOption("warn")
   options("warn"=-1)
   on.exit(options("warn"=owarn))
   if(!.inArgs("log.p", f))
      f0 <- function(x) f(x, lower.tail = FALSE)
   else f0 <- function(x) f(log(x), lower.tail = FALSE, log.p = TRUE)   
   xo <- x
   x1 <- xo/2
   i <- 1
   while( i < 30)
   { i <- i+1
     while(!is.na(f1 <- f0(x1)) && f1 < Inf)
        {xo <- x1
         x1 <- x1/2
         }
     x1 <- (x1+xo)/2}   
   log(xo)}

.fm <- function(x,f){
   owarn <- getOption("warn")
   options("warn"=-1)
   on.exit(options("warn"=owarn))
   if(!.inArgs("log.p", f))
      f0 <- f
   else f0 <- function(x) f(log(x), log.p = TRUE)   
   
   xo <- x
   x1 <- xo/2
   i <- 1
   while( i < 30)
   { i <- i+1
     while(!is.na(f1 <- f0(x1)) && f1 > -Inf)
        {xo <- x1
         x1 <- x1/2
         }
     x1 <- (x1+xo)/2}
   log(xo)}
   
#------------------------------------------------------------------------------
# .presubs : for titles etc
#------------------------------------------------------------------------------

.presubs <- function(inp, frompat, topat){
### replaces in an expression or a string all frompat patterns to topat patterns

logic <- FALSE
inCx <- sapply(inp,
   function(inpx){
      inC <- deparse(inpx)
      l <- length(frompat)
      for(i in 1:l)
         { if (is.language(topat[[i]])){
               totxt <- deparse(topat[[i]])
               totxt <- gsub("expression\\(", "\", ", gsub("\\)$",", \"",totxt))
               if (length(grep(frompat[i],inC))) logic <<- TRUE
               inC <- gsub(frompat[i],totxt,inC)
           }else inC <- gsub(frompat[i], topat[[i]], inC)
         }
      return(inC)
    })
if(length(grep("expression",inCx))>0)
   inCx <- gsub("expression\\(", "", gsub("\\)$","",inCx))
if (length(inCx) > 1) {
   inCx <- paste(inCx, c(rep(",", length(inCx)-1), ""),
                 sep = "", collapse = "\"\\n\",")
   if ( any(as.logical(c(lapply(inp,is.language)))) | logic )
      inCx <- paste("expression(paste(", gsub("\\\\n"," ", inCx), "))", sep ="")
   else
      inCx <- paste("paste(",inCx,")", sep ="")
}else inCx <- paste("expression(paste(",inCx,"))",sep="")
outC <- eval(parse(text = eval(inCx)))
return(outC)
}

#------------------------------------------------------------------------------
# help check functions
#------------------------------------------------------------------------------

.inArgs <- function(arg, fct)
          {as.character(arg) %in% names(formals(fct))}

.isEqual <- function(p0, p1, tol = min( getdistrOption("TruncQuantile")/2,
                                          .Machine$double.eps^.7
                                          ))
                abs(p0-p1)< tol

.isIn <- function(p0, pmat, tol = min( getdistrOption("TruncQuantile")/2,
                                          .Machine$double.eps^.7
                                          ))
                  {## PR 2018 04 13
                   ## detected by Tuomo.OJALA@3ds.com: the gaps matrix can
                   ## have zero rows -> check this in the following line
                   if(nrow(pmat)==0) return(FALSE)
                   list1 <- lapply(1:nrow(pmat), function(x){
                            (p0+tol > pmat[x,1]) & (p0-tol < pmat[x,2]) })
                   apply(matrix(unlist(list1), ncol = nrow(pmat)), 1, any)}           


.isEqual01<- function(x) .isEqual(x,0)|.isEqual(x,1)

.setEqual <- function(x, y, tol = 1e-7){ 
### all elements of x equal to some element of y up tol are set to exactly
###     the respective element of y
   x1 <- round(2*x/tol,0)
   y1 <- round(2*y/tol,0)
   z  <- x
   m  <- match(x1,y1)
   n.ina.m <- !is.na(m)
   z[n.ina.m] <- y[m[n.ina.m]]
   z
}


.notwithLArg <- function(D)  D@.withSim||!.inArgs("lower.tail",p(D))

#------------------------------------------------------------------------------
# other helpers
#------------------------------------------------------------------------------

.getObjName <- function(i = 1)
     {Ca <- sys.call(-4);  as.character(as.list(Ca[[1]]))[i+1]}

.discretizeP <- function(D, lower, upper, h){
   h0 <- 40*(getUp(D)-getLow(D)) /
         2^getdistrOption("DefaultNrFFTGridPointsExponent")
   if(h > h0 )
      warning(paste("Grid for approxfun too wide, ",
                     "increase DefaultNrFFTGridPointsExponent", sep =""))

   x <- seq(from = lower, to = upper, by = h)
   if(TRUE){#.notwithLArg(D)){
      return(diff(p(D)(x)))
#      return((diff(p(D)(x))+diff(rev(p(D)(x,lower=FALSE))))/2)
   }else{
      M <- q.l(D)(0.5);   L <- length(x)
      x.l <- x [ x <= M ];  x.u <- x [ x >= M ]
      L.l <- length(x.l);   L.u <- length(x.u)
      if(L.u+L.l-L>0)
         return(c(diff(p(D)(x.l, lower.tail = TRUE)),
               -diff(p(D)(x.u, lower.tail = FALSE))))
      else
         return(c(diff(p(D)(x.l, lower.tail = TRUE)),
                p(D)(x.u[1])-p(D)(x.l[L.l]),
               -diff(p(D)(x.u, lower.tail = FALSE))))
            }
   }



#------------------------------------------------------------------------------
# .makeD, .makeP, .makeQ
#------------------------------------------------------------------------------

.makeD <- function(object, argList, stand = NULL, fac = NULL)
         { d <- function(x, log = FALSE, ...){}
           if(inA <- .inArgs("log", object@d))
                     argList <- substitute(c(AL, log = log, ...),
                                           list(AL = argList))
           else      argList <- substitute(c(AL, ...),
                                           list(AL = argList))
           myCall <- substitute(d0 <- do.call("@"(objC,d),AL) ,
                                list(objC = quote(object),
                                AL = argList))
           myBod0 <- myCall
           if (!inA) myBod0 <- substitute({myCall
                                           if (log) d0 <- log(d0)})
           myBod1 <- myBod0
           if (!is.null(stand))
                myBod1 <- substitute(
                     {myBod0
                      d0 <- if (log) d0 - log(stand) else d0 / stand },
                      list(stand = stand, myBod0 = myBod0))
           myBod <- myBod1
           if (!is.null(fac))
                myBod <- substitute(
                     {myBod1
                      d0 <- if (log) d0 + log(fac) else d0 * fac },
                      list(fac = fac, myBod1 = myBod1))
           body(d) <- substitute({myBod
                                  return(d0)})
           return(d)
         }

.makeP <- function(object, argList, sign = TRUE, correct = NULL, 
          fac = NULL, fac2 = NULL)
         {
           p <- function(q, lower.tail = TRUE, log.p = FALSE, ...){}
           siY <- substitute(lower.tail)
           siN <- substitute(!lower.tail)
           lowS <-  if( sign) siY else siN
           lowS1 <- if(!sign) siY else siN
           if(inA1 <- .inArgs("lower.tail", object@p))
                      argList <- substitute(c(aL, lower.tail = lowT),
                                list( lowT = lowS,
                                      aL = argList )
                                      )
           else  argList <- substitute(aL, list(aL = argList ))
           if(inA2 <- (.inArgs("log.p", object@p) && inA1))
                     argList <- substitute(c(argList, log.p = log.p, ...))
           else      argList <- substitute(c(argList, ...))

           myCall <- substitute(p0 <- facC * do.call("@"(objC,p),AL) + facD,
                                list(objC = quote(object),
                                     AL = argList,
                                     facC = if(is.null(fac)) 1 else fac,
                                     facD = if(is.null(fac2)) 0 else fac2)
                                     )
           if (!inA1)
               myBod0 <- substitute({myC
                                     if (lowT) p0 <- 1 - p0},
                         list(myC=myCall, lowT = lowS1))
           else
               myBod0 <- substitute(myCall)

           if (!sign && !is.null(correct))
               myBod1 <- substitute({myC
                                     corC},
                         list(myC = myBod0, corC = correct))
           else myBod1 <- substitute(myBod0)

           if (!inA2) myBod <- substitute({myBod1
                                           if (log.p) p0 <- log(p0)
                                           return(p0)}  )

           else myBod <- substitute({myBod1
                                     return(p0)})
           body(p) <- substitute(myBod)
           return(p)
         }


.makeQ <- function(object, lastCall, sign = TRUE, Cont = TRUE)
     ## lastCall of form e1 %op% e2
         {
           siY <- substitute(lower.tail)
           siN <- substitute(!lower.tail)
           lowS <-  if( sign) siY else siN
           lowS1 <- if(!sign) siY else siN

           q <- function(p, lower.tail = TRUE, log.p = FALSE, ...){}
           argList <- alist(p)
           if(inA1 <- .inArgs("lower.tail", object@q))
                     argList1 <- substitute(alist(lower.tail = lowT),
                                list( lowT = lowS))
           else argList1 <- NULL

           if(inA2 <- (.inArgs("log.p", object@q) && inA1))
                      argList1 <- substitute(c(argList1,
                                               alist(log.p = log.p), ...))
           else       argList1 <- substitute(c(argList1, alist(...)))


           argList <- substitute(c(AL,AL1),list(AL=argList,AL1=argList1))
           myCall <- substitute({q0 <- do.call("@"(objC,q),AL)
                                 q0 <- lasC
                                 return(q0)},
                                list(objC = quote(object), AL=argList,
                                     lasC = lastCall))

           if (!Cont && !sign){
               if (inA1){
                     dqfS <- substitute({
                             dq <- getdistrOption("DistrResolution")
                             if (lower.tail) dq <- -dq
                                      })
               }else{
                     dqfS <- substitute({
                             dq <- getdistrOption("DistrResolution")
                                        })
               }
               if (inA2){ indS <- substitute(if(log.p){ ind <- (p>-Inf)&(p<0)
                                               p0[!ind] <- +1 }
                                             else { ind <- (p>0)&(p<1)
                                               p0[!ind] <- -1 } )
               }else    { indS <- substitute({ ind <- (p>0)&(p<1)
                                               p0[!ind] <- -1 })
               }
               myBod1 <- substitute({
                   q01 <- do.call("@"(object,q),AL)
                   p0 <- do.call("@"(object,p), c(alist(q=q01),AL1))
                   indC
                   ind2 <- .isEqual(p,p0)
                   dqfC # dq * if (with lower.tail-arg & lower.tail == FALSE)
                        # -1 else 1
                   p[ind2] <- p[ind2] + dq
                   my0C
               }, list(my0C = substitute(myCall),
                       AL = substitute(argList), AL1 = substitute(argList1),
                       indC = substitute(indS), dqfC = substitute(dqfS) )
                       )
                 ## discrete correction
           }else{
               myBod1 <- substitute(myCall)
           }

           if (!inA1) myBod2 <- substitute({if (lowT) p <- 1 - p
                                            myC},
                                    list(lowT = lowS1, myC = myBod1 ))
           else myBod2 <- substitute(myBod1)

           if (!inA2){
               myBod <- substitute({if (log.p) p <- exp(p)
                                    myC   },
                         list(myC = myBod2))
               }else{ myBod <- substitute(myBod2) }

           body(q) <- substitute(myBod)
           return(q)
         }

#------------------------------------------------------------------------------
# .plusm, .multm
#------------------------------------------------------------------------------

.plusm <- function(e1, e2, Dclass = "DiscreteDistribution"){
            if (length(e2)>1) stop("length of operator must be 1")
            if (.isEqual(e2, 0)) return(e1)

            if(Dclass %in% c("AffLinDiscreteDistribution",
                              "AffLinAbscontDistribution"))
               if(.isEqual(e1@a,1)&&.isEqual(e1@b+e2,0)) return(e1@X0)

            if ((Dclass == "DiscreteDistribution")||
                (Dclass == "AffLinDiscreteDistribution"))
                supportnew <- e1@support + e2

            if ((Dclass == "AbscontDistribution")||
                (Dclass == "AffLinAbscontDistribution"))
                 gapsnew <- if(is.null(e1@gaps)) NULL else e1@gaps + e2

            rnew <- function(n, ...){}
            body(rnew) <- substitute({ f(n, ...) + g },
                                         list(f = e1@r, g = e2))
            dnew <- .makeD(e1, substitute(alist(x = x - e2), list(e2 = e2)))
            pnew <- .makeP(e1, substitute(alist(q = q - e2), list(e2 = e2)))
            qnew <- .makeQ(e1, substitute(q0 + e2, list(e2 = e2)))

            isfe2 <- c((e2>(-Inf)),(e2<Inf))

            if (Dclass == "AffLinDiscreteDistribution"){
                object <- new("AffLinDiscreteDistribution", 
                              r = rnew, d = dnew, p = pnew,
                              q = qnew, support = supportnew,
                              a = e1@a, b = e1@b + e2, X0 = e1@X0,
                             .withSim = FALSE, .withArith = TRUE,
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1),
                    .finSupport = e1@.finSupport& isfe2)
                rm(supportnew)

            }else if (Dclass == "DiscreteDistribution"){
                object <- new("AffLinDiscreteDistribution", 
                              r = rnew, d = dnew, p = pnew,
                              q = qnew, support = supportnew,
                              a = 1, b = e2, X0 = e1,
                             .withSim = FALSE, .withArith = TRUE,
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1),
                    .finSupport = e1@.finSupport&isfe2)
                rm(supportnew)

            }else if (Dclass == "AffLinAbscontDistribution"){
                object <- new("AffLinAbscontDistribution", 
                              r = rnew, d = dnew, p = pnew, q = qnew, 
                              gaps = gapsnew, a = e1@a, b = e1@b + e2, 
                              X0 = e1@X0, .withSim = FALSE, .withArith = TRUE,
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))

            }else if (Dclass == "AbscontDistribution"){  
                object <- new("AffLinAbscontDistribution", r = rnew, 
                              d = dnew, p = pnew, q = qnew, gaps = gapsnew, 
                              a = 1, b = e2, X0 = e1, .withSim = FALSE, 
                              .withArith = TRUE,
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
            }
            rm(pnew, qnew, dnew, rnew)
            object
          }

.multm <- function(e1, e2, Dclass = "DiscreteDistribution"){

            withg <- getdistrOption("withgaps")
            on.exit(distroptions(withgaps=withg))
            distroptions(withgaps=FALSE)

            if (length(e2)>1) stop("length of operator must be 1")

            if (.isEqual(e2, 1)) return(e1)
            if (.isEqual(e2, 0))  return(new("Dirac", location = 0))

            if(Dclass %in% c("AffLinDiscreteDistribution",
                              "AffLinAbscontDistribution"))
               if(.isEqual(e1@a*e2,1)&&.isEqual(e1@b,0)) return(e1@X0)

            rnew <- function(n, ...){}
            body(rnew) <- substitute({ f(n, ...) * g },
                                         list(f = e1@r, g = e2))


            if (Dclass == "AffLinDiscreteDistribution"){
                  if(is.finite(e2)){
                     .finS <- e1@.finSupport
                  }else{
                     ep <- .Machine$double.eps
                    .finS <- c(p(e1)(0)<ep,p(e1)(0)>1-ep)
                  }
            if(e2<0) .finS <- rev(.finS)
                 supportnew <- e1@support * e2
                 if (e2 < 0) supportnew <- rev(supportnew)

                 coR <- substitute({
                             o.warn <- getOption("warn"); options(warn = -1)
                             on.exit(options(warn=o.warn))
                             #
                             x0 <- .setEqual(q / e2C, support(object))
                             d0 <- object@d(x = x0)*(x0 %in% support(object))
                             #
                             options(warn = o.warn)
                             if (!lower.tail) d0 <- -d0
                             p0 <- p0 + d0},
                             list(e2C = e2)
                             )

                 dnew <- .makeD(substitute(e1, list(e1 = e1)),
                                substitute(alist(x = x / e2), list(e2 = e2)))
                 pnew <- .makeP(substitute(e1, list(e1 = e1)),
                                substitute(alist(q = q / e2), list(e2 = e2)),
                                sign = e2>0, correct = coR)
                 qnew <- .makeQ(substitute(e1, list(e1 = e1)),
                                substitute(q0 * e2, list(e2 = e2)),
                                sign = e2>0, Cont = FALSE)

                 object <- new(Dclass, r = rnew, d = dnew, p = pnew,
                               q = qnew, support = supportnew,
                               a = e1@a * e2, b = e2 * e1@b, X0 = e1@X0,                              
                              .withSim = FALSE, .withArith = TRUE,
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1),
                    .finSupport = .finS)
                 rm(supportnew)

            }else if (Dclass == "DiscreteDistribution"){
                  if(is.finite(e2)){
                     .finS <- e1@.finSupport
                  }else{
                     ep <- .Machine$double.eps
                    .finS <- c(p(e1)(0)<ep,p(e1)(0)>1-ep)
                  }
                 supportnew <- e1@support * e2
                 if (e2 < 0) supportnew <- rev(supportnew)

                 coR <- substitute({
                             o.warn <- getOption("warn"); options(warn = -1)
                             on.exit(options(warn=o.warn))
                             d0 <- object@d(x = q / e2C)
                             options(warn = o.warn)
                             if (!lower.tail) d0 <- -d0
                             p0 <- p0 + d0},
                             list(e2C = e2)
                             )

                 dnew <- .makeD(substitute(e1, list(e1 = e1)),
                                substitute(alist(x = x / e2), list(e2 = e2)))
                 pnew <- .makeP(substitute(e1, list(e1 = e1)),
                                substitute(alist(q = q / e2), list(e2 = e2)),
                                sign = e2>0, correct = coR)
                 qnew <- .makeQ(substitute(e1, list(e1 = e1)),
                                substitute(q0 * e2, list(e2 = e2)),
                                sign = e2>0, Cont = FALSE)
                 object <- new("AffLinDiscreteDistribution", r = rnew, d = dnew,
                               p = pnew, q = qnew, support = supportnew,
                               a = e2, b = 0, X0 = e1,                              
                              .withSim = FALSE, .withArith = TRUE,
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1),
                    .finSupport = .finS)
                 rm(supportnew)

            }else if (Dclass == "AffLinAbscontDistribution"){
                 trY <- try(
                 if(is.null(e1@gaps))
                    gapsnew <- NULL
                 else {gapsnew <- e1@gaps
                       if(nrow(gapsnew)){
                       if(is.numeric(gapsnew)&&length(gapsnew)>0){
                          gapsnew <- matrix(gapsnew * e2, ncol=2)
                          if (e2 < 0) gapsnew <-
                             gapsnew[rev(seq(nrow(gapsnew))),c(2,1),drop = FALSE]
                       }}
                 }, silent=TRUE)
                 if(is(trY,"try-error")) gapsnew <- NULL
                 dnew <- .makeD(substitute(e1, list(e1 = e1)),
                                substitute(alist(x = x / e2), list(e2 = e2)),
                                stand = abs(e2))
                 pnew <- .makeP(substitute(e1, list(e1 = e1)),
                                substitute(alist(q = q / e2), list(e2 = e2)),
                                sign = e2>0)
                 qnew <- .makeQ(substitute(e1, list(e1 = e1)),
                                substitute(q0 * e2, list(e2 = e2)),
                                sign = e2>0)
                 object <- new(Dclass, r = rnew, d = dnew, p = pnew, q = qnew, 
                               gaps = gapsnew, a = e1@a * e2, b = e2 * e1@b, 
                               X0 = e1@X0, .withSim = FALSE, .withArith = TRUE,
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
 
            }else if (Dclass == "AbscontDistribution"){
                 trY <- try(
                 if(is.null(e1@gaps))
                    gapsnew <- NULL
                 else {gapsnew <- e1@gaps
                       if(is.numeric(gapsnew)&&length(gapsnew)>0){
                             gapsnew <- matrix(gapsnew * e2, ncol=2)
                             if (e2 < 0) gapsnew <-
                                    gapsnew[rev(seq(nrow(gapsnew))),
                                            c(2,1),drop = FALSE]
                             }
                 }, silent=TRUE)
                 if(is(trY,"try-error")) gapsnew <- NULL

                 dnew <- .makeD(substitute(e1, list(e1 = e1)),
                                substitute(alist(x = x / e2), list(e2 = e2)),
                                stand = abs(e2))
                 pnew <- .makeP(substitute(e1, list(e1 = e1)),
                                substitute(alist(q = q / e2), list(e2 = e2)),
                                sign = e2>0)
                 qnew <- .makeQ(substitute(e1, list(e1 = e1)),
                                substitute(q0 * e2, list(e2 = e2)),
                                sign = e2>0)
                 object <- new("AffLinAbscontDistribution", r = rnew, d = dnew, 
                               p = pnew, q = qnew, gaps = gapsnew, a = e2, 
                               b = 0, X0 = e1, .withSim = FALSE, 
                               .withArith = TRUE,
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
            }
            rm(pnew, qnew, dnew, rnew)
            object
          }
          

#------------------------------------------------------------------------------
# .makeDd, .makePd, .makeQd
#------------------------------------------------------------------------------



.makeDd <- function(x,y, yleft, yright){
   intervall <- getdistrOption("DistrResolution") / 2
   supp.grid <- c(matrix(rbind(x - intervall, x + intervall), nrow = 1))
   prob.grid <- c(matrix(rbind(0, y), nrow = 1), 0)
   df0 <- stepfun(x = supp.grid, y = prob.grid)
   rm(intervall, supp.grid, prob.grid)
   return(df0)
}

.makePd <- function(x,y, yleft, yright){
   stepfun(x = x, y = c(yleft, y))
}

.makeQd <- function(x,y, yleft, yright){
force(y)
force(x)
f <- function(u) {
               q0 <- sapply(u, 
                       function(z) y[min(sum(x < z-.Machine$double.eps) + 1,
                                         length(y)) ] )
               q0[.isEqual(u,0)] <- yleft
               q0[.isEqual(u,1)] <- yright
               return(q0)}
### individualize local variables
#environment(f) <- new.env()
#assign("y", y, environment(f))
#assign("x", x, environment(f))
#assign("yright", yright, environment(f))
#assign("yleft", yleft, environment(f))
return(f)
}

.makeQc <- function(x,y, yleft, yright){
#f0 <- function(u) {
#               q0 <- sapply(u, 
#                       function(z) y[min(sum(x < z-.Machine$double.eps) + 1,
#                                         length(y)) ] )
#               return(q0)}
#eps <- getdistrOption("TruncQuantile")
#x00 <- seq(eps, 1-eps, length = getdistrOption("DefaultNrGridPoints"))
#y00 <- f0(x00)
#idx <- cumsum(rle(y00)[[1]]) ### use only unique y's and corresponding 
#x0 <- x00[idx]               ### maximal x's
#y0 <- y00[idx]
#f1 <- approxfun(x = x0, y = y0, yleft = y0[1], yright = y0[length(y0)])
yleft <- yleft[1]
yright <- yright[1]

isna <- is.na(x)|is.na(y)
x <- x[!isna]
y <- y[!isna]

l0 <- length(unique(x[!.isEqual01(x)]))
if(l0 > 1){
   yl <- if(!is.na(yleft) && is.finite(yleft))  yleft  else y[1]
   yr <- if(!is.na(yright)&& is.finite(yright)) yright else y[length(y)]

   f1 <- approxfun(x = x, y = y, yleft = yl, yright = yr)
}else{ 
   i0 <- (1:length(x))[x==unique(x[!.isEqual01(x)])]
   y0 <- if(l0 ==1) y[min(i0)] else yleft
   f1 <- function(x) return(y0)
}
f <- function(x) 
   {y1 <- f1(x)
    y1[.isEqual(x,0)] <- yleft
    y1[.isEqual(x,1)] <- yright
    return(y1)
    }
return(f)
}


#------------------------------------------------------------------------------
# .makeDNew, .makePNew, .makeQNew
#------------------------------------------------------------------------------


.makeDNew <- function(x, dx, h = NULL, Cont = TRUE, standM = "sum"){
            dx <- (dx >= .Machine$double.eps)*dx
            if( length(dx) < length(x) ) dx <- c(0,dx)

            if (is.null(h)) h <- 1

            dx1 <- dx / h
            mfun <- if (Cont) approxfun else .makeDd

            ## density
            df1 <- mfun(x = x, y = dx1, yleft = 0, yright = 0)

            if (standM == "sum")
                   stand <- sum(dx)
            else   {
            stand <- try(integrate(df1, -Inf, Inf)$value, TRUE)
            if (is(stand,"try-error")){
               if(getdistrOption("warn.makeDNew"))
                  warning("'integrate()' threw an error ---result may be inaccurate.")
               stand <- sum(df1(x))*h*(x[2]-x[1])
               }
            }
            dfun <- function(x, log = FALSE)
                    {if (log)
                          d0 <-    log(df1(x))-log(stand)
                     else d0 <- df1(x) / stand
                     return (d0)}
            rm(x,dx1,h)
            return(dfun)
}

.primefun <- function(f,x, nm = NULL){
 
 h <- diff(x)
 l <- length(x)

 xm <- (x[-l]+x[-1])/2

 fxm <- f(xm)
 fx <- f(x)
 
 
 fxs  <- 2 * cumsum(fx) - fx - fx[1]
 fxsm <- 4 * cumsum(fxm)

 fxx <- c(0, (fxs[-1]+fxsm)* h / 6 )

 if (is.null(nm)) nm <- fxx[l]

 fx1 <- approxfun(x, fxx, yright = nm, yleft = 0)

 ffx <- function(u){
      ffy <- fx1(u) 
      ffy[u > max(x)] <- nm 
      ffy[u < min(x)] <- 0
      return(ffy)
     }

 return(ffx)
}

.csimpsum <- function(fx){
 l <- length(fx)
 l2 <- l%/%2
 if (l%%2 == 0) {
     fx <- c(fx[1:l2],(fx[l2]+fx[l2+1])/2,fx[(l2+1):l])
     l <- l+1}
 f.even <- fx[seq(l) %% 2 == 0]
 f.odd  <- fx[seq(l) %% 2 == 1]
 fs    <- 2 * cumsum(f.odd) - f.odd - f.odd[1]
 fsm   <- 4 * cumsum(f.even)
 ff <- c(0,(fs[2:(l2+1)]+fsm)/3 )
 ff
}

.makePNew <- function(x, dx, h = NULL, notwithLLarg = FALSE,
                      Cont = TRUE, myPf = NULL, pxl = NULL, pxu = NULL){

  if (is.null (h)) h <- 0

  x.u <- x.l <- x
  if (Cont){
         mfun <- if (is.null (myPf)) approxfun else myPf
         l <- length(x)
         if ((l%%2==0)&& is.null(myPf)){
               l2 <- l/2
               if (is.null(pxl))
                   x.l <- c(x[1:l2],(x[l2]+x[l2+1])/2,x[(l2+1):l])
               if (is.null(pxu))
                   x.u <- c(x[1:l2],(x[l2]+x[l2+1])/2,x[(l2+1):l])
               l <- l+1
               }
         cfun <- .csimpsum
         if (is.null(pxl)&& is.null(myPf))
             x.l <- x.l[seq(l)%%2==1]
         if (is.null(pxu)&& is.null(myPf))
             x.u <- x.u[seq(l)%%2==1]
  }else    {
         mfun <- .makePd
         cfun <- cumsum
  }       

  p.l <- if(!is.null(pxl)) pxl else cfun(dx)
  
  nm <- max(p.l)
  p1.l <- mfun(x = x.l, y = p.l, yleft = 0, yright = nm)
  nm <- p1.l(max(x))

  if(notwithLLarg){
      ifElsePS <- substitute(if (lower.tail) p1.l(q) else 1 - p1.l(q))
  }else{
      p.u <- if(!is.null(pxu)) pxu else rev(cfun(rev(dx)))
    ## continuity correction by h/2
      if (!Cont) p.u <- c(p.u[-1],0)
      p1.u <- mfun(x = x.u, y = p.u, yright = 0, yleft = nm)
      rm(p.u)
      ifElsePS <- substitute(if (lower.tail) p1.l(q) else p1.u(q))
  }
  pfun <- function(q, lower.tail = TRUE, log.p = FALSE){}
  body(pfun) <- substitute(
              { p0 <- ifElsePC
                p0 <- if (log.p) log(p0)-log(nm) else p0/nm
                return(p0)
              }, list(ifElsePC = ifElsePS))
  rm(dx, p.l, notwithLLarg)
  return(pfun)
}


.makeQNew <- function(x, px.l, px.u, notwithLLarg = FALSE, yL , yR,
                      Cont = TRUE){
  o.warn <- getOption("warn"); options(warn = -1)
  on.exit(options(warn=o.warn))
  ix <- .isEqual01(px.l)
  mfun <- if (Cont) .makeQc else  .makeQd
  if(!is.finite(yR)||Cont)
     {xx <- px.l[!ix]; yy <- x[!ix]}
  else  
     {xx <- px.l; yy <- x}
  q.l0 <- mfun(x = xx, y = yy, yleft = yL, yright = yR)
  rm(xx,yy)
  if(notwithLLarg){
     ifElseQS <- quote(if (lower.tail) q.l0(p01) else q.l0(1-p01))
  }else{
#         px.u <- rev(px.u);
         x <- rev(x)
         if (Cont) px.u <- rev(px.u)
         ix <- .isEqual01(px.u)
         xx <- px.u[!ix]
         yy <- if (Cont) x[!ix] else x[rev(!ix)]
         q.u <- mfun(x = xx, y = yy, yleft = yR, yright = yL)
         rm(xx,yy)
     ifElseQS <- quote(if (lower.tail) q.l0(p01) else q.u(p01))
  }
  options(warn = o.warn)
  qfun <- function(p, lower.tail = TRUE, log.p = FALSE){}
  body(qfun) <- substitute({
          if (log.p) p <- exp(p)
          if (any((p < -.Machine$double.eps)|(p > 1+.Machine$double.eps)))
              warning(gettextf("q method of %s produced NaN's ", objN))
              i01 <- (-.Machine$double.eps<=p)&(p<=1+.Machine$double.eps)
              p01 <- p[i01] ## only values in [0,1] are used
              q0  <- p*0
              q0[!i01] <- NaN
              q0[ i01] <- ifElseQC
              return(as.numeric(q0))
              }, list(ifElseQC = ifElseQS, objN = quote(.getObjName(1))))
  return(qfun)
}




#setMethod("+", c("LatticeDistribution","AbscontDistribution"),plusDAC)

#------------------------------------------------------------------------------
# .expm.[dc], .logm.[dc] (exact calculation of exp /log) 
#    [ similarly possible : other monotone, smooth trafos, e.g. tan / atan ]
#------------------------------------------------------------------------------
.expm.d <- function(e1){
            supportnew <- exp(e1@support)

            rnew <- function(n, ...){}
            body(rnew) <- substitute({ exp(f(n, ...)) },
                                         list(f = e1@r))
            dnew <- .makeD(substitute(e1, list(e1 = e1)),
                           substitute(alist(x = log(x*(x>0)+(x<=0)))),
                           fac = substitute((x>0)))
            pnew <- .makeP(substitute(e1, list(e1 = e1)),
                           substitute(alist(q = log(q*(q>0)+(q<=0)))),
                           fac = substitute((q>0)),
                           fac2 = substitute((q<=0)& !lower.tail))
            qnew <- .makeQ(substitute(e1, list(e1 = e1)),
                           substitute(exp(q0)),
                           Cont = FALSE)

            object <- new("DiscreteDistribution",
                           r = rnew, d = dnew, p = pnew,
                           q = qnew, support = supportnew,
                          .withSim = FALSE, .withArith = TRUE,
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1),
                    .finSupport = c(TRUE,e1@.finSupport[2]))
            rm(supportnew)
            rm(pnew, qnew, dnew, rnew)
            object
          }
.expm.c <- function(e1){
            gapsnew <- if(is.null(e1@gaps)) NULL else exp(e1@gaps)

            rnew <- function(n, ...){}
            body(rnew) <- substitute({ exp(f(n, ...)) },
                                         list(f = e1@r))
                                    
            
            dnew0 <- .makeD(substitute(e1, list(e1 = e1)),
                            substitute(alist(x = log(x*(x>0)+(x<=0)))),
                            stand = substitute(x+(x==0)),
                            fac = substitute((x>0)))

            # extrapolation for x=0
            x0a <- 10^(-(1:10)/4) 
            f0a <- dnew0(x = x0a, log = FALSE)
            f0  <- max(spline(x0a,f0a, xout = 0)$y,0)
            lf0 <- log(f0)

            dnew <- function(x, log = FALSE, ...){
               d0 <- dnew0(x, log = log, ...)
               i0 <- .isEqual(x,0)
               if(!any(i0)) return(d0)
               if(log) d0[i0] <- lf0 else d0[i0] <- f0
               return(d0)
            }
            pnew <- .makeP(substitute(e1, list(e1 = e1)),
                           substitute(alist(q = log(q*(q>0)+(q<=0)))),
                           fac = substitute((q>0)),
                           fac2 = substitute((q<=0)& !lower.tail))
            qnew <- .makeQ(substitute(e1, list(e1 = e1)),
                           substitute(exp(q0)))

            object <- AbscontDistribution(
                           r = rnew, d = dnew, p = pnew,
                           q = qnew, gaps = gapsnew,
                          .withSim = FALSE, .withArith = TRUE,
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
            if(exists("gapsnew")) rm(gapsnew)
            rm(pnew, qnew, dnew, rnew)
            object
          }
.logm.d <- function(e1){
            supportnew <- log(e1@support)

            rnew <- function(n, ...){}
            body(rnew) <- substitute({ log(f(n, ...)) },
                                         list(f = e1@r))
            dnew <- .makeD(substitute(e1, list(e1 = e1)),
                           substitute(alist(x = exp(x))))
            pnew <- .makeP(substitute(e1, list(e1 = e1)),
                           substitute(alist(q = exp(q))))
            qnew <- .makeQ(substitute(e1, list(e1 = e1)),
                           substitute(log(q0)),
                           Cont = FALSE)

            object <- new("DiscreteDistribution",
                           r = rnew, d = dnew, p = pnew,
                           q = qnew, support = supportnew,
                          .withSim = FALSE, .withArith = TRUE,
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1),
                    .finSupport = c(TRUE,e1@.finSupport[2]))
            rm(supportnew)
            rm(pnew, qnew, dnew, rnew)
            object
          }
.logm.c <- function(e1){
            gapsnew <- if(is.null(e1@gaps)) NULL else log(e1@gaps)

            rnew <- function(n, ...){}
            body(rnew) <- substitute({ log(f(n, ...)) },
                                         list(f = e1@r))
            dnew <- .makeD(substitute(e1, list(e1 = e1)),
                           substitute(alist(x = exp(x))),
                           fac = substitute(exp(x)))
            pnew <- .makeP(substitute(e1, list(e1 = e1)),
                           substitute(alist(q = exp(q))))
            qnew <- .makeQ(substitute(e1, list(e1 = e1)),
                           substitute(log(q0)))

            object <- AbscontDistribution(
                           r = rnew, d = dnew, p = pnew,
                           q = qnew, gaps = gapsnew,
                          .withSim = FALSE, .withArith = TRUE,
                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
            if(exists("gapsnew")) rm(gapsnew)
            rm(pnew, qnew, dnew, rnew)
            object
          }


#------------------------------------------------------------------------------
# .P2D, .D2P, .P2Q, .Q2P casting functions
#------------------------------------------------------------------------------

###Functions for AbscontDistribution 

#determines slot d from p
.P2D <- function(p, xx, ql, qu, ngrid = getdistrOption("DefaultNrGridPoints"))
{if(missing(xx))
    xx <- seq(ql, qu, length = ngrid)
 px <- p(xx)
 dx <- D1ss(xx,px)
 return(.makeDNew(xx, dx, h = NULL, Cont = TRUE, standM = "integrate"))
}


#determines slot q from p

.P2Q <- function(p, xx, ql,qu, ngrid = getdistrOption("DefaultNrGridPoints"), 
                qL = -Inf, qU = Inf){
if(missing(xx))
   {xx <- seq(ql, qu, length = ngrid)
     h <- (qu-ql)/ngrid}
else h <- c(diff(xx),(rev(xx)[1]-rev(xx)[2]))
px.l <- p(xx + 0.5*h)
px.u <- p(xx + 0.5*h, lower.tail = FALSE)
return(.makeQNew(xx + 0.5*h, px.l, px.u, FALSE, qL, qU))
}

#determines slot p from d
.D2P <- function(d, xx, ql, qu,  ngrid = getdistrOption("DefaultNrGridPoints"))
{if(missing(xx))
   { if(ngrid%%2==0) ngrid  <- ngrid+1
     xx <- seq(ql, qu, length = ngrid)
     h <- (qu-ql)/ngrid}
 else h <- c(diff(xx),(rev(xx)[1]-rev(xx)[2]))

 dx <- d(xx)
 return(.makePNew(xx, dx, h = h))
}

#determines slot p from q

.Q2P <- function(q, ngrid = getdistrOption("DefaultNrGridPoints")){
q.l <- q
ep <- getdistrOption("TruncQuantile")^2
xx0 <- seq(ep, 1-ep, length = ngrid)
qx0 <- q.l(xx0)
qx1 <- unique(qx0)
x1 <- tapply(xx0, qx0, max)
p0 <- approxfun(x = qx1, y = x1, yleft = 0, yright = 1, rule = 2)
p01 <- function(x) {
       p11 <- p0(x) 
       p11[x>max(qx1)] <- 1
       p11[x<min(qx1)] <- 0
       return(p11)}
return(function(q, lower.tail = TRUE, log.p = FALSE){
  p1 <- p01(q)
  p1[q==Inf] <- 1
  p1[q==-Inf] <- 0
  if(!lower.tail) p1 <- 1-p1
  if(log.p) p1 <- log(p1)
  return(p1)
})
}

#------------------------------------------------------------------------------
# modify slot q for AbscontDistribution if there are gaps
#------------------------------------------------------------------------------
.modifyqgaps <- function(pfun, qfun, gaps, leftright = "left"){
  ## no modification needed if gaps have no length

  if(length(gaps)==0) return(qfun)
  if(is.null(dim(gaps))) return(qfun)

  finit <- apply(gaps, 1, function(x) all(is.finite(x)&is.numeric(x)))
  if(sum(finit)==0) return(qfun)

  gaps <- gaps[finit,,drop=FALSE]
#  print(gaps)
  ## p-level of constancy region

  if(.inArgs("log.p", pfun)){
       lp.gaps <- matrix(pfun(gaps,log.p=TRUE),nrow=nrow(gaps),ncol=2)
       if(.inArgs("lower.tail", pfun))
          lp.gaps.l <- matrix(pfun(gaps,log.p=TRUE, lower.tail = FALSE),nrow=nrow(gaps),ncol=2)
       else
          lp.gaps.l <- 1-matrix(pfun(gaps,log.p=TRUE),nrow=nrow(gaps),ncol=2)
  }else{
       lp.gaps <- log(matrix(pfun(gaps),nrow=nrow(gaps),ncol=2))
       if(.inArgs("lower.tail", pfun))
          lp.gaps.l <- log(matrix(pfun(gaps,lower.tail = FALSE),nrow=nrow(gaps),ncol=2))
       else
          lp.gaps.l <- log1p(-matrix(pfun(gaps),nrow=nrow(gaps),ncol=2))
  }
#  print(lp.gaps)
#  print(lp.gaps.l)

  ## are we heading for left or right continuous quantile fct?
  lrpmatch <- pmatch(leftright, table = c("left","right"), nomatch = 1)

  ## in order to avoid chaining of qgaps modifications:
  ## place a variable ..q0fun into modified quantile function (after modification)
  ##      which stores the unmodified quantile function
  ## in the first modification round ..q0fun will not yet exist
  ##    in this situation use qfun instead
  qfunN <- qfun
  if(!.inArgs("log.p", qfun) || !.inArgs("lower.tail", qfun)){
    qfunN <- function(p, lower.tail = TRUE, log.p = FALSE){
        if(.inArgs("lower.tail",qfun)) if(log.p) p <- exp(p)
        if(.inArgs("log.p", qfun)){
              p1 <- if(log.p) exp(p) else p
              qval <- if(lower.tail) qfun(p, log.p) else qfun(1-p1, log.p=FALSE)
        }else{
           if(!.inArgs("lower.tail",qfun)){
              qval <- if(lower.tail) qfun(p) else qfun(1-p)
           }else{
              qval <- qfun(p,lower.tail)
           }
        }
    return(qval)
    }
  }


  qfunE <- environment(qfunN)
  qnew <- function(p, lower.tail = TRUE, log.p = FALSE) {}
  qnewE <- environment(qnew) <- new.env()
  body(qnew) <- substitute({
          ## .q0fun is the (gaps-)unmodified quantile function
          .q0fun <- if(exists("..q0fun", envir=qfunE.)){
                       get("..q0fun", envir=qfunE.) } else qfun.
          q0 <- .q0fun(p, lower.tail = lower.tail, log.p = log.p)
          ## the gaps-modification: find out which args p coincide
          ##     (numerically, on log scale) with gaps-plevels;
          ##     depending on "leftright" and lower.tail
          ##     set these return values to left or right endpoit of the gap
          if(length(lp.gaps.)>0){
              i0 <- seq(length=length(p))
              lg <- round(3/2-(2*lower.tail-1)*(2*(lrpmatch.==1)-1)/2)
                   ## ==1 if(lower.tail&&leftright==1) or (!lower.tail&&leftright!=1)
                   ## and == 2 otherwise
              lpgaps0 <- if(lg==1L) lp.gaps. else lp.gaps.l.
              for(i in 1:nrow(lpgaps0)){
                  i0 <- (log(p)>=lpgaps0[i,1])&(log(p)<=lpgaps0[i,2])
                  if(length(i0)) q0[i0] <- gaps.[i,lg]
              }
          }
          return(q0)
  },list(qfunE. = qfunE, qfun.=qfunN, lp.gaps.=lp.gaps,
         lp.gaps.l. = lp.gaps.l[,2:1,drop=FALSE],
         lrpmatch. = lrpmatch, gaps. = gaps, ..isEqual = .isEqual)
  )
  if(exists("..q0fun", envir=qfunE)){
     .q0fun <- get("..q0fun", envir=qfunE)
     assign("..q0fun", .q0fun, envir = qnewE)
  }else{
     assign("..q0fun", qfun, envir = qnewE)
  }
  return(qnew)
}

if(FALSE){ ## old code
.modifyqgaps <- function(pfun, qfun, gaps, leftright = "left"){
  if(length(gaps)==0) return(qfun)
  p.gaps <- pfun(gaps[,1]) 
  p.gaps.l <- pfun(gaps[,1], lower.tail = FALSE)
  dP <- deparse(body(qfun))
  syC <- paste(sys.calls())
  if(length(grep("q\\.r\\(",syC[length(syC)-2])) == 0)
     if(length(grep("q0 <- qfun", dP[2]))>0) 
        return(qfun)
  if(pmatch(leftright, table = c("left","right"), nomatch = 1) == 1){
     qnew <- function(p, lower.tail = TRUE, log.p = FALSE) {
             q0 <- qfun(p, lower.tail = lower.tail, log.p = log.p)
             i0 <- seq(length=length(p))
             if(lower.tail){
                if(log.p) p.gaps <- log(p.gaps)
                for(i in 1:nrow(gaps)){
                    i1 <- .isEqual(p,p.gaps[i])
                    i2 <- i0[p<p.gaps[i]]
                    i3 <- i0[p>p.gaps[i]]
                    q0[i1] <- gaps[i,1]
                    if(length(i2)>0)
                       q0[i2] <- pmin(q0[i2],gaps[i,1])
                    if(length(i3)>0)
                       q0[i3] <- pmax(q0[i3],gaps[i,2])
                }        
             }else{
                if(log.p) p.gaps.l <- log(p.gaps.l)
                for(i in 1:nrow(gaps)){
                    i1 <- .isEqual(p,p.gaps.l[i])
                    i2 <- i0[p<p.gaps.l[i]]
                    i3 <- i0[p>p.gaps.l[i]]
                    q0[i1] <- gaps[i,2]
                    if(length(i2)>0)
                       q0[i2] <- pmax(q0[i2],gaps[i,2])    
                    if(length(i3)>0)
                       q0[i3] <- pmin(q0[i3],gaps[i,1])    
                }        
             }
             return(q0)
     }
  }else{
     qnew <- function(p, lower.tail = TRUE, log.p = FALSE) {
             q0 <- qfun(p, lower.tail = lower.tail, log.p = log.p)
             i0 <- seq(length=length(p))
             if(lower.tail){
                if(log.p) p.gaps <- log(p.gaps)
                for(i in 1:nrow(gaps)){
                    i1 <- .isEqual(p,p.gaps[i])
                    i2 <- i0[p<p.gaps[i]]
                    i3 <- i0[p>p.gaps[i]]
                    q0[i1] <- gaps[i,2]
                    if(length(i2)>0)
                       q0[i2] <- pmin(q0[i2],gaps[i,1])
                    if(length(i3)>0)
                       q0[i3] <- pmax(q0[i3],gaps[i,2])
                }        
             }else{
                if(log.p) p.gaps.l <- log(p.gaps.l)
                for(i in 1:nrow(gaps)){
                    i1 <- .isEqual(p,p.gaps.l[i])
                    i2 <- i0[p<p.gaps.l[i]]
                    i3 <- i0[p>p.gaps.l[i]]
                    q0[i1] <- gaps[i,1]
                    if(length(i2)>0)
                       q0[i2] <- pmax(q0[i2],gaps[i,2])    
                    if(length(i3)>0)
                       q0[i3] <- pmin(q0[i3],gaps[i,1])    
                }        
             }
             return(q0)
     }
  }
  return(qnew)           
}
}
#------------------------------------------------------------------------------
# issue warnings in show / print as to Arith or print
#------------------------------------------------------------------------------
.IssueWarn <- function(Arith,Sim){
    msgA1 <- msgA2 <- msgS1 <- msgS2 <- NULL
    if(Arith && getdistrOption("WarningArith")){ 
      msgA1 <- gettext(
       "arithmetics on distributions are understood as operations on r.v.'s\n")
      msgA2 <- gettext(
       "see 'distrARITH()'; for switching off this warning see '?distroptions'")
       }
    if(Sim && getdistrOption("WarningSim")){ 
      msgS1 <- gettext(
       "slots d,p,q have been filled using simulations; ")
      msgS2 <- gettext(
       "for switching off this warning see '?distroptions'")
       }
    return(list(msgA=c(msgA1,msgA2), msgS = c(msgS1,msgS2)))  
    }

#------------------------------------------------------------------------------
# fill a list acc. recycling rules
#------------------------------------------------------------------------------
.List <- function(list0) if(is.list(list0)) list0 else list(list0)

.fillList <- function(list0, len = length(list0)){
            if(is.null(list0)) return(vector("list",len))
            list0 <- .List(list0)
            if(len == length(list0)) 
               return(list0)
            i <- 0
            ll0 <- length(list0)
            li0 <- vector("list",len)
            if(ll0){
              while(i < len){
                 j <- 1 + ( i %% ll0)
                 i <- i + 1
                 li0[[i]] <- list0[[j]]
              }
            }
            return(li0)
}

#------------------------------------------------------------------------------
# .DistributionAggregate by Jacob van Etten, jacobvanetten@yahoo.com
# on a mail on Feb 27th 2009
#------------------------------------------------------------------------------
.DistrCollapse <- function(support, prob, 
                              eps = getdistrOption("DistrResolution")){
    supp <- support
    prob <- as.vector(prob)
    suppIncr <- diff(c(supp[1]-2*eps,supp)) < eps
    groups <- cumsum(!suppIncr)
    prob <- as.vector(tapply(prob, groups, sum))
    supp <- as.vector(tapply(supp, groups, quantile, probs = 0.5, type = 1)) 
           ### in order to get a "support member" take the leftmost median
    return(list(supp = supp, prob = prob))
#    newDistribution <- DiscreteDistribution(supp=supp,prob=prob)
#    return(newDistribution)
}


.panel.mingle <- function(dots, element){
  pF <- dots[[element]]
  if(is.list(pF)) return(pF)
  pFr <- if(typeof(pF)=="symbol") eval(pF) else{
     pFc <- as.call(pF)
     if(as.list(pFc)[[1]] == "list"){
        lis <- vector("list",length(as.list(pFc))-1)
        for(i in 1:length(lis)){
            lis[[i]] <- pFc[[i+1]]
        }
        lis
     }else pF
  }
  return(pFr)
}

################################
##
## Class: NormParameter
##                             
################################



## Access methods
setMethod("mean", "NormParameter", function(x, ...) x@mean)
setMethod("sd", signature(x = "NormParameter"), function(x, ...) x@sd)
## Replace Methoden
setReplaceMethod("mean", "NormParameter", 
                  function(object, value){ object@mean <- value; object})
setReplaceMethod("sd", "NormParameter", 
                  function(object, value)
                      { object@sd <- as.matrix(value); object})


validNormParameter <- function(object){
    if(!is.matrix(sd(object)) && is.numeric(mean(object))) return(TRUE)
    if(nrow(sd(object)) != ncol((sd(object))))
        stop("Covariance matrix not sqared")
    if(nrow(sd(object)) != length(mean(object)))
        stop("Covariance matrix and mean vector do not have the same dimension")
    return(TRUE)
}

setValidity("NormParameter", validNormParameter)


################################
##
## Class: UniNormParameter
##
################################

setClass("UniNormParameter", contains = "NormParameter")


setValidity("UniNormParameter", function(object){
  if(length(mean(object)) != 1)
    stop("mean has to be a numeric of length 1")    
  if(length(sd(object)) != 1)
    stop("sd has to be a numeric of length 1")    
  sd <- as.numeric(sd(object))
  if(sd <= 0)
    stop("sd has to be positive")
  else return(TRUE)
})


################################
##
## Class: normal distribution
##
################################

Norm <- function(mean = 0, sd = 1) {
   N <- new("Norm", mean = mean, sd = sd)
   N@Symmetry <- SphericalSymmetry(mean)
   N}

## wrapped access methods
setMethod("mean", "Norm", function(x, ...) mean(param(x)))
setMethod("sd", signature(x = "Norm"), function(x) sd(param(x)))
## wrapped replace methods 
setMethod("mean<-", "Norm", 
           function(object, value) new("Norm", mean = value, sd = sd(object)))
setMethod("sd<-", "Norm", 
           function(object, value) new("Norm", mean = mean(object), sd = value))

## clipped moments for normal distribution: found in distrEx...

###setMethod("m1df", "Norm", 
###          function(object){
###            function(t) -d(object)(t) * sd(param(object))^2 
###                        + mean(param(object)) * p(object)(t)
###          })
###
###setMethod("m2df", "Norm", 
###          function(object){
###            mean <- mean(param(object))
###            sd <- sd(param(object))
###            d <- d(object)
###            p <- p(object)
###            function(t) -(t-mean) * d(t) * sd^2 + p(t) * sd^2 - 
###                        2 * mean * d(t) * sd^2 + mean^2 * p(t) 
###          })
###
## convolution operator for normal distributions

setMethod("+", c("Norm","Norm"),
          function(e1,e2){
              N<- new("Norm", sd = sqrt(sd(e1)^2 + sd(e2)^2), 
                   mean = mean(e1) + mean(e2),  .withArith = TRUE)
              N@Symmetry <- SphericalSymmetry(mean(e1)+mean(e2))
              N 
          })

## extra methods for normal distribution
setMethod("+", c("Norm","numeric"),
          function(e1, e2){
            if (length(e2)>1) stop("length of operator must be 1")
            N <- new("Norm", mean = mean(e1) + e2, sd = sd(e1), .withArith = TRUE) 
            N@Symmetry <- SphericalSymmetry(mean(e1)+ e2)
            N           
          })

setMethod("*", c("Norm","numeric"),
          function(e1, e2){
            if (length(e2)>1) stop("length of operator must be 1")
            if (isTRUE(all.equal(e2,0))) 
                return(new("Dirac", location = 0, .withArith = TRUE))
            N<- new("Norm", mean = mean(e1) * e2, 
                 sd = sd(e1) * abs(e2), .withArith = TRUE)
            N@Symmetry <- SphericalSymmetry(mean(e1)*e2)
            N           
          })

######## ExpOrGammaOrChisq - methods

setMethod("*", c("ExpOrGammaOrChisq","numeric"),
          function(e1, e2){
            if(is(e1,"Gammad")){
               if(isTRUE(all.equal(e2,0)))
                  return(new("Dirac", location = 0, .withArith = TRUE))
               if(e2 > 0)
                  return(new("Gammad", shape = shape(e1),
                              scale = scale(e1) * e2, .withArith = TRUE))
               return(getMethod("*",c("AbscontDistribution","numeric"))(
                                      e1 = Gammad(shape = shape(e1),
                                                  scale = scale(e1) * (-e2)), 
                                      e2 = -1)) 
             }                         
             else return(getMethod("*",
                            c("AbscontDistribution","numeric"))(e1, e2) )
          })

setMethod("+", c("ExpOrGammaOrChisq","ExpOrGammaOrChisq"),
          function(e1,e2){
            if(is(e1,"Gammad")&&is(e2,"Gammad"))
               {e10 <- as(e1,"Gammad")
                e20 <- as(e2,"Gammad")
                newshape <- shape(e10) + shape(e20)
                if(is.logical(all.equal(scale(e10),scale(e20))))
                   return(new("Gammad", shape = newshape,
                               scale = scale(e10), .withArith = TRUE))
               }
            return(as(e1, "AbscontDistribution") + e2)
          })

setIs("Exp", "Gammad", 
       coerce = function(obj){new("Gammad", shape = 1, scale = 1/rate(obj))},
       replace = function(obj, value) 
                     {new("Gammad", shape = value@shape, scale = value@scale)}
     )
   ## a Gamma distribution with shape = 1 and scale = 1/rate(obj)

setIs("Exp", "Weibull", 
       coerce = function(obj) {new("Weibull", shape = 1, scale = 1/rate(obj))},
       replace = function(obj, value) 
                     {new("Weibull", shape = value@shape, 
                                     scale = value@scale)}
     )
   ## a Weibull distribution with shape = 1 and scale = 1/rate(obj)

## if ncp == 0 a Gamma distribution with shape = df(obj)/2 and scale = 2
setIs("Chisq", "Gammad", test = function(obj) isTRUE(all.equal(ncp(obj), 0)),
       coerce = function(obj) {new("Gammad", shape = df(obj)/2, scale = 2)},
      replace = function(obj, value) 
                    {new("Gammad", shape = value@shape, scale = value@scale)}
      )


## if location==0 and scale==1 a T-distribution with df = 1
setIs("Cauchy", "Td", test = function(obj) 
                 {isTRUE(all.equal(location(obj),0)) && 
                  isTRUE(all.equal(scale(obj),1))}, 
       coerce = function(obj) {new("Td")},
       replace = function(obj, value)
                     {new("Td", df = value@df, ncp = value@ncp)}
       ) 


## if Min==0 and Max==1 a Beta Distribution with Param's shape1 = 1, shape2 = 2
setIs("Unif", "Beta", test = function(obj) 
             {isTRUE(all.equal(Min(obj),0)) && 
              isTRUE(all.equal(Max(obj),1))}, 
       coerce = function(obj) {new("Beta", shape1 = 1, shape2 = 1)},
       replace = function(obj, value) {new("Beta", shape1 = value@shape1, 
                          shape2 = value@shape2, ncp = value@ncp)}
       ) 

## if support is affine linear, a DiscreteDistribution is a LatticeDistribution
setAs("DiscreteDistribution", "LatticeDistribution",
      function(from){
        if(!.is.vector.lattice(from@support))
            return(from)
        else{ to <- new("LatticeDistribution")
              slotNames <- slotNames(from)
              lst <- sapply(slotNames, function(x) slot(from,x))
              names(lst) <- slotNames
              lst$lattice <- .make.lattice.es.vector(from@support)
              for (i in 1: length(lst))
                   slot(to, name = names(lst)[i]) <- lst[[i]]
              return(to)}
      })

## if support is affine linear, a DiscreteDistribution is a LatticeDistribution
setAs("AffLinDiscreteDistribution", "LatticeDistribution",
      function(from){
        if(!.is.vector.lattice(from@support))
            return(from)
        else{ to <- new("AffLinLatticeDistribution")
              slotNames <- slotNames(from)
              lst <- sapply(slotNames, function(x) slot(from,x))
              names(lst) <- slotNames
              lst$lattice <- .make.lattice.es.vector(from@support)
              for (i in 1: length(lst))
                   slot(to, name = names(lst)[i]) <- lst[[i]]
              return(to)}
      })

#setIs("DiscreteDistribution", "LatticeDistribution",
#      test = function(object) .is.vector.lattice(support(object)),
#      coerce = function(from) 
#               LatticeDistribution(DiscreteDistribution = from),
#      replace = function(from,value)
#                LatticeDistribution( 
#                r = value@r, d = value@d, q = value@q, p = value@p,
#                support = value@support, img =value@img, 
#                .withSim = value@.withSim, .withArith = value@.withArith,
#                lattice = value@lattice)
#      )


#setAs("LatticeDistribution", "DiscreteDistribution", 
#       function(from) 
#            new("DiscreteDistribution",  r = from@r, d = from@d, q = from@q, p = from@p,
#                support = from@support, img =from@img, 
#                .withSim = from@.withSim, .withArith = from@.withArith)
#      )

UnivarMixingDistribution <- function(..., Dlist, mixCoeff,
                                     withSimplify = getdistrOption("simplifyD"))
   {
    ldots <- list(...)
    if(!missing(Dlist)){
        Dlist.L <- as(Dlist, "list")
        if(!is(try(do.call(UnivarDistrList,args=Dlist.L),silent=TRUE),"try-error"))
            ldots <- c(ldots, Dlist.L)
       }
    l <- length(ldots)

    if(l==0) stop ("No components given")
    if(l==1) return(ldots[[1]])
    
    mixDistr <- do.call(UnivarDistrList,args=ldots)
    ep <- .Machine$double.eps
    if(missing(mixCoeff))
       mixCoeff <- rep(1,l)/l
    else{ if (l!=length(mixCoeff))
          stop("argument 'mixCoeff' and the mixing distributions must have the same length")
          if(any(mixCoeff < -ep) || sum(mixCoeff)>1+ep)
             stop("mixing coefficients are no probabilities")
        }
    rnew <- .rmixfun(mixDistr = mixDistr, mixCoeff = mixCoeff)

    pnew <- .pmixfun(mixDistr = mixDistr, mixCoeff = mixCoeff)


    .withArith <- any(as.logical(lapply(mixDistr, function(x) x@".withArith")))
    .withSim   <- any(as.logical(lapply(mixDistr, function(x) x@".withSim")))
    .lowerExact<- all(as.logical(lapply(mixDistr, function(x) x@".lowerExact")))

    if (all( as.logical(lapply(mixDistr, function(x) is(x,"AbscontDistribution")))) ||
        all( as.logical(lapply(mixDistr, function(x) is(x,"DiscreteDistribution")))))
        dnew <- .dmixfun(mixDistr = mixDistr, mixCoeff = mixCoeff)

    gaps <- NULL
    for(i in 1:l){
        if(is.null(gaps)){
           try(gaps <- gaps(mixDistr[[i]]), silent=TRUE)
        }else{
           if(!is(try(gaps0 <- gaps(mixDistr[[i]]), silent=TRUE),"try-error"))
               if(!is.null(gaps0)) gaps <- .mergegaps2(gaps,gaps0)
        }
    }    
    support <- numeric(0)
    for(i in 1:l){
        if(!is(try(support0 <- support(mixDistr[[i]]), silent=TRUE),"try-error"))
               support <- unique(sort(c(support,support0)))
    }    
    
    gaps <- .mergegaps(gaps,support)
    
    qnew <- .qmixfun(mixDistr = mixDistr, mixCoeff = mixCoeff,
                     Cont = TRUE, pnew = pnew, gaps = gaps)

    obj <- new("UnivarMixingDistribution", p = pnew, r = rnew, d = NULL, q = qnew,
         mixCoeff = mixCoeff, mixDistr = mixDistr, .withSim = .withSim,
         .withArith = .withArith,.lowerExact =.lowerExact, gaps = gaps, 
         support = support)

    if (all( as.logical(lapply(mixDistr, function(x) is(x@Symmetry,"SphericalSymmetry"))))){
       sc <- SymmCenter(mixDistr[[1]]@Symmetry) 
       if (all( as.logical(lapply(mixDistr, function(x) .isEqual(SymmCenter(x@Symmetry),sc)))))
           obj@Symmetry <- SphericalSymmetry(sc)    
    }
    
    
    if (withSimplify)
        obj <- simplifyD(obj)

    return(obj)
}


setMethod("mixCoeff", "UnivarMixingDistribution", function(object)object@mixCoeff)
setReplaceMethod("mixCoeff", "UnivarMixingDistribution", function(object,value){
   object@mixCoeff<- value; object})


setMethod("mixDistr", "UnivarMixingDistribution", function(object)object@mixDistr)
setReplaceMethod("mixDistr", "UnivarMixingDistribution", function(object,value){
   object@mixDistr<- value; object})

setMethod("support", "UnivarMixingDistribution", function(object)object@support)
setMethod("gaps", "UnivarMixingDistribution", function(object)object@gaps)


#------------------------------------------------------------------------
# new p.l, q.r methods
#------------------------------------------------------------------------

setMethod("p.l", signature(object = "UnivarMixingDistribution"),  
           function(object) .pmixfun(mixDistr = mixDistr(object), 
                                     mixCoeff = mixCoeff(object), 
                                     leftright = "left"))

setMethod("q.r", signature(object = "UnivarMixingDistribution"),  
           function(object){
                if(!is.null(gaps(object))&&length(gaps(object)))
                   .modifyqgaps(pfun = p(object), qfun = q.l(object),
                                gaps = gaps(object), leftright = "right")
                else
                    q.l(object)
            })

#------------------------------------------------------------------------
# new accessor methods
#------------------------------------------------------------------------

setMethod(".lowerExact", "UnivarMixingDistribution", function(object){ 
             er <- is(try(slot(object, ".lowerExact"), silent = TRUE), "try-error")
             if(er){ object0 <- conv2NewVersion(object)
                     objN <- paste(substitute(object))
                     warning(gettextf("'%s' was generated in an old version of this class.\n",
                                     objN),
                            gettextf("'%s' has been converted to the new version",objN),
                            gettextf(" of this class by a call to 'conv2NewVersion'.\n")
                            )           
                    eval.parent(substitute(object<-object0))                    
                    return(object0@.lowerExact)}
             object@.lowerExact})
setMethod(".logExact", "UnivarMixingDistribution", function(object){
             er <- is(try(slot(object, ".logExact"), silent = TRUE), "try-error")
             if(er){ object0 <- conv2NewVersion(object)
                     objN <- paste(substitute(object))
                     warning(gettextf("'%s' was generated in an old version of this class.\n",
                                     objN),
                            gettextf("'%s' has been converted to the new version",objN),
                            gettextf(" of this class by a call to 'conv2NewVersion'.\n")
                            )           
                    eval.parent(substitute(object<-object0))
                    return(object0@.logExact)}
             object@.logExact})
setMethod("Symmetry", "UnivarMixingDistribution", function(object){
             er <- is(try(slot(object, "Symmetry"), silent = TRUE), "try-error")
             if(er){ object0 <- conv2NewVersion(object)
                     objN <- paste(substitute(object))
                     warning(gettextf("'%s' was generated in an old version of this class.\n",
                                     objN),
                            gettextf("'%s' has been converted to the new version",objN),
                            gettextf(" of this class by a call to 'conv2NewVersion'.\n")
                            )           
                    eval.parent(substitute(object<-object0))
                    return(object0@Symmetry)}
             object@Symmetry})

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Vignette  "How to generate new distributions in packages distr, distrEx"
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%\VignetteIndexEntry{newDistributions}
%\VignetteDepends{startupmsg}
%\VignetteKeywords{probability distribution,estimation}
%\VignettePackage{distr}                     
%\VignetteEngine{knitr::knitr}
%
%
%
\documentclass[10pt]{article}
%
%use svn-multi to fill in revision information
%
\usepackage{svn-multi}
% Version control information:
\svnidlong
{$HeadURL: svn+ssh://ruckdeschel@svn.r-forge.r-project.org/svnroot/distr/pkg/distr/vignettes/newDistributions-knitr.Rnw $}
{$LastChangedDate: 2024-11-05 21:43:18 +0100 (Di, 05 Nov 2024) $}
{$LastChangedRevision: 1481 $}
{$LastChangedBy: ruckdeschel $}
%\svnid{$Id: example_main.tex 146 2008-12-03 13:29:19Z martin $}
% Don't forget to set the svn property 'svn:keywords' to
% 'HeadURL LastChangedDate LastChangedRevision LastChangedBy' or
% 'Id' or both depending if you use \svnidlong and/or \svnid
%
\newcommand{\svnfooter}{Last Changed Rev: \svnkw{LastChangedRevision}}
\svnRegisterAuthor{ruckdeschel}{Peter Ruckdeschel}
\svnRegisterAuthor{stamats}{Matthias Kohl}
\svnRegisterAuthor{florian}{Florian Camphausen}
\svnRegisterAuthor{stabla}{Thomas Stabla}
\svnRegisterAuthor{anhuel}{Anja H{\"u}ller}
\svnRegisterAuthor{ifrin}{Eleonara Feist}
\svnRegisterAuthor{jdospina}{Juan David Ospina}
\svnRegisterAuthor{kowzar}{Kouros Owzar}
%
\usepackage{geometry}
\usepackage[T1]{fontenc}
%
\usepackage{color}
\definecolor{darkblue}{rgb}{0.0,0.0,0.75}
\definecolor{distrCol}{rgb}{0.0,0.4,0.4}
\definecolor{Rcolor}{rgb}{0.0,0.5,0.5}
%
\usepackage{ifpdf}
%
\usepackage{url}
\usepackage{amssymb}
\usepackage[%
baseurl={r-forge.r-project.org},%
pdftitle={How to generate new distributions in packages distr, distrEx},%
pdfauthor={Peter Ruckdeschel, Matthias Kohl},%
pdfsubject={distr},%
pdfkeywords={probability distribution,simulation,estimation},%
pagebackref,bookmarks,colorlinks,linkcolor=darkblue,citecolor=darkblue,%
pagecolor=darkblue,raiselinks,plainpages,pdftex]{hyperref}
%
\markboth{\sl How to generate new distributions in packages ``{\tt distr}'', ``{\tt distrEx}''}%
{\sl How to generate new distributions in packages ``{\tt distr}'', ``{\tt distrEx}''}
%
% -------------------------------------------------------------------------------
\newcommand{\Reals}{\mathbb{R}}
\newcommand{\R}{\mathbb{R}}
\newcommand{\N}{\mathbb{N}}
%
% -------------------------------------------------------------------------------
\RequirePackage{fancyvrb}
\RequirePackage{listings}
%\usepackage{Sweave} no longer needed
% -------------------------------------------------------------------------------
%%\SweaveOpts{keep.source=TRUE}
% -------------------------------------------------------------------------------
%%
%% ------------------------------------------------------------------------------
%% before version 2.7, we used package SweaveListingUtils for syntax highlighting
%% from version 2.7 on, we switch to knitR
%% ------------------------------------------------------------------------------
%%
%%#(ll)SweaveListingsPreparations, results="asis", echo=FALSE(ggeq)
%%#require(SweaveListingUtils)
%%#SweaveListingPreparations()
%%#setToBeDefinedPkgs(pkgs = c("distr","distrEx"),
%%#                   keywordstyles = "\\bf\\color{distrCol}")
%%#(at)
%%
%
% -------------------------------------------------------------------------------
\begin{document}
% -------------------------------------------------------------------------------
%% ------------------------------------------------------------------------------
<<knitRPreparations,include=FALSE>>=
library(knitr)
opts_chunk$set(tidy=FALSE)
@
% -------------------------------------------------------------------------------
\lstdefinelanguage{Rd}[common]{TeX}%
{moretexcs={acronym,alias,arguments,author,bold,cite,%
          code,command,concept,cr,deqn,describe,%
          description,details,dfn,doctype,dots,%
          dontrun,dontshow,donttest,dQuote,%
          email,emph,enc,encoding,enumerate,env,eqn,%
          examples,file,format,if,ifelse,item,itemize,kbd,keyword,%
          ldots,link,linkS4class,method,name,note,%
          option,out,pkg,preformatted,R,Rdopts,Rdversion,%
          references,S3method,S4method,Sexpr,samp,section,%
          seealso,source,sp,special,sQuote,strong,%
          subsection,synopsis,tab,tabular,testonly,%
          title,url,usage,value,var,verb},
   sensitive=true,%
   morecomment=[l]\%% 2008/9 Peter Ruckdeschel
}[keywords,comments]%%
\lstdefinestyle{Rdstyle}{fancyvrb=TRUE,language=Rd, keywordstyle={\bf},%
         basicstyle={\color{black}\footnotesize},%
         commentstyle={\ttfamily\itshape},%
         alsolanguage=R}%
\global\def\Rdlstset{\lstset{style=Rdstyle}}%
% -------------------------------------------------------------------------------
\lstset{language=R,basicstyle={\color{Rcolor}\footnotesize},keywordstyle={\bf}}
\let\code\lstinline
\def\Code#1{{\tt\color{Rcolor} #1}}
\def\file#1{{\tt #1}}
\def\pkg#1{{\tt"#1"}}
\newcommand{\pkgversion}{{\tt 2.7}}
% -------------------------------------------------------------------------------
\title{How to generate new distributions in packages \pkg{distr}, \pkg{distrEx}}
%,version \pkgExversion}
\author{\small Peter Ruckdeschel\thanks{Universit\"at Oldenburg, Oldenburg}
\\[-.5ex]
\small Matthias Kohl\thanks{FH Furtwangen}
\smallskip\\
\small Institut f\"ur Mathematik\\[-0.5ex]
\small Fakult\"at V - Mathematik und Naturwissenschaften\\[-.5ex]
\small Carl von Ossietzky Universit\"at Oldenburg\\[-0.5ex]
\small PObox 2503\\[-.5ex]
\small 26111 Oldenburg (Oldb)\\[-.5ex]
\small Germany\\
\small e-Mail: {\small \tt peter.ruckdeschel@uni-oldenburg.de}\medskip\\
\parbox[t]{5cm}{
\footnotesize\sffamily
 Version control information:
\begin{tabbing}
\footnotesize\sffamily
 Last changes revision: \= \kill
 Head URL: \> \parbox[t]{6cm}{\url{\svnkw{HeadURL}}}\\[1.2ex]
 Last changed date: \> \svndate\\
 Last changes revision: \> \svnrev\\
 Version: \> \svnFullRevision*{\svnrev}\\
 Last changed by: \> \svnFullAuthor*{\svnauthor}\\
\end{tabbing}
}}

\maketitle
% -------------------------------------------------------------------------------
\begin{abstract}
% -------------------------------------------------------------------------------
In this vignette, we give short examples how to produce new
distributions in packages \pkg{distr} and \pkg{distrEx}.
This vignette refers to package version~\pkgversion.
% -------------------------------------------------------------------------------
\end{abstract}
% -------------------------------------------------------------------------------
Basically there are three ways to produce new
distributions in packages \pkg{distr} and \pkg{distrEx}:
\begin{enumerate}
\item automatic generation of single distribution objects by arithmetics and the like
\item using generating functions to produce single distribution objects
\item defining new distribution classes / doing it from scratch
\end{enumerate}
We will give short examples of all three of them.
% -------------------------------------------------------------------------------
\section{Automatic generation by arithmetics and the like}
% -------------------------------------------------------------------------------
We have made available quite general arithmetical operations to our distribution 
objects, generating new image distribution objects automatically. As an example, try
<<Prepa0, include=FALSE, results="hide">>=
require(distr)
@
<<Prepa, echo=FALSE, results="asis">>=
## preparation: set option withSweave to TRUE
require(distr)
distroptions(withSweave = TRUE)
options(width=70)
@
%
<<exam1, eval = TRUE, fig.width=8.0, fig.height=6.5>>=
require(distr)
N <- Norm(mean = 2, sd = 1.3)
P <- Pois(lambda = 1.2)
Z <- 2*N + 3 + P
Z
plot(Z, panel.first = grid(), lwd=3)
p(Z)(0.4)
q(Z)(0.3)
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
Zs <- r(Z)(50)
Zs
@
\par
\noindent{\bf Comment:}\\
Let \code{N} an object of class \code{"Norm"} with parameters  \code{mean=2},
\code{sd=1.3} and let \code{P}  an object of class \code{"Pois"} with parameter 
\code{lambda=1.2}. Assigning to \code{Z} the expression \code{2*N+3+P}, a 
new distribution object is generated ---of class \code{"AbscontDistribution"} in 
our case--- so that identifying \code{N}, \code{P}, \code{Z} with random 
variables distributed according to {\tt N}, {\tt P}, {\tt Z}, 
${\cal L}({\tt Z})={\cal L}(2*{\tt N}+3+{\tt P})$,  and writing \code{p(Z)(0.4)}  
we get $P(Z\leq 0.4)$, \code{ q(Z)(0.3)}  the $30\%$-quantile of {\tt Z},
and with \code{r(Z)(50)} we generate $50$ pseudo random numbers distributed 
according to {\tt Z}, while the \code{plot} command generates the above figure.\\

In the environments of \texttt{RStudio}, see \url{https://posit.co/} and
\texttt{Jupyter IRKernel}, see \url{https://github.com/IRkernel/IRkernel},
calls to \code{q} are caught away from standard {\sf R} evaluation and are treated
in a non-standard way. This non-standard evaluation in particular throws
errors at calls to our accessor methods \code{q} to slot \code{q} of the
respective distribution object. To amend this, from version 2.6 on,
we provide function \code{q.l} (for left-continuous quantile function)
as alias to our accessors \code{q}, so that all our package functionality also
becomes available in \texttt{RStudio} and \texttt{IRKernel}.\\

There are caveats to take care about; for details refer to the (larger) vignette
{\tt distr} in package \pkg{distrDoc}.
% -------------------------------------------------------------------------------
\section{Using generating functions}
% -------------------------------------------------------------------------------
If you want to generate a single distribution object (without any particular parameter)
generating functions are the method of choice:\\


Objects of classes \code{LatticeDistribution} resp.\ 
\code{DiscreteDistribution}, 
\code{AbscontDistribution},  may be generated using the generating functions
\code{LatticeDistribution()} resp.\ \code{DiscreteDistribution()}
resp.\ \code{AbscontDistribution()}; see also
the corresponding help.\\

E.g., to produce a discrete distribution with
support $(1,5,7,21)$ with corresponding probabilities $(0.1,0.1,0.6,0.2)$
we may write
<<DiscrDist, eval = TRUE>>=
D <- DiscreteDistribution(supp = c(1,5,7,21), prob = c(0.1,0.1,0.6,0.2))
D
plot(D, panel.first = grid(lwd=2), lwd = 3)
@
%
and to generate an absolutely continuous distribution with density proportional
to $e^{-|x|^3}$, we write
<<AbscDist, eval = TRUE>>=
AC <- AbscontDistribution(d = function(x) exp(-abs(x)^3), withStand = TRUE)
AC
plot(AC, panel.first = grid(lwd=2), lwd = 3)
@
%
% -------------------------------------------------------------------------------
\section{Doing it from scratch}
% -------------------------------------------------------------------------------
If you would like to create new parametric distributions, using already 
implemented {\tt r}, {\tt d}, {\tt p}, and {\tt q} functions
(e.g.\ implementing additional distributions realized in another 
\href{https://cran.r-project.org}{\tt CRAN} package),
you should probably envisage introducing new distribution {\tt S4} (sub-)classes
and hence better look at the implementation of some discrete and 
continuous parametric distribution classes in package \pkg{distr}.

\noindent{\small Hint: download the {\tt .tar.gz} file; extract it to some {\tt temp} 
folder; look at subdirectories {\tt R} and {\tt man}}\smallskip\\

The general procedure is as follows

\begin{enumerate}
\item introduce a  new subclass of  class \code{Parameter}
\item introduce a new subclass of  \code{LatticeDistribution}/%
\code{DiscreteDistribution} (if discrete)
 or of class \code{AbscontDistribution} (if continuous).
\item define accessor and replacement functions for the ``slots'' of the
parameter (e.g.\ \code{"size"} and \code{"prob"} in the binomial case), 
possibly with new generics
\item (possibly) define a validity function
\item define a generating function
\item if existing, define particular convolution methods or similar particular
methods for this new distribution class
\item create {\tt .Rd} files for the
\begin{itemize}
      \item parameter class
      \item distribution class
\end{itemize}
\item if analytic expressions are available, define particular \code{E}-, \code{var}-, 
\code{skewness}-, and \code{kurtosis}-methods
     and if so, also document\footnote{%
%
this is new, because so far, all \code{E}-, \code{var}-, 
\code{skewness}-, and \code{kurtosis}-methods for ``basic'' 
distributions are documented in the \pkg{distrEx} documentation to 
\code{E}, \code{var}, \ldots, but this would not be operational
any longer for new derived classes, possibly defined in other, new packages
%
     } the corresponding methods in
     the distribution class {\tt .Rd} file\\
\end{enumerate}
    
Let's go through the steps in the example case of the Binomial implementation
in packages \pkg{distr} and \pkg{distrEx}:

\begin{enumerate}
%
\item in \pkg{distr}, see source in \file{R/AllClasses.R},  
%
<<AllClass1, results="asis", echo=TRUE>>=
## Class: BinomParameter
setClass("BinomParameter",
          representation = representation(size = "numeric", prob = "numeric"),
          prototype = prototype(size = 1, prob = 0.5, name =
                      gettext("Parameter of a Binomial distribution")
                      ),
          contains = "Parameter"
          )
@
%
\item in \pkg{distr}, see source in \file{R/AllClasses.R},  
%
<<AllClass2, results="asis", echo=TRUE>>=
## Class: binomial distribution
setClass("Binom",
          prototype = prototype(
                      r = function(n){ rbinom(n, size = 1,prob = 0.5) },
                      d = function(x, log = FALSE){
                              dbinom(x, size = 1, prob = 0.5, log = log)
                                          },
                      p = function(q, lower.tail = TRUE, log.p = FALSE ){
                              pbinom(q, size = 1, prob = 0.5,
                                     lower.tail = lower.tail, log.p = log.p)
                                          },
                      q = function(p, lower.tail = TRUE, log.p = FALSE ){
                              qbinom(p, size = 1, prob = 0.5,
                                     lower.tail = lower.tail, log.p = log.p)
                                          },
                      img = new("Naturals"),
                      param = new("BinomParameter"),
                      support = 0:1,
                      lattice = new("Lattice",
                                pivot = 0, width = 1, Length = 2, name =
                                gettext(
                                  "lattice of a Binomial distribution"
                                       )
                                ),
                     .logExact = TRUE,
                     .lowerExact = TRUE
                      ),
          contains = "LatticeDistribution"
          )
@
%
\item in \pkg{distr}, see source in \file{R/BinomialDistribution.R},  
%
<<BinomDist1, results="asis", echo=TRUE>>=
## Access Methods
setMethod("size", "BinomParameter", function(object) object@size)
setMethod("prob", "BinomParameter", function(object) object@prob)
## Replace Methods
setReplaceMethod("size", "BinomParameter",
                  function(object, value){ object@size <- value; object})
setReplaceMethod("prob", "BinomParameter",
                  function(object, value){ object@prob <- value; object})
@
%
and \file{R/AllGenerics}, 
<<AllGenerics, results="asis", echo=TRUE>>=
if(!isGeneric("size"))
   setGeneric("size", function(object) standardGeneric("size"))
if(!isGeneric("prob"))
   setGeneric("prob", function(object) standardGeneric("prob"))
@
%
\item in \pkg{distr}, see source in \file{R/BinomialDistribution.R},  
%
<<BinomDist2, results="asis", echo=TRUE>>=
setValidity("BinomParameter", function(object){
  if(length(prob(object)) != 1)
    stop("prob has to be a numeric of length 1")
  if(prob(object) < 0)
    stop("prob has to be in [0,1]")
  if(prob(object) > 1)
    stop("prob has to be in [0,1]")
  if(length(size(object)) != 1)
    stop("size has to be a numeric of length 1")
  if(size(object) < 1)
    stop("size has to be a natural greater than 0")
  if(!identical(floor(size(object)), size(object)))
    stop("size has to be a natural greater than 0")
  else return(TRUE)
})
@
%
\item in \pkg{distr}, see source in \file{R/BinomialDistribution.R},  
%
<<BinomDist3, results="asis", echo=TRUE>>=
Binom <- function(size = 1,prob = 0.5) new("Binom", size = size, prob = prob)
@
%
\item in \pkg{distr}, see source in \file{R/BinomialDistribution.R},  
%
<<BinomDist4, results="asis", echo=TRUE>>=
## Convolution for two binomial distributions Bin(n1,p1) and Bin(n2,p2)
## Distinguish cases
## p1 == p2 und p1 != p2


setMethod("+", c("Binom","Binom"),
          function(e1,e2){
            newsize <- size(e1) + size(e2)

            if(isTRUE(all.equal(prob(e1),prob(e2))))
               return(new("Binom", prob = prob(e1), size = newsize,
                          .withArith = TRUE))

            return(as(e1, "LatticeDistribution") + e2)
          })
@
%
\item in \pkg{distr}, see sources in
%
\begin{itemize}
%
\item\file{man/BinomParameter-class.Rd}
%
\begin{lstlisting}[style = Rdstyle]
\name{BinomParameter-class}
\docType{class}
\alias{BinomParameter-class}
\alias{initialize,BinomParameter-method}

\title{Class "BinomParameter"}
\description{ The parameter of a binomial distribution, used by Binom-class}
\section{Objects from the Class}{
Objects can be created by calls of the form
      \code{new("BinomParameter", prob, size)}.
Usually an object of this class is not needed on its own, it is generated
automatically when an object of the class Binom
is instantiated.
}
\section{Slots}{
  \describe{
    \item{\code{prob}}{Object of class \code{"numeric"}:
          the probability of a binomial distribution }
    \item{\code{size}}{Object of class \code{"numeric"}:
          the size of a binomial distribution }
    \item{\code{name}}{Object of class \code{"character"}:
          a name / comment for the parameters }
  }
}
\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "BinomParameter")}:
          initialize method }
    \item{prob}{\code{signature(object = "BinomParameter")}: returns the slot
          \code{prob} of the parameter of the distribution }
    \item{prob<-}{\code{signature(object = "BinomParameter")}: modifies the slot
          \code{prob} of the parameter of the distribution }
    \item{size}{\code{signature(object = "BinomParameter")}: returns the slot
          \code{size} of the parameter of the distribution }
    \item{size<-}{\code{signature(object = "BinomParameter")}: modifies the slot
          \code{size} of the parameter of the distribution}
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }


\seealso{
\code{\link{Binom-class}}
\code{\link{Parameter-class}}
}

\examples{
W <- new("BinomParameter",prob=0.5,size=1)
size(W) # size of this distribution is 1.
size(W) <- 2 # size of this distribution is now 2.
}
\keyword{distribution}
\concept{parameter}
\concept{Binomial distribution}
\concept{S4 parameter class}
\end{lstlisting}

%
\item\file{man/Binom-class.Rd}
\begin{lstlisting}[style = Rdstyle]
\name{Binom-class}
\docType{class}
\alias{Binom-class}
\alias{Binom}
\alias{initialize,Binom-method}

\title{Class "Binom" }
\description{The binomial distribution with \code{size} \eqn{= n}, by default
  \eqn{=1}, and
  \code{prob} \eqn{= p}, by default \eqn{=0.5}, has density
  \deqn{p(x) = {n \choose x} {p}^{x} {(1-p)}^{n-x}}{
    p(x) = choose(n,x) p^x (1-p)^(n-x)}
  for \eqn{x = 0, \ldots, n}.

  C.f.\code{\link[stats:Binomial]{rbinom}}
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Binom(prob, size)}.
This object is a binomial distribution.
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Naturals"}: The space of the
     image of this distribution has got dimension 1 and the
     name "Natural Space". }
    \item{\code{param}}{Object of class \code{"BinomParameter"}: the parameter
          of this distribution (\code{prob}, \code{size}), declared at its
          instantiation }
    \item{\code{r}}{Object of class \code{"function"}: generates random
          numbers (calls function \code{rbinom}) }
    \item{\code{d}}{Object of class \code{"function"}: density function (calls
          function \code{dbinom}) }
    \item{\code{p}}{Object of class \code{"function"}: cumulative function
          (calls function \code{pbinom}) }
    \item{\code{q}}{Object of class \code{"function"}: inverse of the
           cumulative function (calls function \code{qbinom}).
    The quantile is defined as the smallest value x such that F(x) >= p, where
            F is the cumulative function. }
    \item{\code{support}}{Object of class \code{"numeric"}: a (sorted)
            vector containing the support of the discrete density function}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where
    there are explicit formulae for the log version of density, cdf, and
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where
    there are explicit formulae for the lower tail version of cdf and quantile
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"DiscreteDistribution"}, directly.\cr
Class \code{"UnivariateDistribution"}, by class \code{"DiscreteDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"DiscreteDistribution"}.
}
\section{Methods}{
  \describe{
    \item{+}{\code{signature(e1 = "Binom", e2 = "Binom")}: For two binomial
             distributions with equal probabilities the exact convolution
             formula is implemented thereby improving the general numerical
             accuracy.}
    \item{initialize}{\code{signature(.Object = "Binom")}: initialize method }
    \item{prob}{\code{signature(object = "Binom")}: returns the slot \code{prob}
             of the parameter of the distribution }
    \item{prob<-}{\code{signature(object = "Binom")}: modifies the slot
             \code{prob} of the parameter of the distribution }
    \item{size}{\code{signature(object = "Binom")}: returns the slot \code{size}
             of the parameter of the distribution }
    \item{size<-}{\code{signature(object = "Binom")}: modifies the slot
             \code{size} of the parameter of the distribution }
  }
}


\author{
  Thomas Stabla \email{statho3@web.de},\cr
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }


\seealso{
\code{\link{BinomParameter-class}}
\code{\link{DiscreteDistribution-class}}
\code{\link{Naturals-class}}
\code{\link[stats:Binomial]{rbinom}}
}
\examples{
B <- Binom(prob=0.5,size=1) # B is a binomial distribution with prob=0.5 and size=1.
r(B)(1) # # one random number generated from this distribution, e.g. 1
d(B)(1) # Density of this distribution is  0.5 for x=1.
p(B)(0.4) # Probability that x<0.4 is 0.5.
q(B)(.1) # x=0 is the smallest value x such that p(B)(x)>=0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
size(B) # size of this distribution is 1.
size(B) <- 2 # size of this distribution is now 2.
C <- Binom(prob = 0.5, size = 1) # C is a binomial distribution with prob=0.5 and size=1.
D <- Binom(prob = 0.6, size = 1) # D is a binomial distribution with prob=0.6 and size=1.
E <- B + C # E is a binomial distribution with prob=0.5 and size=3.
F <- B + D # F is an object of class LatticeDistribution.
G <- B + as(D,"DiscreteDistribution") ## DiscreteDistribution
}
\keyword{distribution}
\concept{discrete distribution}
\concept{lattice distribution}
\concept{Binomial family}
\concept{Binomial distribution}
\concept{S4 distribution class}
\concept{generating function}
\end{lstlisting}

%
\item {\footnotesize you could have: \file{man/Binom.Rd}
      for the generating function; in the Binomial case, documentation is in
      \file{Binom-class.Rd}; but in case of the Gumbel distribution,
      in package \pkg{RobExtremes}, there is such an extra {\tt .Rd} file}
%
\end{itemize}
%
\item in \pkg{distrEx}, see sources in
%
<<Prepa2, echo=FALSE, results="asis">>=
## preparation: set option withSweave to TRUE
require(distrEx)
@
%
\begin{itemize}
%
\item\file{Expectation.R}, 
<<Expect, results="asis", echo=TRUE>>=
setMethod("E", signature(object = "Binom",
                         fun = "missing",
                         cond = "missing"),
    function(object, low = NULL, upp = NULL, ...){
    if(!is.null(low)) if(low <= min(support(object))) low <- NULL
    if(!is.null(upp)) if(upp >= max(support(object))) upp <- NULL
    if(is.null(low) && is.null(upp))
        return(size(object)*prob(object))
    else{
        if(is.null(low)) low <- -Inf
        if(is.null(upp)) upp <- Inf
        if(low == -Inf){
           if(upp == Inf) return(size(object)*prob(object))
           else return(m1df(object, upper = upp, ...))
        }else{
           E1 <- m1df(object, upper = low, ...)
           E2 <- if(upp == Inf)
                    size(object)*prob(object) else m1df(object, upper = upp, ...)
           return(E2-E1)
        }
    }
   })
@
%
\item\file{Functionals.R}, 
<<var, results="asis", echo=TRUE>>=
setMethod("var", signature(x = "Binom"),
    function(x,...){
    dots <- match.call(call = sys.call(sys.parent(1)),
                       expand.dots = FALSE)$"..."
    fun <- NULL; cond <- NULL; low <- NULL; upp <- NULL
    if(hasArg(low)) low <- dots$low
    if(hasArg(upp)) upp <- dots$upp
    if(hasArg(fun)||hasArg(cond)||!is.null(low)||!is.null(upp))
        return(var(as(x,"DiscreteDistribution"),...))
    else
        return(size(x)*prob(x)*(1-prob(x)))
    })
@
%
\item\file{skewness.R}, 
<<skew, results="asis", echo=TRUE>>=
setMethod("skewness", signature(x = "Binom"),
    function(x,  ...){
    dots <- match.call(call = sys.call(sys.parent(1)),
                       expand.dots = FALSE)$"..."
    fun <- NULL; cond <- NULL; low <- NULL; upp <- NULL
    if(hasArg(low)) low <- dots$low
    if(hasArg(upp)) upp <- dots$upp
    if(hasArg(fun)||hasArg(cond)||!is.null(low)||!is.null(upp))
       return(skewness(as(x,"DiscreteDistribution"),...))
    else
        return((1-2*prob(x))/sqrt(size(x)*prob(x)*(1-prob(x))))
    })
@
%
\item\file{kurtosis.R}, 
<<kurt, results="asis", echo=TRUE>>=
setMethod("kurtosis", signature(x = "Binom"),
    function(x,  ...){
    dots <- match.call(call = sys.call(sys.parent(1)),
                       expand.dots = FALSE)$"..."
    fun <- NULL; cond <- NULL; low <- NULL; upp <- NULL
    if(hasArg(low)) low <- dots$low
    if(hasArg(upp)) upp <- dots$upp
    if(hasArg(fun)||hasArg(cond)||!is.null(low)||!is.null(upp))
       return(kurtosis(as(x,"DiscreteDistribution"),...))
    else
        p <- prob(x)
        return((1-6*p*(1-p))/(size(x)*p*(1-p)))
    })
@
%
\end{itemize}
\end{enumerate}
The procedure will be similar for \textit{any} new class of distributions.\medskip
\begin{itemize}
  \item[Comment] In the classes in package \pkg{distr} (historically the ``oldest''
  in the development of this project), we still use \code{initialize} methods;
  this is no longer needed, if you provide generating functions; for this ``more
  recent'' approach, confer the realization of class \code{Gumbel} in package \pkg{RobExtremes}.
\end{itemize}

% -------------------------------------------------------------------------------
\section{Help needed  / collaboration welcome}
% -------------------------------------------------------------------------------
You are --- as announced on  
\href{https://distr.r-forge.r-project.org}{\tt http://distr.r-forge.r-project.org} ---
         very welcome to collaborate in this project!
See in particular
\href{https://distr.r-forge.r-project.org/HOWTO-collaborate.txt}%
{\tt https://distr.r-forge.r-project.org/HOWTO-collaborate.txt}

With this you should be able to start working.
\begin{thebibliography}{2}

\bibitem{K:R:S:04}
Ruckdeschel P. and Kohl, M. (2014):
\newblock {General Purpose Convolution Algorithm for Distributions in S4-Classes 
by means of FFT}.
\newblock {\em J. Statist. Software\/}, {\bf 59}(4): 1--25.

\bibitem{R:K:S:C:04}
Ruckdeschel P., Kohl M., Stabla T., and Camphausen F. (2006):
\newblock {S4 Classes for Distributions.} 
\newblock {\em R-News\/}, {\bf 6}(2): 10--13.
\newblock https://CRAN.R-project.org/doc/Rnews/Rnews\_2006-2.pdf
%\newblock See also {http://www.uni-bayreuth.de/departments/math/org/mathe7/RUCKDESCHEL/pubs/distr.pdf}

\end{thebibliography}
% no longer needed
%%(ll)cleanup, echo=FALSE(ggeq)
%%#unloadNamespace("SweaveListingUtils")
%%(at)
% -------------------------------------------------------------------------------
\end{document}
% -------------------------------------------------------------------------------

\name{ExpOrGammaOrChisq-class}
\docType{class}
\alias{ExpOrGammaOrChisq-class}
\title{Class "ExpOrGammaOrChisq"}
\description{
To have common methods, a class \code{"ExpOrGammaOrChisq"} is introduced
as subclass of class \code{"AbscontDistribution"}  and as superclass of
classes \code{"Chisq"}, \code{"Exp"}, \code{"Gammad"}. It is only used 
internally.
}
 
\section{Objects from the Class}{
This class is virtual, hence cannot be instantiated.
}

\section{Extends}{
Class \code{"AbscontDistribution"}, directly.\cr
Class \code{"UnivariateDistribution"}, by class \code{"AbscontDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"UnivariateDistribution"}.
}


\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}
}
\seealso{
\code{\link{Chisq-class}}
\code{\link{Gammad-class}}
\code{\link{Exp-class}}
}
\examples{
## class only used internally
}
\keyword{internal}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{Chi square distribution}
\concept{Exp distribution}
\concept{Gamma distribution}
\concept{S4 class union}
\concept{virtual class}

\name{p-methods}
\docType{methods}
\alias{p.r-methods}
\alias{p.r}
\alias{p.r,Distribution-method}
\alias{p-methods}
\alias{p}
\alias{p,Distribution-method}
\title{ Methods for Function p in Package `distr' }
\description{p-methods}

\section{Methods}{\describe{

\item{p}{\code{signature(object = "Distribution")}: returns the cumulative distribution function (c.d.f.),
i.e.; \eqn{p(t) = P(object \le t)}{p(t) = P(object <= t)}  }
\item{p.r}{\code{signature(object  = "Distribution")}: 
from distr-2.6 onwards, we provide this as a synonym for 
method \code{p}; this synonym more explicitely states that we are dealing with
the right-continuous variant of a c.d.f.}
}}
\seealso{
\code{\link{Distribution-class}}, \code{p.l}
}
\examples{
require(distr)
N <- Norm()
p(N)(0.3)
p.r(N)(0.3)
}
\keyword{distribution}
\keyword{methods}
\concept{cumulative distribution function}
\concept{accessor function}
\concept{CDF}
\concept{right-continuous}

\name{makeAbscontDistribution}
\alias{makeAbscontDistribution}

\title{"makeAbscontDistribution"}
\description{Transforms an object of \code{"UnivariateDistribution"} to 
             an object of class \code{"makeAbscontDistribution"}.}
\usage{makeAbscontDistribution(object, gaps = NULL,
                       param = NULL, img = NULL,
                   withgaps = getdistrOption("withgaps"),
                   ngrid = getdistrOption("DefaultNrGridPoints"),
                   ep = getdistrOption("TruncQuantile"))}
\arguments{
  \item{object}{Objects of class \code{"UnivariateDistribution"} (or subclasses)}
  \item{gaps}{slot gaps (of class \code{"matrix"} with two columns) to be filled 
              (i.e. \code{t(gaps)} must be ordered if read as vector)} 
  \item{param}{parameter (of class \code{"OptionalParameter"})} 
  \item{img}{image range of the distribution (of class \code{"rSpace"})} 
  \item{withgaps}{logical; shall gaps be reconstructed empirically?}
  \item{ngrid}{number of gridpoints}
  \item{ep}{tolerance epsilon}
}
\details{takes slot \code{p} of \code{object} and then generates an
\code{"AbscontDistribution"} object using generating function
\code{\link{AbscontDistribution}}.}
%\references{}
\author{Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}
%\note{}
\examples{
  Hu <- Huberize(Norm(), -2,1)
  Hu
  plot(Hu)
  Hu0 <- makeAbscontDistribution(Hu)
  Hu0
  plot(Hu0)
}
\keyword{distribution}
\concept{S4 distribution class}

\name{img-methods}
\docType{methods}
\alias{img-methods}
\alias{img}
\alias{img,Distribution-method}
\title{ Methods for Function img in Package `distr'}
\description{img-methods}

\section{Methods}{\describe{

\item{img}{\code{signature(object  = "Distribution")}: returns the image space / domain of the distribution }
}}
\keyword{methods}
\keyword{distribution}
\concept{space}
\concept{range space of a distribution}
\concept{accessor function}

\name{TParameter-class} 
\docType{class}
\alias{TParameter-class}
\alias{initialize,TParameter-method}

\title{Class "TParameter"}
\description{ The parameter of a t distribution, used by Td-class}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("TParameter", df, ncp)}.
Usually an object of this class is not needed on its own, it is generated automatically when an object of the class
\code{Td} is instantiated. 
}
\section{Slots}{
  \describe{
    \item{\code{df}}{Object of class \code{"numeric"}: the degrees of freedom of a T distribution }
    \item{\code{ncp}}{Object of class \code{"numeric"}: the noncentrality parameter of a T distribution }
    \item{\code{name}}{Object of class \code{"character"}: a name / comment for the parameters }
  }
}
\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "TParameter")}: initialize method }
    \item{df}{\code{signature(object = "TParameter")}: returns the slot \code{df} of the parameter of the distribution }
    \item{df<-}{\code{signature(object = "TParameter")}: modifies the slot \code{df} of the parameter of the distribution }
    \item{ncp}{\code{signature(object = "TParameter")}: returns the slot \code{ncp} of the parameter of the distribution }
    \item{ncp<-}{\code{signature(object = "TParameter")}: modifies the slot \code{ncp} of the parameter of the distribution }
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}

\seealso{
\code{\link{Td-class}}
\code{\link{Parameter-class}}
}

\examples{
W <- new("TParameter",df=1, ncp = 0)
df(W) # df of this distribution is 1.
df(W) <- 2 # df of this distribution is now 2.
}
\keyword{distribution}
\concept{parameter}
\concept{T distribution}
\concept{S4 parameter class}

\name{DistrList}
\alias{DistrList}

\title{Generating function for DistrList-class}
\description{Generates an object of class \code{"DistrList"}.}
\usage{DistrList(..., Dlist)}
\arguments{
  \item{\dots}{Objects of class \code{"Distribution"} (or subclasses)}
  \item{Dlist}{an optional list or object of class \code{"DistrList"};
  if not missing it is appended to argument \code{\dots}; this way
  \code{DistrList} may also be called with a list (or
  \code{"DistrList"}-object) as argument as suggested in an e-mail
  by Krunoslav Sever (thank you!)}
}

%\details{}
\value{Object of class \code{"DistrList"}}
%\references{}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de}}
%\note{}
\seealso{\code{\link{DistrList-class}}, \code{\link{UnivarDistrList-class}},
  \code{\link{UnivarDistrList}}}
\examples{
(DL <- DistrList(Norm(), Exp(), Pois()))
plot(DL)
as(Norm(), "DistrList")

## The function is currently defined as
function(...){ 
    new("DistrList", list(...)) 
}
}
\keyword{distribution}
\keyword{list}
\concept{multivariate distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{getLabel}
\alias{getLabel}
\title{Labels for distribution objects}
\description{
a help function to get reasonable labels for distribution objects}
\usage{
getLabel(x, withnames = TRUE)
}
\arguments{
  \item{x}{ a distribution object}
  \item{withnames}{ logical: are the parameters (if any) of \code{x} to be 
        displayed with names? }
}

\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}

\section{Remark}{The need for this helper function (external to our 
        \code{plot} methods) was brought to our attention in a mail 
        by Kouros Owzar \email{owzar001@mc.duke.edu}.}

\seealso{\code{\link[distr]{plot-methods}}} 
\examples{
## example due to Kouros Owzar:
foo<- function(law,n, withnames = TRUE)
  {
    data.frame(muhat=mean(r(law)(n)),n=n,law= getLabel(law,withnames))
  } 
### a function that groups certain informations on 
##  created with distribution objects
do.call("rbind",lapply(list(Exp(1),Norm(0,1),Weibull(1,1)),foo,n=100))
do.call("rbind",lapply(list(Exp(1),Norm(0,1),Weibull(1,1)),foo,n=100,FALSE))
}
\keyword{distribution}
\concept{utility}

\name{RtoDPQ}
\alias{RtoDPQ}
\title{Default procedure to fill slots d,p,q given r for a.c. distributions}
\description{
  function to do get empirical density, cumulative distribution and quantile function from random numbers}
\usage{
RtoDPQ(r, e = getdistrOption("RtoDPQ.e"), 
       n = getdistrOption("DefaultNrGridPoints"), y = NULL)
}
\arguments{
  \item{r}{ the random number generator }
  \item{e}{ \eqn{10^e} numbers are generated, a higher number leads to a better result. }
  \item{n}{ The number of grid points used to create the approximated functions, a higher number leads to a better result.}
  \item{y}{a (numeric) vector or \code{NULL}}
}
\details{
RtoDPQ generates \eqn{10^e} random numbers, by default \deqn{e = RtoDPQ.e}. 
Instead of using simulated grid points, we have an optional parameter \code{y}
for using N. Horbenko's quantile trick: i.e.; on an equally spaced grid \code{x.grid} on [0,1], apply
\code{f(q(x)(x.grid))} and write the result to \code{y} and produce density and cdf from this
value \code{y} given to \code{RtoDPQ} as argument (instead of simulating grid points).

The density is formed on the basis of \eqn{n}
points using approxfun and density, by default \deqn{n = DefaultNrGridPoints}.
The cumulative distribution function and the quantile function are also created on the basis of \eqn{n} points using 
\code{approxfun} and \code{ecdf}.  Of course, the results are usually not exact as they rely on random numbers.}
\value{ \code{RtoDPQ} returns a list of functions.
  \item{dfun }{ density }
  \item{pfun }{ cumulative distribution function }
  \item{qfun }{ quantile function }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}

\note{ Use \code{RtoDPQ} for absolutely continuous and \code{RtoDPQ.d} for discrete distributions. }

\seealso{ 
\code{\link{UnivariateDistribution-class}}, 
\code{\link{density}}, 
\code{\link{approxfun}}, 
\code{\link{ecdf}}}
\examples{
rn2 <- function(n){rnorm(n)^2}
x <- RtoDPQ(r = rn2, e = 4, n = 512)
# returns density, cumulative distribution and quantile function of
# squared standard normal distribution
x$dfun(4)
## IGNORE_RDIFF_BEGIN
RtoDPQ(r = rn2, e = 5, n = 1024) # for a better result
## IGNORE_RDIFF_END
rp2 <- function(n){rpois(n, lambda = 1)^2}
x <- RtoDPQ.d(r = rp2, e = 5)
# returns density, cumulative distribution and quantile function of
# squared Poisson distribution with parameter lambda=1
}
\keyword{math}
\keyword{distribution}
\keyword{arith}
\concept{random sample}
\concept{image distribution}
\concept{absolutely continuous distribution}
\concept{utility}

\name{Lnorm-class} 
\docType{class}
\alias{Lnorm-class}
\alias{Lnorm}
\alias{initialize,Lnorm-method}

\title{Class "Lnorm"}
\description{ 
  The log normal distribution has density
  \deqn{
    d(x) = \frac{1}{\sqrt{2\pi}\sigma x} e^{-(\log(x) - \mu)^2/2 \sigma^2}%
  }{d(x) = 1/(sqrt(2 pi) sigma x) e^-((log x - mu)^2 / (2 sigma^2))}
  where \eqn{\mu}, by default \eqn{=0}, and \eqn{\sigma}, by default \eqn{=1}, are the mean and standard
  deviation of the logarithm.
C.f. \code{\link[stats:Lognormal]{rlnorm}} }
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Lnorm(meanlog, sdlog)}.
This object is a log normal distribution.
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}: The space of the image of this distribution has got dimension 1
    and the name "Real Space". }
    \item{\code{param}}{Object of class \code{"LnormParameter"}: the parameter of this distribution (meanlog and sdlog),
     declared at its instantiation }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers (calls function \code{rlnorm})}
    \item{\code{d}}{Object of class \code{"function"}: density function (calls function \code{dlnorm})}
    \item{\code{p}}{Object of class \code{"function"}: cumulative function (calls function \code{plnorm})}
    \item{\code{q}}{Object of class \code{"function"}: inverse of the cumulative function (calls function \code{qlnorm})}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"AbscontDistribution"}, directly.\cr
Class \code{"UnivariateDistribution"}, by class \code{"AbscontDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"AbscontDistribution"}.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Lnorm")}: initialize method }
    \item{meanlog}{\code{signature(object = "Lnorm")}: returns the slot \code{meanlog} of the parameter of the distribution }
    \item{meanlog<-}{\code{signature(object = "Lnorm")}: modifies the slot \code{meanlog} of the parameter of the distribution }
    \item{sdlog}{\code{signature(object = "Lnorm")}: returns the slot \code{sdlog} of the parameter of the distribution }
    \item{sdlog<-}{\code{signature(object = "Lnorm")}: modifies the slot \code{sdlog} of the parameter of the distribution }
    \item{*}{\code{signature(e1 = "Lnorm", e2 = "numeric")}:
    For the Lognormal distribution we use its closedness under positive scaling transformations.}
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }

\note{  The mean is \eqn{E(X) = exp(\mu + 1/2 \sigma^2)}, and the variance
  \eqn{Var(X) = exp(2\mu + \sigma^2)(exp(\sigma^2) - 1)}{%
    Var(X) = exp(2*mu + sigma^2)*(exp(sigma^2) - 1)} and
  hence the coefficient of variation is
  \eqn{\sqrt{exp(\sigma^2) - 1}}{sqrt(exp(sigma^2) - 1)} which is
    approximately \eqn{\sigma} when that is small (e.g., \eqn{\sigma < 1/2}).
%% Mode = exp(max(0, mu - sigma^2)); Median = exp(mu)
}

\seealso{
\code{\link{LnormParameter-class}}
\code{\link{AbscontDistribution-class}}
\code{\link{Reals-class}}
\code{\link[stats:Lognormal]{rlnorm}}





}
\examples{
L <- Lnorm(meanlog=1,sdlog=1) # L is a lnorm distribution with mean=1 and sd=1.
r(L)(1) # one random number generated from this distribution, e.g. 3.608011
d(L)(1) # Density of this distribution is 0.2419707 for x=1.
p(L)(1) # Probability that x<1 is 0.1586553.
q(L)(.1) # Probability that x<0.754612 is 0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
meanlog(L) # meanlog of this distribution is 1.
meanlog(L) <- 2 # meanlog of this distribution is now 2.
}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{Log-Normal distribution}
\concept{S4 distribution class}

\name{rSpace-class}
\docType{class}
\alias{rSpace-class}

\title{Class "rSpace" }
\description{ The distribution-classes contain a slot where the sample space is stored. Typically, discrete random variables
take naturals as values. rSpace is the mother-class of the class EuclideanSpace. }
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}
\section{Slots}{
  \describe{
    \item{\code{name}}{Object of class \code{"character"}: the name of the space }
  }
}
\section{Methods}{
  \describe{
    \item{name}{\code{signature(object = "rSpace")}: returns the name of the space }
    \item{name<-}{\code{signature(object = "rSpace")}: changes the name of the space }
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}


\seealso{
\code{\link{Lattice-class}}
\code{\link{Naturals-class}}
\code{\link{EuclideanSpace-class}}
\code{\link{Distribution-class}}
}
\concept{space}
\concept{set}
\concept{S4 space class}
\keyword{distribution}

\name{shape2-methods}
\docType{methods}
\alias{shape2-methods}
\alias{shape2<--methods}
\alias{shape2}
\alias{shape2<-}
\alias{shape2,BetaParameter-method}
\alias{shape2,Beta-method}
\alias{shape2<-,BetaParameter-method}
\alias{shape2<-,Beta-method}
\title{ Methods for Function shape2 in Package `distr' }
\description{shape-methods}
\section{Methods}{\describe{
\item{shape2}{\code{signature(object = "BetaParameter")}: returns the slot shape2 of the parameter of the distribution }
\item{shape2<-}{\code{signature(object = "BetaParameter")}: modifies the slot shape2 of the parameter of the distribution }
\item{shape2}{\code{signature(object = "Beta")}: returns the slot shape2 of the parameter of the distribution }
\item{shape2<-}{\code{signature(object = "Beta")}: modifies the slot shape2 of the parameter of the distribution }
}}
\keyword{distribution}
\keyword{methods}
\concept{shape}
\concept{parameter}
\concept{Beta distribution}
\concept{accessor function}
\concept{replacement function}

\name{mean-methods}
\docType{methods}
\alias{mean-methods}
\alias{mean<--methods}
\alias{mean}
\alias{mean<-}
\alias{mean,NormParameter-method}
\alias{mean,Norm-method}
\alias{mean<-,NormParameter-method}
\alias{mean<-,Norm-method}
\title{ Methods for Function mean in Package `distr' }
\description{mean-methods}
\section{Methods}{\describe{
\item{mean}{\code{signature(object = "NormParameter")}: returns the slot mean of the parameter of the distribution }    
\item{mean<-}{\code{signature(object = "NormParameter")}: modifies the slot mean of the parameter of the distribution }
\item{mean}{\code{signature(object = "Norm")}: returns the slot mean of the parameter of the distribution }
\item{mean<-}{\code{signature(object = "Norm")}: modifies the slot mean of the parameter of the distribution }
}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{Normal distribution}
\concept{Gaussian distribution}
\concept{S4 parameter class}
\concept{location scale family}
\concept{location parameter}
\concept{accessor function}
\concept{replacement function}

\name{getLow,getUp}
\alias{getUp}
\alias{getUp-methods}
\alias{getUp,AbscontDistribution-method}
\alias{getUp,DiscreteDistribution-method}
\alias{getUp,LatticeDistribution-method}
\alias{getUp,UnivarLebDecDistribution-method}
\alias{getUp,UnivarMixingDistribution-method}
\alias{getLow}
\alias{getLow-methods}
\alias{getLow,AbscontDistribution-method}
\alias{getLow,DiscreteDistribution-method}
\alias{getLow,LatticeDistribution-method}
\alias{getLow,UnivarLebDecDistribution-method}
\alias{getLow,UnivarMixingDistribution-method}

\title{getLow, getUp functions of package distr}

\description{
getLow, getUp return lower and upper endpoint of a distribution ---
truncated to lower/upper \code{TruncQuantile} if infinite; in case of an object 
of class \code{"LatticeDistribution"} with infinite lattice length, we search
for the smallest/largest point in the lattice which is returned by succesive
halving of \code{x=0.5} in \code{q(object)(x, lower.tail)} for \code{lower.tail} 
\code{TRUE} resp. \code{false}.}

\usage{
\S4method{getUp}{AbscontDistribution}(object,
                                      eps = getdistrOption("TruncQuantile"))
\S4method{getUp}{DiscreteDistribution}(object, ...)
\S4method{getUp}{LatticeDistribution}(object, ...)
\S4method{getUp}{UnivarLebDecDistribution}(object,
                                      eps = getdistrOption("TruncQuantile"))
\S4method{getUp}{UnivarMixingDistribution}(object,
                                      eps = getdistrOption("TruncQuantile"))
\S4method{getLow}{AbscontDistribution}(object,
                                       eps = getdistrOption("TruncQuantile"))
\S4method{getLow}{DiscreteDistribution}(object, ...)
\S4method{getLow}{LatticeDistribution}(object, ...)
\S4method{getLow}{UnivarLebDecDistribution}(object,
                                      eps = getdistrOption("TruncQuantile"))
\S4method{getLow}{UnivarMixingDistribution}(object,
                                      eps = getdistrOption("TruncQuantile"))
}
\arguments{
  \item{object}{a distribution object}
  \item{eps}{truncation point (numeric)}
  \item{\dots}{for convenience only; makes it possible to call \code{getLow}, 
  \code{getUp} with argument \code{eps} no matter of the class of \code{object};
  is ignored in these functions.}
}



\value{
\item{getLow,getUp}{a \code{numeric} of length 1}
}


\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}
  }

\keyword{methods}
\keyword{distribution}
\concept{support}
\concept{utilities}

\name{liesInSupport}
\alias{liesInSupport}
\alias{liesInSupport-methods}
\alias{liesInSupport,UnivarLebDecDistribution,numeric-method}
\alias{liesInSupport,UnivarMixingDistribution,numeric-method}
\alias{liesInSupport,LatticeDistribution,numeric-method}
\alias{liesInSupport,DiscreteDistribution,numeric-method}
\alias{liesInSupport,Distribution,matrix-method}
\alias{liesInSupport,AbscontDistribution,numeric-method}
\alias{liesInSupport,ExpOrGammaOrChisq,numeric-method}
\alias{liesInSupport,Lnorm,numeric-method}
\alias{liesInSupport,Fd,numeric-method}
\alias{liesInSupport,Norm,numeric-method}
\alias{liesInSupport,DExp,numeric-method}
\alias{liesInSupport,Cauchy,numeric-method}
\alias{liesInSupport,Td,numeric-method}
\alias{liesInSupport,Logis,numeric-method}
\alias{liesInSupport,Weibull,numeric-method}
\alias{liesInSupport,Unif,numeric-method}
\alias{liesInSupport,Beta,numeric-method}


\title{Generic Function for Testing the Support of a Distribution }
\description{
  The function tests if \code{x} lies in the support of the
  distribution \code{object}.
}
\usage{
liesInSupport(object, x, ...)
\S4method{liesInSupport}{UnivarLebDecDistribution,numeric}(object,x, checkFin = FALSE)
\S4method{liesInSupport}{UnivarMixingDistribution,numeric}(object,x, checkFin = FALSE)
\S4method{liesInSupport}{LatticeDistribution,numeric}(object,x, checkFin = FALSE)
\S4method{liesInSupport}{DiscreteDistribution,numeric}(object,x, checkFin = FALSE)
\S4method{liesInSupport}{AbscontDistribution,numeric}(object,x, checkFin = FALSE)
\S4method{liesInSupport}{Distribution,matrix}(object,x, checkFin = FALSE)
\S4method{liesInSupport}{ExpOrGammaOrChisq,numeric}(object,x, checkFin = TRUE)
\S4method{liesInSupport}{Lnorm,numeric}(object,x, checkFin = TRUE)
\S4method{liesInSupport}{Fd,numeric}(object,x, checkFin = TRUE)
\S4method{liesInSupport}{Norm,numeric}(object,x, checkFin = TRUE)
\S4method{liesInSupport}{DExp,numeric}(object,x, checkFin = TRUE)
\S4method{liesInSupport}{Cauchy,numeric}(object,x, checkFin = TRUE)
\S4method{liesInSupport}{Td,numeric}(object,x, checkFin = TRUE)
\S4method{liesInSupport}{Logis,numeric}(object,x, checkFin = TRUE)
\S4method{liesInSupport}{Weibull,numeric}(object,x, checkFin = TRUE)
\S4method{liesInSupport}{Unif,numeric}(object,x, checkFin = TRUE)
\S4method{liesInSupport}{Beta,numeric}(object,x, checkFin = TRUE)
}
\arguments{
  \item{object}{ object of class \code{"Distribution"} }
  \item{x}{ numeric vector or matrix }
  \item{checkFin}{ logical: in case \code{FALSE}, we simply check whether
  \code{x} lies in the \emph{numerical} (i.e., possibly cut
  to relevant quantile range) support; in case \code{TRUE} we try to
  check this by more exact techniques (e.g. in case of lattice distributions)
  and by using slot \code{.finSupport} / the return values of \code{q.l(object)}
  in \code{0} and \code{1}. This is only used on discrete (parts of)
  distributions).}
  \item{\dots}{ used for specific arguments to particular methods. }
}
%\details{}
\value{logical vector}
\section{Methods}{
\describe{
  \item{object = "DiscreteDistribution", x = "numeric":}{ 
    We return a logical vector of the same length as \code{x} with \code{TRUE}
    when \code{x} lies in the support of \code{object}.
    As support we use the value of \code{support(object)}, so this
    is possibly cut to relevant quantile ranges.
    In case \code{checkFin} is \code{TRUE}, in addition, we flag those coordinates
    to \code{TRUE} where \code{x < min(support(object))} if
    \code{is.na(object@.finSupport[1])} or \code{object@.finSupport[1]==FALSE}
    or \code{q.l(object)(0)==-Inf}, and similarly, where
    \code{x > max(support(object))} if \code{is.na(object@.finSupport[2])}
    or \code{object@.finSupport[2]==FALSE} or \code{q.l(object)(1)==Inf}.
    In addition we flag those coordinates to \code{TRUE} where
    \code{q.l(object)(0)<=x<min(support(object))} if
    \code{object@.finSupport[1]==TRUE} and, similarly, where
    \code{q.l(object)(1)>=x>max(support(object))} if
    \code{object@.finSupport[2]==TRUE}.
    }

  \item{object = "Distribution", x = "matrix":}{
    Argument \code{x} is cast to vector and then the respective
    \code{liesInSupport} method for vectors is called. The method throws an
    arror when the dispatch mechanism does not find a suitable, applicable
    respective vector-method.
    }

  \item{object = "AbscontDistribution", x = "numeric":}{
    We return a logical vector of the same length as \code{x} with \code{TRUE}
    where \code{q.l(object)(0)<=x<=q.l(object)(1)} (and replace the boundary
    values by \code{q.l(object)(10*.Machine$double.eps)} resp.
    \code{q.l(object)(1-10*.Machine$double.eps)} once the return values
    for \code{0} or \code{1} return are \code{NaN}.
  }

  \item{object = "LatticeDistribution", x = "numeric":}{
    We return a logical vector of the same length as \code{x} with \code{TRUE}
    when \code{x} lies in the support of \code{object}.
    As support we use the value of \code{support(object)}, so this
    is possibly cut to relevant quantile ranges.
    In case \code{checkFin} is \code{TRUE}, we instead use the lattice
    information: We check whether all values
    \code{(x-pivot(lattice(object))/width(lattice(object))} are non-negative
    integers and are non larger than \code{Length(lattice(object))-1}.
    In addition, we flag those coordinates to \code{TRUE} where
    \code{x < min(support(object))} if
    \code{is.na(object@.finSupport[1])} or \code{object@.finSupport[1]==FALSE},
    and similarly, where \code{x > max(support(object))} if
    \code{is.na(object@.finSupport[2])}
    or \code{object@.finSupport[2]==FALSE}.
    }

  \item{object = "UnivarLebDecDistribution", x = "numeric":}{
    We split up \code{object} into discrete and absolutely continuous
    part and for each of them apply \code{liesInSupport} separately;
    the two return values are combined by a coponentwise logical \code{|}.
    }

  \item{object = "UnivarMixingDistribution", x = "numeric":}{
    We first cast \code{object} to \code{UnivarLebDecDistribution}
    by \code{flat.mix} and then apply the respective method.
    }
}}
%
%\references{}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de} and
Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}
%\note{}
\seealso{\code{\link{Distribution-class}}}
\examples{
liesInSupport(Exp(1), rnorm(10))

# note
x <- rpois(10, lambda = 10)
liesInSupport(Pois(1), x)
# better
liesInSupport(Pois(1), x, checkFin = TRUE)
liesInSupport(Pois(1), 1000*x, checkFin = TRUE)
liesInSupport(-10*Pois(1), -10*x+1, checkFin = TRUE)

xs = c(1000*x,runif(10))
D <- UnivarMixingDistribution(Pois(1),Unif())
liesInSupport(D, xs)

}
\keyword{distribution}
\keyword{methods}
\concept{space}
\concept{S4 space class}
\concept{support}

\name{flat.mix}
\alias{flat.mix}
\title{Default procedure to fill slots d,p,q given r for Lebesgue decomposed distributions}
\description{
  function to do get empirical density, cumulative distribution and quantile function from random numbers}
\usage{
flat.mix(object)
}
\arguments{
  \item{object}{object of class \code{UnivariateMixingDistribution}}
}
\details{
flat.mix generates \eqn{10^e} random numbers, by default \deqn{e = RtoDPQ.e}. 
Replicates are assumed to be part of the discrete part, unique values to be
part of the a.c. part of the distribution. For the replicated ones,
we generate a discrete distribution by a call to \code{\link{DiscreteDistribution}}.
The a.c. density is formed on the basis of \eqn{n}
points using approxfun and density (applied to the unique values), by default \deqn{n = DefaultNrGridPoints}.
The cumulative distribution function is based on all random variables, 
and, as well as the quantile function, is also created on the basis of \eqn{n} points using 
\code{approxfun} and \code{ecdf}.  Of course, the results are usually not exact as they rely on random numbers.}
\value{ \code{flat.mix} returns an object of class \code{UnivarLebDecDistribution}.
}

\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}

\note{ Use \code{RtoDPQ} for absolutely continuous and \code{RtoDPQ.d} for discrete distributions. }

\seealso{ 
\code{\link{UnivariateDistribution-class}}, 
\code{\link{density}}, 
\code{\link{approxfun}}, 
\code{\link{ecdf}}}
\examples{
D1 <- Norm()
D2 <- Pois(1)
D3 <- Binom(1,.4)
D4 <- UnivarMixingDistribution(D1,D2,D3, mixCoeff = c(0.4,0.5,0.1), 
      withSimplify = FALSE)
D <- UnivarMixingDistribution(D1,D4,D1,D2, mixCoeff = c(0.4,0.3,0.1,0.2), 
      withSimplify = FALSE)
D
D0<-flat.mix(D)
D0
plot(D0)
}
\keyword{distribution}
\keyword{arith}
\concept{random sample}
\concept{image distribution}
\concept{absolutely continuous distribution}
\concept{utility}

\name{showobj-methods}
\docType{methods}
\alias{showobj}
\alias{showobj-methods}
\alias{showobj,UnivariateDistribution-method}
\alias{showobj,DistrList-method}
\alias{showobj,UnivarMixingDistribution-method}
\alias{showobj,UnivarLebDecDistribution-method}
\alias{showobj,CompoundDistribution-method}
\title{Helping Methods for Functions print/show in Package `distr' }
\description{showobj-methods}

\usage{
showobj(object, ...)
\S4method{showobj}{UnivariateDistribution}(object, className = class(object)[1])
\S4method{showobj}{CompoundDistribution}(object, className = class(object)[1])
\S4method{showobj}{DistrList}(object, className = class(object)[1])
\S4method{showobj}{UnivarMixingDistribution}(object, className = class(object)[1])
\S4method{showobj}{UnivarLebDecDistribution}(object, className = class(object)[1],
objs)
}
\arguments{
  \item{\dots}{further parameters for \code{showobj}; not yet used.}
  \item{object}{distribution [list] object}
  \item{className}{name of the class of the object}
  \item{objs}{a character capturing the input from \code{show}, \code{print}}
}
\value{a character vector, which is then shown by method \code{show},
but can also be indented if necessary --- like in the possibly nested 
list classes}

\seealso{\code{\link{print-methods}}}
\keyword{print}
\keyword{methods}
\keyword{internal}
\keyword{distribution}

\name{NoSymmetry}
\alias{NoSymmetry}

\title{Generating function for NoSymmetry-class}
\description{Generates an object of class \code{"NoSymmetry"}.}
\usage{NoSymmetry()}
%\details{}
\value{Object of class \code{"NoSymmetry"}}
%\references{}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de}}
%\note{}
\seealso{\code{\link{NoSymmetry-class}}, \code{\link{DistributionSymmetry-class}}}
\examples{
NoSymmetry()

## The function is currently defined as
function(){ new("NoSymmetry") }
}
\concept{symmetry}
\keyword{distribution}

\name{solve-methods}
\docType{methods}
\alias{solve-methods}
\alias{solve}
\alias{solve,ANY,ANY-method}
\alias{solve,PosSemDefSymmMatrix,ANY-method}
\alias{solve,PosDefSymmMatrix,ANY-method}
\alias{solve,ANY-method}
\alias{solve,PosSemDefSymmMatrix-method}
\alias{solve,PosDefSymmMatrix-method}
\title{ Methods for Function solve in Package `distr' }
\description{solve-methods using generalized inverses for various types of matrices}

\usage{solve(a,b, ...)
\S4method{solve}{ANY,ANY}(a, b, generalized = 
getdistrOption("use.generalized.inverse.by.default"), tol = 1e-10)
\S4method{solve}{PosSemDefSymmMatrix,ANY}(a, b, generalized =
getdistrOption("use.generalized.inverse.by.default"), tol = 1e-10)
\S4method{solve}{PosDefSymmMatrix,ANY}(a, b, tol = 1e-10)
}
\arguments{
  \item{a}{matrix to be inverted / to be solved for RHS.}
  \item{b}{a numeric or complex vector or matrix giving the right-hand
          side(s) of the linear system.  If missing, \code{b} is taken to be
          an identity matrix and \code{solve} will return the inverse of
          \code{a}.}
  \item{\dots}{ further arguments to be passed to specific methods (see \code{\link[base]{solve}}).}
  \item{generalized}{ logical: should generalized / Moore-Penrose inverses be used? By default uses
  the corresponding global option to be set by \code{\link{distroptions}}.}
  \item{tol}{ the tolerance for detecting linear dependencies in the
          columns of \code{a}.  Default is \code{.Machine$double.eps}.}
}
\section{Methods}{\describe{
\item{solve}{\code{signature(a = "ANY", b = "ANY")}: tries to evaluate \code{solve.default} method from \pkg{base} in classical way;
if this gives an error, this one is returned if \code{generalized} is \code{TRUE}, else it will then
return \eqn{a^-b} where \eqn{a^-} is the pseudo or Moore-Penrose inverse of \eqn{a}.}
\item{solve}{\code{signature(a = "PosSemDefSymmMatrix", b = "ANY")}: evaluates \eqn{a^-b} where \eqn{a^-} is the pseudo or Moore-Penrose
inverse of \eqn{a}.}
\item{solve}{\code{signature(a = "PosDefSymmMatrix", b = "ANY")}: evaluates solve method from \pkg{base} in classical way.}
}}
\details{The method for the Moore-Penrose inverse for 
\code{signature(a = "PosSemDefSymmMatrix", b = "ANY")} uses \code{eigen} to find the eigenvalue 
decomposition of \code{a} and then simply "pseudo-inverts" the corresponding diagonal matrix built from
\code{eigen(a)$values}, while for \code{signature(a = "ANY", b = "ANY")}  it uses the svd 
decomposition of \code{a} and then simply "pseudo-inverts" the corresponding diagonal matrix built from
\code{svd(a)$d}.}
\author{Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}
\seealso{
\code{\link[base]{solve}} for the default method, \code{\link[base]{eigen}} and \code{\link[base]{svd}} 
for the pseudo inversion}
\keyword{algebra}
\keyword{array}
\concept{pseudo inverse}
\concept{inverse}
\concept{svd}
\concept{eigenvalue decomposition}

\name{DExp-class}
\docType{class}
\alias{DExp-class}
\alias{DExp}
\alias{Laplace}
\alias{DoubleExponential}
\alias{initialize,DExp-method}
\title{Class "DExp"}
\description{
  The double exponential or Laplace distribution with rate \eqn{\lambda} has density
  \deqn{
    f(x) = \frac{1}{2}\lambda {e}^{- \lambda |x|}}{
    f(x) = 1/2 lambda e^(- lambda |x|)}

  C.f. \code{\link{Exp-class}}, \code{\link[stats:Exponential]{rexp}}
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{DExp(rate)}.
This object is a double exponential (or Laplace) distribution. 
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}:
      The space of the image of this distribution has got dimension 1
      and the name "Real Space".}
    \item{\code{param}}{Object of class \code{"ExpParameter"}:
      the parameter of this distribution (rate), declared at its instantiation }
    \item{\code{r}}{Object of class \code{"function"}:
      generates random numbers (calls function rexp)}
    \item{\code{d}}{Object of class \code{"function"}:
      density function (calls function dexp)}
    \item{\code{p}}{Object of class \code{"function"}:
      cumulative function (calls function pexp)}
    \item{\code{q}}{Object of class \code{"function"}:
      inverse of the cumulative function (calls function qexp)}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
  Class \code{"AbscontDistribution"}, directly.\cr
  Class \code{"UnivariateDistribution"}, by class \code{"AbscontDistribution"}.
  Class \code{"Distribution"}, by class \code{"AbscontDistribution"}.
}

\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "DExp")}:
      initialize method}
    \item{rate}{\code{signature(object = "DExp")}:
      returns the slot rate of the parameter of the distribution}
    \item{rate<-}{\code{signature(object = "DExp")}:
      modifies the slot rate of the parameter of the distribution}
    \item{*}{\code{signature(e1 = "DExp", e2 = "numeric")}: 
    For the Laplace distribution we use its closedness under scaling transformations.}
  }
}

\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}
}
\seealso{
\code{\link{Exp-class}}
\code{\link{ExpParameter-class}}
\code{\link{AbscontDistribution-class}}
\code{\link{Reals-class}}
\code{\link[stats:Exponential]{rexp}}
}
\examples{
D <- DExp(rate = 1) # D is a Laplace distribution with rate = 1.
r(D)(1) # one random number generated from this distribution, e.g. 0.4190765
d(D)(1) # Density of this distribution is 0.1839397 for x = 1.
p(D)(1) # Probability that x < 1 is 0.8160603.
q(D)(.1) # Probability that x < -1.609438 is 0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
rate(D) # rate of this distribution is 1.
rate(D) <- 2 # rate of this distribution is now 2.
3*D ###  still a DExp -distribution
}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{Double exponential distribution}
\concept{Laplace distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{Length-methods}
\docType{methods}
\alias{Length-methods}
\alias{Length<--methods}
\alias{Length}
\alias{Length<-}
\alias{Length,Lattice-method}
\alias{Length<-,Lattice-method}
\alias{Length,LatticeDistribution-method}
\alias{Length<-,LatticeDistribution-method}
\title{ Methods for Function Length in Package `distr' }
\description{Length-methods}
\section{Methods}{\describe{
\item{Length}{\code{signature(object = "Lattice")}: returns the slot Length of the lattice }
\item{Length<-}{\code{signature(object = "Lattice")}: modifies the slot Length of the lattice }
\item{Length}{\code{signature(object = "LatticeDistribution")}: returns the slot Length of the lattice slot of the distribution}
\item{Length<-}{\code{signature(object = "LatticeDistribution")}: modifies the slot Length of the lattice slot of the distribution}
}}
\keyword{methods}
\keyword{distribution}
\concept{space}
\concept{S4 space class}
\concept{lattice}
\concept{affine linear lattice}
\concept{accessor function}
\concept{replacement function}

\name{distr-package}
\alias{distr-package}
\alias{distr}
\docType{package}
\title{
distr -- Object Oriented Implementation of Distributions
}
\description{
\pkg{distr} provides a conceptual treatment of distributions by means of S4
classes. A mother class \code{Distribution} is introduced with slots for a
parameter and ---most important--- for the four constitutive methods \code{r},
\code{d}, \code{p}, and \code{q} for simulation respectively for evaluation of
density / c.d.f.\ and quantile function of the corresponding distribution.
Most distributions of  package \pkg{stats} (like normal, Poisson, etc.)
are implemented as subclasses of either \code{AbscontDistribution} or
\code{DiscreteDistribution}, which themselves are again subclasses of
\code{Distribution}.
Up to arguments referring to a parameter of the distribution (like
\code{mean} for the normal distribution), these function slots have the
same arguments as those of package \pkg{stats}, i.e.; for a distribution object
\code{X} we may call these functions as
\itemize{
\item \code{r(X)(n)}
\item \code{d(X)(x, log = FALSE)}
\item \code{p(X)(q, lower.tail = TRUE, log.p = FALSE)}
\item \code{q(X)(p, lower.tail = TRUE, log.p = FALSE)}
}
For the arguments of these function slots see e.g. \code{\link{rnorm}}.
Note that, as usual, slots \code{d}, \code{p}, and \code{q} are vectorized
in their first argument, but are not on the subsequent ones.
In the environments of RStudio, see \url{https://posit.co} and
Jupyter IRKernel, see \url{https://github.com/IRkernel/IRkernel},
calls to \code{q} are caught away from standard R evaluation and are treated
in a non-standard way. This non-standard evaluation in particular throws
errors at calls to our accessor methods \code{q} to slot \code{q} of the
respective distribution object. To amend this, we provide function \code{q.l}
as alias to our accessors \code{q}, so that our packages also become available
 in these environments.
Arithmetics and unary mathematical transformations for distributions are
available: For \code{Distribution} objects \code{X} and \code{Y} expressions
like \code{3*X+sin(exp(-Y/4+3))} have their natural interpretation as
corresponding image distributions.
}
\details{
\tabular{ll}{
Package: \tab distr \cr
Version: \tab 2.9.1 \cr
Date: \tab 2022-11-14 \cr
Depends: \tab R(>= 3.4), methods, graphics, startupmsg, sfsmisc \cr
Suggests: \tab distrEx, svUnit (>= 0.7-11), knitr, distrMod, ROptEst \cr
Imports: \tab stats, grDevices, utils, MASS \cr
LazyLoad: \tab yes \cr
License: \tab LGPL-3 \cr
URL: \tab https://distr.r-forge.r-project.org/\cr
VCS/SVNRevision: \tab 1395 \cr
}}
\section{Classes}{
Distribution classes have a slot \code{param} the class of which
is is specialized for the particualar distributions. The parameter
classes for the particular distributions have slots with names
according to the corresponding \var{[rdpq]<name>} functions of
package \pkg{base}.
From version 1.9 on, \code{AbscontDistribution} and descendants have
a slot \code{\link{gaps}} for gaps in the support.
\code{DiscreteDistribution} and descendants have
an additional slot \code{\link{support}}, which is again specialized
to be a lattice for class \code{LatticeDistribution}. \cr
For saved objects from earlier versions, we provide the methods
\code{\link{isOldVersion}}, and  \code{\link{conv2NewVersion}}
to check whether the object was generated by an older version of
this package and to convert such an object to the new format,
respectively. This applies to objects of subclasses of
\code{AbscontDistribution} lacking a \code{gap}-slot as well as to
to objects of subclasses of \code{LatticeDistribution} lacking a
\code{lattice}-slot.\cr
To enhance accuracy, from version 1.9 on, we also provide
subclasses \code{AffLinAbscontDistribution}, \code{AffLinDiscreteDistribution},
and \code{AffLinLatticeDistribution}, as well as the class union
\code{AffLinDistribution}, so that in particular functionals like \code{E}
from package \pkg{distrEx} can recur to exact formula more frequently:
These classes have additional slots \code{a}, \code{b}, and \code{X0} to
reflect the fact, that a distribution object of theses classes has the
same distribution as \code{a*X0+b}.\cr
For all particular distributions, as well as for classes
\code{AbscontDistribution}, \code{DiscreteDistribution},
\code{LatticeDistribution}, \code{UnivarDistrList} and \code{DistrList}
generating functions are provided, e.g. \code{X <- Norm(mean = 3, sd = 2)}.
The same goes for the space classes.
All slots should be inspected / modified by means of corresponding
accessor- /replacement functions; e.g. \code{mean(X) <- 3}
Again to enhance accuracy, from version 2.0 on, we also provide
subclasses \code{UnivarMixingDistribution} to support mixing distributions,
\code{UnivarLebDecDistribution}, to support Lebesgue decomposed distributions
(with a discrete and an a.c. part) as well as \code{AffLinUnivarLebDecDistribution},
for corresponding affine linear transformations.
Class \code{UnivarLebDecDistribution} is closed under arithmetical operations
   +  \code{/}, \code{*}, \code{^} for pairs of independent variables
   +  \code{+}, \code{-} for pairs of independent variables
   +  affine linear transformations
   +  truncation, huberization, min/max
which are all now available analytically.\cr
(see Parameter classes).
\preformatted{
[*]: there is a generating function with the same name
##########################
Distribution classes
##########################
slots: [<name>(<class>)]
img(rSpace), param(OptionalParameter),
r(function), d(OptionalFunction), p(OptionalFunction), q(OptionalFunction),
.withSim(logical), .withArith(logical), .logExact(logical), .lowerExact(logical),
Symmetry(DistributionSymmetry)
"Distribution"
|>"UnivariateDistribution"
|>|>"UnivarMixingDistribution"            [*]
|>|>|>"UnivarLebDecDistribution"          [*]
|>|>|>|>"AffLinUnivarLebDecDistribution"
|>|>|>"CompoundDistribution"              [*]
|>|>"AbscontDistribution"                 [*]
|>|>|>"AffLinAbscontDistribution"
|>|>|>"Arcsine"                           [*]
|>|>|>"Beta"                              [*]
|>|>|>"Cauchy"                            [*]
|>|>|>"ExpOrGammaOrChisq" (VIRTUAL)
|>|>|>|>"Exp"                             [*]
|>|>|>|>"Gammad"                          [*]
|>|>|>|>"Chisq"                           [*]
|>|>|>"Fd"                                [*]
|>|>|>"Lnorm"                             [*]
|>|>|>"Logis"                             [*]
|>|>|>"Norm"                              [*]
|>|>|>"Td"                                [*]
|>|>|>"Unif"                              [*]
|>|>|>"Weibull"                           [*]
|>|>|"DiscreteDistribution"               [*]
|>|>|>"AffLinDiscreteDistribution"
|>|>|>"LatticeDistribution"               [*]
|>|>|>|>"AffLinLatticeDistribution"
|>|>|>|>"Binom"                           [*]
|>|>|>|>"Dirac"                           [*]
|>|>|>|>"Hyper"                           [*]
|>|>|>|>"NBinom"                          [*]
|>|>|>|>|>"Geom"                          [*]
|>|>|>|>"Pois"                            [*]
"AffLinDistribution" = union ( "AffLinAbscontDistribution",
                               "AffLinDiscreteDistribution",
                               "AffLinUnivarLebDecDistribution" )
"DistrList"
|>"UnivarDistrList"                       [*]
"AcDcLc" = union ( "AbscontDistribution",
                   "DiscreteDistribution",
                   "UnivarLebDecDistribution" )
##########################
Parameter classes
##########################
"OptionalParameter"
|>"Parameter"
|>|>"BetaParameter"
|>|>"BinomParameter"
|>|>"CauchyParameter"
|>|>"ChisqParameter"
|>|>"DiracParameter"
|>|>"ExpParameter"
|>|>"FParameter"
|>|>"GammaParameter"
|>|>"GeomParameter"
|>|>"HyperParameter"
|>|>"LnormParameter"
|>|>"LogisParameter"
|>|>"NbinomParameter"
|>|>"NormParameter"
|>|>"UniNormParameter"
|>|>|>"PoisParameter"
|>|>"TParameter"
|>|>"UnifParameter"
|>|>"WeibullParameter"
##########################
Space classes
##########################
"rSpace"
|>"EuclideanSpace"
|>|>"Reals"
|>"Lattice"
|>"Naturals"
##########################
Symmetry classes
##########################
slots:
type(character), SymmCenter(ANY)
"Symmetry"
|>"NoSymmetry"          [*]
|>"EllipticalSymmetry"  [*]
|>|>"SphericalSymmetry" [*]
|>"DistributionSymmetry"
|>"FunctionSymmetry"
|>|>"NonSymmetric"      [*]
|>|>"EvenSymmetric"     [*]
|>|>"OddSymmetric"      [*]
list thereof
"DistrSymmList"         [*]
"FunSymmList"           [*]
##########################
Matrix classes
##########################
slots:
none
"PosSemDefSymmMatrix" [*] is subclass of class "matrix" of package "base".
|>"PosDefSymmMatrix"  [*]
##########################
Class unions
##########################
"OptionalNumeric" = union("numeric", "NULL")
"OptionalMatrix" = union("matrix","NULL")
}
}
\section{Methods}{
The group \code{Math} of unary (see \link[methods:S4groupGeneric]{Math}) as well as
convolution are made available for distributions, see \link{operators-methods}
;in particular for convolution powers, we have method \link{convpow}.
Besides, there are \code{plot} and \code{print}-methods for distributions.
For the space classes, we have \code{liesIn}, for the
\code{DicreteDistribution} class, we have \code{liesInSupport}, as well
as a generating function.
The "history" of distributions
obtained by chaining operations may be shortened using \code{simplifyr}.
}
\section{Functions}{
\preformatted{
RtoDPQ                  Default procedure to fill slots d,p,q given r
                        for a.c. distributions
RtoDPQ.d                Default procedure to fill slots d,p,q given r
                        for discrete distributions
RtoDPQ.LC               Default procedure to fill slots d,p,q given r
                        for Lebesgue decomposed distributions
decomposePM             decomposes a distribution into positive and negative
                        part and, if discrete, into part '0'
simplifyD               tries to reduce/simplify mixing distribution using
                        that certain weights are 0
flat.LCD                makes a single UnivarLebDecDistribution out of
                        a list of UnivarLebDecDistribution with corresp. weights
flat.mix                makes a single UnivarLebDecDistribution out of
                        a list of a UnivarMixingDistribution
distroptions            Functions to change the global variables of the
                        package 'distr'
standardMethods         Utility to automatically generate accessor and
                        replacement functions
}}
\note{Arithmetics on distribution objects are understood as operations on
corresponding  (independent) r.v.'s and \strong{not} on distribution functions
or densities.\cr
See also \code{distrARITH()}.\cr
Some functions of package \pkg{stats} have intentionally been masked,
but completely retain their functionality --- see \code{distrMASK()}.\cr
Accuracy of these arithmetics is controlled by global options which may
be inspected / set by \code{distroptions()} and \code{getdistrOption()},
confer \link{distroptions} .
}
\section{Extension Packages in distrXXX family}{
Please note that there are extension packages of this packages
available on CRAN,
\describe{
\item{\pkg{distrDoc}}{a documentation package providing joint documentation
for all packages of the distrXXX family of packages in the form of vignette
'distr'; try \code{require(distrDoc); vignette("distr")}.}
\item{\pkg{distrEx}}{provides functionals (like \code{E}, \code{sd}, \code{mad})
operating on distributions, as well as distances between distributions and basic
support for multivariate and conditional distributions.}
\item{\pkg{distrSim}}{for the standardized treatment of simulations,
also under contaminations.}
\item{\pkg{distrTEst}}{with classes and methods for evaluations of statistical procedures
on simulations generated by \pkg{distrSim}.}
\item{\pkg{distrTeach}}{embodies illustrations for basic stats courses using
our distribution classes.}
\item{\pkg{distrMod}}{provides classes for parametric models and hence covers,
in an object orientated way, estimation in statistical models.}
\item{\pkg{distrEllipse}}{provides classes for elliptically symmetric
distributions.}
}
}
\section{Package versions}{
Note: The first two numbers of package versions do not necessarily reflect
 package-individual development, but rather are chosen for the
 distrXXX family as a whole in order to ease updating "depends"
 information.
}
\section{Acknowledgement}{
We thank Martin Maechler, Josef Leydold, John Chambers,
Duncan Murdoch, Gregory Warnes, Paul Gilbert, Kurt Hornik,
Uwe Ligges, Torsten Hothorn, and Seth Falcon for their help
in preparing this package.}
\author{
Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr
Thomas Stabla \email{statho3@web.de},\cr
Florian Camphausen \email{fcampi@gmx.de},\cr
Matthias Kohl \email{Matthias.Kohl@stamats.de}\cr
\emph{Maintainer:} Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}
}
\references{
P. Ruckdeschel, M. Kohl, T. Stabla, F. Camphausen (2006):
S4 Classes for Distributions, \emph{R News}, \emph{6}(2), 2-6.
\url{https://CRAN.R-project.org/doc/Rnews/Rnews_2006-2.pdf}
P. Ruckdeschel and M. Kohl (2014):
   General purpose convolution algorithm for distributions
  in S4-Classes by means of FFT. \emph{J. Statist. Softw.}
  \bold{59}(4): 1-25.
a vignette for packages \pkg{distr}, \pkg{distrSim}, \pkg{distrTEst}, and
\pkg{distrEx} is included into the mere documentation package \pkg{distrDoc}
and may be called by
\code{require("distrDoc");vignette("distr")}
%a more detailed manual for \pkg{distr}, \pkg{distrSim}, \pkg{distrTEst}, and \pkg{distrEx} may be downloaded from
%\url{http://www.uni-bayreuth.de/departments/math/org/mathe7/DISTR/distr.pdf}
a homepage to this package is available under\cr
\url{https://distr.r-forge.r-project.org/}
}
\section{Start-up-Banner}{
You may suppress the start-up banner/message completely by setting
\code{options("StartupBanner"="off")} somewhere before loading this
package by \code{library} or \code{require} in your R-code / R-session.
If option \code{"StartupBanner"} is not defined (default) or setting
\code{options("StartupBanner"=NULL)} or
\code{options("StartupBanner"="complete")} the complete start-up banner is
displayed.
For any other value of option \code{"StartupBanner"}
(i.e., not in \code{c(NULL,"off","complete")})
only the version information is displayed.
The same can be achieved by wrapping the \code{library} or \code{require}
 call into either \code{suppressStartupMessages()} or
 \code{onlytypeStartupMessages(.,atypes="version")}.
 As for general \code{packageStartupMessage}'s, you may also suppress all
 the start-up banner by wrapping the \code{library} or \code{require}
 call into \code{suppressPackageStartupMessages()} from
 \pkg{startupmsg}-version 0.5 on.
 }
\section{Demos}{
Demos are available --- see \code{demo(package="distr")}}
\keyword{package}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{discrete distribution}
\concept{lattice distribution}
\concept{S4 classes}
\concept{arithmetics for distributions}
\concept{S4 parameter class}
\concept{S4 distribution class}
\concept{S4 space class}
\examples{
X <- Unif(2,3)
Y <- Pois(lambda = 3)
Z <- X+Y  # generates Law of corresponding independent variables
p(Z)(0.2)
r(Z)(1000)
plot(Z+sin(Norm()))
}

\name{internals_for_qqplot}
\alias{internals_for_qqplot}
\alias{.inGaps}
\alias{.isReplicated}
\alias{.NotInSupport}
\alias{.SingleDiscrete}
\alias{.makeLenAndOrder}
\alias{.q2kolmogorov}
\alias{.BinomCI.in}
\alias{.BinomCI}
\alias{.BinomCI.nosym}
\alias{.q2pw}
\alias{.confqq}
\alias{.deleteItemsMCL}
\alias{.distrExInstalled}


\title{Internal functions for qqplot of package distr}

\description{These functions are used internally by qqplot of package distr.}

\usage{
.inGaps(x,gapm)
.isReplicated(x, tol = .Machine$double.eps)
.NotInSupport(x,D)
.SingleDiscrete(x,D)
.makeLenAndOrder(x,ord)

.BinomCI.in(t,p.bi,x.i, del.i=0,D.i,n.i,alpha.i)
.BinomCI(x,p.b,D,n,alpha, silent0 = TRUE)
.BinomCI.nosym(x,p.b,D,n,alpha, silent0 = TRUE)

.q2kolmogorov(alpha,n,exact=(n<100), silent0 = TRUE)
.q2pw(x,p.b,D,n,alpha,exact=(n<100),nosym=FALSE, silent0 = TRUE)

.confqq(x,D, datax=FALSE, withConf.pw  = TRUE,  withConf.sim = TRUE, alpha,
                    col.pCI, lty.pCI, lwd.pCI, pch.pCI, cex.pCI,
                    col.sCI, lty.sCI, lwd.sCI, pch.sCI, cex.sCI,
                    n,exact.sCI=(n<100),exact.pCI=(n<100),
                    nosym.pCI = FALSE, with.legend = TRUE,
                    legend.bg = "white", legend.pos = "topleft",
                    legend.cex = 0.8, legend.pref = "", legend.postf = "",
                    legend.alpha = alpha, qqb0 = NULL, transf0=NULL, debug = FALSE)

.deleteItemsMCL(mcl)
.distrExInstalled
}


\arguments{
\item{x}{a (numeric) vector}
\item{gapm}{matrix; the gap matrix as in slot \code{gaps} of
  an \code{"AbscontDistribution"} or \code{"UnivarLebDecDistribution"}
  object.
  }
\item{tol}{numeric; tolerance for separating points.}
\item{D}{object of class \code{"UnivariateDistribution"}}
\item{datax}{logical; (to be used in \pkg{distrMod}) shall data be plotted on x-axis?}
\item{ord}{integer; the result of a call to \code{order}}
\item{alpha}{numeric in [0,1]; confidence level}
\item{n}{integer; sample size}
\item{exact}{logical; shall finite sample version be used?}
\item{t}{current  (half of the) width of the confidence interval.}
\item{p.bi}{(local) (binomial) c.d.f. value at \code{x.i}.}
\item{x.i}{a (numeric) vector}
\item{del.i}{numeric; a (local) asymmetry parameter to pass on to
\code{optim} and \code{uniroot} --- the endpoints of the searched interval
are \code{x.i+t/sqrt(n)+del.i/sqrt(n)} and \code{x.i-t/sqrt(n)+del.i/sqrt(n)}.}
\item{D.i}{object of class \code{"UnivariateDistribution"}}
\item{n.i}{integer; (local) sample size}
\item{alpha.i}{numeric in [0,1]; (local) confidence level}
\item{p.b}{(binomial) c.d.f. value at \code{x}.}
\item{nosym}{logical; shall we compute shortest (asymmetric) confidence intervals;}
\item{withConf.pw}{logical; shall pointwise confidence lines be plotted?}
\item{withConf.sim}{logical; shall simultaneous confidence lines be plotted?}
\item{exact.pCI}{logical; shall pointwise CIs be determined with exact Binomial distribution?}
\item{exact.sCI}{logical; shall simultaneous CIs be determined with exact kolmogorov distribution?}
\item{nosym.pCI}{logical; shall we use (shortest) asymmetric CIs?}
\item{col.pCI}{color for the pointwise CI}
\item{lty.pCI}{line type for the pointwise CI}
\item{lwd.pCI}{line width for the pointwise CI}
\item{pch.pCI}{symbol for points (for discrete mass points) in pointwise CI}
\item{cex.pCI}{magnification factor for points (for discrete mass points) in pointwise CI}
\item{col.sCI}{color for the simultaneous CI}
\item{lty.sCI}{line type for the simultaneous CI}
\item{lwd.sCI}{line width for the simultaneous CI}
\item{pch.sCI}{symbol for points (for discrete mass points) in simultaneous CI}
\item{cex.sCI}{magnification factor for points (for discrete mass points) in simultaneous CI}
\item{with.legend}{logical; shall a legend be plotted?}
\item{legend.bg}{background color for the legend}
\item{legend.pos}{position for the legend}
\item{legend.cex}{magnification factor for the legend}
\item{legend.pref}{character to be prepended to legend text}
\item{legend.postf}{character to be appended to legend text}
\item{legend.alpha}{nominal coverage probability}
\item{mcl}{arguments in call as a list}
\item{qqb0}{precomputed return value of \code{qqbounds}}
\item{transf0}{optional transformation of x-values (by default \code{NULL} and then ignored)}
\item{debug}{logical; if \code{TRUE} additional output to debug confidence bounds. }
\item{silent0}{logical; it is used as argument \code{silent} in \code{try}-catches
          within this function. }
}

\details{
\code{.inGaps} produces a logical vector of same length as \code{x} with
entries \code{TRUE} if the corresponding component of \code{x} lies within a
gap as given by gap matrix \code{gapm} and \code{FALSE} otherwise.

\code{.isReplicated} produces a logical vector of same length as \code{x} with
entries \code{TRUE} if the corresponding component of \code{x} appears at least
twice within \code{x} and \code{FALSE} otherwise.

\code{.NotInSupport} produces a logical vector of same length as \code{x} with
entries \code{TRUE} if the corresponding component of \code{x} does not
lie within the support of \code{D} and \code{FALSE} otherwise.

\code{.SingleDiscrete} produces a numerical vector of same length as \code{x} with
values \code{0}  if the corresponding component of \code{x} is discrete mass point
of \code{D}, \code{1}  if the corresponding component of \code{x} lies within
the continuous support of \code{D}, \code{2} and \code{3}
if the corresponding component of \code{x}
is a left resp. right end point of a gap of \code{D}, and \code{4} if
the corresponding component of \code{x} does not lie within the support of \code{D}
at all.

\code{.makeLenAndOrder} by standard recycling roules respectively by truncation
at the end, forces \code{x} to length \code{length{ord}} and then orders the
result according to \code{ord}.

\code{.q2kolmogorov}, in the finite sample version (\code{exact==TRUE}),
returns the corresponding \code{alpha}-quantile
of the exact Kolmogorov distribution multiplied by \eqn{\sqrt{n}}{sqrt(n)}, and
in the asymptotic version (\code{exact==FALSE}),
the the corresponding (upper) \code{alpha}-quantile
of the asymptotic Kolmogorov distribution. Doing so we make use of
C-function \code{"pkolmogorov2x"} (from \code{\link[stats]{ks.test}} in package \pkg{stats})
and R-function \code{pkstwo} (again from \code{\link[stats]{ks.test}} in package \pkg{stats}).

\code{.BinomCI.in} in a non-vectorized form, computes,
for given \code{t}, \code{x}, \eqn{\alpha}{\code{alpha}}, \eqn{\delta}{\code{del}},
and for \eqn{X\sim D}{X distributed as D}, the discrepancy
\deqn{P(\sqrt{n} |X-x-\delta| \leq t) - \alpha}{P(sqrt(n) abs(X-x-del) <= t) - alpha}


\code{.BinomCI}, in a vectorized form, computes,
for given \code{x}, \eqn{\alpha}{\code{alpha}}, \eqn{\delta}{\code{del}},
values \code{t} such that,
pointwise in \code{x} and for \eqn{X\sim D}{X distributed as D},
\deqn{P(\sqrt{n} |X-x-\delta| \leq t) = \alpha}{P(sqrt(n) abs(X-x-del) <= t) = alpha}

\code{.BinomCI.nosym}, in an outer loop, by varying \code{del} in the former
formula, tries to minimize the length of
a corresponding level alpha confidence interval containing the estimate.


\code{.q2pw} computes  pointwise finite sample or asymptotic confidence widths
by means of binomial probabilities / quantiles, in the former case either
symmetric (default) or shortest asymmetric; in the asymptotic case, for
distributions without a Lebesgue density, for the corresponding
density value at the quantile appearing in the expression for the
asymptotic variance, we make an approximation of \code{(D-E(D))/sd(D)} by
the standard normal, using the density of the latter one; this latter approximation
is only  available if \code{.distrExInstalled == TRUE}; otherwise the corresponding
columns will be filled with \code{NA}.

\code{.confqq} calls \code{qqbound} to compute the confidence intervals
and plots them; returns the return value of qqbound.

\code{.deleteItemsMCL} deletes arguments from a call list which
functions like \code{plot}, \code{lines}, \code{points} cannot digest;
this is necessary in the manipulation of an original call
to a specific \code{qqplot} method to pass on the \code{\dots} argument
correctly to calls the mentioned functions.

\code{.distrExInstalled} is a constant logical --- \code{TRUE} if package
\pkg{distrEx} is installed.

}


\value{
\item{.inGaps}{a logical vector of same length as \code{x}.}
\item{.isReplicated}{a logical vector of same length as \code{x}.}
\item{.NotInSupport}{a logical vector of same length as \code{x}.}
\item{.SingleDiscrete}{a vector of same length as \code{x} with entries in the
set \eqn{\{0,1,2,3,4\}}{{0,1,2,3,4}}.}
\item{.makeLenAndOrder}{a numeric of length \code{length(ord}.}
\item{.BinomCI.in}{a numeric of length 1: the discrepancy
\deqn{P(\sqrt{n} |X-x-\delta| \leq t) - \alpha}{P(sqrt(n) abs(X-x-del) <= t) - alpha}
}
\item{.BinomCI}{a numeric matrix with two columns \code{"left"} and \code{"right"}
with the corresponding pointwise confidence widths.}
\item{.BinomCI.nosym}{a numeric matrix with two columns \code{"left"} and \code{"right"}
with the corresponding pointwise confidence widths.}

\item{.q2kolmogorov}{a numeric of length 1; a corresponding quantile of the
(exact/asymptotic) Kolmogorov distribution}
\item{.q2pw}{a numeric matrix with two columns \code{"left"} and \code{"right"}
with the corresponding pointwise confidence widths.}
\item{.confqq}{\code{invisible(NULL)}}
\item{.deleteItemsMCL}{the manipulated list of arguments}
}

\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},
}

\seealso{\code{\link[stats]{ks.test}}, \code{\link[distr]{qqplot}}
, \code{\link[distrMod]{qqplot}}, \code{\link[RobAStBase]{qqplot}}
}
\keyword{internal}
\concept{utilities}

\name{q.r-methods}
\docType{methods}
\alias{q.r-methods}
\alias{q.r}
\alias{q.r,AbscontDistribution-method}
\alias{q.r,DiscreteDistribution-method}
\alias{q.r,UnivarLebDecDistribution-method}
\alias{q.r,UnivarMixingDistribution-method}
\title{Methods for Function q.r in Package `distr'}
\description{q.r-methods}

\section{Methods}{
return the right-continuous quantile function, i.e.; 
\eqn{{\rm q.r}(s)=\sup\{t \,\big|\, P({\tt object}\leq t)\leq s\}}{q.r(s)=sup\{t|P(object<=t)<=s\}}
\describe{
\item{q.r}{\code{signature(object  = "DiscreteDistribution")}} 
\item{q.r}{\code{signature(object  = "AbscontDistribution")}} 
\item{q.r}{\code{signature(object  = "UnivarLebDecDistribution")}}
\item{q.r}{\code{signature(object  = "UnivarMixingDistribution")}}
}}

\seealso{
\code{\link{DiscreteDistribution-class}}
\code{\link{UnivarLebDecDistribution-class}}
}
\keyword{distribution}
\keyword{methods}
\concept{pseudo-inverse distribution function}
\concept{quantile function}
\concept{left-continuous}
\concept{right-continuous}

\name{UnivarMixingDistribution}
\alias{UnivarMixingDistribution}

\title{Generating function for Class "UnivarMixingDistribution"}
\description{Generates an object of class \code{"UnivarMixingDistribution"}.}
\usage{UnivarMixingDistribution(..., Dlist, mixCoeff, 
                                withSimplify = getdistrOption("simplifyD"))}
\arguments{
  \item{\dots}{Objects of class \code{"UnivariateDistribution"} (or subclasses)}
  \item{Dlist}{an optional list or object of class \code{"UnivarDistrList"};
  if not missing it is appended to argument \code{\dots}; this way 
  \code{UnivarMixingDistribution} may also be called with a list (or 
  \code{"UnivarDistrList"}-object) as argument as suggested in an e-mail
  by Krunoslav Sever (thank you!)}
  \item{mixCoeff}{Objects of class \code{"numeric"} : a vector of 
            probabilities for the mixing components (must be of same length as
            arguments in \dots).}
  \item{withSimplify}{\code{"logical"}: shall the return value be piped through a 
   call to \code{simplifyD}?}
}
\details{If \code{mixCoeff} is missing, all elements in \code{\dots}
  are equally weighted.}
\value{Object of class \code{"UnivarMixingDistribution"}, or if 
argument \code{withSimplify} is \code{TRUE} and the resulting 
object would have one mixing component with probability (almost) 1,
\code{UnivarMixingDistribution} will return this component.}
%\references{}
\author{Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}
%\note{}
\seealso{\code{\link{UnivarMixingDistribution-class}},
  \code{\link{simplifyD}}}
\examples{
mylist <- UnivarMixingDistribution(Binom(3,.3), Dirac(2), Norm(), 
          mixCoeff=c(1/4,1/5,11/20))
}
\keyword{distribution}
\keyword{list}
\concept{multivariate distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{sdlog-methods}
\docType{methods}
\alias{sdlog-methods}
\alias{sdlog<--methods}
\alias{sdlog}
\alias{sdlog<-}
\alias{sdlog,LnormParameter-method}
\alias{sdlog,Lnorm-method}
\alias{sdlog<-,LnormParameter-method}
\alias{sdlog<-,Lnorm-method}
\title{ Methods for Function sdlog in Package `distr' }
\description{sdlog-methods}
\section{Methods}{\describe{

\item{sdlog}{\code{signature(object = "LnormParameter")}: returns the slot \code{sdlog} of the parameter of the distribution }
\item{sdlog<-}{\code{signature(object = "LnormParameter")}: modifies the slot \code{sdlog} of the parameter of the distribution }

\item{sdlog}{\code{signature(object = "Lnorm")}: returns the slot \code{sdlog} of the parameter of the distribution }
\item{sdlog<-}{\code{signature(object = "Lnorm")}: modifies the slot \code{sdlog} of the parameter of the distribution }
}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{Log-Normal distribution}
\concept{S4 parameter class}
\concept{accessor function}
\concept{replacement function}

\name{FParameter-class}
\docType{class}
\alias{FParameter-class}
\alias{initialize,FParameter-method}
\title{Class "FParameter"}
\description{ The parameter of a F distribution, used by Fd-class}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("FParameter", df1, df2, ncp)}.
  Usually an object of this class is not needed on its own, it is
  generated automatically when an object of the class Fd is instantiated. 
}
\section{Slots}{
  \describe{
    \item{\code{df1}}{Object of class \code{"numeric"}: the degrees of freedom of the nominator of an F distribution}
    \item{\code{df2}}{Object of class \code{"numeric"}: the degrees of freedom of the denominator of an F distribution}
    \item{\code{ncp}}{Object of class \code{"numeric"}: the noncentrality parameter of an F distribution }
    \item{\code{name}}{Object of class \code{"character"}: a name / comment for the parameters}
  }
}
\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "FParameter")}: initialize method }
    \item{df1}{\code{signature(object = "FParameter")}:
      returns the slot df1 of the parameter of the distribution}
    \item{df1<-}{\code{signature(object = "FParameter")}:
      modifies the slot df1 of the parameter of the distribution}
    \item{df2}{\code{signature(object = "FParameter")}:
      returns the slot df2 of the parameter of the distribution}
    \item{df2<-}{\code{signature(object = "FParameter")}:
      modifies the slot df2 of the parameter of the distribution}
    \item{ncp}{\code{signature(object = "FParameter")}: 
      returns the slot \code{ncp} of the parameter of the distribution }
    \item{ncp<-}{\code{signature(object = "FParameter")}: 
      modifies the slot \code{ncp} of the parameter of the distribution }
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
}
\seealso{
\code{\link{Fd-class}}
\code{\link{Parameter-class}}
}
\examples{
W <- new("FParameter", df1 = 1, df2 = 1, ncp = 0)
df2(W) # df2 of this distribution is 1.
df2(W) <- 2 # df2 of this distribution is now 2.
}
\keyword{distribution}
\concept{parameter}
\concept{F distribution}
\concept{S4 parameter class}

\name{OptionalParameter-class}
\docType{class}
\alias{OptionalParameter-class}
\alias{OptionalMatrix-class}

\title{Classes "OptionalParameter", "OptionalMatrix" }
\description{ auxiliary classes; may contain either a Parameter or NULL, resp.
a matrix or NULL cf. J. Chambers, "green book".}
\section{Objects from the Class}{"OptionalParameter" is a virtual Class: No objects may be created from it;
"OptionalMatrix" is a class generated by \code{setClassUnion()} so may contain NULL or any matrix}
\section{Methods}{
No methods defined with class "OptionalParameter" in the signature.
}
\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
}
\seealso{\code{\link{Parameter-class}}, \code{\link{AbscontDistribution-class}}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{S4 parameter class}
\concept{class union}
\concept{virtual class}

\name{Cauchy-class}
\docType{class}
\alias{Cauchy-class}
\alias{Cauchy}
\alias{initialize,Cauchy-method}

\title{Class "Cauchy"}
\description{  The Cauchy distribution with location \eqn{l}, by default \eqn{=0}, and scale \eqn{s} , by default \eqn{=1},has
  density
  \deqn{f(x) = \frac{1}{\pi s}
    \left( 1 + \left(\frac{x - l}{s}\right)^2 \right)^{-1}%
  }{f(x) = 1 / (pi s (1 + ((x-l)/s)^2))}
  for all \eqn{x}.
C.f. \code{\link[stats:Cauchy]{rcauchy}}
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Cauchy(location, scale)}.
This object is a Cauchy distribution.
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}: The domain of this distribution has got dimension 1
    and the name "Real Space". }
    \item{\code{param}}{Object of class \code{"CauchyParameter"}: the parameter of this distribution (location and scale),
     declared at its instantiation }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers (calls function \code{rcauchy})}
    \item{\code{d}}{Object of class \code{"function"}: density function (calls function \code{dcauchy})}
    \item{\code{p}}{Object of class \code{"function"}: cumulative function (calls function \code{pcauchy})}
    \item{\code{q}}{Object of class \code{"function"}: inverse of the cumulative function (calls function \code{qcauchy})}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"AbscontDistribution"}, directly.\cr
Class \code{"UnivariateDistribution"}, by class \code{"AbscontDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"AbscontDistribution"}.
}
\section{Is-Relations}{
By means of \code{setIs}, R ``knows'' that a distribution object \code{obj} of class \code{"Cauchy"} with location 0 and scale 1 also is
a T distribution with parameters \code{df = 1, ncp = 0}. 
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Cauchy")}: initialize method }
    \item{location}{\code{signature(object = "Cauchy")}: returns the slot \code{location} of the parameter of the distribution }
    \item{location<-}{\code{signature(object = "Cauchy")}: modifies the slot \code{location} of the parameter of the distribution }
    \item{scale}{\code{signature(object = "Cauchy")}: returns the slot \code{scale} of the parameter of the distribution }
    \item{scale<-}{\code{signature(object = "Cauchy")}: modifies the slot \code{scale} of the parameter of the distribution }
    \item{+}{\code{signature(e1 = "Cauchy", e2 = "Cauchy")}: For the Cauchy distribution the exact convolution formula is
      implemented thereby improving the general numerical approximation.}
    \item{*}{\code{signature(e1 = "Cauchy", e2 = "numeric")}}
    \item{+}{\code{signature(e1 = "Cauchy", e2 = "numeric")}: 
    For the Cauchy location scale family we use its closedness under affine linear transformations.}
  }
further arithmetic methods see \link{operators-methods}
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}


\seealso{
\code{\link{CauchyParameter-class}}
\code{\link{AbscontDistribution-class}}
\code{\link{Reals-class}}
\code{\link[stats:Cauchy]{rcauchy}}





}
\examples{
C <- Cauchy(location = 1, scale = 1) # C is a Cauchy distribution with location=1 and scale=1.
r(C)(1) # one random number generated from this distribution, e.g. 4.104603
d(C)(1) # Density of this distribution is 0.3183099 for x=1.
p(C)(1) # Probability that x<1 is 0.5.
q(C)(.1) # Probability that x<-2.077684 is 0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
location(C) # location of this distribution is 1.
location(C) <- 2 # location of this distribution is now 2.
is(C,"Td") # no
C0 <- Cauchy() # standard, i.e. location = 0, scale = 1
is(C0,"Td") # yes
as(C0,"Td") 
}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{Cauchy distribution}
\concept{T(1) distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{InternalClassUnions}
\docType{class}
\alias{InternalClassUnions}
\alias{OptionalNumeric-class}

\title{Class unions in 'distr'}
\description{Class unions in package \pkg{distr} defined for internal 
 purposes; these are \code{OptionalNumeric},}

\section{Objects from the Class}{All of these classes are virtual: 
 No objects may be created from them.}
\details{
These classes are used internally to make available methods or to allow
slots of classes to be filled with varying types. In particular
\itemize{
\item \code{"OptionalNumeric"}: may contain objects of class  \code{"numeric"}
      or \code{"NULL"}; it is used e.g. for slot \code{nuisance} of class
      \code{"ParamFamParameter"}, as it may or may not be present but if so
      it has to be numeric.
}
}

%\section{Methods}{}
%\references{}
\author{Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}
%\note{}
\seealso{\code{\link[methods:BasicClasses]{numeric-class}},
}
\keyword{classes}
\keyword{internal}

\name{r-methods}
\docType{methods}
\alias{r-methods}
\alias{r}
\alias{r,Distribution-method}
\title{ Methods for Function r in Package `distr' }
\description{r-methods}
\section{Methods}{\describe{
    \item{r}{\code{signature(object = "Distribution")}: generates random deviates
    according to the distribution}
}}
\seealso{
\code{\link{Distribution-class}}
}
\keyword{distribution}
\keyword{methods}
\concept{random number generator}
\concept{RNG}
\concept{accessor function}
\concept{pseudo random number}
\concept{anamorphosis}

\name{SphericalSymmetry-class}
\docType{class}
\alias{SphericalSymmetry-class}

\title{Class for Spherical Symmetric Distributions}
\description{Class for spherical symmetric distributions.}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("SphericalSymmetry")}.
  More frequently they are created via the generating function 
  \code{SphericalSymmetry}. Spherical symmetry for instance leads to 
  a simplification for the computation of optimally robust influence curves.
}
\section{Slots}{
  \describe{
    \item{\code{type}}{Object of class \code{"character"}:
      contains \dQuote{spherical symmetric distribution} }
    \item{\code{SymmCenter}}{Object of class \code{"numeric"}:
      center of symmetry }
  }
}
\section{Extends}{
Class \code{"EllipticalSymmetry"}, directly.\cr
Class \code{"DistributionSymmetry"}, by class \code{"EllipticalSymmetry"}.\cr
Class \code{"Symmetry"}, by class \code{"EllipticalSymmetry"}.
}
%\section{Methods}{}
%\references{}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de}}
%\note{}
\seealso{\code{\link{SphericalSymmetry}}, \code{\link{DistributionSymmetry-class}}}
\examples{
new("SphericalSymmetry")
}
\concept{spherical symmetry}
\keyword{classes}
\keyword{distribution}

\name{Max-methods}
\docType{methods}
\alias{Max-methods}
\alias{Max<--methods}
\alias{Max}
\alias{Max<-}
\alias{Max,UnifParameter-method}
\alias{Max,Unif-method}
\alias{Max<-,UnifParameter-method}
\alias{Max<-,Unif-method}
\title{ Methods for Function Max in Package `distr' }
\description{Max-methods}

\section{Methods}{\describe{

\item{Max}{\code{signature(object = "UnifParameter")}: returns the slot \code{Max} of the parameter of the distribution }
\item{Max<-}{\code{signature(object = "UnifParameter")}: modifies the slot \code{Max} of the parameter of the distribution }

\item{Max}{\code{signature(object = "Unif")}: returns the slot \code{Max} of the parameter of the distribution }
\item{Max<-}{\code{signature(object = "Unif")}: modifies the slot \code{Max} of the parameter of the distribution }
}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{Uniform distribution}
\concept{S4 parameter class}
\concept{accessor function}
\concept{replacement function}

\name{CompoundDistribution-class}
\docType{class}
\alias{CompoundDistribution-class}
\alias{NumbOfSummandsDistr}
\alias{SummandsDistr}
\alias{NumbOfSummandsDistr-methods}
\alias{SummandsDistr-methods}
\alias{NumbOfSummandsDistr,CompoundDistribution-method}
\alias{SummandsDistr,CompoundDistribution-method}
\alias{coerce,CompoundDistribution,UnivarLebDecDistribution-method}
\alias{UnivDistrListOrDistribution-class}

\title{Class "CompoundDistribution"}
\description{\code{CompoundDistribution}-class is a class to formalize 
                   compound distributions; it is a subclass to
                   class \code{UnivarMixingDistribution}.}
\section{Objects from the Class}{
Objects can be created by calls of the form 
\code{new("CompoundDistribution", ...)}.
  More frequently they are created via the generating function 
  \code{\link{CompoundDistribution}}.
}
\section{Slots}{
  \describe{
    \item{\code{NumbOfSummandsDistr}}{Object of class \code{"DiscreteDistribution"},
     the frequency distribution.}
    \item{\code{SummandsDistr}}{Object of class \code{"UnivDistrListOrDistribution"},
     that is, either of class \code{"UnivarDistrList"} (non i.i.d. case) or
     of class \code{"UnivariateDistribution"} (i.i.d. case); the summand distribution(s).}
    \item{\code{mixCoeff}}{Object of class \code{"numeric"}: a vector of 
            probabilities for the mixing components.}
    \item{\code{mixDistr}}{Object of class \code{"UnivarDistrList"}: a list of
    univariate distributions containing the mixing components; must be of same
    length as \code{mixCoeff}.}
    \item{\code{img}}{Object of class \code{"Reals"}: the space of the image of this distribution which has dimension 1
    and the name "Real Space" }
    \item{\code{param}}{Object of class \code{"Parameter"}: the parameter of this distribution, having only the
    slot name "Parameter of a discrete distribution" }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers}
    \item{\code{d}}{fixed to \code{NULL}}
    \item{\code{p}}{Object of class \code{"function"}: cumulative distribution function}
    \item{\code{q}}{Object of class \code{"function"}: quantile function}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"UnivarMixingDistribution"}
class \code{"UnivarDistribution"} by class \code{"UnivarMixingDistribution"},
class \code{"Distribution"} by class \code{"UnivariateDistribution"}.
}
\section{Methods}{
  \describe{
    \item{show}{\code{signature(object = "CompoundDistribution")} prints the object}
    \item{SummandsDistr}{\code{signature(object = "CompoundDistribution")} returns the corresponding slot}
    \item{NumbOfSummandsDistr}{\code{signature(object = "CompoundDistribution")} returns the corresponding slot}
  }
}
\section{setAs relations}{
  There is a coerce method to coerce objects of class \code{"CompoundDistribution"} to
  class \code{UnivarLebDecDistribution}; this is done by a simple call to \code{simplifyD}.
}


\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}
  }

\seealso{
\code{\link{Parameter-class}}, 
\code{\link{UnivariateDistribution-class}}, 
\code{\link{LatticeDistribution-class}}, 
\code{\link{AbscontDistribution-class}}, 
\code{\link{simplifyD}}, 
\code{\link{flat.mix}}
}
\examples{
CP <- CompoundDistribution(Pois(),Norm())
CP
p(CP)(0.3)          
plot(CP)
}
\keyword{distribution}
\concept{lattice distribution}
\concept{support of a distribution}
\concept{S4 distribution class}
\concept{generating function}
\concept{compound distribution}

\name{m-methods}
\docType{methods}
\alias{m-methods}
\alias{m<--methods}
\alias{m}
\alias{m<-}
\alias{m,HyperParameter-method}
\alias{m,Hyper-method}
\alias{m<-,HyperParameter-method}
\alias{m<-,Hyper-method}
\title{ Methods for Function m in Package `distr' }
\description{m-methods}
\section{Methods}{\describe{
\item{m}{\code{signature(object = "HyperParameter")}: returns the slot m of the parameter of the distribution }
\item{m<-}{\code{signature(object = "HyperParameter")}: modifies the slot m of the parameter of the distribution }
\item{m}{\code{signature(object = "Hyper")}: returns the slot m of the parameter of the distribution }
\item{m<-}{\code{signature(object = "Hyper")}: modifies the slot m of the parameter of the distribution }
}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{Hypergeometric distribution}
\concept{S4 parameter class}
\concept{accessor function}
\concept{replacement function}

\name{name-methods}
\docType{methods}
\alias{name-methods}
\alias{name<--methods}
\alias{name}
\alias{name<-}
\alias{name,Parameter-method}
\alias{name<-,Parameter-method}
\alias{name,rSpace-method}
\alias{name<-,rSpace-method}

\title{ Methods for Function name in Package `distr' }
\description{name-methods}
\section{Methods}{\describe{
\item{name}{\code{signature(object = "Parameter")}: returns the slot name of the parameter }
\item{name<-}{\code{signature(object = "Parameter")}: modifies the slot name of the parameter }
\item{name}{\code{signature(object = "rSpace")}: returns the slot name of the space }
\item{name<-}{\code{signature(object = "rSpace")}: modifies the slot name of the space }
}}
\keyword{methods}
\keyword{distribution}
\concept{S4 parameter class}
\concept{S4 space class}
\concept{accessor function}
\concept{replacement function}

\name{Binom-class} 
\docType{class}
\alias{Binom-class}
\alias{Binom}
\alias{initialize,Binom-method}

\title{Class "Binom" }
\description{The binomial distribution with \code{size} \eqn{= n}, by default 
  \eqn{=1}, and
  \code{prob} \eqn{= p}, by default \eqn{=0.5}, has density
  \deqn{p(x) = {n \choose x} {p}^{x} {(1-p)}^{n-x}}{
    p(x) = choose(n,x) p^x (1-p)^(n-x)}
  for \eqn{x = 0, \ldots, n}.

  C.f.\code{\link[stats:Binomial]{rbinom}}
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Binom(prob, size)}.
This object is a binomial distribution. 
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Naturals"}: The space of the 
     image of this distribution has got dimension 1 and the 
     name "Natural Space". }
    \item{\code{param}}{Object of class \code{"BinomParameter"}: the parameter 
          of this distribution (\code{prob}, \code{size}), declared at its 
          instantiation }
    \item{\code{r}}{Object of class \code{"function"}: generates random 
          numbers (calls function \code{rbinom}) }
    \item{\code{d}}{Object of class \code{"function"}: density function (calls 
          function \code{dbinom}) }
    \item{\code{p}}{Object of class \code{"function"}: cumulative function 
          (calls function \code{pbinom}) }
    \item{\code{q}}{Object of class \code{"function"}: inverse of the 
           cumulative function (calls function \code{qbinom}).
    The quantile is defined as the smallest value x such that F(x) >= p, where 
            F is the cumulative function. }
    \item{\code{support}}{Object of class \code{"numeric"}: a (sorted) 
            vector containing the support of the discrete density function}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"DiscreteDistribution"}, directly.\cr
Class \code{"UnivariateDistribution"}, by class \code{"DiscreteDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"DiscreteDistribution"}.
}
\section{Methods}{
  \describe{
    \item{+}{\code{signature(e1 = "Binom", e2 = "Binom")}: For two binomial 
             distributions with equal probabilities the exact convolution 
             formula is implemented thereby improving the general numerical 
             accuracy.}
    \item{initialize}{\code{signature(.Object = "Binom")}: initialize method }
    \item{prob}{\code{signature(object = "Binom")}: returns the slot \code{prob} 
             of the parameter of the distribution }
    \item{prob<-}{\code{signature(object = "Binom")}: modifies the slot 
             \code{prob} of the parameter of the distribution }
    \item{size}{\code{signature(object = "Binom")}: returns the slot \code{size} 
             of the parameter of the distribution }
    \item{size<-}{\code{signature(object = "Binom")}: modifies the slot 
             \code{size} of the parameter of the distribution }
  }
}


\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }


\seealso{
\code{\link{BinomParameter-class}}
\code{\link{DiscreteDistribution-class}}
\code{\link{Naturals-class}}
\code{\link[stats:Binomial]{rbinom}}
}
\examples{
B <- Binom(prob=0.5,size=1) # B is a binomial distribution with prob=0.5 and size=1.
r(B)(1) # # one random number generated from this distribution, e.g. 1
d(B)(1) # Density of this distribution is  0.5 for x=1.
p(B)(0.4) # Probability that x<0.4 is 0.5.
q(B)(.1) # x=0 is the smallest value x such that p(B)(x)>=0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
size(B) # size of this distribution is 1.
size(B) <- 2 # size of this distribution is now 2.
C <- Binom(prob = 0.5, size = 1) # C is a binomial distribution with prob=0.5 and size=1.
D <- Binom(prob = 0.6, size = 1) # D is a binomial distribution with prob=0.6 and size=1.
E <- B + C # E is a binomial distribution with prob=0.5 and size=3.
F <- B + D # F is an object of class LatticeDistribution.
G <- B + as(D,"DiscreteDistribution") ## DiscreteDistribution
}
\keyword{distribution}
\concept{discrete distribution}
\concept{lattice distribution}
\concept{Binomial family}
\concept{Binomial distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{Weibull-class} 
\docType{class}
\alias{Weibull-class}
\alias{Weibull}
\alias{initialize,Weibull-method}

\title{Class "Weibull"}
\description{  The Weibull distribution with \code{shape} parameter \eqn{a}, by default \eqn{=1}, and
  \code{scale} parameter \eqn{\sigma}{b} has density given by, by default \eqn{=1},
  \deqn{d(x) = (a/\sigma) {(x/\sigma)}^{a-1} \exp (-{(x/\sigma)}^{a})}{%
        d(x) = (a/b) (x/b)^(a-1) exp(- (x/b)^a)}
  for \eqn{x > 0}.

C.f. \code{\link[stats:Weibull]{rweibull}}
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Weibull(shape, scale)}.
This object is a Weibull distribution.
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}: The space of the image of this distribution has got dimension 1
    and the name "Real Space". }
    \item{\code{param}}{Object of class \code{"WeibullParameter"}: the parameter of this distribution (shape and scale),
     declared at its instantiation }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers (calls function \code{rweibull})}
    \item{\code{d}}{Object of class \code{"function"}: density function (calls function \code{dweibull})}
    \item{\code{p}}{Object of class \code{"function"}: cumulative function (calls function \code{pweibull})}
    \item{\code{q}}{Object of class \code{"function"}: inverse of the cumulative function (calls function \code{qweibull})}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"AbscontDistribution"}, directly.\cr
Class \code{"UnivariateDistribution"}, by class \code{"AbscontDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"AbscontDistribution"}.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Weibull")}: initialize method }
    \item{scale}{\code{signature(object = "Weibull")}: returns the slot \code{scale} of the parameter of the distribution }
    \item{scale<-}{\code{signature(object = "Weibull")}: modifies the slot \code{scale} of the parameter of the distribution }
    \item{shape}{\code{signature(object = "Weibull")}: returns the slot \code{shape} of the parameter of the distribution }
    \item{shape<-}{\code{signature(object = "Weibull")}: modifies the slot \code{shape} of the parameter of the distribution }
    \item{*}{\code{signature(e1 = "Weibull", e2 = "numeric")}:
    For the Weibull distribution we use its closedness under positive scaling transformations.}
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}

\note{ The density is \eqn{d(x)=0} for \eqn{x < 0}.\cr
The cumulative is
  \eqn{p(x) = 1 - \exp(-{(x/\sigma)}^a)}{p(x) = 1 - exp(- (x/b)^a)},\cr
  the mean is \eqn{E(X) = \sigma \Gamma(1 + 1/a)}{E(X) = b Gamma(1 + 1/a)},\cr
  and the \eqn{Var(X) = \sigma^2(\Gamma(1 + 2/a)-(\Gamma(1 + 1/a))^2)}{%
           Var(X) = b^2 * (gamma(1 + 2/a) - (gamma(1 + 1/a))^2)}.

}

\seealso{
\code{\link{WeibullParameter-class}}
\code{\link{AbscontDistribution-class}}
\code{\link{Reals-class}}
\code{\link[stats:Weibull]{rweibull}}





}
\examples{
W <- Weibull(shape=1,scale=1) # W is a Weibull distribution with shape=1 and scale=1.
r(W)(1) # one random number generated from this distribution, e.g. 0.5204105
d(W)(1) # Density of this distribution is 0.3678794 for x=1.
p(W)(1) # Probability that x<1 is 0.6321206.
q(W)(.1) # Probability that x<0.1053605 is 0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
shape(W) # shape of this distribution is 1.
shape(W) <- 2 # shape of this distribution is now 2.
}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{Weibull distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{print-methods}
\docType{methods}
\alias{print-methods}
\alias{show-methods}
\alias{print,UnivariateDistribution-method}
\alias{show,UnivariateDistribution-method}
\alias{show,LatticeDistribution-method}
\alias{show,UnivarMixingDistribution-method}
\alias{show,UnivarLebDecDistribution-method}
\title{ Methods for Functions print/show in Package `distr' }
\description{print/show-methods}

\section{Methods}{\describe{

\item{print}{\code{signature(x = "UnivariateDistribution")}: returns the class of the object and its parameters }
\item{show}{\code{signature(x = "UnivariateDistribution")}: returns the class of the object and its parameters }

}}
\keyword{print}
\keyword{methods}
\keyword{distribution}

\name{decomposePM-methods}
\docType{methods}
\alias{decomposePM-methods}
\alias{decomposePM}
\alias{decomposePM,AbscontDistribution-method}
\alias{decomposePM,DiscreteDistribution-method}
\alias{decomposePM,UnivarLebDecDistribution-method}
\title{ Methods for function decomposePM in Package `distr' }
\description{decomposePM-methods}

\usage{
decomposePM(object)
}
\arguments{
  \item{object}{Abscont-/Discrete-/UnivarLebDec-Distribution object}
}
\value{the positive and negative part of the distribution together with
corresponding weights as a list.}

\details{
There are particular return types for the following classes
\describe{
\item{"AbscontDistribution"}{a list with components \code{"neg"} and 
\code{"pos"} for the respective negative and positive part; each of these
parts in its turn is a list with components \code{D} for the distribution
(in this case of class \code{"AbscontDistribution"} again) and \code{w} 
for the weight of the respective part; if the weight of
the negative part is 0, the corresponding distribution is set to 
\code{-abs(Norm())}, and respectively, if the weight of the positive part 
is 0, the corresponding distribution is set to 
\code{abs(Norm())}.}
\item{"DiscreteDistribution"}{a list with components \code{"neg"}, 
\code{"pos"} and \code{"0"} for the respective negative, positive and zero 
part; each of these parts in its turn is a list with components \code{D} for 
the distribution (in this case of class \code{"DiscreteDistribution"} again)
and \code{w} for the weight of the respective part; while the distribution of
the zero part is always \code{Dirac(0)}, if the weight of
the negative part is 0, the corresponding distribution is set to 
\code{Dirac(-1)}, and respectively, if the weight of the positive part 
is 0, the corresponding distribution is set to 
\code{Dirac(1)}.}
\item{"UnivarLebDecDistribution"}{a list with components \code{"neg"}, 
\code{"pos"} and \code{"0"} for the respective negative, positive and zero 
part; each of these parts in its turn is a list with components \code{D} for 
the distribution (in case of components \code{"neg"}, 
\code{"pos"} of class \code{"UnivarLebDecDistribution"} again, while 
the distribution of the zero part is always \code{Dirac(0)})
and \code{w} for the weight of the respective part; it is build up by calling
\code{decomposePM} for \code{acPart(object)} and \code{discretePart(object)}
separately, hence if weights of some parts are zero the corresponding
procedure mentionned for these methods applies.}
}

Method \code{decomposePM} is used by our multiplication, division and exponentiation
(\code{"*"}, \code{"/"} \code{"^"}) - methods.
}
\seealso{\code{\link{AbscontDistribution-class}}, 
\code{\link{DiscreteDistribution-class}},
\code{\link{UnivarLebDecDistribution-class}},
\code{\link{operators-methods}}}
\examples{
decomposePM(Norm())
decomposePM(Binom(2,0.3)-Binom(5,.4))
decomposePM(UnivarLebDecDistribution(Norm(),Binom(2,0.3)-Binom(5,.4), 
            acWeight = 0.3))
}
\keyword{distribution}
\keyword{methods}
\concept{truncation}

\name{n-methods}
\docType{methods}
\alias{n-methods}
\alias{n<--methods}
\alias{n}
\alias{n<-}
\alias{n,HyperParameter-method}
\alias{n,Hyper-method}
\alias{n<-,HyperParameter-method}
\alias{n<-,Hyper-method}
\title{ Methods for Function n in Package `distr' }
\description{n-methods}

\section{Methods}{\describe{

\item{n}{\code{signature(object = "HyperParameter")}: returns the slot \code{n} of the parameter of the distribution }
\item{n<-}{\code{signature(object = "HyperParameter")}: modifies the slot \code{n} of the parameter of the distribution }

\item{n}{\code{signature(object = "Hyper")}: returns the slot \code{n} of the parameter of the distribution }
\item{n<-}{\code{signature(object = "Hyper")}: modifies the slot \code{n} of the parameter of the distribution }
}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{Hypergeometric distribution}
\concept{S4 parameter class}
\concept{accessor function}
\concept{replacement function}

\name{dim-methods}
\docType{methods}
\alias{dim-methods}
\alias{dim}
\alias{dim,UnivariateDistribution-method}

\title{ Methods for Function dim in Package `distr' }
\description{dim-methods}
\section{Methods}{\describe{
    \item{dim}{\code{signature(object = "UnivariateDistribution")}: returns the dimension of the distribution }
}}
\seealso{
\code{\link{UnivariateDistribution-class}}
}
\keyword{methods}
\concept{dimension}
\concept{space}
\concept{accessor function}

\name{simplifyD-methods}
\docType{methods}
\alias{simplifyD-methods}
\alias{simplifyD}
\alias{simplifyD,AbscontDistribution-method}
\alias{simplifyD,DiscreteDistribution-method}
\alias{simplifyD,UnivarLebDecDistribution-method}
\alias{simplifyD,UnivarMixingDistribution-method}
\title{Methods for function simplifyD in Package `distr' }
\description{simplifyD-methods}
\usage{
simplifyD(object)
}

\arguments{
  \item{object}{distribution object}
}
\value{the corresponding, possibly simplified distribution}
\section{Methods}{\describe{
\item{simplifyD}{\code{signature(object = "AbscontDistribution")}: 
returns \code{object} unchanged}
\item{simplifyD}{\code{signature(object = "DiscreteDistribution")}: 
returns \code{object} unchanged}
\item{simplifyD}{\code{signature(object = "UnivarLebDecDistribution")}: 
checks whether \code{acWeight} or \code{discreteWeight} is approximately
(i.e.; up to \code{getdistrOption("TruncQuantile")}) zero and if so,
accordingly returns \code{discretePart(object)} or \code{acPart(object)},
respectively.}
\item{simplifyD}{\code{signature(object = "UnivarMixingDistribution")}: 
returns the flattened version of \code{object} (using \code{\link{flat.mix}}).
before doing so, it checks whether any component carries weight  approximately
(i.e.; up to \code{getdistrOption("TruncQuantile")}) one (in slot \code{mixCoeff}) 
and if so, returns this component; else, if not all weights are below
\code{getdistrOption("TruncQuantile")}), it filters out those components
with weight less than \code{getdistrOption("TruncQuantile")}).
}}}
\details{generating functions \code{\link{UnivarMixingDistribution}}
\code{\link{Minimum}}, \code{Maximum}, \code{\link{Truncate}}, and
\code{\link{Huberize}} have an argument  \code{withSimplify} which
decides whether the respective result is filtered by/piped through a 
call to \code{simplifyD}.
By default this argument is set to the distr-option 
\code{getdistrOption("simplifyD"} (for the inspection and modification of such
global options see \code{\link{distroptions}}). Depending on whether or not this 
option is \code{TRUE}, also arithmetic operations \code{"+"}, \code{"*"}, \code{"/"},
\code{"^"} and group \code{Math} give results filtered by/piped through a call 
to \code{simplifyD}.}

\seealso{\code{\link{Huberize}}, \code{\link{Minimum}}}
\examples{
set.seed(123)
Mix1 <- UnivarMixingDistribution(Norm(),Binom(2,.3),
  UnivarLebDecDistribution(acPart = Chisq(df = 2), discretePart = Nbinom(3,.09),
                           acWeight = 0.3),
  Norm()-Chisq(df=3), mixCoeff=c(0,0,0.2,0.8), withSimplify = FALSE)
Mix2 <- UnivarMixingDistribution(Norm(),Mix1, DExp(2),
        mixCoeff = c(0,0.2,0.8), withSimplify = FALSE)
Mix2        
simplifyD(Mix2)
}
\keyword{distribution}
\keyword{methods}
\concept{truncation}

\name{EuclideanSpace-class}
\docType{class}
\alias{EuclideanSpace}
\alias{EuclideanSpace-class}
\alias{initialize,EuclideanSpace-method}

\title{Class "EuclideanSpace" }
\description{ The distribution-classes contain a slot where the sample space is stored. One typical sample space is the
Euclidean Space in dimension k. }
\section{Objects from the Class}{
Objects could theoretically be created by calls of the form \code{new("EuclideanSpace", dimension, name)}.
Usually an object of this class is not needed on its own. \code{EuclideanSpace} is the mother-class of the class 
\code{Reals}, which is generated automatically when a univariate absolutly continuous distribution is instantiated. 
}
\section{Slots}{
  \describe{
    \item{\code{dimension}}{Object of class \code{"numeric"}: the dimension of the space, by default = 1 }
    \item{\code{name}}{Object of class \code{"character"}: the name of the space, by default = "Euclidean Space"}
  }
}
\section{Extends}{
Class \code{"rSpace"}, directly.
}

\usage{
EuclideanSpace(dimension = 1)
}
\arguments{
  \item{dimension}{ positive integer: dimension of 
  the Euclidean space (default \code{=1})}
}

\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "EuclideanSpace")}: initialize method }
    \item{liesIn}{\code{signature(object = "EuclideanSpace", x = "numeric")}: Does a particular vector lie in this space or
    not?
 }
    \item{dimension}{\code{signature(object = "EuclideanSpace")}: returns the dimension of the space }
    \item{dimension<-}{\code{signature(object = "EuclideanSpace")}: modifies the dimension of the space }
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }


\seealso{
\code{\link{rSpace-class}}
\code{\link{Reals-class}}
\code{\link{Distribution-class}}
\code{\link{liesIn-methods}}
}

\examples{
E <- EuclideanSpace(dimension = 2) 
dimension(E) # The dimension of this space is 2.
dimension(E) <- 3 # The dimension of this space is now 3.
liesIn(E,c(0,0,0)) # TRUE
liesIn(E,c(0,0)) # FALSE
}


\concept{space}
\concept{set}
\concept{S4 space class}
\keyword{distribution}

\name{NormParameter-class}
\docType{class}
\alias{NormParameter-class}
\alias{initialize,NormParameter-method}
\title{Class "NormParameter"}
\description{ The parameter of a normal distribution, used by Norm-class}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("NormParameter", sd, mean)}.
  Usually an object of this class is not needed on its own. It is the
  mother-class of the class UniNormParameter, which is generated
  automatically when such a distribution is instantiated.  
}
\section{Slots}{
  \describe{
    \item{\code{sd}}{Object of class \code{"numeric"}: the sd of a normal distribution }
    \item{\code{mean}}{Object of class \code{"numeric"}: the mean of a normal distribution }
    \item{\code{name}}{Object of class \code{"character"}: a name / comment for the parameters }
  }
}
\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "NormParameter")}:
      initialize method}
    \item{mean}{\code{signature(object = "NormParameter")}:
      returns the slot mean of the parameter of the distribution} 
    \item{mean<-}{\code{signature(object = "NormParameter")}:
      modifies the slot mean of the parameter of the distribution} 
    \item{sd}{\code{signature(object = "NormParameter")}:
      returns the slot sd of the parameter of the distribution}
    \item{sd<-}{\code{signature(object = "NormParameter")}:
      modifies the slot sd of the parameter of the distribution}
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}

\seealso{
\code{\link{Norm-class}}
\code{\link{Parameter-class}}
}
\examples{
W <- new("NormParameter", mean = 0, sd = 1)
sd(W) # sd of this distribution is 1.
sd(W) <- 2 # sd of this distribution is now 2.
}
\keyword{distribution}
\concept{parameter}
\concept{Gaussian distribution}
\concept{Normal distribution}
\concept{S4 parameter class}
\concept{location scale family}

\name{DistrList-class}
\docType{class}
\alias{DistrList-class}
\alias{coerce,Distribution,DistrList-method}
\alias{show,DistrList-method}

\title{List of distributions}
\description{Create a list of distributions}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("DistrList", ...)}.
  More frequently they are created via the generating function 
  \code{DistrList}. 
}
\section{Slots}{
  \describe{
    \item{\code{.Data}}{Object of class \code{"list"}. 
      A list of distributions.}
  }
}
\section{Extends}{
Class \code{"list"}, from data part.\cr
Class \code{"vector"}, by class \code{"list"}.
}
\section{Methods}{
  \describe{
    \item{show}{\code{signature(object = "DistrList")}}
    \item{plot}{\code{signature(object = "DistrList")}}
    \item{coerce}{\code{signature(from = "Distribution", to = "DistrList")}:
      create a \code{"DistrList"} object from a \code{"Distribution"} object}
  }
}
%\references{}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de}}
%\note{}
\seealso{\code{\link{DistrList}}, \code{\link[distr]{Distribution-class}}}
\examples{
(DL <- new("DistrList", list(Norm(), Exp())))
plot(DL)
as(Norm(), "DistrList")
}
\keyword{distribution}
\keyword{list}
\concept{multivariate distribution}
\concept{S4 distribution class}

\name{Gammad-class} 
\docType{class}
\alias{Gammad-class}
\alias{Gammad}
\alias{initialize,Gammad-method}

\title{Class "Gammad"}
\description{   The Gammad distribution with parameters \code{shape} \eqn{=\alpha}{= a},
  by default \code{= 1}, and \code{scale} \eqn{=\sigma}{= s}, by default \code{= 1}, has 
  density
  \deqn{
    d(x)= \frac{1}{{\sigma}^{\alpha}\Gamma(\alpha)} {x}^{\alpha-1} e^{-x/\sigma}%
  }{d(x)= 1/(s^a Gamma(a)) x^(a-1) e^-(x/s)}
  for \eqn{x > 0}, \eqn{\alpha > 0}{a > 0} and \eqn{\sigma > 0}{s > 0}.
  The mean and variance are
  \eqn{E(X) = \alpha\sigma}{E(X) = a*s} and
  \eqn{Var(X) = \alpha\sigma^2}{Var(X) = a*s^2}. C.f. \code{\link[stats:GammaDist]{rgamma}} 
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Gammad(scale, shape)}.
This object is a gamma distribution.
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}: The space of the image of this distribution has got dimension 1
    and the name "Real Space". }
    \item{\code{param}}{Object of class \code{"GammaParameter"}: the parameter of this distribution (scale and shape), declared at its instantiation }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers (calls function rgamma)}
    \item{\code{d}}{Object of class \code{"function"}: density function (calls function dgamma)}
    \item{\code{p}}{Object of class \code{"function"}: cumulative function (calls function pgamma)}
    \item{\code{q}}{Object of class \code{"function"}: inverse of the cumulative function (calls function qgamma)}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"ExpOrGammaOrChisq"}, directly.\cr
Class \code{"AbscontDistribution"}, by class \code{"ExpOrGammaOrChisq"}.\cr
Class \code{"UnivariateDistribution"}, by class \code{"AbscontDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"UnivariateDistribution"}.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Gammad")}: initialize method }
    \item{scale}{\code{signature(object = "Gammad")}: returns the slot \code{scale} of the parameter of the distribution }
    \item{scale<-}{\code{signature(object = "Gammad")}: modifies the slot \code{scale} of the parameter of the distribution }
    \item{shape}{\code{signature(object = "Gammad")}: returns the slot \code{shape} of the parameter of the distribution }
    \item{shape<-}{\code{signature(object = "Gammad")}: modifies the slot \code{shape} of the parameter of the distribution }
    \item{+}{\code{signature(e1 = "Gammad", e2 = "Gammad")}: 
    For the Gamma distribution we use its closedness under convolutions.}
    \item{*}{\code{signature(e1 = "Gammad", e2 = "numeric")}: 
    For the Gamma distribution we use its closedness under positive scaling transformations.}
  }
}

\author{Thomas Stabla \email{statho3@web.de},\cr 
        Florian Camphausen \email{fcampi@gmx.de},\cr
        Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
        Matthias Kohl \email{Matthias.Kohl@stamats.de}}


\seealso{
\code{\link{GammaParameter-class}}
\code{\link{AbscontDistribution-class}}
\code{\link{Reals-class}}
\code{\link[stats:GammaDist]{rgamma}}





}
\examples{
G <- Gammad(scale=1,shape=1) # G is a gamma distribution with scale=1 and shape=1.
r(G)(1) # one random number generated from this distribution, e.g. 0.1304441
d(G)(1) # Density of this distribution is 0.3678794 for x=1.
p(G)(1) # Probability that x<1 is 0.6321206.
q(G)(.1) # Probability that x<0.1053605 is 0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
scale(G) # scale of this distribution is 1.
scale(G) <- 2 # scale of this distribution is now 2.
}
\keyword{distribution}
\concept{Gamma distribution}
\concept{absolutely continuous distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{UnivariateDistribution-class}
\docType{class}
\alias{UnivariateDistribution-class}
\alias{initialize,UnivariateDistribution-method}
\title{Class "UnivariateDistribution"}
\description{
  The UnivariateDistribution-class is the mother-class of the classes
  AbscontDistribution and DiscreteDistribution.
}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("UnivariateDistribution")}.
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}:
      the space of the image of this distribution which has dimension 1
      and the name "Real Space"} 
    \item{\code{param}}{Object of class \code{"Parameter"}:
      the parameter of this distribution}
    \item{\code{r}}{Object of class \code{"function"}:
      generates random numbers}
    \item{\code{d}}{Object of class \code{"function"}:
      density function}
    \item{\code{p}}{Object of class \code{"function"}:
      cumulative distribution function}
    \item{\code{q}}{Object of class \code{"function"}:
      quantile function}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"Distribution"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "UnivariateDistribution")}:

      initialize method }
    \item{dim}{\code{signature(x = "UnivariateDistribution")}:

      returns the dimension of the support of the distribution}
    \item{-}{\code{signature(e1 = "UnivariateDistribution")}:

      application of `-' to this univariate distribution}
    \item{*}{\code{signature(e1 = "UnivariateDistribution", e2 = "numeric")}:

      multiplication of this univariate distribution by an object of
      class `numeric'} 
    \item{/}{\code{signature(e1 = "UnivariateDistribution", e2 = "numeric")}:

      division of this univariate distribution by an object of class `numeric'}
    \item{+}{\code{signature(e1 = "UnivariateDistribution", e2 = "numeric")}:

      addition of this univariate distribution to an object of class
      `numeric'} 
    \item{-}{\code{signature(e1 = "UnivariateDistribution", e2 = "numeric")}:

      subtraction of an object of class `numeric' from this univariate
      distribution} 
    \item{*}{\code{signature(e1 = "numeric", e2 = "UnivariateDistribution")}:

      multiplication of this univariate distribution by an object of
      class `numeric'} 
    \item{+}{\code{signature(e1 = "numeric", e2 = "UnivariateDistribution")}:

      addition of this univariate distribution to an object of class
      `numeric'} 
    \item{-}{\code{signature(e1 = "numeric", e2 = "UnivariateDistribution")}:

      subtraction of this univariate distribution from an object of
      class `numeric'}
    \item{+}{\code{signature(e1 = "UnivariateDistribution", e2 = "UnivariateDistribution")}:

      Convolution of two univariate distributions. The slots p, d and q
      are approximated by grids.}
    \item{-}{\code{signature(e1 = "UnivariateDistribution", e2 = "UnivariateDistribution")}:
      
      Convolution of two univariate distributions. The slots p, d and q
      are approximated by grids.}
    \item{simplifyr}{\code{signature(object = "UnivariateDistribution")}:

      simplifies the r-method of a distribution, see there for further information}
    \item{print}{\code{signature(object = "UnivariateDistribution")}:

      returns the class of the object and its parameters}
    \item{show}{\code{signature(object = "UnivariateDistribution")}:
      as print}
      }
  }



\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
}
\seealso{
\code{\link{Parameter-class}}
\code{\link{Distribution-class}}
\code{\link{AbscontDistribution-class}} \cr
\code{\link{DiscreteDistribution-class}} 
\code{\link{Reals-class}} 
\code{\link{RtoDPQ}}
\code{\link{simplifyr-methods}}
}
\keyword{distribution}
\concept{univariate distribution}
\concept{S4 distribution class}

\name{LatticeDistribution-class}
\docType{class}
\alias{AffLinLatticeDistribution-class}
\alias{LatticeDistribution-class}
\alias{lattice}
\alias{lattice-method}
\alias{lattice,LatticeDistribution-method}
\alias{initialize,LatticeDistribution-method}
\alias{initialize,AffLinLatticeDistribution-method}
\alias{sqrt,LatticeDistribution-method}
\alias{coerce,LatticeDistribution,DiscreteDistribution-method}
\alias{coerce,AffLinLatticeDistribution,AffLinDiscreteDistribution-method}


\title{Class "LatticeDistribution"}
\description{The \code{LatticeDistribution}-class is the mother-class of the 
classes \code{Binom}, \code{Dirac}, \code{Geom}, \code{Hyper}, \code{Nbinom} and 
\code{Poisson}. It formalizes a distribution on a regular affine
linear lattice.}
\section{Objects from the Class}{
The usual way to generate objects of class \code{LatticeDistribution} is to call 
the generating function \code{\link{LatticeDistribution}}. \cr
Somewhat more flexible, but also proner to inconsistencies is a call to 
\code{new("LatticeDistribution")}, where you may explicitly specify random 
number generator, (counting) density, cumulative distribution and quantile 
functions. For conveniance, in this call to \code{new("LatticeDistribution")}, 
an additional possibility is to only specify the random number generator. The 
function \code{RtoDPQ.d} then approximates the three remaining slots \code{d}, 
\code{p} and \code{q} by random sampling. 
}


\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}: the space of the image 
    of this distribution which has dimension 1 and the name "Real Space" }
    \item{\code{param}}{Object of class \code{"Parameter"}: the parameter of 
    this distribution, having only the slot name 
    "Parameter of a discrete distribution" }
    \item{\code{r}}{Object of class \code{"function"}: 
                     generates random numbers}
    \item{\code{d}}{Object of class \code{"function"}: 
                     (counting) density/probability function}
    \item{\code{p}}{Object of class \code{"function"}: 
                     cumulative distribution function}
    \item{\code{q}}{Object of class \code{"function"}: 
                     quantile function}
    \item{\code{support}}{Object of class \code{"numeric"}: a (sorted) vector 
                           containing the support of the discrete
    density function}
    \item{\code{lattice}}{Object of class \code{"Lattice"}: the lattice 
                           generating the support.}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"UnivariateDistribution"}, directly.\cr
Class \code{"Distribution"}, by class \code{"UnivariateDistribution"}.
}
\section{Methods}{
  \describe{
    \item{\code{initialize}}{\code{signature(.Object = "LatticeDistribution")}: 
                              initialize method }
    \item{-}{\code{signature(e1 = "LatticeDistribution")}: 
                   application of `-' to this lattice distribution}
    \item{*}{\code{signature(e1 = "LatticeDistribution", e2 = "numeric")}: 
                   multiplication of this lattice distribution
    by an object of class `numeric'}
    \item{/}{\code{signature(e1 = "LatticeDistribution", e2 = "numeric")}: 
    division of this lattice distribution by an object of class `numeric'}
    \item{+}{\code{signature(e1 = "LatticeDistribution", e2 = "numeric")}: 
    addition of this lattice distribution to an object of class `numeric'}
    \item{-}{\code{signature(e1 = "LatticeDistribution", e2 = "numeric")}: 
    subtraction of an object of class `numeric' from this lattice 
    distribution }
    \item{*}{\code{signature(e1 = "numeric", e2 = "LatticeDistribution")}: 
    multiplication of this lattice distribution by an object of class `numeric'}
    \item{+}{\code{signature(e1 = "numeric", e2 = "LatticeDistribution")}: 
    addition of this lattice distribution to an object of class `numeric'}
    \item{-}{\code{signature(e1 = "numeric", e2 = "LatticeDistribution")}: 
    subtraction of this lattice distribution from an object of class `numeric'}
    \item{+}{\code{signature(e1 = "LatticeDistribution", 
    e2 = "LatticeDistribution")}: Convolution of two lattice distributions. 
    Slots p, d and q are approximated by grids.}
    \item{-}{\code{signature(e1 = "LatticeDistribution", 
    e2 = "LatticeDistribution")}: Convolution of two lattice
    distributions. The slots p, d and q are approximated by grids.}
    \item{\code{sqrt}}{\code{signature(x = "LatticeDistribution")}:  exact 
         image distribution of \code{sqrt(x)}.}
    \item{\code{lattice}}{accessor method to the corresponding slot.}
    \item{\code{coerce}}{\code{signature(from = "LatticeDistribution",
    to = "DiscreteDistribution")}: coerces an object from 
    \code{"LatticeDistribution"} to \code{"DiscreteDistribution"}
    thereby cancelling out support points with probability 0.}
  }
}

\section{Internal subclass "AffLinLatticeDistribution"}{
To enhance accuracy of several functionals on distributions,
  mainly from package \pkg{distrEx}, there is an internally used 
  (but exported) subclass \code{"AffLinLatticeDistribution"} which has extra slots 
  \code{a}, \code{b} (both of class \code{"numeric"}),  and \code{X0} 
  (of class \code{"LatticeDistribution"}), to capture the fact 
  that the object has the same distribution as \code{a * X0 + b}. This is 
  the class of the return value of methods 
  \describe{
    \item{-}{\code{signature(e1 = "LatticeDistribution")}}
    \item{*}{\code{signature(e1 = "LatticeDistribution", e2 = "numeric")}}
    \item{/}{\code{signature(e1 = "LatticeDistribution", e2 = "numeric")}}
    \item{+}{\code{signature(e1 = "LatticeDistribution", e2 = "numeric")}}
    \item{-}{\code{signature(e1 = "LatticeDistribution", e2 = "numeric")}}
    \item{*}{\code{signature(e1 = "numeric", e2 = "LatticeDistribution")}}
    \item{+}{\code{signature(e1 = "numeric", e2 = "LatticeDistribution")}}
    \item{-}{\code{signature(e1 = "numeric", e2 = "LatticeDistribution")}}
    \item{-}{\code{signature(e1 = "AffLinLatticeDistribution")}}
    \item{*}{\code{signature(e1 = "AffLinLatticeDistribution", e2 = "numeric")}}
    \item{/}{\code{signature(e1 = "AffLinLatticeDistribution", e2 = "numeric")}}
    \item{+}{\code{signature(e1 = "AffLinLatticeDistribution", e2 = "numeric")}}
    \item{-}{\code{signature(e1 = "AffLinLatticeDistribution", e2 = "numeric")}}
    \item{*}{\code{signature(e1 = "numeric", e2 = "AffLinLatticeDistribution")}}
    \item{+}{\code{signature(e1 = "numeric", e2 = "AffLinLatticeDistribution")}}
    \item{-}{\code{signature(e1 = "numeric", e2 = "AffLinLatticeDistribution")}}
  }
 There is also an explicit \code{coerce}-method from class
 \code{"AffLinLatticeDistribution"} to class \code{"AffLinDiscreteDistribution"}
 which cancels out support points with probability 0.
}

\author{Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}

\note{ Working with a computer, we use a finite interval as support which 
carries at least mass \code{1-getdistrOption("TruncQuantile")}. }

\seealso{
\code{\link{LatticeDistribution}}
\code{\link{Parameter-class}}
\code{\link{Lattice-class}}
\code{\link{UnivariateDistribution-class}}
\code{\link{DiscreteDistribution-class}}
\code{\link{Binom-class}}
\code{\link{Dirac-class}}
\code{\link{Geom-class}}
\code{\link{Hyper-class}}
\code{\link{Nbinom-class}}
\code{\link{Pois-class}}
\code{\link{AbscontDistribution-class}}
\code{\link{Reals-class}}
\code{\link{RtoDPQ.d}}
}
\examples{
B <- Binom(prob = 0.1,size = 10) # B is a Binomial distribution w/ prob=0.1 and size=10.
P <- Pois(lambda = 1) # P is a Poisson distribution with lambda = 1.
D1 <- B+1 # a new Lattice distributions with exact slots d, p, q
D2 <- D1*3 # a new Lattice distributions with exact slots d, p, q
D3 <- B+P # a new Lattice distributions with approximated slots d, p, q
D4 <- D1+P # a new Lattice distributions with approximated slots d, p, q
support(D4) # the (approximated) support of this distribution is 1, 2, ..., 21
r(D4)(1) # one random number generated from this distribution, e.g. 4
d(D4)(1) # The (approximated) density for x=1 is 0.1282716.
p(D4)(1) # The (approximated) probability that x<=1 is 0.1282716.
q(D4)(.5) # The (approximated) 50 percent quantile is 3.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
}
\keyword{distribution}
\concept{discrete distribution}
\concept{lattice distribution}
\concept{lattice of a distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{simplifyr-methods}
\docType{methods}
\alias{simplifyr-methods}
\alias{simplifyr}
\alias{simplifyr,UnivariateDistribution-method}
\title{ Methods for Function simplifyr in Package `distr'}
\description{simplifyr-methods}

\section{Methods}{\describe{

\item{simplifyr}{\code{signature(.Object = "UnivariateDistribution")}: After several transformations of a given distribution it may take quite 
a long time  to generate random numbers from the resulting distribution. simplifyr generates a certain number, 
by default \eqn{10^5}, of random numbers once. This pool of random numbers forms the basis for further uses of the 
r-method. That is, random numbers are generated by sampling with replacement out of this pool. }
}}
\note{ If you want to generate many random numbers, you should use simplifyr with a big size to be sure,
that your numbers are really random. }

\seealso{
\code{\link{Distribution-class}}
}

\examples{
F <- ( Norm() + Binom() + Pois() + Exp() ) * 2 - 10
## IGNORE_RDIFF_BEGIN
system.time(r(F)(10^6))
## IGNORE_RDIFF_END
simplifyr(F, size = 10^6)
## IGNORE_RDIFF_BEGIN
system.time(r(F)(10^6))
## IGNORE_RDIFF_END
}
\keyword{math}
\keyword{distribution}
\keyword{arith}
\concept{random sample}
\concept{image distribution}
\concept{utility}

\name{rate-methods}
\docType{methods}
\alias{rate-methods}
\alias{rate<--methods}
\alias{rate}
\alias{rate<-}
\alias{rate,ExpParameter-method}
\alias{rate,Exp-method}
\alias{rate,DExp-method}
\alias{rate<-,ExpParameter-method}
\alias{rate<-,Exp-method}
\alias{rate<-,DExp-method}
\title{ Methods for Function rate in Package `distr' }
\description{rate-methods}

\section{Methods}{\describe{

\item{rate}{\code{signature(object = "ExpParameter")}: returns the slot \code{rate} of the parameter of the distribution }
\item{rate<-}{\code{signature(object = "ExpParameter")}: modifies the slot \code{rate} of the parameter of the distribution }

\item{rate}{\code{signature(object = "Exp")}: returns the slot \code{rate} of the parameter of the distribution }
\item{rate<-}{\code{signature(object = "Exp")}: modifies the slot \code{rate} of the parameter of the distribution }

\item{rate}{\code{signature(object = "DExp")}: returns the slot \code{rate} of the parameter of the distribution }
\item{rate<-}{\code{signature(object = "DExp")}: modifies the slot \code{rate} of the parameter of the distribution }

}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{Double Exponential distribution}
\concept{Exponential distribution}
\concept{Laplace distribution}
\concept{S4 parameter class}
\concept{accessor function}
\concept{replacement function}

\name{scale-methods}
\docType{methods}
\alias{scale-methods}
\alias{scale<--methods}
\alias{scale}
\alias{scale<-}
\alias{scale,GammaParameter-method}
\alias{scale,Gammad-method}
\alias{scale,Chisq-method}
\alias{scale,LogisParameter-method}
\alias{scale,Logis-method}
\alias{scale,WeibullParameter-method}
\alias{scale,Weibull-method}
\alias{scale,CauchyParameter-method}
\alias{scale,Cauchy-method}
\alias{scale<-,GammaParameter-method}
\alias{scale<-,Gammad-method}
\alias{scale<-,LogisParameter-method}
\alias{scale<-,Logis-method}
\alias{scale<-,WeibullParameter-method}
\alias{scale<-,Weibull-method}
\alias{scale<-,CauchyParameter-method}
\alias{scale<-,Cauchy-method}
\title{ Methods for Function scale in Package `distr' }
\description{scale-methods}

\section{Methods}{\describe{

\item{scale}{\code{signature(object = "GammaParameter")}: returns the slot \code{scale} of the parameter of the distribution }
\item{scale<-}{\code{signature(object = "GammaParameter")}: modifies the slot \code{scale} of the parameter of the distribution }

\item{scale}{\code{signature(object = "Gammad")}: returns the slot \code{scale} of the parameter of the distribution }
\item{scale<-}{\code{signature(object = "Gammad")}: modifies the slot \code{scale} of the parameter of the distribution }

\item{scale}{\code{signature(object = "LogisParameter")}: returns the slot \code{scale} of the parameter of the distribution }
\item{scale<-}{\code{signature(object = "LogisParameter")}: modifies the slot \code{scale} of the parameter of the distribution }

\item{scale}{\code{signature(object = "Logis")}: returns the slot \code{scale} of the parameter of the distribution }
\item{scale<-}{\code{signature(object = "Logis")}: modifies the slot \code{scale} of the parameter of the distribution }

\item{scale}{\code{signature(object = "WeibullParameter")}: returns the slot \code{scale} of the parameter of the distribution }
\item{scale<-}{\code{signature(object = "WeibullParameter")}: modifies the slot \code{scale} of the parameter of the distribution }

\item{scale}{\code{signature(object = "Weibull")}: returns the slot \code{scale} of the parameter of the distribution }
\item{scale<-}{\code{signature(object = "Weibull")}: modifies the slot \code{scale} of the parameter of the distribution }

\item{scale}{\code{signature(object = "CauchyParameter")}: returns the slot \code{scale} of the parameter of the distribution }
\item{scale<-}{\code{signature(object = "CauchyParameter")}: modifies the slot \code{scale} of the parameter of the distribution }

\item{scale}{\code{signature(object = "Cauchy")}: returns the slot \code{scale} of the parameter of the distribution }
\item{scale<-}{\code{signature(object = "Cauchy")}: modifies the slot \code{scale} of the parameter of the distribution }

\item{scale}{\code{signature(object = "Chisq")}:  if \code{ncp(object)} is 0, a Chi-squared distribution is also a Gamma distribution;
     in this case, \code{scale} returns 2 else an error;}

}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{Gamma distribution}
\concept{Logistic distribution}
\concept{Weibull distribution}
\concept{Cauchy distribution}
\concept{Chi Square distribution}
\concept{S4 parameter class}
\concept{location scale family}
\concept{scale parameter}
\concept{accessor function}
\concept{replacement function}

\name{meanlog-methods}
\docType{methods}
\alias{meanlog-methods}
\alias{meanlog<--methods}
\alias{meanlog}
\alias{meanlog<-}
\alias{meanlog,LnormParameter-method}
\alias{meanlog,Lnorm-method}
\alias{meanlog<-,LnormParameter-method}
\alias{meanlog<-,Lnorm-method}
\title{ Methods for Function meanlog in Package `distr' }
\description{meanlog-methods}
\section{Methods}{\describe{
\item{meanlog}{\code{signature(object = "LnormParameter")}: returns the slot meanlog of the parameter of the distribution }
\item{meanlog<-}{\code{signature(object = "LnormParameter")}: modifies the slot meanlog of the parameter of the distribution }
\item{meanlog}{\code{signature(object = "Lnorm")}: returns the slot meanlog of the parameter of the distribution }
\item{meanlog<-}{\code{signature(object = "Lnorm")}: modifies the slot meanlog of the parameter of the distribution }
}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{Log-Normal distribution}
\concept{S4 parameter class}
\concept{accessor function}
\concept{replacement function}

\name{location-methods}
\docType{methods}
\alias{location-methods}
\alias{location<--methods}
\alias{location}
\alias{location<-}
\alias{location,LogisParameter-method}
\alias{location,Logis-method}
\alias{location,CauchyParameter-method}
\alias{location,Cauchy-method}
\alias{location,DiracParameter-method}
\alias{location,Dirac-method}
\alias{location<-,LogisParameter-method}
\alias{location<-,Logis-method}
\alias{location<-,CauchyParameter-method}
\alias{location<-,Cauchy-method}
\alias{location<-,DiracParameter-method}
\alias{location<-,Dirac-method}
\title{ Methods for Function location in Package `distr' }
\description{location-methods}

\section{Methods}{\describe{

\item{location}{\code{signature(object = "LogisParameter")}: returns the slot \code{location} of the parameter of the distribution }
\item{location<-}{\code{signature(object = "LogisParameter")}: modifies the slot \code{location} of the parameter of the
distribution }

\item{location}{\code{signature(object = "Logis")}: returns the slot \code{location} of the parameter of the distribution }
\item{location<-}{\code{signature(object = "Logis")}: modifies the slot \code{location} of the parameter of the distribution }

\item{location}{\code{signature(object = "CauchyParameter")}: returns the slot \code{location} of the parameter of the distribution }
\item{location<-}{\code{signature(object = "CauchyParameter")}: modifies the slot \code{location} of the parameter of the
distribution }

\item{location}{\code{signature(object = "Cauchy")}: returns the slot \code{location} of the parameter of the distribution }
\item{location<-}{\code{signature(object = "Cauchy")}: modifies the slot \code{location} of the parameter of the distribution }

\item{location}{\code{signature(object = "DiracParameter")}: returns the slot \code{location} of the parameter of the distribution }
\item{location<-}{\code{signature(object = "DiracParameter")}: modifies the slot \code{location} of the parameter of the
distribution}

\item{location}{\code{signature(object = "Dirac")}: returns the slot \code{location} of the parameter of the distribution }
\item{location<-}{\code{signature(object = "Dirac")}: modifies the slot \code{location} of the parameter of the distribution }
}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{Dirac distribution}
\concept{Cauchy distribution}
\concept{S4 parameter class}
\concept{accessor function}
\concept{replacement function}
\concept{location scale family}
\concept{location parameter}

\name{p.l-methods}
\docType{methods}
\alias{p.l-methods}
\alias{p.l}
\alias{p.l,AbscontDistribution-method}
\alias{p.l,DiscreteDistribution-method}
\alias{p.l,UnivarLebDecDistribution-method}
\alias{p.l,UnivarMixingDistribution-method}
\title{Methods for Function p.l in Package `distr' }
\description{p-methods}

\section{Methods}{
   return the left continuous cumulative distribution function,
   i.e.; \eqn{p.l(t) = P(object < t)}{p.l(t) = P(object < t)} 
\describe{
\item{p.l}{\code{signature(object = "AbscontDistribution")}} 
\item{p.l}{\code{signature(object = "DiscreteDistribution")}} 
\item{p.l}{\code{signature(object = "UnivarLebDecDistribution")}}
\item{p.l}{\code{signature(object = "UnivarMixingDistribution")}}
}}
\seealso{
\code{\link{DiscreteDistribution-class}}
\code{\link{UnivarLebDecDistribution-class}}
}
\keyword{distribution}
\keyword{methods}
\concept{cumulative distribution function}
\concept{CDF}
\concept{left-continuous}

\name{d-methods}
\docType{methods}
\alias{d-methods}
\alias{d}
\alias{d,Distribution-method}
\title{ Methods for Function d in Package `distr' }
\description{d-methods}

\section{Methods}{\describe{
\item{d}{\code{signature(object = "Distribution")}: returns the density function }
}}
\seealso{
\code{\link{Distribution-class}}
}
\keyword{distribution}
\keyword{methods}
\concept{density}
\concept{probability function}
\concept{accessor function}

\name{PosDefSymmMatrix-class}
\docType{class}
\alias{PosDefSymmMatrix-class}
\alias{PosSemDefSymmMatrix-class}

\title{Positive-[Semi-]definite, symmetric matrices}
\description{The class of positive-[semi-]definite, symmetric matrices.}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("PosSemDefSymmMatrix", ...)}
  resp. \code{new("PosDefSymmMatrix", ...)}.
  More frequently they are created via the generating functions
  \code{PosSemDefSymmMatrix} resp. \code{PosDefSymmMatrix}. 
}
\section{Slots}{
  \describe{
    \item{\code{.Data}}{Object of class \code{"matrix"}. 
      A numeric matrix with finite entries.}
  }
}
\section{Extends}{
[Class \code{"PosSemDefSymmMatrix"}, directly]
Class \code{"matrix"}, from data part.\cr
Class \code{"structure"}, by class \code{"matrix"}.\cr
Class \code{"array"}, by class \code{"matrix"}.\cr
Class \code{"vector"}, by class "matrix", with explicit coerce.\cr
Class \code{"vector"}, by class "matrix", with explicit coerce.
}
%\section{Methods}{}
%\references{}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de},
Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}
%\note{}
\seealso{\code{\link{PosDefSymmMatrix}}, \code{\link[methods:StructureClasses]{matrix-class}}}
\examples{
new("PosDefSymmMatrix", diag(2))
}
\concept{positive-definite matrix}
\concept{positive-semi-definite matrix}
\concept{symmetric matrix}
\keyword{classes}
\keyword{array}

\name{Chisq-class}
\docType{class}
\alias{Chisq-class}
\alias{Chisq}
\alias{initialize,Chisq-method}
\title{Class "Chisq"}
\description{
  The chi-squared distribution with \code{df}\eqn{= n} degrees of
  freedom has density
  \deqn{f_n(x) = \frac{1}{{2}^{n/2} \Gamma (n/2)} {x}^{n/2-1} {e}^{-x/2}}{%
    f_n(x) = 1 / (2^(n/2) Gamma(n/2))  x^(n/2-1) e^(-x/2)}
  for \eqn{x > 0}.  The mean and variance are \eqn{n} and \eqn{2n}.

  The non-central chi-squared distribution with \code{df}\eqn{= n}
  degrees of freedom and non-centrality parameter \code{ncp}
  \eqn{= \lambda} has density
  \deqn{
    f(x) = e^{-\lambda / 2}
      \sum_{r=0}^\infty \frac{(\lambda/2)^r}{r!}\, f_{n + 2r}(x)}{%
    f(x) = exp(-lambda/2) SUM_{r=0}^infty ((lambda/2)^r / r!) dchisq(x, df + 2r)
  }
  for \eqn{x \ge 0}.  For integer \eqn{n}, this is the distribution of
  the sum of squares of \eqn{n} normals each with variance one,
  \eqn{\lambda} being the sum of squares of the normal means.
  
  C.f. \code{\link[stats:Chisquare]{rchisq}}
}
 

\section{Objects from the Class}{
  Objects can be created by calls of the form \code{Chisq(df, ncp)}.
  This object is a chi-squared distribution.
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}:
      The space of the image of this distribution has got dimension 1 and the name "Real Space".}
    \item{\code{param}}{Object of class \code{"ChisqParameter"}:
      the parameter of this distribution (df and ncp), declared at its instantiation}
    \item{\code{r}}{Object of class \code{"function"}:
      generates random numbers (calls function rchisq)}
    \item{\code{d}}{Object of class \code{"function"}:
      density function (calls function dchisq)}
    \item{\code{p}}{Object of class \code{"function"}:
      cumulative function (calls function pchisq)}
    \item{\code{q}}{Object of class \code{"function"}:
      inverse of the cumulative function (calls function qchisq)}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"ExpOrGammaOrChisq"}, directly.\cr
Class \code{"AbscontDistribution"}, by class \code{"ExpOrGammaOrChisq"}.\cr
Class \code{"UnivariateDistribution"}, by class \code{"AbscontDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"UnivariateDistribution"}.
}

\section{Is-Relations}{
By means of \code{setIs}, R ``knows'' that a distribution object \code{obj} of class \code{"Chisq"} with non-centrality 0 also is
a Gamma distribution with parameters \code{shape = df(obj)/2, scale = 2}. 
}

\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Chisq")}: initialize method }
    \item{df}{\code{signature(object = "Chisq")}:
      returns the slot df of the parameter of the distribution }
    \item{df<-}{\code{signature(object = "Chisq")}:
      modifies the slot df of the parameter of the distribution }
    \item{ncp}{\code{signature(object = "Chisq")}:
      returns the slot ncp of the parameter of the distribution }
    \item{ncp<-}{\code{signature(object = "Chisq")}:
      modifies the slot ncp of the parameter of the distribution }
    \item{+}{\code{signature(e1 = "Chisq", e2 = "Chisq")}: 
    For the chi-squared distribution we use its closedness under convolutions.}
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}
\note{
Warning: The code for pchisq and qchisq is unreliable for values of ncp above approximately 290. 
}
\seealso{
\code{\link{ChisqParameter-class}}
\code{\link{AbscontDistribution-class}}
\code{\link{Reals-class}}
\code{\link[stats:Chisquare]{rchisq}}
}
\examples{
C <- Chisq(df = 1, ncp = 1) # C is a chi-squared distribution with df=1 and ncp=1.
r(C)(1) # one random number generated from this distribution, e.g. 0.2557184
d(C)(1) # Density of this distribution is 0.2264666 for x = 1.
p(C)(1) # Probability that x < 1 is 0.4772499.
q(C)(.1) # Probability that x < 0.04270125 is 0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
df(C) # df of this distribution is 1.
df(C) <- 2 # df of this distribution is now 2.
is(C, "Gammad") # no
C0 <- Chisq() # default: Chisq(df=1,ncp=0)
is(C0, "Gammad") # yes
as(C0,"Gammad")
}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{Chi square distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{Beta-class}
\docType{class}
\alias{Beta-class}
\alias{Beta}
\alias{initialize,Beta-method}
\title{Class "Beta"}
\description{
  The Beta distribution with parameters \code{shape1} \eqn{= a} and
  \code{shape2} \eqn{= b} has density
  \deqn{f(x)=\frac{\Gamma(a+b)}{\Gamma(a)\Gamma(b)}{x}^{a-1} {(1-x)}^{b-1}%
  }{Gamma(a+b)/(Gamma(a)Gamma(b))x^(a-1)(1-x)^(b-1)}
  for \eqn{a > 0}, \eqn{b > 0} and \eqn{0 \le x \le 1}{0 <= x <= 1}
  where the boundary values at \eqn{x=0} or \eqn{x=1} are defined as
  by continuity (as limits).
}
\note{ 
  The non-central Beta distribution is defined (Johnson et al, 1995,
  pp. 502) as the distribution of \eqn{X/(X+Y)} where
  \eqn{X \sim \chi^2_{2a}(\lambda)}{X ~ chi^2_2a(lambda)} and
  \eqn{Y \sim \chi^2_{2b}}{Y ~ chi^2_2b}.
  C.f. \code{\link[stats:Beta]{rbeta}} }

\section{Ad hoc methods}{
 For R Version \code{<2.3.0} ad hoc methods are provided for slots \code{q}, \code{r} if \code{ncp!=0};
      for R Version \code{>=2.3.0} the methods from package \pkg{stats} are used.
      }

\section{Objects from the Class}{
  Objects can be created by calls of the form \code{Beta(shape1, shape2)}.
  This object is a beta distribution.
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}:
      The space of the image of this distribution has got dimension 1 and the name "Real Space". }
    \item{\code{param}}{Object of class \code{"BetaParameter"}:
      the parameter of this distribution (shape1 and shape2), declared at its instantiation }
    \item{\code{r}}{Object of class \code{"function"}:
      generates random numbers (calls function rbeta)}
    \item{\code{d}}{Object of class \code{"function"}:
      density function (calls function dbeta)}
    \item{\code{p}}{Object of class \code{"function"}:
      cumulative function (calls function pbeta)}
    \item{\code{q}}{Object of class \code{"function"}:
      inverse of the cumulative function (calls function qbeta)}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
  Class \code{"AbscontDistribution"}, directly. \cr
  Class \code{"UnivariateDistribution"}, by class \code{"AbscontDistribution"}. \cr
  Class \code{"Distribution"}, by class \code{"AbscontDistribution"}.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Beta")}:
      initialize method}
    \item{shape1}{\code{signature(object = "Beta")}:
      returns the slot shape1 of the parameter of the distribution}
    \item{shape1<-}{\code{signature(object = "Beta")}:
      modifies the slot shape1 of the parameter of the distribution}
    \item{shape2}{\code{signature(object = "Beta")}:
      returns the slot shape2 of the parameter of the distribution}
    \item{shape2<-}{\code{signature(object = "Beta")}:
      modifies the slot shape2 of the parameter of the distribution}
    \item{\code{-}}{\code{signature(e1 = "numeric", e2 = "Beta")} if \code{ncp(e2)==0} and \code{e1 == 1},
     an exact (central) \code{Beta(shape1 = shape2(e2), shape2 = shape1(e2))} is returned, else
     the default method is used; exact}
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
}
\seealso{
  \code{\link{BetaParameter-class}}
  \code{\link{AbscontDistribution-class}}
  \code{\link{Reals-class}}
  \code{\link[stats:Beta]{rbeta}}
}
\examples{
B <- Beta(shape1 = 1, shape2 = 1)
# B is a beta distribution with shape1 = 1 and shape2 = 1.
r(B)(1) # one random number generated from this distribution, e.g. 0.6979795
d(B)(1) # Density of this distribution is 1 for x=1.
p(B)(1) # Probability that x < 1 is 1.
q(B)(.1) # Probability that x < 0.1 is 0.1.
shape1(B) # shape1 of this distribution is 1.
shape1(B) <- 2 # shape1 of this distribution is now 2.
Bn <- Beta(shape1 = 1, shape2 = 3, ncp = 5) 
# Bn is a beta distribution with shape1 = 1 and shape2 = 3 and ncp = 5.
B0 <- Bn; ncp(B0) <- 0; 
# B0 is just the same beta distribution as Bn but with ncp = 0
q(B0)(0.1) ## 
q(Bn)(0.1) ## => from R 2.3.0 on ncp no longer ignored...
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{Beta distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{Naturals-class}
\docType{class}
\alias{Naturals}
\alias{Naturals-class}
\alias{initialize,Naturals-method}

\title{Class "Naturals" }
\description{ The distribution-classes contain a slot where the sample space is stored. Typically, discrete random
variables take naturals as values. }
\section{Objects from the Class}{
Objects could theoretically be created by calls of the form \code{new("Naturals", dimension, name)}.
Usually an object of this class is not needed on its own. It is generated automatically when a univariate discrete
distribution is instantiated. 
}
\usage{Naturals()}

\section{Slots}{
  \describe{
    \item{\code{dimension}}{Object of class \code{"character"}: the dimension of the space, by default = 1 }
    \item{\code{name}}{Object of class \code{"character"}: the name of the space, by default = "Natural Space"}
  }
}
\section{Extends}{
Class \code{"Reals"}, directly.\cr
Class \code{"EuclideanSpace"}, by class \code{"Reals"}.\cr
Class \code{"rSpace"}, by class \code{"Reals"}.
}

\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Naturals")}: initialize method }
    \item{liesIn}{\code{signature(object = "Naturals", x = "numeric")}: Does a particular vector only contain naturals? }
}
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}


\seealso{
\code{\link{Reals-class}}
\code{\link{DiscreteDistribution-class}}
}

\examples{
N <- Naturals()
liesIn(N,1) # TRUE
liesIn(N,c(0,1)) # FALSE
liesIn(N,0.1) # FALSE
}

\concept{naturals}
\concept{space}
\concept{set}
\concept{S4 space class}
\keyword{distribution}

\name{DiscreteDistribution}
\alias{DiscreteDistribution}

\title{Generating function "DiscreteDistribution"}
\description{Generates an object of class  \code{"DiscreteDistribution"}}

\usage{
  DiscreteDistribution(supp, prob, .withArith=FALSE, .withSim=FALSE, 
                       .lowerExact = TRUE, .logExact = FALSE,
             .DistrCollapse = getdistrOption("DistrCollapse"),
             .DistrCollapse.Unique.Warn = 
                  getdistrOption("DistrCollapse.Unique.Warn"),
             .DistrResolution = getdistrOption("DistrResolution"),
             Symmetry = NoSymmetry())
}
\arguments{
  \item{supp}{numeric vector which forms the support
    of the discrete distribution. }
  \item{prob}{vector of probability weights for the
    elements of \code{supp}.}
  \item{.withArith}{normally not set by the user, but if determining the entries \code{supp}, \code{prob}
                    distributional arithmetics was involved, you may set this to \code{TRUE}.}
  \item{.withSim}{normally not set by the user, but if determining the entries \code{supp}, \code{prob}
                  simulations were involved, you may set this to \code{TRUE}.}
  \item{.lowerExact}{normally not set by the user: whether the \code{lower.tail=FALSE}
                     part is calculated exactly, avoing a ``\code{1-.}''.}
  \item{.logExact}{normally not set by the user: whether in determining slots \code{d,p,q},
         we make particular use of a logarithmic representation to enhance accuracy.}
\item{.DistrCollapse}{controls whether in generating a new discrete 
     distribution, support points closer together than \code{.DistrResolution} are
     collapsed.}
\item{.DistrCollapse.Unique.Warn}{controls whether there is a warning
      whenever collapsing occurs or when two points are collapsed by a call to
      \code{unique()} (default behaviour if \code{.DistrCollapse} is \code{FALSE})}
\item{.DistrResolution}{minimal spacing between two mass points in a discrete 
                        distribution}
\item{Symmetry}{you may help \R in calculations if you tell it whether
   the distribution is non-symmetric (default) or symmetric with respect
   to a center; in this case use \code{Symmetry=SphericalSymmetry(center)}.}
}
\details{
  If \code{prob} is missing, all elements in \code{supp}
  are equally weighted.\cr
  
  Typical usages are
  \preformatted{
    DiscreteDistribution(supp, prob)
    DiscreteDistribution(supp)
  }
  
}
\value{Object of class \code{"DiscreteDistribution"}}


\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }

\note{ Working with a computer, we use a finite interval as support which 
       carries at least mass \code{1-getdistrOption("TruncQuantile")}. \cr
       
       Also, we require that support points have distance at least       
       \code{.DistrResoltion}, if this condition fails,
       upon a suggestion by Jacob van Etten, \email{jacobvanetten@yahoo.com},
       we use the global option \code{.DistrCollapse} to
       decide whether we use collapsing or not. If we do so, we collapse support 
       points if they are too close to each other, taking
       the (left most) median among them as new support point which accumulates
       all the mass of the collapsed points. 
       With \code{.DistrCollapse==FALSE}, we at least collapse
       points according to the result of \code{unique()}, and if after this
       collapsing, the minimal distance is less than \code{.DistrResoltion},
       we throw an error. By \code{.DistrCollapse.Unique.Warn},
       we control, whether we throw a warning upon collapsing or not.       
       }

\seealso{
\code{\link{DiscreteDistribution-class}}
\code{\link{AbscontDistribution-class}}
\code{\link{RtoDPQ.d}}
}
\examples{
# Dirac-measure at 0
D1 <- DiscreteDistribution(supp = 0)
D1
# simple discrete distribution
D2 <- DiscreteDistribution(supp = c(1:5), prob = c(0.1, 0.2, 0.3, 0.2, 0.2))
D2

plot(D2)
}
\keyword{distribution}
\concept{discrete distribution}
\concept{generating function}

\name{CauchyParameter-class}        
\docType{class}
\alias{CauchyParameter-class}
\alias{initialize,CauchyParameter-method}

\title{Class "CauchyParameter"}
\description{ The parameter of a Cauchy distribution, used by Cauchy-class}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("CauchyParameter", location, scale)}.
Usually an object of this class is not needed on its own, it is generated automatically when an object of the class Cauchy is instantiated. 
}
\section{Slots}{
  \describe{
    \item{\code{location}:}{Object of class \code{"numeric"}: the location of a Cauchy distribution }
    \item{\code{scale}}{Object of class \code{"numeric"}: the scale of a Cauchy distribution }
    \item{\code{name}}{Object of class \code{"character"}: a name / comment for the parameters }
  }
}
\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "CauchyParameter")}: initialize method }
    \item{scale}{\code{signature(object = "CauchyParameter")}: returns the slot \code{scale} of the parameter of the distribution}
    \item{scale<-}{\code{signature(object = "CauchyParameter")}: modifies the slot \code{scale} of the parameter of the
    distribution }
    \item{location}{\code{signature(object = "CauchyParameter")}: returns the slot \code{location} of the parameter of the
    distribution }
    \item{location<-}{\code{signature(object = "CauchyParameter")}: modifies the slot \code{location} of the parameter of the
    distribution }
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }


\seealso{
\code{\link{Cauchy-class}}
\code{\link{Parameter-class}}
}

\examples{
W <- new("CauchyParameter",location=1,scale=1)
location(W) # location of this distribution is 1.
location(W) <- 2 # location of this distribution is now 2.
}
\keyword{distribution}
\concept{parameter}
\concept{location scale family}
\concept{Cauchy distribution}
\concept{S4 parameter class}

\name{sd-methods}
\docType{methods}
\alias{sd-methods}
\alias{sd<--methods}
\alias{sd}
\alias{sd<-}
\alias{sd,NormParameter-method}
\alias{sd,Norm-method}
\alias{sd<-,NormParameter-method}
\alias{sd<-,Norm-method}
\title{ Methods for Function sd in Package `distr' }
\description{sd-methods}
\section{Methods}{\describe{
\item{sd}{\code{signature(x = "Any")}: see \link[stats]{sd}}    
\item{sd}{\code{signature(x = "NormParameter")}: returns the slot sd of the parameter of the distribution }    
\item{sd<-}{\code{signature(object = "NormParameter")}: modifies the slot sd of the parameter of the distribution }
\item{sd}{\code{signature(x = "Norm")}: returns the slot sd of the parameter of the distribution }
\item{sd<-}{\code{signature(object = "Norm")}: modifies the slot sd of the parameter of the distribution }
}}
\seealso{\link[stats]{sd}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{Normal distribution}
\concept{Gaussian distribution}
\concept{S4 parameter class}
\concept{location scale family}
\concept{scale parameter}
\concept{accessor function}
\concept{replacement function}

\name{Dirac-class}
\docType{class}
\alias{Dirac-class}
\alias{Dirac}
\alias{initialize,Dirac-method}
\alias{log,Dirac-method}
\alias{Math,Dirac-method}

\title{Class "Dirac"  }
\description{ The Dirac distribution with location \eqn{l}, by default \eqn{=0}, has density
\eqn{d(x) = 1} for \eqn{x = l}, \eqn{0} else.
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Dirac(location)}.
This object is a \code{Dirac} distribution. 
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Naturals"}: The space of the image of this
     distribution has got dimension 1 and the name "Real Space". }
    \item{\code{param}}{Object of class \code{"DiracParameter"}: the parameter of this distribution (location), declared at its instantiation }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers }
    \item{\code{d}}{Object of class \code{"function"}: density function }
    \item{\code{p}}{Object of class \code{"function"}: cumulative function }
    \item{\code{q}}{Object of class \code{"function"}: inverse of the cumulative function }
    \item{\code{support}}{Object of class \code{"numeric"}: a (sorted) vector containing the support of the discrete density function}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"DiscreteDistribution"}, directly.\cr
Class \code{"UnivariateDistribution"}, by class \code{"DiscreteDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"DiscreteDistribution"}.
}
\section{Methods}{
  \describe{
    \item{-}{\code{signature(e1 = "Dirac", e2 = "Dirac")}}
    \item{+}{\code{signature(e1 = "Dirac", e2 = "Dirac")}} 
    \item{*}{\code{signature(e1 = "Dirac", e2 = "Dirac")}}
    \item{/}{\code{signature(e1 = "Dirac", e2 = "Dirac")}: 
    For the Dirac distribution these operations are trivial.}
    \item{initialize}{\code{signature(.Object = "Dirac")}: initialize method }
    \item{location}{\code{signature(object = "Dirac")}: returns the slot \code{location} of the parameter of the distribution }
    \item{location<-}{\code{signature(object = "Dirac")}: modifies the slot \code{location} of the parameter of the distribution }
    \item{log}{\code{signature(object = "Dirac")}: returns an object of class \code{"Dirac"} distribution with log-transformed
      \code{location} parameter. }
    \item{Math}{\code{signature(object = "Dirac")}: given a \code{"Math"} group generic \code{fun} an object of class 
      \code{"Dirac"} distribution with \code{fun}-transformed \code{location} parameter is returned. }
  }
 further arithmetic methods see \link{operators-methods}
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}

\seealso{
\code{\link{DiracParameter-class}}
\code{\link{DiscreteDistribution-class}}
\code{\link{Naturals-class}}
}
\examples{
D <- Dirac(location = 0) # D is a Dirac distribution with location=0.
r(D)(1)
# r(D)(1) generates a pseudo-random-number according to a Dirac
# distribution with location = 0,
# which of course will take 0 as value almost surely.
d(D)(0) # Density of this distribution is 1 for x = 0.
p(D)(1) # Probability that x < 1 is 1.
q(D)(.1) # q(D)(x) is always 0 (= location).
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
location(D) # location of this distribution is 0.
location(D) <- 2 # location of this distribution is now 2.
}
\keyword{distribution}
\concept{location family}
\concept{lattice distribution}
\concept{discrete distribution}
\concept{Dirac distribution}
\concept{S4 parameter class}

\name{Truncate-methods}
\docType{methods}
\alias{Truncate-methods}
\alias{Truncate}
\alias{Truncate,AbscontDistribution-method}
\alias{Truncate,DiscreteDistribution-method}
\alias{Truncate,LatticeDistribution-method}
\alias{Truncate,UnivarLebDecDistribution-method}
\title{ Methods for function Truncate in Package `distr' }
\description{Truncate-methods}
\usage{
Truncate(object, ...)
\S4method{Truncate}{AbscontDistribution}(object, lower = -Inf, upper = Inf)
\S4method{Truncate}{DiscreteDistribution}(object, lower= -Inf, upper = Inf)
\S4method{Truncate}{LatticeDistribution}(object, lower= -Inf, upper = Inf)
\S4method{Truncate}{UnivarLebDecDistribution}(object, lower = -Inf, upper = Inf, 
                    withSimplify = getdistrOption("simplifyD"))
}
\arguments{
  \item{object}{distribution object}
  \item{\dots}{not yet used; takes up \code{lower}, 
  \code{upper}, \code{withSimplify}.}
  \item{lower}{numeric; lower truncation point}
  \item{upper}{numeric; upper truncation point}
  \item{withSimplify}{logical; is result to be piped through a call to 
         \code{\link{simplifyD}}?}
}
\value{the corresponding distribution of the truncated random variable}
\section{Methods}{\describe{
\item{Truncate}{\code{signature(object = "AbscontDistribution")}: 
returns the distribution of \code{min(upper,max(X,lower))}
conditioned to \code{lower<=X<=upper}, if \code{X} is 
distributed according to \code{object};
if slot \code{.logExact} of argument \code{object} is \code{TRUE} 
and if either there is only one-sided truncation or both truncation
points lie on the same side of the median, we use this representation
to enhance the range of applicability, in particular, for slot \code{r},
we profit from Peter Dalgaard's clever log-tricks as indicated in 
\url{https://stat.ethz.ch/pipermail/r-help/2008-September/174321.html}. To this
end we use the internal functions
(i.e.; non exported to namespace) \code{\link{.trunc.up}} and
\code{\link{.trunc.low}} which provide functional slots \code{r,d,p,q} for 
one-sided truncation. In case of two sided truncation, we simply use one-sided
truncation successively --- first left and then right in case we are right of
the median, and the other way round else;
the result is again of class \code{"AbscontDistribution"};
}
\item{Truncate}{\code{signature(object = "DiscreteDistribution")}: 
returns the distribution of \code{min(upper,max(X,lower))}
conditioned to \code{lower<=X<=upper}, if \code{X} is 
distributed according to \code{object};
the result is again of class \code{"DiscreteDistribution"}}
\item{Truncate}{\code{signature(object = "LatticeDistribution")}: 
if length of the corresp. \code{lattice} is infinite  and 
slot \code{.logExact} of argument \code{object} is \code{TRUE}, 
we proceed similarly as in case of \code{AbscontDistribution}, also 
using internal functions \code{\link{.trunc.up}} and \code{\link{.trunc.low}}; 
else we use the corresponding \code{"DiscreteDistribution"} method;
the result is again of class \code{"LatticeDistribution"}}
\item{Truncate}{\code{signature(object = "UnivarLebDecDistribution")}: 
returns the distribution of \code{min(upper,max(X,lower))}
conditioned to \code{lower<=X<=upper}, if \code{X} is 
distributed according to \code{object};
the result is again of class \code{"UnivarLebDecDistribution"}}
}}
\seealso{\code{\link{Huberize}}, \code{\link{Minimum}}}
\examples{
plot(Truncate(Norm(),lower=-1,upper=2))
TN <- Truncate(Norm(),lower=15,upper=15.7) ### remarkably right!
plot(TN)
r(TN)(30)
TNG <- Truncate(Geom(prob=0.05),lower=325,upper=329) ### remarkably right!
plot(TNG)
}
\keyword{distribution}
\keyword{methods}
\concept{truncation}

\name{Hyper-class}
\docType{class}
\alias{Hyper-class}
\alias{Hyper}
\alias{initialize,Hyper-method}

\title{Class "Hyper" }
\description{ The hypergeometric distribution is used for sampling \emph{without} 
  replacement. The density of this distribution with parameters
  \code{m}, \code{n} and \code{k} (named \eqn{Np}, \eqn{N-Np}, and
  \eqn{n}, respectively in the reference below) is given by
  \deqn{
    p(x) = \left. {m \choose x}{n \choose k-x} \right/ {m+n \choose k}%
  }{p(x) =      choose(m, x) choose(n, k-x) / choose(m+n, k)}
  for \eqn{x = 0, \ldots, k}{x = 0, ..., k}.
  C.f. \code{\link[stats:Hypergeometric]{rhyper}} 
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Hyper(m, n, k)}.
This object is a hypergeometric distribution. 
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Naturals"}: The space of the image of this
     distribution has got dimension 1 and the name "Natural Space". }
    \item{\code{param}}{Object of class \code{"HyperParameter"}: the parameter of this distribution (\code{m}, \code{n}, \code{k}),
     declared at its instantiation }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers (calls function \code{rhyper}) }
    \item{\code{d}}{Object of class \code{"function"}: density function (calls function \code{dhyper}) }
    \item{\code{p}}{Object of class \code{"function"}: cumulative function (calls function \code{phyper}) }
    \item{\code{q}}{Object of class \code{"function"}: inverse of the cumulative function (calls function \code{qhyper}).
    The \eqn{\alpha}{alpha}-quantile is defined as the smallest value \eqn{x} such that 
     \eqn{p(x) \ge \alpha]}{p(x) >= alpha}, where \eqn{p} is the cumulative function. }
    \item{\code{support}:}{Object of class \code{"numeric"}: a (sorted) vector containing the support of the discrete
    density function}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"DiscreteDistribution"}, directly.\cr
Class \code{"UnivariateDistribution"}, by class \code{"DiscreteDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"DiscreteDistribution"}.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Hyper")}: initialize method }
    \item{m}{\code{signature(object = "Hyper")}: returns the slot \code{m} of the parameter of the distribution }
    \item{m<-}{\code{signature(object = "Hyper")}: modifies the slot \code{m} of the parameter of the distribution }
    \item{n}{\code{signature(object = "Hyper")}: returns the slot \code{n} of the parameter of the distribution }
    \item{n<-}{\code{signature(object = "Hyper")}: modifies the slot \code{n} of the parameter of the distribution }
    \item{k}{\code{signature(object = "Hyper")}: returns the slot \code{k} of the parameter of the distribution }
    \item{k<-}{\code{signature(object = "Hyper")}: modifies the slot \code{k} of the parameter of the distribution }
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }


\seealso{
\code{\link{HyperParameter-class}}
\code{\link{DiscreteDistribution-class}}
\code{\link{Naturals-class}}
\code{\link[stats:Hypergeometric]{rhyper}}
}
\examples{
H <- Hyper(m=3,n=3,k=3) # H is a hypergeometric distribution with m=3,n=3,k=3.
r(H)(1) # one random number generated from this distribution, e.g. 2
d(H)(1) # Density of this distribution is  0.45 for x=1.
p(H)(1) # Probability that x<1 is 0.5.
q(H)(.1) # x=1 is the smallest value x such that p(H)(x)>=0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
m(H) # m of this distribution is 3.
m(H) <- 2 # m of this distribution is now 2.
}
\keyword{distribution}
\concept{discrete distribution}
\concept{lattice distribution}
\concept{Hypergeometric distribution}
\concept{S4 parameter class}
\concept{generating function}

\name{NoSymmetry-class}
\docType{class}
\alias{NoSymmetry-class}

\title{Class for Non-symmetric Distributions}
\description{Class for non-symmetric distributions.}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("NoSymmetry")}.
  More frequently they are created via the generating function 
  \code{NoSymmetry}.
}
\section{Slots}{
  \describe{
    \item{\code{type}}{Object of class \code{"character"}:
      contains \dQuote{non-symmetric distribution} }
    \item{\code{SymmCenter}}{Object of class \code{"NULL"} }
  }
}
\section{Extends}{
Class \code{"DistributionSymmetry"}, directly.\cr
Class \code{"Symmetry"}, by class \code{"DistributionSymmetry"}.
}
%\section{Methods}{}
%\references{}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de}}
%\note{}
\seealso{\code{\link{NoSymmetry}}, \code{\link[distr]{Distribution-class}}}
\examples{
new("NoSymmetry")
}
\concept{symmetry}
\keyword{classes}
\keyword{distribution}

\name{shape1-methods}
\docType{methods}
\alias{shape1-methods}
\alias{shape1<--methods}
\alias{shape1}
\alias{shape1<-}
\alias{shape1,BetaParameter-method}
\alias{shape1,Beta-method}
\alias{shape1<-,BetaParameter-method}
\alias{shape1<-,Beta-method}
\title{ Methods for Function shape1 in Package `distr' }
\description{shape-methods}

\section{Methods}{\describe{

\item{shape1}{\code{signature(object = "BetaParameter")}: returns the slot \code{shape1} of the parameter of the distribution }
\item{shape1<-}{\code{signature(object = "BetaParameter")}: modifies the slot \code{shape1} of the parameter of the distribution }

\item{shape1}{\code{signature(object = "Beta")}: returns the slot \code{shape1} of the parameter of the distribution }
\item{shape1<-}{\code{signature(object = "Beta")}: modifies the slot \code{shape1} of the parameter of the distribution }
}}
\keyword{distribution}
\keyword{methods}
\concept{shape}
\concept{parameter}
\concept{Beta distribution}
\concept{accessor function}
\concept{replacement function}

\name{prob-methods} 
\docType{methods}
\alias{prob-methods}
\alias{prob<--methods}
\alias{prob}
\alias{prob<-}
\alias{prob,BinomParameter-method}
\alias{prob,Binom-method}
\alias{prob,NbinomParameter-method}
\alias{prob,Nbinom-method}
\alias{prob,GeomParameter-method}
\alias{prob,Geom-method}
\alias{prob,DiscreteDistribution-method}
\alias{prob,UnivarLebDecDistribution-method}
\alias{prob<-,BinomParameter-method}
\alias{prob<-,Binom-method}
\alias{prob<-,NbinomParameter-method}
\alias{prob<-,Nbinom-method}
\alias{prob<-,GeomParameter-method}
\alias{prob<-,Geom-method}
\alias{prob<-,DiscreteDistribution-method}
\title{ Methods for Function prob in Package `distr' }
\description{prob-methods}

\section{Methods}{\describe{

\item{prob}{\code{signature(object = "BinomParameter")}: returns the slot \code{prop} of the parameter of the distribution }
\item{prob<-}{\code{signature(object = "BinomParameter")}: modifies the slot \code{prob} of the parameter of the distribution }

\item{prob}{\code{signature(object = "Binom")}: returns the slot \code{prop} of the parameter of the distribution }
\item{prob<-}{\code{signature(object = "Binom")}: modifies the slot \code{prob} of the parameter of the distribution }

\item{prob}{\code{signature(object = "NbinomParameter")}: returns the slot \code{prop} of the parameter of the distribution }
\item{prob<-}{\code{signature(object = "NbinomParameter")}: modifies the slot \code{prob} of the parameter of the distribution }

\item{prob}{\code{signature(object = "Nbinom")}: returns the slot \code{prop} of the parameter of the distribution }
\item{prob<-}{\code{signature(object = "Nbinom")}: modifies the slot \code{prob} of the parameter of the distribution }

\item{prob}{\code{signature(object = "GeomParameter")}: returns the slot \code{prop} of the parameter of the distribution (deprecated from 1.9 on)}
\item{prob<-}{\code{signature(object = "GeomParameter")}: modifies the slot \code{prob} of the parameter of the distribution (deprecated from 1.9 on)}

\item{prob}{\code{signature(object = "Geom")}: returns the slot \code{prop} of the parameter of the distribution }
\item{prob<-}{\code{signature(object = "Geom")}: modifies the slot \code{prob} of the parameter of the distribution }

\item{prob}{\code{signature(object = "DiscreteDistribution")}: returns the 
(named) vector of probabilities for the support points of the distribution. }
\item{prob<-}{\code{signature(object = "DiscreteDistribution")}: 
               generates a new object of class \code{"DiscreteDistribution"} with
               the same support as \code{object} as well as the same 
               \code{.withSim}, \code{.withArith}, \code{.lowerExact}, 
               \code{.logExact} slots. }
\item{prob}{\code{signature(object = "UnivarLebDecDistribution")}: returns a
\eqn{2 \times n}{2 x n} matrix where n is the length of the support of
the discrete part of the distribution; the first row named \code{"cond"} 
gives the vector of  probabilities for the support points of the discrete part
of the distribution (i.e.; conditional on being in the discrete part), the second
row named \code{"abs"} is like the first one but multiplied with \code{discreteWeight}
of the distribution, hence gives the absolute probabilities of the support points;
the columns are named by the support values. }
}}
\keyword{distribution}
\concept{parameter}
\concept{Binomial distribution}
\concept{Negative Binomial distribution}
\concept{Geometric distribution}
\concept{S4 parameter class}
\concept{accessor function}
\concept{replacement function}

\name{support-methods}
\docType{methods}
\alias{support-methods}
\alias{support}
\alias{support,DiscreteDistribution-method}
\title{Methods for Function support in Package `distr'}
\description{support-methods}

\section{Methods}{\describe{

\item{support}{\code{signature(object =
    "DiscreteDistribution")}: returns the support}
}}
\keyword{distribution}
\keyword{methods}
\concept{discrete distribution}
\concept{support}
\concept{S4 distribution class}
\concept{accessor function}

\name{size-methods}
\docType{methods}
\alias{size-methods}
\alias{size<--methods}
\alias{size}
\alias{size<-}
\alias{size,BinomParameter-method}
\alias{size,Binom-method}
\alias{size,NbinomParameter-method}
\alias{size,Nbinom-method}
\alias{size,Geom-method}
\alias{size<-,BinomParameter-method}
\alias{size<-,Binom-method}
\alias{size<-,NbinomParameter-method}
\alias{size<-,Nbinom-method}
\alias{size<-,Geom-method}
\title{ Methods for Function size in Package `distr' }
\description{size-methods}

\section{Methods}{\describe{

\item{size}{\code{signature(object  = "BinomParameter")}: returns the slot \code{size} of the parameter of the distribution }
\item{size<-}{\code{signature(object = "BinomParameter")}: modifies the slot \code{size} of the parameter of the distribution }

\item{size}{\code{signature(object = "Binom")}: returns the slot \code{size} of the parameter of the distribution }
\item{size<-}{\code{signature(object = "Binom")}: modifies the slot \code{size} of the parameter of the distribution }

\item{size}{\code{signature(object = "NbinomParameter")}: returns the slot \code{size} of the parameter of the distribution }
\item{size<-}{\code{signature(object = "NbinomParameter")}: modifies the slot \code{size} of the parameter of the distribution }

\item{size}{\code{signature(object = "Nbinom")}: returns the slot \code{size} of the parameter of the distribution }
\item{size<-}{\code{signature(object = "Nbinom")}: modifies the slot \code{size} of the parameter of the distribution }

\item{size}{\code{signature(object = "Geom")}: returns the slot \code{size} of the parameter of the distribution }

}}
\keyword{distribution}
\concept{parameter}
\concept{Binomial distribution}
\concept{Negative Binomial distribution}
\concept{Geometric distribution}
\concept{S4 parameter class}
\concept{accessor function}
\concept{replacement function}

\name{UnifParameter-class} 
\docType{class}
\alias{UnifParameter-class}
\alias{initialize,UnifParameter-method}

\title{Class "UnifParameter"}
\description{ The parameter of a uniform distribution, used by Unif-class}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("UnifParameter", Max, Min)}.
Usually an object of this class is not needed on its own, it is generated automatically when an object of the class 
\code{Unif} is instantiated. 
}
\section{Slots}{
  \describe{
    \item{\code{Max}}{Object of class \code{"numeric"}: the Max of a uniform distribution }
    \item{\code{Min}}{Object of class \code{"numeric"}: the Min of a uniform distribution }
    \item{\code{name}}{Object of class \code{"character"}: a name / comment for the parameters }
  }
}
\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "UnifParameter")}: initialize method }
    \item{Min}{\code{signature(object = "UnifParameter")}: returns the slot \code{Min} of the parameter of the distribution }
    \item{Min<-}{\code{signature(object = "UnifParameter")}: modifies the slot \code{Min} of the parameter of the distribution }
    \item{Max}{\code{signature(object = "UnifParameter")}: returns the slot \code{Max} of the parameter of the distribution }
    \item{Max<-}{\code{signature(object = "UnifParameter")}: modifies the slot \code{Max} of the parameter of the distribution }
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}

\seealso{
\code{\link{Unif-class}}
\code{\link{Parameter-class}}
}

\examples{
W <- new("UnifParameter",Min=0,Max=1)
Max(W) # Max of this distribution is 1.
Max(W) <- 2 # Max of this distribution is now 2.
}
\keyword{distribution}
\concept{parameter}
\concept{Uniform distribution}
\concept{S4 parameter class}

\name{LatticeDistribution}
\alias{LatticeDistribution}

\title{Class "LatticeDistribution"}
\description{The \code{LatticeDistribution}-class is the mother-class of the
classes \code{Binom}, \code{Dirac}, \code{Geom}, \code{Hyper}, \code{Nbinom} and
\code{Poisson}. It formalizes a distribution on a regular affine
linear lattice.}
\section{Objects from the Class}{
The usual way to generate objects of class \code{LatticeDistribution} is to call
the generating function \code{LatticeDistribution()}  (see details). \cr
Somewhat more flexible, but also proner to inconsistencies is a call to
\code{new("LatticeDistribution")}, where you may explicitly specify random
number generator, (counting) density, cumulative distribution and quantile
functions. For conveniance, in this call to \code{new("LatticeDistribution")},
an additional possibility is to only specify the random number generator. The
function \code{RtoDPQ.d} then approximates the three remaining slots \code{d},
\code{p} and \code{q} by random sampling.
}

\usage{
  LatticeDistribution(lattice = NULL, supp = NULL, prob = NULL,
                       .withArith = FALSE, .withSim = FALSE,
                       DiscreteDistribution = NULL, check = TRUE,
                       Symmetry = NoSymmetry())
}
\arguments{
  \item{DiscreteDistribution}{an object of class \code{DiscreteDistribution}
  or \code{AffLinDiscreteDistribution} to be
  coerced to \code{LatticeDistribution} or \code{AffLinLatticeDistribution},
  respectively}
  \item{lattice}{lattice (of class \code{Lattice}) which determines the support
                 of the discrete distribution. }
  \item{supp}{ numeric vector which forms the support
    of the discrete distribution. }
  \item{prob}{ vector of probability weights for the
    elements of \code{supp}.}
  \item{.withArith}{normally not set by the user, but if determining the entries
                    \code{supp}, \code{prob} distributional arithmetics was
                    involved, you may set this to \code{TRUE}.}
  \item{.withSim}{normally not set by the user, but if determining the entries
                  \code{supp}, \code{prob} simulations were involved, you may
                  set this to \code{TRUE}.}
  \item{check}{ logical: if \code{TRUE}, \code{LatticeDistribution()} throws an
               error if argument \code{lattice} and other arguments are
               inconsistent or if there is no way to automatically generate
               a \code{lattice} argument. If \code{check == FALSE},
               \code{LatticeDistribution()}
               returns an object of \code{DiscreteDistribution}, ignoring
               argument \code{lattice}}
  \item{Symmetry}{you may help \R in calculations if you tell it whether
    the distribution is non-symmetric (default) or symmetric with respect
    to a center; in this case use \code{Symmetry=SphericalSymmetry(center)}.}
}
\details{
  Typical usages are
  \preformatted{
  LatticeDistribution(DiscreteDistribution)
  LatticeDistribution(lattice, DiscreteDistribution)
  LatticeDistribution(lattice, supp, prob, .withArith, .withSim, check = FALSE)
  LatticeDistribution(lattice, supp, prob)
  LatticeDistribution(supp)
  }


  For the generating function \code{LatticeDistribution()}, the arguments
  are processed in the following order:\cr
%
  Arguments \code{.withSim} and \code{.withArith} are used in any case.\cr
%
  If there is an argument \code{DiscreteDistribution} (of the respective class),
  all its slots (except for \code{.withSim} and \code{.withArith})
  will be used for filling the slots of the object of class
  \code{LatticeDistribution()}/\code{AffLinLatticeDistribution()}.
  If in addition, there is an argument \code{lattice} of class \code{Lattice},
  it will be checked for consistency
  with argument \code{DiscreteDistribution} and if oK will be used for slot
  lattice of the object of class
  \code{LatticeDistribution()}/\code{AffLinLatticeDistribution()}. In case
  there is no \code{lattice} argument, slot \code{lattice} will be constructed
  from slot \code{support} from argument \code{DiscreteDistribution}.\cr
%
  If there is no argument \code{DiscreteDistribution}, but there are arguments
  \code{supp} and \code{lattice} (the latter of class \code{Lattice}) then
  these are checked for consistency and if oK, generating function
  \code{DiscreteDistribution()} is called with arguments \code{supp},
  \code{prob}, \code{.withArith}, and \code{.withSim} to produce an object
  of class \code{DiscreteDistribution} the slots of which will be used for the
  filling the slots of the object of class
  \code{LatticeDistribution()}/\code{AffLinLatticeDistribution()}.
  If in this case, argument \code{prob} is not given explicitely, all elements
  in \code{supp} are equally weighted.
  \cr
%
  If there is no argument \code{DiscreteDistribution}, but there is an argument
  \code{lattice} of class \code{Lattice} (but no argument \code{slot}) then
 if \code{Length(lattice)} is finite, a corresponding support vector \code{supp}
  is generated from argument \code{lattice} and generating function
  \code{DiscreteDistribution()} is called with arguments \code{supp},
  \code{prob}, \code{.withArith}, and \code{.withSim} to produce an object
  of class \code{DiscreteDistribution} the slots of which will be used for the
  filling the slots of the object of class \code{LatticeDistribution()}.
  If in the same situation \code{Length(lattice)} is not finite, a finite length
  for the support vector is extracted from argument \code{prob} and after
  generating \code{supp} one procedes as in the finite \code{Length(lattice)}
  case. \cr
%
  If there is no argument \code{DiscreteDistribution} and no argument
  \code{lattice} of class \code{Lattice} but an argument \code{supp} then
  it will be checked if \code{supp} makes for a lattice, and if so,
  \code{DiscreteDistribution()} is called with arguments \code{supp},
  \code{prob}, \code{.withArith}, and \code{.withSim} to produce an object
  of class \code{DiscreteDistribution} the slots of which will be used for the
  filling the slots of the object of class \code{LatticeDistribution()}. The
  corresponding \code{lattice}-slot will be filled with information from
  argument \code{supp}.  \cr
  %
  The price for this flexibility of arguments, \code{LatticeDistribution()} may
  be called with, is that you should call \code{LatticeDistribution()} with
  \emph{named arguments} only.\cr
%
  Note that internally we suppress lattice points from the support where
  the probability is 0.
}


\author{Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}

\note{ Working with a computer, we use a finite interval as support which
carries at least mass \code{1-getdistrOption("TruncQuantile")}. }

\seealso{
\code{\link{Parameter-class}}
\code{\link{Lattice-class}}
\code{\link{LatticeDistribution-class}}
\code{\link{Reals-class}}
\code{\link{RtoDPQ.d}}
}
\examples{
  LatticeDistribution(DiscreteDistribution = DiscreteDistribution(supp =
                       c(4,3,2), prob=c(0.3,0.1,0.6)))
  LatticeDistribution(supp = c(4,3,2))
}
\keyword{distribution}
\concept{lattice distribution}
\concept{lattice of a distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{Distribution-class}
\docType{class}
\alias{Distribution-class}
\alias{.logExact}
\alias{.logExact-methods}
\alias{.logExact,Distribution-method}
\alias{.lowerExact}
\alias{.lowerExact-methods}
\alias{.lowerExact,Distribution-method}
\alias{Symmetry}
\alias{Symmetry-methods}
\alias{Symmetry,Distribution-method}

\title{Class "Distribution"}
\description{ The \code{Distribution-class} is the mother-class of 
class \code{UnivariateDistribution}.}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("Distribution")}.
}

\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"rSpace"}: the space of the image }
    \item{\code{param}}{Object of class \code{"OptionalParameter"}: the parameter }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers}
    \item{\code{d}}{Object of class \code{"OptionalFunction"}: density function}
    \item{\code{p}}{Object of class \code{"OptionalFunction"}: cumulative 
    distribution function}
    \item{\code{q}}{Object of class \code{"OptionalFunction"}: quantile function}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}

\section{Methods}{
  \describe{
    \item{img}{\code{signature(object = "Distribution")}: returns the space of 
                 the image}
    \item{param}{\code{signature(object = "Distribution")}: returns the 
                 parameter}
    \item{r}{\code{signature(object = "Distribution")}: returns the random 
              number generator }
    \item{d}{\code{signature(object = "Distribution")}: returns the density 
              function }
    \item{p}{\code{signature(object = "Distribution")}: returns the cumulative 
              distribution function }
    \item{q}{\code{signature(object = "Distribution")}: returns the quantile 
                function}
    \item{.logExact}{\code{signature(object = "Distribution")}: returns slot 
      \code{.logExact} if existing; else tries to convert the object to a newer 
      version of its class by \code{\link{conv2NewVersion}} and
      returns the corresponding slot of the converted object.}
    \item{.lowerExact}{\code{signature(object = "Distribution")}: returns slot 
       \code{.lowerExact} if existing; else tries to convert the object to a 
       newer version of its class by \code{\link{conv2NewVersion}} and
     returns the corresponding slot of the converted object.}
    \item{Symmetry:}{returns slot \code{Symmetry} if existing; else
    tries to convert the object to a 
       newer version of its class by \code{\link{conv2NewVersion}} and
     returns the corresponding slot of the converted object.}
  }
}


\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}


\seealso{
\code{\link{UnivariateDistribution-class}}
\code{\link{Parameter-class}}

}
\keyword{distribution}
\concept{conditional distribution}
\concept{multivariate distribution}
\concept{univariate distribution}
\concept{discrete distribution}
\concept{lattice distribution}
\concept{absolutely continuous distribution}
\concept{S4 distribution class}

\name{gaps-methods}
\docType{methods}
\alias{gaps-methods}
\alias{setgaps-methods}
\alias{gaps<--methods}
\alias{gaps}
\alias{setgaps}
\alias{gaps<-}
\alias{gaps,AbscontDistribution-method}
\alias{gaps<-,AbscontDistribution-method}
\alias{setgaps,AbscontDistribution-method}
\alias{setgaps,UnivarMixingDistribution-method}
\title{ Methods for Functions gaps and setgaps in Package `distr' }
\description{[set]gaps-methods}
\usage{gaps(object)
gaps(object)
gaps(object) <- value
setgaps(object, ...)
\S4method{gaps}{AbscontDistribution}(object)
%\S4method{gaps<-}{AbscontDistribution}(object, value)
\S4method{setgaps}{AbscontDistribution}(object, exactq = 6, 
           ngrid = 50000, ...)
}
\arguments{
  \item{object}{object of class  \code{"AbscontDistribution"} (or subclasses)}
  \item{\dots}{further arguments to be passed to \code{setgaps}; not yet used.}
  \item{value}{\eqn{n \times 2}{n x 2} matrix \code{m} of numerics where 
               \code{c(t(m))} is an ordered vector; value to be assigned to slot 
               \code{gaps}}
  \item{exactq}{density values smaller than 
                \eqn{10^{\scriptsize -{\rm exactq}}}{10^-exactq}
                are considered as \eqn{0}.} 
  \item{ngrid}{number of gridpoints at which the density is evaluated.} 
}

\section{Methods}{\describe{
\item{gaps}{\code{signature(object = "AbscontDistribution")}: returns  slot 
\code{gaps} of an absolutely continuous distribution}
\item{setgaps}{\code{signature(object = "AbscontDistribution")}: tries to find 
out the gaps (where \code{d(object)} is approximately
 0) and fills slot \code{gaps} of \code{object} correspondingly}
\item{setgaps}{\code{signature(object = "UnivarMixingDistribution")}: 
for each mixing component, if it has a slot \code{gaps}, tries to find out the gaps 
 and fills slot \code{gaps} of the component correspondingly, and,
 subsequently merges all found gap-slots of the components to a gap-slot
 for the object, using internal function \code{\link{.mergegaps2}}.}
\item{gaps<-}{\code{signature(object = "AbscontDistribution")}: modifies  slot 
\code{gaps} of an absolutely continuous distribution}
}}
\keyword{methods}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{accessor function}
\concept{replacement function}

\name{UnivarDistrList-class}
\docType{class}
\alias{UnivarDistrList-class}
\alias{coerce,UnivariateDistribution,UnivarDistrList-method}

\title{List of univariate distributions}
\description{Create a list of univariate distributions}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("UnivarDistrList", ...)}.
  More frequently they are created via the generating function 
  \code{\link{DistrList}}. 
}
\section{Slots}{
  \describe{
      \item{\code{.Data}}{Object of class \code{"list"}. 
      A list of univariate distributions.}
  }
}
\section{Extends}{
Class \code{"DistrList"}, directly.\cr
Class \code{"list"}, by class \code{"DistrList"}.\cr
Class \code{"vector"}, by class \code{"DistrList"}.
}
\section{Methods}{
  \describe{
    \item{coerce}{\code{signature(from = "UnivariateDistribution", to = "UnivarDistrList")}:
      create a \code{UnivarDistrList} object from a univariate distribution}
  }
}
%\references{}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de}}
%\note{}
\seealso{\code{\link{UnivarDistrList}}, \code{\link{DistrList-class}},
    \code{\link[distr]{UnivariateDistribution-class}}}
\examples{
(DL <- new("UnivarDistrList", list(Norm(), Exp())))
plot(DL)
as(Norm(), "UnivarDistrList")
}
\keyword{distribution}
\keyword{list}
\concept{multivariate distribution}
\concept{S4 distribution class}

\name{Logis-class}
\docType{class}
\alias{Logis-class}
\alias{Logis}
\alias{initialize,Logis-method}

\title{Class "Logis"}
\description{ The Logistic distribution with \code{location} \eqn{= \mu}{= m}, 
  by default \code{= 0}, and \code{scale} \eqn{= \sigma}{= s}, by default \code{= 1}, 
  has distribution function
  \deqn{
    p(x) = \frac{1}{1 + e^{-(x-\mu)/\sigma}}%
  }{p(x) = 1 / (1 + exp(-(x-m)/s))}  and density
  \deqn{
    d(x)= \frac{1}{\sigma}\frac{e^{(x-\mu)/\sigma}}{(1 + e^{(x-\mu)/\sigma})^2}%
  }{d(x) = 1/s exp((x-m)/s) (1 + exp((x-m)/s))^-2.}

  It is a long-tailed distribution with mean \eqn{\mu}{m} and variance
  \eqn{\pi^2/3 \sigma^2}{pi^2 /3 s^2}. C.f. \code{\link[stats:Logistic]{rlogis}} 
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Logis(location, scale)}.
This object is a logistic distribution.
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}: The space of the image of this distribution has got dimension 1
    and the name "Real Space". }
    \item{\code{param}}{Object of class \code{"LogisParameter"}: the parameter of this distribution (location and scale),
     declared at its instantiation }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers (calls function rlogis)}
    \item{\code{d}}{Object of class \code{"function"}: density function (calls function dlogis)}
    \item{\code{p}}{Object of class \code{"function"}: cumulative function (calls function plogis)}
    \item{\code{q}}{Object of class \code{"function"}: inverse of the cumulative function (calls function qlogis)}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"AbscontDistribution"}, directly.\cr
Class \code{"UnivariateDistribution"}, by class \code{"AbscontDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"AbscontDistribution"}.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Logis")}: initialize method }
    \item{location}{\code{signature(object = "Logis")}: returns the slot \code{location} of the parameter of the distribution }
    \item{location<-}{\code{signature(object = "Logis")}: modifies the slot \code{location} of the parameter of the distribution }
    \item{scale}{\code{signature(object = "Logis")}: returns the slot \code{scale} of the parameter of the distribution }
    \item{scale<-}{\code{signature(object = "Logis")}: modifies the slot \code{scale} of the parameter of the distribution }
    \item{*}{\code{signature(e1 = "Logis", e2 = "numeric")}}
    \item{+}{\code{signature(e1 = "Logis", e2 = "numeric")}: 
    For the logistic location scale family we use its closedness under affine linear transformations.}
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }



\seealso{
\code{\link{LogisParameter-class}}
\code{\link{AbscontDistribution-class}}
\code{\link{Reals-class}}
\code{\link[stats:Logistic]{rlogis}}





}
\examples{
L <- Logis(location = 1,scale = 1)
# L is a logistic distribution with  location = 1 and scale = 1.
r(L)(1) # one random number generated from this distribution, e.g. 5.87557
d(L)(1) # Density of this distribution is 0.25 for x = 1.
p(L)(1) # Probability that x < 1 is 0.5.
q(L)(.1) # Probability that x < -1.197225 is 0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
location(L) # location of this distribution is 1.
location(L) <- 2 # location of this distribution is now 2.
}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{Logistic distribution}
\concept{S4 distribution class}
\concept{location scale family}

\name{DistrSymmList-class}
\docType{class}
\alias{DistrSymmList-class}

\title{List of Symmetries for a List of Distributions}
\description{Create a list of symmetries for a list of distributions}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("DistrSymmList", ...)}.
  More frequently they are created via the generating function 
  \code{DistrSymmList}.
}
\section{Slots}{
  \describe{
    \item{\code{.Data}}{ Object of class \code{"list"}. A list 
      of objects of class \code{"DistributionSymmetry"}. }
  }
}
\section{Extends}{
Class \code{"list"}, from data part.\cr
Class \code{"vector"}, by class \code{"list"}.
}
%\section{Methods}{
%}
%\references{}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de}}
%\note{}
\seealso{\code{\link{DistributionSymmetry-class}}}
\examples{
new("DistrSymmList", list(NoSymmetry(), SphericalSymmetry(SymmCenter = 1), 
                          EllipticalSymmetry(SymmCenter = 2)))
}
\concept{symmetry}
\keyword{classes}
\keyword{distribution}

\name{q-methods}
\docType{methods}
\alias{q-methods}
\alias{q}
\alias{q,Distribution-method}
\alias{q.l-methods}
\alias{q.l}
\alias{q.l,Distribution-method}
\title{Methods for Function q in Package `distr'}
\description{q-methods}

\section{Methods}{\describe{
\item{q}{\code{signature(save  = "Distribution")}: returns the
(left-continuous) quantile function, i.e.;
\eqn{{\rm q}(s)=\inf\{t \,\big|\, P({\tt object}\leq t)\geq s\}}{q(s)=inf\{t|P(object<=t)>=s\}}}
\item{q.l}{\code{signature(object  = "Distribution")}: 
from distr-2.6 onwards, we provide this as a synonym for 
method \code{q}; this synonym more explicitely states that we are dealing with
the left-continuous variant of a quantile function. It is useful in particular 
when used from the console in RStudio, as RStudio catches calls to 
\code{q()} and treats them separately from usual R evaluation.  
The developers of RStudio have been asked to fix this and comply with standard 
R evaluation which explicitely allows overloading \code{q()} as we do it in this 
package, but so far have refused to do so, as they claim overloading 
\code{q()} was insane.
}}}
\seealso{
\code{\link{Distribution-class}}, \code{q.r}
}
\examples{
require(distr)
N <- Norm()
q(N)(0.3)
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
q.l(N)(0.3)
}

\keyword{distribution}
\keyword{methods}
\concept{pseudo-inverse distribution function}
\concept{quantile function}
\concept{accessor function}
\concept{left-continuous}

\name{BinomParameter-class} 
\docType{class}
\alias{BinomParameter-class}
\alias{initialize,BinomParameter-method}

\title{Class "BinomParameter"}
\description{ The parameter of a binomial distribution, used by Binom-class}
\section{Objects from the Class}{
Objects can be created by calls of the form 
      \code{new("BinomParameter", prob, size)}.
Usually an object of this class is not needed on its own, it is generated 
automatically when an object of the class Binom
is instantiated. 
}
\section{Slots}{
  \describe{
    \item{\code{prob}}{Object of class \code{"numeric"}: 
          the probability of a binomial distribution }
    \item{\code{size}}{Object of class \code{"numeric"}: 
          the size of a binomial distribution }
    \item{\code{name}}{Object of class \code{"character"}: 
          a name / comment for the parameters }
  }
}
\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "BinomParameter")}: 
          initialize method }
    \item{prob}{\code{signature(object = "BinomParameter")}: returns the slot 
          \code{prob} of the parameter of the distribution }
    \item{prob<-}{\code{signature(object = "BinomParameter")}: modifies the slot 
          \code{prob} of the parameter of the distribution }
    \item{size}{\code{signature(object = "BinomParameter")}: returns the slot 
          \code{size} of the parameter of the distribution }
    \item{size<-}{\code{signature(object = "BinomParameter")}: modifies the slot 
          \code{size} of the parameter of the distribution}
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }


\seealso{
\code{\link{Binom-class}}
\code{\link{Parameter-class}}
}

\examples{
W <- new("BinomParameter",prob=0.5,size=1)
size(W) # size of this distribution is 1.
size(W) <- 2 # size of this distribution is now 2.
}
\keyword{distribution}
\concept{parameter}
\concept{Binomial distribution}
\concept{S4 parameter class}

\name{qqbounds}
\title{Computation of confidence intervals for qqplot}
\usage{
qqbounds(x,D,alpha,n,withConf.pw, withConf.sim,
         exact.sCI=(n<100),exact.pCI=(n<100),
         nosym.pCI = FALSE, debug = FALSE)

}
\alias{qqbounds}

\arguments{
\item{x}{data to be checked for compatibility with distribution \code{D}.}
\item{D}{object of class \code{"UnivariateDistribution"}, the assumed data
distribution.}
\item{alpha}{confidence level}
\item{n}{sample size}
\item{withConf.pw}{logical; shall pointwise confidence lines be computed?}
\item{withConf.sim}{logical; shall simultaneous confidence lines be computed?}
\item{exact.pCI}{logical; shall pointwise CIs be determined with exact Binomial distribution?}
\item{exact.sCI}{logical; shall simultaneous CIs be determined with exact kolmogorov distribution?}
\item{nosym.pCI}{logical; shall we use (shortest) asymmetric CIs?}
\item{debug}{logical; if \code{TRUE} additional output to debug confidence bounds.}
}
\description{
  We compute confidence intervals for QQ plots.
  These can be simultaneous (to check whether the whole data set is compatible)
  or pointwise (to check whether each (single) data point is compatible);}

\details{
  Both simultaneous  and pointwise confidence intervals come in a
  finite-sample and an asymptotic version;
  the finite sample versions will get quite slow
  for large data sets \code{x}, so in these cases the asymptotic version
  will be preferrable.\cr
   For simultaneous intervals,
  the finite sample version is based on C function \code{"pkolmogorov2x"}
  from package \pkg{stats}, while the asymptotic one uses
   R function \code{pkstwo} again from package \pkg{stats}, both taken
   from the code to \code{\link[stats:ks.test]{ks.test}}.

   Both finite sample and asymptotic versions use the fact,
   that the distribution of the supremal distance between the
   empirical distribution \eqn{\hat F_n}{F.emp} and the corresponding theoretical one
   \eqn{F} (assuming data from \eqn{F})
   does not depend on \eqn{F} for continuous distribution \eqn{F}
   and leads to the Kolmogorov distribution (compare, e.g. Durbin(1973)).
   In case of \eqn{F} with jumps, the corresponding Kolmogorov distribution
   is used to produce conservative intervals.
 \cr
    For pointwise intervals,
  the finite sample version is based on corresponding binomial distributions,
  (compare e.g., Fisz(1963)), while the asymptotic one uses a CLT approximation
  for this binomial distribution. In fact, this approximation is only valid
  for distributions with strictly positive density at the evaluation quantiles.

  In the finite sample version, the binomial distributions will in general not
  be symmetric, so that, by setting \code{nosym.pCI} to \code{TRUE} we may
  produce shortest asymmetric confidence intervals (albeit with a considerable
  computational effort).

  The symmetric intervals returned by default will
  be conservative (which also applies to distributions with jumps in this case).

  For distributions with jumps or with density (nearly) equal to 0 at the
  corresponding quantile, we use the approximation of \code{(D-E(D))/sd(D)}
  by the standard normal at these points; this latter approximation is only
  available if package \pkg{distrEx} is installed; otherwise the corresponding
  columns will be filled with \code{NA}.
}
\value{
  A list with components \code{crit} --- a matrix with the lower and upper confidence
  bounds, and \code{err} a logical vector of length 2.

  Component \code{crit} is a matrix with \code{length(x)} rows
  and four columns \code{c("sim.left","sim.right","pw.left","pw.right")}.
  Entries will be set to \code{NA} if the corresponding \code{x} component
  is not in \code{support(D)} or if the computation method returned an error
  or if the corresponding parts have not been required (if \code{withConf.pw}
  or \code{withConf.sim} is \code{FALSE}).

  \code{err} has components \code{pw}
  ---do we have a non-error return value for the computation of pointwise CI's
  (\code{FALSE} if \code{withConf.pw} is \code{FALSE})---  and \code{sim}
  ---do we have a   non-error return value for the computation of simultaneous CI's
  (\code{FALSE} if \code{withConf.sim} is \code{FALSE}).
}
\references{
  Durbin, J. (1973)
  \emph{Distribution theory for tests based on the sample distribution
    function}.  SIAM.

  Fisz, M. (1963). \emph{Probability Theory and Mathematical Statistics}.
  3rd ed. Wiley, New York.
}
\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}
}
\seealso{
  \code{\link[stats:qqnorm]{qqplot}} from package \pkg{stats} -- the standard QQ plot
  function, \code{\link[stats:ks.test]{ks.test}} again from package \pkg{stats}
  for the implementation of the Kolmogorov distributions;
  \code{\link[distr]{qqplot}} from package \pkg{distr} for
  comparisons of distributions, and
 \code{qqplot} from package \pkg{distrMod} for comparisons
 of data with models, as well as  \code{RobAStBase::qqplot} from package \pkg{RobAStBase} for
 checking of corresponding robust esimators.
}
\examples{
qqplot(Norm(15,sqrt(30)), Chisq(df=15))
## uses:
old.digits <- getOption("digits")
on.exit(options(digits = old.digits))
options(digits = 6)
qqbounds(x = rnorm(30),Norm(),alpha=0.95,n=30,
        withConf.pw = TRUE, withConf.sim  = TRUE,
        exact.sCI=TRUE ,exact.pCI= TRUE,
        nosym.pCI = FALSE)
## other calls:
qqbounds(x = rchisq(30,df=4),Chisq(df=4),alpha=0.95,n=30,
        withConf.pw = TRUE, withConf.sim  = TRUE,
        exact.sCI=FALSE ,exact.pCI= FALSE,
        nosym.pCI = FALSE)
qqbounds(x = rchisq(30,df=4),Chisq(df=4),alpha=0.95,n=30,
        withConf.pw = TRUE, withConf.sim  = TRUE,
        exact.sCI = TRUE ,exact.pCI= TRUE,
        nosym.pCI = TRUE)
options(digits = old.digits)
}
\keyword{hplot}
\keyword{distribution}

\name{width-methods}
\docType{methods}
\alias{width-methods}
\alias{width<--methods}
\alias{width}
\alias{width<-}
\alias{width,Lattice-method}
\alias{width<-,Lattice-method}
\alias{width,LatticeDistribution-method}
\alias{width<-,LatticeDistribution-method}
\title{ Methods for Function width in Package `distr' }
\description{width-methods}
\section{Methods}{\describe{
\item{width}{\code{signature(object = "Lattice")}: returns the slot width of the lattice }
\item{width<-}{\code{signature(object = "Lattice")}: modifies the slot width of the lattice }
\item{width}{\code{signature(object = "LatticeDistribution")}: returns the slot width of the lattice slot of the distribution}
\item{width<-}{\code{signature(object = "LatticeDistribution")}: modifies the slot width of the lattice slot of the distribution}
}}
\keyword{methods}
\keyword{distribution}
\concept{space}
\concept{S4 space class}
\concept{lattice}
\concept{affine linear lattice}
\concept{accessor function}
\concept{replacement function}

\name{Exp-class}
\docType{class}
\alias{Exp-class}
\alias{Exp}
\alias{initialize,Exp-method}
\title{Class "Exp"}
\description{
  The exponential distribution with rate \eqn{\lambda} has density
  \deqn{
    f(x) = \lambda {e}^{- \lambda x}}{
    f(x) = lambda e^(- lambda x)}
  for \eqn{x \ge 0}.

  C.f. \code{\link[stats:Exponential]{rexp}}
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Exp(rate)}.
This object is an exponential distribution. 
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}:
      The space of the image of this distribution has got dimension 1
      and the name "Real Space".}
    \item{\code{param}}{Object of class \code{"ExpParameter"}:
      the parameter of this distribution (rate), declared at its instantiation }
    \item{\code{r}}{Object of class \code{"function"}:
      generates random numbers (calls function rexp)}
    \item{\code{d}}{Object of class \code{"function"}:
      density function (calls function dexp)}
    \item{\code{p}}{Object of class \code{"function"}:
      cumulative function (calls function pexp)}
    \item{\code{q}}{Object of class \code{"function"}:
      inverse of the cumulative function (calls function qexp)}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
  Class \code{"ExpOrGammaOrChisq"}, directly.\cr
  Class \code{"AbscontDistribution"}, by class \code{"ExpOrGammaOrChisq"}.\cr
  Class \code{"UnivariateDistribution"}, by class \code{"AbscontDistribution"}.
  Class \code{"Distribution"}, by class \code{"AbscontDistribution"}.
}
\section{Is-Relations}{
By means of \code{setIs}, R ``knows'' that a distribution object \code{obj} of class \code{"Exp"} also is
a Gamma distribution with parameters \code{shape = 1, scale = 1/rate(obj)} and a Weibull  distribution with 
parameters \code{shape = 1, scale = 1/rate(obj)}
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Exp")}:
      initialize method}
    \item{rate}{\code{signature(object = "Exp")}:
      returns the slot rate of the parameter of the distribution}
    \item{rate<-}{\code{signature(object = "Exp")}:
      modifies the slot rate of the parameter of the distribution}
    \item{*}{\code{signature(e1 = "Exp", e2 = "numeric")}: 
    For the exponential distribution we use its closedness under positive scaling transformations.}
  }
}

\author{
Thomas Stabla \email{statho3@web.de},\cr 
Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
}
\seealso{
\code{\link{ExpParameter-class}}
\code{\link{AbscontDistribution-class}}
\code{\link{Reals-class}}
\code{\link[stats:Exponential]{rexp}}
}
\examples{
E <- Exp(rate = 1) # E is a exp distribution with rate = 1.
r(E)(1) # one random number generated from this distribution, e.g. 0.4190765
d(E)(1) # Density of this distribution is 0.3678794 for x = 1.
p(E)(1) # Probability that x < 1 is 0.6321206.
q(E)(.1) # Probability that x < 0.1053605 is 0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
rate(E) # rate of this distribution is 1.
rate(E) <- 2 # rate of this distribution is now 2.
is(E, "Gammad") # yes
as(E,"Gammad")
is(E, "Weibull") 
E+E+E ###  a Gammad -distribution
2*E+Gammad(scale=1)
}
\keyword{distribution}
\concept{Exponential distribution}
\concept{memoryless distribution}
\concept{absolutely continuous distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{igamma}
\alias{igamma}

\title{Inverse of the digamma function}
\description{
  Function \code{igamma} is a numerical inverse of \code{digamma}.
}
\usage{
igamma(v)
}
\arguments{
  \item{v}{ a numeric in the range [-100000,18] }
  }
\details{
\code{igamma} is vectorized;  it is won
  by spline inversion of a grid; it works well for range 
  [digamma(1e-5);digamma(1e8)] or [-100000,18].
}
\value{\code{igamma(x)} is a value \code{u} such that \code{digamma(u} is approximately \code{x}.}
\author{Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}
%\note{}
\seealso{\code{digamma}}
\examples{
igamma(digamma(c(1e-4,1,20,1e8)))
}
\keyword{math}

\name{PosDefSymmMatrix}
\alias{PosDefSymmMatrix}
\alias{PosSemDefSymmMatrix}

\title{Generating functions for PosSemDefSymmMatrix-class resp. PosDefSymmMatrix-class}
\description{Generates an object of class \code{"PosSemDefSymmMatrix"} resp. of class \code{"PosDefSymmMatrix"}.}
\usage{PosSemDefSymmMatrix(mat)
       PosDefSymmMatrix(mat)}
\arguments{
  \item{mat}{A numeric positive-[semi-]definite, symmetric 
    matrix with finite entries.}
}
\details{
  If \code{mat} is no matrix, \code{as.matrix} is applied.
}
\value{Object of class \code{"PosSemDefSymmMatrix"} resp. of class \code{"PosDefSymmMatrix"}}
%\references{}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de},
Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}
%\note{}
\seealso{\code{\link{PosDefSymmMatrix-class}}}
\examples{
PosSemDefSymmMatrix(1)
PosSemDefSymmMatrix(diag(2))
PosDefSymmMatrix(1)
PosDefSymmMatrix(diag(2))
}
\concept{positive-semi-definite matrix}
\concept{positive-definite matrix}
\concept{symmetric matrix}
\keyword{array}

\name{operators-methods}
\docType{methods}

\alias{operators-methods}
\alias{operators}
\alias{-,UnivariateDistribution,missing-method}
\alias{-,LatticeDistribution,missing-method}
\alias{-,Norm,missing-method}
\alias{+,UnivariateDistribution,numeric-method}
\alias{+,AbscontDistribution,numeric-method}
\alias{+,DiscreteDistribution,numeric-method}
\alias{+,LatticeDistribution,numeric-method}
\alias{+,UnivarLebDecDistribution,numeric-method}
\alias{+,AffLinAbscontDistribution,numeric-method}
\alias{+,AffLinDiscreteDistribution,numeric-method}
\alias{+,AffLinLatticeDistribution,numeric-method}
\alias{+,AffLinUnivarLebDecDistribution,numeric-method}
\alias{+,CompoundDistribution,numeric-method}
\alias{+,Cauchy,numeric-method}
\alias{+,Dirac,numeric-method}
\alias{+,Norm,numeric-method}
\alias{+,Unif,numeric-method}
\alias{+,numeric,UnivariateDistribution-method}
\alias{+,numeric,LatticeDistribution-method}
%%\alias{-,UnivariateDistribution,ANY-method}
\alias{-,UnivariateDistribution,numeric-method}
\alias{-,UnivariateDistribution,UnivariateDistribution-method}
\alias{-,LatticeDistribution,numeric-method}
\alias{-,LatticeDistribution,LatticeDistribution-method}
\alias{-,LatticeDistribution,UnivariateDistribution-method}
\alias{-,UnivariateDistribution,LatticeDistribution-method}
\alias{-,AcDcLcDistribution,AcDcLcDistribution-method}
\alias{-,numeric,UnivariateDistribution-method}
\alias{-,numeric,LatticeDistribution-method}
\alias{*,UnivariateDistribution,numeric-method}
\alias{*,AbscontDistribution,numeric-method}
\alias{*,DiscreteDistribution,numeric-method}
\alias{*,LatticeDistribution,numeric-method}
\alias{*,UnivarLebDecDistribution,numeric-method}
\alias{*,CompoundDistribution,numeric-method}
\alias{*,AffLinAbscontDistribution,numeric-method}
\alias{*,AffLinDiscreteDistribution,numeric-method}
\alias{*,AffLinLatticeDistribution,numeric-method}
\alias{*,AffLinUnivarLebDecDistribution,numeric-method}
\alias{*,DExp,numeric-method}
\alias{*,Exp,numeric-method}
\alias{*,ExpOrGammaOrChisq,numeric-method}
\alias{*,Weibull,numeric-method}
\alias{*,Cauchy,numeric-method}
\alias{*,Dirac,numeric-method}
\alias{*,Norm,numeric-method}
\alias{*,Logis,numeric-method}
\alias{*,Lnorm,numeric-method}
\alias{*,Unif,numeric-method}
\alias{*,numeric,UnivariateDistribution-method}
\alias{*,numeric,LatticeDistribution-method}
\alias{/,UnivariateDistribution,numeric-method}
\alias{/,LatticeDistribution,numeric-method}
\alias{+,UnivariateDistribution,UnivariateDistribution-method}
\alias{+,AbscontDistribution,AbscontDistribution-method}
\alias{+,AbscontDistribution,DiscreteDistribution-method}
\alias{+,DiscreteDistribution,AbscontDistribution-method}
\alias{+,DiscreteDistribution,DiscreteDistribution-method}
\alias{+,LatticeDistribution,DiscreteDistribution-method}
\alias{+,LatticeDistribution,LatticeDistribution-method}
\alias{+,UnivarLebDecDistribution,UnivarLebDecDistribution-method}
\alias{+,AcDcLcDistribution,AcDcLcDistribution-method}
\alias{+,Binom,Binom-method}
\alias{+,Cauchy,Cauchy-method}
\alias{+,Chisq,Chisq-method}
\alias{+,Dirac,Dirac-method}
\alias{+,ExpOrGammaOrChisq,ExpOrGammaOrChisq-method}
\alias{+,Pois,Pois-method}
\alias{+,Nbinom,Nbinom-method}
\alias{+,Norm,Norm-method}
\alias{+,Logis,numeric-method}
\alias{+,Dirac,UnivariateDistribution-method}
\alias{+,Dirac,DiscreteDistribution-method}
\alias{+,UnivariateDistribution,Dirac-method}
\alias{-,numeric,Beta-method}
\alias{-,Dirac,Dirac-method}
\alias{*,Dirac,Dirac-method}
\alias{*,Dirac,UnivariateDistribution-method}
\alias{*,UnivariateDistribution,Dirac-method}
\alias{*,AcDcLcDistribution,AcDcLcDistribution-method}
\alias{/,Dirac,Dirac-method}
\alias{/,numeric,Dirac-method}
\alias{/,numeric,AcDcLcDistribution-method}
\alias{/,AcDcLcDistribution,AcDcLcDistribution-method}
\alias{^,AcDcLcDistribution,numeric-method}
\alias{^,AcDcLcDistribution,Integer-method}
\alias{^,AcDcLcDistribution,AcDcLcDistribution-method}
\alias{^,numeric,AcDcLcDistribution-method}



\title{Methods for operators +,-,*,/,... in Package distr}

\description{Arithmetics and unary mathematical transformations for distributions}

\details{Arithmetics as well as all functions from group \code{Math}, see \code{\link[methods:S4groupGeneric]{Math}}
are provided for distributions; wherever possible exact expressions are used; else 
random variables are generated according to this transformation and subsequently the remaining
slots filled by \code{\link{RtoDPQ}}, \code{\link{RtoDPQ.d}}}

\section{Methods}{\describe{
\item{\code{-}}{\code{signature(e1 = "UnivariateDistribution", e2 = "missing")} unary operator; result again of class \code{"UnivariateDistribution"}; exact}
\item{\code{-}}{\code{signature(e1 = "Norm", e2 = "missing")} unary operator; result again of \code{"Norm"}; exact}
\item{\code{+}}{\code{signature(e1 = "UnivariateDistribution", e2 = "numeric")} result again of class \code{"UnivariateDistribution"}; exact}
\item{\code{+}}{\code{signature(e1 = "AbscontDistribution", e2 = "numeric")} result of 
 class \code{"AffLinAbscontDistribution"}; exact}
\item{\code{+}}{\code{signature(e1 = "DiscreteDistribution", e2 = "numeric")} result of 
 class \code{"AffLinDiscreteDistribution"}; exact}
\item{\code{+}}{\code{signature(e1 = "LatticeDistribution", e2 = "numeric")} result  of 
 class \code{"AffLinLatticeDistribution"}; exact}
\item{\code{+}}{\code{signature(e1 = "UnivarLebDecDistribution", e2 = "numeric")} result of 
 class \code{"AffLinUnivarLebDecDistribution"}; exact}
\item{\code{+}}{\code{signature(e1 = "CompoundDistribution", e2 = "numeric")} result of 
 class \code{"AffLinUnivarLebDecDistribution"}; exact}
\item{\code{+}}{\code{signature(e1 = "AffLinAbscontDistribution", e2 = "numeric")} result again of 
 class \code{"AffLinAbscontDistribution"}; exact}
\item{\code{+}}{\code{signature(e1 = "AffLinDiscreteDistribution", e2 = "numeric")} result again of 
 class \code{"AffLinDiscreteDistribution"}; exact}
\item{\code{+}}{\code{signature(e1 = "AffLinLatticeDistribution", e2 = "numeric")} result again of 
 class \code{"AffLinLatticeDistribution"}; exact}
\item{\code{+}}{\code{signature(e1 = "AffLinUnivarLebDecDistribution", e2 = "numeric")} result of 
 class \code{"AffLinUnivarLebDecDistribution"}; exact}
\item{\code{+}}{\code{signature(e1 = "Cauchy", e2 = "numeric")} result again of class \code{"Cauchy"}; exact}
\item{\code{+}}{\code{signature(e1 = "Dirac", e2 = "numeric")} result again of class \code{"Dirac"}; exact}
\item{\code{+}}{\code{signature(e1 = "Norm", e2 = "numeric")} result again of class \code{"Norm"}; exact}
\item{\code{+}}{\code{signature(e1 = "Unif", e2 = "numeric")} result again of class \code{"Unif"}; exact}
\item{\code{+}}{\code{signature(e1 = "Logis", e2 = "numeric")}  result again of class \code{"Logis"}; exact}
\item{\code{+}}{\code{signature(e1 = "numeric", e2 = "UnivariateDistribution")} is translated to  
         \code{signature(e1 = "UnivariateDistribution", e2 = "numeric")}; exact}
\item{\code{-}}{\code{signature(e1 = "UnivariateDistribution", e2= "ANY")};exact}
\item{\code{-}}{\code{signature(e1 = "UnivariateDistribution", e2 = "numeric")} is translated to 
 \code{e1 + (-e2)}; exact}
\item{\code{-}}{\code{signature(e1 = "numeric", e2 = "UnivariateDistribution")} is translated to \code{(-e1) + e2}; exact}
\item{\code{-}}{\code{signature(e1 = "numeric", e2 = "Beta")} if \code{ncp(e2)==0} and \code{e1 == 1},
an exact (central) \code{Beta(shape1 = shape2(e2), shape2 = shape1(e2))} is returned, else
the default method is used; exact}
\item{\code{*}}{\code{signature(e1 = "UnivariateDistribution", e2 = "numeric")} result again of class \code{"UnivariateDistribution"}; exact}
\item{\code{*}}{\code{signature(e1 = "AbscontDistribution", e2 = "numeric")} result of 
 class \code{"AffLinAbscontDistribution"}; exact}
\item{\code{*}}{\code{signature(e1 = "DiscreteDistribution", e2 = "numeric")} result of 
 class \code{"AffLinDiscreteDistribution"}; exact}
\item{\code{*}}{\code{signature(e1 = "LatticeDistribution", e2 = "numeric")} result  of 
 class \code{"AffLinLatticeDistribution"}; exact}
\item{\code{*}}{\code{signature(e1 = "UnivarLebDecDistribution", e2 = "numeric")} result of 
 class \code{"AffLinUnivarLebDecDistribution"}; exact}
\item{\code{*}}{\code{signature(e1 = "CompoundDistribution", e2 = "numeric")} result of 
 class \code{"AffLinUnivarLebDecDistribution"}; exact}
\item{\code{*}}{\code{signature(e1 = "AffLinAbscontDistribution", e2 = "numeric")} result again of 
 class \code{"AffLinAbscontDistribution"}; exact}
\item{\code{*}}{\code{signature(e1 = "AffLinDiscreteDistribution", e2 = "numeric")} result again of 
 class \code{"AffLinDiscreteDistribution"}; exact}
\item{\code{*}}{\code{signature(e1 = "AffLinLatticeDistribution", e2 = "numeric")} result again of 
 class \code{"AffLinLatticeDistribution"}; exact}
\item{\code{*}}{\code{signature(e1 = "AffLinUnivarLebDecDistribution", e2 = "numeric")} result of 
 class \code{"AffLinUnivarLebDecDistribution"}; exact}
\item{\code{*}}{\code{signature(e1 = "DExp", e2 = "numeric")} if \code{abs(e2)>0} result again of class \code{"DExp"}; exact}
\item{\code{*}}{\code{signature(e1 = "Exp", e2 = "numeric")} if \code{e2>0} result again of class \code{"Exp"}; exact}
\item{\code{*}}{\code{signature(e1 = "ExpOrGammaOrChisq", e2 = "numeric")} if \code{e1} is a Gamma distribution and \code{e2>0} 
                                                                    result of class \code{"Gammad"}; exact}
\item{\code{*}}{\code{signature(e1 = "Weibull", e2 = "numeric")} if \code{e2>0} 
                                                                    result of class \code{"Weibull"}; exact}
\item{\code{*}}{\code{signature(e1 = "Cauchy", e2 = "numeric")}  if \code{abs(e2)>0} result again of class \code{"Cauchy"}; exact}
\item{\code{*}}{\code{signature(e1 = "Dirac", e2 = "numeric")}  result again of class \code{"Dirac"}; exact}
\item{\code{*}}{\code{signature(e1 = "Norm", e2 = "numeric")}  if \code{abs(e2)>0} result again of class \code{"Norm"}; exact}
\item{\code{*}}{\code{signature(e1 = "Unif", e2 = "numeric")}  if \code{abs(e2)>0} result again of class \code{"Unif"}; exact}
\item{\code{*}}{\code{signature(e1 = "Logis", e2 = "numeric")}  if \code{e2>0} result again of class \code{"Logis"}; exact}
\item{\code{*}}{\code{signature(e1 = "Lnorm", e2 = "numeric")}  if \code{e2>0} result again of class \code{"Lnorm"}; exact}
\item{\code{*}}{\code{signature(e1 = "numeric", e2 = "UnivariateDistribution")} is translated to 
         \code{signature(e1 = "UnivariateDistribution", e2 = "numeric")}; exact}
\item{\code{/}}{\code{signature(e1 = "UnivariateDistribution", e2 = "numeric")} is translated to \code{e1 * (1/e2)}; exact}
\item{\code{+}}{\code{signature(e1 = "UnivariateDistribution", e2 = "UnivariateDistribution")}  result again of class 
         \code{"UnivariateDistribution"}; is generated by simulations}
\item{\code{-}}{\code{signature(e1 = "UnivariateDistribution", e2 = "UnivariateDistribution")} is translated to \code{(-e1) + (-e2)}; 
          result again of class   \code{"UnivariateDistribution"}; is generated by simulations}
\item{\code{-}}{\code{signature(e1 = "AcDcLcDistribution", e2 = "AcDcLcDistribution")}: both operands are coerced
                to class \code{"UnivarLebDecDistribution"} and the corresponding method is used.
                }
\item{\code{+}}{\code{signature(e1 = "AbscontDistribution", e2 = "AbscontDistribution")} assumes \code{e1}, \code{e2} independent; result again of class 
         \code{"AbscontDistribution"}; is generated by FFT}
\item{\code{+}}{\code{signature(e1 = "AbscontDistribution", e2 = "DiscreteDistribution")} assumes \code{e1}, \code{e2} independent; result again of class 
         \code{"AbscontDistribution"}; is generated by FFT}
\item{\code{+}}{\code{signature(e1 = "DiscreteDistribution", e2 = "AbscontDistribution")} assumes \code{e1}, \code{e2} independent; result again of class 
         \code{"AbscontDistribution"}; is generated by FFT}
\item{\code{+}}{\code{signature(e1 = "LatticeDistribution", e2 = "LatticeDistribution")} assumes \code{e1}, \code{e2} independent; 
if the larger lattice-width is an integer multiple of the smaller(in abs. value) one: result again of class 
         \code{"LatticeDistribution"}; is generated by D/FFT}
\item{\code{+}}{\code{signature(e1 = "DiscreteDistribution", e2 = "DiscreteDistribution")} assumes \code{e1}, \code{e2} independent; result again of class 
         \code{"DiscreteDistribution"}; is generated by explicite convolution}
\item{\code{+}}{\code{signature(e1 = "LatticeDistribution", e2 = "DiscreteDistribution")} assumes \code{e1}, \code{e2} independent; result again of class 
         \code{"DiscreteDistribution"}; is generated by explicite convolution}
\item{\code{+}}{\code{signature(e1 = "UnivarLebDecDistribution", e2 = "UnivarLebDecDistribution")} assumes \code{e1}, \code{e2} independent; result again of class 
         \code{"UnivarLebDecDistribution"}; is generated by separate explicite convolution of a.c. and discrete parts of \code{e1} and \code{e2}
         and subsequent flattening with \code{\link{flat.LCD}}; if \code{getdistrOption("withSimplify")} is  \code{TRUE}, result is piped
         through a call to \code{\link{simplifyD}}}
\item{\code{+}}{\code{signature(e1 = "AcDcLcDistribution", e2 = "AcDcLcDistribution")}: both operands are coerced
                to class \code{"UnivarLebDecDistribution"} and the corresponding method is used.
                }
\item{\code{+}}{\code{signature(e1 = "Binom", e2 = "Binom")} assumes \code{e1}, \code{e2} independent; 
if \code{prob(e1)==prob(e2)}, result again of class 
         \code{"Binom"}; uses the convolution formula for binomial distributions; exact}
\item{\code{+}}{\code{signature(e1 = "Cauchy", e2 = "Cauchy")}  assumes \code{e1}, \code{e2} independent; result again of class 
         \code{"Cauchy"}; uses the convolution formula for Cauchy distributions; exact}
\item{\code{+}}{\code{signature(e1 = "Chisq", e2 = "Chisq")}  assumes \code{e1}, \code{e2} independent; result again of class 
         \code{"Chisq"}; uses the convolution formula for Chisq distributions; exact}
\item{\code{+}}{\code{signature(e1 = "Dirac", e2 = "Dirac")} result again of class \code{"Dirac"}; exact}
\item{\code{+}}{\code{signature(e1 = "ExpOrGammaOrChisq", e2 = "ExpOrGammaOrChisq")}  assumes \code{e1}, \code{e2} independent; if 
         \code{e1}, \code{e2} are Gamma distributions, result is of class 
         \code{"Gammad"}; uses the convolution formula for Gamma distributions; exact}
\item{\code{+}}{\code{signature(e1 = "Pois", e2 = "Pois")}  assumes \code{e1}, \code{e2} independent; result again of class 
         \code{"Pois"}; uses the convolution formula for Poisson distributions; exact}
\item{\code{+}}{\code{signature(e1 = "Nbinom", e2 = "Nbinom")}  assumes \code{e1}, \code{e2} independent; if 
\code{prob(e1)==prob(e2)}, result again of class 
         \code{"Nbinom"}; uses the convolution formula for negative binomial distributions; exact}
\item{\code{+}}{\code{signature(e1 = "Norm", e2 = "Norm")}  assumes \code{e1}, \code{e2} independent; result again of class 
         \code{"Norm"}; uses the convolution formula for normal distributions; exact}
\item{\code{+}}{\code{signature(e1 = "UnivariateDistribution", e2 = "Dirac")}  translated to \code{e1 + location(e2)}; 
         result again of class \code{"Dirac"}; exact}
\item{\code{+}}{\code{signature(e1 = "Dirac", e2 = "UnivariateDistribution")}  translated to \code{e2 + location(e1)}; 
         result again of class \code{"Dirac"}; exact}
\item{\code{+}}{\code{signature(e1 = "Dirac", e2 = "DiscreteDistribution")}  translated to \code{e2 + location(e1)}; 
         result again of class \code{"Dirac"}; exact}
\item{\code{-}}{\code{signature(e1 = "Dirac", e2 = "Dirac")}  result again of class \code{"Dirac"}; exact}
\item{\code{*}}{\code{signature(e1 = "Dirac", e2 = "Dirac")}  result again of class \code{"Dirac"}; exact}
\item{\code{*}}{\code{signature(e1 = "UnivariateDistribution", e2 = "Dirac")}  translated to \code{e1 * location(e2)}; 
         result again of class \code{"Dirac"}; exact}
\item{\code{*}}{\code{signature(e1 = "Dirac", e2 = "UnivariateDistribution")}  translated to \code{e2 * location(e1)}; 
         result again of class \code{"Dirac"}; exact}
\item{\code{*}}{\code{signature(e1 = "AcDcLcDistribution", e2 = "AcDcLcDistribution")}: by means of \code{\link{decomposePM}}
                \code{e1} and \code{e2} are decomposed into positive and negative parts; of these, convolutions of the
                corresponding logarithms are computed separately and finally \code{exp} is applied to them, again separately;
                the resulting mixing components are then ``flattened'' to one object of class 
                \code{UnivarLebDecDistribution} by \code{\link{flat.LCD}} which according to \code{getdistrOption(withSimplify)} 
                gets piped through a call to \code{\link{simplifyD}}.
                }
\item{\code{/}}{\code{signature(e1 = "Dirac", e2 = "Dirac")}  result again of class \code{"Dirac"}; exact}
\item{\code{/}}{\code{signature(e1 = "numeric", e2 = "Dirac")}  result again of class \code{"Dirac"}; exact}
\item{\code{/}}{\code{signature(e1 = "numeric", e2 = "AcDcLcDistribution")}: if \code{d.discrete(e2)(0)*discreteWeight(e2)>0}
                throws an error (would give division by 0 with positive probability); else by means of \code{\link{decomposePM}}
                \code{e2} is decomposed into positive and negative parts; then, similarly the result obtains as for 
                \code{"*"(signature(e1 = "AcDcLcDistribution", e2 = "AcDcLcDistribution"))} by the exp-log trick
                and is ``flattened'' to one object of class 
                \code{UnivarLebDecDistribution} by \code{\link{flat.LCD}} and 
                according to \code{getdistrOption(withSimplify)} is piped through
                a call to \code{\link{simplifyD}}; exact..
                }
\item{\code{/}}{\code{signature(e1 = "AcDcLcDistribution", e2 = "AcDcLcDistribution")}: translated to \code{e1 * (1/e2)}.
                }
\item{\code{^}}{\code{signature(e1 = "AcDcLcDistribution", e2 = "Integer")}: if \code{e2=0} returns \code{Dirac(1)};
                if \code{e2=1} returns \code{e1}; if \code{e2<0} translated to \code{(1/e1)^(-e2)}; exact. 
                }
\item{\code{^}}{\code{signature(e1 = "AcDcLcDistribution", e2 = "numeric")}: if \code{e2} is integer uses preceding
                item; else if \code{e1< 0} with positive probability, throughs an error; else
                the result obtains similarly to 
                \code{"*"(signature(e1 = "AcDcLcDistribution", e2 = "AcDcLcDistribution"))} by the exp-log trick
                and is ``flattened'' to one object of class 
                \code{UnivarLebDecDistribution} by \code{\link{flat.LCD}} and 
                according to \code{getdistrOption(withSimplify)} is piped through
                a call to \code{\link{simplifyD}}; exact.
                }
\item{\code{^}}{\code{signature(e1 = "AcDcLcDistribution", e2 = "AcDcLcDistribution")}: 
                if \code{e1} is negative with positive probability, 
                throws an error if \code{e2} is non-integer
                with positive probability; if \code{e1} is 0 with positive probability
                throws an error if \code{e2} is non-integer with positive probability. 
                if  \code{e2} is integer with probability 1 uses 
                \code{DiscreteDistribution(supp=e1^(Dirac(x))} for each \code{x} in \code{support(e2)},
                builds up a corresponding mixing distribution; the latter is ``flattened'' to one object of class 
                \code{UnivarLebDecDistribution} by \code{\link{flat.LCD}} and 
                according to \code{getdistrOption(withSimplify)} is piped through
                a call to \code{\link{simplifyD}}.
                Else the result obtains similarly to \code{"*"(signature(e1 = "AcDcLcDistribution", 
                e2 = "AcDcLcDistribution"))} by the exp-log trick
                and is ``flattened'' to one object of class 
                \code{UnivarLebDecDistribution} by \code{\link{flat.LCD}} and 
                according to \code{getdistrOption(withSimplify)} is piped through
                a call to \code{\link{simplifyD}}; exact.
                }
\item{\code{^}}{\code{signature(e1 = "numeric", e2 = "AcDcLcDistribution")}: 
                if \code{e1} is negative, throws an error if \code{e2} is non-integer
                with positive probability; if \code{e1} is 0 throws an error if
                \code{e2} is non-integer with positive probability. 
                if  \code{e2} is integer with probability 1 uses 
                \code{DiscreteDistribution(supp=e1^support(e2), prob=discrete.d(supp))}
                else the result obtains similarly to \code{"*"(signature(e1 = "AcDcLcDistribution", 
                e2 = "AcDcLcDistribution"))} by the exp-log trick
                and is ``flattened'' to one object of class 
                \code{UnivarLebDecDistribution} by \code{\link{flat.LCD}} and 
                according to \code{getdistrOption(withSimplify)} is piped through
                a call to \code{\link{simplifyD}}; exact.
                }
}}

\seealso{
\code{\link{UnivariateDistribution-class}}
\code{\link{AbscontDistribution-class}} \cr
\code{\link{DiscreteDistribution-class}}
\code{\link{LatticeDistribution-class}}\cr
\code{\link{Norm-class}}
\code{\link{Binom-class}}
\code{\link{Pois-class}}
\code{\link{Dirac-class}}\cr
\code{\link{Cauchy-class}}
\code{\link{Gammad-class}}
\code{\link{Logis-class}}
\code{\link{Lnorm-class}}\cr
\code{\link{Exp-class}}
\code{\link{Weibull-class}}
\code{\link{Nbinom-class}}
}

\arguments{
  \item{e1,e2}{ objects of class \code{"UnivariateDistribution"} (or subclasses) or \code{"numeric"}}
}
\examples{
N <- Norm(0,3)
P <- Pois(4)
a <- 3
N + a
N + P
N - a
a * N
a * P
N / a + sin( a * P - N)
N * P
N / N
\donttest{
## takes a little time
N ^ P
}
1.2 ^ N
abs(N) ^ 1.3
}
\references{
Ruckdeschel, P., Kohl, M.(2014):
   General purpose convolution algorithm for distributions 
  in S4-Classes by means of FFT. \emph{J. Statist. Softw.}
  \bold{59}(4): 1-25.
}

\keyword{math}
\keyword{distribution}
\keyword{arith}
\concept{convolution}
\concept{FFT}
\concept{affine linear}
\concept{image distribution}

\name{RtoDPQ.LC}
\alias{RtoDPQ.LC}
\title{Default procedure to fill slots d,p,q given r for Lebesgue decomposed distributions}
\description{
  function to do get empirical density, cumulative distribution and quantile function from random numbers}
\usage{
RtoDPQ.LC(r, e = getdistrOption("RtoDPQ.e"), 
          n = getdistrOption("DefaultNrGridPoints"), y = NULL)
}
\arguments{
  \item{r}{ the random number generator }
  \item{e}{ \eqn{10^e} numbers are generated, a higher number leads to a better result. }
  \item{n}{ The number of grid points used to create the approximated functions, a higher number leads to a better result.}
  \item{y}{a (numeric) vector or \code{NULL}}
}
\details{
RtoDPQ.LC generates \eqn{10^e} random numbers, by default \deqn{e = RtoDPQ.e}. 
Replicates are assumed to be part of the discrete part, unique values to be
part of the a.c. part of the distribution. For the replicated ones,
we generate a discrete distribution by a call to \code{\link{DiscreteDistribution}}.

For the a.c. part, similarly to \code{\link{RtoDPQ}} we have an optional parameter \code{y}
for using N. Horbenko's quantile trick: i.e.; on an equally spaced grid \code{x.grid} on [0,1], apply
\code{f(q(x)(x.grid))}, write the result to \code{y} and use these
values instead of simulated ones.

The a.c. density is formed on the basis of \eqn{n}
points using approxfun and density (applied to the unique values), by default \deqn{n = DefaultNrGridPoints}.
The cumulative distribution function is based on all random variables, 
and, as well as the quantile function, is also created on the basis of \eqn{n} points using 
\code{approxfun} and \code{ecdf}.  Of course, the results are usually not exact as they rely on random numbers.}
\value{ \code{RtoDPQ.LC} returns an object of class \code{UnivarLebDecDistribution}.
}

\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}

\note{ Use \code{RtoDPQ} for absolutely continuous and \code{RtoDPQ.d} for discrete distributions. }

\seealso{ 
\code{\link{UnivariateDistribution-class}}, 
\code{\link{density}}, 
\code{\link{approxfun}}, 
\code{\link{ecdf}}}
\examples{
rn2 <- function(n)ifelse(rbinom(n,1,0.3),rnorm(n)^2,rbinom(n,4,.3))
x <- RtoDPQ.LC(r = rn2, e = 4, n = 512)
plot(x)
# returns density, cumulative distribution and quantile function of
# squared standard normal distribution
d.discrete(x)(4)
x2 <- RtoDPQ.LC(r = rn2, e = 5, n = 1024) # for a better result
plot(x2)
}
\keyword{math}
\keyword{distribution}
\keyword{arith}
\concept{random sample}
\concept{image distribution}
\concept{absolutely continuous distribution}
\concept{utility}

\name{internals_for_distr}
\alias{internals_for_distr}
\alias{.is.vector.lattice}
\alias{.is.consistent}
\alias{.isInteger}
\alias{.isNatural}
\alias{.isNatural0}
\alias{.mergegaps}
\alias{.mergegaps2}
\alias{.pmixfun}
\alias{.dmixfun}
\alias{.qmixfun}
\alias{.rmixfun}
\alias{.del0dmixfun}
\alias{.ULC.cast}
\alias{.loupmixfun}
\alias{.make.lattice.es.vector}
\alias{.presubs}
\alias{.inArgs}
\alias{.EuclidAlgo}
\alias{.getCommonWidth}
\alias{.isEqual}
\alias{.isIn}
\alias{.setEqual}
\alias{.isEqual01}
\alias{.makeD}
\alias{.makeP}
\alias{.makeQ}
\alias{.makeDNew}
\alias{.makePNew}
\alias{.makeQNew}
\alias{.makeDd}
\alias{.makePd}
\alias{.makeQd}
\alias{.makeQc}
\alias{.fM2}
\alias{.fM}
\alias{.fm}
\alias{.notwithLArg}
\alias{.multm}
\alias{.plusm}
\alias{.getObjName}
\alias{.discretizeP}
\alias{.expm.d}
\alias{.expm.c}
\alias{.logm.d}
\alias{.logm.c}
\alias{.P2D}
\alias{.P2Q}
\alias{.D2P}
\alias{.Q2P}
\alias{.csimpsum}
\alias{.primefun}
\alias{.IssueWarn}
\alias{.fillList}
\alias{.trunc.up}
\alias{.trunc.low}
\alias{.modifyqgaps}
\alias{.DistrCollapse}
\alias{.convDiscrDiscr}
\alias{.inWithTol}
\alias{.panel.mingle}
\alias{devNew}

\title{Internal functions of package distr}

\description{
These functions are used internally by package distr.}

\usage{
.is.vector.lattice(x)
.is.consistent(lattice, support, eq.space = TRUE)
.make.lattice.es.vector(x)
.inArgs(arg, fct)
.isEqual(p0, p1, tol = min( getdistrOption("TruncQuantile")/2,
                                          .Machine$double.eps^.7))
.isEqual01(x)
.isIn(p0, pmat, tol = min( getdistrOption("TruncQuantile")/2,
                                          .Machine$double.eps^.7
                                          ))
.isInteger(x, tol = .Machine$double.eps)
.isNatural(x, tol = .Machine$double.eps)
.isNatural0(x, tol = .Machine$double.eps)
.setEqual(x, y, tol = 1e-7)
.presubs(inp, frompat, topat)
.makeD(object, argList,  stand = NULL, fac = NULL)
.makeP(object, argList,  sign = TRUE, correct = NULL, fac =
                 NULL, fac2 = NULL)
.makeQ(object, lastCall, sign = TRUE, Cont = TRUE)
.plusm(e1, e2, Dclass = "DiscreteDistribution")
.multm(e1, e2, Dclass = "DiscreteDistribution")
.notwithLArg(D)
.getObjName(i = 1)
.discretizeP(D, lower, upper, h)
.fm(x,f)
.fM(x,f)
.fM2(x,f)
.makeDd(x,y, yleft, yright)
.makePd(x,y, yleft, yright)
.makeQd(x,y, yleft, yright)
.makeQc(x,y, yleft, yright)
.makeDNew(x, dx, h = NULL, Cont = TRUE, standM = "sum")
.makePNew(x, dx, h = NULL, notwithLLarg = FALSE,
                      Cont = TRUE, myPf = NULL, pxl = NULL, pxu = NULL)
.makeQNew(x, px.l, px.u, notwithLLarg = FALSE, yL , yR, Cont = TRUE)
.mergegaps(gaps, support)
.mergegaps2(gaps1, gaps2)
.consolidategaps(gaps)
.pmixfun(mixDistr, mixCoeff, leftright = "right")
.dmixfun(mixDistr, mixCoeff, withStand = FALSE, supp = NULL)
.rmixfun(mixDistr, mixCoeff)
.qmixfun(mixDistr, mixCoeff, Cont = TRUE, pnew, gaps = NULL, leftright = "left")
.del0dmixfun(mixDistr)
.loupmixfun(mixDistr)
.ULC.cast(x)
.expm.d(e1)
.expm.c(e1)
.logm.d(e1)
.logm.c(e1)
.P2D (p, xx, ql, qu, ngrid = getdistrOption("DefaultNrGridPoints"))
.P2Q (p, xx, ql,qu, ngrid = getdistrOption("DefaultNrGridPoints"), 
                qL = -Inf, qU = Inf)
.D2P (d, xx, ql, qu,  ngrid = getdistrOption("DefaultNrGridPoints"))
.Q2P (q, ngrid = getdistrOption("DefaultNrGridPoints"))
.csimpsum(fx)
.primefun(f,x, nm = NULL)
.IssueWarn(Arith,Sim)
.List(list0)
.fillList(list0, len=length(list0))
.trunc.up(object, upper)
.trunc.low(object, lower)
.modifyqgaps(pfun, qfun, gaps, leftright = "left")
.DistrCollapse(support, prob, eps = getdistrOption("DistrResolution"))
.EuclidAlgo(n1,n2)
.getCommonWidth(x1,x2, tol=.Machine$double.eps)
.convDiscrDiscr(e1,e2)
.inWithTol(x,y,tol=.Machine$double.eps)
.panel.mingle(dots,element)
devNew(...)
}


\arguments{
  \item{x}{a (numeric) vector, or (in case of \code{.ULC.cast}) an object of class \code{"AcDcLcDistribution"}}
  \item{y}{a (numeric) vector}
  \item{f}{in function \code{.primefun}: a function in one (numeric) argument; 
           in functions \code{.fm}, \code{.fM}, \code{.fM2} a vector of function evaluations}
  \item{lattice}{a lattice (of class \code{Lattice})}
  \item{support}{a support vector / support vector of a univariate discrete distribution}
  \item{eq.space}{logical: shall we check for the support to be equally spaced?} 
  \item{arg}{a formal argument as character}
  \item{fct}{a function}
  \item{p0,p1}{(numeric) vectors} 
  \item{pmat}{(matrix) a matrix with two columns where row-wise the left column
              is smaller than the right one} 
  \item{tol}{an error tolerance (numeric)}
  \item{e1}{a distribution object}
  \item{e2}{a numeric}
  \item{object}{a distribution object}  
  \item{argList}{an (unevaluated) list of arguments passed to \code{m(object)}  
        where  \code{m} is in \code{d,p,q}}
  \item{stand}{factor for a (Lebesgue) density to integrate to 1}
  \item{sign}{the sign of the second operand --- for multiplication at 
               the moment}
  \item{correct}{unevaluated R-code to correct for right-continuity 
                  (for multiplication with negative numerics at the moment)}
  \item{fac}{factor to be multiplied with the return value}
  \item{fac2}{factor to be added to the return value}
  \item{lastCall}{unevaluated R-Code ---gives how the result of a call to 
                   \code{q(e1)} is further transformed}
  \item{Cont}{logical: \code{TRUE} if \code{object} is continuous}
  \item{DClass}{character: name of distribution class}
  \item{D}{a distribution object}
  \item{i}{an integer}
  \item{yleft, yright}{extrapolation value beyond left/right endpoint of grid}
  \item{h}{numeric: grid width}
  \item{standM}{standardization method --- summation or integration}
  \item{notwithLLarg}{logical --- can we use \code{log.p}, \code{lower.tail} 
  arguments for \code{p,q}-methods of first operand?}
  \item{dx}{numeric: vector of cell-probabilities for the (discretized) 
  distribution}
  \item{myPf}{function with args \code{x,y, yleft, yright} 
  (as \code{approxfun}): 
              if given: replaces \code{approxfun} as interpolation method for 
              continuos distributions}
  \item{pxl,pxu}{numeric: if given vector of (lower/upper) cumulative 
  probabilities}
  \item{yL, yR}{argmin / argmax of p()-method}
  \item{inp}{either a language object or a character vector}
  \item{frompat}{vector of character strings containing regular expressions (or
          character string for \code{fixed = TRUE}) to be matched in the
          given character vector.  Coerced by \code{as.character} to a
          character string if possible; (as argument \code{pattern} in 
          \code{\link{gsub}} --- but possibly of length >1).}
  \item{topat}{ a (vector of) replacement(s) for matched pattern in 
          \code{.presubs}. 
          Coerced to character if possible.  For \code{fixed = FALSE} this
          can include backreferences \code{"\\1"} to \code{"\\9"} to 
          parenthesized subexpressions of \code{pattern}.  For 
          \code{perl = TRUE} only, it can
          also contain \code{"\\U"} or \code{"\\L"} to convert the rest of the
          replacement to upper or lower case; (as argument \code{replacement} 
          in \code{\link{gsub}}--- but possibly of length >1).}
 \item{gaps,gaps1,gaps2}{matrices \code{m} with two columns, 
             such that \code{t(m)}, interpreted as vector, is ordered}         
 \item{prob}{probability vector for a univariate discrete distribution}         
 \item{mixDistr}{an object of class \code{UnivarDistrList}}
 \item{mixCoeff}{an object of class \code{numeric}; a probability vector}
 \item{pnew}{a function \code{function(q, lower.tail = TRUE, log.p = FALSE}
 realizing slot \code{p} in a distribution object.}
 \item{withStand}{logical; if \code{TRUE} a standardization is made such
 that the sum of the values of the result evaluated at argument \code{supp} is 1}
 \item{supp}{NULL or \code{numeric}; if \code{withStand} is \code{TRUE} used
 to standardize such that the result is a probability density.}
 \item{p,pfun}{slot \code{p} of an object of class \code{"AbscontDistribution"}} 
 \item{d}{slot \code{d} of an object of class \code{"AbscontDistribution"}} 
 \item{q,qfun}{slot \code{q} of an object of class \code{"AbscontDistribution"}} 
 \item{xx}{a given grid of x-values for functions \code{p}, \code{d} to be
 evaluated at}  
 \item{ql,qu}{lower and upper \code{getdistrOption("TruncQuantile")}-quantile of
 the distribution; also, if argument \code{xx} is missing, left and right endpoint
 of a regular grid of \code{ngrid} gridpoints to be used in place of \code{xx}.}  
 \item{qL,qU}{argmin / argmax of p()-method}  
 \item{ngrid}{number of gridpoints}
 \item{fx}{a vector of function evaluations multiplied by the gridwidth}
 \item{nm}{an optional right asymptotic value}
 \item{Arith}{logical; slot \code{.withArith} of a distribution object,
              or logically-``any'' of these slots in a collection of such objects} 
 \item{Sim}{logical; slot \code{.withSim} of a distribution object,
              or logically-``any'' of these slots in a collection of such objects} 
 \item{list0}{list, the elements of which are to be copied to a new list using
              recycling if necessary}
 \item{len}{length of the list to be filled}              
 \item{lower}{lower truncation point}
 \item{upper}{upper truncation point}
 \item{leftright}{character; for slot \code{q}: if partially matched to \code{"right"} 
                  function will return the right continuous version, else
                  the left continuous version; for slot \code{p}: if partially 
                  matched to \code{"left"} the left continuous version, else
                  the right continuous version;}
 \item{n1}{integer argument for \code{.EuclidAlgo}}
 \item{n2}{integer argument for \code{.EuclidAlgo}}
 \item{x1}{width argument for \code{.getCommonWidth}}
 \item{x2}{width argument for \code{.getCommonWidth}}
 \item{dots}{the unevaluated \code{\dots} argument}
 \item{element}{the name of the item in the unevaluated \code{\dots} argument}
 \item{...}{arguments passed through to other functions}
}

\details{
\code{.is.vector.lattice} checks whether a given vector \code{x} is equally 
spaced.
\code{.is.consistent} checks whether a given support vector \code{support} is 
consistent to a given lattice \code{lattice} --- with or without checking
if \code{support} is equally spaced.  \code{.make.lattice.es.vector} 
makes an object of class \code{Lattice} out of a given (equally spaced) vector 
\code{x}.

\code{.inArgs} checks whether an argument \code{arg} is a formal argument of 
     \code{fct} --- not vectorized.

\code{.isEqual} checks whether \code{p0} and \code{p1} are equal to given
      tolerance.
\code{.isIn} checks whether \code{p0} lies in any of the intervals given by
     matrix \code{pmat} to given tolerance.
\code{.isEqual01}(x) checks whether \code{x} is 0 or 1 to given tolerance. 
\code{.setEqual} sets all elements of x which are equal to some element of y 
         up to tolerance tol, to exactly the respective element of y. 
         
\code{.notwithLArg} checks whether object \code{D} was generated by simulations 
or if its slots \code{p,q} do not have \code{lower.tail} arguments.

\code{.getObjName} returns the name of the object in the \code{i}th operand.
\code{.discretizeP} discretizes \code{D} to a grid of probabilities from 
     \code{lower} to \code{upper} with width \code{h}.   

\code{.fm}, \code{.fM} return the smallest / biggest value in (0,1) such that
  \code{f}(x) is finite; \code{.fM2} is a variant of \code{.fM} using a 
  \code{lower.tail = FALSE} argument.

\code{.makeD}, \code{.makeP}, \code{.makeQ} generate slots \code{p,d,q} for
    binary operations \code{e1 /op/ e2} for a distribution object \code{e1} 
    and a numeric \code{e2} ---for the moment only \code{/op/}'s 
     \code{+,-,*,/} are implemented.

\code{.plusm}, \code{.multm} more specifically use \code{.makeD}, \code{.makeP}, 
\code{.makeQ} to generate slots \code{p,d,q} for \code{+}, \code{*}, 
      respectively. 


\code{.makeDd}, \code{.makePd}, \code{.makeQd} provide discrete analogues to 
\code{approxfun} for interpolation at non grid-values 

\code{.makeQc} is an analogue to \code{makeQd} for absolutely continuous 
distributions using \code{approxfun}. 

\code{.makeDNew} generates slot \code{d} for a new distribution object.
In case of a discrete distribution it produces a step function with 
\code{stepfun} (using \code{.makeDd}) and standardizes to 1 by summation.
In case of a continuous distribution it produces a density function with 
\code{approxfun} and standardizes to 1 by integration if the latter fails,
  it uses a trapezoid rule / summation for this purpose.

\code{.makePNew} generates slot \code{p} for a new distribution object.
In case of a discrete distribution it produces a step function from 
\code{cumsum} applied to \code{dx} ---or from \code{pxl} if this is given, with 
\code{stepfun} (using \code{.makePd}). 
In case of a continuous distribution it produces a cdf with 
\code{approxfun}. In case of \code{RtoDPQ}, \code{approxfun} is replaced by
\code{myPf} which calls \code{ecdf} directly.

\code{.makeQNew} generates slot \code{q} for a new distribution object.
In case of a discrete distribution it produces a step function 
(using \code{.makeQd}). Special care is taken for left continuity...
In case of a continuous distribution it produces a quantile function with 
\code{approxfun}. 

\code{.isInteger}, \code{.isNatural}, and \code{.isNatural0} test for each 
coordinate of argument \code{x} whether it
is integer [natural / natural or 0] or not.

\code{.mergegaps} modifies the gaps matrix of an a.c. distribution according to
the support slot of a discrete distribution; if necessary, a gap interval
[a,b] is split into [a,c],[c,b] if \code{a<c<b}.
\code{.mergegaps2} merges two gap matrices of two a.c. distributions \code{X1} 
and \code{X2} such that in the intervals of the resulting gap matrix, 
neither \code{X1} nor \code{X2} carries mass.
\code{.consolidategaps} consolidates a gap matrix, i.e. joins adjacent 
gap intervals.  

\code{.pmixfun}, \code{.dmixfun}, \code{.rmixfun}, and \code{.qmixfun}
fill the slots \code{p}, \code{d}, \code{r}, and \code{q}
of a corresponding mixing distribution according to the arguments
in \code{mixDistr}, \code{mixCoeff}.

\code{.loupmixfun} finds commun lower and upper bounds for the support of
the mixing distribution.

\code{.del0dmixfun} sets (if slot \code{d.ac} is not \code{NULL}) the return
value of slot function \code{d.ac} of \code{mixDistr} 
for argument \code{0} to \code{0}.

\code{.ULC.cast} coerces an object of class \code{"AcDcLcDistribution"} to
class \code{"UnivarLebDecDistribution"}, using \code{simplifyD}.


\code{.expm.d,.expm.c} for discrete, resp. a.c. argument \code{e1} fill the 
slots \code{p}, \code{d}, \code{r}, and \code{q}
of the transformation \code{exp(e1)} exactly.
\code{.logm.d,.logm.c} for discrete, resp. a.c. argument \code{e1} fill the 
slots \code{p}, \code{d}, \code{r}, and \code{q}
of the transformation \code{log(e1)} exactly.

For objects of class \code{\link{AbscontDistribution}},
\code{.P2D} and \code{.P2Q} reconstruct function slots \code{d} resp. 
\code{q} from function slot \code{p}
by means of function \code{D1ss} from package \pkg{sfsmisc};
and of function \code{.makeQNew},  respectively. The other way round, 
\code{.D2P} and \code{.Q2P} reconstruct function slot \code{p} from 
from function slots  \code{d} resp. \code{q} 
by means of function \code{.makePNew} and explicite numeric inversion,  
respectively.

\code{.csimpsum} is used internally in \code{.makePNew} to produce
a primitive function out of function evaluations by means of vectorized
Simpson quadrature method, returning already the function values
of the prime function on a grid; it is to mimick the behaviour
of \code{cumsum}. \code{.primefun} is similar but more flexible and
produces the prime function as a function.

\code{.List} checks if argument already is a list, and if so leaves it as
             it is, otherwise casts it to a list by a call to \code{list}.

\code{.fillList} fills a new list with the elements of a given list \code{list0}
                 until length \code{len} is reached using recycling if necessary.
                 Argument \code{list0} is cast to \code{list} by a call
                 to \code{.List} if necessary.

\code{.trunc.up}, \code{.trunc.low}  provide common routines for
classes \code{DiscreteDistribution} and \code{AbscontDistribution} for 
one-sided truncation, using (for slot \code{r}) Peter Dalgaard's clever 
log-tricks as indicated in 
\url{https://stat.ethz.ch/pipermail/r-help/2008-September/174321.html}.

\code{.modifyqgaps} modifies slot \code{q} for objects of class 
     \code{AbscontDistribution} in the presence of gaps, i.e.; if slot
     \code{gaps} is not \code{NULL}. If argument \code{leftright} does not
     partially match \code{"right"} (default) returns the left continuous
     version of the quantile function, else the right continuous one.
 
\code{.EuclidAlgo} computes the greatest common divisor of two integers by 
      means of the Euclidean algorithm.
\code{.getCommonWidth} for two lattices with widths \code{x1} and \code{x2}
      computes the smallest common lattice width for convolution.
\code{.convDiscrDiscr} computes the convolution of two discrete distributions by
    brute force.
\code{.inWithTol} works like \code{\%in\%} but with a given tolerance.

\code{.panel.mingle} is used for mingling arguments \code{panel.first},
\code{panel.last} in a plot; it returns the evaluated argument \code{element}
within dots, if it is a symbol; else if it can be interpreted as a call, and if
the top call is \code{list}, it returns a list of the items of the call to \code{list},
unevaluated, and otherwise the unchanged argument.

\code{devNew} opens a new device. This function is for back compatibility
with R versions < 2.8.0. To control the number of opened devices, when
\code{length(dev.list())>20}, in interactive mode we ask the user to shut
some windows until \code{length(dev.list())<=20}; in non-interactive mode
we shut the first 15 open devices (except for the first one) before opening
a new one.
}


\value{
\item{.is.vector.lattice}{\code{logical} (length 1).}
\item{.is.consistent}{\code{logical} (length 1).}
\item{.notwithLArg}{\code{logical} (length 1).}
\item{.make.lattice.es.vector}{an object of class \code{Lattice}.}
\item{.inArgs}{\code{logical} (length 1).}
\item{.isIn, .isEqual,.isEqual01}{vector of \code{logical}.}
\item{.fm,.fM, .fM2}{a \code{numeric} of length 1.}
\item{.plusm,.multm}{an object of class \code{DiscreteDistribution} or 
       \code{AbscontDistribution} according to argument \code{DClass}.}
\item{.getObjName}{\code{character}.}
\item{.discretizeP}{\code{numeric} --- the probabilities for the grid-values.}
\item{.makeDd,.makePd, .makeQd}{a function with args 
       \code{x, y, yleft, yright}.}
\item{.makeD,.makeDNew}{a function with args \code{x, log = FALSE}.}
\item{.makeP,.makePNew}{a function with args \code{q, lower.tail = TRUE, 
       log.p = FALSE}.}
\item{.makeQ,.makeQNew}{a function with args \code{p, lower.tail = TRUE, 
       log.p = FALSE}.}
\item{.isInteger,.isNatural,.isNatural0}{\code{logical} (same length as argument \code{x}).}
\item{.mergegaps,.mergegaps2}{a \code{gaps}-matrix, i.e.; a matrix \code{m} with two columns, 
              such that \code{t(m)}, interpreted as vector, is ordered.}
\item{.pmixfun}{slot \code{p} for a mixing distribution, i.e. a function 
                \code{function(q, lower.tail = TRUE, log.p = FALSE)}, which
                is  the cdf of the distribution.}
\item{.dmixfun}{slot \code{d} for a mixing distribution, i.e. a function 
                \code{function(x, log = FALSE)}, which
                is  the density of the distribution.}
\item{.qmixfun}{slot \code{q} for a mixing distribution, i.e. a function 
                \code{function(p, lower.tail = TRUE, log.p = FALSE)}, which
                is  the quantile function of the distribution.}
\item{.rmixfun}{slot \code{r} for a mixing distribution, i.e. a function 
                \code{function(n)} generating r.v.'s according to the distribution.}
\item{.deldmixfun}{a possibly modified argument \code{mixDistr}.}
\item{.loupmixfun}{a list of four components: \code{qL}, the minimal value of 
\code{q(x)(0)}, \code{ql}, the minimal value of 
\code{q(x)(getdistrOption("TruncQuantile"))}, \code{qU}, the maximal value of 
\code{q(x)(1)}, \code{qu}, the maximal value of 
\code{q(x)(getdistrOption("TruncQuantile"), lower.tail = FALSE)},
\code{x} running through the members of \code{mixDistr} in each case.}
\item{.ULC.cast}{an object of class \code{"UnivarLebDecDistribution"}.}
\item{.expm.d,.logm.d}{an object of class \code{"DiscreteDistribution"}.} 
\item{.expm.c,.logm.c}{an object of class \code{"AbscontDistribution"}.} 
\item{.P2D}{a density \code{d} as function \code{function(x,  log = FALSE)}.}
\item{.P2Q}{a quantile function \code{q} as function \code{function(p, 
            lower.tail = TRUE, log.p = FALSE)}} 
\item{.D2P, .Q2P}{a cdf \code{p} as function \code{function(q, 
            lower.tail = TRUE, log.p = FALSE)}.}
\item{.csimpsum}{a vector of evaluations of the prime function at the grid points.}
\item{.primefun}{the prime function as a function.}
\item{.IssueWarn}{a list with two warnings to be issued each of which may be empty.}
\item{.List}{a list.}
\item{.fillList}{a list.}
\item{.trunc.up,.trunc.low}{a list with elements \code{r,p,d,q} (in this order).}
\item{.DistrCollapse}{upon a suggestion by Jacob van Etten, 
       \email{jacobvanetten@yahoo.com}: help function to collapse the support 
       points of a discrete distributions if they are too close to each other;
       here argument \code{support} is the (original; already sorted) support
       and \code{prob} a corresponding probability vector of same length.
       Criterium for collapsing: a distance smaller than argument
       \code{eps}.
       }
\item{.EuclidAlgo}{returns the greatest common divisor (an integer).}
\item{.getCommonWidth}{returns the  smallest common lattice width (a numeric).}
\item{.convDiscrDiscr}{returns the convolution of two discrete distributions.}
\item{.inWithTol}{returns a logical vector of same lenght as \code{x} for the
    matches (up to tolerance) with vector \code{y}.}
\item{.panel.mingle}{used for mingling arguments \code{panel.first},
\code{panel.last}; returns the evaluated argument \code{element} within dots,
if it is a symbol; else if it can be interpreted as a call, and if the top
call is \code{list}, it returns a list of the items of the call to \code{list},
unevaluated, and otherwise the unchanged argument.}
\item{devNew}{returns the return value of the device opened,
usually invisible \code{NULL}.}
}

\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
}

\seealso{
\code{\link{AbscontDistribution}},
\code{\link{DiscreteDistribution}},
\code{\link{LatticeDistribution}},
\code{\link{RtoDPQ}},
\code{\link{RtoDPQ.d}},
\code{\link{convpow}},
\code{\link{operators}},
\code{\link{plot-methods}}
\code{\link[grDevices:dev]{dev.new}}
}
\keyword{internal}
\concept{utilities}

\name{shape-methods}
\docType{methods}
\alias{shape-methods}
\alias{shape<--methods}
\alias{shape}
\alias{shape<-}
\alias{shape,GammaParameter-method}
\alias{shape,Gammad-method}
\alias{shape,Chisq-method}
\alias{shape,Exp-method}
\alias{shape,WeibullParameter-method}
\alias{shape,Weibull-method}
\alias{shape<-,GammaParameter-method}
\alias{shape<-,Gammad-method}
\alias{shape<-,WeibullParameter-method}
\alias{shape<-,Weibull-method}
\title{ Methods for Function shape in Package `distr' }
\description{shape-methods}

\section{Methods}{\describe{

\item{shape}{\code{signature(object  = "GammaParameter")}: returns the slot \code{shape} of a parameter of a Gamma distribution }
\item{shape<-}{\code{signature(object = "GammaParameter")}: modifies the slot \code{shape} of a parameter of a Gamma distribution }

\item{shape}{\code{signature(object = "Gammad")}: returns the slot \code{shape} of the parameter slot of a Gamma distribution }
\item{shape<-}{\code{signature(object = "Gammad")}: modifies the slot shape of the parameter slot of a Gamma distribution }

\item{shape}{\code{signature(object  = "WeibullParameter")}: returns the slot \code{shape} of a parameter of a Weibull distribution }
\item{shape<-}{\code{signature(object = "WeibullParameter")}: modifies the slot \code{shape} of a parameter of a Weibull distribution }

\item{shape}{\code{signature(object  = "Weibull")}: returns the slot \code{shape} of the parameter slot of the distribution }
\item{shape<-}{\code{signature(object = "Weibull")}: modifies the slot \code{shape} of the parameter slot of the distribution }

\item{shape}{\code{signature(object = "Chisq")}: if \code{ncp(object)} is 0, a Chi-squared distribution is also a Gamma distribution;
     in this case, \code{shape} returns \code{df(object)/2} else an error;}

\item{shape}{\code{signature(object  = "Exp")}: returns the slot \code{shape} of the parameter slot of the Exp distribution  (=1)}
}}
\keyword{distribution}
\keyword{methods}
\concept{shape}
\concept{parameter}
\concept{Gamma distribution}
\concept{Chi square distribution}
\concept{Exp distribution}
\concept{Weibull distribution}
\concept{accessor function}
\concept{replacement function}

\name{convpow-methods}
\docType{methods}
\alias{convpow}
\alias{convpow-methods}
\alias{convpow,AcDcLcDistribution-method}
\alias{convpow,AbscontDistribution-method}
\alias{convpow,LatticeDistribution-method}
\alias{convpow,DiscreteDistribution-method}
\alias{convpow,AcDcLcDistribution-method}
\alias{convpow,Norm-method}
\alias{convpow,Binom-method}
\alias{convpow,Nbinom-method}
\alias{convpow,ExpOrGammaOrChisq-method}
\alias{convpow,Cauchy-method}
\alias{convpow,Pois-method}
\alias{convpow,Dirac-method}

\title{Distribution of the sum of univariate i.i.d r.v's}
\description{
  Method \code{convpow} determines the distribution of the sum of N univariate
  i.i.d r.v's by means of DFT}


\usage{
  convpow(D1,...)
  \S4method{convpow}{AbscontDistribution}(D1,N)
  \S4method{convpow}{LatticeDistribution}(D1,N, 
                     ep = getdistrOption("TruncQuantile"))
  \S4method{convpow}{DiscreteDistribution}(D1,N)
  \S4method{convpow}{AcDcLcDistribution}(D1,N, 
                     ep = getdistrOption("TruncQuantile"))
}

\arguments{
  \item{D1}{an object of (a sub)class (of) \code{"AbscontDistribution"} or  
            \code{"LatticeDistribution"} or of \code{"UnivarLebDecDistribution"}}
  \item{\dots}{not yet used; meanwhile takes up N }
  \item{N}{ an integer or 0 (for 0 returns Dirac(0), for 1 D1)}
  \item{ep}{ numeric of length 1 in (0,1) ---
             for \code{"LatticeDistribution"}: support points will be 
             cancelled if their probability is less than \code{ep};
             for \code{"UnivarLebDecDistribution"}: if \code{(acWeight(object)<ep)} 
             we work with the discrete parts only, and, similarly, if
             \code{(discreteWeight(object)<ep)} we with the absolutely continuous 
             parts only.}
  }
\details{
  in the methods implemented a second argument \code{N} is obligatory;
  the general methods use a general purpose convolution algorithm for 
  distributions by means of D/FFT. In case of an argument of class 
  \code{"UnivarLebDecDistribution"}, the result will in generally be
   again of class \code{"UnivarLebDecDistribution"}. However, if \code{acWeight(D1)}
   is positive, \code{discreteWeight(convpow(D1,N))} will decay exponentially
   in \code{N}, hence from some (small) \eqn{N_0} on, the result will be of
   class \code{"AbscontDistribution"}. This is used algorithmically, too, as 
   then only the a.c. part needs to be convolved.
   In case of an argument \code{D1} of class \code{"DiscreteDistribution"},
   for \code{N} equal to 0,1 we return the obvious solutions, and for \code{N==2} 
   the return value is \code{D1+D1}. For \code{N>2}, we split up \code{N} into
   \code{N=N1+N2}, \code{N1=floor(N/2)} and recursively return
   \code{convpow(D1,N1)+convpow(D1,N2)}.
   }

\value{Object of class \code{"AbscontDistribution"}, \code{"DiscreteDistribution"},
       \code{"LatticeDistribution"} resp. \code{"AcDcLcDistribution"}}

\section{further S4-Methods}{
  There are particular methods for the following classes, using explicit 
  convolution formulae:
  \describe{
    \item{\code{signature(D1="Norm")}}{returns class \code{"Norm"}}
    \item{\code{signature(D1="Nbinom")}}{returns class \code{"Nbinom"}}
    \item{\code{signature(D1="Binom")}}{returns class \code{"Binom"}}
    \item{\code{signature(D1="Cauchy")}}{returns class \code{"Cauchy"}}
    \item{\code{signature(D1="ExpOrGammaOrChisq")}}{returns class 
         \code{"Gammad"} ---if \code{D1} may be coerced to \code{Gammad}}
    \item{\code{signature(D1="Pois")}}{returns class \code{"Pois"}}
    \item{\code{signature(D1="Dirac")}}{returns class \code{"Dirac"}}
  }
}

\author{Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}\cr
Matthias Kohl \email{matthias.kohl@stamats.de}
Thomas Stabla \email{statho3@web.de}
}
%\note{}
\seealso{\code{\link{operators}}, \code{distrARITH()}}

\examples{
convpow(Exp()+Pois(),4)
}

\references{
Kohl, M., Ruckdeschel, P., (2014):
   General purpose convolution algorithm for distributions 
  in S4-Classes by means of FFT. \emph{J. Statist. Softw.}
  \bold{59}(4): 1-25.
}

\concept{convolution for distributions}
\concept{arithmetics for distributions}
\concept{info file}
\concept{FFT}
\concept{DFT}
\keyword{distribution}

\name{EmpiricalDistribution}
\alias{EmpiricalDistribution}

\title{Generating function "EmpiricalDistribution"}
\description{Generates an object of class  \code{"DiscreteDistribution"}}

\usage{
  EmpiricalDistribution(data, .withArith=FALSE, .withSim=FALSE, 
                        .lowerExact = TRUE, .logExact = FALSE,
                        .DistrCollapse = getdistrOption("DistrCollapse"),
                        .DistrCollapse.Unique.Warn = 
                             getdistrOption("DistrCollapse.Unique.Warn"),
                        .DistrResolution = getdistrOption("DistrResolution"),
                        Symmetry = NoSymmetry())
}
\arguments{
  \item{data}{numeric vector with data. }
  \item{.withArith}{normally not set by the user, but if determining the entries \code{supp}, \code{prob}
                    distributional arithmetics was involved, you may set this to \code{TRUE}.}
  \item{.withSim}{normally not set by the user, but if determining the entries \code{supp}, \code{prob}
                  simulations were involved, you may set this to \code{TRUE}.}
  \item{.lowerExact}{normally not set by the user: whether the \code{lower.tail=FALSE}
                     part is calculated exactly, avoing a ``\code{1-.}''.}
  \item{.logExact}{normally not set by the user: whether in determining slots \code{d,p,q},
         we make particular use of a logarithmic representation to enhance accuracy.}
\item{.DistrCollapse}{controls whether in generating a new discrete 
     distribution, support points closer together than \code{.DistrResolution} are
     collapsed.}
\item{.DistrCollapse.Unique.Warn}{controls whether there is a warning
      whenever collapsing occurs or when two points are collapsed by a call to
      \code{unique()} (default behaviour if \code{.DistrCollapse} is \code{FALSE})}
\item{.DistrResolution}{minimal spacing between two mass points in a discrete 
                        distribution}
\item{Symmetry}{you may help \R in calculations if you tell it whether
   the distribution is non-symmetric (default) or symmetric with respect
   to a center; in this case use \code{Symmetry=SphericalSymmetry(center)}.}
}
\details{
  The function is a simple utility function providing a wrapper to the 
  generating function \code{\link{DiscreteDistribution}}.

  Typical usage is
  \preformatted{
    EmpiricalDistribution(data)
  }
  
}
\value{Object of class \code{"DiscreteDistribution"}}


\author{
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
}


\seealso{
\code{\link{DiscreteDistribution}}
\code{\link{DiscreteDistribution-class}}
}
\examples{
x <- rnorm(20)
D1 <- EmpiricalDistribution(data = x)
D1

plot(D1)
}
\keyword{distribution}
\concept{empirical distribution}
\concept{generating function}

\name{plot-methods}
\docType{methods}
\alias{plot}
\alias{plot-methods}
\alias{plot,AffLinUnivarLebDecDistribution,missing-method}
\alias{plot,UnivarLebDecDistribution,missing-method}
\alias{plot,AbscontDistribution,missing-method}
\alias{plot,DiscreteDistribution,missing-method}
\alias{plot,CompoundDistribution,missing-method}
\alias{plot,DistrList,missing-method}
\title{ Methods for Function plot in Package `distr' }
\description{plot-methods}
\usage{plot(x, y, ...)
\S4method{plot}{AbscontDistribution,missing}(x, width = 10, height = 5.5,
     withSweave = getdistrOption("withSweave"), xlim = NULL, ylim = NULL,
     ngrid = 1000, verticals = TRUE, do.points = TRUE, main = FALSE,
     inner = TRUE, sub = FALSE, bmar = par("mar")[1], tmar = par("mar")[3], ...,
     cex.main = par("cex.main"), cex.inner = 1.2, cex.sub = par("cex.sub"), 
     col.points = par("col"), col.vert = par("col"), col.main = par("col.main"),  
     col.inner = par("col.main"), col.sub = par("col.sub"), cex.points = 2.0, 
     pch.u = 21, pch.a = 16, mfColRow = TRUE,
     to.draw.arg = NULL, withSubst = TRUE)
\S4method{plot}{DiscreteDistribution,missing}(x, width = 10, height = 5.5,
     withSweave = getdistrOption("withSweave"), xlim = NULL, ylim = NULL,
     verticals = TRUE, do.points = TRUE, main = FALSE, inner = TRUE, sub = FALSE,
     bmar = par("mar")[1], tmar = par("mar")[3], ..., 
     cex.main = par("cex.main"), cex.inner = 1.2, cex.sub = par("cex.sub"), 
     col.points = par("col"), col.hor = par("col"), col.vert = par("col"), 
     col.main = par("col.main"), col.inner = par("col.main"), 
     col.sub = par("col.sub"),  cex.points = 2.0, pch.u = 21, pch.a = 16,
     mfColRow = TRUE, to.draw.arg = NULL, withSubst = TRUE)
\S4method{plot}{AffLinUnivarLebDecDistribution,missing}(x, width = 10, 
     height = 5.5, withSweave = getdistrOption("withSweave"), xlim = NULL,
     ylim = NULL, ngrid = 1000, verticals = TRUE, do.points = TRUE, main = FALSE,
     inner = TRUE, sub = FALSE, bmar = par("mar")[1], tmar = par("mar")[3], ...,
     cex.main = par("cex.main"), cex.inner = 1.2, cex.sub = par("cex.sub"),
     col.points = par("col"), col.hor = par("col"), col.vert = par("col"),
     col.main = par("col.main"), col.inner = par("col.main"),
     col.sub = par("col.sub"),  cex.points = 2.0, pch.u = 21, pch.a = 16,
     mfColRow = TRUE, to.draw.arg = NULL, withSubst = TRUE)
\S4method{plot}{UnivarLebDecDistribution,missing}(x, width = 10, 
     height = 14.5, withSweave = getdistrOption("withSweave"), xlim = NULL,
     ylim = NULL, ngrid = 1000, verticals = TRUE, do.points = TRUE, main = FALSE,
     inner = TRUE, sub = FALSE, bmar = par("mar")[1], tmar = par("mar")[3], ...,
     cex.main = par("cex.main"), cex.inner = 0.9, cex.sub = par("cex.sub"),
     col.points = par("col"), col.hor = par("col"), col.vert = par("col"),
     col.main = par("col.main"), col.inner = par("col.main"),
     col.sub = par("col.sub"),  cex.points = 2.0, pch.u = 21, pch.a = 16,
     mfColRow = TRUE, to.draw.arg = NULL, withSubst = TRUE)
\S4method{plot}{DistrList,missing}(x, y, ...)
\S4method{plot}{CompoundDistribution,missing}(x, y, ...)
}
\arguments{
  \item{x}{object of class 
            \code{"AffLinUnivarLebDecDistribution"} or class
            \code{"UnivarLebDecDistribution"} or class
            \code{"AbscontDistribution"} or class
            \code{"DiscreteDistribution"} or class
            \code{"DistrList"}: (list of) distribution(s) to be plotted }
  \item{y}{ missing }
  \item{xlim}{the x limits (x1, x2) of the plot.  Note that \code{x1 > x2} is
              allowed and leads to a "reversed axis". As in \code{plot.default}.}
  \item{ylim}{the y limits of the plot. Either as in \code{plot.default}
  (i.e. a vector of length 2) or a vector of length 4, where the first two
   elements are the values for \code{ylim} in panel "d", and the last
   two elements are the values for \code{ylim} resp. \code{xlim} in panels "p",
   and "q".}
  \item{width}{width (in inches) of the graphics device opened}
  \item{height}{height (in inches) of the graphics device opened}
  \item{withSweave}{logical: if \code{TRUE} (for working with \command{Sweave}) 
        no extra device is opened and height/width are not set}
  \item{ngrid}{integer: number of grid points used for plots of absolutely continuous 
               distributions}
  \item{main}{logical: is a main title to be used? or \cr
              just as argument \code{main} in \code{\link{plot.default}}.}
  \item{inner}{logical: do panels for density/probability function -
                        cdf - quantile function have their own titles? or \cr
               list which is filled to length 3  (resp. 8 for class 
               \code{UnivarLebDecDistribution}) if necessary
               (possibly using recycling rules):  titles for 
               density/probability function -
               cdf - quantile function (each of the same form as argument 
               \code{main} in \code{\link{plot.default}})} 
  \item{sub}{logical: is a sub-title to be used? or \cr
              just as argument \code{sub} in \code{\link{plot.default}}.}
  \item{tmar}{top margin -- useful for non-standard main title sizes}
  \item{bmar}{bottom margin -- useful for non-standard sub title sizes}
  \item{verticals}{logical: if \code{TRUE}, draw vertical lines at steps;
                   as in \code{\link[stats]{plot.stepfun}}} 
  \item{do.points}{logical: if \code{TRUE}, draw  also draw points at the 
         (\code{xlim} restricted) knot locations; as 
         in \code{\link[stats]{plot.stepfun}}} 
  \item{cex.points}{numeric; character expansion factor; as in 
                    \code{\link[stats]{plot.stepfun}}}              
  \item{col.points}{character or integer code; color of points; as in 
                    \code{\link[stats]{plot.stepfun}}}              
  \item{col.hor}{character or integer code; color of horizontal lines; as in 
                    \code{\link[stats]{plot.stepfun}}}              
  \item{col.vert}{character or integer code; color of vertical lines; as in 
                    \code{\link[stats]{plot.stepfun}}}              
  \item{cex.main}{magnification to be used for main titles relative
          to the current setting of \code{cex}; as in 
          \code{\link[graphics]{par}}}
  \item{cex.inner}{magnification to be used for inner titles relative
          to the current setting of \code{cex}; as in 
          \code{\link[graphics]{par}}}
  \item{cex.sub}{magnification to be used for sub titles relative
          to the current setting of \code{cex}; as in 
          \code{\link[graphics]{par}}}
  \item{col.main}{character or integer code; color for the main title}              
  \item{col.inner}{character or integer code; color for the inner title}              
  \item{col.sub}{character or integer code; color for the sub title}              
  \item{pch.u}{character or integer code; plotting characters or symbols for
               unattained value; see \code{\link[graphics]{points}}}              
  \item{pch.a}{character or integer code; plotting characters or symbols for
               attained value; see \code{\link[graphics]{points}}}              
  \item{mfColRow}{shall default partition in panels be used --- defaults to \code{TRUE}}
  \item{to.draw.arg}{Either \code{NULL} (default; everything
  is plotted) or a vector of either integers 
  (the indices of the subplots to be drawn) or characters --- the names of
  the subplots to be drawn: in case of an object \code{x} of class 
  \code{"DiscreteDistribution"} or \code{"AbscontDistribution"} \code{c("d","p","q")}
  for density, c.d.f. and quantile function; in case of \code{x} a proper
  \code{"UnivarLebDecDistribution"} (with pos. weights for both discrete and
  abs. continuous part) names are 
  \code{c("p","q","d.c","p.c","q.c","d.d","p.d","q.d")}) 
  for c.d.f. and quantile function of the composed distribution and the respective
  three panels for the absolutely continuous and the discrete part, respectively; 
  }
  \item{withSubst}{logical; if \code{TRUE} (default) pattern substitution for
     titles and lables is used; otherwise no substitution is used. }
  \item{\dots}{addtional arguments for \code{plot} --- see 
               \code{\link[graphics]{plot}}, 
               \code{\link[graphics]{plot.default}}, 
               \code{\link[stats]{plot.stepfun}} }
}
\details{
\describe{
\item{plot}{\code{signature(x = "AffLinUnivarLebDecDistribution", y = "missing")}: plots  
   cumulative distribution function and the quantile function }
\item{plot}{\code{signature(x = "UnivarLebDecDistribution", y = "missing")}: plots a set of
  eight plots: in the first row, it plots the 
   cumulative distribution function and the quantile function; in the second
   row the absolutely continuous part (with density, cdf and quantile fct.),
   and in the last row the discrete part (with prob.fct., cdf and quantile fct.). }
\item{plot}{\code{signature(x = "CompoundDistribution", y = "missing")}: 
  coerces \code{x} to \code{"UnivarLebDecDistribution"} and uses the corresponding
  method. }
\item{plot}{\code{signature(x = "AbscontDistribution", y = "missing")}: plots density, 
   cumulative distribution function and the quantile function }
\item{plot}{\code{signature(x = "DiscreteDistribution", y = "missing")}: plots probability 
  function, cumulative distribution function and the quantile function}
\item{plot}{\code{signature(x = "DistrList", y = "missing")}: plots a list of distributions}
}
Any parameters of \code{plot.default} may be passed on to this particular
\code{plot} method. 

For main-, inner, and subtitles given as arguments \code{main}, 
\code{inner}, and \code{sub}, top and bottom margins are enlarged to 5 resp. 
6 by default but may also be specified by \code{tmar} / \code{bmar} arguments. 
If \code{main} / \code{inner} / \code{sub} are 
logical then if the respective argument is \code{FALSE} nothing is done/plotted, 
but if it is \code{TRUE}, we use a default main title taking up the calling 
argument \code{x} in case of \code{main}, default inner titles taking up the
class and (named) parameter slots of argument \code{x} in case of \code{inner},
and a "generated on <data>"-tag in case of \code{sub}.
Of course, if \code{main} / \code{inner} / \code{sub} are \code{character}, this
is used for the title; in case of \code{inner} it is then checked whether it
has length 3. In all title and axis label arguments, if \code{withSubst} is \code{TRUE},
the following patterns are substituted:
\describe{
\item{\code{"\%C"}}{class of argument \code{x}}
\item{\code{"\%P"}}{parameters of \code{x} in form of a comma-separated list of 
                   <value>'s coerced to character}
\item{\code{"\%Q"}}{parameters of \code{x} in form of a comma-separated list of 
                   <value>'s coerced to character and in parenthesis --- unless 
                   empty; then ""}
\item{\code{"\%N"}}{parameters of \code{x} in form of a comma-separated list 
                   <name> = <value> coerced to character}
\item{\code{"\%A"}}{deparsed argument \code{x}}
\item{\code{"\%D"}}{time/date-string when the plot was generated}
}

If not explicitly set, \code{col.points}, \code{col.vert},
\code{col.hor}, \code{col.main}, \code{col.inner}, \code{col.sub} are set 
to \code{col} if this arg is given and else to
\code{par("col")} resp. for the titles \code{par("col.main")},
\code{par("col.main")}, \code{par("col.sub")}.  

If not explicitly set, \code{pch.a}, \code{pch.u} are set 
to \code{pch} if this arg is given and else to 16, 21, respectively.

If not explicitly set, \code{cex} is set to 1. If not explicitly set, 
\code{cex.points} is set to $2.0 \code{cex}$ (if \code{cex} is given)
and to 2.0 else.

If general \code{plot} arguments \code{xlab}, \code{ylab} are not specified,
they are set to \code{"x"}, \code{"q"}, \code{"p"} for \code{xlab} and
to \code{"d(x)"}, \code{"p(q)"}, \code{"q(p)"} for \code{ylab} for
density, cdf and quantile function respectively.
Otherwise, according to the respective content of \code{to.draw.arg},
it is supposed to be a list with one entry for each selected panel, i.e.,
in case \code{x} is an object of class \code{DiscreteDistribution} or
\code{AbscontDistribution} a list of maximal length maximally 3, respectively,
in case \code{x} is an object of class \code{UnivarLebDecDistribution}
In these label arguments, the same pattern substitutions are made as
for titles. If no character substitutions and mathematical expressions
are needed, character vectors of respective length instead of lists are
also allowed for arguments \code{xlab}, \code{ylab}.

In addition, argument \code{\dots} may contain arguments \code{panel.first},
\code{panel.last}, i.e., hook expressions to be evaluated at the very beginning
and at the very end of each panel (within the then valid coordinates).
To be able to use these hooks for each panel individually, they may also be
lists of expressions (of the same length as the number of panels and
run through in the same order as the panels).
}

\value{An S3 object of class \code{c("plotInfo","DiagnInfo")}, i.e., a list
  containing the information needed to produce the
  respective plot, which at a later stage could be used by different
  graphic engines (like, e.g. \code{ggplot}) to produce the plot
  in a different framework. A more detailed description will follow in
  a subsequent version.
}

\examples{
plot(Binom(size = 4, prob = 0.3))
plot(Binom(size = 4, prob = 0.3), do.points = FALSE)
plot(Binom(size = 4, prob = 0.3), verticals = FALSE)
plot(Binom(size = 4, prob = 0.3), main = TRUE)
plot(Binom(size = 4, prob = 0.3), main = FALSE)
plot(Binom(size = 4, prob = 0.3), cex.points = 1.2, pch = 20)
plot(Binom(size = 4, prob = 0.3), xlab = list("a1","a2", "a3"),
           ylab=list("p"="U","q"="V","d"="W"))
B <- Binom(size = 4, prob = 0.3)
plot(B, col = "red", col.points = "green", main = TRUE, col.main = "blue", 
     col.sub = "orange", sub = TRUE, cex.sub = 0.6, col.inner = "brown")
plot(Nbinom(size = 4,prob = 0.3), cex.points = 1.2, col = "red", 
     col.points = "green")
plot(Nbinom(size = 4,prob = 0.3), cex.points = 1.2, pch.u = 20, pch.a = 10)
plot(Norm(), main = TRUE, cex.main = 3, tmar = 6)
plot(Norm(), inner = FALSE, main = TRUE, cex.main = 3, tmar = 6)
plot(Norm(), lwd = 3, col = "red", ngrid = 200, lty = 3, las = 2)
plot(Norm(), main = "my Distribution: \%A", 
     inner = list(expression(paste(lambda,"-density of \%C(\%P)")), "CDF",
                  "Pseudo-inverse with param's \%N"), 
     sub = "this plot was correctly generated on \%D", 
     cex.inner = 0.9, cex.sub = 0.8)

plot(Norm(),panel.first=grid(4,4))
## does not (yet) work as desired:
plot(Norm(),panel.first=list(grid(5,5),grid(3,3),grid(4,4)))
li <- list(substitute(grid(5,5)),substitute(grid(3,3)),substitute(grid(4,4)))
plot(Norm(),panel.first=li)

plot(Cauchy())
plot(Cauchy(), xlim = c(-4,4))
plot(Chisq())
### the next ylab argument is just for illustration purposes
plot(Chisq(),mfColRow = FALSE,to.draw.arg="d",
     xlab="x",ylab=list(expression(paste(lambda,"-density of \%C(\%P)"))))
## substitution can be switched off
plot(Chisq(),mfColRow = FALSE,to.draw.arg="d",
     xlab="x",ylab=list(expression(paste(lambda,"-density of \%C(\%P)"))), withSubst=FALSE)
plot(Chisq(), log = "xy", ngrid = 100)
Ch <- Chisq(); setgaps(Ch); plot(Ch, do.points = FALSE)
setgaps(Ch, exactq = 3); plot(Ch, verticals = FALSE)
plot(Ch, cex = 1.2, pch.u = 20, pch.a = 10, col.points = "green", 
     col.vert = "red")

\dontrun{ # to save time 
## some distribution with gaps
wg <- flat.mix(UnivarMixingDistribution(Unif(0,1),Unif(4,5), 
               withSimplify=FALSE))
# some Lebesgue decomposed distribution 
mymix <- UnivarLebDecDistribution(acPart = wg, discretePart = Binom(4,.4),
         acWeight = 0.4)
plot(mymix)         
#
## selection of subpanels for plotting
N <- Norm()
par(mfrow=c(1,2))
plot(N, mfColRow = FALSE, to.draw.arg=c("d","q"))
plot(N, mfColRow = FALSE, to.draw.arg=c(2,3))
par(mfrow=c(1,1))

wg <- flat.mix(UnivarMixingDistribution(Unif(0,1),Unif(4,5),
               withSimplify=FALSE))
myLC <- UnivarLebDecDistribution(discretePart=Binom(3,.3), acPart = wg,
          discreteWeight=.2)
layout(matrix(c(rep(1,6),2,2,3,3,4,4,5,5,5,6,6,6), 
              nrow=3, byrow=TRUE))
plot(myLC,mfColRow = FALSE,
     to.draw.arg=c("p","d.c","p.c","q.c", "p.d","q.d"))

P <- Pois(2)
plot(as(P,"UnivarLebDecDistribution"),mfColRow = FALSE,to.draw.arg=c("d.d"))
### the next ylab argument is just for illustration purposes
plot(as(P,"UnivarLebDecDistribution"),mfColRow = FALSE,to.draw.arg=c("d.d"),
     xlab="x",ylab=list(expression(paste(lambda,"-density of \%C(\%P)"))))
}
}
\seealso{\code{\link[graphics]{plot}},\code{\link[graphics]{plot.default}}, 
         \code{\link[stats]{plot.stepfun}},  \code{\link[graphics]{par}}}
\keyword{methods}
\keyword{hplot}
\keyword{distribution}

\name{internalGenerics}
\alias{internalGenerics}
\alias{distribution}
\alias{samplesize}
\alias{samplesize<-}
\title{Internal: Common Generics 'distribution' and 'samplesize', 'samplesize<-'}
\description{
  In order to be able to use packages \pkg{distrSim} and \pkg{distrMod}
  resp. \pkg{RobAStBase} independently,
  it is necessary to import the respective generic from a prior package, i.e.,
  \pkg{distr}.
  }
\usage{
distribution(object)
samplesize(object, ...)
x <- samplesize(object, value)
}
\arguments{
  \item{object}{ the first argument to dispatch on in the actual methods. }
  \item{value}{ the value to be assigned. }
  \item{\dots}{ additional arguments for function \code{samplesize}. }
}

\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}

\keyword{internal}

\name{RtoDPQ.d}
\alias{RtoDPQ.d}
\title{Default procedure to fill slots d,p,q given r for discrete distributions}
\description{
function to do get empirical density, cumulative distribution and quantile function from random numbers}
\usage{
RtoDPQ.d(r, e = getdistrOption("RtoDPQ.e"))
}
\arguments{
  \item{r}{ the random number generator }
  \item{e}{ \eqn{10^e} numbers are generated, a higher number leads to a better result. }
}
\details{
  \code{RtoDPQ.d} generates \eqn{10^e} random numbers, by default \eqn{e =}
  \code{RtoDPQ.e} which are used to produce a density, cdf and
  quantile function.
  Of course, the results are usually not exact as they rely on random numbers.}
\value{ RtoDPQ returns a list of functions.
  \item{dfun }{ density }
  \item{pfun }{ cumulative distribution function }
  \item{qfun }{ quantile function }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
}
\note{
  Use RtoDPQ for absolutely continuous and RtoDPQ.d for discrete
  distributions.
}
\seealso{ 
\code{\link{UnivariateDistribution-class}}, 
\code{\link{density}}, 
\code{\link{approxfun}}, 
\code{\link{ecdf}}}
\examples{
rn2 <- function(n){rnorm(n)^2}
x <- RtoDPQ(r = rn2, e = 4, n = 512)
# returns density, cumulative distribution and quantile function of
# squared standard  normal distribution
x$dfun(4)
## IGNORE_RDIFF_BEGIN
RtoDPQ(r = rn2, e = 5, n = 1024) # for a better result
## IGNORE_RDIFF_END
rp2 <- function(n){rpois(n, lambda = 1)^2}
x <- RtoDPQ.d(r = rp2, e = 5)
# returns density, cumulative distribution and quantile function of
# squared Poisson distribution with parameter lambda=1
}
\keyword{math}
\keyword{distribution}
\keyword{arith}
\concept{random sample}
\concept{image distribution}
\concept{discrete distribution}
\concept{utility}

\name{distrMASK}
\alias{distrMASK}
\alias{MASKING}
\title{Masking of/by other functions in package "distr"}
\description{Provides information on the (intended) masking of
and (non-intended) masking by other other functions in package \pkg{distr}}
\usage{
distrMASK(library = NULL)
}
\arguments{
  \item{library}{a character vector with path names of R libraries, or \code{NULL}.
                 The default value of \code{NULL} corresponds to all libraries
                 currently known.  If the default is used, the loaded packages
                 are searched before the libraries}
    }
\value{no value is returned
}

\author{Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}
\examples{
distrMASK()
}
\keyword{programming}
\keyword{distribution}
\keyword{documentation}
\concept{masking}
\concept{info file}

\name{UnivarDistrList}
\alias{UnivarDistrList}

\title{Generating function for UnivarDistrList-class}
\description{Generates an object of class \code{"UnivarDistrList"}.}
\usage{UnivarDistrList(..., Dlist)}
\arguments{
  \item{\dots}{Objects of class \code{"UnivariateDistribution"} (or subclasses)}
  \item{Dlist}{an optional list or object of class \code{"UnivarDistrList"};
  if not missing it is appended to argument \code{\dots}; this way
  \code{UnivarMixingDistribution} may also be called with a list (or
  \code{"UnivarDistrList"}-object) as argument as suggested in an e-mail
  by Krunoslav Sever (thank you!)}
}
%\details{}
\value{Object of class \code{"UnivarDistrList"}}
%\references{}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de}}
%\note{}
\seealso{\code{\link{DistrList-class}}, \code{\link{UnivarDistrList-class}},
  \code{\link{UnivarDistrList}}}
\examples{
(DL <- UnivarDistrList(Norm(), Exp(), Pois()))
plot(DL)
as(Norm(), "UnivarDistrList")

## The function is currently defined as
function(...){ 
    new("UnivarDistrList", list(...)) 
}
}
\keyword{list}
\keyword{distribution}
\concept{univariate distribution}
\concept{S4 distribution class}

\name{Huberize-methods}
\docType{methods}
\alias{Huberize-methods}
\alias{Huberize}
\alias{Huberize,AcDcLcDistribution-method}
\title{ Methods for function Huberize in Package `distr' }
\description{Huberize-methods}
\usage{
Huberize(object, ...)
\S4method{Huberize}{AcDcLcDistribution}(object,lower,upper,
                    withSimplify = getdistrOption("simplifyD"))
}
\arguments{
  \item{object}{distribution object}
  \item{\ldots}{further arguments for \code{Huberize}; takes up \code{lower}, 
  \code{upper}, \code{withSimplify}.}
  \item{lower}{numeric; lower truncation point}
  \item{upper}{numeric; upper truncation point}
  \item{withSimplify}{logical; is result to be piped through a call to
         \code{\link{simplifyD}}?}

}
\value{the corresponding distribution of the truncated random variable}
\section{Methods}{\describe{
\item{Huberize}{\code{signature(object = "AcDcLcDistribution")}:
returns the unconditioned
distribution of \code{min(upper,max(X,lower))}, if \code{X} is
distributed according to \code{object};
the result is of class \code{"UnivarLebDecDistribution"} in general.}
}}
\examples{
Hub <- Huberize(Norm(),lower=-1,upper=2)
Hub 
plot(Hub)
}
\seealso{\code{\link{Truncate}}}
\keyword{distribution}
\keyword{methods}
\concept{truncation}

\name{Math-methods}
\docType{methods}
\alias{Math-methods}
\alias{log-methods}
\alias{log10-methods}
\alias{lgamma-methods}
\alias{gamma-methods}
\alias{sign-methods}
\alias{Math,AcDcLcDistribution-method}
\alias{Math,AbscontDistribution-method}
\alias{Math,DiscreteDistribution-method}
\alias{abs,AbscontDistribution-method}
\alias{abs,DiscreteDistribution-method}
\alias{exp,AbscontDistribution-method}
\alias{exp,DiscreteDistribution-method}
\alias{log,AbscontDistribution-method}
\alias{log,DiscreteDistribution-method}
\alias{sign,AbscontDistribution-method}
\alias{sign,DiscreteDistribution-method}
\alias{log10,AbscontDistribution-method}
\alias{log10,DiscreteDistribution-method}
\alias{digamma,AbscontDistribution-method}
\alias{digamma,DiscreteDistribution-method}
\alias{lgamma,AbscontDistribution-method}
\alias{lgamma,DiscreteDistribution-method}
\alias{gamma,AbscontDistribution-method}
\alias{gamma,DiscreteDistribution-method}
\title{ Methods for Functions from group `Math' in Package `distr' }
\description{Math-methods provide automatical generation of image distributions
for random variables transformed by functions from group
\code{\link[methods:S4groupGeneric]{Math}}}
\section{Methods}{\describe{
    \item{Math}{\code{signature(x = "AbscontDistribution")}: application of a
      mathematical function from group \code{\link[methods:S4groupGeneric]{Math}}, e.g.
      \code{sin} or \code{exp}  (including
      \code{log, log10, gamma, lgamma, digamma}), to this
      absolutely continouos distribution}
    \item{Math}{\code{signature(x = "DiscreteDistribution")}: application of a
      mathematical function, e.g. sin or exp  (including
      \code{log, log10, gamma, lgamma, digamma}), to this
      discrete distribution}
    \item{Math}{\code{signature(x = "UnivarLebDecDistribution")}: application of a
      mathematical function from group \code{\link[methods:S4groupGeneric]{Math}}, e.g.
      \code{sin} or \code{exp}  (including
      \code{log, log10, gamma, lgamma}), to this
      Lebesgue decomposed distribution}
    \item{Math}{\code{signature(x = "UnivarLebDecDistribution")}: application of a
      mathematical function from group \code{\link[methods:S4groupGeneric]{Math}}, e.g.
      \code{sin} or \code{exp}  (including
      \code{log, log10, gamma, lgamma}), to this
       distribution of class \code{"AcDcLcDistribution"}}
    \item{abs}{\code{signature(x = "AbscontDistribution")}: application of 
      function \code{abs} to this absolutely continouos distribution; 
      (exactly)}
    \item{abs}{\code{signature(x = "DiscreteDistribution")}: application of 
      function \code{abs} to this discrete distribution; 
      (exactly)}
    \item{sign}{\code{signature(x = "AbscontDistribution")}: application of 
      function \code{abs} to this absolutely continouos distribution; 
      (exactly)}
    \item{sign}{\code{signature(x = "DiscreteDistribution")}: application of 
      function \code{abs} to this discrete continouos distribution; 
      (exactly)}
    \item{exp}{\code{signature(x = "AbscontDistribution")}: application of
      function \code{exp} to this absolutely continouos distribution;
      (exactly)}
    \item{exp}{\code{signature(x = "DiscreteDistribution")}: application of
      function \code{exp} to this discrete distribution;
      (exactly)}
    \item{log}{\code{signature(x = "AbscontDistribution")}: application of
      function \code{log} to this absolutely continouos distribution;
      (exactly for R-version \code{>2.5.1})}
    \item{log}{\code{signature(x = "DiscreteDistribution")}: application of
      function \code{log} to this discrete distribution;
      (exactly for R-version \code{>2.5.1})}
}}
\keyword{arith}
\keyword{distribution}
\concept{image distribution}

\name{UnivarMixingDistribution-class}
\docType{class}
\alias{UnivarMixingDistribution-class}
\alias{mixCoeff}
\alias{mixDistr}
\alias{mixCoeff<-}
\alias{mixDistr<-}
\alias{mixCoeff<--methods}
\alias{mixDistr<--methods}
\alias{mixCoeff<-,UnivarMixingDistribution-method}
\alias{mixDistr<-,UnivarMixingDistribution-method}
\alias{mixCoeff-methods}
\alias{mixDistr-methods}
\alias{mixCoeff,UnivarMixingDistribution-method}
\alias{mixDistr,UnivarMixingDistribution-method}
\alias{support,UnivarMixingDistribution-method}
\alias{gaps,UnivarMixingDistribution-method}
\alias{.logExact,UnivarMixingDistribution-method}
\alias{.lowerExact,UnivarMixingDistribution-method}
\alias{Symmetry,UnivarMixingDistribution-method}

\title{Class "UnivarMixingDistribution"}
\description{\code{UnivarMixingDistribution}-class is a class to formalize 
                   univariate mixing distributions; it is a subclass to
                   class \code{UnivariateDistribution}.}
\section{Objects from the Class}{
Objects can be created by calls of the form 
\code{new("UnivarMixingDistribution", ...)}.
  More frequently they are created via the generating function 
  \code{\link{UnivarMixingDistribution}}.
}
\section{Slots}{
  \describe{
    \item{\code{mixCoeff}}{Object of class \code{"numeric"}: a vector of 
            probabilities for the mixing components.}
    \item{\code{mixDistr}}{Object of class \code{"UnivarDistrList"}: a list of
    univariate distributions containing the mixing components; must be of same
    length as \code{mixCoeff}.}
    \item{\code{img}}{Object of class \code{"Reals"}: the space of the image of this distribution which has dimension 1
    and the name "Real Space" }
    \item{\code{param}}{Object of class \code{"Parameter"}: the parameter of this distribution, having only the
    slot name "Parameter of a discrete distribution" }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers}
    \item{\code{d}}{fixed to \code{NULL}}
    \item{\code{p}}{Object of class \code{"function"}: cumulative distribution function}
    \item{\code{q}}{Object of class \code{"function"}: quantile function}
    \item{\code{support}}{numeric vector --- the union of all support slots of components, if existing}
    \item{\code{gaps}}{(numeric) matrix or \code{NULL}; the merged \code{gaps} slots of all components, 
    if existing (else NULL)}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where there are explicit formulae for the
                              log version of density, cdf, and quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where there are explicit formulae for the
                              lower tail version of cdf and quantile function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"UnivariateDistribution"}
class \code{"Distribution"} by class \code{"UnivariateDistribution"}.
}
\section{Methods}{
  \describe{
    \item{show}{\code{signature(object = "UnivarMixingDistribution")} prints the object}
    \item{mixCoeff<-}{\code{signature(object = "UnivarMixingDistribution")} replaces the corresponding slot}
    \item{mixCoeff}{\code{signature(object = "UnivarMixingDistribution")} returns the corresponding slot}
    \item{mixDistr<-}{\code{signature(object = "UnivarMixingDistribution")} replaces the corresponding slot}
    \item{mixDistr}{\code{signature(object = "UnivarMixingDistribution")} returns the corresponding slot}
    \item{support}{\code{signature(object = "UnivarMixingDistribution")} returns the corresponding slot}
    \item{gaps}{\code{signature(object = "UnivarMixingDistribution")} returns the corresponding slot}
    \item{.logExact}{\code{signature(object = "Distribution")}: returns slot 
      \code{.logExact} if existing; else tries to convert the object to a newer 
      version of its class by \code{\link{conv2NewVersion}} and
      returns the corresponding slot of the converted object.}
    \item{.lowerExact}{\code{signature(object = "Distribution")}: returns slot 
       \code{.lowerExact} if existing; else tries to convert the object to a 
       newer version of its class by \code{\link{conv2NewVersion}} and
     returns the corresponding slot of the converted object.}
    \item{Symmetry}{returns slot \code{Symmetry} if existing; else
    tries to convert the object to a 
       newer version of its class by \code{\link{conv2NewVersion}} and
     returns the corresponding slot of the converted object.}
  }
}


\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}
  }

\seealso{
\code{\link{Parameter-class}}, 
\code{\link{UnivariateDistribution-class}}, 
\code{\link{LatticeDistribution-class}}, 
\code{\link{AbscontDistribution-class}}, 
\code{\link{simplifyD}}, 
\code{\link{flat.mix}}
}
\examples{
mylist <- UnivarMixingDistribution(Binom(3,.3), Dirac(2), Norm(), 
          mixCoeff=c(1/4,1/5,11/20))
mylist2 <- UnivarMixingDistribution(Binom(3,.3), mylist, 
          mixCoeff=c(.3,.7))
mylist2
p(mylist)(0.3)          
mixDistr(mylist2)
}
\keyword{distribution}
\concept{discrete distribution}
\concept{lattice distribution}
\concept{support of a distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{Td-class}
\docType{class}
\alias{Td-class}
\alias{Td}
\alias{initialize,Td-method}

\title{Class "Td"}
\description{  The \eqn{t} distribution with \code{df} \eqn{= \nu}{= n} degrees of
  freedom has density
  \deqn{
    f(x) = \frac{\Gamma ((\nu+1)/2)}{\sqrt{\pi \nu} \Gamma (\nu/2)}
    (1 + x^2/\nu)^{-(\nu+1)/2}%
  }{f(x) = Gamma((n+1)/2) / (sqrt(n pi) Gamma(n/2)) (1 + x^2/n)^-((n+1)/2)}
  for all real \eqn{x}.
  It has mean \eqn{0} (for \eqn{\nu > 1}{n > 1}) and
  variance \eqn{\frac{\nu}{\nu-2}}{n/(n-2)} (for \eqn{\nu > 2}{n > 2}).
 C.f. \code{\link[stats:TDist]{rt}}   
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Td(df)}.
This object is a \eqn{t} distribution.
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}: The domain of this distribution has got dimension 1
    and the name "Real Space". }
    \item{\code{param}}{Object of class \code{"TParameter"}: the parameter of this distribution (df),
     declared at its instantiation }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers (calls function \code{rt})}
    \item{\code{d}}{Object of class \code{"function"}: density function (calls function \code{dt})}
    \item{\code{p}}{Object of class \code{"function"}: cumulative function (calls function \code{pt})}
    \item{\code{q}}{Object of class \code{"function"}: inverse of the cumulative function (calls function \code{qt})}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"AbscontDistribution"}, directly.\cr
Class \code{"UnivariateDistribution"}, by class \code{"AbscontDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"AbscontDistribution"}.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Td")}: initialize method }
    \item{df}{\code{signature(object = "Td")}: returns the slot df of the parameter of the distribution }
    \item{df<-}{\code{signature(object = "Td")}: modifies the slot df of the parameter of the distribution }
    \item{ncp}{\code{signature(object = "Td")}: returns the slot ncp of the parameter of the distribution }
    \item{ncp<-}{\code{signature(object = "Td")}: modifies the slot ncp of the parameter of the distribution }
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}

\note{  The general \emph{non-central} \eqn{t}
  with parameters \eqn{(\nu,\delta)}{(df,Del)} \code{= (df, ncp)}
  is defined as a the distribution of
  \eqn{T_{\nu}(\delta) := \frac{U + \delta}{\chi_{\nu}/\sqrt{\nu}}}{%
      T(df,Del) := (U + Del) / (Chi(df) / sqrt(df)) }
  where \eqn{U} and \eqn{\chi_{\nu}}{Chi(df)}  are independent random
  variables, \eqn{U \sim {\cal N}(0,1)}{U \~ N(0,1)}, and
%%fails  \eqn{{\chi_{\nu}}^2}{(Chi(df))^2}
  \eqn{\chi^2_\nu}{Chi(df)^2}
  is chi-squared, see \code{\link[stats:Chisquare]{rchisq}}.

  The most used applications are power calculations for \eqn{t}-tests:\cr
  Let \eqn{T= \frac{\bar{X} - \mu_0}{S/\sqrt{n}}}{T= (mX - m0) / (S/sqrt(n))}
  where
  \eqn{\bar{X}}{mX} is the \code{\link{mean}} and \eqn{S} the sample standard
  deviation (\code{\link{sd}}) of \eqn{X_1,X_2,\dots,X_n} which are i.i.d.
%%fails \eqn{{\cal N}(\mu,\sigma^2)}{N(mu,sigma^2)}
        \eqn{       N(\mu,\sigma^2)}{N(mu,sigma^2)}.
  Then \eqn{T} is distributed as non-centrally \eqn{t} with
  \code{df}\eqn{= n-1}
  degrees of freedom and \bold{n}on-\bold{c}entrality \bold{p}arameter
  \code{ncp}\eqn{= (\mu - \mu_0) \sqrt{n}/\sigma}{= (mu - m0) * sqrt(n)/sigma}.

}
\seealso{
\code{\link{TParameter-class}}, 
\code{\link{AbscontDistribution-class}}, 
\code{\link{Reals-class}}, 
\code{\link[stats:TDist]{rt}}
}
\section{Ad hoc methods}{
      For R Version \code{<2.3.0} ad hoc methods are provided for slots \code{q}, \code{r} if \code{ncp!=0};
      for R Version \code{>=2.3.0} the methods from package \pkg{stats} are used.
      }

\examples{
T <- Td(df = 1) # T is a t distribution with df = 1.
r(T)(1) # one random number generated from this distribution, e.g. -0.09697573
d(T)(1) # Density of this distribution is 0.1591549 for x = 1.
p(T)(1) # Probability that x < 1 is 0.75.
q(T)(.1) # Probability that x < -3.077684 is 0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
df(T) # df of this distribution is 1.
df(T) <- 2 # df of this distribution is now 2.
Tn <- Td(df = 1, ncp = 5) 
  # T is a noncentral t distribution with df = 1 and ncp = 5.
d(Tn)(1) ## from R 2.3.0 on ncp no longer ignored...
}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{T distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{AbscontDistribution}
\alias{AbscontDistribution}

\title{Generating function "AbscontDistribution"}
\description{Generates an object of class  \code{"AbscontDistribution"}}

\usage{
AbscontDistribution(r = NULL, d = NULL, p = NULL, q = NULL,
                   gaps = NULL, param = NULL, img = new("Reals"),
                   .withSim = FALSE, .withArith = FALSE,
                    .lowerExact = FALSE, .logExact = FALSE,
                   withgaps = getdistrOption("withgaps"),
                   low1 = NULL, up1 = NULL, low = -Inf, up =Inf,
                   withStand = FALSE,
                   ngrid = getdistrOption("DefaultNrGridPoints"),
                   ep = getdistrOption("TruncQuantile"),
                   e = getdistrOption("RtoDPQ.e"),
                   Symmetry = NoSymmetry())
}
\arguments{
  \item{r}{slot \code{r} to be filled} 
  \item{d}{slot \code{d} to be filled} 
  \item{p}{slot \code{p} to be filled} 
  \item{q}{slot \code{q} to be filled} 
  \item{gaps}{slot gaps (of class \code{"matrix"} with two columns) to be filled 
              (i.e. \code{t(gaps)} must be ordered if read as vector)} 
  \item{param}{parameter (of class \code{"OptionalParameter"})} 
  \item{img}{image range of the distribution (of class \code{"rSpace"})} 
  \item{low1}{lower bound (to be the lower TruncQuantile-quantile of the distribution)} 
  \item{up1}{upper bound (to be the upper TruncQuantile-quantile of the distribution)} 
  \item{low}{lower bound (to be the 100-percent-quantile of the distribution)}   
  \item{up}{upper bound (to be the 100-percent-quantile of the distribution)}  
  \item{withStand}{logical: shall we standardize argument function \code{d}
  to integrate to 1 --- default is no resp. \code{FALSE}}  
  \item{ngrid}{number of gridpoints}
  \item{ep}{tolerance epsilon}
  \item{e}{exponent to base 10 to be used for simulations}
  \item{withgaps}{logical; shall gaps be reconstructed empirically?}
  \item{.withArith}{normally not set by the user, but if determining the entries \code{supp}, \code{prob}
                    distributional arithmetics was involved, you may set this to \code{TRUE}.}
  \item{.withSim}{normally not set by the user, but if determining the entries \code{supp}, \code{prob}
                  simulations were involved, you may set this to \code{TRUE}.}
  \item{.lowerExact}{normally not set by the user: whether the \code{lower.tail=FALSE}
                     part is calculated exactly, avoing a ``\code{1-.}''.}
  \item{.logExact}{normally not set by the user: whether in determining slots \code{d,p,q},
         we make particular use of a logarithmic representation to enhance accuracy.}
  \item{Symmetry}{you may help \R in calculations if you tell it whether
    the distribution is non-symmetric (default) or symmetric with respect
    to a center; in this case use \code{Symmetry=SphericalSymmetry(center)}.}
}
\details{
  Typical usages are
  \preformatted{
  AbscontDistribution(r)
  AbscontDistribution(r = NULL, d)
  AbscontDistribution(r = NULL, d = NULL, p)
  AbscontDistribution(r = NULL, d = NULL, p = NULL, d)
  AbscontDistribution(r, d, p, q)
  }


Minimally, only one of the slots \code{r}, \code{d}, \code{p} or \code{q} needs to be given as argument.
The other non-given slots are then reconstructed according to the following scheme:

\tabular{ccccl}{
r\tab d\tab p\tab q\tab proceding\cr
-\tab -\tab -\tab -\tab excluded\cr
-\tab +\tab -\tab -\tab p by \code{.D2P}, q by \code{.P2Q}, r by \code{q(runif(n))}\cr
-\tab -\tab +\tab -\tab d by \code{.P2D}, q by \code{.P2Q}, r by \code{q(runif(n))}\cr
-\tab +\tab +\tab -\tab q by \code{.P2Q}, r by \code{q(runif(n))}\cr
-\tab -\tab -\tab +\tab p by \code{.Q2P}, d by \code{.P2D}, r by \code{q(runif(n))}\cr
-\tab +\tab -\tab +\tab p by \code{.Q2P}, r by \code{q(runif(n))}\cr
-\tab -\tab +\tab +\tab d by \code{.P2D}, r by \code{q(runif(n))}\cr
-\tab +\tab +\tab +\tab r by \code{q(runif(n))}\cr
+\tab -\tab -\tab -\tab call to \code{\link{RtoDPQ}}\cr
+\tab +\tab -\tab -\tab p by \code{.D2P}, q by \code{.P2Q}\cr
+\tab -\tab +\tab -\tab d by \code{.P2D}, q by \code{.P2Q}\cr
+\tab +\tab +\tab -\tab q by \code{.P2Q}\cr
+\tab -\tab -\tab +\tab p by \code{.Q2P}, d by \code{.P2D}\cr
+\tab +\tab -\tab +\tab p by \code{.Q2P}\cr
+\tab -\tab +\tab +\tab d by \code{.P2D}\cr
+\tab +\tab +\tab +\tab nothing\cr}

For this purpose, one may alternatively give arguments \code{low1} and \code{up1} (\code{NULL} each by default,
and determined through slot \code{q}, resp. \code{p}, resp. \code{d}, resp. \code{r} in this order
according to availability),
for the (finite) range of values in the support of this distribution, 
as well as the possibly infinite theoretical range given by
arguments \code{low} and \code{up} with default values \code{-Inf}, \code{Inf}, respectively.
Of course all other slots may be specified as arguments.}


\value{Object of class \code{"AbscontDistribution"}}


\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}
  }


\seealso{
\code{\link{AbscontDistribution-class}},
\code{\link{DiscreteDistribution-class}}, 
\code{\link{RtoDPQ}}
}
\examples{
plot(Norm())
plot(AbscontDistribution(r = rnorm))
plot(AbscontDistribution(d = dnorm))
plot(AbscontDistribution(p = pnorm))
plot(AbscontDistribution(q = qnorm))
plot(Ac <- AbscontDistribution(d = function(x, log = FALSE){
                                   d <- exp(-abs(x^3))
                                   ## unstandardized!!
                                   if(log) d <- log(d)
                                   return(d)}, 
                         withStand = TRUE))
}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{generating function}

\name{BetaParameter-class}
\docType{class}
\alias{BetaParameter-class}
\alias{initialize,BetaParameter-method}
\title{Class "BetaParameter"}
\description{
  The parameter of a beta distribution, used by \code{Beta-class}
}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("BetaParameter", shape1, shape2, ncp)}.
  Usually an object of this class is not needed on its own, it is
  generated   automatically when an object of the class Beta
  is instantiated. 
}
\section{Slots}{
  \describe{
    \item{\code{shape1}}{Object of class \code{"numeric"}: the shape1 of a beta distribution }
    \item{\code{shape2}}{Object of class \code{"numeric"}: the shape2 of a beta distribution }
    \item{\code{ncp}}{Object of class \code{"numeric"}: the noncentrality parameter of a beta distribution }
    \item{\code{name}}{Object of class \code{"character"}: a name / comment for the parameters }
  }
}
\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "BetaParameter")}:
      initialize method}
    \item{shape1}{\code{signature(object = "BetaParameter")}:
      returns the slot shape1 of the parameter of the distribution}
    \item{shape1<-}{\code{signature(object = "BetaParameter")}:
      modifies the slot shape1 of the parameter of the distribution}
    \item{shape2}{\code{signature(object = "BetaParameter")}:
      returns the slot shape2 of the parameter of the distribution}
    \item{shape2<-}{\code{signature(object = "BetaParameter")}:
      modifies the slot shape2 of the parameter of the distribution}
    \item{ncp}{\code{signature(object = "BetaParameter")}: returns the slot \code{ncp} of the parameter of the distribution }
    \item{ncp<-}{\code{signature(object = "BetaParameter")}: modifies the slot \code{ncp} of the parameter of the distribution }
}
}
\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
}
\seealso{
  \code{\link{Beta-class}}
  \code{\link{Parameter-class}}
}
\examples{
W <- new("BetaParameter", shape1 = 1, shape2 = 1, ncp = 0)
shape2(W) # shape2 of this distribution is 1.
shape2(W) <- 2 # shape2 of this distribution is now 2.
}
\keyword{distribution}
\concept{parameter}
\concept{Beta distribution}
\concept{S4 parameter class}

\name{Geom-class}
\docType{class}
\alias{Geom-class}
\alias{Geom}
\alias{initialize,Geom-method}

\title{Class "Geom" }
\description{
  The geometric distribution with \code{prob} \eqn{= p} has density
  \deqn{p(x) = p {(1-p)}^{x}}{p(x) = p (1-p)^x}
  for \eqn{x = 0, 1, 2, \ldots}{x = 0, 1, 2, ...}
  
  C.f. \code{\link[stats:Geometric]{rgeom}}
}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{Geom(prob)}.
  This object is a geometric distribution. 
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Naturals"}:
      The space of the image of this distribution has got dimension 1
      and the name "Natural Space".}
    \item{\code{param}}{Object of class \code{"NbinomParameter"}:
      the parameter of this distribution (prob),  
      declared at its instantiation (size=1)}
    \item{\code{r}}{Object of class \code{"function"}:
      generates random numbers (calls function rgeom)}
    \item{\code{d}}{Object of class \code{"function"}:
      density function (calls function dgeom)}
    \item{\code{p}}{Object of class \code{"function"}:
      cumulative function (calls function pgeom)}
    \item{\code{q}}{Object of class \code{"function"}:
      inverse of the cumulative function (calls function qgeom).
      The quantile is defined as the smallest value \eqn{x} such that
      \eqn{F(x) \ge p}, where \eqn{F} is the distribution function.}      
    \item{\code{support}}{Object of class \code{"numeric"}:
      a (sorted) vector containing the support of the discrete density
      function}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"DiscreteDistribution"}, directly.\cr 
Class \code{"Nbinom"}, directly.\cr
Class \code{"UnivariateDistribution"}, by class \code{"DiscreteDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"DiscreteDistribution"}.
}
\section{Contains-Relations}{
By means of a \code{contains} argument in the class declaration, R ``knows'' 
that a distribution object \code{obj} of class \code{"Geom"} also is a 
negative Binomial distribution with parameters \code{size = 1, prob = prob(obj)}
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Geom")}: initialize method}
    \item{prob}{\code{signature(object = "Geom")}:
      returns the slot prob of the parameter of the distribution}
    \item{prob<-}{\code{signature(object = "Geom")}:
      modifies the slot prob of the parameter of the distribution}
  }
}

\author{
Thomas Stabla \email{statho3@web.de},\cr 
Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
}
\note{Working with a computer, we use a finite interval as support which carries 
      at least mass \code{1-getdistrOption("TruncQuantile")}. }

\seealso{
\code{\link{Nbinom-class}}
\code{\link{GeomParameter-class}}
\code{\link{DiscreteDistribution-class}}
\code{\link{Naturals-class}}
\code{\link[stats:Geometric]{rgeom}}
}
\examples{
G <- Geom(prob = 0.5) # G is a geometric distribution with prob = 0.5.
r(G)(1) # one random number generated from this distribution, e.g. 0
d(G)(1) # Density of this distribution is 0.25 for x = 1.
p(G)(1) # Probability that x<1 is 0.75.
q(G)(.1) # x = 0 is the smallest value x such that p(G)(x) >= 0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
prob(G) # prob of this distribution is 0.5.
prob(G) <- 0.6 # prob of this distribution is now 0.6.
as(G,"Nbinom")
G+G+G
}
\keyword{distribution}
\concept{discrete distribution}
\concept{lattice distribution}
\concept{Geometric distribution}
\concept{Negative Binomial distribution}
\concept{S4 parameter class}
\concept{generating function}

\name{liesIn-methods}
\docType{methods}
\alias{liesIn-methods}
\alias{liesIn}
\alias{liesIn,EuclideanSpace,numeric-method}
\alias{liesIn,Naturals,numeric-method}
\title{ Methods for Function liesIn in Package `distr' }
\description{liesIn-methods}
\section{Methods}{\describe{
    \item{liesIn}{\code{signature(object = "EuclideanSpace", x =
	"numeric")}}:
    
    Does a particular vector lie in this space or not?
    \item{liesIn}{\code{signature(object = "Naturals", x =
	"numeric")}}:
    
    Does a particular vector only contain naturals?}
  }
\keyword{distribution}
\keyword{methods}
\concept{space}
\concept{S4 space class}
\concept{set}

\name{Parameter-class}
\docType{class}
\alias{Parameter-class}
\title{Class "Parameter"}
\description{
  Parameter is the mother-class of all Parameter classes.
}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("Parameter")}.
}
\section{Slots}{
  \describe{
    \item{\code{name}}{Object of class \code{"character"}:
      a name / comment for the parameters}
  }
}
\section{Methods}{
  \describe{
    \item{name}{\code{signature(object = "Parameter")}:
      returns the name of the parameter}
    \item{name<-}{\code{signature(object = "Parameter")}:
      modifies the name of the parameter}
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }

\seealso{
\code{\link{Distribution-class}}
}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{S4 parameter class}

\name{EllipticalSymmetry}
\alias{EllipticalSymmetry}

\title{Generating function for EllipticalSymmetry-class}
\description{Generates an object of class \code{"EllipticalSymmetry"}.}
\usage{EllipticalSymmetry(SymmCenter = 0)}
\arguments{
  \item{SymmCenter}{ numeric: center of symmetry }
}
%\details{}
\value{Object of class \code{"EllipticalSymmetry"}}
%\references{}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de}}
%\note{}
\seealso{\code{\link{EllipticalSymmetry-class}}, \code{\link{DistributionSymmetry-class}}}
\examples{
EllipticalSymmetry()

## The function is currently defined as
function(SymmCenter = 0){ 
    new("EllipticalSymmetry", SymmCenter = SymmCenter) 
}
}
\concept{elliptical symmetry}
\concept{symmetry}
\keyword{distribution}

\name{HyperParameter-class} 
\docType{class}
\alias{HyperParameter-class}
\alias{initialize,HyperParameter-method}

\title{Class "HyperParameter"}
\description{ The parameter of a hypergeometric distribution, used by \code{Hyper}-class}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("HyperParameter", k, m, n)}.
Usually an object of this class is not needed on its own, it is generated automatically when an object of the class \code{Hyper}
is instantiated. 
}
\section{Slots}{
  \describe{
    \item{\code{k}}{Object of class \code{"numeric"}: k of a hypergeometric distribution }
    \item{\code{m}}{Object of class \code{"numeric"}: m of a hypergeometric distribution }
    \item{\code{n}}{Object of class \code{"numeric"}: n of a hypergeometric distribution }
    \item{\code{name}}{Object of class \code{"character"}: a name / comment for the parameters }
  }
}
\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "HyperParameter")}: initialize method }
    \item{k}{\code{signature(object = "HyperParameter")}: returns the slot \code{k} of the parameter of the distribution }
    \item{k<-}{\code{signature(object = "HyperParameter")}: modifies the slot \code{k} of the parameter of the distribution }
    \item{m}{\code{signature(object = "HyperParameter")}: returns the slot \code{m} of the parameter of the distribution }
    \item{m<-}{\code{signature(object = "HyperParameter")}: modifies the slot \code{m} of the parameter of the distribution }
    \item{n}{\code{signature(object = "HyperParameter")}: returns the slot \code{n} of the parameter of the distribution }
    \item{n<-}{\code{signature(object = "HyperParameter")}: modifies the slot \code{n} of the parameter of the distribution }
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }


\seealso{
\code{\link{Hyper-class}}
\code{\link{Parameter-class}}
}

\examples{
W <- new("HyperParameter",k=3, m=3, n=3)
m(W) # m of this distribution is 3.
m(W) <- 2 # m of this distribution is now 2.
}
\keyword{distribution}
\concept{parameter}
\concept{Hypergeometric distribution}
\concept{S4 parameter class}

\name{standardMethods}
\alias{standardMethods}
\title{Utility to automatically generate accessor and replacement functions}
\description{
Creates definitions for accessor and replacement functions of an given class.
}
\usage{
standardMethods(class, writetofile = FALSE, directory)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{class}{the class for which accessor and replacement functions
    are to be produced, given as a string}
  \item{writetofile}{logical value, indicating wheter output is to be written to a file}
  \item{directory}{if writetofile = TRUE, the output is
    written to a file in the given directory, the name of the file
    starting with "classname" and ending with "StandardMethods.txt"}
}
\value{no value is returned
}
\author{Thomas Stabla \email{statho@web.de}}
\examples{
setClass("testclass", representation(a = "numeric", b = "character"))
standardMethods("testclass")
}
\keyword{utilities}
\keyword{programming}

\name{distroptions}
\alias{distroptions}
\alias{getdistrOption}
\alias{DefaultNrGridPoints}
\alias{DistrResolution}
\alias{DistrCollapse}
\alias{TruncQuantile}
\alias{WarningSim}
\alias{WarningArith}
\alias{DefaultNrFFTGridPointsExponent}
\alias{RtoDPQ.e}
\alias{WarningArith}
\alias{WarningSim}
\alias{withSweave}
\alias{warn.makeDNew}
%\alias{simplifyD}
\alias{withgaps}
\alias{use.generalized.inverse.by.default}
\alias{DistributionAggregate.Unique.Warn}


\title{
  functions to change the global variables of the package `distr'
}
\description{
  With \code{distroptions} and \code{getdistrOption} you may inspect and change the global
  variables used by package \pkg{distr}.
}
\usage{
distroptions(...)
getdistrOption(x)
}
\arguments{
  \item{\dots}{ any options can be defined, using name = value or by passing a list of such tagged values. }
  \item{x}{ a character string holding an option name.}
}
\details{Invoking \code{distroptions()} with no arguments returns a list with the current values of the options.  
 To access the value of a single option, one should use \code{getdistrOption("WarningSim")}, e.g., rather than
 \code{distroptions("WarningSim")} which is a \emph{list} of length one.}
\value{
\code{distroptions()} returns a list of the global options of \pkg{distr}. \cr
\code{distroptions("RtoDPQ.e")} returns  the global option \code{RtoDPQ.e} as a list of length 1. \cr
\code{distroptions("RtoDPQ.e" = 3)} sets the value of the global option  \code{RtoDPQ.e} to 3. 
\code{getdistrOption("RtoDPQ.e")} the current value set for option \code{RtoDPQ.e}. 
}
\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}

\section{Currently available options}{
\describe{
\item{\code{DefaultNrGridPoints}}{default number of grid points in integration, default value: \code{2^12}}
\item{\code{DistrResolution}}{minimal spacing between two mass points in a discrete distribution, default value: \code{1e-6}}
\item{\code{DistrCollapse}}{logical; in discrete distributions, shall support points
with distance smaller than \code{DistrResolution} be collapsed; default value:  \code{TRUE}}
\item{\code{TruncQuantile}}{argument for \code{q}-slot at which to truncate; also, for discrete distributions, 
 support is restricted to [\code{q(TruncQuantile)},\code{q(1-TruncQuantile)}], default value: \code{1e-5}}
\item{\code{DefaultNrFFTGridPointsExponent}}{by default, for e = \code{DefaultNrFFTGridPointsExponent}, 
 FFT uses \eqn{2^e} gridpoints; default value: \code{12}}
\item{\code{RtoDPQ.e}}{by default, for reconstructing the \code{d}-,\code{p}-,\code{q}-slots out of simulations by slot \code{r},
\code{RtoDPQ} resp. \code{RtoDPQ.d} use \eqn{10^e} simulations, where e = \code{RtoDPQ.e}, default value: \code{5}}
\item{\code{WarningSim}}{if \code{WarningSim==TRUE}, \code{print}/\code{show} issue a warning as to the precision of 
\code{d}-,\code{p}-,\code{q}-slots when these are obtained by \code{RtoDPQ} resp. \code{RtoDPQ.d}, default value: \code{TRUE}}
\item{\code{WarningArith}}{if \code{WarningArith==TRUE}, \code{print}/\code{show} issue a warning as to the interpretation of
arithmetics operating on distributions, when the corresponding distribution to be plotted/shown is obtained by such an operation;
keep in mind that arithmetics in fact operate on random variables distributed according to the given distributions and \bold{not}
on corresponding cdf's or densities; default value: \code{TRUE}}
\item{\code{withSweave}}{is code run in Sweave (then no new graphic devices are opened), 
                         default value: \code{FALSE}}
  \item{\code{withgaps}}{controls whether in the return value of arithmetic
  operations the slot \code{gaps} of an the \code{AbscontDistribution} part
  is filled automatically based on empirical evaluations via  \code{setgaps}
   ---default \code{TRUE}}
  \item{\code{simplifyD}}{ controls whether in the return value of arithmetic
  operations there is a call to \code{simplifyD} or not ---default \code{TRUE}}
\item{use.generalized.inverse.by.default}{logical; decides whether by default
(i.e., if argument \code{generalized} of \code{\link{solve}} is not explicitely
set), \code{\link{solve}} is to use generalized inverses if the original 
\code{solve}-method from package \pkg{base} fails; if the option is \code{FALSE}, 
in case of failure, and unless argument \code{generalized} is not explicitely 
set to \code{TRUE}, \code{solve} will throw an error as is 
the \pkg{base}-method behavior. The default value is \code{TRUE}.  
  }
\item{\code{DistrCollapse.Unique.Warn}}{controls whether there is a warning
      whenever collapsing occurs or when two points are collapsed by a call to
      \code{unique()} (default behaviour if \code{DistrCollapse} is \code{FALSE});
      ---default \code{FALSE}}
\item{\code{warn.makeDNew}}{controls whether a warning is issued once in
      internal utility \code{.makeDNew} standard integration with
      \code{integrate} throws an error---default \code{TRUE}}
}
} 
\examples{
distroptions("RtoDPQ.e") # returns the value of RtoDPQ.e, by default = 5
currentDistrOptions <- distroptions()
distroptions(RtoDPQ.e = 6)
distroptions("RtoDPQ.e") 
getdistrOption("RtoDPQ.e") 
distroptions(c("WarningSim","WarningArith"))   
getdistrOption("WarningSim")   
distroptions("WarningSim" = FALSE)   
         # switches off warnings as to (In)accuracy due to simulations
distroptions("WarningArith" = FALSE) 
         # switches off warnings as to arithmetics
distroptions(currentDistrOptions)
}
\seealso{\code{\link[base]{options}}, \code{\link[base:options]{getOption}}}
\keyword{distribution}
\concept{global options}
\concept{options}

\name{DiscreteDistribution-class}
\docType{class}
\alias{DiscreteDistribution-class}
\alias{AffLinDiscreteDistribution-class}
\alias{initialize,DiscreteDistribution-method}
\alias{initialize,AffLinDiscreteDistribution-method}
\alias{sqrt,DiscreteDistribution-method}
\alias{coerce,DiscreteDistribution,LatticeDistribution-method}
\alias{coerce,AffLinDiscreteDistribution,LatticeDistribution-method}

\title{Class "DiscreteDistribution"}
\description{The \code{DiscreteDistribution}-class is the mother-class of the class \code{LatticeDistribution}.}
\section{Objects from the Class}{
Objects can be created by calls to \code{new("DiscreteDistribution", ...)}, but more
easily is the use of the generating function \code{"DiscreteDistribution"}.
This generating function, from version 1.9 on, has been moved to this package from package \pkg{distrEx}.
}

\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}: the space of the image of this distribution which has dimension 1
    and the name "Real Space" }
    \item{\code{param}}{Object of class \code{"Parameter"}: the parameter of this distribution, having only the
    slot name "Parameter of a discrete distribution" }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers}
    \item{\code{d}}{Object of class \code{"function"}: density/probability function}
    \item{\code{p}}{Object of class \code{"function"}: cumulative distribution function}
    \item{\code{q}}{Object of class \code{"function"}: quantile function}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{.finSupport}}{logical: used internally to check whether the
     true support is finite; in case \code{img} is one-dimensional, it is
     of length 2 (left and right end).}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"UnivariateDistribution"}, directly.\cr
Class \code{"Distribution"}, by class \code{"UnivariateDistribution"}.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "DiscreteDistribution")}: initialize method }
    \item{coerce}{\code{signature(from = "DiscreteDistribution", 
                   to = "LatticeDistribution")}: coerce method to class \code{"LatticeDistribution"}
                   (checks if support is a lattice)}
    \item{Math}{\code{signature(x = "DiscreteDistribution")}: application of a mathematical function, e.g. \code{sin} or \code{tan} to this discrete distribution
    \itemize{
      \item \code{abs}: \code{signature(x = "DiscreteDistribution")}:  exact image distribution of \code{abs(x)}.
      \item \code{exp}: \code{signature(x = "DiscreteDistribution")}:  exact image distribution of \code{exp(x)}.
      \item \code{sign}: \code{signature(x = "DiscreteDistribution")}:  exact image distribution of \code{sign(x)}.
      \item \code{sqrt}: \code{signature(x = "DiscreteDistribution")}:  exact image distribution of \code{sqrt(x)}.
      \item \code{log}: \code{signature(x = "DiscreteDistribution")}:  (with optional further argument \code{base}, defaulting to \code{exp(1)}) exact image distribution of \code{log(x)}.
      \item \code{log10}: \code{signature(x = "DiscreteDistribution")}:  exact image distribution of \code{log10(x)}.
      \item \code{gamma}: \code{signature(x = "DiscreteDistribution")}:  exact image distribution of \code{gamma(x)}.
      \item \code{lgamma}: \code{signature(x = "DiscreteDistribution")}:  exact image distribution of \code{lgamma(x)}.
      \item \code{digamma}: \code{signature(x = "DiscreteDistribution")}:  exact image distribution of \code{digamma(x)}.
    }}
    \item{-}{\code{signature(e1 = "DiscreteDistribution")}: application of `-' to this discrete distribution}
    \item{*}{\code{signature(e1 = "DiscreteDistribution", e2 = "numeric")}: multiplication of this discrete distribution
    by an object of class `numeric'}
    \item{/}{\code{signature(e1 = "DiscreteDistribution", e2 = "numeric")}: division of this discrete distribution
    by an object of class `numeric'}
    \item{+}{\code{signature(e1 = "DiscreteDistribution", e2 = "numeric")}: addition of this discrete distribution
    to an object of class `numeric'}
    \item{-}{\code{signature(e1 = "DiscreteDistribution", e2 = "numeric")}: subtraction of an object of class `numeric'
    from this discrete distribution }
    \item{*}{\code{signature(e1 = "numeric", e2 = "DiscreteDistribution")}: multiplication of this discrete distribution
    by an object of class `numeric'}
    \item{+}{\code{signature(e1 = "numeric", e2 = "DiscreteDistribution")}: addition of this discrete distribution
    to an object of class `numeric'}
    \item{-}{\code{signature(e1 = "numeric", e2 = "DiscreteDistribution")}: subtraction of this discrete distribution
    from an object of class `numeric'}
    \item{+}{\code{signature(e1 = "DiscreteDistribution", e2 = "DiscreteDistribution")}: Convolution of two discrete
    distributions. The slots p, d and q are approximated on a common grid.}
    \item{-}{\code{signature(e1 = "DiscreteDistribution", e2 = "DiscreteDistribution")}: Convolution of two discrete
    distributions. The slots p, d and q are approximated on a common grid.}
    \item{support}{\code{signature(object = "DiscreteDistribution")}: returns the support}
    \item{p.l}{\code{signature(object = "DiscreteDistribution")}: returns the 
               left continuous cumulative distribution function, i.e.; 
               \eqn{p.l(t) = P(object < t)}{p.l(t) = P(object < t)} }
    \item{q.r}{\code{signature(object = "DiscreteDistribution")}: returns the 
               right-continuous quantile function, i.e.; 
    \eqn{{\rm q.r}(s)=\sup\{t \,\big|\, P({\tt object}\ge t)\leq s\}}{q.r(s)=sup\{t|P(object>=t)<=s\}}}
    \item{plot}{\code{signature(object = "DiscreteDistribution")}: plots density, cumulative distribution and quantile
    function }
  }
}

\section{Internal subclass "AffLinDiscreteDistribution"}{
To enhance accuracy of several functionals on distributions,
  mainly from package \pkg{distrEx}, from version 1.9 of this package on, 
  there is an internally used (but exported) subclass 
  \code{"AffLinDiscreteDistribution"} which has extra slots 
  \code{a}, \code{b} (both of class \code{"numeric"}),  and \code{X0} 
  (of class \code{"DiscreteDistribution"}), to capture the fact 
  that the object has the same distribution as \code{a * X0 + b}. This is 
  the class of the return value of methods 
  \describe{
    \item{-}{\code{signature(e1 = "DiscreteDistribution")}}
    \item{*}{\code{signature(e1 = "DiscreteDistribution", e2 = "numeric")}}
    \item{/}{\code{signature(e1 = "DiscreteDistribution", e2 = "numeric")}}
    \item{+}{\code{signature(e1 = "DiscreteDistribution", e2 = "numeric")}}
    \item{-}{\code{signature(e1 = "DiscreteDistribution", e2 = "numeric")}}
    \item{*}{\code{signature(e1 = "numeric", e2 = "DiscreteDistribution")}}
    \item{+}{\code{signature(e1 = "numeric", e2 = "DiscreteDistribution")}}
    \item{-}{\code{signature(e1 = "numeric", e2 = "DiscreteDistribution")}}
    \item{-}{\code{signature(e1 = "AffLinDiscreteDistribution")}}
    \item{*}{\code{signature(e1 = "AffLinDiscreteDistribution", e2 = "numeric")}}
    \item{/}{\code{signature(e1 = "AffLinDiscreteDistribution", e2 = "numeric")}}
    \item{+}{\code{signature(e1 = "AffLinDiscreteDistribution", e2 = "numeric")}}
    \item{-}{\code{signature(e1 = "AffLinDiscreteDistribution", e2 = "numeric")}}
    \item{*}{\code{signature(e1 = "numeric", e2 = "AffLinDiscreteDistribution")}}
    \item{+}{\code{signature(e1 = "numeric", e2 = "AffLinDiscreteDistribution")}}
    \item{-}{\code{signature(e1 = "numeric", e2 = "AffLinDiscreteDistribution")}}
  }
  There also is a class union of \code{"AffLinAbscontDistribution"},
  \code{"AffLinDiscreteDistribution"}, \code{"AffLinUnivarLebDecDistribution"}
  and called \code{"AffLinDistribution"}
  which is used for functionals.
}
\section{Internal virtual superclass "AcDcLcDistribution"}{
As many operations should be valid no matter whether the operands
are of class \code{"AbscontDistribution"},
  \code{"DiscreteDistribution"}, or \code{"UnivarLebDecDistribution"},
there is a class union of these classes called \code{"AcDcLcDistribution"};
in partiucalar methods for \code{"*"}, \code{"/"}, 
\code{"^"} (see \link{operators-methods}) and methods
\code{\link{Minimum}}, \code{Maximum}, \code{\link{Truncate}}, and
 \code{\link{Huberize}}, and \code{\link{convpow}} are defined for this 
 class union.   
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }

\note{ Working with a computer, we use a finite interval as support which 
       carries at least mass \code{1-getdistrOption("TruncQuantile")}. \cr
       
       Also, we require that support points have distance at least       
       \code{getdistrOption("DistrResoltion")}, if this condition fails,
       upon a suggestion by Jacob van Etten, \email{jacobvanetten@yahoo.com},
       we use the global option \code{getdistrOption("DistrCollapse")} to
       decide whether we use collapsing or not. If we do so, we collapse support 
       points if they are too close to each other, taking
       the (left most) median among them as new support point which accumulates
       all the mass of the collapsed points. 
       With \code{getdistrOption("DistrCollapse")==FALSE}, we at least collapse
       points according to the result of \code{unique()}, and if after this
       collapsing, the minimal distance is less than \code{getdistrOption("DistrResoltion")},
       we throw an error. By \code{getdistrOption("DistrCollapse.Unique.Warn")},
       we control, whether we throw a warning upon collapsing or not.       
       }

\seealso{
\code{\link{Parameter-class}}
\code{\link{UnivariateDistribution-class}}
\code{\link{LatticeDistribution-class}}
\code{\link{AbscontDistribution-class}}
\code{\link{Reals-class}}
\code{\link{RtoDPQ.d}}
}
\examples{
# Dirac-measure at 0
D1 <- DiscreteDistribution(supp = 0)
support(D1)

# simple discrete distribution
D2 <- DiscreteDistribution(supp = c(1:5), prob = c(0.1, 0.2, 0.3, 0.2, 0.2))
plot(D2)
(pp <- p(D2)(support(D2)))
p(D2)(support(D2)-1e-5)
p(D2)(support(D2)+1e-5)
p.l(D2)(support(D2))
p.l(D2)(support(D2)-1e-5)
p.l(D2)(support(D2)+1e-5)
q(D2)(pp)
q(D2)(pp-1e-5)
q(D2)(pp+1e-5)
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
q.r(D2)(pp)
q.r(D2)(pp-1e-5)
q.r(D2)(pp+1e-5)
}
\keyword{distribution}
\concept{discrete distribution}
\concept{lattice distribution}
\concept{support of a distribution}
\concept{S4 distribution class}

\name{NbinomParameter-class} 
\docType{class}
\alias{NbinomParameter-class}
\alias{initialize,NbinomParameter-method}

\title{Class "NbinomParameter"}
\description{ The parameter of a negative binomial distribution, used by Nbinom-class}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("NbinomParameter", prob, size)}.
Usually an object of this class is not needed on its own, it is generated automatically when an object of the class Nbinom
is prepared. 
}
\section{Slots}{
  \describe{
    \item{\code{prob}}{Object of class \code{"numeric"}: the probability of a negative binomial distribution }
    \item{\code{size}}{Object of class \code{"numeric"}: the size of a negative binomial distribution }
    \item{\code{name}}{Object of class \code{"character"}: a name / comment for the parameters }
  }
}
\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "NbinomParameter")}: initialize method }
    \item{prob}{\code{signature(object = "NbinomParameter")}: returns the slot \code{prob} of the parameter of the
    distribution }
    \item{prob<-}{\code{signature(object = "NbinomParameter")}: modifies the slot \code{prob} of the parameter of the
    distribution }
    \item{size}{\code{signature(object = "NbinomParameter")}: returns the slot \code{size} of the parameter of the distribution }
    \item{size<-}{\code{signature(object = "NbinomParameter")}: modifies the slot \code{size} of the parameter of the
    distribution }
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}

\seealso{
\code{\link{Nbinom-class}}
\code{\link{Parameter-class}}
}

\examples{
W <- new("NbinomParameter",prob=0.5,size=1)
size(W) # size of this distribution is 1.
size(W) <- 2 # size of this distribution is now 2.
}
\keyword{distribution}
\concept{parameter}
\concept{Negative Binomial distribution}
\concept{S4 parameter class}

\name{dimension-methods}
\docType{methods}
\alias{dimension-methods}
\alias{dimension<--methods}
\alias{dimension}
\alias{dimension<-}
\alias{dimension,EuclideanSpace-method}
\alias{dimension<-,EuclideanSpace-method}

\title{ Methods for Function dimension in Package `distr' }
\description{dimension-methods}
\section{Methods}{\describe{
    \item{dimension}{\code{signature(object = "EuclideanSpace")}: returns the dimension of the space }
    \item{dimension<-}{\code{signature(object = "EuclideanSpace")}: modifies the dimension of the space }
}}
\keyword{methods}
\concept{dimension}
\concept{space}
\concept{S4 space class}
\concept{accessor function}
\concept{replacement function}

\name{Nbinom-class} 
\docType{class}
\alias{Nbinom-class}
\alias{Nbinom}
\alias{initialize,Nbinom-method}

\title{Class "Nbinom" }
\description{
  The negative binomial distribution with \code{size} \eqn{= n}, by default \eqn{=1}, and
  \code{prob} \eqn{= p}, by default \eqn{=0.5}, has density
  \deqn{
    d(x) = \frac{\Gamma(x+n)}{\Gamma(n) x!} p^n (1-p)^x}{%
    d(x) = Gamma(x+n)/(Gamma(n) x!) p^n (1-p)^x}
  for \eqn{x = 0, 1, 2, \ldots}

  This represents the number of failures
  which occur in a sequence of Bernoulli trials before a target number
  of successes is reached.
  C.f. \code{\link[stats:NegBinomial]{rnbinom}} 
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Nbinom(prob, size)}.
This object is a negative binomial distribution. 
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Naturals"}: The space of the image of this
     distribution has got dimension 1 and the name "Natural Space". }
    \item{\code{param}}{Object of class \code{"NbinomParameter"}: the parameter of this distribution (prob, size),
     declared at its instantiation }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers (calls function \code{rnbinom}) }
    \item{\code{d}}{Object of class \code{"function"}: density function (calls function \code{dnbinom}) }
    \item{\code{p}}{Object of class \code{"function"}: cumulative function (calls function \code{pnbinom}) }
    \item{\code{q}}{Object of class \code{"function"}: inverse of the cumulative function (calls function \code{qnbinom}).
  The quantile is defined as the smallest value \eqn{x} such that
  \eqn{F(x) \ge p}, where \eqn{F} is the distribution function.}
    \item{\code{support}}{Object of class \code{"numeric"}: a (sorted) vector containing the support of the discrete
    density function}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"DiscreteDistribution"}, directly.\cr
Class \code{"UnivariateDistribution"}, by class \code{"DiscreteDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"DiscreteDistribution"}.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Nbinom")}: initialize method }
    \item{prob}{\code{signature(object = "Nbinom")}: returns the slot \code{prob} of the parameter of the distribution }
    \item{prob<-}{\code{signature(object = "Nbinom")}: modifies the slot \code{prob} of the parameter of the distribution }
    \item{size}{\code{signature(object = "Nbinom")}: returns the slot \code{size} of the parameter of the distribution }
    \item{size<-}{\code{signature(object = "Nbinom")}: modifies the slot \code{size} of the parameter of the distribution }
    \item{+}{\code{signature(e1 = "Nbinom", e2 = "Nbinom")}: 
    For the negative binomial distribution we use its closedness under convolutions.}
  }
}

\author{
 Thomas Stabla \email{statho3@web.de},\cr 
 Florian Camphausen \email{fcampi@gmx.de},\cr
 Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
 Matthias Kohl \email{Matthias.Kohl@stamats.de}}

\note{Working with a computer, we use a finite interval as support which carries at least mass \code{1-getdistrOption("TruncQuantile")}. }

\seealso{
\code{\link{NbinomParameter-class}}
\code{\link{Geom-class}}
\code{\link{DiscreteDistribution-class}}
\code{\link{Naturals-class}}
\code{\link[stats:NegBinomial]{rnbinom}}
}
\examples{
N <- Nbinom(prob = 0.5, size = 1) # N is a binomial distribution with prob=0.5 and size=1.
r(N)(1) # one random number generated from this distribution, e.g. 3
d(N)(1) # Density of this distribution is  0.25 for x=1.
p(N)(0.4) # Probability that x<0.4 is 0.5.
q(N)(.1) # x=0 is the smallest value x such that p(B)(x)>=0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
size(N) # size of this distribution is 1.
size(N) <- 2 # size of this distribution is now 2.
}
\keyword{distribution}
\concept{discrete distribution}
\concept{lattice distribution}
\concept{Negative Binomial distribution}
\concept{S4 parameter class}
\concept{generating function}

\name{LnormParameter-class} 
\docType{class}
\alias{LnormParameter-class}
\alias{initialize,LnormParameter-method}

\title{Class "LnormParameter"}
\description{ The parameter of a log normal distribution, used by Lnorm-class}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("LnormParameter", meanlog, sdlog)}.
Usually an object of this class is not needed on its own, it is generated automatically when an object of the class Lnorm
is instantiated. 
}
\section{Slots}{
  \describe{
    \item{\code{meanlog}}{Object of class \code{"numeric"}: the mean of a log normal distribution }
    \item{\code{sdlog}}{Object of class \code{"numeric"}: the sd of a log normal distribution }
    \item{\code{name}}{Object of class \code{"character"}: a name / comment for the parameters }
  }
}
\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "LnormParameter")}: initialize method }
    \item{sdlog}{\code{signature(object = "LnormParameter")}: returns the slot sdlog of the parameter of the distribution }
    \item{sdlog<-}{\code{signature(object = "LnormParameter")}: modifies the slot sdlog of the parameter of the
    distribution }
    \item{meanlog}{\code{signature(object = "LnormParameter")}: returns the slot meanlog of the parameter of the
    distribution}
    \item{meanlog<-}{\code{signature(object = "LnormParameter")}: modifies the slot meanlog of the parameter of the
    distribution }
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }



\seealso{
\code{\link{Lnorm-class}}
\code{\link{Parameter-class}}
}

\examples{
W <- new("LnormParameter",sdlog=1,meanlog=0)
meanlog(W) # meanlog of this distribution is 0.
meanlog(W) <- 2 # meanlog of this distribution is now 2.
}
\keyword{distribution}
\concept{parameter}
\concept{Log-Normal distribution}
\concept{S4 parameter class}

\name{Integer-class}
\docType{class}
\alias{Integer-class}
\alias{coerce,numeric,Integer-method}

\title{Internal Class "Integer"}
\description{For the ease of method dispatch, there is an internal
S4 class \code{Integer}, which is a subclass of \code{numeric} and has a 
straightforward validity method.}
\section{Objects from the Class}{
new("Integer",
}
\section{Slots}{
\describe{
    \item{\code{.Data}}{Object of class \code{"numeric"}}
  }
}
\section{Extends}{
Class \code{"\linkS4class{numeric}"}, from data part.
Class \code{"\linkS4class{vector}"}, by class "numeric", distance 2.
}
\section{Methods}{
\describe{   \item{coerce}{\code{signature(from = "numeric", to = "Integer")}:
      create a \code{"Integer"} object from a \code{"numeric"} vector.}
}}
%\references{ ~put references to the literature/web site here ~ }
\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}
}
%\note{ ~~further notes~~ }
\seealso{
\code{\link{numeric}}, \code{\link{vector}}
}
%\examples{}
\keyword{classes}
\keyword{internal}

\name{df2-methods}
\docType{methods}
\alias{df2-methods}
\alias{df2<--methods}
\alias{df2}
\alias{df2<-}
\alias{df2,FParameter-method}
\alias{df2,Fd-method}
\alias{df2<-,FParameter-method}
\alias{df2<-,Fd-method}
\title{ Methods for Function df2 in Package `distr' }
\description{df-methods}

\section{Methods}{\describe{

\item{df2}{\code{signature(object = "FParameter")}: returns the slot \code{df2} of the parameter of an F-distribution }
\item{df2<-}{\code{signature(object = "FParameter")}: modifies the slot \code{df2} of the parameter of an F-distribution }

\item{df2}{\code{signature(object  = "Fd")}: returns the slot \code{df2} of the slot \code{param} of the distribution}
\item{df2<-}{\code{signature(object = "Fd")}: modifies the slot \code{df2} of the slot \code{param} of the distribution }
}}
\keyword{distribution}
\keyword{methods}
\concept{degree of freedom}
\concept{parameter}
\concept{F distribution}
\concept{accessor function}
\concept{replacement function}

\name{Reals-class}
\docType{class}
\alias{Reals}
\alias{Reals-class}
\alias{initialize,Reals-method}

\title{Class "Reals" }
\description{ Particular case of a one-dimensional Euclidean Space }
\section{Objects from the Class}{
Objects could theoretically be created by calls of the form \code{new("Reals", dimension, name)}.
Usually an object of this class is not needed on its own. It is generated automatically when a univariate absolutly
continuous distribution is instantiated. 
}
\section{Slots}{
  \describe{
    \item{\code{dimension}}{Object of class \code{"character"}: the dimension of the space, by default = 1 }
    \item{\code{name}}{Object of class \code{"character"}: the name of the space, by default = "Real Space"}
  }
}
\usage{Reals()}

\section{Extends}{
Class \code{"EuclideanSpace"}, directly.\cr
Class \code{"rSpace"}, by class \code{"EuclideanSpace"}.
}

\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Reals")}: initialize method }
}
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }


\seealso{
\code{\link{EuclideanSpace-class}}
\code{\link{Naturals-class}}
\code{\link{AbscontDistribution-class}}
}

\examples{
R <- Reals()
liesIn(R,c(0,0)) # FALSE
}


\keyword{distribution}
\concept{space}
\concept{set}
\concept{S4 space class}

\name{Lattice-class}
\docType{class}
\alias{Lattice-class}
\alias{Lattice}

\title{Class "Lattice" }
\description{Class \code{Lattice} formalizes an affine linearly generated grid
   of (support) points  \code{pivot + (0:(Length-1)) * width};
 this is used for subclass \code{LatticeDistribution} of class
 \code{DiscreteDistribution} which in addition to the latter contains a slot 
 \code{lattice} of class \code{Lattice}. 
 }

\section{Objects from the Class}{
Objects may be generated by calling the generating function \code{Lattice}.
}

\usage{
  Lattice(pivot = 0, width = 1, Length = 2, name = "a lattice")
}
\arguments{
  \item{pivot}{the (finite) utmost left or right value of the lattice}
  \item{width}{the (finite) grid-width; if negative the lattice is expanded to 
               the left, else to the right}
  \item{Length}{the (possibly infinite) length of the lattice}
  \item{name}{the (possibly empty) name of the lattice 
              (inherited from class \code{rSpace})}
}

\section{Slots}{
  \describe{
    \item{\code{pivot}}{Object of class \code{"numeric"}: 
                         --- the pivot of the lattice; must be of length 1}
    \item{\code{width}}{Object of class \code{"numeric"}: 
                         --- the width of the lattice; must be of length 1 and 
                         must not be 0}
    \item{\code{Length}}{Object of class \code{"numeric"}: 
                          --- the width of the lattice; must be an integer > 0 
                          of length 1}
    \item{\code{name}}{Object of class \code{"character"}: 
                        the name of the space, by default = "a lattice"}
  }
}
\section{Extends}{
Class \code{"rSpace"}, directly.\cr
}

\section{Methods}{
  \describe{
    \item{pivot}{\code{signature(.Object = "Lattice")}: 
         returns the \code{'pivot'} slot}
    \item{pivot<-}{\code{signature(.Object = "Lattice")}:
         modifies the \code{'pivot'} slot}
    \item{width}{\code{signature(.Object = "Lattice")}: 
         returns the \code{'width'} slot }
    \item{width<-}{\code{signature(.Object = "Lattice")}: 
         modifies the \code{'width'} slot}
    \item{Length}{\code{signature(.Object = "Lattice")}: 
         returns the \code{'Length'} slot }
    \item{Length<-}{\code{signature(.Object = "Lattice")}: 
         modifies the \code{'Length'} slot }
}
}

\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}

\seealso{
\code{\link{rSpace-class}}
\code{\link{LatticeDistribution-class}}
}

\examples{
L <- Lattice(pivot = 0, width = 1, Length = Inf, name = "the Naturals")
name(L)
pivot(L) <- 1 ### now starting from 1
}

\keyword{distribution}
\concept{space}
\concept{S4 space class}
\concept{set}

\name{Min-methods}
\docType{methods}
\alias{Min-methods}
\alias{Min<--methods}
\alias{Min}
\alias{Min<-}
\alias{Min,UnifParameter-method}
\alias{Min,Unif-method}
\alias{Min<-,UnifParameter-method}
\alias{Min<-,Unif-method}
\title{ Methods for Function Min in Package `distr' }
\description{Min-methods}

\section{Methods}{\describe{

\item{Min}{\code{signature(object = "UnifParameter")}: returns the slot \code{Min} of the parameter of the distribution }
\item{Min<-}{\code{signature(object = "UnifParameter")}: modifies the slot \code{Min} of the parameter of the distribution }

\item{Min}{\code{signature(object = "Unif")}: returns the slot \code{Min} of the parameter of the distribution }
\item{Min<-}{\code{signature(object = "Unif")}: modifies the slot \code{Min} of the parameter of the distribution }
}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{Uniform distribution}
\concept{S4 parameter class}
\concept{accessor function}
\concept{replacement function}

\name{k-methods}
\docType{methods}
\alias{k-methods}
\alias{k<--methods}
\alias{k}
\alias{k<-}
\alias{k,HyperParameter-method}
\alias{k,Hyper-method}
\alias{k<-,HyperParameter-method}
\alias{k<-,Hyper-method}
\title{ Methods for Function k in Package `distr' }
\description{k-methods}

\section{Methods}{\describe{

\item{k}{\code{signature(object = "HyperParameter")}: returns the slot \code{k} of the parameter of the distribution }
\item{k<-}{\code{signature(object = "HyperParameter")}: modifies the slot \code{k} of the parameter of the distribution }

\item{k}{\code{signature(object = "Hyper")}: returns the slot \code{k} of the parameter of the distribution }
\item{k<-}{\code{signature(object = "Hyper")}: modifies the slot \code{k} of the parameter of the distribution }
}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{Hypergeometric distribution}
\concept{S4 parameter class}
\concept{accessor function}
\concept{replacement function}

\name{qqplot}
\docType{methods}
\alias{qqplot}
\alias{qqplot-methods}
\alias{qqplot,ANY,ANY-method}
\alias{qqplot,UnivariateDistribution,UnivariateDistribution-method}

\title{Methods for Function qqplot in Package `distr'}
\usage{
qqplot(x, y, ...)
\S4method{qqplot}{UnivariateDistribution,UnivariateDistribution}(x, y,
    n = 30, withIdLine = TRUE, withConf = TRUE,
    withConf.pw  = withConf,  withConf.sim = withConf,
    plot.it = TRUE, xlab = deparse(substitute(x)),
    ylab = deparse(substitute(y)), ...,
    width = 10, height = 5.5, withSweave = getdistrOption("withSweave"),
    mfColRow = TRUE, n.CI = n, col.IdL = "red", lty.IdL = 2, lwd.IdL = 2,
    alpha.CI = .95, exact.pCI = (n<100), exact.sCI = (n<100), nosym.pCI = FALSE,
    col.pCI = "orange", lty.pCI = 3, lwd.pCI = 2, pch.pCI = par("pch"),
    cex.pCI = par("cex"),
    col.sCI = "tomato2", lty.sCI = 4, lwd.sCI = 2, pch.sCI = par("pch"),
    cex.sCI = par("cex"),
    cex.pch = par("cex"), col.pch = par("col"),
    jit.fac = 0, check.NotInSupport = TRUE,
    col.NotInSupport = "red", with.legend = TRUE, legend.bg = "white",
    legend.pos = "topleft", legend.cex = 0.8, legend.pref = "", 
    legend.postf = "", legend.alpha = alpha.CI, debug = FALSE, withSubst = TRUE)
\S4method{qqplot}{ANY,ANY}(x, y,
    plot.it = TRUE, xlab = deparse(substitute(x)),
    ylab = deparse(substitute(y)), ...)
}

\arguments{
\item{x}{object of class \code{"ANY"} (\pkg{stats}-method) or
of code \code{"UnivariateDistribution"}; to be compared to \code{y}.}
\item{y}{object of class \code{"ANY"} (\pkg{stats}-method) or
of class \code{"UnivariateDistribution"}.}
\item{n}{numeric; number of quantiles at which to do the comparison.}
\item{withIdLine}{logical; shall line \code{y = x} be plotted in?}
\item{withConf}{logical; shall confidence lines be plotted?}
\item{withConf.pw}{logical; shall pointwise confidence lines be plotted?}
\item{withConf.sim}{logical; shall simultaneous confidence lines be plotted?}
\item{plot.it}{logical; shall be plotted at all (inherited from \code{\link[stats:qqnorm]{qqplot}})?}
\item{xlab}{x-label}
\item{ylab}{y-label}
\item{\dots}{further parameters for function \code{plot}}
\item{width}{width (in inches) of the graphics device opened}
\item{height}{height (in inches) of the graphics device opened}
\item{withSweave}{logical: if \code{TRUE} (for working with \command{Sweave})
        no extra device is opened and height/width are not set}
\item{mfColRow}{shall default partition in panels be used --- defaults to \code{TRUE}}
\item{n.CI}{numeric; number of points to be used for confidence interval}
\item{col.IdL}{color for the identity line}
\item{lty.IdL}{line type for the identity line}
\item{lwd.IdL}{line width for the identity line}
\item{alpha.CI}{confidence level}
\item{exact.pCI}{logical; shall pointwise CIs be determined with exact Binomial distribution?}
\item{exact.sCI}{logical; shall simultaneous CIs be determined with exact kolmogorov distribution?}
\item{nosym.pCI}{logical; shall we use (shortest) asymmetric CIs?}
\item{col.pCI}{color for the pointwise CI}
\item{lty.pCI}{line type for the pointwise CI}
\item{lwd.pCI}{line width for the pointwise CI}
\item{pch.pCI}{symbol for points (for discrete mass points) in pointwise CI}
\item{cex.pCI}{magnification factor for points (for discrete mass points) in pointwise CI}
\item{col.sCI}{color for the simultaneous CI}
\item{lty.sCI}{line type for the simultaneous CI}
\item{lwd.sCI}{line width for the simultaneous CI}
\item{pch.sCI}{symbol for points (for discrete mass points) in simultaneous CI}
\item{cex.sCI}{magnification factor for points (for discrete mass points) in simultaneous CI}
\item{cex.pch}{magnification factor for the plotted symbols}
\item{col.pch}{color for the plotted symbols}
\item{jit.fac}{jittering factor used for discrete distributions}
\item{check.NotInSupport}{logical; shall we check if all \code{x}-quantiles lie in \code{support(y)}?}
\item{col.NotInSupport}{logical; if preceding check \code{TRUE} color of \code{x}-quantiles if not in \code{support(y)}}
\item{with.legend}{logical; shall a legend be plotted?}
\item{legend.bg}{background color for the legend}
\item{legend.pos}{position for the legend}
\item{legend.cex}{magnification factor for the legend}
\item{legend.pref}{character to be prepended to legend text}
\item{legend.postf}{character to be appended to legend text}
\item{legend.alpha}{nominal coverage probability}
\item{debug}{logical; if \code{TRUE} additional output to debug confidence bounds.}
\item{withSubst}{logical; if \code{TRUE} (default) pattern substitution for
      titles and lables is used; otherwise no substitution is used. }
}

\description{
  We generalize function \code{\link[stats:qqnorm]{qqplot}} from package \pkg{stats} to
  be applicable to distribution objects.  In this context,
  \code{qqplot} produces a QQ plot of two distributions, i.e.; argument \code{x}
  is the distribution to be checked for compatibility, and \code{y} is the
  model (\eqn{H_0}{H0}-)distribution.
   Graphical parameters may be given as arguments to \code{qqplot}.
   The \pkg{stats} function
  is just the method for signature \code{x=ANY,y=ANY}.
  In all title and axis label arguments, if \code{withSubst} is \code{TRUE},
  the following patterns are substituted:
\describe{
\item{\code{"\%C"}}{class of argument \code{x}}
\item{\code{"\%A"}}{deparsed argument \code{x}}
\item{\code{"\%D"}}{time/date-string when the plot was generated}
}
}
\details{
\describe{
\item{qqplot}{\code{signature(x = "ANY", y = "ANY")}: function \code{qqplot} from
package \pkg{stats}.}
\item{qqplot}{\code{signature(x = "UnivariateDistribution", y = "UnivariateDistribution")}:
produces a QQ plot for two univariate distributions.}
}
}
\value{
  A list of elements containing the information needed to compute the 
  respective QQ plot, in particular it extends the elements of the 
  return value of function \code{\link[stats:qqnorm]{qqplot}} 
  from package \pkg{stats}, i.e., a
  list with components \code{x} and \code{y} for x and y coordinates
  of the plotted points; more specifically it contains
  \item{x}{The x coordinates of the points that were/would be plotted}
  \item{y}{The corresponding quantiles of the second distribution,
           \emph{including \code{\link{NA}}s}.}
  \item{crit}{A matrix with the lower and upper confidence bounds
               (computed by \code{qqbounds}).}
  \item{err}{logical vector of length 2.}
  (elements \code{crit} and \code{err} are taken from the return
   value(s) of \code{qqbounds}).
   The return value allows to recover all information used to produce the plot
   for later use in enhanced graphics (e.g. with ggplot).   
}
\references{
  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
  \emph{The New S Language}.
  Wadsworth & Brooks/Cole.
}
\seealso{
  \code{\link[stats:qqnorm]{qqplot}} from package \pkg{stats} -- the standard QQ plot
  function and
  \code{\link{qqbounds}}, used by \code{qqplot} to produce confidence
   intervals.
}
\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}
}
\examples{
## IGNORE_RDIFF_BEGIN
qqplot(Norm(15,sqrt(30)), Chisq(df=15))
## some discrete Distributions:
P <- Pois(5)
B <- Binom(size=2000,prob=5/2000)
qqplot(B,P)
## IGNORE_RDIFF_END
\donttest{
## takes too much time for R CMD check --as-cran
qqplot(B,P, nosym.pCI=TRUE)
}
## some Lebesgue-Decomposed distributions:
mylist <- UnivarLebDecDistribution(discretePart=Binom(3,.3), acPart=Norm(2,2),
               acWeight=11/20)
mylist2 <- mylist+0.1

## IGNORE_RDIFF_BEGIN
qqplot(mylist,mylist2)
qqplot(mylist,mylist2,exact.pCI=FALSE,exact.sCI=FALSE)
## IGNORE_RDIFF_END

\donttest{
## takes too much time for R CMD check --as-cran
qqplot(mylist,mylist2,nosym.pCI=TRUE)
## some ac. distribution with a gap
mylist3 <- UnivarMixingDistribution(Unif(0,0.3),Unif(0.6,1),mixCoeff=c(0.8,0.2))
gaps(mylist3)
mylist4 <- UnivarMixingDistribution(Unif(0,0.3),Unif(0.6,1),mixCoeff=c(0.6,0.4))
qqplot(mylist3,mylist4)
qqplot(mylist3,mylist4,nosym.pCI=TRUE)
}
}
\keyword{hplot}
\keyword{distribution}

\name{Pois-class}
\docType{class}
\alias{Pois-class}
\alias{Pois}
\alias{initialize,Pois-method}
\title{Class "Pois" }
\description{
  The Poisson distribution has density
%-- please leave the linebreaking for the next two ! --
  \deqn{p(x) = \frac{\lambda^x e^{-\lambda}}{x!}}{%
        p(x) = lambda^x exp(-lambda)/x!}
  for \eqn{x = 0, 1, 2, \ldots}.  The mean and variance are
  \eqn{E(X) = Var(X) = \lambda}.

  C.f. \code{\link[stats:Poisson]{rpois}}
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Pois(lambda)}.
This object is a Poisson distribution. 
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Naturals"}:
      The space of the image of this distribution has got dimension 1
      and the name "Natural Space".}
    \item{\code{param}}{Object of class \code{"PoisParameter"}:
      the parameter of this distribution (lambda), declared at its
      instantiation}
    \item{\code{r}}{Object of class \code{"function"}:
      generates random numbers (calls function rpois)}
    \item{\code{d}}{Object of class \code{"function"}:
      density function (calls function dpois)}
    \item{\code{p}}{Object of class \code{"function"}:
      cumulative function (calls function ppois)}
    \item{\code{q}}{Object of class \code{"function"}:
      inverse of the cumulative function (calls function qpois).
      The quantile is defined as the smallest value \eqn{x} such that
      \eqn{F(x) \ge p}, where \eqn{F} is the distribution function.}
    \item{\code{support}}{Object of class \code{"numeric"}: a (sorted)
      vector containing the support of the discrete density function}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"DiscreteDistribution"}, directly.
Class \code{"UnivariateDistribution"}, by class \code{"DiscreteDistribution"}.
Class \code{"Distribution"}, by class \code{"DiscreteDistribution"}.
}
\section{Methods}{
  \describe{
    \item{+}{\code{signature(e1 = "Pois", e2 = "Pois")}:
      For the Poisson distribution the exact convolution formula is
      implemented thereby improving the general numerical approximation.}
    \item{initialize}{\code{signature(.Object = "Pois")}:
      initialize method}
    \item{lambda}{\code{signature(object = "Pois")}:
      returns the slot lambda of the parameter of the distribution}
    \item{lambda<-}{\code{signature(object = "Pois")}:
      modifies the slot lambda of the parameter of the distribution}
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
}
\note{Working with a computer, we use a finite interval as support which carries at least mass \code{1-getdistrOption("TruncQuantile")}. }
\seealso{
\code{\link{PoisParameter-class}}
\code{\link{DiscreteDistribution-class}}
\code{\link{Naturals-class}}
\code{\link[stats:Poisson]{rpois}}
}
\examples{
P <- Pois(lambda = 1) # P is a Poisson distribution with lambda = 1.
r(P)(1) # one random number generated from this distribution, e.g. 1
d(P)(1) # Density of this distribution is 0.3678794 for x = 1.
p(P)(0.4) # Probability that x < 0.4 is 0.3678794.
q(P)(.1) # x = 0 is the smallest value x such that p(B)(x) >= 0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
lambda(P) # lambda of this distribution is 1.
lambda(P) <- 2 # lambda of this distribution is now 2.
R <- Pois(lambda = 3) # R is a Poisson distribution with lambda = 2.
S <- P + R # R is a Poisson distribution with lambda = 5(=2+3).
}
\keyword{distribution}
\concept{discrete distribution}
\concept{lattice distribution}
\concept{Poisson distribution}
\concept{S4 parameter class}
\concept{generating function}

\name{DistrSymmList}
\alias{DistrSymmList}

\title{Generating function for DistrSymmList-class}
\description{
  Generates an object of class \code{"DistrSymmList"}.
}
\usage{
DistrSymmList(...)
}
\arguments{
  \item{\dots}{ Objects of class \code{"DistributionSymmetry"} which
    shall form the list of symmetry types. }
}
%\details{}
\value{ Object of class \code{"DistrSymmList"} }
%\references{}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de}}
%\note{}
\seealso{\code{\link{DistrSymmList-class}}}
\examples{
DistrSymmList(NoSymmetry(), SphericalSymmetry(SymmCenter = 1), 
              EllipticalSymmetry(SymmCenter = 2))

## The function is currently defined as
function (...){
    new("DistrSymmList", list(...))
}
}
\concept{symmetry}
\keyword{distribution}

\name{sqrt-methods}
\docType{methods}
\alias{sqrt-methods}
\alias{sqrt}
\alias{sqrt,PosSemDefSymmMatrix-method}
\title{ Methods for Function sqrt in Package `distr' }
\description{sqrt-methods using generalized inverses for p.s.d. matrices}

\usage{
\S4method{sqrt}{PosSemDefSymmMatrix}(x)
}
\arguments{
  \item{x}{a p.s.d. matrix (of class \code{PosSemDefSymmMatrix}}
}
\section{Methods}{\describe{
\item{sqrt}{\code{signature(x = "PosSemDefSymmMatrix")}: produces a symmetric,
 p.s.d. matrix \eqn{y} such that  \eqn{x = y^2}.}
}}
\author{Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}
\seealso{
\code{\link{solve}}}
\keyword{algebra}
\keyword{array}
\concept{pseudo inverse}
\concept{inverse}
\concept{svd}
\concept{eigenvalue decomposition}

\name{Unif-class} 
\docType{class}
\alias{Unif-class}
\alias{Unif}
\alias{initialize,Unif-method}

\title{Class "Unif"}
\description{ 
  The uniform distribution has density
  \deqn{d(x) = \frac{1}{max-min}}{d(x) = 1/(max-min)}
  for \eqn{min}, by default \eqn{=0}, \eqn{\le x \le max}, by default \eqn{=1}.
C.f. \code{\link[stats:Uniform]{runif}} 
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Unif(Min, Max)}.
This object is a uniform distribution.
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}: The space of the image of this distribution has got dimension 1
    and the name "Real Space". }
    \item{\code{param}}{Object of class \code{"UnifParameter"}: the parameter of this distribution (Min and Max),
     declared at its instantiation }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers (calls function \code{runif})}
    \item{\code{d}}{Object of class \code{"function"}: density function (calls function \code{dunif})}
    \item{\code{p}}{Object of class \code{"function"}: cumulative function (calls function \code{punif})}
    \item{\code{q}}{Object of class \code{"function"}: inverse of the cumulative function (calls function \code{qunif})}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"AbscontDistribution"}, directly.\cr
Class \code{"UnivariateDistribution"}, by class \code{"AbscontDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"AbscontDistribution"}.
}
\section{Is-Relations}{
By means of \code{setIs}, R ``knows'' that a distribution object \code{obj} of class \code{"Unif"} with Min 0 and Max 1 also is
a Beta distribution with parameters \code{shape1 = 1, shape2 = 1, ncp = 0}. 
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Unif")}: initialize method }
    \item{Min}{\code{signature(object = "Unif")}: returns the slot \code{Min} of the parameter of the distribution }
    \item{Min<-}{\code{signature(object = "Unif")}: modifies the slot \code{Min} of the parameter of the distribution }
    \item{Max}{\code{signature(object = "Unif")}: returns the slot \code{Max} of the parameter of the distribution }
    \item{Max<-}{\code{signature(object = "Unif")}: modifies the slot \code{Max} of the parameter of the distribution }
    \item{*}{\code{signature(e1 = "Unif", e2 = "numeric")}: multiplication of this uniform distribution
    by an object of class `numeric'}
    \item{+}{\code{signature(e1 = "Unif", e2 = "numeric")}: addition of this uniform distribution
    to an object of class `numeric'}
}}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}


\seealso{
\code{\link{UnifParameter-class}}
\code{\link{AbscontDistribution-class}}
\code{\link{Reals-class}}
\code{\link[stats:Uniform]{runif}}

}
\examples{
U <- Unif(Min=0,Max=2) # U is a uniform distribution with Min=0 and Max=2.
r(U)(1) # one random number generated from this distribution, e.g. 1.984357
d(U)(1) # Density of this distribution is 0.5 for x=1.
p(U)(1) # Probability that x<1 is 0.5.
q(U)(.1) # Probability that x<0.2 is 0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
Min(U) # Min of this distribution is 0.
Min(U) <- 1 # Min of this distribution is now 1.
Min(U) # Min of this distribution is 1.
Min(U) <- 0
is(U/2,"Beta") # yes
V <- U/2; as(V,"Beta")
}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{Uniform distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{LogisParameter-class} 
\docType{class}
\alias{LogisParameter-class}
\alias{initialize,LogisParameter-method}

\title{Class "LogisParameter"}
\description{ The parameter of a logistic distribution, used by \code{Logis}-class}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("LogisParameter", scale, location)}.
Usually an object of this class is not needed on its own, it is generated automatically when an object of the class 
\code{Logis} is instantiated. 
}
\section{Slots}{
  \describe{
    \item{\code{scale}}{Object of class \code{"numeric"}: the scale of a logistic distribution }
    \item{\code{location}}{Object of class \code{"numeric"}: the location of a logistic distribution }
    \item{\code{name}}{Object of class \code{"character"}: a name / comment for the parameters }
  }
}
\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "LogisParameter")}: initialize method }
    \item{location}{\code{signature(object = "LogisParameter")}: returns the slot \code{location} of the parameter of the
    distribution }
    \item{location<-}{\code{signature(object = "LogisParameter")}: modifies the slot \code{location} of the parameter of the
    distribution }
    \item{scale}{\code{signature(object = "LogisParameter")}: returns the slot \code{scale} of the parameter of the distribution }
    \item{scale<-}{\code{signature(object = "LogisParameter")}: modifies the slot \code{scale} of the parameter of the
    distribution }
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }


\seealso{
\code{\link{Logis-class}}
\code{\link{Parameter-class}}
}

\examples{
W <- new("LogisParameter",location=0,scale=1)
scale(W) # scale of this distribution is 1.
scale(W) <- 2 # scale of this distribution is now 2.
}
\keyword{distribution}
\concept{parameter}
\concept{Logistic distribution}
\concept{S4 parameter class}
\concept{location scale family}

\name{Symmetry-class}
\docType{class}
\alias{Symmetry-class}
\alias{show,Symmetry-method}
\alias{type}
\alias{type,Symmetry-method}
\alias{SymmCenter}
\alias{SymmCenter,Symmetry-method}

\title{Class of Symmetries}
\description{Class of symmetries of various objects.}
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}
\section{Slots}{
  \describe{
    \item{\code{type}}{Object of class \code{"character"}:
      discribes type of symmetry. }
    \item{\code{SymmCenter}}{Object of class \code{"ANY"}:
      center of symmetry. }
  }
}
\section{Methods}{
  \describe{
    \item{type}{\code{signature(object = "Symmetry")}: 
      accessor function for slot \code{type}}

    \item{SymmCenter}{\code{signature(object = "Symmetry")}: 
      accessor function for slot \code{SymmCenter}}

    \item{show}{\code{signature(object = "Symmetry")}}
  }
}
%\references{}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de}}
%\note{}
\seealso{\code{\link{DistributionSymmetry-class}}, \code{\link{OptionalNumeric-class}}}
%\examples{}
\concept{symmetry}
\keyword{classes}

\name{ncp-methods}
\docType{methods}
\alias{ncp-methods}
\alias{ncp<--methods}
\alias{ncp}
\alias{ncp<-}
\alias{ncp,ChisqParameter-method}
\alias{ncp,Chisq-method}
\alias{ncp<-,ChisqParameter-method}
\alias{ncp<-,Chisq-method}
\alias{ncp,FParameter-method}
\alias{ncp,Fd-method}
\alias{ncp<-,FParameter-method}
\alias{ncp<-,Fd-method}
\alias{ncp,BetaParameter-method}
\alias{ncp,Beta-method}
\alias{ncp<-,BetaParameter-method}
\alias{ncp<-,Beta-method}
\alias{ncp,TParameter-method}
\alias{ncp,Td-method}
\alias{ncp<-,TParameter-method}
\alias{ncp<-,Td-method}
\title{ Methods for Function ncp in Package `distr' }
\description{ncp-methods}

\section{Methods}{\describe{

\item{ncp}{\code{signature(object = "BetaParameter")}: returns the slot \code{ncp} of the parameter of the distribution }
\item{ncp<-}{\code{signature(object = "BetaParameter")}: modifies the slot \code{ncp} of the parameter of the distribution }

\item{ncp}{\code{signature(object = "Beta")}: returns the slot \code{ncp} of the parameter of the distribution }
\item{ncp<-}{\code{signature(object = "Beta")}: modifies the slot \code{ncp} of the parameter of the distribution }

\item{ncp}{\code{signature(object = "ChisqParameter")}: returns the slot \code{ncp} of the parameter of the distribution }
\item{ncp<-}{\code{signature(object = "ChisqParameter")}: modifies the slot \code{ncp} of the parameter of the distribution }

\item{ncp}{\code{signature(object = "Chisq")}: returns the slot \code{ncp} of the parameter of the distribution }
\item{ncp<-}{\code{signature(object = "Chisq")}: modifies the slot \code{ncp} of the parameter of the distribution }

\item{ncp}{\code{signature(object = "FParameter")}: returns the slot \code{ncp} of the parameter of the distribution }
\item{ncp<-}{\code{signature(object = "FParameter")}: modifies the slot \code{ncp} of the parameter of the distribution }

\item{ncp}{\code{signature(object = "Fd")}: returns the slot \code{ncp} of the parameter of the distribution }
\item{ncp<-}{\code{signature(object = "Fd")}: modifies the slot \code{ncp} of the parameter of the distribution }

\item{ncp}{\code{signature(object = "TParameter")}: returns the slot \code{ncp} of the parameter of the distribution }
\item{ncp<-}{\code{signature(object = "TParameter")}: modifies the slot \code{ncp} of the parameter of the distribution }

\item{ncp}{\code{signature(object = "Td")}: returns the slot \code{ncp} of the parameter of the distribution }
\item{ncp<-}{\code{signature(object = "Td")}: modifies the slot \code{ncp} of the parameter of the distribution }

}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{Chi square distribution}
\concept{beta distribution}
\concept{T distribution}
\concept{F distribution}
\concept{non centrality parameter}
\concept{S4 parameter class}
\concept{accessor function}
\concept{replacement function}

\name{CompoundDistribution}
\alias{CompoundDistribution}

\title{Generating function for Class "CompoundDistribution"}
\description{Generates an object of class \code{"CompoundDistribution"}.}
\usage{
CompoundDistribution(NumbOfSummandsDistr, SummandsDistr, .withSim = FALSE,
                                 withSimplify = FALSE)
                                }
\arguments{
  \item{NumbOfSummandsDistr}{Object of class \code{"DiscreteDistribution"},
     the frequency distribution; it is checked that support is contained in
     0,1,2,\dots}
  \item{SummandsDistr}{Object of class \code{"UnivDistrListOrDistribution"},
     that is, either of class \code{"UnivarDistrList"} (non i.i.d. case) or
     of class \code{"UnivariateDistribution"} (i.i.d. case); 
     the summand distribution(s).}
  \item{.withSim}{logical; value of the corresponding slot.} 
  \item{withSimplify}{\code{"logical"}: shall the return value be piped through a 
   call to \code{simplifyD}?}
}
\value{Object of class \code{"CompoundDistribution"}, or if 
argument \code{withSimplify} is \code{TRUE} the result of
\code{\link{simplifyD}} applied to the compound distribution,
i.e. an object of class \code{"UnivarLebDecDistribution"}, or if degenerate,
of class \code{"AbscontDistribution"} or \code{"DiscreteDistribution"}.}
%\references{}
\author{Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}
%\note{}
\seealso{\code{\link{CompoundDistribution-class}},
  \code{\link{simplifyD}}}
\examples{
CP0 <- CompoundDistribution(Pois(), Norm())
CP0
CP1 <- CompoundDistribution(DiscreteDistribution(supp = c(1,5,9,11),
                            prob = dbinom(0:3, size = 3,prob = 0.3)),Norm())
CP1
UL <- UnivarDistrList(Norm(), Binom(10,0.3), Chisq(df=4), Norm(),
                      Binom(10,0.3), Chisq(df=4), Norm(), Binom(10,0.3),
                      Chisq(df=4), Td(5), Td(10))
CP2 <- CompoundDistribution(DiscreteDistribution(supp = c(1,5,9,11),
                      prob = dbinom(0:3, size = 3, prob = 0.3)),UL)
plot(CP2)
}
\keyword{distribution}
\keyword{list}
\concept{compound distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{ExpParameter-class}
\docType{class}
\alias{ExpParameter-class}
\alias{initialize,ExpParameter-method}

\title{Class "ExpParameter"}
\description{ The parameter of an exponential distribution, used by Exp-class and DExp-class}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("ExpParameter", rate)}.
Usually an object of this class is not needed on its own, it is generated automatically when an object of the class Exp
is instantiated. 
}
\section{Slots}{
  \describe{
    \item{\code{rate}}{Object of class \code{"numeric"}: the rate of an exponential distribution }
    \item{\code{name}}{Object of class \code{"character"}: a name / comment for the parameters }
  }
}
\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "ExpParameter")}: initialize method }
    \item{rate}{\code{signature(object = "ExpParameter")}: returns the slot \code{rate} of the parameter of the distribution }
    \item{rate<-}{\code{signature(object = "ExpParameter")}: modifies the slot \code{rate} of the parameter of the distribution }
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }


\seealso{
\code{\link{Exp-class}}
\code{\link{DExp-class}}
\code{\link{Parameter-class}}
}

\examples{
W <- new("ExpParameter", rate = 1)
rate(W) # rate of this distribution is 1.
rate(W) <- 2 # rate of this distribution is now 2.
}
\keyword{distribution}
\concept{parameter}
\concept{Exp distribution}
\concept{S4 parameter class}

\name{lambda-methods}
\docType{methods}
\alias{lambda-methods}
\alias{lambda<--methods}
\alias{lambda}
\alias{lambda<-}
\alias{lambda,PoisParameter-method}
\alias{lambda,Pois-method}
\alias{lambda<-,PoisParameter-method}
\alias{lambda<-,Pois-method}
\title{ Methods for Function lambda  in Package `distr'}
\description{lambda-methods}

\section{Methods}{\describe{

\item{lambda}{\code{signature(object = "PoisParameter")}: returns the slot \code{lambda} of the parameter of the distribution }
\item{lambda<-}{\code{signature(object = "PoisParameter")}: modifies the slot \code{lambda} of the parameter of the distribution }

\item{lambda}{\code{signature(object  = "Pois")}: returns the slot \code{lambda} of the parameter of the distribution }
\item{lambda<-}{\code{signature(object = "Pois")}: modifies the slot \code{lambda} of the parameter of the distribution }

}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{Poisson distribution}
\concept{S4 parameter class}
\concept{accessor function}
\concept{replacement function}

\name{UnivarLebDecDistribution-class}
\docType{class}
\alias{UnivarLebDecDistribution-class}
\alias{AffLinUnivarLebDecDistribution-class}
\alias{AcDcLcDistribution-class}
\alias{acPart}
\alias{discretePart}
\alias{acPart<-}
\alias{discretePart<-}
\alias{acPart-methods}
\alias{discretePart-methods}
\alias{acPart<--methods}
\alias{discretePart<--methods}
\alias{acPart,UnivarLebDecDistribution-method}
\alias{discretePart,UnivarLebDecDistribution-method}
\alias{acPart<-,UnivarLebDecDistribution-method}
\alias{discretePart<-,UnivarLebDecDistribution-method}
\alias{acWeight}
\alias{discreteWeight}
\alias{acWeight<-}
\alias{discreteWeight<-}
\alias{acWeight-methods}
\alias{discreteWeight-methods}
\alias{acWeight<--methods}
\alias{discreteWeight<--methods}
\alias{acWeight,UnivarLebDecDistribution-method}
\alias{discreteWeight,UnivarLebDecDistribution-method}
\alias{acWeight<-,UnivarLebDecDistribution-method}
\alias{discreteWeight<-,UnivarLebDecDistribution-method}
\alias{p.discrete,UnivarLebDecDistribution-method}
\alias{d.discrete,UnivarLebDecDistribution-method}
\alias{q.discrete,UnivarLebDecDistribution-method}
\alias{r.discrete,UnivarLebDecDistribution-method}
\alias{p.ac,UnivarLebDecDistribution-method}
\alias{d.ac,UnivarLebDecDistribution-method}
\alias{q.ac,UnivarLebDecDistribution-method}
\alias{r.ac,UnivarLebDecDistribution-method}
\alias{p.discrete-methods}
\alias{d.discrete-methods}
\alias{q.discrete-methods}
\alias{r.discrete-methods}
\alias{p.ac-methods}
\alias{d.ac-methods}
\alias{q.ac-methods}
\alias{r.ac-methods}
\alias{p.discrete}
\alias{d.discrete}
\alias{q.discrete}
\alias{r.discrete}
\alias{p.ac}
\alias{d.ac}
\alias{q.ac}
\alias{r.ac}
\alias{coerce,AbscontDistribution,UnivarLebDecDistribution-method}
\alias{coerce,DiscreteDistribution,UnivarLebDecDistribution-method}
\alias{coerce,AffLinUnivarLebDecDistribution,UnivarLebDecDistribution-method}
\alias{abs,UnivarLebDecDistribution-method}
\alias{log,UnivarLebDecDistribution-method}
\alias{log10,UnivarLebDecDistribution-method}
\alias{Math,UnivarLebDecDistribution-method}
\alias{^,AcDcLcDistribution,Dirac-method}
\alias{exp,UnivarLebDecDistribution-method}
\alias{sign,UnivarLebDecDistribution-method}
\alias{sign,AcDcLcDistribution-method}
\alias{sqrt,UnivarLebDecDistribution-method}
\alias{sqrt,AcDcLcDistribution-method}

\title{Class "UnivarLebDecDistribution"}
\description{\code{UnivarLebDecDistribution}-class is a class to formalize
                   a Lebesgue decomposed distribution with a discrete and an
                   absolutely continuous part; it is a subclass to
                   class \code{UnivarMixingDistribution}.}
\section{Objects from the Class}{
Objects can be created by calls of the form
\code{new("UnivarLebDecDistribution", ...)}.
  More frequently they are created via the generating function
  \code{\link{UnivarLebDecDistribution}}.
}
\section{Slots}{
  \describe{
    \item{\code{mixCoeff}}{Object of class \code{"numeric"}: a vector of length
            2 of probabilities for the respective a.c. and discrete part of
            the object}
    \item{\code{mixDistr}}{Object of class \code{"UnivarDistrList"}: a list of
    univariate distributions containing the a.c. and discrete components; must be of
    length 2; the first component must be of class \code{"AbscontDistribution"},
    the second of class \code{"DiscreteDistribution"}.}
    \item{\code{img}}{Object of class \code{"Reals"}: the space of the image of this distribution which has dimension 1
    and the name "Real Space" }
    \item{\code{param}}{Object of class \code{"Parameter"}: the parameter of this distribution, having only the
    slot name "Parameter of a discrete distribution" }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers}
    \item{\code{d}}{fixed to \code{NULL}}
    \item{\code{p}}{Object of class \code{"function"}: cumulative distribution function}
    \item{\code{q}}{Object of class \code{"function"}: quantile function}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
    \item{\code{support}}{numeric vector --- the support slot of the discrete part}
    \item{\code{gaps}}{(numeric) matrix or \code{NULL}; --- the gaps slot of 
                         the absolutely continuous part}
  }
}
\section{Extends}{
Class \code{"UnivarMixingDistribution"}, directly;
class \code{"UnivariateDistribution"} by class \code{"UnivarMixingDistribution"}
class \code{"Distribution"} by class \code{"UnivariateDistribution"}.
}
\section{Methods}{
  \describe{
    \item{show}{\code{signature(object = "UnivarLebDecDistribution")}}
    \item{plot}{\code{signature(object = "UnivarLebDecDistribution")}}
    \item{acPart}{\code{signature(object = "UnivarLebDecDistribution")}}
    \item{acPart<-}{\code{signature(object = "UnivarLebDecDistribution")}}
    \item{discretePart}{\code{signature(object = "UnivarLebDecDistribution")}}
    \item{discretePart<-}{\code{signature(object = "UnivarLebDecDistribution")}}
    \item{acWeight}{\code{signature(object = "UnivarLebDecDistribution")}}
    \item{acWeight<-}{\code{signature(object = "UnivarLebDecDistribution")}}
    \item{discreteWeight}{\code{signature(object = "UnivarLebDecDistribution")}}
    \item{discreteWeight<-}{\code{signature(object = "UnivarLebDecDistribution")}}
    \item{p.ac}{\code{signature(object = "UnivarLebDecDistribution")} accessor to 
                slot \code{p} of \code{acPart(object)}, possibly weighted 
                by \code{acWeight(object)}; 
                it has an extra argument \code{CondOrAbs} with default value 
                \code{"cond"} which if it does not partially match 
               (by \code{\link{pmatch}}) \code{"abs"}, returns exactly
               slot \code{p} of \code{acPart(object)} else weighted by 
               \code{acWeight(object)}.}
    \item{d.ac}{\code{signature(object = "UnivarLebDecDistribution")}accessor to 
                slot \code{d} of the absolutely continuous part of
                the distribution, possibly weighted by \code{acWeight(object)}; 
                it has an extra argument \code{CondOrAbs} which acts as the one
                in \code{p.ac}.}
    \item{q.ac}{\code{signature(object = "UnivarLebDecDistribution")} accessor to 
                slot \code{q} of \code{acPart(object)}.}
    \item{r.ac}{\code{signature(object = "UnivarLebDecDistribution")} accessor to 
                slot \code{q} of \code{acPart(object)}.}
    \item{p.discrete}{\code{signature(object = "UnivarLebDecDistribution")} 
                      accessor to slot \code{p} of \code{discretePart(object)}, 
                      possibly weighted by \code{discreteWeight(object)}; 
                      it has an extra argument \code{CondOrAbs} which acts 
                      as the one in \code{p.ac}.}
    \item{d.discrete}{\code{signature(object = "UnivarLebDecDistribution")} 
                      accessor to slot \code{d} of \code{discretePart(object)}, 
                      possibly weighted by \code{discreteWeight(object)}; 
                      it has an extra argument \code{CondOrAbs} which acts as 
                      the one in \code{p.ac}.}
    \item{q.discrete}{\code{signature(object = "UnivarLebDecDistribution")} 
                      accessor to slot \code{q} of \code{discretePart(object)}.}
    \item{r.discrete}{\code{signature(object = "UnivarLebDecDistribution")} 
                      accessor to slot \code{r} of \code{discretePart(object)}.}
    \item{coerce}{\code{signature(from = "AffLinUnivarLebDecDistribution", to = "UnivarLebDecDistribution")}:
      create a \code{"UnivarLebDecDistribution"} object from a \code{"AffLinUnivarLebDecDistribution"} object}
    \item{coerce}{\code{signature(from = "AbscontDistribution", to = "UnivarLebDecDistribution")}:
      create a \code{"UnivarLebDecDistribution"} object from a \code{"AbscontDistribution"} object}
    \item{coerce}{\code{signature(from = "DiscreteDistribution", to = "UnivarLebDecDistribution")}:
      create a \code{"UnivarLebDecDistribution"} object from a \code{"DiscreteDistribution"} object}
    \item{Math}{\code{signature(x = "UnivarLebDecDistribution")}: application of a mathematical function, e.g. \code{sin} or \code{tan} to this discrete distribution
    \itemize{
      \item \code{abs}: \code{signature(x = "UnivarLebDecDistribution")}:  exact image distribution of \code{abs(x)}.
      \item \code{exp}: \code{signature(x = "UnivarLebDecDistribution")}:  exact image distribution of \code{exp(x)}.
      \item \code{sign}: \code{signature(x = "UnivarLebDecDistribution")}:  exact image distribution of \code{sign(x)}.
      \item \code{sign}: \code{signature(x = "AcDcLcDistribution")}:  exact image distribution of \code{sign(x)}.
      \item \code{sqrt}: \code{signature(x = "AcDcLcDistribution")}:  exact image distribution of \code{sqrt(x)}.
      \item \code{log}: \code{signature(x = "UnivarLebDecDistribution")}:  (with optional further argument \code{base}, defaulting to \code{exp(1)}) exact image distribution of \code{log(x)}.
      \item \code{log10}: \code{signature(x = "UnivarLebDecDistribution")}:  exact image distribution of \code{log10(x)}.
      \item \code{sqrt}: \code{signature(x = "UnivarLebDecDistribution")}:  exact 
          image distribution of \code{sqrt(x)}.
      \item \code{sqrt}: \code{signature(x = "AcDcLcDistribution")}:  exact image distribution of \code{sqrt(x)}.
    }}
    \item{-}{\code{signature(e1 = "UnivarLebDecDistribution")}: application of `-' to this distribution}
    \item{*}{\code{signature(e1 = "UnivarLebDecDistribution", e2 = "numeric")}: multiplication of this distribution
    by an object of class `numeric'}
    \item{/}{\code{signature(e1 = "UnivarLebDecDistribution", e2 = "numeric")}: division of this distribution
    by an object of class `numeric'}
    \item{+}{\code{signature(e1 = "UnivarLebDecDistribution", e2 = "numeric")}: addition of this distribution
    to an object of class `numeric'}
    \item{-}{\code{signature(e1 = "UnivarLebDecDistribution", e2 = "numeric")}: subtraction of an object of class `numeric'
    from this distribution }
    \item{*}{\code{signature(e1 = "numeric", e2 = "UnivarLebDecDistribution")}: multiplication of this distribution
    by an object of class `numeric'}
    \item{+}{\code{signature(e1 = "numeric", e2 = "UnivarLebDecDistribution")}: addition of this distribution
    to an object of class `numeric'}
    \item{-}{\code{signature(e1 = "numeric", e2 = "UnivarLebDecDistribution")}: subtraction of this distribution
    from an object of class `numeric'}
    \item{+}{\code{signature(e1 = "UnivarLebDecDistribution", e2 = "UnivarLebDecDistribution")}: Convolution of two Lebesgue
    decomposed distributions. Result is again of class \code{"UnivarLebDecDistribution"}, but if option
    \code{getdistrOption("withSimplify")} is \code{TRUE} it is piped through a call to \code{\link{simplifyD}},
    hence may also be of class \code{AbscontDistribution} or \code{DiscreteDistribution}}.
    \item{-}{\code{signature(e1 = "UnivarLebDecDistribution", e2 = "UnivarLebDecDistribution")}: Convolution of two Lebesgue
    decomposed distributions. The same applies as for the preceding item.}
  }
}
\section{Internal subclass "AffLinUnivarLebDecDistribution"}{
To enhance accuracy of several functionals on distributions,
  mainly from package \pkg{distrEx},  
  there is an internally used (but exported) subclass 
  \code{"AffLinUnivarLebDecDistribution"} which has extra slots 
  \code{a}, \code{b} (both of class \code{"numeric"}),  and \code{X0} 
  (of class \code{"UnivarLebDecDistribution"}), to capture the fact 
  that the object has the same distribution as \code{a * X0 + b}. This is 
  the class of the return value of methods 
  \describe{
    \item{-}{\code{signature(e1 = "UnivarLebDecDistribution")}}
    \item{*}{\code{signature(e1 = "UnivarLebDecDistribution", e2 = "numeric")}}
    \item{/}{\code{signature(e1 = "UnivarLebDecDistribution", e2 = "numeric")}}
    \item{+}{\code{signature(e1 = "UnivarLebDecDistribution", e2 = "numeric")}}
    \item{-}{\code{signature(e1 = "UnivarLebDecDistribution", e2 = "numeric")}}
    \item{*}{\code{signature(e1 = "numeric", e2 = "UnivarLebDecDistribution")}}
    \item{+}{\code{signature(e1 = "numeric", e2 = "UnivarLebDecDistribution")}}
    \item{-}{\code{signature(e1 = "numeric", e2 = "UnivarLebDecDistribution")}}
    \item{-}{\code{signature(e1 = "AffLinUnivarLebDecDistribution")}}
    \item{*}{\code{signature(e1 = "AffLinUnivarLebDecDistribution", e2 = "numeric")}}
    \item{/}{\code{signature(e1 = "AffLinUnivarLebDecDistribution", e2 = "numeric")}}
    \item{+}{\code{signature(e1 = "AffLinUnivarLebDecDistribution", e2 = "numeric")}}
    \item{-}{\code{signature(e1 = "AffLinUnivarLebDecDistribution", e2 = "numeric")}}
    \item{*}{\code{signature(e1 = "numeric", e2 = "AffLinUnivarLebDecDistribution")}}
    \item{+}{\code{signature(e1 = "numeric", e2 = "AffLinUnivarLebDecDistribution")}}
    \item{-}{\code{signature(e1 = "numeric", e2 = "AffLinUnivarLebDecDistribution")}}
  }
  There also is a class union of \code{"AffLinAbscontDistribution"},
  \code{"AffLinDiscreteDistribution"}, \code{"AffLinUnivarLebDecDistribution"}
  and called \code{"AffLinDistribution"}
  which is used for functionals.
}

\section{Internal virtual superclass "AcDcLcDistribution"}{
As many operations should be valid no matter whether the operands
are of class \code{"AbscontDistribution"},
  \code{"DiscreteDistribution"}, or \code{"UnivarLebDecDistribution"},
there is a class union of these classes called \code{"AcDcLcDistribution"};
in particular methods for \code{"*"}, \code{"/"}, 
\code{"^"} (see \link{operators-methods}) and methods
\code{\link{Minimum}}, \code{Maximum}, \code{\link{Truncate}}, and
 \code{\link{Huberize}}, and \code{\link{convpow}} are defined for this 
 class union.   
}
\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}
  }

\seealso{
\code{\link{Parameter-class}}
\code{\link{UnivarMixingDistribution-class}}
\code{\link{DiscreteDistribution-class}}
\code{\link{AbscontDistribution-class}}
\code{\link{simplifyD}}
\code{\link{flat.LCD}}
}
\examples{
wg <- flat.mix(UnivarMixingDistribution(Unif(0,1),Unif(4,5),
               withSimplify=FALSE))
myLC <- UnivarLebDecDistribution(discretePart=Binom(3,.3), acPart = wg,
          discreteWeight=.2)
myLC
p(myLC)(0.3)
r(myLC)(30)
q(myLC)(0.9)
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
acPart(myLC)
plot(myLC)
d.discrete(myLC)(2)
p.ac(myLC)(0)
acWeight(myLC)
plot(acPart(myLC))
plot(discretePart(myLC))
gaps(myLC)
support(myLC)
plot(as(Norm(),"UnivarLebDecDistribution"))
}
\keyword{distribution}
\concept{Lebesgue decomposed distribution}
\concept{absolutely continuous distribution}
\concept{discrete distribution}
\concept{S4 distribution class}

\name{Norm-class} 
\docType{class}
\alias{Norm-class}
\alias{Norm}
\alias{initialize,Norm-method}

\title{Class "Norm"}
\description{  The normal distribution has density
  \deqn{
    f(x) =
    \frac{1}{\sqrt{2\pi}\sigma} e^{-(x-\mu)^2/2\sigma^2}}{
    f(x) = 1/(sqrt(2 pi) sigma) e^-((x - mu)^2/(2 sigma^2))
  }
  where \eqn{\mu}{mu} is the mean of the distribution and
  \eqn{\sigma}{sigma} the standard deviation.
 C.f. \code{\link[stats:Normal]{rnorm}}
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Norm(mean, sd)}.
This object is a normal distribution.
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}: The domain of this distribution has got dimension 1
    and the name "Real Space". }
    \item{\code{param}}{Object of class \code{"UniNormParameter"}: the parameter of this distribution (mean and sd),
     declared at its instantiation }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers (calls function \code{rnorm})}
    \item{\code{d}}{Object of class \code{"function"}: density function (calls function \code{dnorm})}
    \item{\code{p}}{Object of class \code{"function"}: cumulative function (calls function \code{pnorm})}
    \item{\code{q}}{Object of class \code{"function"}: inverse of the cumulative function (calls function \code{qnorm})}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"AbscontDistribution"}, directly.\cr
Class \code{"UnivariateDistribution"}, by class \code{"AbscontDistribution"}.\cr
Class \code{"Distribution"}, by class \code{"AbscontDistribution"}.
}
\section{Methods}{
  \describe{
    \item{-}{\code{signature(e1 = "Norm", e2 = "Norm")}}
    \item{+}{\code{signature(e1 = "Norm", e2 = "Norm")}: 
    For the normal distribution the exact convolution formulas are implemented thereby improving the general numerical
    approximation.}
    \item{*}{\code{signature(e1 = "Norm", e2 = "numeric")}}
    \item{+}{\code{signature(e1 = "Norm", e2 = "numeric")}: 
    For the normal distribution we use its closedness under affine linear transformations.}
    \item{initialize}{\code{signature(.Object = "Norm")}: initialize method }
    \item{mean}{\code{signature(object = "Norm")}: returns the slot \code{mean} of the parameter of the distribution }
    \item{mean<-}{\code{signature(object = "Norm")}: modifies the slot \code{mean} of the parameter of the distribution }
    \item{sd}{\code{signature(object = "Norm")}: returns the slot \code{sd} of the parameter of the distribution  }
    \item{sd<-}{\code{signature(object = "Norm")}: modifies the slot \code{sd} of the parameter of the distribution }
  }
  further arithmetic methods see \link{operators-methods}
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}

\seealso{
\code{\link{UniNormParameter-class}}
\code{\link{AbscontDistribution-class}}
\code{\link{Reals-class}}
\code{\link[stats:Normal]{rnorm}}

}
\examples{
N <- Norm(mean=1,sd=1) # N is a normal distribution with mean=1 and sd=1.
r(N)(1) # one random number generated from this distribution, e.g. 2.257783
d(N)(1) # Density of this distribution is  0.3989423 for x=1.
p(N)(1) # Probability that x<1 is 0.5.
q(N)(.1) # Probability that x<-0.2815516 is 0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
mean(N) # mean of this distribution is 1.
sd(N) <- 2 # sd of this distribution is now 2.
M <- Norm() # M is a normal distribution with mean=0 and sd=1.
O <- M+N # O is a normal distribution with mean=1 (=1+0) and sd=sqrt(5) (=sqrt(2^2+1^2)).
}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{Gaussian distribution}
\concept{Normal distribution}
\concept{S4 distribtution class}
\concept{location scale family}

\name{param-methods}
\docType{methods}
\alias{param-methods}
\alias{param}
\alias{param,Distribution-method}
\title{ Methods for Function param in Package `distr' }
\description{param-methods}
\section{Methods}{\describe{
\item{param}{\code{signature(object = "Distribution")}: returns the parameter }
}}
\keyword{methods}
\keyword{distribution}
\concept{parameter}
\concept{S4 parameter class}
\concept{accessor function}

\name{Version Management}
\docType{methods}
\alias{isOldVersion}
\alias{isOldVersion-methods}
\alias{isOldVersion,ANY-method}
\alias{conv2NewVersion}
\alias{conv2NewVersion-methods}
\alias{conv2NewVersion,ANY-method}
\alias{conv2NewVersion,LatticeDistribution-method}
\title{ Methods for Version Management in Package `distr' }
\description{Version-Management-methods}
\usage{isOldVersion(object)
conv2NewVersion(object)
\S4method{isOldVersion}{ANY}(object)
\S4method{conv2NewVersion}{ANY}(object)
\S4method{conv2NewVersion}{LatticeDistribution}(object)
}
\arguments{
  \item{object}{object of class  \code{"ANY"} (or subclasses)}
}
\details{
 From version 1.9 of this package on, class \code{"AbscontDistribution"} has an extra slot
 \code{gaps}. As the addition of new slots will probably happen again in the future development
 of our packages, we provide the following two help functions
 \code{isOldVersion} and \code{conv2NewVersion} to check whether the object was generated by an 
 older version of this package and to convert such an object to the new format, respectively. 
Also, the intermediate class \code{"LatticeDistribution"} is introduced at version 1.9
so that all subclasses of \code{"DiscreteDistribution"} like \code{"Binom"}, \code{"Nbinom"} etc,
now have an extra slot \code{lattice}. \code{conv2NewVersion} takes this up and provides
a particular method for signature \code{"LatticeDistribution"} which fills slot
\code{lattice} accordingly.

 \describe{
\item{isOldVersion}{\code{signature(object = "ANY")}: 
throws an error if \code{isClass(class(object))} is \code{FALSE}, i.e.; if
the class of \code{object} is no formal (S4) class. Else it checks whether
all slots of the actual class definition may be accessed and if so
returns \code{FALSE} and else \code{TRUE} and issues a warning.
 }
\item{conv2NewVersion}{\code{signature(object = "ANY")}: 
Generates a valid copy of \code{object} (according to the actual class definition), 
using the slots of \code{object} where possible and for the slots which are not yet
present in \code{object} (because it was generated by an older version of 
the class definition), it generates a prototype object of the class 
of \code{object} with \code{new(class(object))} and uses
the slot values of this prototype to fill the missing slots.
 }
\item{conv2NewVersion}{\code{signature(object = "LatticeDistribution")}: 
Generates a valid copy of \code{object} (according to the actual class definition,
i.e.; with a corresponding \code{lattice}-slot), by generating a new instance of
this object by \code{new(class(object), <list-of-parameters>}.
 }
}
}
\keyword{utilities}
\concept{version management}

\name{Fd-class} 
\docType{class}
\alias{Fd-class}
\alias{Fd}
\alias{initialize,Fd-method}

\title{Class "Fd"}
\description{   The F distribution with \code{df1 =} \eqn{n_1}{n1}, by default \code{= 1}, 
  and \code{df2 =} \eqn{n_2}{n2}, by default \code{= 1}, degrees of freedom has density
  \deqn{
    d(x) = \frac{\Gamma(n_1/2 + n_2/2)}{\Gamma(n_1/2)\Gamma(n_2/2)}
    \left(\frac{n_1}{n_2}\right)^{n_1/2} x^{n_1/2 -1}
    \left(1 + \frac{n_1 x}{n_2}\right)^{-(n_1 + n_2) / 2}%
  }{d(x) = Gamma((n1 + n2)/2) / (Gamma(n1/2) Gamma(n2/2))
    (n1/n2)^(n1/2) x^(n1/2 - 1)
    (1 + (n1/n2) x)^-(n1 + n2)/2}
  for \eqn{x > 0}.

  C.f. \code{\link[stats:Fdist]{rf}}
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{Fd(df1, df2)}.
This object is a F distribution.
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}: The space of the image of this distribution has got dimension 1 and the name "Real Space". }
    \item{\code{param}}{Object of class \code{"FParameter"}: the parameter of this distribution (df1 and df2), declared at its instantiation }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers (calls function rf)}
    \item{\code{d}}{Object of class \code{"function"}: density function (calls function df)}
    \item{\code{p}}{Object of class \code{"function"}: cumulative function (calls function pf)}
    \item{\code{q}}{Object of class \code{"function"}: inverse of the cumulative function (calls function qf)}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
Class \code{"AbscontDistribution"}, directly. \cr
Class \code{"UnivariateDistribution"}, by class \code{"AbscontDistribution"}. \cr
Class \code{"Distribution"}, by class \code{"AbscontDistribution"}.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Fd")}: initialize method }
    \item{df1}{\code{signature(object = "Fd")}: returns the slot \code{df1} of the parameter of the distribution }
    \item{df1<-}{\code{signature(object = "Fd")}: modifies the slot \code{df1} of the parameter of the distribution }
    \item{df2}{\code{signature(object = "Fd")}: returns the slot \code{df2} of the parameter of the distribution }
    \item{df2<-}{\code{signature(object = "Fd")}: modifies the slot \code{df2} of the parameter of the distribution }
  }
}

\author{Thomas Stabla \email{statho3@web.de},\cr 
        Florian Camphausen \email{fcampi@gmx.de},\cr
        Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
        Matthias Kohl \email{Matthias.Kohl@stamats.de}}

\note{It is the distribution of the ratio of the mean squares of n1 and n2 independent standard normals, and hence of the
ratio of two independent chi-squared variates each divided by its degrees of freedom. Since the ratio of a normal and the
root mean-square of m independent normals has a Student's \eqn{t_m} distribution, the square of a \eqn{t_m} variate has a F
distribution on 1 and m degrees of freedom. 

The non-central F distribution is again the ratio of mean squares of independent normals of unit variance, but those in the
numerator are allowed to have non-zero means and ncp is the sum of squares of the means. 
}

\section{Ad hoc methods}{\itemize{
\item An ad hoc method is provided for slot \code{d} if \code{ncp!=0}.
\item For R Version \code{<2.3.0} ad hoc methods are provided for slots \code{q}, \code{r} if \code{ncp!=0};
      for R Version \code{>=2.3.0} the methods from package \pkg{stats} are used.
      }}

\seealso{
\code{\link{FParameter-class}}
\code{\link{AbscontDistribution-class}}
\code{\link{Reals-class}}
\code{\link[stats:Fdist]{rf}}
}
\examples{
F <- Fd(df1 = 1, df2 = 1) # F is a F distribution with df=1 and df2=1.
r(F)(1) # one random number generated from this distribution, e.g. 29.37863
d(F)(1) # Density of this distribution is 0.1591549 for x=1 .
p(F)(1) # Probability that x<1 is 0.5.
q(F)(.1) # Probability that x<0.02508563 is 0.1.
## in RStudio or Jupyter IRKernel, use q.l(.)(.) instead of q(.)(.)
df1(F) # df1 of this distribution is 1.
df1(F) <- 2 # df1 of this distribution is now 2.
Fn <- Fd(df1 = 1, df2 = 1, ncp = 0.5) 
  # Fn is a F distribution with df=1, df2=1 and ncp =0.5.
d(Fn)(1) ## from R 2.3.0 on ncp no longer ignored...
}
\keyword{distribution}
\concept{F distribution}
\concept{absolutely continuous distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{distrARITH}
\alias{distrARITH}
\alias{ARITHMETICS}
\title{Arithmetics on Distributions}
\description{Provides information on the interpretation of
arithmetics operating on Distributions in package \pkg{distr}}
\usage{
distrARITH(library = NULL)
}
\arguments{
  \item{library}{a character vector with path names of R libraries, or \code{NULL}.
                 The default value of \code{NULL} corresponds to all libraries
                 currently known.  If the default is used, the loaded packages
                 are searched before the libraries}
    }
\value{no value is returned
}

\author{Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}
\examples{
distrARITH()
}
\keyword{math}
\keyword{distribution}
\keyword{arith}
\keyword{documentation}
\concept{arithmetics for distributions}
\concept{info file}

\name{UnivarLebDecDistribution}
\alias{UnivarLebDecDistribution}

\title{Generating function for Class "UnivarLebDecDistribution"}
\description{Generates an object of class \code{"UnivarLebDecDistribution"}.}
\usage{
UnivarLebDecDistribution(acPart, discretePart, acWeight, discreteWeight,
                                     r = NULL, e = NULL, n = NULL, y = NULL)
}
\arguments{
  \item{acPart}{Object of class \code{"AbscontDistribution"} (or subclasses); 
                a.c. part of the distribution}
  \item{discretePart}{Object of class \code{"AbscontDistribution"} (or subclasses);
  discrete part of the distribution}
  \item{acWeight}{Object of class \code{"numeric"}; weight of the a.c. part of 
                  the distribution}
  \item{discreteWeight}{Object of class \code{"numeric"}; weight of the discrete 
        part of the distribution}
  \item{r}{optional argument; if given, this is a random number generator as function
           \code{r <- function(n){....}} to produce r.v.'s distributed 
            according to the distribution; used in a call to \code{\link{RtoDPQ.LC}}
            if  \code{acPart} and \code{discretePart} are missing.}
  \item{e}{optional argument; if argument \code{r} is given, this is the number
           of r.v.'s drawn to fill the empty slots of this object; if missing filled
           with \code{getdistrOption("RtoDPQ.e")}.}
  \item{n}{optional argument; if argument \code{r} is given, this is the number
           gridpoints used in filling the empty p,d,q slots of this object; if missing filled
           with \code{getdistrOption("DefaultNrGridPoints")}.}
  \item{y}{a (numeric) vector or \code{NULL}}
}

\details{At least one of arguments \code{discretePart}, \code{acPart}, or \code{r}
         must be given; if the first two are missing, slots are filled by a call
         to \code{RtoDPQ.LC}. For this purpose argument \code{r} is used together
         with arguments \code{e} and \code{n}. If the latter are missing they are
         filled with \code{getdistrOption("RtoDPQ.e")} and 
         \code{getdistrOption("DefaultNrGridPoints")}, respectively.
         For the a.c. part, similarly to \code{\link{RtoDPQ}} we have an optional 
         parameter \code{y} for using N. Horbenko's quantile trick: i.e.; on an 
         equally spaced grid \code{x.grid} on [0,1], apply 
         \code{f(q(x)(x.grid))}, write the result to \code{y} and use these
         values instead of simulated ones.
         
         If argument \code{discretePart} is missing but  \code{acPart} is not,
         \code{discreteWeight} is set to 0 and \code{discretePart} is set to \code{Dirac(0)}.
         If argument   \code{acPart}  is missing but \code{discretePart} is not,
         \code{acWeight} is set to 0 and \code{discretePart} is set to \code{Norm()}.
         If both arguments   \code{acPart} and \code{discretePart} are given,
         at least one of arguments \code{discreteWeight} and \code{acWeight} must
         be given and lie in [0,1], else an error is thrown.
         If only one argument \code{acWeight} or \code{discreteWeight} is given
         the other one is gotten as 1-[ac/discrete]Weight.
         Else if both are given, they must sum up to 1.
         If a weight is smaller than \code{getdistrOption("TruncQuantile")}, it 
         is set to 0.
         }
\value{Object of class \code{"UnivarLebDecDistribution"}.}
%\references{}
\author{Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}
%\note{}
\seealso{\code{\link{UnivarLebDecDistribution-class}},
  \code{\link{simplifyD}}}
\examples{
mylist <- UnivarLebDecDistribution(discretePart=Binom(3,.3), acPart=Norm(2,2),
          acWeight=11/20)
mylist          
}
\keyword{distribution}
\keyword{list}
\concept{multivariate distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{distr-defunct}
\alias{distr-defunct}
\alias{GeomParameter-class}
\alias{initialize,GeomParameter-method}

\title{Class "GeomParameter"}
\description{ The parameter of a geometric distribution, used by Geom-class}
\section{Objects from the Class}{
Objects were created by calls of the form \code{new("GeomParameter", prob)}.
Usually an object of this class was not needed on its own, it was generated
automatically when an object of the class Geom
is instantiated.
}
\section{Slots}{
  \describe{
    \item{\code{prob}}{Object of class \code{"numeric"}: the probability of a
                        geometric distribution }
    \item{\code{name}}{Object of class \code{"character"}:
                        a name / comment for the parameters }
  }
}

\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "GeomParameter")}:
                       initialize method }
    \item{prob}{\code{signature(object = "GeomParameter")}:
                 returns the slot \code{prob} of the parameter of the
                 distribution }
    \item{prob<-}{\code{signature(object = "GeomParameter")}:
                  modifies the slot \code{prob} of the parameter of the
                  distribution }
  }
}

\section{Defunct}{
The use of class \code{GeomParameter} is defunct as of version 2.8.0;
it is to be replaced by a corresponding use
of class \code{NbinomParameter} with slot \code{size = 1} which may be
generated, e.g. by
\code{new("NbinomParameter", prob, size = 1,
           name = "Parameter of a Geometric distribution")}
}

\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}
  }

\seealso{
  \code{\link{Defunct}}
}
\keyword{misc}

\name{flat.LCD}
\alias{flat.LCD}
\title{Flattening a list of Lebesgue decomposed distributions}
\description{
  flattens a list of Lebesgue decomposed distributions endowed with weights to 
  give one Lebesgue decomposed distribution}
\usage{
flat.LCD(..., mixCoeff = NULL, withgaps = getdistrOption("withgaps"))
}
\arguments{
  \item{\dots}{list of Lebesgue decomposed distributions}
  \item{mixCoeff}{Object of class \code{"numeric"} of the same length
                  as \dots: a vector of probabilities for the mixing components.}
  \item{withgaps}{logical; shall \code{gaps} be detected empirically?}
}
\details{
\code{flat.LCD} flattens a list of Lebesgue decomposed distributions given through \code{\dots},
i.e., it takes all list elements and mixing coefficients and builds up the mixed
distribution (forgetting about the components); the result will be one distribution of 
class \code{UnivarLebDecDistribution}. If \code{mixCoeff} is missing, all list elements 
are equally weighted. It is used internally in our methods for \code{"*"}, \code{"/"},
\code{"^"} (see \link{operators-methods}), 
\code{\link{Minimum}}, and \code{\link{convpow}}, as well in method \code{\link{flat.mix}}.}

\value{ \code{flat.LCD} returns an object of class \code{UnivarLebDecDistribution}.}

\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}}

\seealso{ 
\code{\link{UnivarLebDecDistribution-class}}, 
\code{\link{operators-methods}}} 
\examples{
D1 <- as(Norm(),"UnivarLebDecDistribution")
D2 <- as(Pois(1),"UnivarLebDecDistribution")
D3 <- as(Binom(1,.4),"UnivarLebDecDistribution")
flat.LCD(D1,D2,D3, mixCoeff = c(0.4,0.5,0.1))
}
\keyword{math}
\keyword{distribution}
\keyword{arith}
\concept{Lebesgue decomposed distribution}
\concept{utility}

\name{PoisParameter-class}
\docType{class}
\alias{PoisParameter-class}
\alias{initialize,PoisParameter-method}
\title{Class "PoisParameter"}
\description{ The parameter of a Poisson distribution, used by Pois-class}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("PoisParameter", lambda)}.
  Usually an object of this class is not needed on its own, it is
  generated automatically when an object of the class Pois is prepared. 
}
\section{Slots}{
  \describe{
    \item{\code{lambda}}{Object of class \code{"numeric"}:
      the lambda of a Poisson distribution}
    \item{\code{name}}{Object of class \code{"character"}:
      a name / comment for the parameters}
  }
}
\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "PoisParameter")}:
      initialize method }
    \item{lambda}{\code{signature(object = "PoisParameter")}:
      returns the slot lambda of the parameter of the distribution}
    \item{lambda<-}{\code{signature(object = "PoisParameter")}:
      modifies the slot lambda of the parameter of the distribution}
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }

\seealso{
\code{\link{Pois-class}}
\code{\link{Parameter-class}}
}
\examples{
W <- new("PoisParameter",lambda = 1)
lambda(W) # lambda of this distribution is 1.
lambda(W) <- 2 # lambda of this distribution is now 2.
}
\keyword{distribution}
\concept{parameter}
\concept{Poisson distribution}
\concept{S4 parameter class}

\name{options}
\alias{options}
\alias{newDevice}

\title{
  additional options in package `distr'
}
\description{
  In package \pkg{distr}, we add an extra option \code{"newDevice"};
  it is inspected and manipulated as usual.
}
\details{We do not change the behaviour of \code{options} or \code{getOption};
for the general documentation to these two functions, 
confer \code{\link[base]{options}}, \code{\link[base:options]{getOption}}. Here
we only document added options.}
\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}}

\section{Additionally available options in package 'distr'}{
\describe{
\item{\code{"newDevice"}}{logical; controls behaviour when generating several plots within
one function; if \code{TRUE}, before each call to  call to \code{\link[graphics:frame]{plot.new}},
a call to \code{\link{devNew}} is inserted; if \code{FALSE}, we reproduce the usual behaviour
in \pkg{graphics}, i.e.; we do not call \code{\link{devNew}}. Defaults to \code{FALSE}.}
}
} 
\examples{
getOption("newDevice") 
options("newDevice"=TRUE) 
}
\seealso{\code{\link[base]{options}}, \code{\link[base:options]{getOption}}}
\keyword{environment}
\keyword{iplot}
\keyword{dplot}
\concept{global options}
\concept{options}

\name{ChisqParameter-class} 
\docType{class}
\alias{ChisqParameter-class}
\alias{initialize,ChisqParameter-method}

\title{Class "ChisqParameter"}
\description{ The parameter of a chi-squared distribution, used by Chisq-class}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("ChisqParameter", ncp, df)}.
Usually an object of this class is not needed on its own, it is generated automatically when an object of the class Chisq
is instantiated. 
}
\section{Slots}{
  \describe{
    \item{\code{ncp}}{Object of class \code{"numeric"}: the ncp of a chi-squared distribution }
    \item{\code{df}}{Object of class \code{"numeric"}: the df of a chi-squared distribution }
    \item{\code{name}}{Object of class \code{"character"}: a name / comment for the parameters }
  }
}
\section{Extends}{
Class \code{"Parameter"}, directly.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "ChisqParameter")}: initialize method }
    \item{df}{\code{signature(object = "ChisqParameter")}: returns the slot \code{df} of the parameter of the distribution }
    \item{df<-}{\code{signature(object = "ChisqParameter")}: modifies the slot \code{df} of the parameter of the distribution }
    \item{ncp}{\code{signature(object = "ChisqParameter")}: returns the slot \code{ncp} of the parameter of the distribution }
    \item{ncp<-}{\code{signature(object = "ChisqParameter")}: modifies the slot \code{ncp} of the parameter of the distribution }
  }
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }


\seealso{
\code{\link{Chisq-class}}
\code{\link{Parameter-class}}
}

\examples{
W <- new("ChisqParameter",df=1,ncp=1)
ncp(W) # ncp of this distribution is 1.
ncp(W) <- 2 # ncp of this distribution is now 2.
}
\keyword{distribution}
\concept{parameter}
\concept{Chi square distribution}
\concept{S4 parameter class}

\name{AbscontDistribution-class} 
\docType{class}
\alias{AbscontDistribution-class}
\alias{AffLinDistribution-class}
\alias{AffLinAbscontDistribution-class}
\alias{sqrt,AbscontDistribution-method}
\alias{initialize,AbscontDistribution-method}
\alias{initialize,AffLinAbscontDistribution-method}
\alias{sqrt,AbscontDistribution-method}

\title{Class "AbscontDistribution"}
\description{The \code{AbscontDistribution}-class is the mother-class of the classes \code{Beta}, \code{Cauchy}, 
\code{Chisq}, \code{Exp}, \code{F}, \code{Gammad}, \code{Lnorm}, \code{Logis}, \code{Norm}, \code{T}, \code{Unif} and 
\code{Weibull}. Further absolutely continuous distributions can be defined either by declaration of
own random number generator, density, cumulative distribution and quantile functions, or as result of a 
convolution of two absolutely continuous distributions or by application of a mathematical operator to an absolutely
continuous distribution.}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("AbscontDistribution", r, d, p, q)}.
More comfortably, you may use the generating function \code{\link{AbscontDistribution}}.
The result of these calls is an absolutely continuous distribution. 
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}: the space of the image of this distribution which has dimension 1
    and the name "Real Space" }
    \item{\code{param}}{Object of class \code{"Parameter"}: the parameter of this distribution, having only 
    the slot name "Parameter of an absolutely continuous distribution" }
    \item{\code{r}}{Object of class \code{"function"}: generates random numbers}
    \item{\code{d}}{Object of class \code{"function"}: density function}
    \item{\code{p}}{Object of class \code{"function"}: cumulative distribution function}
    \item{\code{q}}{Object of class \code{"function"}: quantile function}
    \item{\code{gaps}}{[from version 1.9 on] Object of class \code{"OptionalMatrix"}, 
          i.e.; an object which may either be \code{NULL} ora \code{matrix}. 
          This slot, if non-\code{NULL}, contains left and right 
          endpoints of intervals where the density of the object is 0. This slot 
          may be inspected by the accessor \code{\link{gaps}()} and modified by a corresponding
          replacement method. It may also be filled automatically by 
         \code{\link{setgaps}()}. For saved objects from earlier versions, we provide functions
         \code{\link{isOldVersion}} and \code{\link{conv2NewVersion}}.}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
}}
\section{Extends}{
Class \code{"UnivariateDistribution"}, directly. \cr
Class \code{"Distribution"}, by class \code{"UnivariateDistribution"}.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "AbscontDistribution")}: initialize method }
    \item{Math}{\code{signature(x = "AbscontDistribution")}: application of a mathematical function, e.g. \code{sin} or
    \code{exp} (does not work with \code{log}, \code{sign}!), to this absolutely continouos distribution
    \itemize{
      \item \code{abs}: \code{signature(x = "AbscontDistribution")}: exact image distribution of \code{abs(x)}.
      \item \code{exp}: \code{signature(x = "AbscontDistribution")}:  exact image distribution of \code{exp(x)}.
      \item \code{sign}: \code{signature(x = "AbscontDistribution")}:  exact image distribution of \code{sign(x)}.
      \item \code{sqrt}: \code{signature(x = "AbscontDistribution")}:  exact image distribution of \code{sqrt(x)}.
      \item \code{log}: \code{signature(x = "AbscontDistribution")}:  (with optional further argument \code{base}, defaulting to \code{exp(1)}) exact image distribution of \code{log(x)}.
      \item \code{log10}: \code{signature(x = "AbscontDistribution")}:  exact image distribution of \code{log10(x)}.
      \item \code{gamma}: \code{signature(x = "AbscontDistribution")}:  exact image distribution of \code{gamma(x)}.
      \item \code{lgamma}: \code{signature(x = "AbscontDistribution")}:  exact image distribution of \code{lgamma(x)}.
      \item \code{digamma}: \code{signature(x = "AbscontDistribution")}:  exact image distribution of \code{digamma(x)}.
      \item \code{sqrt}: \code{signature(x = "AbscontDistribution")}:  exact image distribution of \code{sqrt(x)}.
    }}
    \item{-}{\code{signature(e1 = "AbscontDistribution")}: application of `-' to this absolutely continuous distribution.}
    \item{*}{\code{signature(e1 = "AbscontDistribution", e2 = "numeric")}: multiplication of this absolutely continuous distribution by an object of class \code{"numeric"}}
    \item{/}{\code{signature(e1 = "AbscontDistribution", e2 = "numeric")}: division of this absolutely continuous distribution by an object of class \code{"numeric"}}
    \item{+}{\code{signature(e1 = "AbscontDistribution", e2 = "numeric")}: addition of this absolutely continuous distribution to an object of class \code{"numeric"}.}
    \item{-}{\code{signature(e1 = "AbscontDistribution", e2 = "numeric")}: subtraction of an object of class \code{"numeric"} from this absolutely continuous distribution.}
    \item{*}{\code{signature(e1 = "numeric", e2 = "AbscontDistribution")}: multiplication of this absolutely continuous distribution by an object of class \code{"numeric"}.}
    \item{+}{\code{signature(e1 = "numeric", e2 = "AbscontDistribution")}: addition of this absolutely continuous distribution to an object of class \code{"numeric"}.}
    \item{-}{\code{signature(e1 = "numeric", e2 = "AbscontDistribution")}: subtraction of this absolutely continuous distribution from an object of class \code{"numeric"}.}
    \item{+}{\code{signature(e1 = "AbscontDistribution", e2 = "AbscontDistribution")}: Convolution of two absolutely continuous distributions. The slots p, d and q are approximated by grids.}
    \item{-}{\code{signature(e1 = "AbscontDistribution", e2 = "AbscontDistribution")}: Convolution of two absolutely continuous distributions. The slots p, d and q are approximated by grids.}
    \item{plot}{\code{signature(object = "AbscontDistribution")}: plots density, cumulative distribution and quantile function.}
  }
}

\section{Internal subclass "AffLinAbscontDistribution"}{
To enhance accuracy of several functionals on distributions,
  mainly from package \pkg{distrEx}, from version 1.9 of this package on, 
  there is an internally used (but exported) subclass 
  \code{"AffLinAbscontDistribution"} which has extra slots 
  \code{a}, \code{b} (both of class \code{"numeric"}),  and \code{X0} 
  (of class \code{"AbscontDistribution"}), to capture the fact 
  that the object has the same distribution as \code{a * X0 + b}. This is 
  the class of the return value of methods 
  \describe{
    \item{-}{\code{signature(e1 = "AbscontDistribution")}}
    \item{*}{\code{signature(e1 = "AbscontDistribution", e2 = "numeric")}}
    \item{/}{\code{signature(e1 = "AbscontDistribution", e2 = "numeric")}}
    \item{+}{\code{signature(e1 = "AbscontDistribution", e2 = "numeric")}}
    \item{-}{\code{signature(e1 = "AbscontDistribution", e2 = "numeric")}}
    \item{*}{\code{signature(e1 = "numeric", e2 = "AbscontDistribution")}}
    \item{+}{\code{signature(e1 = "numeric", e2 = "AbscontDistribution")}}
    \item{-}{\code{signature(e1 = "numeric", e2 = "AbscontDistribution")}}
    \item{-}{\code{signature(e1 = "AffLinAbscontDistribution")}}
    \item{*}{\code{signature(e1 = "AffLinAbscontDistribution", e2 = "numeric")}}
    \item{/}{\code{signature(e1 = "AffLinAbscontDistribution", e2 = "numeric")}}
    \item{+}{\code{signature(e1 = "AffLinAbscontDistribution", e2 = "numeric")}}
    \item{-}{\code{signature(e1 = "AffLinAbscontDistribution", e2 = "numeric")}}
    \item{*}{\code{signature(e1 = "numeric", e2 = "AffLinAbscontDistribution")}}
    \item{+}{\code{signature(e1 = "numeric", e2 = "AffLinAbscontDistribution")}}
    \item{-}{\code{signature(e1 = "numeric", e2 = "AffLinAbscontDistribution")}}
  }
  There also is a class union of \code{"AffLinAbscontDistribution"},
  \code{"AffLinDiscreteDistribution"}, \code{"AffLinUnivarLebDecDistribution"}
  and called \code{"AffLinDistribution"}
  which is used for functionals.
}
\section{Internal virtual superclass "AcDcLcDistribution"}{
As many operations should be valid no matter whether the operands
are of class \code{"AbscontDistribution"},
  \code{"DiscreteDistribution"}, or \code{"UnivarLebDecDistribution"},
there is a class union of these classes called \code{"AcDcLcDistribution"};
in partiucalar methods for \code{"*"}, \code{"/"}, 
\code{"^"} (see \link{operators-methods}) and methods
\code{\link{Minimum}}, \code{Maximum}, \code{\link{Truncate}}, and
 \code{\link{Huberize}}, and \code{\link{convpow}} are defined for this 
 class union.   
}

\author{
  Thomas Stabla \email{statho3@web.de},\cr 
  Florian Camphausen \email{fcampi@gmx.de},\cr
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de},\cr 
  Matthias Kohl \email{Matthias.Kohl@stamats.de}
  }


\seealso{
\code{\link{AbscontDistribution}}
\code{\link{Parameter-class}}
\code{\link{UnivariateDistribution-class}}
\code{\link{Beta-class}}
\code{\link{Cauchy-class}}
\code{\link{Chisq-class}}
\code{\link{Exp-class}}
\code{\link{Fd-class}}
\code{\link{Gammad-class}}
\code{\link{Lnorm-class}}
\code{\link{Logis-class}}
\code{\link{Norm-class}}
\code{\link{Td-class}}
\code{\link{Unif-class}}
\code{\link{Weibull-class}}
\code{\link{DiscreteDistribution-class}}
\code{\link{Reals-class}}
\code{\link{RtoDPQ}}

}
\examples{
N <-  Norm() # N is a normal distribution with mean=0 and sd=1.
E <-  Exp() # E is an exponential distribution with rate=1.
A1 <-  E+1 # a new absolutely continuous distributions with exact slots d, p, q
A2 <-  A1*3 # a new absolutely continuous distributions with exact slots d, p, q
A3 <- N*0.9 + E*0.1 # a new absolutely continuous distribution with approximated slots d, p, q
r(A3)(1) # one random number generated from this distribution, e.g. -0.7150937
d(A3)(0) # The (approximated) density for x=0 is 0.43799.
p(A3)(0) # The (approximated) probability that x <= 0 is 0.45620.
q(A3)(.1) # The (approximated) 10 percent quantile is -1.06015.
## in RStudio or Jupytier IRKernel, use q.l(.)(.) instead of q(.)(.)
}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{S4 distribution class}

\name{DistributionSymmetry-class}
\docType{class}
\alias{DistributionSymmetry-class}

\title{Class of Symmetries for Distributions}
\description{Class of symmetries for distributions.}
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}
\section{Slots}{
  \describe{
    \item{\code{type}}{Object of class \code{"character"}:
      discribes type of symmetry. }
    \item{\code{SymmCenter}}{Object of class \code{"OptionalNumeric"}:
      center of symmetry. }
  }
}
\section{Extends}{
Class \code{"Symmetry"}, directly.
}
%\section{Methods}{}
%\references{}
\author{Matthias Kohl \email{Matthias.Kohl@stamats.de}}
%\note{}
\seealso{\code{\link{Symmetry-class}}, \code{\link{Distribution-class}}, 
    \code{\link{OptionalNumeric-class}}}
%\examples{}
\concept{symmetry}
\keyword{classes}
\keyword{distribution}

\name{df-methods}
\docType{methods}
\alias{df-methods}
\alias{df<--methods}
\alias{df}
\alias{df<-}
\alias{df,TParameter-method}
\alias{df,Td-method}
\alias{df,ChisqParameter-method}
\alias{df,Chisq-method}
\alias{df<-,TParameter-method}
\alias{df<-,Td-method}
\alias{df<-,ChisqParameter-method}
\alias{df<-,Chisq-method}
\title{ Methods for Function df in Package `distr' }
\description{df-methods}

\section{Methods}{\describe{

\item{df}{\code{signature(object = "TParameter")}: returns the slot \code{df} of the parameter of the distribution }
\item{df<-}{\code{signature(object = "TParameter")}: modifies the slot \code{df} of the parameter of the distribution }

\item{df}{\code{signature(object = "Td")}: returns the slot \code{df} of the parameter of the distribution }
\item{df<-}{\code{signature(object = "Td")}: modifies the slot \code{df} of the parameter of the distribution }

\item{df}{\code{signature(object = "ChisqParameter")}: returns the slot \code{df} of the parameter of the distribution }
\item{df<-}{\code{signature(object = "ChisqParameter")}: modifies the slot \code{df} of the parameter of the distribution }

\item{df}{\code{signature(object = "Chisq")}: returns the slot \code{df} of the parameter of the distribution }
\item{df<-}{\code{signature(object = "Chisq")}: modifies the slot \code{df} of the parameter of the distribution }
}}
\keyword{distribution}
\keyword{methods}
\concept{degree of freedom}
\concept{parameter}
\concept{T distribution}
\concept{Chi square distribution}
\concept{accessor function}
\concept{replacement function}

\name{Arcsine-class}
\docType{class}
\alias{Arcsine-class}
\alias{Arcsine}
\alias{initialize,Arcsine-method}
\title{Class "Arcsine"}
\description{
  The Arcsine distribution has density
  \deqn{f(x)=\frac{1}{\pi \sqrt{1-x^2}%
  }}{1/sqrt(1+x^2)/pi}
  for \eqn{-1 < x < 1}{-1 < x < 1}.
}

\section{Objects from the Class}{
  Objects can be created by calls of the form \code{Arcsine()}.
  This object is an Arcsine distribution.
}
\section{Slots}{
  \describe{
    \item{\code{img}}{Object of class \code{"Reals"}:
      The space of the image of this distribution has got dimension 1 and the name "Real Space". }
    \item{\code{r}}{Object of class \code{"function"}:
      generates random numbers (calls function rArcsine)}
    \item{\code{d}}{Object of class \code{"function"}:
      density function (calls function dArcsine)}
    \item{\code{p}}{Object of class \code{"function"}:
      cumulative function (calls function pArcsine)}
    \item{\code{q}}{Object of class \code{"function"}:
      inverse of the cumulative function (calls function qArcsine)}
    \item{\code{.withArith}}{logical: used internally to issue warnings as to 
            interpretation of arithmetics}
    \item{\code{.withSim}}{logical: used internally to issue warnings as to 
          accuracy}
    \item{\code{.logExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the log version of density, cdf, and 
    quantile function}
    \item{\code{.lowerExact}}{logical: used internally to flag the case where 
    there are explicit formulae for the lower tail version of cdf and quantile 
    function}
    \item{\code{Symmetry}}{object of class \code{"DistributionSymmetry"};
     used internally to avoid unnecessary calculations.}
  }
}
\section{Extends}{
  Class \code{"AbscontDistribution"}, directly. \cr
  Class \code{"UnivariateDistribution"}, by class \code{"AbscontDistribution"}. \cr
  Class \code{"Distribution"}, by class \code{"AbscontDistribution"}.
}
\section{Methods}{
  \describe{
    \item{initialize}{\code{signature(.Object = "Arcsine")}:
      initialize method}
  }
}

\author{
  Peter Ruckdeschel \email{peter.ruckdeschel@uni-oldenburg.de}
  }
\seealso{
  \code{\link{AbscontDistribution-class}}
  \code{\link{Reals-class}}
}
\examples{
A <- Arcsine()
# A is a Arcsine distribution with shape1 = 1 and shape2 = 1.
r(A)(3) # three random number generated from this distribution, e.g. 0.6979795
d(A)(c(-2,-1,-0.2,0,0.2,1,2)) # Density at x=c(-1,-0.2,0,0.2,1).
p(A)(c(-2,-1,-0.2,0,0.2,1,2)) # cdf at q=c(-1,-0.2,0,0.2,1).
q(A)(c(0,0.2,1,2)) # quantile function at at x=c(0,0.2,1).
## in RStudio or Jupyter IRKernel, use q.l(A)(c(0,0.2,1,2)) instead
}
\keyword{distribution}
\concept{absolutely continuous distribution}
\concept{Arcsine distribution}
\concept{S4 distribution class}
\concept{generating function}

\name{Minimum-methods}
\docType{methods}
\alias{Minimum-methods}
\alias{Minimum}
\alias{Maximum-methods}
\alias{Maximum}
\alias{Minimum,AbscontDistribution,AbscontDistribution-method}
\alias{Minimum,DiscreteDistribution,DiscreteDistribution-method}
\alias{Minimum,AbscontDistribution,Dirac-method}
\alias{Minimum,AcDcLcDistribution,AcDcLcDistribution-method}
\alias{Minimum,AbscontDistribution,numeric-method}
\alias{Minimum,DiscreteDistribution,numeric-method}
\alias{Minimum,AcDcLcDistribution,numeric-method}
\alias{Maximum,AcDcLcDistribution,AcDcLcDistribution-method}
\alias{Maximum,AcDcLcDistribution,numeric-method}
\title{ Methods for functions Minimum and Maximum in Package `distr' }
\description{Minimum and Maximum-methods}
\usage{
Minimum(e1, e2, ...)
Maximum(e1, e2, ...) 
\S4method{Minimum}{AbscontDistribution,AbscontDistribution}(e1,e2, ...)
\S4method{Minimum}{DiscreteDistribution,DiscreteDistribution}(e1,e2, ...)
\S4method{Minimum}{AbscontDistribution,Dirac}(e1,e2, 
                   withSimplify = getdistrOption("simplifyD"))
\S4method{Minimum}{AcDcLcDistribution,AcDcLcDistribution}(e1,e2, 
                   withSimplify = getdistrOption("simplifyD"))
\S4method{Maximum}{AcDcLcDistribution,AcDcLcDistribution}(e1,e2, 
                   withSimplify = getdistrOption("simplifyD"))
\S4method{Minimum}{AbscontDistribution,numeric}(e1,e2, ...)
\S4method{Minimum}{DiscreteDistribution,numeric}(e1,e2, ...)
\S4method{Minimum}{AcDcLcDistribution,numeric}(e1,e2,
                   withSimplify = getdistrOption("simplifyD"))
\S4method{Maximum}{AcDcLcDistribution,numeric}(e1,e2, 
                   withSimplify = getdistrOption("simplifyD"))
}
\arguments{
  \item{e1}{distribution object}
  \item{e2}{distribution object or numeric}
  \item{\dots}{further arguments (to be able to call various methods  with the same
               arguments}
  \item{withSimplify}{logical; is result to be piped through a call to 
         \code{\link{simplifyD}}?}
}
\value{the corresponding distribution of the minimum / maximum}

\section{Methods}{\describe{
\item{Minimum}{\code{signature(e1 = "AbscontDistribution", e2 = "AbscontDistribution")}: 
returns the distribution of \code{min(X1,X2)}, if \code{X1},\code{X2} are independent
and distributed according to \code{e1} and \code{e2} respectively;
the result is again of class \code{"AbscontDistribution"}}
\item{Minimum}{\code{signature(e1 = "DiscreteDistribution", e2 = "DiscreteDistribution")}: 
returns the distribution of \code{min(X1,X2)}, if \code{X1},\code{X2} are independent
and distributed according to \code{e1} and \code{e2} respectively;
the result is again of class \code{"DiscreteDistribution"}}
\item{Minimum}{\code{signature(e1 = "AbscontDistribution", e2 = "Dirac")}: 
returns the distribution of \code{min(X1,X2)}, if \code{X1},\code{X2} are  
distributed according to \code{e1} and \code{e2} respectively;
the result is of class \code{"UnivarLebDecDistribution"}}
\item{Minimum}{\code{signature(e1 = "AcDcLcDistribution", e2 = "AcDcLcDistribution")}: 
returns the distribution of \code{min(X1,X2)}, if \code{X1},\code{X2} are  
distributed according to \code{e1} and \code{e2} respectively;
the result is of class \code{"UnivarLebDecDistribution"}}
\item{Minimum}{\code{signature(e1 = "AcDcLcDistribution", e2 = "numeric")}: 
if \code{e2} = \eqn{n}, returns the distribution of \code{min(X1,X2,...,Xn)}, if \code{X1},\code{X2}, 
..., \code{Xn} are i.i.d. according to \code{e1};
the result is of class \code{"UnivarLebDecDistribution"}}
\item{Maximum}{\code{signature(e1 = "AcDcLcDistribution", e2 = "AcDcLcDistribution")}: 
returns the distribution of \code{max(X1,X2)}, if \code{X1},\code{X2} are  
distributed according to \code{e1} and \code{e2} respectively;
translates into \code{-Minimum(-e1,-e2)};
the result is of class \code{"UnivarLebDecDistribution"}}
\item{Maximum}{\code{signature(e1 = "AcDcLcDistribution", e2 = "numeric")}: 
if \code{e2} = \eqn{n}, returns the distribution of \code{max(X1,X2,...,Xn)}, if \code{X1},\code{X2}, 
..., \code{Xn} are i.i.d. according to \code{e1}; translates into 
\code{-Minimum(-e1,e2)}; the result is of class \code{"UnivarLebDecDistribution"}}
}}
\seealso{\code{\link{Huberize}}, \code{\link{Truncate}}}
\examples{
## IGNORE_RDIFF_BEGIN
plot(Maximum(Unif(0,1), Minimum(Unif(0,1), Unif(0,1))))
plot(Minimum(Exp(4),4))
## IGNORE_RDIFF_END

\donttest{
## a sometimes lengthy example...
plot(Minimum(Norm(),Pois()))}
}
\keyword{distribution}
\keyword{methods}
\concept{minimum}
\concept{maximum}
